SystemOrganization addCategory: #'DrGeoII-Core-App'!
SystemOrganization addCategory: #'DrGeoII-Core-Builder'!
SystemOrganization addCategory: #'DrGeoII-Core-Command'!
SystemOrganization addCategory: #'DrGeoII-Core-Factory'!
SystemOrganization addCategory: #'DrGeoII-Core-Item'!
SystemOrganization addCategory: #'DrGeoII-Core-Item-View'!
SystemOrganization addCategory: #'DrGeoII-Core-Macro'!
SystemOrganization addCategory: #'DrGeoII-Core-Script'!
SystemOrganization addCategory: #'DrGeoII-Core-Tool'!
SystemOrganization addCategory: #'DrGeoII-Core-UI'!

!Rectangle methodsFor: '*DrGeoII-Core-intersection' stamp: 'HilaireFernandes 6/19/2010 18:03'!
intersectionWith: aLineSeg 
	| intersections |
	intersections := Set new.
	(aLineSeg
		intersectionWith: (LineSegment from: self origin to: self bottomLeft))
		ifNotNilDo: [:inter | intersections add: inter].
	(aLineSeg
		intersectionWith: (LineSegment from: self origin to: self topRight))
		ifNotNilDo: [:inter | intersections add: inter].
	(aLineSeg
		intersectionWith: (LineSegment from: self corner to: self bottomLeft))
		ifNotNilDo: [:inter | intersections add: inter].
	(aLineSeg
		intersectionWith: (LineSegment from: self corner to: self topRight))
		ifNotNilDo: [:inter | intersections add: inter].
		intersections := intersections asOrderedCollection.
	intersections size = 2
		ifTrue: [^ LineSegment from: intersections first to: intersections second].
	intersections size = 0 
		ifTrue: 
			[(self containsPoint: aLineSeg start) 
				ifTrue: [(self containsPoint: aLineSeg end) ifTrue: [^aLineSeg ]]
				ifFalse: [(self containsPoint: aLineSeg end) ifFalse: [^nil]]].

	(self containsPoint: aLineSeg start) 
		ifTrue: [
			(self containsPoint: aLineSeg start) 
				ifTrue: [^aLineSeg]
				ifFalse: [^LineSegment from: aLineSeg start to: intersections first]].
	(self containsPoint: aLineSeg end) 
		ifTrue: [^LineSegment from: aLineSeg end to: intersections first].
	^nil! !

Model subclass: #DrGMathItem
	instanceVariableNames: 'parents name descriptiveName longDescription exist actionMap dirty id'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'DrGeoII-Core-Item'!

!DrGMathItem commentStamp: 'HilaireFernandes 7/22/2011 15:12' prior: 0!
This is the root class of the mathematic items hierarchy.
The classes in this hierarchy are models of the mathematical items like point, line, circle.
These models will have different type of view depending on the ApplicationModel/View/Presenter.

id                            - unique id used when importing Dr. Geo 1.1 files / it is used to cache the index position in the factory pool 
name 				- for example A, (AB), [AD], d
descriptiveName	- 'this point A', 'this line (AB)'
longDescription	- the point A, the symmetric of the point B in respect to O'
basicType 			- object familly the instance belong to (#point, #line,...), it is part of the class name
nodeType 			- the sub type the instance belong to (#2pts, #translation, #intersection), it is the value of XML  'type' attribute
!

DrGMathItem subclass: #DrGArcItem
	instanceVariableNames: 'radius center origin length'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'DrGeoII-Core-Item'!

DrGArcItem subclass: #DrGArc3ptsItem
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'DrGeoII-Core-Item'!

!DrGArc3ptsItem methodsFor: 'accessing' stamp: 'HilaireFernandes 3/21/2006 10:53'!
endPoint
	^parents third point! !

!DrGArc3ptsItem methodsFor: 'xml writing' stamp: 'HilaireFernandes 1/20/2008 08:44'!
nodeType
	^#'3pts'! !

!DrGArc3ptsItem methodsFor: 'accessing' stamp: 'HilaireFernandes 1/17/2006 16:35'!
parentQty
	^3! !

!DrGArc3ptsItem methodsFor: 'as yet unclassified' stamp: 'HilaireFernandes 4/18/2011 22:12'!
parentsEqual: aMathItem 
	^ self parents = aMathItem parents
		or: [self parents reverse = aMathItem parents]! !

!DrGArc3ptsItem methodsFor: 'accessing' stamp: 'HilaireFernandes 3/21/2006 10:53'!
startPoint
	^parents first point! !

!DrGArc3ptsItem methodsFor: 'updating' stamp: 'HilaireFernandes 10/17/2010 11:43'!
update
	| direction1 direction2 a b c ab |
	self doParentsExist 
		ifTrue: 
			[a := parents first point.
			b := parents second point.
			c := parents third point.
			a = b 
				ifTrue: 
					[exist := false.
					^self].
			c = b 
				ifTrue: 
					[exist := false.
					^self ].
			direction1 := (DrGDirectionItem new)
						origin: (a + b) / 2;
						direction: (a - b) normal.
			direction2 := (DrGDirectionItem new)
						origin: (b + c) / 2;
						direction: (b - c) normal.
			center := direction1 intersectionWithDirection: direction2.
			center ifNil: [^self updateAsSegmentOrRay].
			a := a - center.
			b := b - center.
			c := c - center.
			radius := a r.
			origin := a y asFloat arcTan2: a x.
			ab := (b y asFloat arcTan2: b x) - origin.
			length := (c y asFloat arcTan2: c x) - origin.
			length = 0 
				ifTrue: 
					[exist := false.
					^self].
			(length < 0 and: [ab < length or: [ab > 0]]) 
				ifTrue: 
					[length := length + DrGDirectionItem twoPi.
					^self].
			(length > 0 and: [ab > length or: [ab < 0]]) 
				ifTrue: [length := length - DrGDirectionItem twoPi]].! !

!DrGArc3ptsItem methodsFor: 'updating' stamp: 'HilaireFernandes 1/24/2008 10:32'!
updateAsSegmentOrRay
"the three point are aligned, the arc is a segment"
	exist := false.! !

DrGArcItem subclass: #DrGArcHomothetyItem
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'DrGeoII-Core-Item'!

!DrGArcHomothetyItem methodsFor: 'updating' stamp: 'HilaireFernandes 3/25/2006 15:41'!
moveParent: aPoint
	^false! !

!DrGArcHomothetyItem methodsFor: 'xml writing' stamp: 'HilaireFernandes 1/20/2008 12:03'!
nodeType
	^#Scale! !

!DrGArcHomothetyItem methodsFor: 'updating' stamp: 'HilaireFernandes 10/17/2010 11:43'!
update
|arcItem centerHomothety|
	self doParentsExist ifTrue:
		[arcItem := parents first.
		centerHomothety := parents second point.
		center := centerHomothety +
			(parents third valueItem * (arcItem center - centerHomothety)).
		radius := arcItem radius * parents third valueItem abs.
		origin := arcItem origin + (parents third valueItem sign -1 * Float halfPi).
		length := arcItem length].! !

!DrGArcItem methodsFor: 'accessing' stamp: 'HilaireFernandes 1/17/2006 15:39'!
abscissaOf: aPoint
	^(self angleOf: aPoint) / length! !

!DrGArcItem methodsFor: 'private' stamp: 'jlrr 9/29/2007 15:01'!
adaptiveDescriptiveName
	^ 'This Arc Circle %1' translated! !

!DrGArcItem methodsFor: 'private' stamp: 'HilaireFernandes 6/22/2006 08:50'!
angleOf: aPoint 
	"Compute the angle position of aPoint in the circle"

	| angleM pointM |
	pointM := aPoint - center.
	angleM := (pointM y asFloat arcTan2: pointM x) - origin.
	"Transcript show: 'angle(1): ', angleM asString, 'origin: ', origin asString."
	length > 0 
		ifTrue: 
			[angleM < 0 ifTrue: [angleM := angleM + DrGDirectionItem twoPi].
			angleM > length ifTrue: [angleM := length]]
		ifFalse: 
			[angleM > 0 ifTrue: [angleM := angleM - DrGDirectionItem twoPi].
			angleM < length ifTrue: [angleM := length]].
	"	Transcript show: 'angle: ', angleM asString , ' length: ', length asString;cr."
	^angleM! !

!DrGArcItem methodsFor: 'accessing' stamp: 'HilaireFernandes 1/16/2006 21:00'!
basicType
	^#arc! !

!DrGArcItem methodsFor: 'accessing' stamp: 'HilaireFernandes 1/16/2006 21:01'!
center
	^center! !

!DrGArcItem methodsFor: 'accessing' stamp: 'HilaireFernandes 1/16/2006 21:01'!
center: aPoint
	center := aPoint ! !

!DrGArcItem methodsFor: 'computing' stamp: 'HilaireFernandes 1/17/2006 14:37'!
closestPointTo: aPoint
	| angle |
	angle := self angleOf: aPoint.
	^center + (radius * ((origin + angle) cos @ (origin + angle) sin))! !

!DrGArcItem methodsFor: 'testing' stamp: 'HilaireFernandes 9/5/2009 22:53'!
contains: aPoint 
	| angleM pointM |
	((center dist: aPoint) closeTo: radius) ifFalse: [^false].
	pointM := aPoint - center.
	angleM := (pointM y asFloat arcTan2: pointM x) - origin.
	^ length > 0 
		ifTrue: 
			[angleM < 0 ifTrue: [angleM := angleM + DrGDirectionItem twoPi].
			length >= angleM]
		ifFalse: 
			[angleM > 0 ifTrue: [angleM := angleM - DrGDirectionItem twoPi].
			length <= angleM]! !

!DrGArcItem methodsFor: 'accessing' stamp: 'HilaireFernandes 3/21/2006 10:51'!
endPoint
	self subclassResponsibility! !

!DrGArcItem methodsFor: 'computing' stamp: 'HilaireFernandes 1/17/2006 18:58'!
intersectionWith: aCurveItem flag: aFlag
	| aPoint |
	aCurveItem isLineItem 
		ifTrue: 
			[aPoint := self intersectionWithDirection: aCurveItem flag: aFlag.
			aPoint ifNil: [^nil].
			(self contains: aPoint)
				ifTrue: [^aPoint]
				ifFalse: [^nil]].
	(aCurveItem isRayItem or: [aCurveItem isSegmentItem])
		ifTrue:
			[aPoint := self intersectionWithDirection: aCurveItem flag: aFlag.
			aPoint ifNil: [^nil].
			((self contains: aPoint) and: [aCurveItem contains: aPoint])
				ifTrue: [^aPoint]
				ifFalse: [^nil]].
	aCurveItem isCircleItem
		ifTrue:
			[aPoint := self intersectionWithCircle: aCurveItem flag: aFlag.
			aPoint ifNil: [^nil].
			(self contains: aPoint)
				ifTrue: [^aPoint]
				ifFalse: [^nil]].
	aCurveItem isArcItem
		ifTrue:
			[aPoint := self intersectionWithCircle: aCurveItem flag: aFlag.
			aPoint ifNil: [^nil].
			((self contains: aPoint) and: [aCurveItem contains: aPoint])
				ifTrue: [^aPoint]
				ifFalse: [^nil]].
! !

!DrGArcItem methodsFor: 'computing' stamp: 'HilaireFernandes 1/24/2006 22:55'!
intersectionWithCircle: aCircleOrArcItem flag: aFlag 
	| circleItem |
	circleItem := (DrGCircleItem new)
				center: self center;
				radius: self radius.
	^circleItem intersectionWithCircle: aCircleOrArcItem flag: aFlag! !

!DrGArcItem methodsFor: 'computing' stamp: 'HilaireFernandes 1/24/2006 22:55'!
intersectionWithDirection: aDirectionItem flag: aFlag 
	| circleItem |
	circleItem := (DrGCircleItem new)
				center: self center;
				radius: self radius.
	^circleItem intersectionWithDirection: aDirectionItem flag: aFlag! !

!DrGArcItem methodsFor: '*DrGeoII-Core-testing' stamp: 'HilaireFernandes 1/25/2006 09:34'!
isArcItem
	^true! !

!DrGArcItem methodsFor: '*DrGeoII-Core-testing' stamp: 'HilaireFernandes 1/25/2006 09:34'!
isCurveItem
	^true! !

!DrGArcItem methodsFor: '*DrGeoII-Core-testing' stamp: 'HilaireFernandes 1/25/2006 09:34'!
isGeometricItem
	^true! !

!DrGArcItem methodsFor: 'accessing' stamp: 'HilaireFernandes 1/16/2006 21:02'!
length
	^length! !

!DrGArcItem methodsFor: 'accessing' stamp: 'HilaireFernandes 1/16/2006 21:02'!
length: aFloat
	length := aFloat ! !

!DrGArcItem methodsFor: 'accessing' stamp: 'HilaireFernandes 1/16/2006 21:02'!
origin
	^origin! !

!DrGArcItem methodsFor: 'accessing' stamp: 'HilaireFernandes 1/16/2006 21:02'!
origin: aFloat
	origin := aFloat ! !

!DrGArcItem methodsFor: 'computing' stamp: 'HilaireFernandes 1/17/2006 14:39'!
pointAt: anAbscissa
	|angle|
	angle := origin + (anAbscissa * length).
	^center + (radius * (angle cos @ angle sin))
	! !

!DrGArcItem methodsFor: 'accessing' stamp: 'HilaireFernandes 1/16/2006 21:01'!
radius
	^radius! !

!DrGArcItem methodsFor: 'accessing' stamp: 'HilaireFernandes 1/16/2006 21:01'!
radius: aFloat
	radius := aFloat! !

!DrGArcItem methodsFor: 'accessing' stamp: 'HilaireFernandes 3/21/2006 10:51'!
startPoint
	self subclassResponsibility! !

DrGArcItem subclass: #DrGArcReflexionItem
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'DrGeoII-Core-Item'!

!DrGArcReflexionItem methodsFor: 'updating' stamp: 'HilaireFernandes 3/25/2006 15:42'!
moveParent: aPoint
	^false! !

!DrGArcReflexionItem methodsFor: 'xml writing' stamp: 'HilaireFernandes 1/20/2008 11:58'!
nodeType
	^#Reflexion! !

!DrGArcReflexionItem methodsFor: 'updating' stamp: 'HilaireFernandes 10/17/2010 11:43'!
update
|pointA|
	self doParentsExist ifTrue:
		[origin := parents first origin.
		radius := parents first radius.
		center := parents first center.
		length := parents first length negated.
		pointA := center + (radius * (origin cos @ origin sin)).
		pointA := pointA reflexionBy: parents second origin 
						direction: parents second direction.
		center := center reflexionBy: parents second origin 
						direction: parents second direction.
		pointA := pointA - center.
		origin := pointA y asFloat arcTan2: pointA x].
! !

DrGArcItem subclass: #DrGArcRotationItem
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'DrGeoII-Core-Item'!

!DrGArcRotationItem methodsFor: 'updating' stamp: 'HilaireFernandes 3/25/2006 15:42'!
moveParent: aPoint
	^false! !

!DrGArcRotationItem methodsFor: 'xml writing' stamp: 'HilaireFernandes 1/20/2008 12:04'!
nodeType
	^#Reflexion! !

!DrGArcRotationItem methodsFor: 'updating' stamp: 'HilaireFernandes 10/17/2010 11:43'!
update
|arcItem|
	self doParentsExist ifTrue:
		[arcItem := parents first.
		center := arcItem center
			rotateBy: parents third valueItem negated 
			about: parents second point.
		origin := arcItem origin + parents third valueItem.
		length := arcItem length.
		radius := arcItem radius].
! !

DrGArcItem subclass: #DrGArcSymmetryItem
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'DrGeoII-Core-Item'!

!DrGArcSymmetryItem methodsFor: 'updating' stamp: 'HilaireFernandes 3/25/2006 15:42'!
moveParent: aPoint
	^false! !

!DrGArcSymmetryItem methodsFor: 'xml writing' stamp: 'HilaireFernandes 1/20/2008 12:01'!
nodeType
	^#Symmetry! !

!DrGArcSymmetryItem methodsFor: 'updating' stamp: 'HilaireFernandes 10/17/2010 11:43'!
update
	self doParentsExist 
		ifTrue: 
			[origin := parents first origin + Float pi.
			origin > Float pi ifTrue: [origin := origin - DrGDirectionItem twoPi].
			radius := parents first radius.
			center := 2 * parents second point - parents first center.
			length := parents first length].
! !

DrGArcItem subclass: #DrGArcTranslationItem
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'DrGeoII-Core-Item'!

!DrGArcTranslationItem methodsFor: 'updating' stamp: 'HilaireFernandes 3/25/2006 15:42'!
moveParent: aPoint
	^false! !

!DrGArcTranslationItem methodsFor: 'xml writing' stamp: 'HilaireFernandes 1/20/2008 12:02'!
nodeType
	^#Translation! !

!DrGArcTranslationItem methodsFor: 'updating' stamp: 'HilaireFernandes 10/17/2010 11:43'!
update
	self doParentsExist ifTrue:
		[origin := parents first origin.
		radius := parents first radius.
		center := parents first center + parents second direction.
		length := parents first length].
! !

DrGMathItem subclass: #DrGCircleItem
	instanceVariableNames: 'radius center'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'DrGeoII-Core-Item'!

!DrGCircleItem commentStamp: '<historical>' prior: 0!
Instance variables radius and center are used to cache these values!

DrGCircleItem subclass: #DrGCircle2ptsItem
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'DrGeoII-Core-Item'!

!DrGCircle2ptsItem methodsFor: 'xml writing' stamp: 'HilaireFernandes 1/20/2008 12:03'!
nodeType
	^#'2pts'! !

!DrGCircle2ptsItem methodsFor: 'accessing' stamp: 'HilaireFernandes 1/17/2006 16:35'!
parentQty
	^2! !

!DrGCircle2ptsItem methodsFor: 'updating' stamp: 'HilaireFernandes 10/17/2010 11:43'!
update
	self doParentsExist
		ifTrue: [center := parents first point.
					radius := center dist: parents second point].! !

DrGCircleItem subclass: #DrGCircleHomothetyItem
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'DrGeoII-Core-Item'!

!DrGCircleHomothetyItem methodsFor: 'updating' stamp: 'HilaireFernandes 3/25/2006 15:42'!
moveParent: aPoint
	^false! !

!DrGCircleHomothetyItem methodsFor: 'xml writing' stamp: 'HilaireFernandes 1/20/2008 11:58'!
nodeType
	^#Scale! !

!DrGCircleHomothetyItem methodsFor: 'updating' stamp: 'HilaireFernandes 10/17/2010 11:43'!
update
|centerHomothety|
	self doParentsExist ifTrue:
		[centerHomothety := parents second point.
		radius := parents first radius * parents third valueItem abs.
		center := centerHomothety +
			(parents third valueItem * (parents first center - centerHomothety))].! !

!DrGCircleItem methodsFor: 'accessing' stamp: 'HilaireFernandes 1/24/2006 22:55'!
abscissaOf: aPoint 
	| angle vector |
	vector := aPoint - self center.
	angle := vector y arcTan: vector x.
	angle < 0 ifTrue: [angle := angle + (2 * Float pi)].
	^angle * 0.5 * DrGDirectionItem invPi! !

!DrGCircleItem methodsFor: 'private' stamp: 'HilaireFernandes 2/1/2008 12:35'!
adaptiveDescriptiveName
	^ 'This circle %1' translated! !

!DrGCircleItem methodsFor: 'accessing' stamp: 'HilaireFernandes 1/2/2006 10:41'!
basicType
	^#circle! !

!DrGCircleItem methodsFor: 'accessing' stamp: 'HilaireFernandes 1/2/2006 10:23'!
center
	^center! !

!DrGCircleItem methodsFor: 'accessing' stamp: 'HilaireFernandes 1/3/2006 16:27'!
center: aPoint
	center := aPoint ! !

!DrGCircleItem methodsFor: 'accessing' stamp: 'HilaireFernandes 1/2/2006 11:12'!
closestPointTo: aPoint
	^self center + (self radius * (aPoint - self center) normalized)! !

!DrGCircleItem methodsFor: 'testing' stamp: 'HilaireFernandes 4/17/2011 10:43'!
contains: aPoint
	^ (center dist: aPoint) closeTo: radius  ! !

!DrGCircleItem methodsFor: 'computing' stamp: 'HilaireFernandes 1/17/2006 17:49'!
intersectionWith: aCurveItem flag: aFlag
|aPoint|
	aCurveItem isLineItem 
		ifTrue: [^self intersectionWithDirection: aCurveItem  flag: aFlag].
	(aCurveItem isSegmentItem or: [aCurveItem isRayItem])
		ifTrue:
			[aPoint := self intersectionWithDirection: aCurveItem  flag: aFlag.
			aPoint ifNil: [^nil].
			(aCurveItem contains: aPoint)
				ifTrue: [^aPoint]
				ifFalse: [^nil]].
	aCurveItem isCircleItem
		ifTrue: [^self intersectionWithCircle: aCurveItem flag: aFlag].
	aCurveItem isArcItem
		ifTrue: [^aCurveItem intersectionWith: self flag: aFlag]! !

!DrGCircleItem methodsFor: 'computing' stamp: 'HilaireFernandes 1/24/2006 22:55'!
intersectionWithCircle: aCircleItem flag: aFlag 
	| directionItem aVector aRadius x d |
	directionItem := DrGDirectionItem new.
	aVector := aCircleItem center - center.
	aRadius := aCircleItem radius.
	d := aVector dotProduct: aVector.
	x := (d + radius squared - aRadius squared) / (2 * d).
	directionItem direction: aVector normal.
	directionItem origin: x * aVector + center.
	^self intersectionWithDirection: directionItem flag: aFlag! !

!DrGCircleItem methodsFor: 'computing' stamp: 'HilaireFernandes 1/12/2006 22:10'!
intersectionWithDirection: aDirectionItem flag: aFlag
|a b c d k vector|
	vector := aDirectionItem direction.
	a := vector dotProduct: vector.
	b := vector dotProduct: (aDirectionItem origin - self center).
	c := (aDirectionItem origin squaredDistanceTo: self center) - self radius squared.
	d := b squared - (a * c).
	(d < 0 or: [a = 0]) ifTrue: [^ nil].
	k := (b negated + (aFlag * d sqrt)) / a.
	^ aDirectionItem origin + (k * vector)! !

!DrGCircleItem methodsFor: '*DrGeoII-Core-testing' stamp: 'HilaireFernandes 1/25/2006 09:35'!
isCircleItem
	^true
! !

!DrGCircleItem methodsFor: '*DrGeoII-Core-testing' stamp: 'HilaireFernandes 1/25/2006 09:35'!
isCurveItem
	^true
! !

!DrGCircleItem methodsFor: '*DrGeoII-Core-testing' stamp: 'HilaireFernandes 1/25/2006 09:35'!
isGeometricItem
	^true
! !

!DrGCircleItem methodsFor: 'accessing' stamp: 'HilaireFernandes 1/2/2006 11:06'!
pointAt: anAbscissa
	|angle|
	angle := 2 * Float pi * anAbscissa.
	^ (Point x: angle cos y: angle sin) * self radius + self center.! !

!DrGCircleItem methodsFor: 'accessing' stamp: 'HilaireFernandes 1/2/2006 10:24'!
radius
	^radius! !

!DrGCircleItem methodsFor: 'accessing' stamp: 'HilaireFernandes 1/3/2006 16:27'!
radius: aFlaot
	radius := aFlaot ! !

DrGCircleItem subclass: #DrGCircleRadiusItem
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'DrGeoII-Core-Item'!

!DrGCircleRadiusItem methodsFor: 'updating' stamp: 'HilaireFernandes 10/28/2011 16:27'!
moveParent: aPoint 
	| thePoint |
	(parents
			noneSatisfy: [:item | item isValueItem or: [item isSegmentItem]])
		ifTrue: [^ super moveParent: aPoint].
	"we just want to move the center point, not the radius value"
	thePoint := parents first.
	thePoint moveEvent: aPoint.
	^ true! !

!DrGCircleRadiusItem methodsFor: 'xml writing' stamp: 'HilaireFernandes 1/22/2008 17:52'!
nodeType
	^ #radius! !

!DrGCircleRadiusItem methodsFor: 'accessing' stamp: 'HilaireFernandes 1/22/2008 17:51'!
parentQty
	^ 2! !

!DrGCircleRadiusItem methodsFor: 'updating' stamp: 'HilaireFernandes 10/17/2010 11:43'!
update
	self doParentsExist
		ifTrue: [center := parents first point.
			radius := parents second valueItem].! !

DrGCircleRadiusItem subclass: #DrGCircleSegmentItem
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'DrGeoII-Core-Item'!

!DrGCircleSegmentItem methodsFor: 'xml writing' stamp: 'HilaireFernandes 1/28/2008 22:21'!
nodeType
	^ #segment! !

!DrGCircleSegmentItem methodsFor: 'updating' stamp: 'HilaireFernandes 10/17/2010 11:43'!
update
	self doParentsExist
		ifTrue: [center := parents first point.
			radius := parents second direction r]! !

DrGCircleItem subclass: #DrGCircleReflexionItem
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'DrGeoII-Core-Item'!

!DrGCircleReflexionItem methodsFor: 'updating' stamp: 'HilaireFernandes 3/25/2006 15:42'!
moveParent: aPoint
	^false! !

!DrGCircleReflexionItem methodsFor: 'xml writing' stamp: 'HilaireFernandes 1/20/2008 12:00'!
nodeType
	^#Reflexion! !

!DrGCircleReflexionItem methodsFor: 'updating' stamp: 'HilaireFernandes 10/17/2010 11:43'!
update
	self doParentsExist ifTrue:
		[self radius: parents first radius.
		self center: (parents first center
							reflexionBy: parents second origin 
							direction: parents second direction)].
! !

DrGCircleItem subclass: #DrGCircleRotationItem
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'DrGeoII-Core-Item'!

!DrGCircleRotationItem methodsFor: 'updating' stamp: 'HilaireFernandes 3/25/2006 15:42'!
moveParent: aPoint
	^false! !

!DrGCircleRotationItem methodsFor: 'xml writing' stamp: 'HilaireFernandes 1/20/2008 12:01'!
nodeType
	^#Rotation! !

!DrGCircleRotationItem methodsFor: 'updating' stamp: 'HilaireFernandes 10/17/2010 11:43'!
update
	self doParentsExist ifTrue:
		[radius := parents first radius.
		center := parents first center
			rotateBy: parents third valueItem negated 
			about: parents second point].
! !

DrGCircleItem subclass: #DrGCircleSymmetryItem
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'DrGeoII-Core-Item'!

!DrGCircleSymmetryItem methodsFor: 'updating' stamp: 'HilaireFernandes 3/25/2006 15:42'!
moveParent: aPoint
	^false! !

!DrGCircleSymmetryItem methodsFor: 'xml writing' stamp: 'HilaireFernandes 1/20/2008 12:01'!
nodeType
	^#Symmetry! !

!DrGCircleSymmetryItem methodsFor: 'updating' stamp: 'HilaireFernandes 10/17/2010 11:43'!
update
	self doParentsExist ifTrue:
		[self radius: parents first radius.
		self center: (2 * parents second point) - parents first center].
! !

DrGCircleItem subclass: #DrGCircleTranslationItem
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'DrGeoII-Core-Item'!

!DrGCircleTranslationItem methodsFor: 'updating' stamp: 'HilaireFernandes 3/25/2006 15:42'!
moveParent: aPoint
	^false! !

!DrGCircleTranslationItem methodsFor: 'xml writing' stamp: 'HilaireFernandes 1/20/2008 12:02'!
nodeType
	^#Translation! !

!DrGCircleTranslationItem methodsFor: 'updating' stamp: 'HilaireFernandes 10/17/2010 11:43'!
update
	self doParentsExist ifTrue:
		[self center: parents first center + parents second direction.
		self radius: parents first radius].
! !

DrGMathItem subclass: #DrGCompositeItem
	instanceVariableNames: 'subItems'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'DrGeoII-Core-Item'!

!DrGCompositeItem commentStamp: '<historical>' prior: 0!
I am the composite item, I represent a set of math items.!

!DrGCompositeItem class methodsFor: 'as yet unclassified' stamp: 'HilaireFernandes 3/2/2006 18:17'!
newWith: theSubItems
	^self new
		subItems: theSubItems! !

!DrGCompositeItem methodsFor: 'comparing' stamp: 'HilaireFernandes 4/10/2010 12:36'!
= aMathItem
	^ aMathItem 
		ifNil: [false] 
		ifNotNil: [self className = aMathItem className 
			and: [self subItems = aMathItem subItems]]! !

!DrGCompositeItem methodsFor: 'private' stamp: 'jlrr 9/30/2007 14:01'!
adaptiveDescriptiveName
	^ 'This composite object %1' translated! !

!DrGCompositeItem methodsFor: 'comparing' stamp: 'HilaireFernandes 3/25/2006 17:56'!
hash
	^ self subItems hash bitXor: self className hash! !

!DrGCompositeItem methodsFor: '*DrGeoII-Core-testing' stamp: 'HilaireFernandes 3/1/2006 10:01'!
isCompositeItem
	^true! !

!DrGCompositeItem methodsFor: '*DrGeoII-Core-testing' stamp: 'HilaireFernandes 3/3/2006 12:08'!
isGeometricItem
"if all subitem are geometric item, considere ourself as a geometric item"
	subItems do: [:aMathItem |
		(aMathItem isGeometricItem) 
			ifFalse: [^false]].
	^true! !

!DrGCompositeItem methodsFor: 'updating' stamp: 'HilaireFernandes 10/16/2010 16:13'!
move: aPoint
	subItems do: [:item|
		(item class ~= DrGPointFreeItem
			and: [item class ~= DrGPointOncurveItem])
			ifTrue:
				[item moveEvent: aPoint]].! !

!DrGCompositeItem methodsFor: 'updating' stamp: 'HilaireFernandes 3/25/2006 18:27'!
moveParent: aPoint
	^false! !

!DrGCompositeItem methodsFor: 'accessing' stamp: 'HilaireFernandes 3/1/2006 10:26'!
subItems
	^subItems ! !

!DrGCompositeItem methodsFor: 'accessing' stamp: 'HilaireFernandes 3/1/2006 10:26'!
subItems: theSubItems
	subItems := theSubItems ! !

DrGMathItem subclass: #DrGDirectionItem
	instanceVariableNames: 'origin vector'
	classVariableNames: 'Inv2Pi InvPi TwoPi'
	poolDictionaries: ''
	category: 'DrGeoII-Core-Item'!

!DrGDirectionItem class methodsFor: 'class initialization' stamp: 'HilaireFernandes 2/22/2006 15:15'!
initialize
	InvPi := 0.31830988618379067154.
	Inv2Pi := 0.636619772367581.
	TwoPi	:= 6.28318530717958647692528676655900576! !

!DrGDirectionItem class methodsFor: 'constants' stamp: 'HilaireFernandes 2/22/2006 15:16'!
inv2Pi
	^Inv2Pi! !

!DrGDirectionItem class methodsFor: 'constants' stamp: 'HilaireFernandes 12/31/2005 09:25'!
invPi
	^InvPi! !

!DrGDirectionItem class methodsFor: 'constants' stamp: 'HilaireFernandes 1/16/2006 22:47'!
twoPi
	^TwoPi! !

!DrGDirectionItem methodsFor: 'accessing' stamp: 'HilaireFernandes 12/21/2005 16:34'!
direction
	^vector ! !

!DrGDirectionItem methodsFor: 'accessing' stamp: 'HilaireFernandes 12/26/2005 09:30'!
direction: aDirection
	vector := aDirection ! !

!DrGDirectionItem methodsFor: 'updating' stamp: 'HilaireFernandes 10/17/2010 11:43'!
homothetyUpdate
|center|
	self doParentsExist ifTrue:
		[center := parents second point.
		origin := center +
			(parents third valueItem * (parents first origin - center)).
		vector := parents first direction * parents third valueItem].! !

!DrGDirectionItem methodsFor: 'computing' stamp: 'HilaireFernandes 3/21/2006 10:44'!
intersectionWithDirection: aDirectionItem
| det c1 c2 p direction |
	direction := aDirectionItem  direction.
	det := vector crossProduct: direction.
	(det closeTo: 0) ifTrue: [^ nil].
	c1 := origin crossProduct: vector.
	c2 := aDirectionItem  origin crossProduct: direction.
	p := ((c1 negated * direction x + (c2 * vector x)) / det) @
		((vector y * c2 - (direction y * c1)) / det).
	^ p! !

!DrGDirectionItem methodsFor: 'testing' stamp: 'HilaireFernandes 4/17/2011 09:35'!
isCollinearWith: aDirection 
	^ (vector crossProduct: aDirection direction)
		closeTo: 0! !

!DrGDirectionItem methodsFor: '*DrGeoII-Core-testing' stamp: 'HilaireFernandes 1/25/2006 09:35'!
isDirectionItem
	^true! !

!DrGDirectionItem methodsFor: '*DrGeoII-Core-testing' stamp: 'HilaireFernandes 1/25/2006 09:36'!
isGeometricItem
	^true! !

!DrGDirectionItem methodsFor: 'accessing' stamp: 'HilaireFernandes 12/21/2005 16:37'!
normal
	^ vector normal! !

!DrGDirectionItem methodsFor: 'accessing' stamp: 'HilaireFernandes 12/21/2005 16:34'!
origin
	^origin ! !

!DrGDirectionItem methodsFor: 'accessing' stamp: 'HilaireFernandes 12/26/2005 09:30'!
origin: anOrigin
	origin := anOrigin ! !

!DrGDirectionItem methodsFor: 'updating' stamp: 'HilaireFernandes 10/17/2010 11:43'!
reflexionUpdate
	self doParentsExist ifTrue:
		[self origin: (parents first origin
							reflexionBy: parents second origin 
							direction: parents second direction).
		self direction: (parents first direction 
								vectorReflexionBy: parents second direction)].! !

!DrGDirectionItem methodsFor: 'updating' stamp: 'HilaireFernandes 10/17/2010 11:43'!
rotationUpdate
	self doParentsExist ifTrue:
		[origin := parents first origin
			rotateBy: parents third valueItem negated
			about: parents second point.
		vector := parents first direction 
			rotateVector: parents third valueItem].! !

!DrGDirectionItem methodsFor: 'updating' stamp: 'HilaireFernandes 10/17/2010 11:43'!
symmetryUpdate
	self doParentsExist ifTrue:
		[self origin: (2 * parents second point) - parents first origin.
		self direction: parents first direction negated].! !

!DrGDirectionItem methodsFor: 'updating' stamp: 'HilaireFernandes 10/17/2010 11:43'!
translationUpdate
	self doParentsExist ifTrue: 
		[self origin: parents first origin + parents second direction.
		self direction: parents first direction].! !

!DrGDirectionItem methodsFor: 'updating' stamp: 'HilaireFernandes 10/17/2010 11:43'!
update
	self doParentsExist ifTrue:
		["default behavior for segment, line and ray defined by 2 points"
		self origin: (parents at: 1) point.
		self direction: (parents at: 2) point - (self origin).
		self direction =( 0@0) ifTrue: [exist := false]].! !

DrGDirectionItem subclass: #DrGLineItem
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'DrGeoII-Core-Item'!

DrGLineItem subclass: #DrGLine2ptsItem
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'DrGeoII-Core-Item'!

!DrGLine2ptsItem methodsFor: 'xml writing' stamp: 'HilaireFernandes 1/20/2008 09:02'!
nodeType
	^#'2pts'! !

!DrGLine2ptsItem methodsFor: 'accessing' stamp: 'HilaireFernandes 1/17/2006 16:36'!
parentQty
	^2! !

DrGLineItem subclass: #DrGLineHomothetyItem
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'DrGeoII-Core-Item'!

!DrGLineHomothetyItem methodsFor: 'updating' stamp: 'HilaireFernandes 3/25/2006 15:42'!
moveParent: aPoint
	^false! !

!DrGLineHomothetyItem methodsFor: 'xml writing' stamp: 'HilaireFernandes 1/20/2008 11:58'!
nodeType
	^#Scale! !

!DrGLineHomothetyItem methodsFor: 'updating' stamp: 'HilaireFernandes 2/7/2006 22:54'!
update
	self homothetyUpdate ! !

!DrGLineItem methodsFor: 'computing' stamp: 'HilaireFernandes 1/24/2006 22:55'!
abscissaOf: aPoint 
	^(aPoint - origin dotProduct: vector normalized) arcTan 
		* DrGDirectionItem invPi + 0.5! !

!DrGLineItem methodsFor: 'private' stamp: 'jlrr 9/30/2007 14:22'!
adaptiveDescriptiveName
	^ 'This line %1' translated! !

!DrGLineItem methodsFor: 'accessing' stamp: 'HilaireFernandes 12/31/2005 10:04'!
basicType
	^#line! !

!DrGLineItem methodsFor: 'computing' stamp: 'HilaireFernandes 12/31/2005 11:41'!
closestPointTo: aPoint
	|unitVector|
	unitVector := vector normalized.
	^origin + (((aPoint - origin) dotProduct: unitVector) * unitVector)! !

!DrGLineItem methodsFor: 'computing' stamp: 'HilaireFernandes 2/9/2011 22:05'!
contains: aPoint
	^ (vector crossProduct: (aPoint - origin)) closeTo: 0! !

!DrGLineItem methodsFor: 'computing' stamp: 'HilaireFernandes 4/30/2010 22:18'!
intersectionWith: aCurveItem flag: aFlag
	aCurveItem isLineItem 
		ifTrue: [^ self intersectionWithDirection: aCurveItem].
	(aCurveItem isSegmentItem 
		or: [aCurveItem isRayItem 
			or: [aCurveItem isCircleItem
				or: [aCurveItem isArcItem]]])
			ifTrue: [^ aCurveItem intersectionWith: self flag: aFlag].! !

!DrGLineItem methodsFor: '*DrGeoII-Core-testing' stamp: 'HilaireFernandes 1/25/2006 09:36'!
isCurveItem
	^true! !

!DrGLineItem methodsFor: '*DrGeoII-Core-testing' stamp: 'HilaireFernandes 1/25/2006 09:36'!
isLineItem
	^true! !

!DrGLineItem methodsFor: 'computing' stamp: 'HilaireFernandes 5/13/2010 10:56'!
pointAt:	anAbscissa
	|point abscissa|
	"In the interval [-0.5 ; 0.5]"
	abscissa := anAbscissa - 0.5.
	(abscissa = 0.5 or: [abscissa = -0.5]) 
		ifTrue:
			[point := abscissa * vector.
			point x > 0 
				ifTrue: [point := Float infinity @ point y] 
				ifFalse: [point := 0 - Float infinity @ point y].
			point y > 0
				ifTrue: [point := point x @ Float infinity] 
				ifFalse: [point := point x @ (0 - Float infinity)]]
		ifFalse:
			[abscissa := abscissa * Float pi.
			point := origin + (abscissa tan * vector normalized)].
	^point! !

DrGLineItem subclass: #DrGLineParallelItem
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'DrGeoII-Core-Item'!

!DrGLineParallelItem methodsFor: 'private' stamp: 'jlrr 9/30/2007 15:03'!
adaptiveDescriptiveName
	^ 'This parallel line %1' translated! !

!DrGLineParallelItem methodsFor: 'updating' stamp: 'HilaireFernandes 1/24/2008 10:34'!
move: aPoint
	parents size = 1 ifTrue: [origin := origin + aPoint].
! !

!DrGLineParallelItem methodsFor: 'updating' stamp: 'HilaireFernandes 8/2/2011 14:28'!
moveParent: aPoint
"Only move the origin parent if it is a PointItem instance"
	parents isEmptyOrNil ifFalse: [parents second moveEvent: aPoint].
	^true! !

!DrGLineParallelItem methodsFor: 'xml writing' stamp: 'HilaireFernandes 1/20/2008 09:04'!
nodeType
	^#parallel! !

!DrGLineParallelItem methodsFor: 'accessing' stamp: 'HilaireFernandes 4/10/2010 12:38'!
parentQty 
	^ 2! !

!DrGLineParallelItem methodsFor: 'updating' stamp: 'HilaireFernandes 10/17/2010 11:43'!
update
	self doParentsExist ifTrue:
		[self direction: (parents at: 1) direction.
		parents size = 2 	ifTrue: [self origin: (parents at: 2) point]].
! !

DrGLineParallelItem subclass: #DrGLinePerpendicularItem
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'DrGeoII-Core-Item'!

!DrGLinePerpendicularItem methodsFor: 'private' stamp: 'jlrr 9/30/2007 15:03'!
adaptiveDescriptiveName
	^ 'This perpendicular line %1' translated! !

!DrGLinePerpendicularItem methodsFor: 'xml writing' stamp: 'HilaireFernandes 1/20/2008 09:05'!
nodeType
	^#perpendicular! !

!DrGLinePerpendicularItem methodsFor: 'updating' stamp: 'HilaireFernandes 10/17/2010 11:43'!
update
	self doParentsExist ifTrue:
		[self direction: (parents at: 1) normal.
		parents size = 2 	ifTrue: [self origin: (parents at: 2) point]].
! !

DrGLineItem subclass: #DrGLineReflexionItem
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'DrGeoII-Core-Item'!

!DrGLineReflexionItem methodsFor: 'updating' stamp: 'HilaireFernandes 3/25/2006 15:43'!
moveParent: aPoint
	^false! !

!DrGLineReflexionItem methodsFor: 'xml writing' stamp: 'HilaireFernandes 1/20/2008 12:00'!
nodeType
	^#Reflexion! !

!DrGLineReflexionItem methodsFor: 'updating' stamp: 'HilaireFernandes 1/3/2006 14:01'!
update
	self reflexionUpdate! !

DrGLineItem subclass: #DrGLineRotationItem
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'DrGeoII-Core-Item'!

!DrGLineRotationItem methodsFor: 'updating' stamp: 'HilaireFernandes 3/25/2006 15:43'!
moveParent: aPoint
	^false! !

!DrGLineRotationItem methodsFor: 'xml writing' stamp: 'HilaireFernandes 1/20/2008 12:01'!
nodeType
	^#Rotation! !

!DrGLineRotationItem methodsFor: 'updating' stamp: 'HilaireFernandes 2/7/2006 21:58'!
update
	self rotationUpdate ! !

DrGLineItem subclass: #DrGLineSymmetryItem
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'DrGeoII-Core-Item'!

!DrGLineSymmetryItem methodsFor: 'updating' stamp: 'HilaireFernandes 3/25/2006 15:43'!
moveParent: aPoint
	^false! !

!DrGLineSymmetryItem methodsFor: 'xml writing' stamp: 'HilaireFernandes 1/20/2008 12:01'!
nodeType
	^#Symmetry! !

!DrGLineSymmetryItem methodsFor: 'updating' stamp: 'HilaireFernandes 1/3/2006 17:20'!
update
	self symmetryUpdate ! !

DrGLineItem subclass: #DrGLineTranslationItem
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'DrGeoII-Core-Item'!

!DrGLineTranslationItem methodsFor: 'updating' stamp: 'HilaireFernandes 3/25/2006 15:43'!
moveParent: aPoint
	^false! !

!DrGLineTranslationItem methodsFor: 'xml writing' stamp: 'HilaireFernandes 1/20/2008 12:02'!
nodeType
	^#Translation! !

!DrGLineTranslationItem methodsFor: 'updating' stamp: 'HilaireFernandes 1/4/2006 06:56'!
update
	self translationUpdate! !

DrGDirectionItem subclass: #DrGRayItem
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'DrGeoII-Core-Item'!

DrGRayItem subclass: #DrGRay2ptsItem
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'DrGeoII-Core-Item'!

!DrGRay2ptsItem methodsFor: 'xml writing' stamp: 'HilaireFernandes 1/20/2008 09:08'!
nodeType
	^#'2pts'! !

!DrGRay2ptsItem methodsFor: 'accessing' stamp: 'HilaireFernandes 1/17/2006 16:36'!
parentQty
	^2! !

DrGRayItem subclass: #DrGRayHomothetyItem
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'DrGeoII-Core-Item'!

!DrGRayHomothetyItem methodsFor: 'updating' stamp: 'HilaireFernandes 3/25/2006 15:43'!
moveParent: aPoint
	^false! !

!DrGRayHomothetyItem methodsFor: 'xml writing' stamp: 'HilaireFernandes 1/20/2008 11:58'!
nodeType
	^#Scale! !

!DrGRayHomothetyItem methodsFor: 'updating' stamp: 'HilaireFernandes 2/7/2006 23:00'!
update
	self homothetyUpdate! !

!DrGRayItem methodsFor: 'computing' stamp: 'HilaireFernandes 2/22/2006 15:16'!
abscissaOf: aPoint 
	^(aPoint - origin dotProduct: vector normalized) arcTan 
		* DrGDirectionItem inv2Pi! !

!DrGRayItem methodsFor: 'private' stamp: 'jlrr 9/30/2007 15:03'!
adaptiveDescriptiveName
	^ 'This half-line %1' translated! !

!DrGRayItem methodsFor: 'accessing' stamp: 'HilaireFernandes 12/31/2005 11:31'!
basicType
	^#ray! !

!DrGRayItem methodsFor: 'computing' stamp: 'HilaireFernandes 12/31/2005 11:35'!
closestPointTo: aPoint
	|unitVector abscissa|
	unitVector := vector normalized.
	abscissa := aPoint - origin dotProduct: unitVector.
	abscissa < 0 ifTrue: [abscissa := 0].
	^origin + (abscissa * unitVector)! !

!DrGRayItem methodsFor: 'testing' stamp: 'HilaireFernandes 4/17/2011 10:32'!
contains: aPoint
	^ ((vector crossProduct: (aPoint - origin)) closeTo: 0) 
		and: [((aPoint - origin) dotProduct: vector) >= 0]! !

!DrGRayItem methodsFor: 'computing' stamp: 'HilaireFernandes 1/18/2008 11:25'!
intersectionWith: aCurveItem flag: aFlag
|aPoint|
	aCurveItem isLineItem 
		ifTrue: 
			[aPoint := self intersectionWithDirection: aCurveItem.
			(self contains: aPoint)
				ifTrue: [^aPoint]
				ifFalse: [^nil]].
	(aCurveItem isSegmentItem or: [aCurveItem isRayItem])
		ifTrue:
			[aPoint := self intersectionWithDirection: aCurveItem.
			((self contains: aPoint) and: [aCurveItem contains: aPoint])
				ifTrue: [^aPoint]
				ifFalse: [^nil]].
	(aCurveItem isCircleItem
		or: [aCurveItem isArcItem])
		ifTrue:
			[^aCurveItem intersectionWith: self flag: aFlag].! !

!DrGRayItem methodsFor: '*DrGeoII-Core-testing' stamp: 'HilaireFernandes 1/30/2006 16:16'!
isCurveItem
	^true! !

!DrGRayItem methodsFor: '*DrGeoII-Core-testing' stamp: 'HilaireFernandes 1/30/2006 16:15'!
isRayItem
	^true! !

!DrGRayItem methodsFor: 'computing' stamp: 'HilaireFernandes 2/22/2006 15:47'!
pointAt: anAbscissa
	|point|
	anAbscissa = 1 
		ifTrue:
			[|infX infY|
			point := vector.
			point x > 0 
				ifTrue: [infX := Float infinity] 
				ifFalse: [infX := 0 - Float infinity].
			point y > 0
				ifTrue: [infY :=	Float infinity] 
				ifFalse: [infY := 0 - Float infinity].
			point := infX@infY]
		ifFalse:
			[anAbscissa = 0
				ifTrue: [point := origin]
				ifFalse:
					[point := origin + ((anAbscissa * Float halfPi) tan * vector normalized)]].
	^point
! !

DrGRayItem subclass: #DrGRayReflexionItem
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'DrGeoII-Core-Item'!

!DrGRayReflexionItem methodsFor: 'updating' stamp: 'HilaireFernandes 3/25/2006 15:44'!
moveParent: aPoint
	^false! !

!DrGRayReflexionItem methodsFor: 'xml writing' stamp: 'HilaireFernandes 1/20/2008 12:00'!
nodeType
	^#Reflexion! !

!DrGRayReflexionItem methodsFor: 'updating' stamp: 'HilaireFernandes 1/3/2006 16:00'!
update
	self reflexionUpdate ! !

DrGRayItem subclass: #DrGRayRotationItem
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'DrGeoII-Core-Item'!

!DrGRayRotationItem methodsFor: 'updating' stamp: 'HilaireFernandes 3/25/2006 15:44'!
moveParent: aPoint
	^false! !

!DrGRayRotationItem methodsFor: 'xml writing' stamp: 'HilaireFernandes 1/20/2008 12:01'!
nodeType
	^#Rotation! !

!DrGRayRotationItem methodsFor: 'updating' stamp: 'HilaireFernandes 2/7/2006 22:01'!
update
	self rotationUpdate ! !

DrGRayItem subclass: #DrGRaySymmetryItem
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'DrGeoII-Core-Item'!

!DrGRaySymmetryItem methodsFor: 'updating' stamp: 'HilaireFernandes 3/25/2006 15:44'!
moveParent: aPoint
	^false! !

!DrGRaySymmetryItem methodsFor: 'xml writing' stamp: 'HilaireFernandes 1/20/2008 12:02'!
nodeType
	^#Symmetry! !

!DrGRaySymmetryItem methodsFor: 'updating' stamp: 'HilaireFernandes 1/3/2006 17:23'!
update
	self symmetryUpdate ! !

DrGRayItem subclass: #DrGRayTranslationItem
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'DrGeoII-Core-Item'!

!DrGRayTranslationItem methodsFor: 'updating' stamp: 'HilaireFernandes 3/25/2006 15:44'!
moveParent: aPoint
	^false! !

!DrGRayTranslationItem methodsFor: 'xml writing' stamp: 'HilaireFernandes 1/20/2008 12:02'!
nodeType
	^#Translation! !

!DrGRayTranslationItem methodsFor: 'updating' stamp: 'HilaireFernandes 1/4/2006 07:03'!
update
	self translationUpdate ! !

DrGDirectionItem subclass: #DrGSegmentItem
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'DrGeoII-Core-Item'!

DrGSegmentItem subclass: #DrGSegment2ptsItem
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'DrGeoII-Core-Item'!

!DrGSegment2ptsItem methodsFor: 'xml writing' stamp: 'HilaireFernandes 1/20/2008 12:01'!
nodeType
	^#'2pts'! !

!DrGSegment2ptsItem methodsFor: 'accessing' stamp: 'HilaireFernandes 1/17/2006 16:36'!
parentQty
	^2! !

!DrGSegment2ptsItem methodsFor: 'comparing' stamp: 'HilaireFernandes 4/18/2011 22:12'!
parentsEqual: aMathItem 
	^ self parents asSet = aMathItem parents asSet! !

DrGSegmentItem subclass: #DrGSegmentHomothetyItem
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'DrGeoII-Core-Item'!

!DrGSegmentHomothetyItem methodsFor: 'updating' stamp: 'HilaireFernandes 3/25/2006 15:44'!
moveParent: aPoint
	^false! !

!DrGSegmentHomothetyItem methodsFor: 'xml writing' stamp: 'HilaireFernandes 1/20/2008 11:58'!
nodeType
	^#Scale! !

!DrGSegmentHomothetyItem methodsFor: 'updating' stamp: 'HilaireFernandes 2/7/2006 23:00'!
update
	self homothetyUpdate ! !

!DrGSegmentItem methodsFor: 'computing' stamp: 'HilaireFernandes 12/31/2005 11:40'!
abscissaOf: aPoint
	^((aPoint - origin) dotProduct: vector) / (vector dotProduct: vector)! !

!DrGSegmentItem methodsFor: 'private' stamp: 'jlrr 9/30/2007 15:07'!
adaptiveDescriptiveName
	^ 'This segment %1' translated! !

!DrGSegmentItem methodsFor: 'accessing' stamp: 'HilaireFernandes 12/21/2005 16:33'!
basicType
	^#segment! !

!DrGSegmentItem methodsFor: 'computing' stamp: 'HilaireFernandes 3/13/2006 16:16'!
closestPointTo: aPoint
	|abscissa squareNorm|
	abscissa := (aPoint - origin) dotProduct: vector.
	squareNorm := vector dotProduct: vector.
	abscissa < 0 ifTrue: [abscissa := 0].
	abscissa > squareNorm ifTrue: 
		[abscissa := squareNorm].
	^ origin + (abscissa * vector / squareNorm asFloat)! !

!DrGSegmentItem methodsFor: 'testing' stamp: 'HilaireFernandes 7/24/2010 19:29'!
contains: aPoint
|anAbscissa|
	^ ((vector crossProduct: (aPoint - origin)) closeTo: 0) 
		and: [(anAbscissa := self abscissaOf: aPoint) >= 0
			and: [anAbscissa <= 1]]! !

!DrGSegmentItem methodsFor: 'computing' stamp: 'HilaireFernandes 11/20/2010 09:48'!
intersectionWith: aCurveItem flag: aFlag
|aPoint|
	aCurveItem isLineItem
		ifTrue:
			[aPoint := self intersectionWithDirection: aCurveItem.
			aPoint ifNil: [^nil].
			(self contains: aPoint)
				ifTrue: [^aPoint]
				ifFalse: [^nil]].
	aCurveItem isSegmentItem
		ifTrue:
			[aPoint := self intersectionWithDirection: aCurveItem.
			aPoint ifNil: [^nil].
			((self contains: aPoint) and: [aCurveItem contains: aPoint])
				ifTrue: [^aPoint]
				ifFalse: [^nil]].
	(aCurveItem isRayItem 
		or: [aCurveItem isCircleItem
			or: [aCurveItem isArcItem]])
		ifTrue:
			[^aCurveItem intersectionWith: self flag: aFlag].
			! !

!DrGSegmentItem methodsFor: '*DrGeoII-Core-testing' stamp: 'HilaireFernandes 3/21/2010 20:37'!
isCurveItem
	^true! !

!DrGSegmentItem methodsFor: '*DrGeoII-Core-testing' stamp: 'HilaireFernandes 1/25/2006 09:51'!
isSegmentItem
	^true! !

!DrGSegmentItem methodsFor: 'computing' stamp: 'HilaireFernandes 12/31/2005 11:40'!
pointAt: anAbscissa
	^ origin + (anAbscissa * vector)! !

DrGSegmentItem subclass: #DrGSegmentReflexionItem
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'DrGeoII-Core-Item'!

!DrGSegmentReflexionItem methodsFor: 'updating' stamp: 'HilaireFernandes 3/25/2006 15:44'!
moveParent: aPoint
	^false! !

!DrGSegmentReflexionItem methodsFor: 'xml writing' stamp: 'HilaireFernandes 1/20/2008 12:01'!
nodeType
	^#Reflexion! !

!DrGSegmentReflexionItem methodsFor: 'updating' stamp: 'HilaireFernandes 1/3/2006 16:02'!
update
	self reflexionUpdate ! !

DrGSegmentItem subclass: #DrGSegmentRotationItem
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'DrGeoII-Core-Item'!

!DrGSegmentRotationItem methodsFor: 'updating' stamp: 'HilaireFernandes 3/25/2006 15:44'!
moveParent: aPoint
	^false! !

!DrGSegmentRotationItem methodsFor: 'xml writing' stamp: 'HilaireFernandes 1/20/2008 12:01'!
nodeType
	^#Rotation! !

!DrGSegmentRotationItem methodsFor: 'updating' stamp: 'HilaireFernandes 2/7/2006 22:02'!
update
	self rotationUpdate ! !

DrGSegmentItem subclass: #DrGSegmentSymmetryItem
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'DrGeoII-Core-Item'!

!DrGSegmentSymmetryItem methodsFor: 'updating' stamp: 'HilaireFernandes 3/25/2006 15:44'!
moveParent: aPoint
	^false! !

!DrGSegmentSymmetryItem methodsFor: 'xml writing' stamp: 'HilaireFernandes 1/20/2008 12:02'!
nodeType
	^#Symmetry! !

!DrGSegmentSymmetryItem methodsFor: 'updating' stamp: 'HilaireFernandes 1/3/2006 17:24'!
update
	self symmetryUpdate ! !

DrGSegmentItem subclass: #DrGSegmentTranslationItem
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'DrGeoII-Core-Item'!

!DrGSegmentTranslationItem methodsFor: 'updating' stamp: 'HilaireFernandes 3/25/2006 15:44'!
moveParent: aPoint
	^false! !

!DrGSegmentTranslationItem methodsFor: 'xml writing' stamp: 'HilaireFernandes 1/20/2008 12:02'!
nodeType
	^#Translation! !

!DrGSegmentTranslationItem methodsFor: 'updating' stamp: 'HilaireFernandes 1/4/2006 07:02'!
update
	self translationUpdate ! !

DrGSegmentItem subclass: #DrGVectorItem
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'DrGeoII-Core-Item'!

DrGVectorItem subclass: #DrGVector2ptsItem
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'DrGeoII-Core-Item'!

!DrGVector2ptsItem methodsFor: 'initialize-release' stamp: 'HilaireFernandes 1/2/2006 09:43'!
initialize: theParents
	super initialize: theParents.
	self origin: (theParents first point).! !

!DrGVector2ptsItem methodsFor: 'xml writing' stamp: 'HilaireFernandes 1/20/2008 09:22'!
nodeType
	^#'2pts'! !

!DrGVector2ptsItem methodsFor: 'accessing' stamp: 'HilaireFernandes 1/17/2006 16:36'!
parentQty
	^2! !

!DrGVector2ptsItem methodsFor: 'updating' stamp: 'HilaireFernandes 10/17/2010 11:43'!
update
	self doParentsExist ifTrue:
		[self direction: (parents at: 2) point - (parents at: 1) point].
! !

DrGVectorItem subclass: #DrGVectorHomothetyItem
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'DrGeoII-Core-Item'!

!DrGVectorHomothetyItem methodsFor: 'initialize-release' stamp: 'HilaireFernandes 2/7/2006 23:07'!
initialize: theParents
	super initialize: theParents.
	self origin: parents second point.! !

!DrGVectorHomothetyItem methodsFor: 'updating' stamp: 'HilaireFernandes 3/25/2006 15:44'!
moveParent: aPoint
	^false! !

!DrGVectorHomothetyItem methodsFor: 'xml writing' stamp: 'HilaireFernandes 1/20/2008 11:58'!
nodeType
	^#Scale! !

!DrGVectorHomothetyItem methodsFor: 'updating' stamp: 'HilaireFernandes 10/17/2010 11:43'!
update
	self doParentsExist ifTrue:
		[vector := parents first direction * parents third valueItem].! !

!DrGVectorItem methodsFor: 'private' stamp: 'jlrr 9/30/2007 15:07'!
adaptiveDescriptiveName
	^ 'This vector %1' translated! !

!DrGVectorItem methodsFor: 'accessing' stamp: 'HilaireFernandes 1/2/2006 09:06'!
basicType
	^#vector! !

!DrGVectorItem methodsFor: '*DrGeoII-Core-testing' stamp: 'HilaireFernandes 3/13/2006 15:09'!
isCurveItem
	^false! !

!DrGVectorItem methodsFor: '*DrGeoII-Core-testing' stamp: 'HilaireFernandes 1/28/2008 16:36'!
isSegmentItem
	^false! !

!DrGVectorItem methodsFor: '*DrGeoII-Core-testing' stamp: 'HilaireFernandes 1/25/2006 09:39'!
isVectorItem
	^true! !

!DrGVectorItem methodsFor: 'updating' stamp: 'HilaireFernandes 1/24/2008 10:36'!
move: aPoint
	origin := origin + aPoint.
! !

!DrGVectorItem methodsFor: 'updating' stamp: 'HilaireFernandes 3/25/2006 15:44'!
moveParent: aPoint
"We don't want to move the parents, as the whole vector representation can be move all arround"
	^false! !

!DrGVectorItem methodsFor: 'as yet unclassified' stamp: 'HilaireFernandes 11/1/2010 22:10'!
writeAsXmlTo: aNode
	|node|
	node := super writeAsXmlTo: aNode.
	node attributeAt: #origin put: origin asString.
	^ node! !

DrGVectorItem subclass: #DrGVectorReflexionItem
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'DrGeoII-Core-Item'!

!DrGVectorReflexionItem methodsFor: 'initialize-release' stamp: 'HilaireFernandes 1/3/2006 16:26'!
initialize: theParents
	super initialize: theParents.
	self origin: parents first origin.! !

!DrGVectorReflexionItem methodsFor: 'updating' stamp: 'HilaireFernandes 3/25/2006 15:44'!
moveParent: aPoint
	^false! !

!DrGVectorReflexionItem methodsFor: 'xml writing' stamp: 'HilaireFernandes 1/20/2008 12:01'!
nodeType
	^#Reflexion! !

!DrGVectorReflexionItem methodsFor: 'updating' stamp: 'HilaireFernandes 10/17/2010 11:43'!
update
	self doParentsExist ifTrue:
		[self direction: (parents first direction 
								vectorReflexionBy: parents second direction)].
! !

DrGVectorItem subclass: #DrGVectorRotationItem
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'DrGeoII-Core-Item'!

!DrGVectorRotationItem methodsFor: 'initialize-release' stamp: 'HilaireFernandes 2/7/2006 22:03'!
initialize: theParents
	super initialize: theParents.
	self origin: parents first origin.! !

!DrGVectorRotationItem methodsFor: 'updating' stamp: 'HilaireFernandes 3/25/2006 15:44'!
moveParent: aPoint
	^false! !

!DrGVectorRotationItem methodsFor: 'xml writing' stamp: 'HilaireFernandes 1/20/2008 12:01'!
nodeType
	^#Rotation! !

!DrGVectorRotationItem methodsFor: 'updating' stamp: 'HilaireFernandes 10/17/2010 11:43'!
update
	self doParentsExist ifTrue:
		[vector := parents first direction 
			rotateVector: parents third valueItem].
! !

DrGVectorItem subclass: #DrGVectorSymmetryItem
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'DrGeoII-Core-Item'!

!DrGVectorSymmetryItem methodsFor: 'initialize-release' stamp: 'HilaireFernandes 1/3/2006 17:25'!
initialize: theParents
	super initialize: theParents.
	self origin: (parents first origin)! !

!DrGVectorSymmetryItem methodsFor: 'updating' stamp: 'HilaireFernandes 3/25/2006 15:44'!
moveParent: aPoint
	^false! !

!DrGVectorSymmetryItem methodsFor: 'xml writing' stamp: 'HilaireFernandes 1/20/2008 12:02'!
nodeType
	^#Symmetry! !

!DrGVectorSymmetryItem methodsFor: 'updating' stamp: 'HilaireFernandes 10/17/2010 11:43'!
update
	self doParentsExist ifTrue:
		[self direction: parents first direction negated].
! !

DrGMathItem subclass: #DrGLocus2ptsItem
	instanceVariableNames: 'sampleNumber p'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'DrGeoII-Core-Item'!

!DrGLocus2ptsItem methodsFor: 'accessing' stamp: 'jlrr 11/13/2007 12:08'!
abscissaOf: aPoint 
	| i distance |
	distance := self squareNorm: aPoint
					- (p at: 1).
	i := 1.
	2
		to: sampleNumber
		do: [:n | (self squareNorm: (p at: n)
						- aPoint)
					< distance
				ifTrue: [distance := self squareNorm: aPoint
									- (p at: n).
					i := n]].
	^ (i / sampleNumber) asFloat! !

!DrGLocus2ptsItem methodsFor: 'private' stamp: 'jlrr 11/13/2007 12:08'!
adaptiveDescriptiveName
	^ 'This locus %1' translated! !

!DrGLocus2ptsItem methodsFor: 'accessing' stamp: 'jlrr 11/13/2007 12:09'!
basicType
	^ #locus! !

!DrGLocus2ptsItem methodsFor: 'accessing' stamp: 'jlrr 11/13/2007 12:09'!
closestPointTo: aPoint 
	| distance d m |
	m := p at: 1.
	distance := self squareNorm: aPoint - m.
	2
		to: sampleNumber
		do: [:n | 
			d := self squareNorm: (p at: n)
							- aPoint.
			d < distance
				ifTrue: [m := p at: n.
					distance := d]].
	^ m! !

!DrGLocus2ptsItem methodsFor: 'accessing' stamp: 'jlrr 11/13/2007 12:09'!
constrainedPoint
	^ parents second! !

!DrGLocus2ptsItem methodsFor: 'accessing' stamp: 'jlrr 11/13/2007 12:09'!
freePoint
	^ parents first! !

!DrGLocus2ptsItem methodsFor: '*DrGeoII-Core-testing' stamp: 'jlrr 11/13/2007 12:10'!
isCurveItem
	^ true! !

!DrGLocus2ptsItem methodsFor: '*DrGeoII-Core-testing' stamp: 'jlrr 11/13/2007 12:10'!
isGeometricItem
	^ true! !

!DrGLocus2ptsItem methodsFor: '*DrGeoII-Core-testing' stamp: 'HilaireFernandes 1/25/2008 22:59'!
isLocusItem
	^ true! !

!DrGLocus2ptsItem methodsFor: 'xml writing' stamp: 'HilaireFernandes 10/16/2010 16:07'!
nodeType
	^#'2pts'! !

!DrGLocus2ptsItem methodsFor: 'accessing' stamp: 'jlrr 11/13/2007 12:09'!
parentQty
	^ 2! !

!DrGLocus2ptsItem methodsFor: 'accessing' stamp: 'jlrr 11/13/2007 12:09'!
pointAt: anAbscissa 
	| m |
	anAbscissa = 1.0
		ifTrue: [m := p at: sampleNumber]
		ifFalse: [m := p at: (anAbscissa * sampleNumber) rounded].
	^ m! !

!DrGLocus2ptsItem methodsFor: 'accessing' stamp: 'jlrr 11/13/2007 12:09'!
points: aCollection 
	p := aCollection! !

!DrGLocus2ptsItem methodsFor: 'accessing' stamp: 'jlrr 11/13/2007 12:09'!
samples: aNumber 
	sampleNumber := aNumber! !

!DrGLocus2ptsItem methodsFor: 'private' stamp: 'jlrr 11/13/2007 12:08'!
squareNorm: aPoint 
	^ aPoint x * aPoint x + (aPoint y * aPoint y)! !

!DrGLocus2ptsItem methodsFor: 'updating' stamp: 'HilaireFernandes 1/24/2008 10:33'!
update! !

!DrGMathItem class methodsFor: 'as yet unclassified' stamp: 'HilaireFernandes 11/8/2010 22:45'!
basicNewWith: theParents
	"Instantiate this mathItem with these parents, but do not update it yet.
	Parents contains other MathItem instance, sometime it contains other object as Point"
	^self new 
		parents: theParents; 
		initialize: theParents! !

!DrGMathItem class methodsFor: 'as yet unclassified' stamp: 'HilaireFernandes 12/29/2005 09:49'!
newWith: theParents
	"Instantiate this mathItem with these parents
	Parents contains other MathItem instance, sometime it contains other object as Point"
	^self new 
		parents: theParents; 
		initialize: theParents;
		update! !

!DrGMathItem methodsFor: 'comparing' stamp: 'HilaireFernandes 4/18/2011 22:13'!
= aMathItem 
	^ aMathItem
		ifNil: [false]
		ifNotNil: [self basicType = aMathItem basicType
				and: [self nodeType = aMathItem nodeType
						and: [self parentsEqual: aMathItem]]]! !

!DrGMathItem methodsFor: 'computing' stamp: 'HilaireFernandes 12/29/2005 09:27'!
abscissaOf: aPoint
	self subclassResponsibility! !

!DrGMathItem methodsFor: 'event' stamp: 'HilaireFernandes 3/22/2006 12:05'!
actionMap
	actionMap ifNil: [^self createActionMap].
	^ actionMap ! !

!DrGMathItem methodsFor: 'event' stamp: 'HilaireFernandes 11/27/2011 12:00'!
actionMap: anObject
	actionMap := anObject ! !

!DrGMathItem methodsFor: 'private' stamp: 'jlrr 9/30/2007 15:16'!
adaptiveDescriptiveName
	^ 'This math item %1' translated! !

!DrGMathItem methodsFor: 'accessing' stamp: 'HilaireFernandes 12/19/2005 15:20'!
basicType
	^#mathItem! !

!DrGMathItem methodsFor: 'updating' stamp: 'HilaireFernandes 10/26/2011 23:40'!
changed
	self triggerEvent: #updateMyView! !

!DrGMathItem methodsFor: 'computing' stamp: 'HilaireFernandes 12/29/2005 09:26'!
closestPointTo: aPoint
	self subclassResponsibility ! !

!DrGMathItem methodsFor: 'testing' stamp: 'HilaireFernandes 1/12/2006 17:28'!
contains: aPoint
	self subclassResponsibility ! !

!DrGMathItem methodsFor: 'accessing' stamp: 'HilaireFernandes 8/4/2008 13:15'!
descriptiveName
	^ self adaptiveDescriptiveName 
		copyReplaceAll: '%1'
		with: self safeName! !

!DrGMathItem methodsFor: 'accessing' stamp: 'HilaireFernandes 7/22/2011 15:12'!
dirty: boolean
	dirty := boolean ! !

!DrGMathItem methodsFor: 'testing' stamp: 'HilaireFernandes 10/29/2010 09:15'!
doParentsExist
	exist := true.
	parents 
		ifNotNil: 
			[parents do: [:parent |
			parent exist ifFalse: [exist := false]]]
		ifNil: [exist := false].
	^exist
				! !

!DrGMathItem methodsFor: 'accessing' stamp: 'HilaireFernandes 1/12/2006 17:12'!
exist
	^exist! !

!DrGMathItem methodsFor: 'comparing' stamp: 'HilaireFernandes 1/28/2008 22:44'!
hash
	^ (self parents hash bitXor: self nodeType hash) bitXor: self basicType hash! !

!DrGMathItem methodsFor: 'accessing' stamp: 'HilaireFernandes 1/14/2008 22:45'!
id
	^id! !

!DrGMathItem methodsFor: 'accessing' stamp: 'HilaireFernandes 1/14/2008 22:46'!
id: aSymbol
	id := aSymbol! !

!DrGMathItem methodsFor: 'initialize-release' stamp: 'HilaireFernandes 11/6/2011 21:42'!
initialize
	parents := nil.
	exist := true.! !

!DrGMathItem methodsFor: 'initialize-release' stamp: 'HilaireFernandes 12/29/2005 10:20'!
initialize: theParents
! !

!DrGMathItem methodsFor: 'computing' stamp: 'HilaireFernandes 1/12/2006 16:51'!
intersectionWith: aCurveItem flag: aFlag
	self subclassResponsibility ! !

!DrGMathItem methodsFor: 'testing' stamp: 'HilaireFernandes 7/22/2011 15:12'!
isDirty
	^ dirty! !

!DrGMathItem methodsFor: 'testing' stamp: 'HilaireFernandes 10/27/2011 10:01'!
isPropertyEditable
	^ false! !

!DrGMathItem methodsFor: 'accessing' stamp: 'HilaireFernandes 1/12/2006 10:25'!
longDescription
	^'A long description of this mathematic item'! !

!DrGMathItem methodsFor: 'updating' stamp: 'HilaireFernandes 7/7/2011 19:11'!
move: aPoint
! !

!DrGMathItem methodsFor: 'updating' stamp: 'HilaireFernandes 1/23/2008 11:49'!
moveAt: aPoint
	self shouldNotImplement! !

!DrGMathItem methodsFor: 'updating' stamp: 'HilaireFernandes 1/23/2008 11:04'!
moveEvent: aPoint
	"are we dirty?"
	"self dirty ifFalse: [^self]."
	(self moveParent: aPoint)
		ifFalse:
			["parents are not moving, we can upate ourself"
			self move: aPoint].
	"notify the dependant mathItems"
	"self triggerEvent: #movedMathItem."! !

!DrGMathItem methodsFor: 'updating' stamp: 'HilaireFernandes 1/23/2008 11:39'!
moveParent: aPoint
"By default we try reverse dragging (move parents).
This behaviour should be overridden for some mathItem, for example a free point on a curve should not reverse dragging the curve it belongs to.
Return true if we are doing reverse dragging, otherwise false"
	parents ifNotNil:
		[parents do: [:parent|
			parent moveEvent: aPoint]].
	^true.! !

!DrGMathItem methodsFor: 'accessing' stamp: 'HilaireFernandes 1/12/2006 10:18'!
name
	^name! !

!DrGMathItem methodsFor: 'accessing' stamp: 'HilaireFernandes 1/11/2006 10:42'!
name: aString
	name := aString ! !

!DrGMathItem methodsFor: 'xml writing' stamp: 'HilaireFernandes 1/23/2008 12:08'!
nodeType
	^ #mathItem! !

!DrGMathItem methodsFor: 'accessing' stamp: 'HilaireFernandes 4/18/2011 22:51'!
parentQty
	"how many parents do need this item to be fully defined"
	self subclassResponsibility ! !

!DrGMathItem methodsFor: 'accessing' stamp: 'HilaireFernandes 12/6/2005 16:00'!
parents
	^ parents ! !

!DrGMathItem methodsFor: 'accessing' stamp: 'HilaireFernandes 1/23/2008 11:42'!
parents: aCollection
	parents := aCollection.
! !

!DrGMathItem methodsFor: 'comparing' stamp: 'HilaireFernandes 4/18/2011 22:12'!
parentsEqual: aMathItem 
	^ self parents = aMathItem parents! !

!DrGMathItem methodsFor: 'computing' stamp: 'HilaireFernandes 12/29/2005 09:27'!
pointAt: anAbscissa
	self subclassResponsibility ! !

!DrGMathItem methodsFor: 'accessing' stamp: 'HilaireFernandes 11/5/2011 18:10'!
position
	self shouldBeImplemented.! !

!DrGMathItem methodsFor: 'updating' stamp: 'HilaireFernandes 10/27/2011 11:34'!
property
	self subclassResponsibility ! !

!DrGMathItem methodsFor: 'updating' stamp: 'HilaireFernandes 10/27/2011 11:34'!
property: aProperty 
	self subclassResponsibility ! !

!DrGMathItem methodsFor: 'initialize-release' stamp: 'HilaireFernandes 11/27/2011 12:00'!
release
	self triggerEvent: #deletedMathItem.
	"Do it last otherwise our deletedMathItem will never be sent"
	super release.! !

!DrGMathItem methodsFor: 'event' stamp: 'HilaireFernandes 3/22/2006 16:54'!
releaseActionMap
	actionMap := nil! !

!DrGMathItem methodsFor: 'accessing' stamp: 'HilaireFernandes 9/26/2008 20:57'!
safeName
	^ (name isNil or: [name isEmpty]) 
		ifTrue: [''] 
		ifFalse: [ name ]! !

!DrGMathItem methodsFor: 'updating' stamp: 'HilaireFernandes 1/24/2008 10:33'!
update
! !

!DrGMathItem methodsFor: 'event' stamp: 'HilaireFernandes 3/22/2006 12:04'!
updateableActionMap
	actionMap ifNil:
		[actionMap := self createActionMap].
	^actionMap! !

!DrGMathItem methodsFor: 'xml writing' stamp: 'HilaireFernandes 10/31/2010 14:47'!
writeAsXmlTo: aNode
	"return the newly created element"
	| node |
	node :=  XMLElement named: self basicType attributes: Dictionary new.
	node attributeAt: #type put: self nodeType;
		attributeAt: #name put: (name ifNil: ['']);
		attributeAt: #id put: self hash asString.
	self writeParentsAsXmlTo: node.
	aNode addElement: node.
	^node! !

!DrGMathItem methodsFor: 'xml writing' stamp: 'HilaireFernandes 1/20/2008 11:48'!
writeParents: collection asXmlTo: aNode
"write parents reference in the natural order"
|attr|
	collection do: [:each | 
		attr := Dictionary new.
		attr at: #ref put: each hash asString.
		aNode addElement: (XMLElement named: #parent attributes: attr)]! !

!DrGMathItem methodsFor: 'xml writing' stamp: 'HilaireFernandes 9/23/2010 20:58'!
writeParentsAsXmlTo: aNode
	parents ifNotNil:
		[self writeParents: parents asXmlTo: aNode]! !

DrGMathItem subclass: #DrGPointItem
	instanceVariableNames: 'point'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'DrGeoII-Core-Item'!

DrGPointItem subclass: #DrGPointCoordinateItem
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'DrGeoII-Core-Item'!

DrGPointCoordinateItem subclass: #DrGPointBlockItem
	instanceVariableNames: 'pointBlock'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'DrGeoII-Core-Item'!

!DrGPointBlockItem commentStamp: '<historical>' prior: 0!
Point item defined by a block closure!

!DrGPointBlockItem methodsFor: 'testing' stamp: 'HilaireFernandes 2/4/2011 14:32'!
doParentsExist 
	^ super doParentsExist and: [pointBlock isNil not]! !

!DrGPointBlockItem methodsFor: 'xml writing' stamp: 'HilaireFernandes 2/4/2011 14:26'!
nodeType
	^#Block! !

!DrGPointBlockItem methodsFor: 'accessing' stamp: 'HilaireFernandes 7/20/2011 17:59'!
parents: aCollection
	aCollection isEmptyOrNil ifTrue: [
		parents := aCollection.
		^ self].
	aCollection first isBlock ifTrue: [
		self pointBlock: aCollection first.
		parents := aCollection allButFirst]
	ifFalse: [parents := aCollection]! !

!DrGPointBlockItem methodsFor: 'accessing' stamp: 'HilaireFernandes 2/3/2011 22:41'!
pointBlock: aBlock
	pointBlock := aBlock ! !

!DrGPointBlockItem methodsFor: 'updating' stamp: 'HilaireFernandes 2/4/2011 15:20'!
update
	self doParentsExist ifTrue:
		[parents size = 1
			ifTrue: [self point: (pointBlock value: parents first )] "To reduce indirection in the bc"
			ifFalse: [self point: (pointBlock value: parents)]]. 
! !

!DrGPointBlockItem methodsFor: 'as yet unclassified' stamp: 'HilaireFernandes 2/4/2011 15:57'!
writeAsXmlTo: aNode
	|node|
	node := super writeAsXmlTo: aNode.
	node addElement: ((XMLElement named: #code) addContent: (XMLStringNode string: pointBlock asString)).
	^node! !

!DrGPointCoordinateItem methodsFor: '*DrGeoII-Core-testing' stamp: 'HilaireFernandes 11/13/2007 20:28'!
isConstrainedPointItem
	^true! !

!DrGPointCoordinateItem methodsFor: 'xml writing' stamp: 'HilaireFernandes 1/20/2008 12:00'!
nodeType
	^#Coordinate! !

!DrGPointCoordinateItem methodsFor: 'updating' stamp: 'HilaireFernandes 10/17/2010 11:43'!
update
	self doParentsExist ifTrue:
		[self point: parents first valueItem @ parents second valueItem]. 
! !

DrGPointCoordinateItem subclass: #DrGPointScriptItem
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'DrGeoII-Core-Item'!

!DrGPointScriptItem commentStamp: 'HilaireFernandes 8/6/2008 13:40' prior: 0!
A DrGPointByScriptItem is definied by the returned value of a script.
The script valueItem should understand asPoint.

Instance Variables
!

!DrGPointScriptItem methodsFor: 'xml writing' stamp: 'HilaireFernandes 8/6/2008 13:43'!
nodeType
	^#Script! !

!DrGPointScriptItem methodsFor: 'updating' stamp: 'HilaireFernandes 10/17/2010 11:43'!
update
	self doParentsExist ifTrue:
		[self point: parents first valueItem asPoint]. 
! !

DrGPointItem subclass: #DrGPointFreeItem
	instanceVariableNames: 'locked'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'DrGeoII-Core-Item'!

!DrGPointFreeItem commentStamp: 'HilaireFernandes 11/27/2011 10:56' prior: 0!
I represent a free point in the plane!

!DrGPointFreeItem methodsFor: 'testing' stamp: 'HilaireFernandes 10/29/2010 09:39'!
doParentsExist
	^ exist := true.! !

!DrGPointFreeItem methodsFor: 'comparing' stamp: 'HilaireFernandes 12/21/2005 14:51'!
hash
	^ super hash bitXor: point hash! !

!DrGPointFreeItem methodsFor: 'initialize-release' stamp: 'HilaireFernandes 12/29/2005 09:44'!
initialize: theParents
	super initialize: theParents.
	self point: theParents first.! !

!DrGPointFreeItem methodsFor: '*DrGeoII-Core-testing' stamp: 'HilaireFernandes 3/10/2006 19:26'!
isFreePointItem
	^true! !

!DrGPointFreeItem methodsFor: 'accessing' stamp: 'HilaireFernandes 11/6/2011 21:44'!
isLocked
	^ locked ifNil: [locked := false]! !

!DrGPointFreeItem methodsFor: 'accessing' stamp: 'HilaireFernandes 11/6/2011 21:46'!
locked: aBoolean
	locked := aBoolean ! !

!DrGPointFreeItem methodsFor: 'updating' stamp: 'HilaireFernandes 11/6/2011 21:45'!
move: aPoint
	self isLocked ifFalse: [point := point + aPoint]
	! !

!DrGPointFreeItem methodsFor: 'updating' stamp: 'HilaireFernandes 11/6/2011 21:45'!
moveAt: aPoint
	self isLocked ifFalse: [point := aPoint]
	! !

!DrGPointFreeItem methodsFor: 'xml writing' stamp: 'HilaireFernandes 4/22/2010 07:59'!
nodeType
	^#Free! !

!DrGPointFreeItem methodsFor: 'accessing' stamp: 'HilaireFernandes 12/29/2005 09:48'!
parents: aCollection
	parents := nil! !

!DrGPointFreeItem methodsFor: 'comparing' stamp: 'HilaireFernandes 4/18/2011 22:11'!
parentsEqual: aMathItem
	^ self point = aMathItem point! !

!DrGPointFreeItem methodsFor: 'as yet unclassified' stamp: 'HilaireFernandes 10/27/2011 11:37'!
property 
	^ self point! !

!DrGPointFreeItem methodsFor: 'as yet unclassified' stamp: 'HilaireFernandes 10/27/2011 11:38'!
property: aProperty 
	self point: aProperty ! !

!DrGPointFreeItem methodsFor: 'xml writing' stamp: 'HilaireFernandes 10/31/2010 15:11'!
writeAsXmlTo: aNode
	"It is important to return the node"
	| node|
	node :=  super writeAsXmlTo: aNode.
	node addElement:  ((XMLElement named: #x) addContent: (XMLStringNode string: point x asString));
		addElement: ((XMLElement named: #y) addContent: (XMLStringNode string: point y asString)).
	^node! !

DrGPointItem subclass: #DrGPointHomothetyItem
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'DrGeoII-Core-Item'!

!DrGPointHomothetyItem methodsFor: '*DrGeoII-Core-testing' stamp: 'HilaireFernandes 11/13/2007 20:27'!
isConstrainedPointItem
	^true! !

!DrGPointHomothetyItem methodsFor: 'xml writing' stamp: 'HilaireFernandes 1/20/2008 11:57'!
nodeType
	^#Scale! !

!DrGPointHomothetyItem methodsFor: 'updating' stamp: 'HilaireFernandes 10/17/2010 11:43'!
update
|center|
	self doParentsExist ifTrue:
		[center := parents second point.
		point := center + 
			(parents third valueItem * (parents first point - center))].! !

DrGPointItem subclass: #DrGPointIntersectionItem
	instanceVariableNames: 'k'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'DrGeoII-Core-Item'!

!DrGPointIntersectionItem commentStamp: '<historical>' prior: 0!
k 				- (-1) or 1 when they are 2 intersection points!

!DrGPointIntersectionItem class methodsFor: 'as yet unclassified' stamp: 'HilaireFernandes 10/30/2011 17:14'!
newWith: theParents k: integer
	"Instantiate this mathItem with these parents
	Parents contains other MathItem instance, sometime it contains other object as Point"
	^self new 
		parents: theParents; 
		k: integer;
		update! !

!DrGPointIntersectionItem methodsFor: 'comparing' stamp: 'HilaireFernandes 1/12/2006 22:22'!
= aMathItem
	^ super = aMathItem and: [k = aMathItem k]! !

!DrGPointIntersectionItem methodsFor: 'comparing' stamp: 'HilaireFernandes 1/12/2006 22:22'!
hash
	^super hash bitXor: k hash! !

!DrGPointIntersectionItem methodsFor: 'initialize-release' stamp: 'HilaireFernandes 4/30/2010 22:17'!
initialize: theParents
"check for the right intersection point when one parent is a circle or an arc"
|pointA pointB clicPoint|
	super initialize: theParents.
	(theParents first isCircleItem 
		or: [theParents second isCircleItem
	 		or: [theParents first isArcItem
				or: [theParents second isArcItem]]])
		ifFalse: [^self].
	pointA := theParents first intersectionWith: theParents second flag: -1.
	pointB := theParents first intersectionWith: theParents second flag: 1.
	pointA ifNil: [k := 1. ^self].
	pointB ifNil: [k := -1.	^self].
	clicPoint := theParents third.
	(pointA squaredDistanceTo: clicPoint) < (pointB squaredDistanceTo: clicPoint)
		ifTrue: [k := -1] 
		ifFalse: [k := 1]! !

!DrGPointIntersectionItem methodsFor: '*DrGeoII-Core-testing' stamp: 'HilaireFernandes 11/13/2007 20:28'!
isConstrainedPointItem
	^true! !

!DrGPointIntersectionItem methodsFor: '*DrGeoII-Core-testing' stamp: 'HilaireFernandes 3/10/2006 19:26'!
isIntersectionPointItem
	^true! !

!DrGPointIntersectionItem methodsFor: 'accessing' stamp: 'HilaireFernandes 1/12/2006 22:21'!
k
	^k! !

!DrGPointIntersectionItem methodsFor: 'accessing' stamp: 'HilaireFernandes 1/17/2008 15:26'!
k: integer 
	k := integer.
	self update! !

!DrGPointIntersectionItem methodsFor: 'xml writing' stamp: 'HilaireFernandes 1/20/2008 11:59'!
nodeType
	^#Intersection! !

!DrGPointIntersectionItem methodsFor: 'accessing' stamp: 'HilaireFernandes 7/20/2011 17:37'!
parents: aCollection 
	"We only need the 1st and 2nd mathItem in the collection. 
The 3rd one is a Point instance (position the user clicked) only used at initialization time"
	aCollection isEmptyOrNil ifTrue: 
		[parents := aCollection.
		^self].
	parents := OrderedCollection new
		add: aCollection first;
		add: aCollection second;
		yourself! !

!DrGPointIntersectionItem methodsFor: 'updating' stamp: 'HilaireFernandes 10/17/2010 11:43'!
update
	|aPoint|
	self doParentsExist ifTrue:
		[aPoint := (parents first intersectionWith: parents second flag: k).
		aPoint 
			ifNil: [exist := false]
			ifNotNil: 
				[exist := true.
				self point: aPoint]].! !

!DrGPointIntersectionItem methodsFor: 'xml writing' stamp: 'HilaireFernandes 4/25/2010 22:38'!
writeAsXmlTo: aNode 
	| node |
	node := super writeAsXmlTo: aNode.
	node
		attributeAt: #extra
		put: (k = 1
				ifTrue: ['1']
				ifFalse: ['0']).
	^node! !

!DrGPointItem methodsFor: 'private' stamp: 'jlrr 9/30/2007 15:07'!
adaptiveDescriptiveName
	^ 'This point %1' translated! !

!DrGPointItem methodsFor: 'accessing' stamp: 'HilaireFernandes 12/19/2005 15:20'!
basicType
	^#point! !

!DrGPointItem methodsFor: 'computing' stamp: 'HilaireFernandes 2/14/2011 22:06'!
closestPointTo: aPoint
	^ self point  ! !

!DrGPointItem methodsFor: '*DrGeoII-Core-testing' stamp: 'HilaireFernandes 1/25/2006 09:38'!
isGeometricItem
	^true! !

!DrGPointItem methodsFor: '*DrGeoII-Core-testing' stamp: 'HilaireFernandes 1/25/2006 09:38'!
isPointItem
	^true! !

!DrGPointItem methodsFor: 'as yet unclassified' stamp: 'HilaireFernandes 10/27/2011 10:12'!
isPropertyEditable
	^ self isFreePointItem ! !

!DrGPointItem methodsFor: 'updating' stamp: 'HilaireFernandes 3/27/2006 11:02'!
moveAt: aPoint
! !

!DrGPointItem methodsFor: 'updating' stamp: 'HilaireFernandes 3/25/2006 15:44'!
moveParent: aPoint
	^false! !

!DrGPointItem methodsFor: 'scripting' stamp: 'HilaireFernandes 2/9/2006 14:19'!
moveTo: aPoint
	self moveAt: aPoint.
	self triggerEvent: #movedMathItem! !

!DrGPointItem methodsFor: 'accessing' stamp: 'HilaireFernandes 12/19/2005 15:33'!
point
	^point ! !

!DrGPointItem methodsFor: 'accessing' stamp: 'HilaireFernandes 12/19/2005 15:33'!
point: aPoint
	point := aPoint ! !

DrGPointItem subclass: #DrGPointMiddle2ptsItem
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'DrGeoII-Core-Item'!

!DrGPointMiddle2ptsItem methodsFor: '*DrGeoII-Core-testing' stamp: 'HilaireFernandes 11/13/2007 20:28'!
isConstrainedPointItem
	^true! !

!DrGPointMiddle2ptsItem methodsFor: 'xml writing' stamp: 'HilaireFernandes 1/20/2008 11:59'!
nodeType
	^#'Middle_2pts'! !

!DrGPointMiddle2ptsItem methodsFor: 'as yet unclassified' stamp: 'HilaireFernandes 4/16/2011 11:16'!
parentsEqual: aMathItem
	^ self parents asSet = aMathItem parents asSet  ! !

!DrGPointMiddle2ptsItem methodsFor: 'updating' stamp: 'HilaireFernandes 10/17/2010 11:43'!
update
	self doParentsExist ifTrue:
		[self point: (parents first point + parents second point) / 2].
! !

DrGPointMiddle2ptsItem subclass: #DrGPointMiddlesegmentItem
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'DrGeoII-Core-Item'!

!DrGPointMiddlesegmentItem methodsFor: 'xml writing' stamp: 'HilaireFernandes 1/20/2008 10:23'!
nodeType
	^ #'Middle_segment'! !

!DrGPointMiddlesegmentItem methodsFor: 'updating' stamp: 'HilaireFernandes 10/17/2010 11:43'!
update
	self doParentsExist ifTrue:
		[self point: (parents first origin + (parents first direction / 2))].
! !

DrGPointItem subclass: #DrGPointOncurveItem
	instanceVariableNames: 'abscissa locked'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'DrGeoII-Core-Item'!

!DrGPointOncurveItem methodsFor: 'comparing' stamp: 'HilaireFernandes 10/29/2010 09:36'!
= aMathItem
	^ super = aMathItem 
		and: [self point = aMathItem point]! !

!DrGPointOncurveItem methodsFor: 'accessing' stamp: 'HilaireFernandes 12/29/2005 09:57'!
abscissa
	^abscissa! !

!DrGPointOncurveItem methodsFor: 'accessing' stamp: 'HilaireFernandes 2/9/2006 11:28'!
abscissa: anAbscissa
	abscissa := anAbscissa.
	abscissa > 1 ifTrue: [abscissa := 1].
	abscissa < 0 ifTrue: [abscissa := 0]! !

!DrGPointOncurveItem methodsFor: 'comparing' stamp: 'HilaireFernandes 10/29/2010 09:37'!
hash
	^ super hash bitXor: point hash! !

!DrGPointOncurveItem methodsFor: 'initialize-release' stamp: 'HilaireFernandes 11/13/2011 21:52'!
initialize: theParents
	super initialize: theParents. 
	self point: (theParents first closestPointTo: theParents second).
	self abscissa: (theParents first abscissaOf: self point).! !

!DrGPointOncurveItem methodsFor: '*DrGeoII-Core-testing' stamp: 'HilaireFernandes 10/29/2010 09:38'!
isFreePointItem
	^true! !

!DrGPointOncurveItem methodsFor: 'accessing' stamp: 'HilaireFernandes 11/6/2011 21:50'!
isLocked
	^ locked ifNil: [locked := false]! !

!DrGPointOncurveItem methodsFor: '*DrGeoII-Core-testing' stamp: 'HilaireFernandes 3/10/2006 19:25'!
isPointItemOnCurve
	^true! !

!DrGPointOncurveItem methodsFor: 'accessing' stamp: 'HilaireFernandes 11/6/2011 21:50'!
locked: aBoolean
	locked := aBoolean ! !

!DrGPointOncurveItem methodsFor: 'updating' stamp: 'HilaireFernandes 11/6/2011 21:50'!
move: aPoint
	self isLocked ifFalse: [
		self point: (parents first closestPointTo: point + aPoint).
		self abscissa: (parents first abscissaOf: self point)]
! !

!DrGPointOncurveItem methodsFor: 'updating' stamp: 'HilaireFernandes 11/6/2011 21:51'!
moveAt: aPoint
	self isLocked ifFalse: [
		self point: (parents first closestPointTo: aPoint).
		self abscissa: (parents first abscissaOf: self point)]
! !

!DrGPointOncurveItem methodsFor: 'scripting' stamp: 'HilaireFernandes 1/24/2008 10:34'!
moveTo: aPoint
	self point: (parents first closestPointTo: aPoint).
	self abscissa: (parents first abscissaOf: self point).
! !

!DrGPointOncurveItem methodsFor: 'xml writing' stamp: 'HilaireFernandes 1/20/2008 09:59'!
nodeType
	^#'On_curve'! !

!DrGPointOncurveItem methodsFor: 'accessing' stamp: 'HilaireFernandes 7/20/2011 17:37'!
parents: aCollection
"We only need the first mathItem in the collection. 
The second one is a Point instance (position the user clicked) only used at initialization time"
	aCollection isEmptyOrNil ifTrue: 
		[parents := aCollection.
		^self].
	parents := OrderedCollection new add: aCollection first;yourself.! !

!DrGPointOncurveItem methodsFor: 'accessing' stamp: 'HilaireFernandes 10/27/2011 11:35'!
property 
	^ self abscissa ! !

!DrGPointOncurveItem methodsFor: 'accessing' stamp: 'HilaireFernandes 10/27/2011 11:35'!
property: aProperty 
	self abscissa: aProperty ! !

!DrGPointOncurveItem methodsFor: 'scripting' stamp: 'HilaireFernandes 1/28/2008 16:20'!
setCurveAbscissa: aFloat 
	self abscissa: aFloat.
	self update! !

!DrGPointOncurveItem methodsFor: 'updating' stamp: 'HilaireFernandes 10/17/2010 11:43'!
update
	self doParentsExist ifTrue:
		[self point: (parents first pointAt: abscissa).
		(point x isInfinite or: [point y isInfinite ]) ifTrue: [exist := false]].
! !

!DrGPointOncurveItem methodsFor: 'xml writing' stamp: 'HilaireFernandes 10/31/2010 15:12'!
writeAsXmlTo: aNode
	"It is important to return the node"
	| node|
	node := super writeAsXmlTo: aNode.
	node addElement:  ((XMLElement named: #value) addContent: (XMLStringNode string: abscissa  asString)).
	^node! !

DrGPointItem subclass: #DrGPointReflexionItem
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'DrGeoII-Core-Item'!

!DrGPointReflexionItem methodsFor: '*DrGeoII-Core-testing' stamp: 'HilaireFernandes 11/13/2007 20:29'!
isConstrainedPointItem
	^true! !

!DrGPointReflexionItem methodsFor: 'xml writing' stamp: 'HilaireFernandes 1/20/2008 12:00'!
nodeType
	^#Reflexion! !

!DrGPointReflexionItem methodsFor: 'updating' stamp: 'HilaireFernandes 10/17/2010 11:43'!
update
	self doParentsExist	ifTrue:
		[self point: (parents first point
					reflexionBy: parents second origin 
					direction: parents second direction)].
! !

DrGPointItem subclass: #DrGPointRotationItem
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'DrGeoII-Core-Item'!

!DrGPointRotationItem methodsFor: '*DrGeoII-Core-testing' stamp: 'HilaireFernandes 11/13/2007 20:29'!
isConstrainedPointItem
	^true! !

!DrGPointRotationItem methodsFor: 'xml writing' stamp: 'HilaireFernandes 1/20/2008 12:01'!
nodeType
	^#Rotation! !

!DrGPointRotationItem methodsFor: 'updating' stamp: 'HilaireFernandes 10/17/2010 11:43'!
update
	self doParentsExist	ifTrue:
		[self point: (parents first point
			rotateBy: parents third valueItem negated 
			about: parents second point)].
! !

DrGPointItem subclass: #DrGPointSymmetryItem
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'DrGeoII-Core-Item'!

!DrGPointSymmetryItem methodsFor: '*DrGeoII-Core-testing' stamp: 'HilaireFernandes 11/13/2007 20:29'!
isConstrainedPointItem
	^true! !

!DrGPointSymmetryItem methodsFor: 'xml writing' stamp: 'HilaireFernandes 1/20/2008 12:02'!
nodeType
	^#Symmetry! !

!DrGPointSymmetryItem methodsFor: 'updating' stamp: 'HilaireFernandes 10/17/2010 11:43'!
update
	self doParentsExist ifTrue: 
		[self point: (2 * parents second point) - parents first point].
! !

DrGPointItem subclass: #DrGPointTranslationItem
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'DrGeoII-Core-Item'!

!DrGPointTranslationItem methodsFor: '*DrGeoII-Core-testing' stamp: 'HilaireFernandes 11/13/2007 20:29'!
isConstrainedPointItem
	^true! !

!DrGPointTranslationItem methodsFor: 'xml writing' stamp: 'HilaireFernandes 1/20/2008 12:04'!
nodeType
	^#Translation! !

!DrGPointTranslationItem methodsFor: 'updating' stamp: 'HilaireFernandes 10/17/2010 11:43'!
update
	self doParentsExist ifTrue:
		[self point: parents first point + parents second direction].
! !

DrGMathItem subclass: #DrGPolygonItem
	instanceVariableNames: 'points length sidesLength'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'DrGeoII-Core-Item'!

DrGPolygonItem subclass: #DrGPolygonHomothetyItem
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'DrGeoII-Core-Item'!

!DrGPolygonHomothetyItem methodsFor: 'updating' stamp: 'HilaireFernandes 3/25/2006 15:44'!
moveParent: aPoint
	^false! !

!DrGPolygonHomothetyItem methodsFor: 'xml writing' stamp: 'HilaireFernandes 1/20/2008 11:58'!
nodeType
	^#Scale! !

!DrGPolygonHomothetyItem methodsFor: 'updating' stamp: 'HilaireFernandes 10/17/2010 11:43'!
update
|center factor|
	self doParentsExist ifTrue:
		[center := parents second point.
		factor := parents third valueItem.
		points := OrderedCollection new.
		parents first points do: [:point |
			points add: (center + (factor * (point - center)))].
		self computeLength].! !

!DrGPolygonItem methodsFor: 'computing' stamp: 'HilaireFernandes 4/18/2011 21:32'!
abscissaOf: aPoint in: side
"To speed up calculs, we better know the side where is the point"
|distanceToPoint|
	distanceToPoint := 0.
	1 to: side - 1 do: [:i |
		distanceToPoint := distanceToPoint + (sidesLength at: i)].
	distanceToPoint := distanceToPoint + (aPoint dist: (points at: side)).
	^ distanceToPoint / length! !

!DrGPolygonItem methodsFor: 'private' stamp: 'HilaireFernandes 4/18/2011 21:32'!
adaptiveDescriptiveName
	^ 'This polygon %1' translated! !

!DrGPolygonItem methodsFor: 'accessing' stamp: 'HilaireFernandes 4/18/2011 21:32'!
basicType
	^#polygon! !

!DrGPolygonItem methodsFor: 'computing' stamp: 'HilaireFernandes 4/18/2011 21:32'!
closestPointInfoTo: aPoint
|closestPoint smallestDistance side|
	smallestDistance := 1e10.
	points withIndexDo: [:p :i | | distance point |
		point := self closestPointTo: aPoint 
					from: p
					to:  (points at: i + 1 ifAbsent: [points at: 1]).
		distance := aPoint dist: point.
		distance < smallestDistance
			ifTrue:
				[smallestDistance := distance.
				closestPoint := point.
				side := i]].
	^{closestPoint . self abscissaOf: closestPoint in: side}! !

!DrGPolygonItem methodsFor: 'computing' stamp: 'HilaireFernandes 4/18/2011 21:32'!
closestPointTo: aPoint
	^ (self closestPointInfoTo: aPoint) first! !

!DrGPolygonItem methodsFor: 'computing' stamp: 'HilaireFernandes 4/18/2011 21:32'!
closestPointTo: aPoint from: pointA to: pointB
|abscissa vector squareNorm|
	vector := pointB - pointA.
	squareNorm := vector dotProduct: vector.
	abscissa := (aPoint - pointA) dotProduct: vector.
	abscissa < 0 ifTrue: [abscissa := 0].
	abscissa > squareNorm ifTrue: 
		[abscissa := squareNorm].
	^ pointA + (abscissa * vector / squareNorm asFloat)! !

!DrGPolygonItem methodsFor: 'updating' stamp: 'HilaireFernandes 4/18/2011 21:32'!
computeLength
	length := 0.
	sidesLength := OrderedCollection new.
	points  withIndexDo: [:p :i |
		length := length + 
			(sidesLength add: (p dist: (points at: i + 1 ifAbsent: [points at: 1])))].
	^length! !

!DrGPolygonItem methodsFor: '*DrGeoII-Core-testing' stamp: 'HilaireFernandes 4/18/2011 21:32'!
isGeometricItem
	^true! !

!DrGPolygonItem methodsFor: '*DrGeoII-Core-testing' stamp: 'HilaireFernandes 4/18/2011 21:32'!
isPolygonItem
	^true! !

!DrGPolygonItem methodsFor: 'accessing' stamp: 'HilaireFernandes 4/18/2011 21:32'!
length
	^length ! !

!DrGPolygonItem methodsFor: 'computing' stamp: 'HilaireFernandes 4/18/2011 21:33'!
pointAt: anAbscissa
|distanceToPoint leftDistanceToPoint sum|
	distanceToPoint := leftDistanceToPoint := anAbscissa * length.
	sum := 0.
	sidesLength withIndexDo: [:s :i |
		sum := sum + s.
		sum >= distanceToPoint 
			ifTrue:
				["point is on this side"
				|pointA pointB vector|
				pointA := points at: i.
				pointB := points at: i + 1 ifAbsent: [points at: 1].
				pointA = pointB 
					ifTrue: [vector := 0@0]
					ifFalse:	[vector := (pointB - pointA) / (pointA dist: pointB)].
				^pointA + (vector * leftDistanceToPoint)].
		leftDistanceToPoint := leftDistanceToPoint - s]! !

!DrGPolygonItem methodsFor: 'accessing' stamp: 'HilaireFernandes 4/18/2011 21:33'!
points
	^points! !

DrGPolygonItem subclass: #DrGPolygonNptsItem
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'DrGeoII-Core-Item'!

!DrGPolygonNptsItem methodsFor: 'xml writing' stamp: 'HilaireFernandes 4/28/2010 22:20'!
nodeType
	^ #npts! !

!DrGPolygonNptsItem methodsFor: 'accessing' stamp: 'HilaireFernandes 4/18/2011 22:51'!
parentQty 
	^ 3! !

!DrGPolygonNptsItem methodsFor: 'comparing' stamp: 'HilaireFernandes 4/19/2011 22:15'!
parentsEqual: aPolygon
	|shiftedCollection|
	self parents size = aPolygon parents size ifFalse: [^false].
	shiftedCollection := self parents.
	shiftedCollection size timesRepeat: [
		shiftedCollection = aPolygon parents ifTrue: [^true].
		shiftedCollection := shiftedCollection shiftRight].
	shiftedCollection := self parents reverse.
	shiftedCollection size timesRepeat: [
		shiftedCollection = aPolygon parents ifTrue: [^true].
		shiftedCollection := shiftedCollection shiftRight].
	^ false! !

!DrGPolygonNptsItem methodsFor: 'updating' stamp: 'HilaireFernandes 10/17/2010 11:43'!
update
	self doParentsExist ifTrue:
		[points := OrderedCollection new.
		parents do: [:pointItem |
			points add: pointItem point].
		self computeLength].
! !

DrGPolygonItem subclass: #DrGPolygonReflexionItem
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'DrGeoII-Core-Item'!

!DrGPolygonReflexionItem methodsFor: 'updating' stamp: 'HilaireFernandes 3/25/2006 15:44'!
moveParent: aPoint
	^false! !

!DrGPolygonReflexionItem methodsFor: 'xml writing' stamp: 'HilaireFernandes 1/20/2008 12:00'!
nodeType
	^#Reflexion! !

!DrGPolygonReflexionItem methodsFor: 'accessing' stamp: 'HilaireFernandes 4/18/2011 21:39'!
parentQty
	^3! !

!DrGPolygonReflexionItem methodsFor: 'updating' stamp: 'HilaireFernandes 10/17/2010 11:43'!
update
|axe|
	self doParentsExist 	ifTrue:
		[axe := parents second.
		points := OrderedCollection new.
		parents first points do: [:point |
			points add: (point reflexionBy: axe origin direction: axe direction)].
		self computeLength].
! !

DrGPolygonItem subclass: #DrGPolygonRotationItem
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'DrGeoII-Core-Item'!

!DrGPolygonRotationItem methodsFor: 'updating' stamp: 'HilaireFernandes 3/25/2006 15:44'!
moveParent: aPoint
	^false! !

!DrGPolygonRotationItem methodsFor: 'xml writing' stamp: 'HilaireFernandes 1/20/2008 12:01'!
nodeType
	^#Rotation! !

!DrGPolygonRotationItem methodsFor: 'accessing' stamp: 'HilaireFernandes 4/18/2011 21:39'!
parentQty
	^3! !

!DrGPolygonRotationItem methodsFor: 'updating' stamp: 'HilaireFernandes 10/17/2010 11:43'!
update
|center angle|
	self doParentsExist ifTrue:
		[center := parents second point.
		angle := parents third valueItem negated.
		points := OrderedCollection new.
		parents first points do: [:point |
			points add: (point 
				rotateBy: angle
				about: center)].
		self computeLength].
! !

DrGPolygonItem subclass: #DrGPolygonSymmetryItem
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'DrGeoII-Core-Item'!

!DrGPolygonSymmetryItem methodsFor: 'updating' stamp: 'HilaireFernandes 3/25/2006 15:44'!
moveParent: aPoint
	^false! !

!DrGPolygonSymmetryItem methodsFor: 'xml writing' stamp: 'HilaireFernandes 1/20/2008 12:02'!
nodeType
	^#Symmetry! !

!DrGPolygonSymmetryItem methodsFor: 'accessing' stamp: 'HilaireFernandes 4/18/2011 21:39'!
parentQty
	^3! !

!DrGPolygonSymmetryItem methodsFor: 'updating' stamp: 'HilaireFernandes 10/17/2010 11:43'!
update
|center|
	self doParentsExist 	ifTrue:
		[center := parents second point.
		points := OrderedCollection new.
		parents first points do: [:point |
			points add: (2 * center - point)].
		self computeLength].
! !

DrGPolygonItem subclass: #DrGPolygonTranslationItem
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'DrGeoII-Core-Item'!

!DrGPolygonTranslationItem methodsFor: 'updating' stamp: 'HilaireFernandes 3/25/2006 15:44'!
moveParent: aPoint
	^false! !

!DrGPolygonTranslationItem methodsFor: 'xml writing' stamp: 'HilaireFernandes 1/20/2008 12:02'!
nodeType
	^#Translation! !

!DrGPolygonTranslationItem methodsFor: 'accessing' stamp: 'HilaireFernandes 4/18/2011 21:39'!
parentQty
	^3! !

!DrGPolygonTranslationItem methodsFor: 'updating' stamp: 'HilaireFernandes 10/17/2010 11:43'!
update
|translation|
	self doParentsExist 	ifTrue:
		[translation := parents second direction.
		points := OrderedCollection new.
		parents first points do: [:point |
			points add: point + translation].
		self computeLength].
! !

DrGMathItem subclass: #DrGTextItem
	instanceVariableNames: 'text position locked'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'DrGeoII-Core-Item'!

!DrGTextItem methodsFor: 'comparing' stamp: 'HilaireFernandes 11/5/2011 23:20'!
= aMathItem
	^ super = aMathItem and: [text = aMathItem text and: [position = aMathItem position]]! !

!DrGTextItem methodsFor: 'private' stamp: 'HilaireFernandes 11/5/2011 22:25'!
adaptiveDescriptiveName
	^ 'This text' translated! !

!DrGTextItem methodsFor: 'accessing' stamp: 'HilaireFernandes 11/5/2011 14:31'!
basicType
	^#text! !

!DrGTextItem methodsFor: 'accessing' stamp: 'HilaireFernandes 11/5/2011 14:33'!
descriptiveName 
	^ self adaptiveDescriptiveName
! !

!DrGTextItem methodsFor: 'testing' stamp: 'HilaireFernandes 11/5/2011 14:44'!
doParentsExist 
	^ true! !

!DrGTextItem methodsFor: 'comparing' stamp: 'HilaireFernandes 11/5/2011 23:18'!
hash
	^ (self basicType hash bitXor: text hash) bitXor: position hash! !

!DrGTextItem methodsFor: 'initialize-release' stamp: 'HilaireFernandes 11/5/2011 17:42'!
initialize 
	super initialize.
	text := 'Edit me' translated! !

!DrGTextItem methodsFor: 'initialize-release' stamp: 'HilaireFernandes 11/5/2011 14:35'!
initialize: theParents
	super initialize: theParents.
	self position: theParents last! !

!DrGTextItem methodsFor: 'accessing' stamp: 'HilaireFernandes 11/6/2011 21:59'!
isLocked
	^ locked ifNil: [locked := false]! !

!DrGTextItem methodsFor: 'testing' stamp: 'HilaireFernandes 11/5/2011 21:09'!
isPropertyEditable 
	^ true! !

!DrGTextItem methodsFor: '*DrGeoII-Core-testing' stamp: 'HilaireFernandes 11/5/2011 14:35'!
isTextItem
	^ true! !

!DrGTextItem methodsFor: 'accessing' stamp: 'HilaireFernandes 11/6/2011 21:59'!
locked: aBoolean
	locked := aBoolean ! !

!DrGTextItem methodsFor: 'updating' stamp: 'HilaireFernandes 11/6/2011 21:59'!
move: aPoint
	self isLocked ifFalse: [position := position + aPoint]! !

!DrGTextItem methodsFor: 'updating' stamp: 'HilaireFernandes 11/5/2011 14:39'!
moveParent: aPoint
	^ false! !

!DrGTextItem methodsFor: 'xml writing' stamp: 'HilaireFernandes 11/6/2011 13:07'!
nodeType
	^#free! !

!DrGTextItem methodsFor: 'accessing' stamp: 'HilaireFernandes 11/5/2011 18:02'!
parents: aCollection
	parents := nil! !

!DrGTextItem methodsFor: 'accessing' stamp: 'HilaireFernandes 11/5/2011 18:11'!
position
	^ position! !

!DrGTextItem methodsFor: 'accessing' stamp: 'HilaireFernandes 11/5/2011 17:41'!
position: aPoint 
	position := aPoint! !

!DrGTextItem methodsFor: 'updating' stamp: 'HilaireFernandes 11/5/2011 21:09'!
property
	^ self text! !

!DrGTextItem methodsFor: 'updating' stamp: 'HilaireFernandes 11/5/2011 21:09'!
property: aProperty 
	self text: aProperty ! !

!DrGTextItem methodsFor: 'accessing' stamp: 'HilaireFernandes 11/5/2011 14:40'!
text
	^ text! !

!DrGTextItem methodsFor: 'accessing' stamp: 'HilaireFernandes 11/5/2011 14:40'!
text: aText
	text := aText! !

!DrGTextItem methodsFor: 'xml writing' stamp: 'HilaireFernandes 11/5/2011 14:43'!
writeAsXmlPositionTo: aNode 
	"return the newly created element"
	aNode
		addElement: ((XMLElement named: #x)
				addContent: (XMLStringNode string: position x asString));
		addElement: ((XMLElement named: #y)
				addContent: (XMLStringNode string: position y asString)).
	^ aNode! !

!DrGTextItem methodsFor: 'xml writing' stamp: 'HilaireFernandes 11/5/2011 15:29'!
writeAsXmlTo: aNode
	|node|
	node := super writeAsXmlTo: aNode.
	node addElement:  ((XMLElement named: #contents) addContent: (XMLStringNode string: self text)).
	^ self writeAsXmlPositionTo: node
! !

DrGMathItem subclass: #DrGValueItem
	instanceVariableNames: 'value position locked'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'DrGeoII-Core-Item'!

DrGValueItem subclass: #DrGAngleItem
	instanceVariableNames: 'radianValue degreeValue'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'DrGeoII-Core-Item'!

DrGAngleItem subclass: #DrGAngle3ptsItem
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'DrGeoII-Core-Item'!

!DrGAngle3ptsItem commentStamp: '<historical>' prior: 0!
Define a geometric angle in the range  [0 ; 180]!

!DrGAngle3ptsItem methodsFor: 'private' stamp: 'jlrr 9/29/2007 14:36'!
adaptiveDescriptiveName
	^ 'This geometric angle %1' translated! !

!DrGAngle3ptsItem methodsFor: 'updating' stamp: 'HilaireFernandes 2/4/2006 22:45'!
move: aPoint
"Don't move when user drag around"! !

!DrGAngle3ptsItem methodsFor: 'xml writing' stamp: 'HilaireFernandes 1/20/2008 10:39'!
nodeType
	^#'3pts'! !

!DrGAngle3ptsItem methodsFor: 'updating' stamp: 'HilaireFernandes 10/17/2010 11:43'!
update
|u v center|
	self doParentsExist ifTrue:
		[center := parents second point.
		u := parents first point - center.
		v := parents third point - center.
		(u = (0@0) or: [v = (0@0)]) 
			ifTrue: [exist := false]
			ifFalse:
				[radianValue := value := ((u crossProduct: v) asFloat arcTan2: (u dotProduct: v)) abs.
				degreeValue := radianValue radiansToDegrees]].! !

!DrGAngleItem methodsFor: 'accessing' stamp: 'HilaireFernandes 2/4/2006 11:56'!
basicType
	^#angle! !

!DrGAngleItem methodsFor: 'accessing' stamp: 'HilaireFernandes 2/4/2006 10:25'!
degreeAngle
"return the angle in degree"
	^degreeValue ! !

!DrGAngleItem methodsFor: '*DrGeoII-Core-testing' stamp: 'HilaireFernandes 2/4/2006 09:58'!
isAngleItem
	^true! !

!DrGAngleItem methodsFor: 'accessing' stamp: 'HilaireFernandes 2/4/2006 10:25'!
radianAngle
"return the angle in radian"
	^radianValue ! !

DrGAngleItem subclass: #DrGAngleVectorsItem
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'DrGeoII-Core-Item'!

!DrGAngleVectorsItem commentStamp: '<historical>' prior: 0!
Define an oriented angle, in the range ]-180 ; 180]!

!DrGAngleVectorsItem methodsFor: 'private' stamp: 'jlrr 9/29/2007 14:40'!
adaptiveDescriptiveName
	^ 'This oriented angle %1' translated! !

!DrGAngleVectorsItem methodsFor: 'xml writing' stamp: 'HilaireFernandes 10/16/2010 09:15'!
nodeType
	^ #vectors! !

!DrGAngleVectorsItem methodsFor: 'updating' stamp: 'HilaireFernandes 10/17/2010 11:43'!
update
|u v|
	self doParentsExist ifTrue:
		[u := parents first direction.
		v := parents second direction.
		(u = (0@0) or: [v = (0@0)]) 
			ifTrue: [exist := false]
			ifFalse:
				[radianValue := value := ((u crossProduct: v) asFloat arcTan2: (u dotProduct: v)).
				degreeValue := radianValue radiansToDegrees]].	! !

!DrGAngleVectorsItem methodsFor: 'xml writing' stamp: 'HilaireFernandes 10/31/2010 14:41'!
writeAsXmlPositionTo: aNode 
	"return the newly created element"
	aNode
		addElement: ((XMLElement named: #x)
				addContent: (XMLStringNode string: position x asString));
		addElement: ((XMLElement named: #y)
				addContent: (XMLStringNode string: position y asString)).
	^ aNode! !

!DrGAngleVectorsItem methodsFor: 'xml writing' stamp: 'HilaireFernandes 10/31/2010 15:05'!
writeAsXmlTo: aNode 
	"return the newly created element"
	| node |
	node := super writeAsXmlTo: aNode.
	^ self writeAsXmlPositionTo: node.! !

DrGValueItem subclass: #DrGComputedValueItem
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'DrGeoII-Core-Item'!

!DrGComputedValueItem methodsFor: 'xml writing' stamp: 'HilaireFernandes 10/31/2010 15:08'!
writeAsXmlTo: aNode 
	"return the newly created element"
	| node |
	node := super writeAsXmlTo: aNode.
	^ self writeAsXmlPositionTo: node.! !

DrGComputedValueItem subclass: #DrGValueArclengthItem
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'DrGeoII-Core-Item'!

!DrGValueArclengthItem methodsFor: 'private' stamp: 'jlrr 9/30/2007 15:14'!
adaptiveDescriptiveName
	^ 'This arc length %1' translated! !

!DrGValueArclengthItem methodsFor: 'xml writing' stamp: 'HilaireFernandes 1/20/2008 11:06'!
nodeType
	^#'arc_length'! !

!DrGValueArclengthItem methodsFor: 'updating' stamp: 'HilaireFernandes 10/17/2010 11:43'!
update
	self doParentsExist ifTrue: 
		[value := parents first length abs * parents first radius].
! !

DrGComputedValueItem subclass: #DrGValueCircleperimeterItem
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'DrGeoII-Core-Item'!

!DrGValueCircleperimeterItem methodsFor: 'private' stamp: 'jlrr 9/30/2007 11:39'!
adaptiveDescriptiveName
	^ 'This circle perimeter %1' translated! !

!DrGValueCircleperimeterItem methodsFor: 'xml writing' stamp: 'HilaireFernandes 1/20/2008 11:16'!
nodeType
	^#'circle_perimeter'! !

!DrGValueCircleperimeterItem methodsFor: 'updating' stamp: 'HilaireFernandes 10/17/2010 11:43'!
update
	self doParentsExist ifTrue:
		[value := parents first radius * DrGDirectionItem twoPi].
! !

DrGComputedValueItem subclass: #DrGValueDistance2ptsItem
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'DrGeoII-Core-Item'!

!DrGValueDistance2ptsItem methodsFor: 'private' stamp: 'jlrr 9/30/2007 11:41'!
adaptiveDescriptiveName
	^ 'This distance between two points %1' translated! !

!DrGValueDistance2ptsItem methodsFor: 'xml writing' stamp: 'HilaireFernandes 1/20/2008 11:17'!
nodeType
	^# 'distance_2pts'! !

!DrGValueDistance2ptsItem methodsFor: 'updating' stamp: 'HilaireFernandes 4/8/2011 19:07'!
update
	self doParentsExist ifTrue:
		[value := parents first point dist: parents second point].
! !

DrGComputedValueItem subclass: #DrGValueDistanceptlineItem
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'DrGeoII-Core-Item'!

!DrGValueDistanceptlineItem methodsFor: 'private' stamp: 'jlrr 9/30/2007 11:40'!
adaptiveDescriptiveName
	^ 'This point-line distance %1' translated! !

!DrGValueDistanceptlineItem methodsFor: 'xml writing' stamp: 'HilaireFernandes 1/20/2008 11:17'!
nodeType
	^#'distance_pt_line'! !

!DrGValueDistanceptlineItem methodsFor: 'updating' stamp: 'HilaireFernandes 10/17/2010 11:43'!
update
	self doParentsExist ifTrue:
		[value := ((parents first point - parents second origin) 
					dotProduct: parents second direction normal) abs].
! !

DrGComputedValueItem subclass: #DrGValueSegmentlengthItem
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'DrGeoII-Core-Item'!

!DrGValueSegmentlengthItem methodsFor: 'private' stamp: 'jlrr 9/30/2007 15:14'!
adaptiveDescriptiveName
	^ 'This segment length %1' translated! !

!DrGValueSegmentlengthItem methodsFor: 'xml writing' stamp: 'HilaireFernandes 1/20/2008 11:18'!
nodeType
	^#'segment_length'! !

!DrGValueSegmentlengthItem methodsFor: 'updating' stamp: 'HilaireFernandes 10/17/2010 11:43'!
update
	self doParentsExist ifTrue:
		[value := parents first direction r].
! !

DrGValueSegmentlengthItem subclass: #DrGValueVectornormItem
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'DrGeoII-Core-Item'!

!DrGValueVectornormItem methodsFor: 'private' stamp: 'jlrr 9/30/2007 15:15'!
adaptiveDescriptiveName
	^ 'This Vector''s norm %1' translated! !

!DrGValueVectornormItem methodsFor: 'xml writing' stamp: 'HilaireFernandes 1/20/2008 11:18'!
nodeType
	^#'vector_norm'! !

DrGComputedValueItem subclass: #DrGValueSlopeItem
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'DrGeoII-Core-Item'!

!DrGValueSlopeItem methodsFor: 'private' stamp: 'HilaireFernandes 1/16/2008 17:22'!
adaptiveDescriptiveName
	^ 'This line slope %1' translated! !

!DrGValueSlopeItem methodsFor: 'xml writing' stamp: 'HilaireFernandes 1/20/2008 11:17'!
nodeType
	^#slope! !

!DrGValueSlopeItem methodsFor: 'updating' stamp: 'HilaireFernandes 10/17/2010 11:43'!
update
	|v|
	self doParentsExist ifTrue:
		[v := parents first direction.
		v x= 0 ifTrue: [exist := false]
		ifFalse: [value := v y / v x.]].
! !

!DrGValueItem methodsFor: 'private' stamp: 'jlrr 9/30/2007 15:14'!
adaptiveDescriptiveName
	^ 'This value %1' translated! !

!DrGValueItem methodsFor: 'accessing' stamp: 'HilaireFernandes 2/2/2006 11:18'!
basicType
	 ^#value! !

!DrGValueItem methodsFor: 'initialize-release' stamp: 'HilaireFernandes 2/2/2006 11:59'!
initialize
	super initialize.
	value := 0! !

!DrGValueItem methodsFor: 'initialize-release' stamp: 'HilaireFernandes 8/3/2008 12:04'!
initialize: theParents
	super initialize: theParents.
	self position: theParents last! !

!DrGValueItem methodsFor: 'accessing' stamp: 'HilaireFernandes 11/6/2011 21:56'!
isLocked
	^ locked ifNil: [locked := false]! !

!DrGValueItem methodsFor: '*DrGeoII-Core-testing' stamp: 'HilaireFernandes 2/2/2006 11:19'!
isValueItem
	^true! !

!DrGValueItem methodsFor: 'accessing' stamp: 'HilaireFernandes 11/6/2011 21:56'!
locked: aBoolean
	locked := aBoolean ! !

!DrGValueItem methodsFor: 'updating' stamp: 'HilaireFernandes 11/6/2011 21:57'!
move: aPoint
	self isLocked ifFalse: [position := position + aPoint].
! !

!DrGValueItem methodsFor: 'updating' stamp: 'HilaireFernandes 3/25/2006 15:44'!
moveParent: aPoint
"Don't move my parent"
	^false! !

!DrGValueItem methodsFor: 'accessing' stamp: 'HilaireFernandes 7/20/2011 17:38'!
parents: aCollection 
	"last is the initial screen position of the value"
	aCollection isEmptyOrNil ifTrue: 
		[ parents := aCollection .
		^ self ].
	aCollection last isPoint 
		ifTrue: [ parents := aCollection allButLast ]
		ifFalse: [ parents := aCollection ]! !

!DrGValueItem methodsFor: 'accessing' stamp: 'HilaireFernandes 2/2/2006 10:48'!
position
	^position! !

!DrGValueItem methodsFor: 'accessing' stamp: 'HilaireFernandes 2/2/2006 10:47'!
position: aPoint
	position := aPoint! !

!DrGValueItem methodsFor: 'accessing' stamp: 'HilaireFernandes 2/2/2006 10:49'!
valueItem
	^value! !

!DrGValueItem methodsFor: 'xml writing' stamp: 'HilaireFernandes 10/31/2010 14:41'!
writeAsXmlPositionTo: aNode 
	"return the newly created element"
	aNode
		addElement: ((XMLElement named: #x)
				addContent: (XMLStringNode string: position x asString));
		addElement: ((XMLElement named: #y)
				addContent: (XMLStringNode string: position y asString)).
	^ aNode! !

DrGValueItem subclass: #DrGValueScriptItem
	instanceVariableNames: 'script'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'DrGeoII-Core-Item'!

!DrGValueScriptItem commentStamp: '<historical>' prior: 0!
Script!

!DrGValueScriptItem methodsFor: 'comparing' stamp: 'HilaireFernandes 8/23/2008 22:45'!
= aScriptItem
	^super = aScriptItem and: [script = aScriptItem script]! !

!DrGValueScriptItem methodsFor: 'private' stamp: 'HilaireFernandes 9/19/2010 20:25'!
adaptiveDescriptiveName
	| string |
	parents size > 0 
		ifFalse: [string := 'This script {1}>>{2}' translated]
		ifTrue: 
			[parents size = 1 
				ifTrue: [string := 'This script {1}>>{2} with argument: %2' translated]
				ifFalse: [string := 'This script {1}>>{2} with arguments: %2' translated].
			string := string copyReplaceAll: '%2' with: self argumentName].
	^string format: {'DrGeoUserScripts'. script asString}.! !

!DrGValueScriptItem methodsFor: 'private' stamp: 'HilaireFernandes 8/4/2008 13:15'!
argumentName
	"collect parents name in a string as A, B, C"
	| stream |
	parents size = 0 ifTrue: [ ^ '' ].
	stream := ReadWriteStream on: String new.
	parents allButLast do: 
		[ :item | 
		stream
			nextPutAll: item safeName;
			nextPutAll: ', ' ].
	stream nextPutAll: parents last safeName.
	^ stream contents! !

!DrGValueScriptItem methodsFor: 'testing' stamp: 'HilaireFernandes 8/23/2008 22:34'!
hash
	^super hash bitXor: script hash! !

!DrGValueScriptItem methodsFor: 'initialization' stamp: 'HilaireFernandes 8/4/2008 10:31'!
initialize: theParents 
	super initialize: theParents.
	script := theParents at: theParents size - 1! !

!DrGValueScriptItem methodsFor: 'testing' stamp: 'HilaireFernandes 10/27/2011 10:07'!
isPropertyEditable
	^ true! !

!DrGValueScriptItem methodsFor: 'testing' stamp: 'HilaireFernandes 8/3/2008 18:29'!
isScriptItem
	^true! !

!DrGValueScriptItem methodsFor: 'xml writing' stamp: 'HilaireFernandes 11/5/2010 21:48'!
nodeType
	^#script! !

!DrGValueScriptItem methodsFor: 'accessing' stamp: 'HilaireFernandes 7/20/2011 17:37'!
parents: aCollection 
	"last is the initial screen position of the value, and previous last is the script symbol"
	aCollection isEmptyOrNil ifTrue: 
		[ parents := aCollection.
		^ self ].
	aCollection last isPoint 
		ifTrue: [ parents := aCollection allButLast: 2 ]
		ifFalse: 
			[ "play nicely with macro"
			parents := aCollection ]! !

!DrGValueScriptItem methodsFor: 'accessing' stamp: 'HilaireFernandes 8/23/2008 22:55'!
script
	^script! !

!DrGValueScriptItem methodsFor: 'updating' stamp: 'HilaireFernandes 10/17/2010 11:43'!
update
	self doParentsExist ifTrue: 
		[ value := DrGeoUserScripts instance
			perform: script
			withArguments: parents asArray ]! !

!DrGValueScriptItem methodsFor: 'xml writing' stamp: 'HilaireFernandes 11/5/2010 22:02'!
writeAsXmlTo: aNode
	|node|
	node := super writeAsXmlTo: aNode.
	node addElement: (
		(XMLElement named: #code) addContent: (
			XMLStringNode string: (DrGeoUserScripts compiledMethodAt: script) getSource asString)).
	^ self writeAsXmlPositionTo: node.! !

DrGValueItem subclass: #DrGValueValueItem
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'DrGeoII-Core-Item'!

!DrGValueValueItem commentStamp: '<historical>' prior: 0!
Free value!

!DrGValueValueItem methodsFor: 'comparing' stamp: 'HilaireFernandes 7/24/2010 19:16'!
= aMathItem
	^super = aMathItem 
		and: [self valueItem = aMathItem valueItem
			and: [self position = aMathItem position]]! !

!DrGValueValueItem methodsFor: 'testing' stamp: 'HilaireFernandes 10/29/2010 09:13'!
doParentsExist
	^ exist := true.! !

!DrGValueValueItem methodsFor: 'comparing' stamp: 'HilaireFernandes 2/2/2006 15:33'!
hash
	^ (super hash bitXor: position hash) bitXor: value hash! !

!DrGValueValueItem methodsFor: 'testing' stamp: 'HilaireFernandes 10/27/2011 10:07'!
isPropertyEditable
	^ true! !

!DrGValueValueItem methodsFor: 'xml writing' stamp: 'HilaireFernandes 1/20/2008 10:42'!
nodeType
	^#value! !

!DrGValueValueItem methodsFor: 'as yet unclassified' stamp: 'HilaireFernandes 10/27/2011 11:36'!
property
	^ self valueItem! !

!DrGValueValueItem methodsFor: 'as yet unclassified' stamp: 'HilaireFernandes 10/27/2011 11:36'!
property: aProperty 
	self valueItem: aProperty ! !

!DrGValueValueItem methodsFor: 'accessing' stamp: 'HilaireFernandes 1/24/2008 10:33'!
valueItem: aValue
	value := aValue.
! !

!DrGValueValueItem methodsFor: 'xml writing' stamp: 'HilaireFernandes 10/31/2010 15:09'!
writeAsXmlTo: aNode 
	"return the newly created element"
	| node |
	node := super writeAsXmlTo: aNode.
	node addElement:  ((XMLElement named: #value) addContent: (XMLStringNode string: value  asString)).
	^ self writeAsXmlPositionTo: node.! !

Model subclass: #DrGeoDomain
	instanceVariableNames: 'factory cmdManager actionMap'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'DrGeoII-Core-App'!

!DrGeoDomain commentStamp: 'HilaireFernandes 4/18/2010 12:42' prior: 0!
This is the domain object for DrGeo, aka the top level Model instance for one DrGeo instance!

!DrGeoDomain methodsFor: 'event' stamp: 'HilaireFernandes 3/22/2006 12:03'!
actionMap
	actionMap ifNil: [^self createActionMap].
	^ actionMap ! !

!DrGeoDomain methodsFor: 'user interface' stamp: 'HilaireFernandes 6/15/2010 22:18'!
addModelItemsToWindowMenu: aMenu 
	aMenu addLine.
	aMenu
		add: 'About Dr. Geo II...'
		target: self
		selector: #inform:
		argument: 'Copyright 1996-2010 Hilaire Fernandes'! !

!DrGeoDomain methodsFor: 'building' stamp: 'HilaireFernandes 3/6/2006 09:27'!
createCompositeItem: aMathItemSelection
	cmdManager compositeCommandFor: aMathItemSelection.! !

!DrGeoDomain methodsFor: 'building' stamp: 'HilaireFernandes 7/27/2008 11:19'!
createFromMathItem: aMathItem
	"return the mathItem (newly created or the one in the pool, 
in case the caller want to do something with it"
	(factory pushAsLastWhenInPool: aMathItem) ifTrue:
		[^{factory last. false}].
	"create a new command and execute it"
	cmdManager buildCommandFor: aMathItem.
	^{ aMathItem. true }! !

!DrGeoDomain methodsFor: 'building' stamp: 'HilaireFernandes 7/27/2008 11:15'!
createFromMathItemNoStack: aMathItem
	"return the mathItem (newly created or the one in the pool, 
in case the caller want to do something with it, without stacking in the undo/redo stack"
	(factory pushAsLastWhenInPool: aMathItem) ifTrue: [^factory last].
	factory add: aMathItem.
	self triggerEvent: #newMathItem with: aMathItem.
	^aMathItem! !

!DrGeoDomain methodsFor: 'building' stamp: 'HilaireFernandes 10/28/2011 18:03'!
createFromMathItemNoStackNoFactoryCheck: aMathItem
	"return the mathItem. we do not check in the facory pool, without stacking in the undo/redo stack"
	factory add: aMathItem.
	self triggerEvent: #newMathItem with: aMathItem.
	^aMathItem! !

!DrGeoDomain methodsFor: 'building' stamp: 'HilaireFernandes 10/17/2010 10:35'!
createMacro: aBuilder 
	(DrGMacroFactory new pushAsLastWhenInPool: (aBuilder getItem: self)) ifFalse: 
		[ "Create macro and register it in the factory"
		DrGMacroFactory new add: (aBuilder getItem: self) ]! !

!DrGeoDomain methodsFor: 'building' stamp: 'HilaireFernandes 8/23/2008 22:56'!
createMathItem: aBuilder 
	"return the mathItem (newly created or the one in the pool, 
in case the caller want to do something with it"
	(factory pushAsLastWhenInPool: aBuilder getItem first) ifTrue: 
		[ ^ {  (factory last). false  } ].
	"create a new command and execute it"
	^ {  (cmdManager buildCommandWith: aBuilder). true  }! !

!DrGeoDomain methodsFor: 'building' stamp: 'HilaireFernandes 1/30/2006 17:08'!
deleteMathItem: aMathItem 
	"create a delete command and execute it"
	cmdManager deleteCommandFor: aMathItem! !

!DrGeoDomain methodsFor: 'accessing' stamp: 'HilaireFernandes 12/7/2005 10:58'!
factory
	^factory! !

!DrGeoDomain methodsFor: 'building' stamp: 'HilaireFernandes 7/31/2008 23:30'!
in: macroTree replaceParent: item with: newItem 
	| index |
	macroTree do: [ :anItem | 
		(anItem parents notNil and: [ (index := anItem parents indexOf: item) ~= 0 ])
			ifTrue: [anItem parents at: index put: newItem ] ]! !

!DrGeoDomain methodsFor: 'initialize-release' stamp: 'HilaireFernandes 1/30/2006 17:04'!
initialize
	super initialize.
	factory := DrGMathItemFactory new.
	cmdManager := DrGCommandManager new domain: self! !

!DrGeoDomain methodsFor: 'accessing' stamp: 'HilaireFernandes 4/9/2010 13:22'!
labelString
	^ 'Dr. Geo II - ', Date today asString! !

!DrGeoDomain methodsFor: 'updating' stamp: 'HilaireFernandes 1/23/2008 11:49'!
moveEvent: aMathItem at: aPoint
	aMathItem moveAt: aPoint.
	self updateAllMathItems! !

!DrGeoDomain methodsFor: 'updating' stamp: 'HilaireFernandes 7/20/2011 16:58'!
moveEvent: aMathItem at: aPoint withDirty: mathItems
	aMathItem moveAt: aPoint.
	self updateDirty: mathItems! !

!DrGeoDomain methodsFor: 'updating' stamp: 'HilaireFernandes 7/18/2008 13:57'!
moveEvent: aMathItem inDirection: aPoint 
	aMathItem moveEvent: aPoint.
	self updateAllMathItems ! !

!DrGeoDomain methodsFor: 'updating' stamp: 'HilaireFernandes 7/20/2011 16:58'!
moveEvent: aMathItem inDirection: aPoint withDirty: mathItems
	aMathItem moveEvent: aPoint.
	self updateDirty: mathItems! !

!DrGeoDomain methodsFor: 'building' stamp: 'HilaireFernandes 1/31/2006 11:40'!
moveMathItem: aMathItem with: aPoint
	"create a move command"
	cmdManager moveCommandFor: aMathItem with: aPoint! !

!DrGeoDomain methodsFor: 'updating' stamp: 'HilaireFernandes 11/27/2011 12:01'!
mutate: aPointItem asFreeOn: aCurveItem at: aPosition
	|item|
	item := DrGPointOncurveItem newWith: {aCurveItem .  aPosition}.
	item name: aPointItem name.
	item actionMap: aPointItem actionMap.
	aPointItem become: item.
	self updateAllMathItems ! !

!DrGeoDomain methodsFor: 'updating' stamp: 'HilaireFernandes 11/27/2011 12:01'!
mutate: aPointItem asIntersectionWith: curveA and: curveB at: aPosition
	|item|
	item :=  DrGPointIntersectionItem newWith: {curveA. curveB.  aPosition}.
	item name: aPointItem name.
	item actionMap: aPointItem actionMap.
	aPointItem become: item.
	self updateAllMathItems.! !

!DrGeoDomain methodsFor: 'updating' stamp: 'HilaireFernandes 11/27/2011 12:01'!
mutateAsFreePoint: aPointItem
	|freePointItem|
	(aPointItem isFreePointItem and: [aPointItem isPointItemOnCurve not]) ifTrue: [^aPointItem].
	freePointItem := DrGPointFreeItem new point: aPointItem point.
	freePointItem name: aPointItem name.
	freePointItem actionMap: aPointItem actionMap.
	aPointItem become: freePointItem.
	^ freePointItem! !

!DrGeoDomain methodsFor: 'building' stamp: 'HilaireFernandes 4/19/2010 13:11'!
playMacro: aBuilder in: app 
	| answer macroTree |
	macroTree := aBuilder getItem.
	macroTree 
		with: aBuilder constructedNodes
		do: 
			[ :item :node | 
			answer := self createFromMathItem: item.
			answer last 
				ifTrue: 
					[ node isHiddenNode ifTrue: 
						[ (app costumeOf: item) ifNotNilDo: 
							[ :aCostume | 
							aCostume style isHidden: true.
							aCostume forceVisible: false ] ] ]
				ifFalse: 
					[ "item twin in the factory, use it in the macro tree"
					self 
						in: macroTree
						replaceParent: item
						with: answer first ] ]! !

!DrGeoDomain methodsFor: 'building' stamp: 'HilaireFernandes 10/27/2011 11:46'!
propertyCommandFor: aMathItem with: aProperty 
	"create a move command"
	cmdManager propertyCommandFor: aMathItem with: aProperty ! !

!DrGeoDomain methodsFor: 'accessing' stamp: 'HilaireFernandes 1/31/2006 00:11'!
redo
	cmdManager redo! !

!DrGeoDomain methodsFor: 'initialize-release' stamp: 'HilaireFernandes 8/18/2008 12:42'!
release
	super release.
	factory release.
	cmdManager release.! !

!DrGeoDomain methodsFor: 'event' stamp: 'HilaireFernandes 3/22/2006 16:54'!
releaseActionMap
	actionMap := nil! !

!DrGeoDomain methodsFor: 'accessing' stamp: 'HilaireFernandes 1/31/2006 00:11'!
undo
	cmdManager undo! !

!DrGeoDomain methodsFor: 'updating' stamp: 'HilaireFernandes 8/4/2008 14:29'!
update: symbol
	Transcript show: symbol asString;cr.
	symbol = #contents ifTrue: [self updateAllMathItems]! !

!DrGeoDomain methodsFor: 'updating' stamp: 'HilaireFernandes 7/18/2008 14:18'!
updateAllMathItems
	factory updateAllMathItems.
	self triggerEvent: #updatedItems! !

!DrGeoDomain methodsFor: 'updating' stamp: 'HilaireFernandes 1/25/2008 23:10'!
updateAllMathItemsButLocus
	factory updateAllMathItemsButLocus.
	"We don't need to update the costumem we just want updated mathiteem'"
	"self triggerEvent: #updatedItems"! !

!DrGeoDomain methodsFor: 'updating' stamp: 'HilaireFernandes 7/20/2011 16:58'!
updateDirty: mathItems
	mathItems do: [:item | item update].
	self triggerEvent: #updatedDirtyItems! !

!DrGeoDomain methodsFor: 'event' stamp: 'HilaireFernandes 3/22/2006 12:03'!
updateableActionMap
	actionMap ifNil:
		[actionMap := self createActionMap].
	^actionMap! !

PolygonMorph subclass: #DrGPolylineMorph
	instanceVariableNames: 'blink costume'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'DrGeoII-Core-Item-View'!

DrGPolylineMorph subclass: #DrGArcMorph
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'DrGeoII-Core-Item-View'!

!DrGArcMorph methodsFor: 'accessing' stamp: 'HilaireFernandes 11/18/2011 22:16'!
center: aPoint radius: aFloat origin: anOrigin length: aLength
|step costep sinstep csteX csteY mobile|
	vertices := OrderedCollection new.
	aFloat ~= 0 
		ifTrue: [
			step := 5 / aFloat * aLength sign.
			step  ~= 0 ifTrue: [
				(aLength / step) abs > 100 ifTrue: [step := aLength / 100].
				costep := step cos.	sinstep := step sin.
				csteX := aPoint x * (1 - costep) + (aPoint y * sinstep).
				csteY := aPoint y * (1 - costep) - (aPoint x * sinstep).
				mobile := aPoint + (aFloat * (anOrigin cos @ anOrigin sin)).
				vertices add: mobile.
				0 to: (aLength / step) truncated -1  do: [:i|
					mobile := (mobile x * costep - (mobile y * sinstep) + csteX)
						@(mobile x * sinstep + (mobile y * costep) + csteY).
					vertices add: mobile]].
			mobile := aPoint + (aFloat * ((anOrigin + aLength) cos @ (anOrigin + aLength) sin)).
			vertices add: mobile]
		ifFalse: [vertices add: aPoint].
	self computeBounds! !

!DrGArcMorph methodsFor: 'accessing' stamp: 'HilaireFernandes 11/16/2011 12:16'!
color: aColor
	|theColor|
	costume style 
		ifNotNil: [theColor := costume style isTranslucent ifTrue: [aColor alpha: 0.3] ifFalse: [aColor]]
	 	ifNil: [theColor := aColor].
	super color: theColor! !

DrGPolylineMorph subclass: #DrGLocusMorph
	instanceVariableNames: 'free constrained drawable factory minAbscissaDelta maxScreenDelta maxAbscissaDelta freeIndex constrainedIndex samples worldBound'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'DrGeoII-Core-Item-View'!

!DrGLocusMorph commentStamp: 'HilaireFernandes 4/2/2011 08:57' prior: 0!
- list : a linked list of DrGLocusSample
- minAbscissaDelta : the minimum abscissa delta between two samples
- maxAbscissaDelta : the maximum abscissa delta between two samples
- maxScreenDelta : the maximum delta between two samples in the screen!

!DrGLocusMorph methodsFor: 'accessing' stamp: 'HilaireFernandes 4/1/2011 16:57'!
costume:  aCostume
	super costume: aCostume.
	drawable := aCostume drawable.
	factory := aCostume domain factory.! !

!DrGLocusMorph methodsFor: 'as yet unclassified' stamp: 'HilaireFernandes 4/2/2011 10:51'!
drawBorderOn: aCanvas usingEnds: anArray 
	"Display my border on the canvas."
	"NOTE: Much of this code is also copied in 
	drawDashedBorderOn: 
	(should be factored)"
	| bigClipRect style |
	borderDashSpec
		ifNotNil: [^ self drawDashedBorderOn: aCanvas usingEnds: anArray].
	style := self borderStyle.
	bigClipRect := aCanvas clipRect expandBy: self borderWidth + 1 // 2.
	self
		lineSegmentsDo: [:p1 :p2 | 
			| p1i p2i |
			p1i := p1 asIntegerPoint.
			p2i := p2 asIntegerPoint.
			self hasArrows
				ifTrue: ["Shorten line ends so as not to interfere with tip 
					of arrow."
					((arrows == #back
								or: ["Shorten line ends so as not to
									interfere with tip 
									of arrow."
									arrows == #both
									"Shorten line ends so as not to
									interfere with tip 
									of arrow."])
							and: [p1 = vertices first])
						ifTrue: [p1i := anArray first asIntegerPoint].
					((arrows == #forward
								or: [arrows == #both])
							and: [p2 = vertices last])
						ifTrue: [p2i := anArray last asIntegerPoint]].
			(closed
					or: ["bigClipRect intersects: (p1i rect: p2i) 
						optimized:"
						((p1i min: p2i)
							max: bigClipRect origin)
							<= ((p1i max: p2i)
									min: bigClipRect corner)
						"bigClipRect intersects: (p1i rect: p2i) 
						optimized:"])
				ifTrue: [(p1i squaredDistanceTo: p2i) < 100000 ifTrue:
					[style
						drawLineFrom: p1i
						to: p2i
						on: aCanvas]]]! !

!DrGLocusMorph methodsFor: 'accessing' stamp: 'HilaireFernandes 4/2/2011 11:00'!
free: freeItem constrained: constrainedItem 
	| link0 link1 x0 validSamples|
	x0 := freeItem abscissa.
	worldBound := drawable worldBound.
	free := freeItem.
	constrained := constrainedItem.
	freeIndex := factory indexOf: free.
	constrainedIndex := factory indexOf: constrained.
	samples := LinkedList new.
	link0 := samples
				add: (self sample: 0.0001).
	link1 := samples
				add: (self sample: 0.9999).
	self samplingFrom: link0 to: link1.
	validSamples := self selectValidSamples.
	validSamples ifEmpty: [
		validSamples := Array with: link0 value with: link1 value].
	self costume mathItem
		points: (validSamples
				collect: [:s | s pointWorld]);
		 samples: validSamples size.
	vertices := validSamples
				collect: [:s | s pointScreen].
	free abscissa: x0.
	factory updateAllMathItemsButLocusFrom: freeIndex to: constrainedIndex.
	self computeBounds! !

!DrGLocusMorph methodsFor: 'initialization' stamp: 'HilaireFernandes 11/16/2011 09:52'!
initialize
	super initialize.
	minAbscissaDelta := 0.001.
	maxScreenDelta := 5 squared.
	maxAbscissaDelta := 0.1! !

!DrGLocusMorph methodsFor: 'sampling' stamp: 'HilaireFernandes 4/2/2011 10:23'!
sample: abscissa 
	| sample |
	sample := DrGLocusSample new x: abscissa.
	free abscissa: abscissa.
	factory updateAllMathItemsButLocusFrom: freeIndex to: constrainedIndex.
	sample pointWorld: constrained point;
		offScreen: (worldBound containsPoint: constrained point) not;  
		pointScreen: (drawable worldToMorph: constrained point).
	^ sample! !

!DrGLocusMorph methodsFor: 'sampling' stamp: 'HilaireFernandes 4/2/2011 07:19'!
samplingFrom: link1 to: link2 
	| sample1 sample2 sample link |
	sample1 := link1 value.
	sample2 := link2 value.
	(sample2 x - sample1 x > minAbscissaDelta
			and: [sample2 x - sample1 x > maxAbscissaDelta
					or: [(sample1 pointScreen squaredDistanceTo: sample2 pointScreen)
							> maxScreenDelta]])
		ifTrue: [
			sample := self sample: (sample1 x + sample2 x / 2).
			link := samples add: sample afterLink: link1.
			self samplingFrom: link1 to: link.
			self samplingFrom: link to: link2]! !

!DrGLocusMorph methodsFor: 'sampling' stamp: 'HilaireFernandes 4/2/2011 10:34'!
selectValidSamples
	| validSamples prevLink|
	validSamples := OrderedCollection new.
	samples
		linksDo: [:link | link value offScreen
				ifFalse: [validSamples add: link value]
				ifTrue: [link nextLink
						ifNotNil: [link nextLink value offScreen
								ifFalse: [validSamples add: link value]
								ifTrue: [prevLink ifNotNil: [prevLink value offScreen ifFalse: [validSamples add: link value]]]]].
				prevLink := link].
	^ validSamples ! !

DrGPolylineMorph subclass: #DrGPolygoneMorph
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'DrGeoII-Core-Item-View'!

!DrGPolygoneMorph methodsFor: 'accessing' stamp: 'HilaireFernandes 2/6/2006 21:47'!
color: aColor
	self borderColor: aColor.
	super color: (aColor alpha: 0.3)! !

!DrGPolygoneMorph methodsFor: 'as yet unclassified' stamp: 'HilaireFernandes 9/10/2009 18:49'!
containsPoint: aPoint
	(self bounds containsPoint: aPoint) ifFalse: [^ false].
	
	^ (self filledForm pixelValueAt: aPoint - bounds topLeft + 1) > 0! !

!DrGPolygoneMorph methodsFor: 'initialization' stamp: 'HilaireFernandes 2/6/2006 21:56'!
initialize
	super initialize.
	self makeClosed.! !

!DrGPolygoneMorph methodsFor: 'as yet unclassified' stamp: 'HilaireFernandes 4/30/2010 22:23'!
isCurvier
	^ false! !

!DrGPolylineMorph methodsFor: 'accessing' stamp: 'HilaireFernandes 1/22/2006 12:46'!
color: aColor
	self borderColor: aColor.
	super color: aColor.! !

!DrGPolylineMorph methodsFor: 'geometry testing' stamp: 'HilaireFernandes 6/29/2011 08:43'!
containsPoint: aPoint
	self lineSegmentsDo:
		[:p1 :p2 |
		(aPoint onLineFrom: p1 to: p2 within: 15.0)
				ifTrue: [^ true]].
	^ false! !

!DrGPolylineMorph methodsFor: 'accessing' stamp: 'HilaireFernandes 12/31/2005 10:43'!
costume
	^costume! !

!DrGPolylineMorph methodsFor: 'accessing' stamp: 'HilaireFernandes 12/31/2005 10:43'!
costume: aCostume
	costume := aCostume! !

!DrGPolylineMorph methodsFor: 'drawing' stamp: 'HilaireFernandes 1/5/2012 10:15'!
drawOn: aCanvas 
	|array |
	closed ifTrue:
		[aCanvas drawPolygon: self getVertices fillStyle: self fillStyle].
	array := self drawArrowsOn: aCanvas.
	self drawBorderOn: aCanvas usingEnds: array.
! !

!DrGPolylineMorph methodsFor: 'event handling' stamp: 'HilaireFernandes 12/15/2005 15:44'!
handlesMouseDown: evt
	^ false! !

!DrGPolylineMorph methodsFor: 'initialization' stamp: 'HilaireFernandes 11/16/2011 09:52'!
initialize
	super initialize.
	self makeOpen.
	self beStraightSegments.
	blink := false.
	self sticky: true.! !

!DrGPolylineMorph methodsFor: '*DrGeoII-Core-testing' stamp: 'HilaireFernandes 3/9/2006 17:09'!
isMorphCostume
	^true! !

!DrGPolylineMorph methodsFor: 'stepping and presenter' stamp: 'HilaireFernandes 12/30/2005 12:20'!
startBlinking
	blink := true.
	self startStepping.! !

!DrGPolylineMorph methodsFor: 'stepping and presenter' stamp: 'HilaireFernandes 1/24/2006 23:02'!
step
	blink ifFalse: [^self].
	super step.
	self borderColor = DrGCostumeStyle blinkingColor 
		ifTrue: [self borderColor: costume style color]
		ifFalse: [self borderColor: DrGCostumeStyle blinkingColor]! !

!DrGPolylineMorph methodsFor: 'stepping and presenter' stamp: 'HilaireFernandes 1/21/2006 11:42'!
stepTime
	^500! !

!DrGPolylineMorph methodsFor: 'stepping and presenter' stamp: 'HilaireFernandes 1/20/2006 16:01'!
stopBlinking
	blink := false.
	self stopStepping.! !

!DrGPolylineMorph methodsFor: 'stepping and presenter' stamp: 'HilaireFernandes 1/22/2006 16:47'!
stopStepping
	super stopStepping.
	self color: costume style color.! !

DrGPolylineMorph subclass: #DrGSegmentMorph
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'DrGeoII-Core-Item-View'!

DrGSegmentMorph subclass: #DrGLineMorph
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'DrGeoII-Core-Item-View'!

!DrGLineMorph commentStamp: '<historical>' prior: 0!
A DGLineMorph is extented to the boundary of its parent morph, a drawable in Dr. Geo jargon.!

!DrGLineMorph methodsFor: 'layout' stamp: 'HilaireFernandes 4/21/2010 08:39'!
extendFrom: aPoint direction: u
| p1 p2 drawable r|
	drawable := self costume drawable.
	r := u x asFloat / u y.
	p1 := Point x: ((drawable top - aPoint y) * r + aPoint x) asInteger y: drawable top.
	p2 := Point x: ((drawable bottom - aPoint y) * r + aPoint x) asInteger 
		y: drawable bottom.
	super from: p1 to: p2.! !

!DrGLineMorph methodsFor: 'layout' stamp: 'HilaireFernandes 12/31/2005 10:45'!
extendHorizontally: aPoint
| p1 p2 drawable|
	drawable := self costume drawable.
	p1 := Point x: drawable left y: aPoint y.
	p2 := Point x: drawable right y: aPoint y.
	super from: p1 to: p2! !

!DrGLineMorph methodsFor: 'accessing' stamp: 'HilaireFernandes 12/31/2005 11:07'!
from: aPoint direction: u
	u y abs < 0.001 
		ifTrue: [self extendHorizontally: aPoint]
		ifFalse: [self extendFrom: aPoint direction: u]! !

DrGSegmentMorph subclass: #DrGRayMorph
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'DrGeoII-Core-Item-View'!

!DrGRayMorph commentStamp: '<historical>' prior: 0!
A DGRayMorph is extended in one 'sens' to the boundary of its parents morph, a drawable in Dr. Geo   jargon.!

!DrGRayMorph methodsFor: 'layout' stamp: 'HilaireFernandes 12/31/2005 11:56'!
extendFrom: origin direction: u
| border drawable r|
	drawable := costume drawable.
	r := u x asFloat / u y.
	u y > 0
		ifTrue: [border := Point x: ((drawable bottom - origin y) * r + origin x) asInteger 
			y: drawable bottom]
		ifFalse: [border := Point x: ((drawable top - origin y) * r + origin x) asInteger 
			y: drawable top].
	super from: origin to: border.! !

!DrGRayMorph methodsFor: 'layout' stamp: 'HilaireFernandes 12/31/2005 11:56'!
extendHorizontally: origin direction: u
| border drawable|
	drawable := costume drawable.
	u x > 0
		ifTrue: [border := Point x: drawable right y: origin y]
		ifFalse: [border := Point x: drawable left y: origin y].
	super from: origin to: border! !

!DrGRayMorph methodsFor: 'accessing' stamp: 'HilaireFernandes 12/31/2005 11:05'!
from: aPoint direction: u
	u y abs < 0.001 
		ifTrue: [self extendHorizontally: aPoint direction: u]
		ifFalse: [self extendFrom: aPoint direction: u]! !

DrGSegmentMorph subclass: #DrGSegmentMarkMorph
	instanceVariableNames: 'forcedMark'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'DrGeoII-Core-Item-View'!

!DrGSegmentMarkMorph methodsFor: 'as yet unclassified' stamp: 'HilaireFernandes 11/16/2011 21:37'!
computeBounds 
	super computeBounds. 
	self costumeMark ~= #none ifTrue: [
		bounds := bounds merge: (Rectangle
			center: (vertices at: 1) + (vertices at: 2) //2
			extent: (self borderWidth * 6) )].
	self layoutChanged.
	self changed
! !

!DrGSegmentMarkMorph methodsFor: 'accessing' stamp: 'HilaireFernandes 11/16/2011 21:38'!
costumeMark
	forcedMark ifNotNil: [^forcedMark].
	costume ifNil: [^#none].
	costume style ifNil: [^#none].
	^ costume style mark ifNil: [#none]! !

!DrGSegmentMarkMorph methodsFor: 'drawing' stamp: 'HilaireFernandes 11/16/2011 20:45'!
drawCircleOn: aCanvas 
	|rectangle|
	rectangle := Rectangle 
		center:  (vertices first + vertices second / 2)
		extent: (4 @ 4 + (self borderWidth  * 2)).
	aCanvas frameOval: rectangle width: self borderWidth color: self borderColor.! !

!DrGSegmentMarkMorph methodsFor: 'drawing' stamp: 'HilaireFernandes 11/16/2011 20:45'!
drawDiscOn: aCanvas 
	| rectangle|
	rectangle := Rectangle 
		center: (vertices first + vertices second // 2)
		extent: (4 @ 4 + (self borderWidth  * 2)).
	aCanvas fillOval: rectangle color: self borderColor.! !

!DrGSegmentMarkMorph methodsFor: 'drawing' stamp: 'HilaireFernandes 11/23/2011 09:28'!
drawMark1On: aCanvas 
	|center extent vector u n startPoint endPoint |
	startPoint := vertices at: 1.
	endPoint := vertices at: 2.
	center := startPoint + endPoint / 2.
	vector := endPoint - startPoint.
	[u := vector normalized] on: Error do: [u := 0@1].
	n := u y @ u x negated.
	vector :=  (u + n ) normalized * (self borderWidth + 1 * 3).
	aCanvas 
		line: center - vector
		to: center + vector
		width: self borderWidth  
		color: self borderColor.! !

!DrGSegmentMarkMorph methodsFor: 'drawing' stamp: 'HilaireFernandes 11/23/2011 09:29'!
drawMark2On: aCanvas 
	|center extent vector u n startPoint endPoint |
	startPoint := vertices at: 1.
	endPoint := vertices at: 2.
	vector := endPoint - startPoint.
	[u := vector normalized] on: Error do: [u := 0@1].
	n := u y @ u x negated.
	vector :=  (u + n ) normalized * (self borderWidth + 1 * 3).
	center := (startPoint + endPoint / 2) - (u * (self borderWidth  * 1.5)).
	2 timesRepeat: [
		aCanvas 
			line: center - vector
			to: center + vector
			width: self borderWidth  
			color: self borderColor.
		center := center + (u * (self borderWidth  * 3))].! !

!DrGSegmentMarkMorph methodsFor: 'drawing' stamp: 'HilaireFernandes 11/23/2011 09:30'!
drawMark3On: aCanvas 
	|center extent vector u n startPoint endPoint |
	startPoint := vertices at: 1.
	endPoint := vertices at: 2.
	vector := endPoint - startPoint.
	[u := vector normalized] on: Error do: [u := 0@1].
	n := u y @ u x negated.
	vector :=  (u + n ) normalized * (self borderWidth + 1 * 3).
	center := (startPoint + endPoint / 2) - (u * (self borderWidth * 3)).
	3 timesRepeat: [
		aCanvas 
			line: center - vector
			to: center + vector
			width: self borderWidth 
			color: self borderColor.
		center := center + (u * (self borderWidth  * 3))].! !

!DrGSegmentMarkMorph methodsFor: 'drawing' stamp: 'HilaireFernandes 11/16/2011 19:57'!
drawMarkOn: aCanvas
	|mark|
	forcedMark 
		ifNotNil: [mark := forcedMark]
		ifNil: [mark := costume style mark].
	mark == #none ifTrue: [^self].
	mark == #mark1 ifTrue: [^ self drawMark1On: aCanvas].
	mark == #mark2 ifTrue: [^ self drawMark2On: aCanvas].
	mark == #mark3 ifTrue: [^ self drawMark3On: aCanvas].
	mark == #disc ifTrue: [^ self drawDiscOn: aCanvas].
	mark == #circle ifTrue: [^ self drawCircleOn: aCanvas].
	
! !

!DrGSegmentMarkMorph methodsFor: 'drawing' stamp: 'HilaireFernandes 11/16/2011 20:13'!
drawOn: aCanvas
	super drawOn: aCanvas.
	self drawMarkOn: aCanvas! !

!DrGSegmentMarkMorph methodsFor: 'accessing' stamp: 'HilaireFernandes 11/16/2011 19:57'!
forcedMark: symbol
	forcedMark := symbol! !

!DrGSegmentMorph methodsFor: 'accessing' stamp: 'HilaireFernandes 11/16/2011 17:53'!
from: startPoint to: endPoint  
	| clippedSeg|
	clippedSeg := self costume drawable bounds
				intersectionWith: (LineSegment from: startPoint to: endPoint).
	clippedSeg
		ifNotNil: [
			self visible ifFalse: [ self show].
			vertices at: 1 put: clippedSeg start asIntegerPoint.
			vertices at: 2 put: clippedSeg end asIntegerPoint.
			self computeBounds]
		ifNil: [self visible ifTrue: [self hide]]! !

!DrGSegmentMorph methodsFor: 'initializing' stamp: 'HilaireFernandes 11/16/2011 19:58'!
initialize
	super initialize.
	vertices := Array with: 0@0 with: 0@0.! !

DrGSegmentMorph subclass: #DrGVectorMorph
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'DrGeoII-Core-Item-View'!

!DrGVectorMorph methodsFor: 'accessing' stamp: 'HilaireFernandes 1/28/2008 22:40'!
from: startPoint to: endPoint
	"We don't calculate clipped vector, otherwise the arrow may be misplaced when part of the vector extremity is outisde the drawing area'"
	vertices at: 1 put: startPoint asIntegerPoint .
	vertices at: 2 put: endPoint  asIntegerPoint.
	self computeBounds
! !

!DrGVectorMorph methodsFor: 'initializing' stamp: 'HilaireFernandes 1/2/2006 09:49'!
initialize
	super initialize.
	arrows := #forward.! !

!SequenceableCollection methodsFor: '*DrGeoII-Core' stamp: 'HilaireFernandes 4/19/2011 22:15'!
shiftRight
	^ self allButFirst, (Array with: self first)! !

!SettingTreeBuilder methodsFor: '*DrGeoII-Core-pragmas' stamp: 'HilaireFernandes 12/21/2011 10:49'!
drgeosettings
	self systemsettings! !

!MouseOverHandler methodsFor: '*DrGeoII-Core-accessing' stamp: 'HilaireFernandes 2/22/2006 21:27'!
mouseOverMorphs
	^mouseOverMorphs! !

StringMorph subclass: #DrGStringMorph
	instanceVariableNames: 'costume blink'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'DrGeoII-Core-Item-View'!

DrGStringMorph subclass: #DrGLabelMorph
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'DrGeoII-Core-Item-View'!

!DrGLabelMorph methodsFor: 'editing' stamp: 'HilaireFernandes 3/12/2006 21:47'!
acceptContents
	costume mathItem name: contents asString.! !

!DrGLabelMorph methodsFor: 'event handling' stamp: 'HilaireFernandes 2/22/2006 15:39'!
mouseMove: evt
	costume 
		ifNotNil: 
			[costume constraintTextPosition: evt cursorPoint.
			self position: costume textPosition].
! !

!DrGStringMorph methodsFor: 'accessing' stamp: 'HilaireFernandes 1/25/2006 17:53'!
costume
	"Answer the value of costume"

	^ costume! !

!DrGStringMorph methodsFor: 'accessing' stamp: 'CedrickBeler 4/23/2010 16:49'!
costume: anObject
	"Set the value of costume"

	costume := anObject! !

!DrGStringMorph methodsFor: 'event handling' stamp: 'HilaireFernandes 2/4/2006 12:17'!
handlesMouseDown: evt
	^true! !

!DrGStringMorph methodsFor: 'initialization' stamp: 'HilaireFernandes 1/26/2008 14:38'!
initialize
	super initialize.
	blink := false.
	self font: DrGDefault font
! !

!DrGStringMorph methodsFor: 'stepping and presenter' stamp: 'HilaireFernandes 2/4/2006 12:04'!
startBlinking
	blink := true.
	self startStepping ! !

!DrGStringMorph methodsFor: 'stepping and presenter' stamp: 'HilaireFernandes 2/4/2006 12:04'!
step
	blink ifFalse: [^self].
	super step.
	self color = DrGCostumeStyle blinkingColor 
		ifTrue: [self color: costume style color]
		ifFalse: [self color: DrGCostumeStyle blinkingColor]! !

!DrGStringMorph methodsFor: 'testing' stamp: 'HilaireFernandes 2/4/2006 12:04'!
stepTime
	^500! !

!DrGStringMorph methodsFor: 'stepping and presenter' stamp: 'HilaireFernandes 2/4/2006 12:04'!
stopBlinking
	blink := false.
	self stopStepping.! !

!DrGStringMorph methodsFor: 'stepping and presenter' stamp: 'HilaireFernandes 2/4/2006 12:04'!
stopStepping
	super stopStepping.
	self color: costume style color.! !

DrGStringMorph subclass: #DrGValueMorph
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'DrGeoII-Core-Item-View'!

!DrGValueMorph methodsFor: 'editing' stamp: 'HilaireFernandes 8/5/2011 15:45'!
acceptContents
	costume mathItem valueItem: contents asNumber.
	self contents: costume roundedValueString.! !

!DrGValueMorph methodsFor: 'events-processing' stamp: 'HilaireFernandes 3/12/2006 18:54'!
handleMouseDown: anEvent
	^false! !

!DrGValueMorph methodsFor: '*DrGeoII-Core-testing' stamp: 'HilaireFernandes 3/9/2006 17:10'!
isMorphCostume
	^true! !

!DrGValueMorph methodsFor: 'editing' stamp: 'HilaireFernandes 4/20/2011 22:32'!
lostFocusWithoutAccepting
	"no update in that case"
	^ self! !

TextMorph subclass: #DrGTextMorph
	instanceVariableNames: 'costume blink'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'DrGeoII-Core-Item-View'!

!DrGTextMorph methodsFor: 'accessing' stamp: 'HilaireFernandes 11/5/2011 18:57'!
color: aColor
	self textColor: aColor! !

!DrGTextMorph methodsFor: 'accessing' stamp: 'HilaireFernandes 11/5/2011 17:48'!
costume
	^ costume! !

!DrGTextMorph methodsFor: 'accessing' stamp: 'HilaireFernandes 11/5/2011 17:42'!
costume: aDrGTextCostume 
	costume := aDrGTextCostume! !

!DrGTextMorph methodsFor: 'events-processing' stamp: 'HilaireFernandes 11/9/2011 11:39'!
handleMouseMove: anEvent
	^ self isOnPlaceEditable ifTrue: [super handleMouseMove: anEvent]! !

!DrGTextMorph methodsFor: 'events-processing' stamp: 'HilaireFernandes 11/6/2011 12:46'!
handlesMouseDown: anEvent
	^ self isOnPlaceEditable ! !

!DrGTextMorph methodsFor: 'events-processing' stamp: 'HilaireFernandes 11/6/2011 12:46'!
handlesMouseOver: anEvent
	^ self isOnPlaceEditable ! !

!DrGTextMorph methodsFor: 'initialization' stamp: 'HilaireFernandes 11/6/2011 12:45'!
initialize 
	super initialize.
	blink := false.
	self crAction: (MessageSend receiver: self selector: #ok).! !

!DrGTextMorph methodsFor: 'as yet unclassified' stamp: 'HilaireFernandes 11/6/2011 12:46'!
isOnPlaceEditable
	^ self crAction notNil! !

!DrGTextMorph methodsFor: 'as yet unclassified' stamp: 'HilaireFernandes 11/6/2011 12:52'!
ok
	self crAction: nil.
	self costume mathItem 
		text: self contents string;
		changed
	! !

!DrGTextMorph methodsFor: 'stepping and presenter' stamp: 'HilaireFernandes 11/5/2011 18:47'!
startBlinking
	blink := true.
	self startStepping ! !

!DrGTextMorph methodsFor: 'stepping and presenter' stamp: 'HilaireFernandes 11/5/2011 18:47'!
step
	blink ifFalse: [^self].
	super step.
	self color = DrGCostumeStyle blinkingColor 
		ifTrue: [self color: costume style color]
		ifFalse: [self color: DrGCostumeStyle blinkingColor]! !

!DrGTextMorph methodsFor: 'stepping and presenter' stamp: 'HilaireFernandes 11/5/2011 18:47'!
stepTime
	^500! !

!DrGTextMorph methodsFor: 'stepping and presenter' stamp: 'HilaireFernandes 11/5/2011 18:48'!
stopBlinking
	blink := false.
	self stopStepping.! !

!DrGTextMorph methodsFor: 'stepping and presenter' stamp: 'HilaireFernandes 11/5/2011 18:48'!
stopStepping
	super stopStepping.
	self color: costume style color.! !

Morph subclass: #DrGWizardPage
	uses: TEasilyThemed - {#theme}
	instanceVariableNames: 'cancel previous nextOrApply buttons model content title'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'DrGeoII-Core-UI'!

!DrGWizardPage commentStamp: 'HilaireFernandes 7/20/2008 19:44' prior: 0!
A DrGWizardPage is a page morph to add in a wizard dialog..
Content can be added with the #content: message
Alternatively, it can be subclassed this class, to do so
override intialiaze with super call and at the end of 
initialize use #content: message

Instance Variables
	buttons:		<Object>
	cancel:		<Object>
	content:		<Object>
	model:		<Object>
	nextOrApply:		<Object>
	previous:		<Object>

buttons
	- xxxxx

cancel
	- xxxxx

content
	- xxxxx

model
	- xxxxx

nextOrApply
	- xxxxx

previous
	- xxxxx
!

DrGWizardPage class
	uses: TEasilyThemed classTrait
	instanceVariableNames: ''!

!DrGWizardPage class methodsFor: 'as yet unclassified' stamp: 'HilaireFernandes 7/17/2008 11:25'!
applyPage
	"page with cancel button, previous button, and apply button"! !

!DrGWizardPage class methodsFor: 'as yet unclassified' stamp: 'HilaireFernandes 7/17/2008 16:27'!
applyPage: title
	"page with cancel button, previous button, and apply button"
		^ self new applyPage: title! !

!DrGWizardPage class methodsFor: 'as yet unclassified' stamp: 'HilaireFernandes 7/17/2008 16:24'!
firstPage: title
	"page with cancel button, gray and locked previous button, and next button"
	^ self new firstPage: title! !

!DrGWizardPage class methodsFor: 'as yet unclassified' stamp: 'HilaireFernandes 7/17/2008 16:04'!
lastPage: title
	"page with cancel button, previous button, and gray and locked next button"
	^self new lastPage: title! !

!DrGWizardPage class methodsFor: 'instance creation' stamp: 'HilaireFernandes 7/17/2008 13:12'!
standardPage: title
	"page with cancel, previous, next buttons"
	^ self new standardPage: title.! !

!DrGWizardPage methodsFor: 'button' stamp: 'HilaireFernandes 7/17/2008 12:50'!
addButton: btn
	buttons addMorphBack: btn! !

!DrGWizardPage methodsFor: 'callback' stamp: 'HilaireFernandes 7/17/2008 16:39'!
apply
	model apply! !

!DrGWizardPage methodsFor: 'button' stamp: 'HilaireFernandes 7/6/2010 15:29'!
applyBtn
	nextOrApply := self newButtonFor: self
		action: #apply 
		label:  'apply' translated
		help: nil.
	^nextOrApply ! !

!DrGWizardPage methodsFor: 'initialization' stamp: 'HilaireFernandes 7/17/2008 16:57'!
applyPage: title
	self title: title.
	self 
		addButton: self cancelBtn;
		addButton:  self previousBtn;
		addButton: self applyBtn.! !

!DrGWizardPage methodsFor: 'button' stamp: 'HilaireFernandes 7/6/2010 15:59'!
buttons
	buttons := Morph new.
	buttons color: Color transparent.
	buttons layoutPolicy: TableLayout new;
		listDirection: #leftToRight;
		vResizing: #shrinkWrap;
		hResizing: #spaceFill;
		cellInset: 5;
		listCentering: #bottomRight.
	^ buttons! !

!DrGWizardPage methodsFor: 'callback' stamp: 'HilaireFernandes 7/17/2008 16:39'!
cancel
	model cancel! !

!DrGWizardPage methodsFor: 'button' stamp: 'HilaireFernandes 7/6/2010 15:28'!
cancelBtn
	cancel := self newCancelButtonFor: self.
 	^ cancel! !

!DrGWizardPage methodsFor: 'accessing' stamp: 'HilaireFernandes 8/4/2008 10:27'!
content: aMorph
	aMorph 
		vResizing: #spaceFill;
		hResizing: #spaceFill.
	content 
		ifNotNil: [self replaceSubmorph: content by: aMorph]
		ifNil: [self addMorph: aMorph].
	content := aMorph.
	content isTextMorph ifTrue: 
		[content wrapFlag: true.]! !

!DrGWizardPage methodsFor: 'accessing' stamp: 'HilaireFernandes 8/22/2008 16:41'!
defaultContent
	self content: (Morph new
		color:  (Color r: 0.85 g: 0.9 b: 1.0)).! !

!DrGWizardPage methodsFor: 'submorphs-add/remove' stamp: 'HilaireFernandes 7/28/2008 10:01'!
delete
	super delete.
	model := nil.
	content release; delete! !

!DrGWizardPage methodsFor: 'initialization' stamp: 'HilaireFernandes 7/6/2010 15:26'!
firstPage: title
	self title: title.
	self 
		addButton: self cancelBtn;
		addButton: (self previousBtn disable);
		addButton: self nextBtn! !

!DrGWizardPage methodsFor: 'initialization' stamp: 'HilaireFernandes 7/6/2010 17:22'!
initialize
	super initialize.
	self color: Color transparent .
	self clipSubmorphs: true.
	self layoutPolicy: TableLayout new;
		listDirection: #topToBottom;
		hResizing: #spaceFill;
		vResizing:  #spaceFill;
		layoutInset: 3.
	self addMorph: self buttons.
	self defaultContent.! !

!DrGWizardPage methodsFor: 'initialization' stamp: 'HilaireFernandes 7/6/2010 15:26'!
lastPage: title
	self title: title.
	self
		addButton: self cancelBtn;
		addButton: self previousBtn;
		addButton: (self nextBtn disable)! !

!DrGWizardPage methodsFor: 'accessing' stamp: 'HilaireFernandes 7/17/2008 16:39'!
model: aMacroMorph 
	model := aMacroMorph! !

!DrGWizardPage methodsFor: 'callback' stamp: 'HilaireFernandes 7/17/2008 16:39'!
next
	model next! !

!DrGWizardPage methodsFor: 'button' stamp: 'HilaireFernandes 7/6/2010 15:28'!
nextBtn
	nextOrApply := self newButtonFor: self
		action: #next 
		label:  'next' translated
		help: nil.
	^ nextOrApply! !

!DrGWizardPage methodsFor: 'callback' stamp: 'HilaireFernandes 7/17/2008 16:39'!
previous
	model previous! !

!DrGWizardPage methodsFor: 'button' stamp: 'HilaireFernandes 7/6/2010 15:28'!
previousBtn
	previous := self newButtonFor: self
		action: #previous 
		label:  'previous' translated
		help: nil.
	^ previous! !

!DrGWizardPage methodsFor: 'initialization' stamp: 'HilaireFernandes 7/17/2008 16:58'!
standardPage: title
	self title: title.
	self 
		addButton: self cancelBtn;
		addButton: self previousBtn;
		addButton: self nextBtn! !

!DrGWizardPage methodsFor: 'accessing' stamp: 'HilaireFernandes 7/6/2010 16:14'!
title
	^ title! !

!DrGWizardPage methodsFor: 'accessing' stamp: 'HilaireFernandes 7/6/2010 16:12'!
title: aTitle
	title := aTitle! !

!Morph methodsFor: '*DrGeoII-Core-testing' stamp: 'HilaireFernandes 3/9/2006 17:11'!
isMorphCostume
	^false! !

!Point methodsFor: '*DrGeoII-Core-printing' stamp: 'HilaireFernandes 4/8/2011 19:38'!
printShowingDecimalPlaces: placeDesired 
"do not round when coordinate is a fraction"
	|stringX stringY|
	x isFraction
		ifTrue: [stringX := x printString]
		ifFalse: [stringX := x printShowingDecimalPlaces: placeDesired].
	y isFraction
		ifTrue: [stringY := y printString] 
		ifFalse: [stringY := y printShowingDecimalPlaces: placeDesired].		
	^ stringX, '@', stringY! !

!Point methodsFor: '*DrGeoII-Core-point functions' stamp: 'HilaireFernandes 1/3/2006 14:06'!
reflexionBy: aPoint direction: aDirection
"the reflected point according to the axe defined by aPoint and aDirection"
	^ ((self - aPoint) vectorReflexionBy: aDirection) + aPoint! !

!Point methodsFor: '*DrGeoII-Core-point functions' stamp: 'HilaireFernandes 2/7/2006 21:56'!
rotateVector: anAngle
|cosa sina|
	cosa := anAngle cos.
	sina := anAngle sin.
	^ (x * cosa - (y * sina))@(x * sina + (y * cosa))! !

!Point methodsFor: '*DrGeoII-Core-point functions' stamp: 'HilaireFernandes 1/3/2006 13:47'!
vectorReflexionBy: aDirection
"the reflected vector according to the axe defined by aPoint and aVector"
|unit normal|
	unit := aDirection normalized.
	normal := unit normal.
	^((self dotProduct: unit) * unit) - ((self dotProduct: normal) * normal)
! !

!UndefinedObject methodsFor: '*DrGeoII-Core-accessing' stamp: 'HilaireFernandes 4/19/2010 10:27'!
area
	^nil! !

!UndefinedObject methodsFor: '*DrGeoII-Core-accessing' stamp: 'HilaireFernandes 4/19/2010 12:46'!
gridVisible
	^false! !

DialogWindow subclass: #DrGWizard
	instanceVariableNames: 'pages currentPage panel'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'DrGeoII-Core-UI'!

!DrGWizard commentStamp: 'HilaireFernandes 7/20/2008 10:58' prior: 0!
A DrGMacroMorph is an abstractact dialog to build and play macro.
It emits event when 
 - the user changes the page with the next and previous buttons
 - the user press apply and cancel

Instance Variables
	pages:		dictionary of pages Morph

pages
	- xxxxx
!

!DrGWizard class methodsFor: 'as yet unclassified' stamp: 'HilaireFernandes 7/20/2008 19:51'!
example
	"DrGWizard example openInWorld"
	| wizard |
	wizard := self new.
	wizard
		addPage: ((DrGWizardPage firstPage: 'Step 1/3') model: wizard);
		addPage: ((DrGWizardPage standardPage: 'Step 2/3') model: wizard;
			content: 'We can change content :
	1. One
	2. Two
	3. Three' asTextMorph);
		addPage: ((DrGWizardPage applyPage: 'Step 3/3') model: wizard).
	^ wizard! !

!DrGWizard methodsFor: 'accessing' stamp: 'HilaireFernandes 7/6/2010 17:07'!
addPage: aPage
	pages add: aPage.
	currentPage ifNil: 
		[self panel addMorph: aPage.
		self title: aPage title.
		currentPage := 1]! !

!DrGWizard methodsFor: 'callback' stamp: 'HilaireFernandes 7/20/2008 10:10'!
apply
	self triggerEvent: #apply! !

!DrGWizard methodsFor: 'callback' stamp: 'HilaireFernandes 7/20/2008 10:10'!
cancel
	self triggerEvent: #cancel! !

!DrGWizard methodsFor: 'meta-actions' stamp: 'HilaireFernandes 7/28/2008 10:07'!
delete
	super delete.
	pages do: [:page | page delete].! !

!DrGWizard methodsFor: 'accessing' stamp: 'HilaireFernandes 7/6/2010 17:09'!
goPage: integer
"	(contentMorph findA: DrGWizardPage) ifNotNilDo: 
		[:page | self removeMorph: page]."
	self panel removeAllMorphs.
	self panel addMorph: (self pageAt: integer).
	self title: (self pageAt: integer) title.
	self triggerEvent: #page with: integer.! !

!DrGWizard methodsFor: 'initialization' stamp: 'HilaireFernandes 8/4/2010 11:17'!
initialExtent
	^550@350! !

!DrGWizard methodsFor: 'initialization' stamp: 'HilaireFernandes 11/11/2010 14:37'!
initialize
	super initialize.
	pages := OrderedCollection new.
	self setProperty: #morphicLayerNumber toValue: 5! !

!DrGWizard methodsFor: 'testing' stamp: 'HilaireFernandes 7/6/2010 16:31'!
isResizeable 
	^ true! !

!DrGWizard methodsFor: 'actions' stamp: 'HilaireFernandes 7/6/2010 17:08'!
newMainPanel
	^ self panel! !

!DrGWizard methodsFor: 'callback' stamp: 'HilaireFernandes 7/20/2008 10:09'!
next
	currentPage < pages size ifFalse: [^ self].
	self goPage: (currentPage := currentPage + 1)! !

!DrGWizard methodsFor: 'accessing' stamp: 'HilaireFernandes 7/20/2008 12:10'!
pageAt: integer
	^ pages at: integer ! !

!DrGWizard methodsFor: 'initialization' stamp: 'HilaireFernandes 7/6/2010 17:34'!
pageModel
	|page|
	page := Morph new color: Color transparent .
	page
		layoutPolicy: TableLayout new;
		listDirection: #bottomToTop;
		cellPositioning: #topLeft;
		layoutInset: 0@3;
		cellInset: 0@5.
	^ page! !

!DrGWizard methodsFor: 'actions' stamp: 'HilaireFernandes 7/6/2010 17:08'!
panel
	^ panel ifNil: [ panel := self newDialogPanel]! !

!DrGWizard methodsFor: 'callback' stamp: 'HilaireFernandes 7/20/2008 10:10'!
previous
	currentPage > 1 ifFalse: [^self].
	self goPage: (currentPage := currentPage - 1)! !

DrGWizard subclass: #DrGWizardMacroBuild
	instanceVariableNames: 'builder'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'DrGeoII-Core-UI'!

!DrGWizardMacroBuild class methodsFor: 'instance creation' stamp: 'HilaireFernandes 7/25/2008 10:06'!
builder: aBuilder 
	^ self basicNew
		builder: aBuilder;
		initialize ! !

!DrGWizardMacroBuild methodsFor: 'accessing' stamp: 'HilaireFernandes 7/22/2008 13:19'!
builder: aBuilder
	builder := aBuilder ! !

!DrGWizardMacroBuild methodsFor: 'meta-actions' stamp: 'HilaireFernandes 7/28/2008 10:07'!
delete
	super delete.
	builder := nil.! !

!DrGWizardMacroBuild methodsFor: 'initialization' stamp: 'HilaireFernandes 7/28/2008 11:03'!
firstPage
	^ (DrGWizardPage firstPage: 'Build a macro-construction' translated) 
				model: self;
				content: 'To build a macro-construction,

 1. First select the input paramaters,
 2. Next select the output parameters,
 3. Next chose a name and a description,
 4. Last apply the selection.' translated asTextMorph lock! !

!DrGWizardMacroBuild methodsFor: 'initialization' stamp: 'HilaireFernandes 7/6/2010 17:35'!
fourthPage
	| page morph |
	page := self pageModel.
	
	page addMorph: 'Title:' translated asStringMorph.
	morph := PluggableTextMorph 
		on: builder text: #title accept: #title:.
	morph acceptTextInModel.
	morph
		color: Color white;
		acceptOnCR: true;
		hResizing: #spaceFill;
		height: 16;
		hideScrollBarsIndefinitely.
	page addMorph: morph.
	page addMorph: 'Description:' translated asStringMorph.
	morph := PluggableTextMorph 
		on: builder text: #description accept: #description:.
	morph
		color: Color white;
		hResizing: #spaceFill;
		vResizing: #spaceFill.
	page addMorph: morph.
	morph acceptTextInModel.
	^(DrGWizardPage applyPage: 'Give a name and a description' translated) 
				model: self;
				content: page! !

!DrGWizardMacroBuild methodsFor: 'initialization' stamp: 'HilaireFernandes 7/6/2010 17:27'!
initialize
	super initialize.
	self 
		addPage: self firstPage;
		addPage: self secondPage;
		addPage: self thirdPage;
		addPage: self fourthPage.
		! !

!DrGWizardMacroBuild methodsFor: 'testing' stamp: 'HilaireFernandes 7/20/2008 18:05'!
isInputPage
	^ currentPage  = 2! !

!DrGWizardMacroBuild methodsFor: 'testing' stamp: 'HilaireFernandes 7/20/2008 18:05'!
isOutputPage
	^ currentPage  = 3! !

!DrGWizardMacroBuild methodsFor: 'initialization' stamp: 'HilaireFernandes 7/6/2010 17:39'!
secondPage
	| list page |
	page := self pageModel.
	list := PluggableListMorph 
		on: builder
		list: #inputName 
		selected: nil 
		changeSelected: nil.
	list color: Color white;
		hResizing: #spaceFill;
		vResizing: #spaceFill.
	page addMorph: list.
	^ (DrGWizardPage new standardPage: 'Select input parameters' translated) 
		model: self;
		content: page! !

!DrGWizardMacroBuild methodsFor: 'initialization' stamp: 'HilaireFernandes 7/6/2010 17:40'!
thirdPage
	| list page |
	page := self pageModel.
	list := PluggableListMorph 
		on: builder 
		list: #outputName
		selected: nil 
		changeSelected: nil.
	list color: Color white;
		hResizing: #spaceFill;
		vResizing: #spaceFill.
	page addMorph: list .
	^ (DrGWizardPage new standardPage: 'Select output parameters' translated) 
		model: self;
		content: page! !

DrGWizard subclass: #DrGWizardMacroPlay
	instanceVariableNames: 'builder macroFactories'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'DrGeoII-Core-UI'!

!DrGWizardMacroPlay class methodsFor: 'instance creation' stamp: 'HilaireFernandes 7/28/2008 11:59'!
builder: aBuilder 
	^ self basicNew
		builder: aBuilder;
		initialize ! !

!DrGWizardMacroPlay methodsFor: 'accessing' stamp: 'HilaireFernandes 7/28/2008 12:19'!
builder: aBuilder
	builder := aBuilder! !

!DrGWizardMacroPlay methodsFor: 'meta-actions' stamp: 'HilaireFernandes 7/28/2008 10:48'!
delete
	super delete.
	builder := nil.! !

!DrGWizardMacroPlay methodsFor: 'initialization' stamp: 'HilaireFernandes 8/4/2008 10:10'!
firstPage
	^ (DrGWizardPage firstPage: 'Execute a macro-construction' translated) 
				model: self;
				content: 'To execute a macro-construction,

 1. First select a macro-construction from the list,
 2. Select items on the figure. Only items relevant
 to the selected macro-construction are selectable.
 Once enought items are selected, the macro is
 automaticly executed.
 To start press the ''next'' button.' translated asTextMorph lock! !

!DrGWizardMacroPlay methodsFor: 'initialization' stamp: 'HilaireFernandes 10/17/2010 10:30'!
initialize
	super initialize.
	macroFactories := DrGMacroFactory new.
	self
		addPage: self firstPage;
		addPage: self secondPage.! !

!DrGWizardMacroPlay methodsFor: 'testing' stamp: 'HilaireFernandes 7/28/2008 11:10'!
isSelectPage
	^ currentPage = 2! !

!DrGWizardMacroPlay methodsFor: 'accessing' stamp: 'HilaireFernandes 8/22/2008 22:04'!
macroList
	^ macroFactories titles! !

!DrGWizardMacroPlay methodsFor: 'initialization' stamp: 'HilaireFernandes 7/6/2010 17:30'!
secondPage
	|list page morph |
	page := Morph new color: Color transparent.
	page
		layoutPolicy: TableLayout new;
		listDirection: #bottomToTop;
		cellPositioning: #topLeft;
		layoutInset: 0@3;
		cellInset: 0@5.
	page addMorph: 'Title:' translated asStringMorph.
	list := PluggableListMorph
		on: self
		list: #macroList
		selected: #selectedMacro
		changeSelected: #selectedMacro:.
	list hResizing: #spaceFill;
		height: 50;
		color: Color white.
	page addMorph: list.
	page addMorph: 'Description:' translated asStringMorph.
	morph := PluggableTextMorph 
		on: builder text: #description accept: #description:.
	morph
		hResizing: #spaceFill;
		vResizing: #spaceFill;
		color: Color white.
	page addMorph: morph.
	morph acceptTextInModel.
	^(DrGWizardPage lastPage: 'Select a macro-construction then the figure items' translated) 
		model: self;
		content: page! !

!DrGWizardMacroPlay methodsFor: 'accessing' stamp: 'HilaireFernandes 8/22/2008 22:04'!
selectedMacro
	^ macroFactories indexOf: builder selectedMacro! !

!DrGWizardMacroPlay methodsFor: 'accessing' stamp: 'HilaireFernandes 8/22/2008 22:04'!
selectedMacro: anIndex
	builder selectedMacro: (macroFactories at: anIndex).
	self changed: #selectedMacro.
	builder changed: #description.! !

DrGWizard subclass: #DrGWizardScript
	instanceVariableNames: 'builder methodList instanceList'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'DrGeoII-Core-UI'!

!DrGWizardScript class methodsFor: 'instance creation' stamp: 'HilaireFernandes 8/2/2008 21:40'!
builder: aBuilder 
	^ self basicNew
		builder: aBuilder;
		initialize ! !

!DrGWizardScript methodsFor: 'accessing' stamp: 'HilaireFernandes 8/2/2008 20:46'!
builder: anObject
	builder := anObject ! !

!DrGWizardScript methodsFor: 'meta-actions' stamp: 'HilaireFernandes 8/2/2008 21:38'!
delete
	super delete.
	builder := nil! !

!DrGWizardScript methodsFor: 'initialization' stamp: 'HilaireFernandes 11/11/2010 14:52'!
firstPage
	^ (DrGWizardPage firstPage: 'Use a script in the figure' translated) 
				model: self;
				content: 'To use a script,

 1. First select a script from the list,
 2. Select items on the figure,
3.  Clic somewhere in the background. 
 Once enought items are selected, the script is
 inserted in the figure, at the user selected position.
 To start press the ''next'' button.' translated asTextMorph lock! !

!DrGWizardScript methodsFor: 'accessing' stamp: 'HilaireFernandes 8/4/2008 11:36'!
goPage: integer
	super goPage: integer.
	self changed: #methodList! !

!DrGWizardScript methodsFor: 'initialization' stamp: 'HilaireFernandes 7/6/2010 17:27'!
initialize
	super initialize.
	self 
		addPage: self firstPage;
		addPage: self secondPage.! !

!DrGWizardScript methodsFor: 'accessing' stamp: 'HilaireFernandes 9/27/2008 18:47'!
instanceList
	^ instanceList
		ifNil: [instanceList := DrGeoScripts instance list]! !

!DrGWizardScript methodsFor: 'testing' stamp: 'HilaireFernandes 8/2/2008 21:40'!
isSelectPage
	^ currentPage = 2! !

!DrGWizardScript methodsFor: 'accessing' stamp: 'HilaireFernandes 9/27/2008 18:47'!
methodList
	^ methodList ifNil: [methodList := DrGeoScripts instance selectors]! !

!DrGWizardScript methodsFor: 'initialization' stamp: 'HilaireFernandes 7/6/2010 17:29'!
secondPage
	| list page morph |
	page := Morph new color: Color transparent.
	page layoutPolicy: TableLayout new;
		 listDirection: #bottomToTop;
		 cellPositioning: #topLeft;
		 layoutInset: 3 @ 0;
		cellInset: 0@5.
	page addMorph: 'Script name:' translated asStringMorph.
	list := PluggableListMorph
				on: self
				list: #methodList
				selected: #selectedMethod
				changeSelected: #selectedMethod:.
	list hResizing: #spaceFill;
		 height: 80;
		color: Color white.
	page addMorph: list.
	page addMorph: 'Description:' translated asStringMorph.
	morph := PluggableTextMorph
				on: builder
				text: #description
				accept: #description:.
	morph hResizing: #spaceFill;
		color: Color white;
		vResizing: #spaceFill.
	page addMorph: morph.
	morph acceptTextInModel.
	^ (DrGWizardPage lastPage: 'Select a script then the figure items' translated) model: self;
		 content: page! !

!DrGWizardScript methodsFor: 'accessing' stamp: 'HilaireFernandes 8/4/2008 10:07'!
selectedMethod
	^ builder selectedMethod 
		ifNil: [0]
		ifNotNil: [self methodList indexOf: builder selectedMethod selector asString]! !

!DrGWizardScript methodsFor: 'accessing' stamp: 'HilaireFernandes 9/27/2008 18:48'!
selectedMethod: anIndex 
	builder
		selectedMethod: (self instanceList
				at: anIndex
				ifAbsent: []).
	self changed: #selectedMethod.
	builder changed: #description! !

Object subclass: #Carre
	instanceVariableNames: 'canvas'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'DrGeoII-Core-Script'!

!Carre methodsFor: 'as yet unclassified' stamp: 'HilaireFernandes 1/22/2008 22:28'!
carre: s1 and: s2 and: s3 and: s4 n: integer 
	integer > 0
		ifTrue: [self
				carre: (self segment: s1 and: s2)
				and: (self segment: s2 and: s3)
				and: (self segment: s3 and: s4)
				and: (self segment: s4 and: s1)
				n: integer - 1]! !

!Carre methodsFor: 'initialize-release' stamp: 'HilaireFernandes 8/10/2008 12:18'!
initialize
	| p1 |
	canvas := DrGeoCanvas new.
	self
		carre: (canvas
				segment: (p1 := canvas point: -2 @ 2)
				to: (canvas point: 2 @ 2))
		and: (canvas
				segment: (canvas point: 2 @ 2)
				to: (canvas point: 2 @ -2))
		and: (canvas
				segment: (canvas point: 2 @ -2)
				to: (canvas point: -2 @ -2))
		and: (canvas
				segment: (canvas point: -2 @ -2)
				to: (canvas point: -2 @ 2))
		n: 30.
	"MessageTally
		spyOn: [canvas domain moveEvent: p1 mathItem inDirection: 1 @ -1]"! !

!Carre methodsFor: 'as yet unclassified' stamp: 'HilaireFernandes 1/22/2008 22:24'!
segment: s1 and: s2 
	| p1 p2 |
	p1 := canvas pointOnCurve: s1 at: 1 / 8.
	p2 := canvas pointOnCurve: s2 at: 1 / 8.
	^ canvas segment: p1 hide to: p2 hide! !

Object subclass: #DrGCommand
	instanceVariableNames: 'domain'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'DrGeoII-Core-Command'!

!DrGCommand methodsFor: 'accessing' stamp: 'HilaireFernandes 12/26/2005 09:01'!
domain: aDomain
	domain := aDomain.! !

!DrGCommand methodsFor: 'command' stamp: 'HilaireFernandes 1/30/2006 23:52'!
execute
	self subclassResponsibility ! !

!DrGCommand methodsFor: 'command' stamp: 'HilaireFernandes 1/30/2006 23:52'!
unexecute
	self subclassResponsibility ! !

DrGCommand subclass: #DrGCompositeCommand
	instanceVariableNames: 'compositeItem mathItems'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'DrGeoII-Core-Command'!

!DrGCompositeCommand class methodsFor: 'as yet unclassified' stamp: 'HilaireFernandes 3/2/2006 22:02'!
with: theMathItems in: aDomain
	^self new
		domain: aDomain ;
		compileComposite: theMathItems! !

!DrGCompositeCommand methodsFor: 'as yet unclassified' stamp: 'HilaireFernandes 3/2/2006 18:18'!
compileComposite: theMathItems
	mathItems := theMathItems.
	compositeItem := DrGCompositeItem newWith: theMathItems.! !

!DrGCompositeCommand methodsFor: 'as yet unclassified' stamp: 'HilaireFernandes 3/2/2006 22:22'!
execute
	domain triggerEvent: #groupCostumes with: compositeItem.! !

!DrGCompositeCommand methodsFor: 'as yet unclassified' stamp: 'HilaireFernandes 8/18/2008 12:47'!
release
	super release.
	compositeItem release.
! !

!DrGCompositeCommand methodsFor: 'as yet unclassified' stamp: 'HilaireFernandes 3/2/2006 22:22'!
unexecute
	domain triggerEvent: #ungroupCostumes with: compositeItem.! !

DrGCommand subclass: #DrGDeleteCommand
	instanceVariableNames: 'mathItems'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'DrGeoII-Core-Command'!

!DrGDeleteCommand commentStamp: '<historical>' prior: 0!
A command to delete a mathItem and its dependant.!

!DrGDeleteCommand class methodsFor: 'as yet unclassified' stamp: 'HilaireFernandes 1/24/2006 10:23'!
for: aMathItem in: aDomain
	^self new
		domain: aDomain;
		deleteMathItem: aMathItem.! !

!DrGDeleteCommand methodsFor: 'as yet unclassified' stamp: 'HilaireFernandes 11/5/2011 23:19'!
deleteMathItem: aMathItem 
	mathItems := domain factory orderedDependentMathItem: aMathItem.! !

!DrGDeleteCommand methodsFor: 'command' stamp: 'HilaireFernandes 1/24/2006 14:25'!
execute
	domain factory remove: mathItems! !

!DrGDeleteCommand methodsFor: 'as yet unclassified' stamp: 'HilaireFernandes 8/18/2008 12:47'!
release
	super release.
	mathItems do: [:m | m release].! !

!DrGDeleteCommand methodsFor: 'command' stamp: 'HilaireFernandes 9/20/2010 22:43'!
unexecute
	mathItems do: [:aMathItem|
		domain factory add: aMathItem.
		domain triggerEvent: #redoMathItem with: aMathItem]
	! !

DrGCommand subclass: #DrGMathItemCommand
	instanceVariableNames: 'mathItems'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'DrGeoII-Core-Command'!

!DrGMathItemCommand commentStamp: '<historical>' prior: 0!
A command to record a mathItem construction!

!DrGMathItemCommand class methodsFor: 'as yet unclassified' stamp: 'HilaireFernandes 2/2/2006 09:50'!
for: aMathItem in: aDomain 
	^self new
		domain: aDomain;
		fromMathItem: aMathItem .! !

!DrGMathItemCommand class methodsFor: 'as yet unclassified' stamp: 'HilaireFernandes 12/7/2005 15:57'!
with: aBuilder in: aDomain 
	^self new
		domain: aDomain;
		createMathItem: aBuilder.! !

!DrGMathItemCommand methodsFor: 'accessing' stamp: 'HilaireFernandes 3/5/2006 14:49'!
createMathItem: aBuilder
	mathItems := aBuilder getItem.
! !

!DrGMathItemCommand methodsFor: 'command' stamp: 'HilaireFernandes 9/21/2010 21:25'!
execute
	"Remove any previous dependant"
	mathItems do: [:aMathItem| 
		"record the mathItem in the factory"
		domain factory add: aMathItem.
		domain triggerEvent: #redoMathItem with: aMathItem].
	^mathItems! !

!DrGMathItemCommand methodsFor: 'accessing' stamp: 'HilaireFernandes 3/5/2006 14:49'!
fromMathItem: aMathItem
	mathItems := {aMathItem}.
! !

!DrGMathItemCommand methodsFor: 'dependents access' stamp: 'HilaireFernandes 8/18/2008 12:48'!
release
	super release.
	mathItems do: [:m | m release]! !

!DrGMathItemCommand methodsFor: 'command' stamp: 'HilaireFernandes 3/5/2006 14:50'!
unexecute
	"remove the mathItem in the factory"
	domain factory remove: mathItems
! !

DrGCommand subclass: #DrGMoveCommand
	instanceVariableNames: 'vector mathItem'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'DrGeoII-Core-Command'!

!DrGMoveCommand class methodsFor: 'as yet unclassified' stamp: 'HilaireFernandes 1/31/2006 11:40'!
for: aMathItem with: aPoint in: aDomain
	^self new
		domain: aDomain;
		mathItem: aMathItem;
		vector: aPoint! !

!DrGMoveCommand methodsFor: 'command' stamp: 'HilaireFernandes 1/23/2008 11:39'!
execute
	domain moveEvent: mathItem inDirection: vector! !

!DrGMoveCommand methodsFor: 'accessing' stamp: 'HilaireFernandes 1/31/2006 11:28'!
mathItem: aMathItem
	mathItem := aMathItem ! !

!DrGMoveCommand methodsFor: 'command' stamp: 'HilaireFernandes 1/23/2008 11:39'!
unexecute
	domain moveEvent: mathItem inDirection: vector negated! !

!DrGMoveCommand methodsFor: 'accessing' stamp: 'HilaireFernandes 1/31/2006 11:27'!
vector: aPoint
	vector := aPoint
	! !

DrGCommand subclass: #DrGPropertyCommand
	instanceVariableNames: 'property mathItem'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'DrGeoII-Core-Command'!

!DrGPropertyCommand commentStamp: 'HilaireFernandes 10/27/2011 11:30' prior: 0!
A command to record property editing!

!DrGPropertyCommand class methodsFor: 'as yet unclassified' stamp: 'HilaireFernandes 10/27/2011 11:31'!
for: aMathItem with: aProperty in: aDomain
	^self new
		domain: aDomain;
		mathItem: aMathItem;
		property: aProperty! !

!DrGPropertyCommand methodsFor: 'command' stamp: 'HilaireFernandes 10/27/2011 11:39'!
execute
	|temp|
	temp := mathItem property.
	mathItem property: property.
	property := temp. 
	mathItem changed! !

!DrGPropertyCommand methodsFor: 'accessing' stamp: 'HilaireFernandes 10/27/2011 11:32'!
mathItem: aMathItem 
	mathItem := aMathItem ! !

!DrGPropertyCommand methodsFor: 'accessing' stamp: 'HilaireFernandes 10/27/2011 11:31'!
property: aProperty
	property := aProperty ! !

!DrGPropertyCommand methodsFor: 'command' stamp: 'HilaireFernandes 10/27/2011 11:39'!
unexecute
	self execute! !

Object subclass: #DrGCommandManager
	instanceVariableNames: 'stack domain'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'DrGeoII-Core-Command'!

!DrGCommandManager methodsFor: 'command exec' stamp: 'HilaireFernandes 2/2/2006 09:49'!
buildCommandFor: aMathItem
	|command|
	command := stack nextPut: (DrGMathItemCommand for: aMathItem in: domain).
	^command execute! !

!DrGCommandManager methodsFor: 'command exec' stamp: 'HilaireFernandes 1/30/2006 17:05'!
buildCommandWith: aBuilder
	|command|
	command := stack nextPut: (DrGMathItemCommand with: aBuilder in: domain).
	^command execute! !

!DrGCommandManager methodsFor: 'command exec' stamp: 'HilaireFernandes 3/2/2006 22:01'!
compositeCommandFor: aMathItemSelection
|command|
	command := stack nextPut: (DrGCompositeCommand with: aMathItemSelection in: domain).
	command execute.! !

!DrGCommandManager methodsFor: 'command exec' stamp: 'HilaireFernandes 1/30/2006 17:08'!
deleteCommandFor: aMathItem
|command|
	command := stack nextPut: (DrGDeleteCommand for: aMathItem in: domain).
	command execute! !

!DrGCommandManager methodsFor: 'accessing' stamp: 'HilaireFernandes 1/30/2006 17:03'!
domain: aDomain
	domain := aDomain! !

!DrGCommandManager methodsFor: 'initialization' stamp: 'HilaireFernandes 1/30/2006 16:46'!
initialize
	stack := DrGCommandStack new! !

!DrGCommandManager methodsFor: 'command exec' stamp: 'HilaireFernandes 1/31/2006 11:33'!
moveCommandFor: aMathItem with: aPoint
	stack nextPut: (DrGMoveCommand for: aMathItem with: aPoint in: domain).! !

!DrGCommandManager methodsFor: 'command exec' stamp: 'HilaireFernandes 10/27/2011 11:46'!
propertyCommandFor: aMathItem with: aProperty
	|command|
	command := stack nextPut: (DrGPropertyCommand for: aMathItem with: aProperty in: domain).
	command execute! !

!DrGCommandManager methodsFor: 'command exec' stamp: 'HilaireFernandes 1/31/2006 00:14'!
redo
|command|
	command := stack next.
	command ifNotNil:	[command execute]! !

!DrGCommandManager methodsFor: 'as yet unclassified' stamp: 'HilaireFernandes 8/18/2008 12:48'!
release
	stack contents do: [:c | c release].
	super release.! !

!DrGCommandManager methodsFor: 'command exec' stamp: 'HilaireFernandes 1/31/2006 00:14'!
undo
|command|
	command := stack previous.
	command ifNotNil: [command unexecute]! !

Object subclass: #DrGCostumeStyle
	instanceVariableNames: 'costume color isHidden'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'DrGeoII-Core-Item-View'!

!DrGCostumeStyle class methodsFor: 'as yet unclassified' stamp: 'HilaireFernandes 1/22/2006 17:20'!
blinkingColor
	^Color lightGray lighter lighter! !

!DrGCostumeStyle class methodsFor: 'as yet unclassified' stamp: 'HilaireFernandes 11/16/2011 09:55'!
colors
	^#(#black #blue #brown #green #magenta #orange #red #yellow #white)! !

!DrGCostumeStyle class methodsFor: 'as yet unclassified' stamp: 'HilaireFernandes 1/22/2006 16:38'!
hiddenColor
	^Color lightGray lighter lighter lighter! !

!DrGCostumeStyle class methodsFor: 'as yet unclassified' stamp: 'HilaireFernandes 1/21/2006 10:39'!
newWith: aCostume
	^self basicNew costume: aCostume; initialize! !

!DrGCostumeStyle methodsFor: 'menu' stamp: 'HilaireFernandes 10/29/2010 10:56'!
addColorMenu: theMenu 
	| subMenu |
	subMenu := MenuMorph new.
	self class colors do: [:aColor | |theColor|
		theColor := Color perform: aColor.
		subMenu
			add: ''
			target: self
			selector: #color:
			argument: theColor .
		subMenu lastItem 
			icon: (DrGIcons colorForm: theColor)].
	theMenu add: 'colour' translated subMenu: subMenu! !

!DrGCostumeStyle methodsFor: 'menu' stamp: 'HilaireFernandes 11/6/2011 21:48'!
addLockedMenuItemTo: theMenu
"if the mathiItem is lockable, add this menu item"
	costume isLockable ifTrue: [
		theMenu
			add: (costume isLocked
					ifTrue: ['unlock' translated]
					ifFalse: ['lock' translated])
			target: costume
			action: #toggleLocked]
! !

!DrGCostumeStyle methodsFor: 'menu' stamp: 'HilaireFernandes 1/20/2006 21:33'!
addMyMenus: theMenu
"Add specific subclasses menuItem and subMenu"! !

!DrGCostumeStyle methodsFor: 'accessing' stamp: 'HilaireFernandes 4/25/2010 22:32'!
baseColor
	"Answer color whatever it is hidden or not"

	^ color! !

!DrGCostumeStyle methodsFor: 'accessing' stamp: 'HilaireFernandes 1/24/2006 23:02'!
color
	"Answer the value of color"

	^isHidden ifTrue: [DrGCostumeStyle hiddenColor] ifFalse: [color]! !

!DrGCostumeStyle methodsFor: 'accessing' stamp: 'HilaireFernandes 3/3/2006 12:01'!
color: anObject
	"Set the value of color"
	color := anObject.
	costume color: color.! !

!DrGCostumeStyle methodsFor: 'accessing' stamp: 'HilaireFernandes 1/20/2006 22:55'!
costume
	"Answer the value of costume"

	^ costume! !

!DrGCostumeStyle methodsFor: 'accessing' stamp: 'CedrickBeler 4/23/2010 16:39'!
costume: anObject
	"Set the value of costume"

	costume := anObject! !

!DrGCostumeStyle methodsFor: 'initialize-release' stamp: 'HilaireFernandes 1/22/2006 13:03'!
initialize
	isHidden := false.
	self color: Color black.! !

!DrGCostumeStyle methodsFor: 'accessing' stamp: 'HilaireFernandes 1/20/2006 17:35'!
isHidden
	"Answer the value of isHidden"

	^ isHidden! !

!DrGCostumeStyle methodsFor: 'accessing' stamp: 'HilaireFernandes 1/24/2008 11:04'!
isHidden: anObject 
	"Set the value of isHidden"

	isHidden := anObject.
	isHidden 
		ifTrue: 
			[costume stopBlinking.
			costume color: DrGCostumeStyle hiddenColor.
			costume changed]
		ifFalse: [costume color: self color]! !

!DrGCostumeStyle methodsFor: 'accessing' stamp: 'HilaireFernandes 1/22/2006 01:31'!
isNotHidden
	"Answer the value of isHidden"

	^ isHidden not! !

!DrGCostumeStyle methodsFor: 'testing' stamp: 'HilaireFernandes 11/30/2011 17:38'!
isTranslucent 
	^ false! !

!DrGCostumeStyle methodsFor: 'menu' stamp: 'HilaireFernandes 11/6/2011 21:37'!
popupMenu
	| menu |
	menu := MenuMorph new title: ' edit style... ' translated.
	self addColorMenu: menu.
	self addMyMenus: menu.
	menu
		add: 'rename' translated
		target: self
		action: #rename.
	self addLockedMenuItemTo: menu.
	menu
		add: (self isHidden
				ifTrue: ['show' translated]
				ifFalse: ['hide' translated])
		target: self
		action: #toggleHidden.
	menu addStayUpIcons.
	menu popUpInWorld: World! !

!DrGCostumeStyle methodsFor: 'accessing' stamp: 'HilaireFernandes 1/26/2008 21:23'!
rename
	| |
	costume textMorph
		ifNil: [costume textMorph: 'edit me' translated].
	costume textMorph launchMiniEditor: ActiveEvent! !

!DrGCostumeStyle methodsFor: 'accessing' stamp: 'HilaireFernandes 1/22/2006 16:53'!
toggleHidden
	self isHidden: self isHidden not! !

!DrGCostumeStyle methodsFor: 'accessing' stamp: 'HilaireFernandes 10/27/2011 10:44'!
view
	^ costume view! !

!DrGCostumeStyle methodsFor: 'xml writing' stamp: 'HilaireFernandes 10/31/2010 22:33'!
writeAttributesTo: aNode
	aNode 
		attributeAt: #color put: self baseColor name asString capitalized;
		attributeAt: #masked put: self isHidden asString capitalized! !

DrGCostumeStyle subclass: #DrGCurveCostumeStyle
	instanceVariableNames: 'thickness lineStyle'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'DrGeoII-Core-Item-View'!

!DrGCurveCostumeStyle methodsFor: 'menu' stamp: 'HilaireFernandes 11/16/2011 18:08'!
addLineStyleMenu: theMenu
	|subMenu |
	subMenu := MenuMorph new.
	#(#dot #dash #plain) do: [:style | 
		subMenu 
			add: ''
			target: self
			selector: #lineStyle:
			argument: style.
		subMenu lastItem 
			icon: (DrGIcons segment: self width color: self color dash: (self lineStyleFor: style))].
	theMenu add: 'line style' translated subMenu: subMenu.! !

!DrGCurveCostumeStyle methodsFor: 'menu' stamp: 'HilaireFernandes 1/21/2006 15:57'!
addMyMenus: theMenu
	super addMyMenus: theMenu.
	self addLineStyleMenu: theMenu.
	self addThicknessMenu: theMenu! !

!DrGCurveCostumeStyle methodsFor: 'menu' stamp: 'HilaireFernandes 4/28/2010 17:29'!
addThicknessMenu: theMenu
|subMenu|
	subMenu := MenuMorph new.
	#(#small #medium #large) do: [:aThickness|
		subMenu 
			add: ''
			target: self
			selector: #thickness:
			argument: aThickness.
		subMenu lastItem
			icon: (DrGIcons segment: (self widthFor: aThickness) color: self color dash: nil)].
	theMenu add: 'size' translated subMenu: subMenu.! !

!DrGCurveCostumeStyle methodsFor: 'initialize-release' stamp: 'HilaireFernandes 1/21/2006 15:15'!
initialize
	super initialize.
	self lineStyle: #plain.
	self thickness: #small.! !

!DrGCurveCostumeStyle methodsFor: 'accessing' stamp: 'HilaireFernandes 1/21/2006 14:57'!
lineStyle
	"Answer the value of lineStyle"

	^ lineStyle! !

!DrGCurveCostumeStyle methodsFor: 'accessing' stamp: 'HilaireFernandes 4/28/2010 17:22'!
lineStyle: aSymbol
	"Set the value of lineStyle"
	lineStyle := aSymbol.
	costume dash: (self lineStyleFor: aSymbol)
! !

!DrGCurveCostumeStyle methodsFor: 'accessing' stamp: 'HilaireFernandes 4/28/2010 17:23'!
lineStyleFor: aStyle
	"possible lineStyle"
	aStyle = #dot ifTrue: [^{1. 5 . Color transparent}].
	aStyle = #dash ifTrue: [^ {5. 8 . Color transparent}].
	^ nil! !

!DrGCurveCostumeStyle methodsFor: 'accessing' stamp: 'HilaireFernandes 1/21/2006 14:49'!
thickness
	"Answer the value of thickness"

	^ thickness! !

!DrGCurveCostumeStyle methodsFor: 'accessing' stamp: 'HilaireFernandes 4/28/2010 16:32'!
thickness: anObject
	"Set the value of thickness"
	thickness := anObject.
	costume width: self width.! !

!DrGCurveCostumeStyle methodsFor: 'accessing' stamp: 'HilaireFernandes 4/28/2010 17:17'!
width
	"give the width"
	^ self widthFor: thickness! !

!DrGCurveCostumeStyle methodsFor: 'accessing' stamp: 'HilaireFernandes 4/28/2010 17:17'!
widthFor: aThickness
	"give the width"
	aThickness = #small ifTrue: [^1].
	aThickness = #medium ifTrue: [^2].
	aThickness = #large ifTrue: [^3].
	^ 0! !

!DrGCurveCostumeStyle methodsFor: 'xml writing' stamp: 'HilaireFernandes 10/31/2010 22:37'!
writeAttributesTo: aNode
	|attr|
	super writeAttributesTo: aNode.
	(self lineStyle = #dot or: [self lineStyle =#dashed] )
		ifTrue: [attr := 'Dashed']
		ifFalse: [attr := 'Normal'].
	aNode attributeAt: #style put: self lineStyle.
	(self thickness = #large or: [self thickness = #medium]) ifTrue: [attr := 'Thick'].
	aNode attributeAt: #thickness put: attr.! !

DrGCurveCostumeStyle subclass: #DrGFinitCurveCostumeStyle
	instanceVariableNames: 'arrow'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'DrGeoII-Core-Item-View'!

DrGFinitCurveCostumeStyle subclass: #DrGCompositeStyle
	instanceVariableNames: 'shape'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'DrGeoII-Core-Item-View'!

!DrGCompositeStyle methodsFor: 'menu' stamp: 'jlrr 9/29/2007 15:03'!
addMyMenus: theMenu 
	super addMyMenus: theMenu.
	self addShapeMenu: theMenu.
	theMenu
		add: 'disband this group' translated
		target: costume
		action: #disband! !

!DrGCompositeStyle methodsFor: 'menu' stamp: 'jlrr 9/30/2007 15:12'!
addShapeMenu: theMenu 
	| subMenu |
	subMenu := MenuMorph new.
	#(#cross #round #square )
		do: [:aShape | subMenu
				add: aShape asString translated
				target: self
				selector: #shape:
				argument: aShape].
	theMenu add: 'shape' translated subMenu: subMenu! !

!DrGCompositeStyle methodsFor: 'accessing' stamp: 'HilaireFernandes 3/3/2006 14:46'!
shape
	^shape! !

!DrGCompositeStyle methodsFor: 'accessing' stamp: 'HilaireFernandes 3/3/2006 14:51'!
shape: anObject
	shape := anObject.
	costume shape: anObject! !

!DrGFinitCurveCostumeStyle methodsFor: 'menu' stamp: 'HilaireFernandes 4/28/2010 18:08'!
addArrowMenu: theMenu
	|subMenu|
	subMenu := MenuMorph new.
	#(#makeBackArrow #makeForwardArrow #makeBothArrows #makeNoArrows) do: [:anArrow |
		self arrow ~= anArrow ifTrue:	[
			subMenu 
				add: ''
				target: self
				selector: #arrow:
				argument: anArrow.
			subMenu lastItem 
				icon: (DrGIcons 
					segment: self width 
					color: self color 
					dash: (self lineStyleFor: self lineStyle) 
					arrow: anArrow)]].
	theMenu add: 'arrow' translated subMenu: subMenu.! !

!DrGFinitCurveCostumeStyle methodsFor: 'menu' stamp: 'HilaireFernandes 11/19/2011 10:39'!
addMyMenus: theMenu
	super addMyMenus: theMenu.
	costume isSegmentCostume ifTrue: [self addArrowMenu: theMenu] ! !

!DrGFinitCurveCostumeStyle methodsFor: 'accessing' stamp: 'HilaireFernandes 1/21/2006 18:05'!
arrow
	"Answer the value of arrows"

	^ arrow! !

!DrGFinitCurveCostumeStyle methodsFor: 'accessing' stamp: 'HilaireFernandes 4/28/2010 18:04'!
arrow: anObject
	"Set the value of arrows"
	arrow := anObject.
	costume perform: arrow.! !

!DrGFinitCurveCostumeStyle methodsFor: 'initialize-release' stamp: 'HilaireFernandes 11/16/2011 10:29'!
initialize
	super initialize.
	costume isCircleCostume ifFalse: [self arrow: #makeNoArrows]! !

!DrGFinitCurveCostumeStyle methodsFor: 'xml writing' stamp: 'HilaireFernandes 11/16/2011 12:45'!
writeAttributesTo: aNode
	super writeAttributesTo: aNode.
	costume isCircleCostume ifFalse: [aNode attributeAt: #arrow put: self arrow.]! !

DrGFinitCurveCostumeStyle subclass: #DrGFinitFilledCostumeStyle
	instanceVariableNames: 'isTranslucent isFilled'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'DrGeoII-Core-Item-View'!

DrGFinitFilledCostumeStyle subclass: #DrGArcCostumeStyle
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'DrGeoII-Core-Item-View'!

!DrGArcCostumeStyle methodsFor: 'accessing' stamp: 'HilaireFernandes 11/16/2011 11:57'!
isFilled: boolean
	(isFilled := boolean) 
		ifTrue: [costume morph makeClosed]
		ifFalse: [costume morph makeOpen].
	costume changed! !

!DrGFinitFilledCostumeStyle methodsFor: 'menu' stamp: 'HilaireFernandes 11/16/2011 10:23'!
addFilledMenuItemTo: theMenu
	theMenu
		add: (self isFilled
			ifTrue: ['unfill' translated]
			ifFalse: ['fill' translated])
		target: self
		action: #toggleFilled
! !

!DrGFinitFilledCostumeStyle methodsFor: 'menu' stamp: 'HilaireFernandes 11/16/2011 10:24'!
addMyMenus: theMenu
	super addMyMenus: theMenu.
	self addFilledMenuItemTo: theMenu.
	self isFilled ifTrue: [
		self addTranslucentMenuItemTo: theMenu]! !

!DrGFinitFilledCostumeStyle methodsFor: 'menu' stamp: 'HilaireFernandes 11/16/2011 10:22'!
addTranslucentMenuItemTo: theMenu
	theMenu
		add: (self isTranslucent
			ifTrue: ['opaque' translated]
			ifFalse: ['translucent' translated])
		target: self
		action: #toggleTranslucent
! !

!DrGFinitFilledCostumeStyle methodsFor: 'accessing' stamp: 'HilaireFernandes 11/16/2011 10:48'!
isFilled
	^ isFilled ifNil: [isFilled := false]! !

!DrGFinitFilledCostumeStyle methodsFor: 'accessing' stamp: 'HilaireFernandes 11/16/2011 10:42'!
isFilled: boolean
	isFilled := boolean.
	costume changed! !

!DrGFinitFilledCostumeStyle methodsFor: 'accessing' stamp: 'HilaireFernandes 11/16/2011 10:49'!
isTranslucent
	^ isTranslucent ifNil: [isTranslucent := false]! !

!DrGFinitFilledCostumeStyle methodsFor: 'accessing' stamp: 'HilaireFernandes 11/16/2011 12:13'!
isTranslucent: boolean
	isTranslucent := boolean.
	costume color: color.
	costume changed! !

!DrGFinitFilledCostumeStyle methodsFor: 'accessing' stamp: 'HilaireFernandes 11/16/2011 10:42'!
toggleFilled
	self isFilled: isFilled not.
! !

!DrGFinitFilledCostumeStyle methodsFor: 'accessing' stamp: 'HilaireFernandes 11/16/2011 10:42'!
toggleTranslucent
	self isTranslucent: isTranslucent not! !

!DrGFinitFilledCostumeStyle methodsFor: 'as yet unclassified' stamp: 'HilaireFernandes 11/16/2011 12:34'!
writeAttributesTo: aNode
	super writeAttributesTo: aNode.
	aNode attributeAt: #filled put: self isFilled asString capitalized.
	aNode attributeAt: #translucent put: self isTranslucent asString capitalized! !

DrGFinitCurveCostumeStyle subclass: #DrGSegmentCostumeStyle
	instanceVariableNames: 'mark'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'DrGeoII-Core-Item-View'!

!DrGSegmentCostumeStyle methodsFor: 'menu' stamp: 'HilaireFernandes 11/16/2011 21:31'!
addMarkStyleMenu: theMenu
	|subMenu|
	subMenu := MenuMorph new.
	#(#none #mark1 #mark2 #mark3 #disc #circle) do: [:aMark |
		subMenu
			add: ''
			target: self
			selector: #mark:
			argument: aMark.
		subMenu lastItem 
			icon: (DrGIcons segment: 2 color: Color black mark: aMark)].
	theMenu add: 'mark' translated subMenu: subMenu.! !

!DrGSegmentCostumeStyle methodsFor: 'menu' stamp: 'HilaireFernandes 11/16/2011 21:31'!
addMyMenus: theMenu
	super addMyMenus: theMenu.
	self addMarkStyleMenu: theMenu! !

!DrGSegmentCostumeStyle methodsFor: 'accessing' stamp: 'HilaireFernandes 11/16/2011 18:07'!
mark
	^ mark! !

!DrGSegmentCostumeStyle methodsFor: 'accessing' stamp: 'HilaireFernandes 11/16/2011 21:40'!
mark: aSymbol
	mark := aSymbol.
	costume changed! !

!DrGSegmentCostumeStyle methodsFor: 'xml writing' stamp: 'HilaireFernandes 11/16/2011 21:45'!
writeAttributesTo: aNode
	super writeAttributesTo: aNode.
	aNode attributeAt: #mark put: self mark! !

DrGCostumeStyle subclass: #DrGPointCostumeStyle
	instanceVariableNames: 'shape thickness'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'DrGeoII-Core-Item-View'!

!DrGPointCostumeStyle methodsFor: 'menu' stamp: 'HilaireFernandes 1/21/2006 12:34'!
addMyMenus: theMenu
	self addSizeMenu: theMenu.
	self addShapeMenu: theMenu.! !

!DrGPointCostumeStyle methodsFor: 'menu' stamp: 'HilaireFernandes 10/29/2010 10:50'!
addShapeMenu: theMenu 
	| subMenu theShape|
	theShape := self shape.
	subMenu := MenuMorph new.
	#(#cross #round #square ) do: [:aShape | 
		subMenu
			add: aShape asString translated
			target: self
			selector: #shape:
			argument: aShape.
		self shape: aShape.
		subMenu lastItem
			 icon: (DrGIcons iconWithBackground: self costume morph)].
	theMenu add: 'shape' translated subMenu: subMenu.
	self shape: theShape ! !

!DrGPointCostumeStyle methodsFor: 'menu' stamp: 'HilaireFernandes 10/29/2010 10:50'!
addSizeMenu: theMenu 
	| subMenu theThickness|
	theThickness := self thickness.
	subMenu := MenuMorph new.
	#(#small #medium #large) do: [:aThickness | 
		subMenu
			add: aThickness asString translated
			target: self
			selector: #thickness:
			argument: aThickness.
		self thickness: aThickness.
		subMenu lastItem
			icon: (DrGIcons iconWithBackground: self costume morph)].
	theMenu add: 'size' translated subMenu: subMenu.
	self thickness: theThickness.! !

!DrGPointCostumeStyle methodsFor: 'initialize-release' stamp: 'HilaireFernandes 1/26/2008 21:17'!
initialize
	super initialize.
	self color: Color red.
	self thickness: #medium.
	self shape: #cross! !

!DrGPointCostumeStyle methodsFor: 'accessing' stamp: 'HilaireFernandes 1/21/2006 11:57'!
shape
	"Answer the value of shape"

	^ shape! !

!DrGPointCostumeStyle methodsFor: 'accessing' stamp: 'HilaireFernandes 3/3/2006 14:41'!
shape: anObject
	"Set the value of shape"

	shape := anObject.
	costume changed.! !

!DrGPointCostumeStyle methodsFor: 'accessing' stamp: 'HilaireFernandes 1/21/2006 11:59'!
thickness
	"Answer the value of thickness"

	^ thickness! !

!DrGPointCostumeStyle methodsFor: 'accessing' stamp: 'HilaireFernandes 1/16/2008 15:23'!
thickness: anObject
	"Set the value of thickness"
	thickness := anObject.
	thickness = #small ifTrue: [costume pointSize: 6@6].
	thickness = #medium ifTrue: [costume pointSize: 8@8].
	thickness = #large ifTrue: [costume pointSize: 10@10].! !

!DrGPointCostumeStyle methodsFor: 'xml writing' stamp: 'HilaireFernandes 10/31/2010 22:32'!
writeAttributesTo: aNode
	|attr|
	super writeAttributesTo: aNode.
	self thickness = #small ifTrue: [attr := 'Dashed'].
	self thickness = #medium ifTrue: [attr := 'Normal'].
	self thickness = #large ifTrue: [attr := 'Thick'].
	aNode attributeAt: #thickness put: attr.
	attr := self shape asString capitalized.
	self shape = #square ifTrue: [attr := 'Rectangular'].
	self shape = #round ifTrue: [attr := 'Round'].
	aNode attributeAt: #style put: attr! !

DrGCostumeStyle subclass: #DrGValueCostumeStyle
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'DrGeoII-Core-Item-View'!

!DrGValueCostumeStyle methodsFor: 'accessing' stamp: 'HilaireFernandes 10/27/2011 10:44'!
rename
|answer|
	costume mathItem name 
		ifNotNil:	[
			answer := self view 
				textEntry:  'Rename this value' translated 
				title: 'Rename this value' translated 
				entryText: costume mathItem name.]
		ifNil: [
			answer := self view 
				textEntry:  'Name this value' translated 
				title: 'Name this value' translated].
	answer ifNil: [^self].
	answer size = 0 ifTrue: [answer := nil].
	costume mathItem name: answer.
	costume redraw! !

Object subclass: #DrGDefault
	instanceVariableNames: ''
	classVariableNames: 'Host NetworkDirectory NetworkPassword NetworkType NetworkUser UseNetwork'
	poolDictionaries: ''
	category: 'DrGeoII-Core-App'!

DrGDefault class
	instanceVariableNames: 'isOlpc isIpad isAndroid'!

DrGDefault class
	instanceVariableNames: 'isOlpc isIpad isAndroid'!

!DrGDefault class methodsFor: 'host mode' stamp: 'HilaireFernandes 12/26/2011 18:29'!
beAndroidMode
	"self beAndroidMode"
	isIpad := isOlpc := false.
	isAndroid := true.
	WorldState desktopMenuTitle: 'Dr. Geo'.
	WorldState desktopMenuPragmaKeyword:  'drgeoMenu'.
	DrGeo addStartUp.

	PolymorphSystemSettings uiThemeClass:  DrGeoTheme.
	UITheme currentSettings fastDragging: true.
	UITheme currentSettings fadedBackgroundWindows: false.
	SystemWindow fullScreenMargin: 0.
	UITheme currentSettings preferRoundCorner: false.

	TaskbarMorph showTaskbar: false.
	World color: Color white.! !

!DrGDefault class methodsFor: 'host mode' stamp: 'HilaireFernandes 12/26/2011 13:03'!
beOlpcMode
	"self beOlpcMode"
	isOlpc := true.
	isAndroid := false.
	isIpad := false.
	WorldState desktopMenuTitle: 'Dr. Geo'.
	WorldState desktopMenuPragmaKeyword:  'drgeoMenu'.

	DrGeo addStartUp.

	Display newDepth: 16.
	(Smalltalk at: #UIThemeSugar) resetDefaultSettings.
	PolymorphSystemSettings uiThemeClass:  (Smalltalk at: #UIThemeSugar).
	UITheme currentSettings fastDragging: true.
	UITheme currentSettings fadedBackgroundWindows: false.

	World color: Color white.
	SystemWindow fullScreenMargin: 0.
! !

!DrGDefault class methodsFor: 'host mode' stamp: 'HilaireFernandes 12/26/2011 13:04'!
beWorkstationMode
	"self beWorkstationMode"
	isIpad := isOlpc := isAndroid := false.
	DrGeo removeStartUp.
	Display newDepth: 32.
	PolymorphSystemSettings uiThemeClass:  UIThemeWatery2.
	TaskbarMorph showTaskbar: true.
	SystemWindow fullScreenMargin: 0.
! !

!DrGDefault class methodsFor: 'host mode' stamp: 'HilaireFernandes 12/26/2011 13:00'!
beiPadMode
	"self beiPadMode"
	isIpad := true.
	isOlpc := false.
	WorldState desktopMenuTitle: 'Dr. Geo'.
	WorldState desktopMenuPragmaKeyword:  'drgeoMenu'.

	PolymorphSystemSettings uiThemeClass:  DrGeoTheme.
	UITheme currentSettings fastDragging: true.
	UITheme currentSettings fadedBackgroundWindows: false.
	SystemWindow fullScreenMargin: 0.
	UITheme currentSettings preferRoundCorner: false.

	TaskbarMorph showTaskbar: false.
	World color: Color white.
	HostWindowProxy basicNew primitiveWindowSize: 1	x: 1024	y: 768. "does not work when building from Linux"
	World doOneCycleNow.
	DrGeo new view extent: 1024@768; position: 0@0.
! !

!DrGDefault class methodsFor: 'path' stamp: 'HilaireFernandes 12/10/2011 11:37'!
defaultPath
	^ (self isOlpc ifTrue: [self olpcPath] ifFalse: [
		self isIpad ifTrue: [self iPadPath] ifFalse: [
			self othersPath]])! !

!DrGDefault class methodsFor: 'settings' stamp: 'HilaireFernandes 12/21/2011 10:35'!
drgeoSettingsOn: aBuilder
	<drgeosettings>
	(aBuilder group: #drgeo)
		label: 'Dr. Geo';
		description: 'All Dr. Geo related settings' translated;
		noOrdering;
		with: [
			(aBuilder setting: #useNetwork)
				label: 'Use network resources' translated;
				noOrdering;
				description: 'If checked you can set a server name, user and password to load and to save sketches.' translated;
				with: [
					(aBuilder pickOne: #networkType)
						label: 'Server type' translated;
						description: 'The server type, FTP or WebDav';
						default: #ftp;
						domainValues: #(#ftp #http).
					(aBuilder setting: #host)
						label: 'Host' translated;
						description: 'The server (i.e. ftp.drgeo.eu)' translated .
					(aBuilder setting: #networkUser)
						label: 'User name' translated;
						description: 'The server user name' translated.
					(aBuilder setting: #networkPassword)
						type: #Password;
						label: 'Password' translated;
						description: 'The user password' translated.
					(aBuilder setting: #networkDirectory)
						label: 'Directory' translated;
						description: 'The server resources directory (i.e. private/drgeo)' translated ]]! !

!DrGDefault class methodsFor: 'resources' stamp: 'HilaireFernandes 12/19/2011 14:21'!
exampleResources
	^ DrGDirectoryLocal new directory: DrGDefault examplesPath.! !

!DrGDefault class methodsFor: 'path' stamp: 'HilaireFernandes 11/18/2011 22:44'!
examplesPath
	^ self defaultPath, FileDirectory slash, 'examples'! !

!DrGDefault class methodsFor: 'font' stamp: 'HilaireFernandes 1/26/2008 14:39'!
font
	^ StrikeFont familyName: self fontName pointSize: self fontPointSize! !

!DrGDefault class methodsFor: 'font' stamp: 'HilaireFernandes 6/15/2011 17:03'!
fontName
	^ 'Bitmap DejaVu Sans'! !

!DrGDefault class methodsFor: 'font' stamp: 'HilaireFernandes 11/9/2011 11:26'!
fontPointSize
	^ (self isEtoys or: [self isOlpc])
		ifTrue: [14]
		ifFalse: [11]! !

!DrGDefault class methodsFor: 'settings' stamp: 'HilaireFernandes 12/20/2011 22:45'!
host
	^ Host ifNil: [Host := '']! !

!DrGDefault class methodsFor: 'settings' stamp: 'HilaireFernandes 12/20/2011 22:46'!
host: string
	Host := string! !

!DrGDefault class methodsFor: 'path' stamp: 'BertFreudenberg 6/30/2011 16:01'!
iPadPath
	"Smalltalk imagePath would work too, normally, but not if the image was copied to the Documents folder"
	^ Smalltalk vmPath, 'DrGeo.app'! !

!DrGDefault class methodsFor: 'testing' stamp: 'HilaireFernandes 6/29/2011 08:36'!
ifIpad: aBlock
	^ isIpad ifTrue: [aBlock value]! !

!DrGDefault class methodsFor: 'testing' stamp: 'HilaireFernandes 12/4/2011 16:30'!
ifNotLiteDo: block
	^ self isLite 
		ifTrue:  [UIManager default inform: 'You need the premium version of Dr. Geo for this function.' translated]
		ifFalse: [block value]! !

!DrGDefault class methodsFor: 'testing' stamp: 'HilaireFernandes 12/27/2011 00:01'!
ifTablet: aBlock
	^ self isTablet ifTrue: [aBlock value]! !

!DrGDefault class methodsFor: 'as yet unclassified' stamp: 'HilaireFernandes 6/15/2011 21:29'!
initialize 
"DrGDefault initialize"
	super initialize.
	isOlpc := nil.
	isIpad := false! !

!DrGDefault class methodsFor: 'testing' stamp: 'HilaireFernandes 12/26/2011 13:10'!
isAndroid
	^ isAndroid! !

!DrGDefault class methodsFor: 'testing' stamp: 'HilaireFernandes 5/19/2010 21:34'!
isEtoys
	^ SystemVersion current version beginsWith:  'etoys'! !

!DrGDefault class methodsFor: 'testing' stamp: 'HilaireFernandes 6/15/2011 21:29'!
isIpad
	^ isIpad! !

!DrGDefault class methodsFor: 'testing' stamp: 'HilaireFernandes 12/26/2011 13:12'!
isLite
"Are we the lite version of DrGeo"
	^self isPremium not and: [self isTablet]! !

!DrGDefault class methodsFor: 'testing' stamp: 'HilaireFernandes 8/3/2010 13:49'!
isOlpc
	^ isOlpc ifNil: [
		isOlpc := (SmalltalkImage  current getSystemAttribute: 1) includesSubString:  'olpc']! !

!DrGDefault class methodsFor: 'testing' stamp: 'HilaireFernandes 8/3/2011 10:17'!
isPremium
"Are we the premium version of DrGeo"
	^ true! !

!DrGDefault class methodsFor: 'testing' stamp: 'HilaireFernandes 12/26/2011 13:11'!
isTablet
	^ self isAndroid or: [self isIpad]! !

!DrGDefault class methodsFor: 'testing' stamp: 'HilaireFernandes 12/26/2011 13:12'!
isWorkstation
"Are we workstation version of DrGeo"
	^self isOlpc not and: [self isTablet not]! !

!DrGDefault class methodsFor: 'path' stamp: 'HilaireFernandes 11/18/2011 22:45'!
myExportsPath
	^ self defaultPath, FileDirectory slash, 'MyExports'! !

!DrGDefault class methodsFor: 'resources' stamp: 'HilaireFernandes 12/19/2011 14:22'!
mySketchResources
	^ DrGDirectoryLocal new directory: DrGDefault mySketchesPath.! !

!DrGDefault class methodsFor: 'path' stamp: 'HilaireFernandes 11/18/2011 22:44'!
mySketchesPath
	^ self defaultPath, FileDirectory slash, 'MySketches'! !

!DrGDefault class methodsFor: 'settings' stamp: 'HilaireFernandes 12/20/2011 22:47'!
networkDirectory
	^ NetworkDirectory ifNil: [NetworkDirectory := '']! !

!DrGDefault class methodsFor: 'settings' stamp: 'HilaireFernandes 12/20/2011 22:47'!
networkDirectory: string
	NetworkDirectory := string ! !

!DrGDefault class methodsFor: 'settings' stamp: 'HilaireFernandes 12/20/2011 22:46'!
networkPassword
	^ NetworkPassword ifNil: [NetworkPassword := '']! !

!DrGDefault class methodsFor: 'settings' stamp: 'HilaireFernandes 12/20/2011 22:47'!
networkPassword: string
	NetworkPassword := string ! !

!DrGDefault class methodsFor: 'resources' stamp: 'HilaireFernandes 12/20/2011 23:20'!
networkResources
	^ (DrGDirectoryFTP host: self host)
		login: self networkUser password: self networkPassword;
		directory: self networkDirectory.! !

!DrGDefault class methodsFor: 'settings' stamp: 'HilaireFernandes 12/21/2011 10:02'!
networkType
	^ NetworkType ifNil: [NetworkType = #ftp]! !

!DrGDefault class methodsFor: 'settings' stamp: 'HilaireFernandes 12/21/2011 10:03'!
networkType: symbol
	NetworkType := symbol! !

!DrGDefault class methodsFor: 'settings' stamp: 'HilaireFernandes 12/20/2011 22:46'!
networkUser
	^ NetworkUser ifNil: [NetworkUser := '']! !

!DrGDefault class methodsFor: 'settings' stamp: 'HilaireFernandes 12/20/2011 22:47'!
networkUser: string
	NetworkUser := string ! !

!DrGDefault class methodsFor: 'path' stamp: 'HilaireFernandes 12/10/2011 11:37'!
olpcPath
	^ FileDirectory dirPathFor: (SmalltalkImage current getSystemAttribute: 1)! !

!DrGDefault class methodsFor: 'path' stamp: 'HilaireFernandes 11/18/2011 22:41'!
othersPath
	| path |
	path := SmalltalkImage current imagePath.
	2 timesRepeat: [path := path copyUpToLast: FileDirectory pathNameDelimiter].
	^ path! !

!DrGDefault class methodsFor: 'font' stamp: 'HilaireFernandes 6/4/2011 15:27'!
smallFont
	^ StrikeFont familyName: self fontName pointSize: self smallFontPointSize! !

!DrGDefault class methodsFor: 'font' stamp: 'HilaireFernandes 11/9/2011 11:26'!
smallFontPointSize
	^ (self isEtoys or: [self isOlpc])
		ifTrue: [12]
		ifFalse: [9]! !

!DrGDefault class methodsFor: 'settings' stamp: 'HilaireFernandes 12/21/2011 10:00'!
useNetwork
	^ UseNetwork ifNil: [UseNetwork := false]! !

!DrGDefault class methodsFor: 'settings' stamp: 'HilaireFernandes 12/21/2011 10:00'!
useNetwork: aBoolean
	UseNetwork := aBoolean! !

!DrGDefault class methodsFor: 'as yet unclassified' stamp: 'HilaireFernandes 1/28/2008 21:30'!
viewXml: filename 
	DrGeo serviceViewXml: self examplesPath , FileDirectory slash , filename! !

!DrGDefault class methodsFor: 'as yet unclassified' stamp: 'HilaireFernandes 12/26/2011 13:12'!
wheelX
	^ self isTablet
		ifTrue: [50]
		ifFalse: [25]! !

!DrGDefault class methodsFor: 'as yet unclassified' stamp: 'HilaireFernandes 12/26/2011 13:12'!
wheelY
	^ self isTablet
		ifTrue: [180]
		ifFalse: [100]! !

Object subclass: #DrGDirectory
	instanceVariableNames: 'filename'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'DrGeoII-Core-App'!

!DrGDirectory commentStamp: 'HilaireFernandes 12/19/2011 15:00' prior: 0!
A DrGDirectory is Facade to the DrGeo file on local file system or remotre ressources on sever, as FTP or WebDav.
	- filename : hold a filename for regular use (the user does not have to enter it again)!

!DrGDirectory methodsFor: 'protocol' stamp: 'HilaireFernandes 12/17/2011 22:31'!
directory
	self subclassResponsibility ! !

!DrGDirectory methodsFor: 'protocol' stamp: 'HilaireFernandes 12/17/2011 20:36'!
directory: aDirectory
	self subclassResponsibility ! !

!DrGDirectory methodsFor: 'accessing' stamp: 'HilaireFernandes 12/17/2011 22:02'!
fgeoFiles
	"return only the drgeo files"
	^ self getFileList select: [:filename | filename endsWith: '.fgeo' ]! !

!DrGDirectory methodsFor: 'testing' stamp: 'HilaireFernandes 12/17/2011 21:27'!
fileExists: filename
	self subclassResponsibility ! !

!DrGDirectory methodsFor: 'accessing' stamp: 'HilaireFernandes 12/19/2011 15:07'!
filename
	^ filename! !

!DrGDirectory methodsFor: 'accessing' stamp: 'HilaireFernandes 12/19/2011 15:00'!
filename: string
	filename := string! !

!DrGDirectory methodsFor: 'accessing' stamp: 'HilaireFernandes 12/17/2011 20:45'!
getFile: filename
	"return a stream on filename content"
	self subclassResponsibility ! !

!DrGDirectory methodsFor: 'accessing' stamp: 'HilaireFernandes 12/17/2011 14:40'!
getFileList
	"return an array with file list entry"
	self subclassResponsibility ! !

!DrGDirectory methodsFor: 'accessing' stamp: 'HilaireFernandes 12/18/2011 18:54'!
getTextFile: filename
	self subclassResponsibility ! !

!DrGDirectory methodsFor: 'testing' stamp: 'HilaireFernandes 12/17/2011 22:12'!
isLocal
	^ false! !

!DrGDirectory methodsFor: 'testing' stamp: 'HilaireFernandes 12/17/2011 22:12'!
isRemote
	^ false! !

!DrGDirectory methodsFor: 'accessing' stamp: 'HilaireFernandes 12/19/2011 15:02'!
put: stream
	filename ifNotNil: [self put: stream into: filename]! !

!DrGDirectory methodsFor: 'accessing' stamp: 'HilaireFernandes 12/18/2011 11:03'!
put: stream into: filename
	self subclassResponsibility ! !

DrGDirectory subclass: #DrGDirectoryFTP
	instanceVariableNames: 'client'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'DrGeoII-Core-App'!

!DrGDirectoryFTP commentStamp: 'HilaireFernandes 12/17/2011 14:36' prior: 0!
I access DrGeo sketches located in a FTP server. 
	- client : the FTP connection client!

!DrGDirectoryFTP class methodsFor: 'instance creation' stamp: 'HilaireFernandes 12/17/2011 14:49'!
host: hostName
	^ self new host: hostName! !

!DrGDirectoryFTP methodsFor: 'protocol' stamp: 'HilaireFernandes 12/17/2011 14:39'!
directory: aPath
	client changeDirectoryTo: aPath! !

!DrGDirectoryFTP methodsFor: 'testing' stamp: 'HilaireFernandes 12/17/2011 21:27'!
fileExists: filename
	^ self getFileList includes: filename! !

!DrGDirectoryFTP methodsFor: 'accessing' stamp: 'HilaireFernandes 12/18/2011 18:54'!
getFile: filename
	^ReadStream on: (client getFileNamed: filename) asByteArray! !

!DrGDirectoryFTP methodsFor: 'accessing' stamp: 'HilaireFernandes 12/17/2011 22:34'!
getFileList
	^ (client getFileList subStrings: String crlf) 
		collect: [:filename | filename withBlanksTrimmed] 
		thenSelect: [:filename | filename notEmpty]! !

!DrGDirectoryFTP methodsFor: 'accessing' stamp: 'HilaireFernandes 12/18/2011 18:55'!
getTextFile: filename
	^ReadStream on: (client getFileNamed: filename)! !

!DrGDirectoryFTP methodsFor: 'protocol' stamp: 'HilaireFernandes 12/18/2011 11:44'!
host: hostName
	client := FTPClient openOnHostNamed: hostName.! !

!DrGDirectoryFTP methodsFor: 'testing' stamp: 'HilaireFernandes 12/17/2011 22:12'!
isRemote
	^ true! !

!DrGDirectoryFTP methodsFor: 'protocol' stamp: 'HilaireFernandes 12/18/2011 11:45'!
login: user password: password
	client loginUser: user password: password.
	client binary! !

!DrGDirectoryFTP methodsFor: 'accessing' stamp: 'HilaireFernandes 12/19/2011 14:34'!
put: stream into: filename
	|readonlyStream|
	[client deleteFileNamed: filename] on: Error do: [].
	client binary.
	[readonlyStream := ReadStream on: stream contents.
	client putFileStreamContents: readonlyStream as: filename] ensure: 
		[readonlyStream close]! !

DrGDirectory subclass: #DrGDirectoryLocal
	instanceVariableNames: 'directory'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'DrGeoII-Core-App'!

!DrGDirectoryLocal commentStamp: 'HilaireFernandes 12/17/2011 15:43' prior: 0!
A DrGDirectoryLocal is facade to local resources on the host.!

!DrGDirectoryLocal methodsFor: 'protocol' stamp: 'HilaireFernandes 12/17/2011 22:31'!
directory 
	^ directory! !

!DrGDirectoryLocal methodsFor: 'protocol' stamp: 'HilaireFernandes 12/17/2011 20:38'!
directory: aPath
	directory := aPath ! !

!DrGDirectoryLocal methodsFor: 'testing' stamp: 'HilaireFernandes 12/17/2011 21:26'!
fileExists: filename
	^ (FileDirectory on: directory) fileExists: filename! !

!DrGDirectoryLocal methodsFor: 'helpers' stamp: 'HilaireFernandes 12/18/2011 11:22'!
fullname: filename
	^ directory, FileDirectory slash, filename! !

!DrGDirectoryLocal methodsFor: 'accessing' stamp: 'HilaireFernandes 12/18/2011 18:30'!
getFile: filename
	^ FileStream readOnlyFileNamed: (self fullname: filename)! !

!DrGDirectoryLocal methodsFor: 'accessing' stamp: 'HilaireFernandes 12/17/2011 22:00'!
getFileList
	^ (FileDirectory on: directory) entries collect: [:directoryEntry | directoryEntry name]! !

!DrGDirectoryLocal methodsFor: 'accessing' stamp: 'HilaireFernandes 12/18/2011 18:55'!
getTextFile: filename
	^ self getFile: filename! !

!DrGDirectoryLocal methodsFor: 'testing' stamp: 'HilaireFernandes 12/17/2011 22:13'!
isLocal
	^ true! !

!DrGDirectoryLocal methodsFor: 'accessing' stamp: 'HilaireFernandes 12/18/2011 19:15'!
put: stream into: filename
	|streamOnDisk|
	[streamOnDisk := MultiByteFileStream forceNewFileNamed: (self fullname: filename).
	streamOnDisk nextPutAll: stream contents] ensure: 
		[streamOnDisk close]! !

Object subclass: #DrGDynamicTool
	instanceVariableNames: 'state presenter spot last costume mathItemUnderMouse'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'DrGeoII-Core-Tool'!

!DrGDynamicTool commentStamp: 'HilaireFernandes 7/20/2011 16:34' prior: 0!
The root of the class hierarchy to handle tool

 state 					- 	state of the tool
 presenter 				- 	from which presenter is used this tool
 spot 					- 	xy-postition of the selected item when selecting one among several
 last 					- 	last mouse xy-position (used during drag)
 costume 				- 	selected costume during the last multiple choice selection
 mathItemUnderMouse 	- 	mathItem under mouse
 !

DrGDynamicTool subclass: #DrGBuildTool
	instanceVariableNames: 'selectedCostumes builder'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'DrGeoII-Core-Tool'!

!DrGBuildTool class methodsFor: 'instance creation' stamp: 'HilaireFernandes 1/5/2006 17:12'!
with: aBuilderClassName
	^self new builder: aBuilderClassName new.! !

!DrGBuildTool methodsFor: 'adding' stamp: 'HilaireFernandes 8/9/2008 16:13'!
add: aCostume
	self stopBlinking.
	selectedCostumes add: aCostume.
	self startBlinking.
	"the flyCostume need this to catch the selected mathItem"
	self triggerEvent: #selectedCostume with: aCostume.! !

!DrGBuildTool methodsFor: 'accessing' stamp: 'HilaireFernandes 1/5/2006 17:13'!
builder
	^builder! !

!DrGBuildTool methodsFor: 'accessing' stamp: 'HilaireFernandes 1/5/2006 17:12'!
builder: aBuilder
	builder := aBuilder ! !

!DrGBuildTool methodsFor: 'accessing' stamp: 'HilaireFernandes 3/13/2011 21:04'!
description 
	^ builder class description! !

!DrGBuildTool methodsFor: 'initialize-release' stamp: 'HilaireFernandes 7/22/2008 16:27'!
initialize
	super initialize.
	self switchState:  DrGBuildToolState.
	selectedCostumes := OrderedCollection new! !

!DrGBuildTool methodsFor: 'testing' stamp: 'HilaireFernandes 1/6/2006 10:09'!
isWanted: aMathItemCollection
	^builder isWanted: aMathItemCollection  ! !

!DrGBuildTool methodsFor: 'initialize-release' stamp: 'HilaireFernandes 1/5/2006 17:50'!
registerDependant: anObject
	builder when: #readyToBuild send: #readyToBuild to: anObject! !

!DrGBuildTool methodsFor: 'initialize-release' stamp: 'HilaireFernandes 8/18/2008 12:27'!
release
	super release.
	builder release.
	self releaseActionMap. ! !

!DrGBuildTool methodsFor: 'updating' stamp: 'HilaireFernandes 6/15/2010 22:29'!
reset
	super reset.
	self stopBlinking.
	self builder reset.
	selectedCostumes := OrderedCollection new! !

!DrGBuildTool methodsFor: 'updating' stamp: 'HilaireFernandes 7/24/2008 23:49'!
startBlinking
	selectedCostumes do: [ :aCostume | aCostume startBlinking ]! !

!DrGBuildTool methodsFor: 'updating' stamp: 'HilaireFernandes 7/24/2008 23:49'!
stopBlinking
	selectedCostumes do: [ :aCostume | aCostume stopBlinking ]! !

DrGBuildTool subclass: #DrGFlyPointBuildTool
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'DrGeoII-Core-Tool'!

!DrGFlyPointBuildTool commentStamp: '<historical>' prior: 0!
This tool lets the user create free point, point on curve and intersection point on the fly.
On the fly means create the point in one-clic operation.!

!DrGFlyPointBuildTool methodsFor: 'accessing' stamp: 'HilaireFernandes 3/21/2010 20:57'!
costumesAt: aPoint
	^self intersectableCurves: ((self myCostumesAt: aPoint) select: 
			[:aCostume|
				aCostume isCurveCostume])
		at: aPoint! !

!DrGFlyPointBuildTool methodsFor: 'initialize' stamp: 'HilaireFernandes 7/22/2008 16:29'!
initialize
	super initialize.
	self switchState: DrGFlyPointBuildToolState! !

!DrGFlyPointBuildTool methodsFor: 'accessing' stamp: 'HilaireFernandes 3/21/2010 20:54'!
intersectableCurves: aCollection at: aPoint 
	"As long as we can't get proper intersection with the first two, we remove the first one an try again "
	aCollection size <= 1 ifTrue: [ ^ aCollection ].
	^ (	DrGFlyPointBuilder new isCurveIntersectionValid: { 
			(aCollection first mathItem).
			(aCollection second mathItem).
			aPoint}) 
		ifTrue: [ aCollection ]
		ifFalse: [self intersectableCurves: aCollection allButFirst at: aPoint ]! !

DrGBuildTool subclass: #DrGMacroBuildTool
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'DrGeoII-Core-Tool'!

!DrGMacroBuildTool methodsFor: 'updating' stamp: 'HilaireFernandes 7/27/2008 10:11'!
cancel
	"the builder informs us the user wants to cancel the macro building process"
	presenter selectTool: DrGSelectTool! !

!DrGMacroBuildTool methodsFor: 'accessing' stamp: 'HilaireFernandes 7/22/2008 21:27'!
costumesAt: aPoint
	^self pointCostumesFirstAt: aPoint ! !

!DrGMacroBuildTool methodsFor: 'initialization' stamp: 'HilaireFernandes 8/4/2008 10:18'!
initialize
	super initialize.
	self switchState: DrGMacroToolStateNeutral ! !

!DrGMacroBuildTool methodsFor: 'initialize-release' stamp: 'HilaireFernandes 7/27/2008 10:26'!
registerDependant: anObject
	builder when: #changedState send: #switchState: to: self.
	builder when: #cancel send: #cancel to: self.
	builder when: #readyToBuild send: #readyToBuildMacro to: anObject! !

!DrGMacroBuildTool methodsFor: 'accessing' stamp: 'HilaireFernandes 4/19/2010 13:19'!
selectCostumesFrom: aMathItemCollection 
	presenter ifNil: [^self].
	self stopBlinking.
	selectedCostumes := self app costumesOf: aMathItemCollection.
	self startBlinking! !

!DrGMacroBuildTool methodsFor: 'accessing' stamp: 'HilaireFernandes 8/18/2008 12:27'!
selectInputCostume
	self selectCostumesFrom: builder input! !

!DrGMacroBuildTool methodsFor: 'accessing' stamp: 'HilaireFernandes 7/25/2008 13:18'!
selectNoCostume
	self selectCostumesFrom: OrderedCollection new! !

!DrGMacroBuildTool methodsFor: 'accessing' stamp: 'HilaireFernandes 8/18/2008 12:27'!
selectOutputCostume
	self selectCostumesFrom: builder output! !

DrGMacroBuildTool subclass: #DrGMacroPlayTool
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'DrGeoII-Core-Tool'!

!DrGMacroPlayTool methodsFor: 'updating' stamp: 'HilaireFernandes 7/29/2008 09:24'!
changedMacro
" the builder inform us the user selected another macro-construction,
do some cleaning"
	self selectNoCostume! !

!DrGMacroPlayTool methodsFor: 'initialize-release' stamp: 'HilaireFernandes 7/28/2008 22:06'!
registerDependant: anObject
	builder when: #changedState send: #switchState: to: self.
	builder when: #cancel send: #cancel to: self.
	builder when: #readyToBuild send: #readyToPlayMacro to: anObject.
	builder when: #changedMacro send: #changedMacro to: self! !

DrGMacroBuildTool subclass: #DrGScriptPlayTool
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'DrGeoII-Core-Tool'!

!DrGScriptPlayTool methodsFor: 'updating' stamp: 'HilaireFernandes 8/3/2008 11:54'!
changedScript
" the builder inform us the user selected another script,
do some cleaning"
	self selectNoCostume! !

!DrGScriptPlayTool methodsFor: 'initialize-release' stamp: 'HilaireFernandes 8/3/2008 11:44'!
registerDependant: anObject
	builder when: #changedState send: #switchState: to: self.
	builder when: #cancel send: #cancel to: self.
	builder when: #readyToBuild send: #readyToBuild to: anObject.
	builder when: #changedScript send: #changedScript to: self! !

!DrGDynamicTool methodsFor: 'accessing' stamp: 'HilaireFernandes 4/19/2010 13:17'!
app 
	^presenter app! !

!DrGDynamicTool methodsFor: 'accessing' stamp: 'HilaireFernandes 4/19/2010 13:15'!
area 
	^presenter area! !

!DrGDynamicTool methodsFor: 'handling' stamp: 'HilaireFernandes 5/19/2010 21:48'!
chooseCostume: costumes 
	| aChoice |
	aChoice := UIManager default
		chooseFrom: (costumes
			collect: [:aCostume | aCostume mathItem descriptiveName])
		title: 'Select an object' translated.
	aChoice isZero
		ifTrue: [^ self handleChoice: {}].
	^ self handleChoice: {costumes at: aChoice}! !

!DrGDynamicTool methodsFor: 'accessing' stamp: 'HilaireFernandes 12/30/2005 09:05'!
costume
	^costume ! !

!DrGDynamicTool methodsFor: 'accessing' stamp: 'HilaireFernandes 3/2/2006 18:06'!
costume: aCostume
	costume ifNotNil: [costume stopBlinking].
	costume := aCostume.
	costume ifNotNil: [costume startBlinking]
	! !

!DrGDynamicTool methodsFor: 'accessing' stamp: 'HilaireFernandes 7/22/2008 16:51'!
costumesAt: aPoint
"return the list of costume at position aPoint"
	^self myCostumesAt: aPoint ! !

!DrGDynamicTool methodsFor: 'accessing' stamp: 'HilaireFernandes 3/13/2011 21:05'!
description 
	^ self subclassResponsibility! !

!DrGDynamicTool methodsFor: 'accessing' stamp: 'HilaireFernandes 4/19/2010 13:24'!
domain 
	^presenter domain! !

!DrGDynamicTool methodsFor: 'accessing' stamp: 'HilaireFernandes 7/20/2011 16:40'!
factory  
	^self domain factory! !

!DrGDynamicTool methodsFor: 'updating' stamp: 'HilaireFernandes 4/19/2010 13:17'!
gridPoint: ungriddedPoint
	^ self area gridPoint: ungriddedPoint.! !

!DrGDynamicTool methodsFor: 'handling' stamp: 'HilaireFernandes 11/13/2011 12:06'!
handleChoice: aCostumeCollection
	^ self state handleChoice: aCostumeCollection ! !

!DrGDynamicTool methodsFor: 'handling' stamp: 'HilaireFernandes 1/12/2006 15:01'!
handleMouseAt: aPoint
	^self state handleMouseAt: aPoint! !

!DrGDynamicTool methodsFor: 'handling' stamp: 'HilaireFernandes 1/8/2006 19:17'!
handlePress: aPoint
	^self state handlePress: aPoint ! !

!DrGDynamicTool methodsFor: 'handling' stamp: 'HilaireFernandes 11/12/2011 22:25'!
handlePressShiftKey: aPoint
	^self state handlePressShiftKey: aPoint ! !

!DrGDynamicTool methodsFor: 'handling' stamp: 'HilaireFernandes 12/12/2005 18:25'!
handleRelease: aPoint
	self state handleRelease: aPoint ! !

!DrGDynamicTool methodsFor: 'handling' stamp: 'HilaireFernandes 11/12/2011 22:41'!
handleReleaseShiftKey: aPoint
	self state handleReleaseShiftKey: aPoint ! !

!DrGDynamicTool methodsFor: 'handling' stamp: 'HilaireFernandes 11/12/2011 22:23'!
handleShiftKey: aPoint
	^self state handleShiftKey: aPoint ! !

!DrGDynamicTool methodsFor: 'handling' stamp: 'HilaireFernandes 11/12/2011 22:30'!
handleShiftKeyMouseAt: aPoint
	^self state handleShiftKeyMouseAt: aPoint! !

!DrGDynamicTool methodsFor: 'initialize-release' stamp: 'HilaireFernandes 7/18/2008 16:48'!
initialize
	costume := nil.
	self last: 0@0.! !

!DrGDynamicTool methodsFor: 'testing' stamp: 'HilaireFernandes 4/9/2010 21:42'!
isGridded
	^presenter isGridded! !

!DrGDynamicTool methodsFor: 'testing' stamp: 'HilaireFernandes 1/5/2006 17:53'!
isWanted: aMathItem
	^true! !

!DrGDynamicTool methodsFor: 'accessing' stamp: 'HilaireFernandes 12/18/2005 21:35'!
last
	^last! !

!DrGDynamicTool methodsFor: 'accessing' stamp: 'HilaireFernandes 12/18/2005 21:35'!
last: aPoint
	last := aPoint! !

!DrGDynamicTool methodsFor: 'accessing' stamp: 'HilaireFernandes 12/30/2005 09:06'!
mathItem
	^self costume mathItem! !

!DrGDynamicTool methodsFor: 'private' stamp: 'HilaireFernandes 11/5/2011 23:04'!
myCostumesAt: aPoint
"return the list of costume at position aPoint"
	^self app costumes select: [:aCostume | 
		aCostume style isNotHidden 
			and: [(self isWanted: {aCostume mathItem})
				and: [aCostume containsPoint: aPoint]]]! !

!DrGDynamicTool methodsFor: 'private' stamp: 'HilaireFernandes 11/5/2011 23:08'!
pointCostumesFirstAt: aPoint 
	"When the user moves the mouse over a collection of geometric items, if there
are any point items, we discard all the other geometric items"
	| myCostumes pointCostumes |
	myCostumes := self myCostumesAt: aPoint.
	pointCostumes := myCostumes select: [:aCostume | aCostume isPointCostume].
	pointCostumes ifNotEmpty: [ ^ pointCostumes ].
	^ myCostumes! !

!DrGDynamicTool methodsFor: 'accessing' stamp: 'HilaireFernandes 12/15/2005 23:25'!
presenter
	^presenter ! !

!DrGDynamicTool methodsFor: 'accessing' stamp: 'HilaireFernandes 12/15/2005 23:25'!
presenter: aPresenter
	presenter := aPresenter ! !

!DrGDynamicTool methodsFor: 'initialize-release' stamp: 'HilaireFernandes 1/5/2006 17:37'!
registerDependant: anObject! !

!DrGDynamicTool methodsFor: 'initialize-release' stamp: 'HilaireFernandes 8/18/2008 12:23'!
release
	super release.
	self reset.! !

!DrGDynamicTool methodsFor: 'initialize-release' stamp: 'HilaireFernandes 8/18/2008 12:57'!
reset
	self costume: nil;
		last: nil;
		spot: nil.
	mathItemUnderMouse := nil! !

!DrGDynamicTool methodsFor: 'initialize-release' stamp: 'HilaireFernandes 10/25/2011 18:15'!
resetForFreshStart 
	self reset.! !

!DrGDynamicTool methodsFor: 'accessing' stamp: 'HilaireFernandes 12/30/2005 09:08'!
spot
	^spot! !

!DrGDynamicTool methodsFor: 'accessing' stamp: 'HilaireFernandes 12/30/2005 09:08'!
spot: aPoint
	spot := aPoint ! !

!DrGDynamicTool methodsFor: 'accessing' stamp: 'HilaireFernandes 12/12/2005 18:05'!
state
	^state! !

!DrGDynamicTool methodsFor: 'accessing' stamp: 'HilaireFernandes 12/12/2005 18:05'!
state: aState
	state :=	aState ! !

!DrGDynamicTool methodsFor: 'updating' stamp: 'HilaireFernandes 7/20/2008 09:54'!
switchState: aStateClass
	self state: (aStateClass new context: self)! !

DrGDynamicTool subclass: #DrGEditGroupTool
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'DrGeoII-Core-Tool'!

!DrGEditGroupTool methodsFor: 'accessing' stamp: 'HilaireFernandes 7/22/2008 20:52'!
compositeCostumesAt: aPoint
	^self myCostumesAt: aPoint! !

!DrGEditGroupTool methodsFor: 'accessing' stamp: 'HilaireFernandes 7/22/2008 16:43'!
costumesAt: aPoint
	^self pointCostumesFirstAt: aPoint! !

!DrGEditGroupTool methodsFor: 'initialization' stamp: 'HilaireFernandes 7/18/2008 16:52'!
initialize
	super initialize.
	self reset! !

!DrGEditGroupTool methodsFor: 'private' stamp: 'HilaireFernandes 7/24/2010 19:15'!
myCostumesAt: aPoint
	^self app costumes select: [:aCostume |
		aCostume style isNotHidden
			and: [aCostume isCompositeCostume
				and: [aCostume containsPoint: aPoint]]]! !

!DrGEditGroupTool methodsFor: 'private' stamp: 'HilaireFernandes 3/3/2006 17:49'!
pointCostumesFirstAt: aPoint
	^(super pointCostumesFirstAt: aPoint) select: [:aCostume |
		aCostume isCompositeCostume not]! !

!DrGEditGroupTool methodsFor: 'initialize-release' stamp: 'HilaireFernandes 7/31/2008 15:07'!
release
	costume ifNotNil:
		[costume stopBlinking].
	super release.! !

!DrGEditGroupTool methodsFor: 'updating' stamp: 'HilaireFernandes 7/20/2008 09:55'!
reset
	super reset.
	self switchState:  DrGEditGroupToolStateNeutral! !

DrGDynamicTool subclass: #DrGFlyBuildTool
	instanceVariableNames: 'flyPointTool mathItemTool flyingCostume'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'DrGeoII-Core-Tool'!

!DrGFlyBuildTool commentStamp: 'HilaireFernandes 7/3/2011 10:21' prior: 0!
This tool is designed for touch screen, to construct line with sliding finger!

!DrGFlyBuildTool class methodsFor: 'as yet unclassified' stamp: 'HilaireFernandes 1/7/2006 11:25'!
with: aBuilderClassName
	^self new mathItemTool: aBuilderClassName	! !

!DrGFlyBuildTool methodsFor: 'adding' stamp: 'HilaireFernandes 1/7/2006 11:28'!
add: aCostume
	mathItemTool add: aCostume ! !

!DrGFlyBuildTool methodsFor: 'accessing' stamp: 'HilaireFernandes 1/8/2006 19:44'!
builder
"We have two builder, the flyPointBuilder and the other one (line, segment,...)"
	flyPointTool builder readyToBuild 
		ifTrue: [^flyPointTool builder]
		ifFalse: [^mathItemTool builder]! !

!DrGFlyBuildTool methodsFor: 'accessing' stamp: 'HilaireFernandes 1/7/2006 11:29'!
builder: aBuilder
	mathItemTool builder: aBuilder.! !

!DrGFlyBuildTool methodsFor: 'accessing' stamp: 'HilaireFernandes 3/13/2011 21:10'!
description 
	^ mathItemTool builder class description ! !

!DrGFlyBuildTool methodsFor: 'accessing' stamp: 'HilaireFernandes 1/7/2006 11:43'!
flyPointTool
	^flyPointTool! !

!DrGFlyBuildTool methodsFor: 'accessing' stamp: 'HilaireFernandes 4/19/2010 13:15'!
flyingCostume
	flyingCostume ifNil: 
			[flyingCostume := DrGFlyingCostume with: self mathItemTool builder
						in: self area.
			self mathItemTool 
				when: #selectedCostume
				send: #add:
				to: flyingCostume].
	^flyingCostume! !

!DrGFlyBuildTool methodsFor: 'initialize-release' stamp: 'HilaireFernandes 7/20/2008 09:56'!
initialize
	super initialize.
	self switchState: DrGFlyBuildToolState.
	flyPointTool := DrGFlyPointBuildTool with: DrGFlyPointBuilder! !

!DrGFlyBuildTool methodsFor: 'testing' stamp: 'HilaireFernandes 1/7/2006 11:30'!
isWanted: aMathItem
	^mathItemTool isWanted: aMathItem! !

!DrGFlyBuildTool methodsFor: 'accessing' stamp: 'HilaireFernandes 1/7/2006 11:43'!
mathItemTool
	^mathItemTool! !

!DrGFlyBuildTool methodsFor: 'accessing' stamp: 'HilaireFernandes 1/24/2006 23:11'!
mathItemTool: aBuilderClassName
	mathItemTool ifNil: 
		[mathItemTool := DrGBuildTool with: aBuilderClassName] ! !

!DrGFlyBuildTool methodsFor: 'accessing' stamp: 'HilaireFernandes 1/7/2006 11:27'!
presenter: aPresenter
	super presenter: aPresenter.
	mathItemTool presenter: aPresenter.
	flyPointTool presenter: aPresenter.! !

!DrGFlyBuildTool methodsFor: 'initialize-release' stamp: 'HilaireFernandes 8/18/2008 12:26'!
registerDependant: anObject
	mathItemTool registerDependant: anObject.
	flyPointTool registerDependant: anObject! !

!DrGFlyBuildTool methodsFor: 'initialize-release' stamp: 'HilaireFernandes 7/31/2008 22:23'!
release
	super release
	mathItemTool release.
	flyPointTool release.
	flyingCostume release.
	flyingCostume := nil.! !

!DrGFlyBuildTool methodsFor: 'updating' stamp: 'HilaireFernandes 10/25/2011 18:11'!
reset
"Why do we reset?"
	"because intermediate point where created"
	flyPointTool builder readyToBuild ifTrue: [flyPointTool reset].
	"or the final mathItem was created"
	mathItemTool builder readyToBuild ifTrue:	
		[self resetForFreshStart].

! !

!DrGFlyBuildTool methodsFor: 'updating' stamp: 'HilaireFernandes 10/25/2011 18:11'!
resetForFreshStart
"Reset to restart over the user  build process"
	mathItemTool reset.
	flyPointTool reset.
	flyingCostume release.
	flyingCostume := nil.
	self flyingCostume

! !

DrGFlyBuildTool subclass: #DrGFlyBuildTouchTool
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'DrGeoII-Core-Tool'!

!DrGFlyBuildTouchTool commentStamp: 'HilaireFernandes 7/3/2011 10:22' prior: 0!
This tool is designed for touch screen, to construct line with sliding finger!

!DrGFlyBuildTouchTool methodsFor: 'initialize-release' stamp: 'HilaireFernandes 7/3/2011 11:00'!
initialize 
	super initialize.
	self switchState: DrGFlyBuildTouchToolState.! !

DrGDynamicTool subclass: #DrGSelectTool
	instanceVariableNames: 'start mathItems costumes'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'DrGeoII-Core-Tool'!

!DrGSelectTool commentStamp: 'HilaireFernandes 7/20/2011 16:34' prior: 0!
start					- the position where start the dragging
mathItems                    - the mathItems impacted by the dragging operation
costumes                     - the costumes impacted by the draggin opration!

!DrGSelectTool class methodsFor: 'documentation' stamp: 'HilaireFernandes 3/13/2011 21:16'!
description
	^ 'Select and move an object.' translated! !

!DrGSelectTool class methodsFor: 'documentation' stamp: 'HilaireFernandes 3/13/2011 21:15'!
title 
	^ 'Select and Move' translated! !

!DrGSelectTool methodsFor: 'accessing' stamp: 'HilaireFernandes 7/20/2011 16:48'!
costumes
	^ costumes ! !

!DrGSelectTool methodsFor: 'accessing' stamp: 'HilaireFernandes 7/22/2008 16:53'!
costumesAt: aPoint
	^ self pointCostumesFirstAt: aPoint! !

!DrGSelectTool methodsFor: 'accessing' stamp: 'HilaireFernandes 3/13/2011 21:17'!
description
	^ self class description! !

!DrGSelectTool methodsFor: 'initialize-release' stamp: 'HilaireFernandes 1/20/2006 12:02'!
initialize
	self reset! !

!DrGSelectTool methodsFor: 'accessing' stamp: 'HilaireFernandes 7/20/2011 16:48'!
mathItems 
	^ mathItems ! !

!DrGSelectTool methodsFor: 'accessing' stamp: 'HilaireFernandes 11/14/2011 14:23'!
mutableCostumesAt: aPoint
"select the costume we can mutate"
	^ (self costumesAt: aPoint) select: [:aCostume | 
		aCostume mathItem isFreePointItem or: [aCostume mathItem isIntersectionPointItem]]! !

!DrGSelectTool methodsFor: 'updating' stamp: 'HilaireFernandes 7/20/2011 16:39'!
reset
	super reset.
	start := nil.
	self switchState: DrGSelectToolStateNeutral.
	costumes := nil.
	mathItems := nil! !

!DrGSelectTool methodsFor: 'accessing' stamp: 'HilaireFernandes 12/27/2005 12:00'!
start
	^start! !

!DrGSelectTool methodsFor: 'accessing' stamp: 'HilaireFernandes 12/27/2005 12:00'!
start: aPoint
	start := aPoint ! !

!DrGSelectTool methodsFor: 'accessing' stamp: 'HilaireFernandes 11/14/2011 15:18'!
targetMutationCostumeAt: aPoint
	"Only the relevant item as target to mute a point, in our case curve,
	We need to fine tune this target list"
	^ (self myCostumesAt: aPoint) select: [:aCostume |
		aCostume mathItem isCurveItem and: [
			self domain factory is: aCostume mathItem before: self mathItem]]! !

!DrGSelectTool methodsFor: 'as yet unclassified' stamp: 'HilaireFernandes 11/16/2011 17:42'!
updateDirtyItemsList
"dirty items are the one needing update, both model and view"
"	mathItems := self factory dirtyItemsWhen: costume mathItem.
	costumes := self app costumesOf: mathItems. "
	mathItems := self factory pool.
	costumes := self app costumes! !

DrGDynamicTool subclass: #DrGStyleTool
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'DrGeoII-Core-Tool'!

!DrGStyleTool commentStamp: '<historical>' prior: 0!
A tool to edit the style of the costume.!

DrGStyleTool subclass: #DrGDeleteTool
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'DrGeoII-Core-Tool'!

!DrGDeleteTool commentStamp: '<historical>' prior: 0!
A tool to delete the mathItem. With subclass from StyleTool to have hidden costume to be display, so we can delete these ones aswell.!

!DrGDeleteTool class methodsFor: 'documentation' stamp: 'HilaireFernandes 11/5/2011 22:14'!
description 
	^'Erase an object and all its depedencies.' translated! !

!DrGDeleteTool class methodsFor: 'documentation' stamp: 'HilaireFernandes 3/13/2011 21:19'!
title 
	^'Eraser' translated ! !

!DrGDeleteTool methodsFor: 'updating' stamp: 'HilaireFernandes 7/20/2008 09:54'!
reset
	super reset.
	self switchState: DrGDeleteToolState! !

DrGStyleTool subclass: #DrGPropertyTool
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'DrGeoII-Core-Tool'!

!DrGPropertyTool commentStamp: 'HilaireFernandes 10/26/2011 23:04' prior: 0!
A tool to edit the property of some object like free point, free point on curve,  free value and script!

!DrGPropertyTool class methodsFor: 'documentation' stamp: 'HilaireFernandes 10/26/2011 23:05'!
description
	^ 'Edit an item''s property' translated! !

!DrGPropertyTool class methodsFor: 'documentation' stamp: 'HilaireFernandes 10/26/2011 23:05'!
title
	^ 'Property' translated! !

!DrGPropertyTool methodsFor: 'testing' stamp: 'HilaireFernandes 10/27/2011 10:03'!
isWanted: aMathItem
" only items whose property are editable "
	^ aMathItem isPropertyEditable! !

!DrGPropertyTool methodsFor: 'as yet unclassified' stamp: 'HilaireFernandes 10/28/2011 18:08'!
presenter: aPresenter
	super presenter: aPresenter.
	self app forceVisible: false.! !

!DrGPropertyTool methodsFor: 'updating' stamp: 'HilaireFernandes 10/27/2011 10:23'!
reset
	super reset.
	self switchState: DrGPropertyToolState! !

!DrGStyleTool class methodsFor: 'documentation' stamp: 'HilaireFernandes 3/13/2011 21:18'!
description 
	^ 'Edit an object style.' translated! !

!DrGStyleTool class methodsFor: 'documentation' stamp: 'HilaireFernandes 3/13/2011 21:17'!
title
	^ 'Edit Style' translated! !

!DrGStyleTool methodsFor: 'accessing' stamp: 'HilaireFernandes 7/22/2008 20:32'!
costumesAt: aPoint
	^self pointCostumesFirstAt: aPoint ! !

!DrGStyleTool methodsFor: 'accessing' stamp: 'HilaireFernandes 3/13/2011 21:16'!
description 
	^ self class description! !

!DrGStyleTool methodsFor: 'initialize-release' stamp: 'HilaireFernandes 7/18/2008 16:49'!
initialize
	super initialize.
	self reset.! !

!DrGStyleTool methodsFor: 'private' stamp: 'HilaireFernandes 10/27/2011 10:07'!
myCostumesAt: aPoint
"return the list of costume at position aPoint"
	^self app costumes select: [:aCostume |
		(self isWanted: aCostume mathItem)
		and: [aCostume containsPoint: aPoint]]! !

!DrGStyleTool methodsFor: 'accessing' stamp: 'HilaireFernandes 4/19/2010 13:20'!
presenter: aPresenter
	super presenter: aPresenter.
	self app forceVisible: true.! !

!DrGStyleTool methodsFor: 'initialize-release' stamp: 'HilaireFernandes 4/19/2010 13:20'!
release
	self app forceVisible: false.
	super release.! !

!DrGStyleTool methodsFor: 'updating' stamp: 'HilaireFernandes 7/20/2008 09:59'!
reset
	super reset.
	self switchState: DrGStyleToolState! !

DrGStyleTool subclass: #DrGViewerTool
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'DrGeoII-Core-Tool'!

!DrGViewerTool commentStamp: '<historical>' prior: 0!
A tool to select an item to open its viewer!

!DrGViewerTool class methodsFor: 'documentation' stamp: 'HilaireFernandes 3/13/2011 21:21'!
description 
	^ 'Open the Etoys script viewer for a given geometric object.' translated! !

!DrGViewerTool class methodsFor: 'documentation' stamp: 'HilaireFernandes 3/13/2011 21:20'!
title
	^'Etoys Viewer' translated  ! !

!DrGViewerTool methodsFor: 'updating' stamp: 'HilaireFernandes 7/20/2008 09:59'!
reset
	super reset.
	self switchState: DrGViewerToolState! !

Object subclass: #DrGDynamicToolState
	instanceVariableNames: 'context'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'DrGeoII-Core-Tool'!

!DrGDynamicToolState commentStamp: '<historical>' prior: 0!
The context is a tool (selectTool, buildTool, ...) where the state operate.!

DrGDynamicToolState subclass: #DrGBuildToolState
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'DrGeoII-Core-Tool'!

!DrGBuildToolState methodsFor: 'handling' stamp: 'HilaireFernandes 3/21/2006 10:15'!
handleChoice: aCostumeCollection
|aMathItemCollection|
	aMathItemCollection := OrderedCollection new.
	aCostumeCollection do: [:aCostume|
		aMathItemCollection add: aCostume mathItem.
		self context add: aCostume].
	^self context builder 
		add: aMathItemCollection 
		at: (self area gridWorldPoint: self context last)! !

!DrGBuildToolState methodsFor: 'handling' stamp: 'HilaireFernandes 7/22/2008 16:26'!
handlePress: aPoint 
	"return true if we process something (including additional user choice)"

	| costumes |
	self area hideTip.
	self context last: aPoint.
	costumes := self context costumesAt: aPoint.
	costumes size = 1 
		ifTrue: 
			[self handleChoice: costumes.
			^true].
	(costumes size = 0 and: [self context builder isWanted: { }]) 
		ifTrue: 
			[self handleChoice: costumes.
			^true].
	"More than one math item under mouse, user must choose one item"
	costumes size >= 2 
		ifTrue: 
			["Display a pop-up menu to select one item"
			self context chooseCostume: costumes.
			^true].
	^false! !

DrGBuildToolState subclass: #DrGFlyPointBuildToolState
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'DrGeoII-Core-Tool'!

!DrGFlyPointBuildToolState methodsFor: 'handling' stamp: 'HilaireFernandes 7/22/2008 20:41'!
handleMouseAt: aPoint 
	| costumes aTip |
	costumes := self context costumesAt: aPoint.
	costumes size = 1
		ifTrue: [aTip := costumes first mathItem descriptiveName].
	costumes size > 1
		ifTrue: [aTip := 'This intersection' translated].
	aTip
		ifNil: [self area hideTip]
		ifNotNil: [self area tip: aTip at: aPoint].
	^ aTip notNil! !

!DrGFlyPointBuildToolState methodsFor: 'handling' stamp: 'HilaireFernandes 8/23/2008 15:55'!
handlePress: aPoint 
	"return true if did something"
	| costumes |
	self area hideTip.
	self context last: (self context gridPoint: aPoint).
	costumes := self context costumesAt: aPoint.
	"user clicked in the background or in one or several curve item, handle it immediately"
	self handleChoice: costumes.
	^ true! !

DrGBuildToolState subclass: #DrGMacroBuildToolStateInput
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'DrGeoII-Core-Tool'!

!DrGMacroBuildToolStateInput commentStamp: 'HilaireFernandes 7/25/2008 09:05' prior: 0!
When user selects input page, blink the cosumes related to this state. In this state the user can select items as input parameters for the macro. He can also unselect items.!

!DrGMacroBuildToolStateInput methodsFor: 'accessing' stamp: 'HilaireFernandes 7/25/2008 13:16'!
context: aContext
	super context: aContext.
	self context selectInputCostume ! !

!DrGMacroBuildToolStateInput methodsFor: 'handling' stamp: 'HilaireFernandes 8/4/2008 10:46'!
handleChoice: aCostumeCollection 
	self context builder
		add: (aCostumeCollection collect: [ :c | c mathItem ])	
			at: (self area gridWorldPoint: self context last);
		changed: #inputName.
	self context selectInputCostume! !

DrGMacroBuildToolStateInput subclass: #DrGMacroPlayToolStateSelect
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'DrGeoII-Core-Tool'!

!DrGMacroPlayToolStateSelect methodsFor: 'handling' stamp: 'HilaireFernandes 8/4/2008 10:47'!
handleChoice: aCostumeCollection 
	self context builder 
		add: (aCostumeCollection collect: [ :c | c mathItem ])
		at: (self area gridWorldPoint: self context last).
	self context selectInputCostume! !

DrGBuildToolState subclass: #DrGMacroBuildToolStateOutput
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'DrGeoII-Core-Tool'!

!DrGMacroBuildToolStateOutput commentStamp: 'HilaireFernandes 7/25/2008 09:06' prior: 0!
When user selects  output page, blink the cosumes related to this state. In this state the user can select items as output parameters for the macro. He can also unselect items.!

!DrGMacroBuildToolStateOutput methodsFor: 'handling' stamp: 'HilaireFernandes 7/25/2008 13:36'!
context: aContext
	super context: aContext.
	self context selectOutputCostume ! !

!DrGMacroBuildToolStateOutput methodsFor: 'handling' stamp: 'HilaireFernandes 8/4/2008 10:47'!
handleChoice: aCostumeCollection 
	self context builder
		add: (aCostumeCollection collect: [ :c | c mathItem ])
			at: (self area gridWorldPoint: self context last);
		changed: #outputName.
	self context selectOutputCostume! !

!DrGDynamicToolState methodsFor: 'accessing' stamp: 'HilaireFernandes 4/19/2010 13:21'!
area
	^self context area! !

!DrGDynamicToolState methodsFor: 'accessing' stamp: 'HilaireFernandes 12/12/2005 18:03'!
context
	^context! !

!DrGDynamicToolState methodsFor: 'accessing' stamp: 'HilaireFernandes 12/12/2005 18:02'!
context: aContext
	context := aContext ! !

!DrGDynamicToolState methodsFor: 'handling' stamp: 'HilaireFernandes 7/20/2008 01:18'!
handleChoice: aCostumeCollection
	^false! !

!DrGDynamicToolState methodsFor: 'handling' stamp: 'HilaireFernandes 11/5/2011 22:41'!
handleMouseAt: aPoint 
	"Return true if we display a tip"
	| costumes aTip |
	costumes := self context costumesAt: aPoint.
	costumes size = 1
		ifTrue: [aTip := costumes first mathItem descriptiveName.].
	costumes size > 1
		ifTrue: [aTip := 'Several objects can be selected. Please, select one clicking the mouse.' translated. ].
	aTip
		ifNil: [self area hideTip]
		ifNotNil: [self area tip: aTip at: aPoint].
	^ aTip notNil! !

!DrGDynamicToolState methodsFor: 'handling' stamp: 'HilaireFernandes 7/20/2008 01:17'!
handlePress: aPoint
	^ false! !

!DrGDynamicToolState methodsFor: 'handling' stamp: 'HilaireFernandes 11/12/2011 22:32'!
handlePressShiftKey: aPoint
"handle pressed mouse bouton with shift key, by default  just handle mouse button press"
	^ self handlePress: aPoint! !

!DrGDynamicToolState methodsFor: 'handling' stamp: 'HilaireFernandes 7/20/2008 01:12'!
handleRelease: aPoint 
! !

!DrGDynamicToolState methodsFor: 'handling' stamp: 'HilaireFernandes 11/12/2011 22:46'!
handleReleaseShiftKey: aPoint 
" handle mouse bouton release while shift key down, by default only handle release"
	self handleRelease: aPoint! !

!DrGDynamicToolState methodsFor: 'handling' stamp: 'HilaireFernandes 11/12/2011 22:32'!
handleShiftKey: aPoint
"handle shift key down, do nothing by default"
	^ false! !

!DrGDynamicToolState methodsFor: 'handling' stamp: 'HilaireFernandes 11/12/2011 22:31'!
handleShiftKeyMouseAt: aPoint 
"handle moved mouse while shift key down, by default just handle mouse at"
	^ self handleMouseAt: aPoint! !

!DrGDynamicToolState methodsFor: 'state' stamp: 'HilaireFernandes 7/20/2008 09:54'!
switchState: aStateClass
	self context switchState: aStateClass! !

!DrGDynamicToolState methodsFor: 'accessing' stamp: 'HilaireFernandes 4/19/2010 10:38'!
view
	^self context presenter view! !

DrGDynamicToolState subclass: #DrGFlyBuildToolState
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'DrGeoII-Core-Tool'!

!DrGFlyBuildToolState methodsFor: 'processing' stamp: 'HilaireFernandes 7/3/2011 11:49'!
addFlyingPointToBuiltItemAt: aPoint
	self context mathItemTool 
		add: self context app lastCostume.
	self context mathItemTool builder 
		add: {self context domain factory last} 
		at: aPoint.! !

!DrGFlyBuildToolState methodsFor: 'handling' stamp: 'HilaireFernandes 7/3/2011 11:16'!
handleMouseAt: aPoint 
	self moveFlyingCostumeAt: aPoint.
	self context last: aPoint.
	"dispatch handleMouseAt"
	^ (self context mathItemTool handleMouseAt: aPoint) 
		ifFalse: [self isFlyingPointWantedAt: aPoint]
		ifTrue: [true]! !

!DrGFlyBuildToolState methodsFor: 'handling' stamp: 'HilaireFernandes 7/3/2011 11:48'!
handlePress: aPoint 
	self context last: aPoint.
	"do we select an item underneath?"
	(self context mathItemTool handlePress: aPoint) ifTrue: [^true].
	"do we want a flyingPoint ?"
	(self isFlyingPointConstructedAt: aPoint) ifFalse: [^false].
	self addFlyingPointToBuiltItemAt: aPoint.
	^ true! !

!DrGFlyBuildToolState methodsFor: 'handling' stamp: 'HilaireFernandes 1/8/2006 19:11'!
handleRelease: aPoint
	self context mathItemTool handleRelease: aPoint.
	self context flyPointTool handleRelease: aPoint.! !

!DrGFlyBuildToolState methodsFor: 'testing' stamp: 'HilaireFernandes 7/3/2011 11:29'!
isFlyingPointConstructedAt: aPoint
"return true whenever a flypoint is constructed, the new point instance is then ready to use, from the flyPointTool builder"
	^ (self context mathItemTool builder isWanted: { (DrGPointItem new) }) 
		and: [self context flyPointTool handlePress: aPoint]! !

!DrGFlyBuildToolState methodsFor: 'testing' stamp: 'HilaireFernandes 7/3/2011 11:16'!
isFlyingPointWantedAt: aPoint
"are we interested by a point at position aPoint"
	^ (self context mathItemTool builder isWanted: { (DrGPointItem new) }) 
		ifTrue: [self context flyPointTool handleMouseAt: aPoint]
		ifFalse: [false]! !

!DrGFlyBuildToolState methodsFor: 'accessing' stamp: 'HilaireFernandes 7/19/2011 19:11'!
moveFlyingCostumeAt: aPoint
	self context flyingCostume 
		moveAt: (self area morphToWorld: aPoint).
! !

DrGFlyBuildToolState subclass: #DrGFlyBuildTouchToolState
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'DrGeoII-Core-Tool'!

!DrGFlyBuildTouchToolState methodsFor: 'handling' stamp: 'HilaireFernandes 7/4/2011 18:14'!
handlePress: aPoint
	| answer |
	answer := super handlePress: aPoint.
	self switchState: DrGFlyBuildTouchToolStateGrabbed.
	^ answer! !

DrGFlyBuildTouchToolState subclass: #DrGFlyBuildTouchToolStateGrabbed
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'DrGeoII-Core-Tool'!

!DrGFlyBuildTouchToolStateGrabbed commentStamp: 'HilaireFernandes 7/3/2011 10:55' prior: 0!
I am grabbing: i.e. the mouse move with bouton pressed or the user finger is moving on the touch screen!

!DrGFlyBuildTouchToolStateGrabbed methodsFor: 'handling' stamp: 'HilaireFernandes 7/4/2011 18:14'!
handleMouseAt: aPoint
	self switchState: DrGFlyBuildTouchToolStateSlided! !

!DrGFlyBuildTouchToolStateGrabbed methodsFor: 'handling' stamp: 'HilaireFernandes 7/4/2011 18:20'!
handlePress: aPoint
	"We cannot clic againt as the finger is stuck on the screen"
	^ false! !

!DrGFlyBuildTouchToolStateGrabbed methodsFor: 'handling' stamp: 'HilaireFernandes 7/4/2011 18:16'!
handleRelease: aPoint
	self switchState: DrGFlyBuildTouchToolState! !

DrGFlyBuildTouchToolState subclass: #DrGFlyBuildTouchToolStateSlided
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'DrGeoII-Core-Tool'!

!DrGFlyBuildTouchToolStateSlided methodsFor: 'handling' stamp: 'HilaireFernandes 7/4/2011 18:22'!
handlePress: aPoint
	"the finger is stuck on the screen, we cannot clic again"
	^ false! !

!DrGFlyBuildTouchToolStateSlided methodsFor: 'handling' stamp: 'HilaireFernandes 7/4/2011 18:33'!
handleRelease: aPoint
	self switchState: DrGFlyBuildTouchToolState.
	self context last: aPoint.
	"do we select an item underneath?"
	(self context mathItemTool handlePress: aPoint) ifTrue: [^true].
	"do we want a flyingPoint ?"
	(self isFlyingPointConstructedAt: aPoint) ifFalse: [^false].
	self addFlyingPointToBuiltItemAt: aPoint.
	^ true! !

DrGDynamicToolState subclass: #DrGMacroToolStateNeutral
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'DrGeoII-Core-Tool'!

!DrGMacroToolStateNeutral commentStamp: 'HilaireFernandes 7/25/2008 09:05' prior: 0!
When user switch to info pages (info and description pages), we unselect all selected costumes. In this state the user can't select any items.!

!DrGMacroToolStateNeutral methodsFor: 'accessing' stamp: 'HilaireFernandes 7/25/2008 13:18'!
context: aContext
	super context: aContext.
	self context selectNoCostume! !

!DrGMacroToolStateNeutral methodsFor: 'as yet unclassified' stamp: 'HilaireFernandes 8/4/2008 14:46'!
handleMouseAt: aPoint
	^false! !

DrGDynamicToolState subclass: #DrGSelectToolState
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'DrGeoII-Core-Tool'!

DrGSelectToolState subclass: #DrGEditGroupToolStateNeutral
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'DrGeoII-Core-Tool'!

!DrGEditGroupToolStateNeutral methodsFor: 'handling' stamp: 'HilaireFernandes 7/20/2008 09:56'!
handleChoice: aCostumeCollection
	aCostumeCollection 
		ifNotEmpty: 
			[self context costume: aCostumeCollection first.
			self switchState: DrGEditGroupToolStateSelected.
			^true]
		ifEmpty:
			[self context reset.
			^false]
		
! !

!DrGEditGroupToolStateNeutral methodsFor: 'handling' stamp: 'HilaireFernandes 7/22/2008 20:26'!
handleMouseAt: aPoint 
	| costumes aTip |
	costumes := self context compositeCostumesAt: aPoint.
	costumes
		ifEmpty: [self area hideTip.
			^ false].
	costumes size > 1
		ifTrue: [aTip := 'Several objects can be selected. Please, select one clicking the mouse.' translated]
		ifFalse: [aTip := costumes first mathItem descriptiveName].
	self area tip: aTip at: aPoint.
	^ true! !

!DrGEditGroupToolStateNeutral methodsFor: 'handling' stamp: 'HilaireFernandes 3/22/2006 14:14'!
handlePress: aPoint
	|costumes|
	self area hideTip.
	costumes := self context compositeCostumesAt: aPoint.
	costumes isEmptyOrNil ifTrue: [^false].
	costumes size = 1
		ifTrue:
			[^self handleChoice: costumes.]
		ifFalse:
			[^self context chooseCostume: costumes].! !

DrGSelectToolState subclass: #DrGEditGroupToolStateSelected
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'DrGeoII-Core-Tool'!

!DrGEditGroupToolStateSelected methodsFor: 'handling' stamp: 'HilaireFernandes 8/18/2008 12:35'!
handleChoice: aCostumeCollection
"if the costume does not bellong to the composite add it, 
otherwise remove it from the composite"
|compositeCostume itemCostume|
	aCostumeCollection ifEmpty:
			[self context reset.
			^false].
	compositeCostume := self context costume.
	itemCostume := aCostumeCollection first.
	(compositeCostume mathItem subItems includes: itemCostume mathItem)
		ifTrue:  [compositeCostume removeCostume: itemCostume]
		ifFalse: [compositeCostume addCostume: itemCostume].
	compositeCostume stopBlinking;startBlinking.
	^true.! !

!DrGEditGroupToolStateSelected methodsFor: 'handling' stamp: 'HilaireFernandes 7/22/2008 16:44'!
handlePress: aPoint
	| costumes |
	self area hideTip.
	costumes := self context costumesAt: aPoint.
	costumes size = 1
		ifTrue:
			[^self handleChoice: costumes].
	costumes size > 1 
		ifTrue:
			[^self context chooseCostume: costumes].
	self context reset.
	^false.! !

!DrGSelectToolState methodsFor: 'handling' stamp: 'HilaireFernandes 11/13/2011 12:06'!
handleChoice: aCostumeCollection 
	aCostumeCollection 
		ifNotEmpty: 
			[self context costume: aCostumeCollection first.
			self switchState: DrGSelectToolStateSelected.
			^true]
		ifEmpty: 
			[self context reset.
			^false]! !

!DrGSelectToolState methodsFor: 'accessing' stamp: 'HilaireFernandes 11/13/2011 21:27'!
mathItem 
"the selected math item"
	^ self context mathItem ! !

DrGSelectToolState subclass: #DrGSelectToolStateDragged
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'DrGeoII-Core-Tool'!

DrGSelectToolStateDragged subclass: #DrGMutatorToolStateDragged
	instanceVariableNames: 'mutator originalPointClass'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'DrGeoII-Core-Tool'!

!DrGMutatorToolStateDragged methodsFor: 'accessing' stamp: 'HilaireFernandes 11/27/2011 11:29'!
context: aContext
	super context: aContext.
	originalPointClass := context mathItem class.! !

!DrGMutatorToolStateDragged methodsFor: 'handling' stamp: 'HilaireFernandes 11/27/2011 11:40'!
handleMouseAt: aPoint
	|costumes aTip|
	"First handle the redraw"
	super handleMouseAt: aPoint.
	"Detect for mutation candidate"
	costumes := self context targetMutationCostumeAt: aPoint.
	(costumes isEmpty and: [self isMutantFreePoint not]) ifTrue: [
		aTip := 'Change as a free point on the plane.' translated].
	costumes size = 1 ifTrue: [
		aTip := 'Change as a free point on this curve.' translated].
	costumes size > 1 ifTrue: [
		aTip := 'Change as this intersection.' translated].
	aTip 
		ifNil: [self area hideTip]
		ifNotNil: [self area tip: aTip at: aPoint]! !

!DrGMutatorToolStateDragged methodsFor: 'handling' stamp: 'HilaireFernandes 11/13/2011 21:59'!
handleRelease: aPoint
	|costumes|
	self area hideTip.
	"Detect for mutation candidate"
	costumes := self context targetMutationCostumeAt: aPoint.
	costumes size = 1 ifTrue: [
		self context domain
			mutate: self mathItem asFreeOn: costumes first mathItem at: (self area gridWorldPoint: aPoint)].
	costumes size > 1 ifTrue: [
		self context domain 
			mutate: self mathItem 
			asIntersectionWith: costumes first mathItem  
			and: costumes second mathItem  
			at: (self area gridWorldPoint: aPoint)]. 

	self switchState: DrGSelectToolStateNeutral.
	self context reset
! !

!DrGMutatorToolStateDragged methodsFor: 'as yet unclassified' stamp: 'HilaireFernandes 11/13/2011 14:59'!
isMutantFreePoint
	^ originalPointClass = DrGPointFreeItem! !

!DrGSelectToolStateDragged methodsFor: 'handling' stamp: 'HilaireFernandes 11/13/2011 21:27'!
handleMouseAt: aPoint
|griddedPoint|
	griddedPoint := self context gridPoint: aPoint.
	(self context isGridded and: [self mathItem isFreePointItem])
		ifFalse:
			[self context domain 
				moveEvent: self mathItem
				inDirection: (self area morphToVector: 
									griddedPoint - self context last)
				withDirty: self context mathItems.]
		ifTrue:
			[self context domain 
				moveEvent: self mathItem
				at: (self area gridWorldPoint: griddedPoint)
				withDirty: self context mathItems].
	self context last: griddedPoint.! !

!DrGSelectToolStateDragged methodsFor: 'handling' stamp: 'HilaireFernandes 11/13/2011 21:28'!
handleRelease: aPoint 
	self switchState: DrGSelectToolStateNeutral.
	"issue a moveCommand"
	self context domain 
		moveMathItem: self mathItem 
		with: (self area morphToVector: 
			self context last - self context start).
	self context reset! !

DrGSelectToolState subclass: #DrGSelectToolStateGrabbed
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'DrGeoII-Core-Tool'!

DrGSelectToolStateGrabbed subclass: #DrGMutatorToolStateGrabbed
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'DrGeoII-Core-Tool'!

!DrGMutatorToolStateGrabbed methodsFor: 'handling' stamp: 'HilaireFernandes 11/27/2011 11:39'!
handleMouseAt: aPoint 
	"The user is moving, detach the point as a free point and switch to dragging state"
	|griddedPoint|
	self switchState: DrGMutatorToolStateDragged.
	griddedPoint := self context gridPoint: aPoint.
	self context
		start: griddedPoint;
		last: griddedPoint.
	self context domain mutateAsFreePoint: self mathItem.

! !

!DrGSelectToolStateGrabbed methodsFor: 'handling' stamp: 'HilaireFernandes 7/20/2008 09:58'!
handleMouseAt: aPoint 
	"The user is moving, switch to dragging state"
|griddedPoint|
	griddedPoint := self context gridPoint: aPoint.
	self switchState: DrGSelectToolStateDragged.
	self context
		start: griddedPoint;
		last: griddedPoint! !

!DrGSelectToolStateGrabbed methodsFor: 'handling' stamp: 'HilaireFernandes 1/24/2006 23:13'!
handleRelease: aPoint 
	self switchState: DrGSelectToolStateNeutral.
	self context reset! !

DrGSelectToolState subclass: #DrGSelectToolStateNeutral
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'DrGeoII-Core-Tool'!

!DrGSelectToolStateNeutral methodsFor: 'handling' stamp: 'HilaireFernandes 8/2/2011 14:19'!
handlePress: aPoint 
	| costumes griddedPoint|
	self area hideTip.
	griddedPoint := self context gridPoint: aPoint.
	costumes := self context costumesAt: aPoint.
	costumes size = 1 
		ifTrue: 
			[self context last: griddedPoint.
			self context costume: costumes first.
			self context updateDirtyItemsList.
			self switchState: DrGSelectToolStateGrabbed.
			^true].
	"More than one math item under mouse"
	costumes size > 1 
		ifTrue: 
			[self context last: griddedPoint.
			self context chooseCostume: costumes.
			^true].
	"The user clicked in the background, clear the selection"
	self context reset.
	^false! !

!DrGSelectToolStateNeutral methodsFor: 'handling' stamp: 'HilaireFernandes 11/27/2011 11:38'!
handlePressShiftKey: aPoint 
	| costumes griddedPoint|
	self area hideTip.
	griddedPoint := self context gridPoint: aPoint.
	costumes := self context mutableCostumesAt: aPoint.
	costumes size = 1 
		ifTrue: 
			[self context last: griddedPoint.
			self context costume: costumes first.
			self context updateDirtyItemsList.
			self switchState: DrGMutatorToolStateGrabbed. 
			^true].
	"More than one math item under mouse"
	costumes size > 1 
		ifTrue: 
			[self context last: griddedPoint.
			(self context chooseCostume: costumes) 
				ifTrue: [self switchState: DrGMutatorToolStateSelected].
			^true].
	"The user clicked in the background, clear the selection"
	self context reset.
	^false! !

!DrGSelectToolStateNeutral methodsFor: 'handling' stamp: 'HilaireFernandes 11/13/2011 14:48'!
handleShiftKeyMouseAt: aPoint
	"Let's inform about the mutate mode"
	| costumes aTip |
	costumes := self context mutableCostumesAt: aPoint.
	costumes ifEmpty: [
		aTip := 'Pressing down the [Shift] key you can drag and change the nature of free point or intersection point.' translated].
	costumes size = 1 ifTrue: [
		aTip := 'Drag this point to change its nature.' translated].
	costumes size > 1 ifTrue: [
		aTip := 'Several objects can be selected. Please, select one clicking the mouse.' translated. ].
	self area tip: aTip at: aPoint.
	^ aTip notNil! !

DrGSelectToolState subclass: #DrGSelectToolStateSelected
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'DrGeoII-Core-Tool'!

DrGSelectToolStateSelected subclass: #DrGMutatorToolStateSelected
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'DrGeoII-Core-Tool'!

!DrGMutatorToolStateSelected methodsFor: 'handling' stamp: 'HilaireFernandes 11/13/2011 11:54'!
handlePress: aPoint 
	| costumes |
	self area hideTip.
	costumes := self context costumesAt: aPoint.
	(costumes includes: self context costume) 
		ifTrue: [
			self context updateDirtyItemsList.
			self switchState: DrGMutatorToolStateGrabbed.
			^true]
		ifFalse: [ 
			self context reset.
			^false]! !

!DrGSelectToolStateSelected methodsFor: 'handling' stamp: 'HilaireFernandes 11/13/2011 21:29'!
handleMouseAt: aPoint
	|costumes|
	costumes := self context costumesAt: aPoint.
	(costumes includes: self context costume)
		ifTrue:
			[self area 
				tip: self mathItem descriptiveName
				at: aPoint]
		ifFalse: [self area hideTip]! !

!DrGSelectToolStateSelected methodsFor: 'handling' stamp: 'HilaireFernandes 11/13/2011 11:59'!
handlePress: aPoint 
	| costumes |
	self area hideTip.
	costumes := self context costumesAt: aPoint.
	(costumes includes: self context costume) 
		ifTrue: [
			self context updateDirtyItemsList.
			self switchState: DrGSelectToolStateGrabbed.
			^true]
		ifFalse: [ 
			self context reset.
			^false]! !

DrGSelectToolState subclass: #DrGStyleToolState
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'DrGeoII-Core-Tool'!

DrGStyleToolState subclass: #DrGDeleteToolState
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'DrGeoII-Core-Tool'!

!DrGDeleteToolState methodsFor: 'handling' stamp: 'HilaireFernandes 11/27/2011 11:51'!
handleChoice: aCostumeCollection
	aCostumeCollection 
		ifNotEmpty: 
			[self context presenter 
				deleteMathItem: aCostumeCollection first;
				selectToolAfterBuilding].
	self context reset! !

DrGStyleToolState subclass: #DrGPropertyToolState
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'DrGeoII-Core-Tool'!

!DrGPropertyToolState methodsFor: 'handling' stamp: 'HilaireFernandes 10/27/2011 10:23'!
handleChoice: aCostumeCollection
	aCostumeCollection 
		ifNotEmpty: 
			[aCostumeCollection first editMyProperty].
	self context reset! !

!DrGStyleToolState methodsFor: 'handling' stamp: 'HilaireFernandes 1/24/2006 14:30'!
handleChoice: aCostumeCollection
	aCostumeCollection 
		ifNotEmpty: 
			[self context costume: aCostumeCollection first.
			self context costume style popupMenu].
	self context reset! !

!DrGStyleToolState methodsFor: 'handling' stamp: 'HilaireFernandes 8/18/2008 12:39'!
handlePress: aPoint
	|costumes|
	self area hideTip.
	costumes := self context costumesAt: aPoint.
	costumes size = 1 ifTrue:
		[self context last: aPoint.
		self handleChoice: costumes.
		^true].
	"More than one math item under mouse"
	costumes size > 1 ifTrue:
		[self context last: aPoint.
		self context chooseCostume: costumes.
		^true].
	"The user clicked in the background, clear the selection"
	self context reset.
	^false! !

DrGStyleToolState subclass: #DrGViewerToolState
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'DrGeoII-Core-Tool'!

!DrGViewerToolState methodsFor: 'handling' stamp: 'HilaireFernandes 4/2/2006 11:10'!
handleChoice: aCostumeCollection
	aCostumeCollection 
		ifNotEmpty: 
			[self context presenter 
				openViewerForCostume: aCostumeCollection first].
	self context reset! !

Object subclass: #DrGFactory
	instanceVariableNames: 'pool last'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'DrGeoII-Core-Factory'!

!DrGFactory commentStamp: 'HilaireFernandes 7/18/2008 10:22' prior: 0!
The Factory is where are manufactured new mathematic or macro-constrcution items.
It owns a pool of the already manufactured items. 
In last is the latested created (or recreated) item, 
Carefull!! last may not be equal to 'pool last' when the user try
to instantiate an already created object, in this case last return 
the original copy somewhere in the pool.

When manufacturing a new item it checks for its unicity.
It works with the help of the MathItem and macro Builder.
Related patterns: flyweight.!

!DrGFactory methodsFor: 'accessing' stamp: 'HilaireFernandes 10/17/2010 18:46'!
add: item 
	self last: item.
	pool addLast: item.
	^ item! !

!DrGFactory methodsFor: 'accessing' stamp: 'HilaireFernandes 8/22/2008 21:56'!
at: integer
	^ self pool at: integer ifAbsent: [nil]! !

!DrGFactory methodsFor: 'accessing' stamp: 'HilaireFernandes 7/28/2008 12:31'!
findInPool: item 
	"Try to find a twin of this mathItem, if so return the twin, otherwise nil"
	^ self at: (self indexOf: item)! !

!DrGFactory methodsFor: 'accessing' stamp: 'HilaireFernandes 8/22/2008 21:55'!
indexOf: anItem 
	^ anItem ifNil: [0] ifNotNil: [self pool indexOf: anItem]! !

!DrGFactory methodsFor: 'initialize-release' stamp: 'HilaireFernandes 7/18/2008 10:17'!
initialize
	super initialize.
	pool := OrderedCollection new.
! !

!DrGFactory methodsFor: 'testing' stamp: 'HilaireFernandes 11/14/2011 15:16'!
is: item1 before: item2
	^ (self indexOf: item1) < (self indexOf: item2)! !

!DrGFactory methodsFor: 'testing' stamp: 'HilaireFernandes 7/27/2008 11:06'!
isAnyInPool: itemList 
	^itemList anySatisfy: [:item | (self findInPool: item) isNil not].! !

!DrGFactory methodsFor: 'testing' stamp: 'HilaireFernandes 7/27/2008 11:07'!
isInPool: item
	^ (self findInPool: item) isNil not! !

!DrGFactory methodsFor: 'accessing' stamp: 'HilaireFernandes 7/18/2008 10:17'!
last
	^last! !

!DrGFactory methodsFor: 'accessing' stamp: 'HilaireFernandes 7/27/2008 10:59'!
last: item 
	"push item as the last instantiated item in the pool, purelly artificial however
	as the items order in the pool is not changed"
	last := item! !

!DrGFactory methodsFor: 'accessing' stamp: 'HilaireFernandes 7/18/2008 10:18'!
pool
	^pool ! !

!DrGFactory methodsFor: 'removing' stamp: 'HilaireFernandes 10/27/2010 10:57'!
purge
	pool do: [ :aMathItem | aMathItem release].
	pool := OrderedCollection new.! !

!DrGFactory methodsFor: 'updating' stamp: 'HilaireFernandes 7/27/2008 11:25'!
pushAsLastWhenInPool: anItem
"if this item has a twin in the pool, push as last this last one and
return true, otherwise return false"
	(self isInPool: anItem) ifTrue:
		[self last: (self findInPool: anItem).
		^true].
	^false! !

!DrGFactory methodsFor: 'initialize-release' stamp: 'HilaireFernandes 8/18/2008 12:42'!
release
	pool do: [ :item | item release].
	super release.! !

!DrGFactory methodsFor: 'removing' stamp: 'HilaireFernandes 7/18/2008 10:25'!
remove: itemCollection 
	pool := pool copyWithoutAll: itemCollection.
	itemCollection do: [ :aMathItem | aMathItem release ]! !

!DrGFactory methodsFor: 'removing' stamp: 'HilaireFernandes 8/13/2008 20:10'!
removeAt: index
	| item |
	item := self at: index.
	last = item ifTrue: [last := nil].
	pool removeAt: index! !

DrGFactory subclass: #DrGMacroFactory
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'DrGeoII-Core-Factory'!

DrGMacroFactory class
	instanceVariableNames: 'uniqueInstance'!

!DrGMacroFactory commentStamp: '<historical>' prior: 0!
DrGMacroFactory is a unique factory to hold macro construction definition.

Class Instance Variables:
	uniqueInstance	is the unique macro factory instance!

DrGMacroFactory class
	instanceVariableNames: 'uniqueInstance'!

!DrGMacroFactory class methodsFor: 'instance creation' stamp: 'HilaireFernandes 10/17/2010 11:23'!
new
	^ uniqueInstance ifNil: [uniqueInstance := super new]! !

!DrGMacroFactory class methodsFor: 'as yet unclassified' stamp: 'HilaireFernandes 1/8/2012 14:54'!
purge
	"self purge"
	self new purge! !

!DrGMacroFactory methodsFor: 'accessing' stamp: 'HilaireFernandes 10/17/2010 18:47'!
addUnique: aMacro
	^ (self findInPool: aMacro) 
		ifNil: [self add: aMacro].! !

!DrGMacroFactory methodsFor: 'accessing' stamp: 'HilaireFernandes 8/22/2008 21:55'!
titles
	^ self pool collect: [:m | m title]! !

DrGFactory subclass: #DrGMathItemFactory
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'DrGeoII-Core-Factory'!

!DrGMathItemFactory commentStamp: 'HilaireFernandes 7/18/2008 10:26' prior: 0!
See superclass comments.
This factory interface in extended to handle mathItem specific action!

!DrGMathItemFactory methodsFor: 'updating' stamp: 'HilaireFernandes 7/22/2011 15:13'!
allClean
	pool withIndexDo: [:item :index| 
			item dirty: false;
				id: index]! !

!DrGMathItemFactory methodsFor: 'accessing' stamp: 'HilaireFernandes 7/22/2011 09:35'!
dependentFromIndex: mathItemIndex to: endIndex
	"Return a collection of the dependents items of theMathItem, 
	Caution: it is not following the creation order and may have duplicate, further traitment is need for that"
	| theMathItem |
	theMathItem := self at: mathItemIndex.
	theMathItem dirty: true.
	mathItemIndex to: endIndex do: [:index |  |aMathItem |
		aMathItem := self at: index.
		(aMathItem parents notNil and: [aMathItem parents includes: theMathItem]) ifTrue: [
			self dependentFromIndex: index to: endIndex]].! !

!DrGMathItemFactory methodsFor: 'accessing' stamp: 'HilaireFernandes 7/18/2008 16:36'!
dependentMathItem: theMathItem 
	"Return a collection of the dependents items of theMathItem, 
	Caution: it is not following the creation order, further traitment is need for that"
	| dependents |
	dependents := Set with: theMathItem.
	pool do: [:aMathItem |
		(aMathItem parents notNil and: 
			[aMathItem parents includes: theMathItem]) ifTrue: 
				[dependents addAll: (self dependentMathItem: aMathItem)]].
	^ dependents! !

!DrGMathItemFactory methodsFor: 'accessing' stamp: 'HilaireFernandes 7/22/2011 15:40'!
dirtyItemsWhen: rootItem
| parents endIndex |
	self allClean.
	rootItem dirty: true.
	endIndex := pool size.
	parents := OrderedCollection new.
	self moveableParentsOf: rootItem in: parents.
	parents asSet do: [:parent |
		parent dirty: true.
		endIndex to: parent id by: -1 do: [: index |
			(self isParentsOf: index dependsOn: parent id) ifTrue: [(self at: index) dirty: true]]].
	endIndex to: rootItem id by: -1 do: [:index |
		(self isParentsOf:  index dependsOn: rootItem id) ifTrue: [(self at: index) dirty: true]].
	^ pool select: [:item | item isDirty]! !

!DrGMathItemFactory methodsFor: 'accessing' stamp: 'HilaireFernandes 1/14/2008 22:51'!
findInPoolById: anID
"Try to find an item given its unique ID, it used to load Dr. Geo 1.1 file"
	^ pool detect: [:each | each id = anID ]
	! !

!DrGMathItemFactory methodsFor: 'accessing' stamp: 'HilaireFernandes 7/22/2011 15:16'!
isParentsOf: itemIndex dependsOn: rootIndex
	"does my parents chain depends on rootIndex?"
	| theMathItem |
	theMathItem := self at: itemIndex.
	theMathItem isDirty ifTrue: [^true]. "one of my parents is already dirty, so I'm dirty!!"
	theMathItem parents isEmptyOrNil ifTrue: [^false].
	^ theMathItem parents anySatisfy: [:parent | 
		parent id = rootIndex 
			ifTrue: [
				theMathItem dirty: true.
				true]
			ifFalse: [self isParentsOf: parent id dependsOn: rootIndex]]! !

!DrGMathItemFactory methodsFor: 'accessing' stamp: 'HilaireFernandes 7/22/2011 10:24'!
moveableParentsOf: theMathItem in: parents
	"Return a collection of the parents items of theMathItem, sensidble to reverse dragging
	Caution: we may have duplicated item"
	| safeParents areParentsMoveable |
	safeParents := theMathItem parents.
	theMathItem parents: #().
	areParentsMoveable := theMathItem moveParent: 0@0.
	theMathItem parents: safeParents.
	areParentsMoveable ifTrue: [
		(parents addAll: safeParents) do: [:aParent |
			self moveableParentsOf: aParent in: parents]].
! !

!DrGMathItemFactory methodsFor: 'accessing' stamp: 'HilaireFernandes 7/18/2008 16:41'!
ordered: aMathItemCollection
"sort the the math item according to the pool order"
	^ pool select: [:item | aMathItemCollection includes: item]! !

!DrGMathItemFactory methodsFor: 'accessing' stamp: 'HilaireFernandes 7/18/2008 16:40'!
orderedDependentMathItem: aMathItem
	^self ordered: (self dependentMathItem: aMathItem )! !

!DrGMathItemFactory methodsFor: 'updating' stamp: 'HilaireFernandes 1/23/2008 10:55'!
updateAllMathItems
	"stupidly update all math item"
	pool
		do: [:aMathItem | aMathItem update]! !

!DrGMathItemFactory methodsFor: 'updating' stamp: 'HilaireFernandes 7/18/2008 10:27'!
updateAllMathItemsButLocus
	"stupidly update all math item but the loci ones"
	pool
		do: [:aMathItem | aMathItem isLocusItem not ifTrue: [ aMathItem update]]! !

!DrGMathItemFactory methodsFor: 'updating' stamp: 'HilaireFernandes 3/28/2011 21:55'!
updateAllMathItemsButLocusFrom: startIndex to: endIndex 
	"stupidly update math item in the given range"
	pool
		from: startIndex
		to: endIndex
		do: [:aMathItem | aMathItem isLocusItem not
				ifTrue: [aMathItem update]]! !

!DrGMathItemFactory methodsFor: 'updating' stamp: 'HilaireFernandes 7/20/2011 16:53'!
updateDirtyMathItems: aCollection
	"stupidly update all math item"
	aCollection
		do: [:aMathItem | aMathItem update]! !

Object subclass: #DrGIcons
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'DrGeoII-Core-UI'!

!DrGIcons commentStamp: '<historical>' prior: 0!
I just execute-print something like:
	(FileStream fileNamed: 'middle-.png') binary contents asByteArray 
in the class message to get the string!

!DrGIcons class methodsFor: 'accessing' stamp: 'HilaireFernandes 11/26/2005 18:58'!
angle
	^self formFromGraphic: #(137 80 78 71 13 10 26 10 0 0 0 13 73 72 68 82 0 0 0 48 0 0 0 48 16 6 0 0 0 7 146 37 196 0 0 0 6 98 75 71 68 255 255 255 255 255 255 9 88 247 220 0 0 0 9 112 72 89 115 0 0 0 72 0 0 0 72 0 70 201 107 62 0 0 9 93 73 68 65 84 120 218 237 90 107 76 147 215 27 255 189 212 118 109 41 72 105 144 75 239 165 88 196 41 136 34 157 12 47 212 133 44 129 13 5 151 177 57 62 240 97 178 108 81 251 69 179 132 100 155 95 220 178 236 195 98 178 65 150 200 134 44 91 140 19 198 36 217 133 218 45 113 153 98 22 116 72 128 181 180 20 74 116 114 81 161 148 123 159 255 135 238 245 69 253 51 160 92 138 218 223 151 183 231 125 207 229 247 60 207 57 207 121 158 115 10 132 16 66 8 33 44 17 136 130 205 224 41 21 120 181 240 8 154 192 193 82 192 226 199 13 11 14 241 199 29 172 226 25 38 216 76 22 72 120 174 239 179 61 3 237 119 165 229 88 117 152 47 225 213 46 216 106 231 183 104 194 171 85 192 229 231 181 66 123 192 66 87 66 176 13 178 114 62 126 153 6 152 175 2 231 18 112 165 55 187 199 110 115 93 168 96 43 213 110 181 142 243 40 214 4 107 224 249 41 96 185 103 226 19 59 227 87 59 136 36 18 137 68 34 9 52 252 13 33 64 204 150 65 7 79 241 79 73 38 60 183 171 201 203 203 203 203 203 123 184 126 8 139 196 194 51 233 162 162 162 162 162 162 144 43 90 36 22 175 56 255 138 8 25 98 129 88 122 69 101 100 100 100 100 100 16 137 197 98 177 88 28 50 196 44 88 126 197 68 69 69 69 69 69 17 25 141 70 163 209 24 50 196 191 120 124 239 1 30 115 4 79 1 219 182 109 219 182 109 27 145 86 171 213 106 181 79 157 33 86 155 192 139 231 179 226 41 120 73 73 73 73 73 9 81 103 167 221 222 209 1 184 221 110 183 219 13 244 247 15 12 220 185 3 120 189 94 239 232 40 87 95 44 22 139 69 34 64 38 147 201 100 50 64 161 80 40 148 74 64 175 215 235 245 122 160 186 186 186 186 186 122 229 143 18 50 51 51 51 51 51 137 60 30 143 199 227 1 90 91 91 91 91 91 23 206 99 217 136 151 149 149 149 149 149 17 89 173 86 171 213 10 116 116 116 116 116 116 0 10 69 66 130 76 6 100 101 109 221 186 118 45 144 158 190 113 163 64 0 40 149 10 69 124 60 240 218 107 239 188 99 181 2 191 253 118 246 236 158 61 192 237 219 253 253 30 15 208 209 225 116 2 192 181 107 109 109 30 15 112 229 74 115 243 205 155 64 79 79 111 239 221 187 64 114 178 193 160 215 3 187 119 239 217 99 50 1 21 21 21 21 149 149 79 209 41 170 217 108 54 155 205 68 42 149 74 165 84 114 241 115 110 110 78 142 193 192 149 29 142 198 198 164 36 34 151 235 226 69 165 242 209 39 91 207 233 180 88 18 19 185 247 221 221 23 47 202 229 254 167 66 65 212 213 101 177 168 213 92 253 252 252 189 123 19 18 184 178 74 165 82 41 20 68 102 179 217 124 244 232 242 185 46 185 92 46 151 203 137 10 11 11 11 11 11 131 224 34 77 38 147 105 207 30 78 240 157 59 141 70 181 154 168 174 174 162 34 37 197 175 72 173 150 200 229 178 88 20 10 174 30 91 254 251 239 31 127 212 106 137 174 93 171 175 79 73 225 190 55 52 84 85 237 216 65 244 211 79 95 125 149 154 74 116 233 210 183 223 174 95 79 116 253 122 125 253 76 131 118 118 254 242 139 86 203 25 232 194 133 202 74 157 142 40 59 219 104 212 104 184 122 38 147 201 148 147 179 244 10 218 191 127 255 254 253 251 137 116 58 157 78 167 91 120 255 1 47 153 228 100 131 33 41 137 168 189 189 163 195 102 227 222 119 117 89 44 26 141 255 183 207 7 76 79 251 124 60 30 48 53 53 53 69 4 12 13 121 189 12 3 100 100 20 22 58 157 192 215 95 127 242 137 86 11 120 189 99 99 12 3 140 142 142 143 19 1 68 68 236 130 102 24 128 97 24 198 231 3 14 31 62 113 194 233 4 170 170 62 252 80 163 1 100 50 169 116 205 26 64 167 83 40 248 124 32 50 82 36 26 26 2 134 135 39 39 149 74 96 227 198 220 220 203 151 57 126 27 54 36 39 107 181 64 91 91 123 187 211 185 212 46 99 225 174 104 193 4 212 106 255 210 246 120 134 135 7 6 128 207 63 127 239 61 153 12 48 26 55 111 6 0 149 42 39 199 237 6 90 90 26 26 82 82 0 151 171 183 119 98 2 24 24 184 115 103 114 18 184 113 195 110 159 152 0 174 95 111 107 27 27 3 156 78 183 123 124 28 184 117 171 191 127 98 2 24 25 241 122 167 167 185 241 36 18 177 152 207 7 226 226 214 173 147 72 128 196 68 141 70 42 5 82 82 18 19 137 0 181 58 62 158 8 136 136 8 15 7 0 157 78 173 142 137 1 118 237 122 245 213 166 38 160 179 179 177 209 96 0 254 252 243 198 141 145 17 224 237 183 79 156 184 125 27 136 136 136 136 88 187 22 112 56 92 174 190 190 197 27 34 50 50 50 50 50 146 104 104 104 104 104 104 104 25 12 192 206 248 254 254 190 190 158 30 160 166 230 163 143 164 82 96 195 6 157 142 199 243 219 158 97 128 225 225 209 81 161 16 216 180 41 47 207 102 3 14 29 42 46 142 139 3 44 150 63 254 184 119 15 176 219 187 186 70 71 1 141 70 165 138 142 6 182 108 73 75 75 73 1 12 134 245 235 147 147 1 135 195 225 176 219 129 236 236 93 187 94 120 1 168 171 171 171 171 175 7 164 210 232 232 184 56 160 185 185 185 249 175 191 128 206 78 135 195 233 4 146 146 180 218 136 8 32 39 199 104 12 15 7 42 43 191 249 230 214 45 192 98 57 115 102 203 22 64 46 95 183 110 98 2 16 10 215 172 25 30 6 218 219 29 142 201 73 224 141 55 142 31 239 235 3 98 99 99 99 227 226 128 27 55 218 218 220 238 192 13 193 230 7 74 165 82 169 84 2 181 181 181 181 181 181 115 247 55 231 113 52 235 59 89 87 211 223 63 56 56 54 230 159 129 97 97 156 226 239 221 27 25 17 137 128 150 22 155 205 231 243 71 57 145 145 156 66 132 66 145 40 58 26 120 247 221 99 199 14 29 2 210 211 183 110 205 206 6 106 107 235 235 47 93 98 152 246 118 187 189 191 31 56 123 246 252 249 95 127 101 24 139 197 106 189 122 21 176 88 172 214 43 87 24 102 106 106 122 154 97 88 197 51 76 118 118 118 118 86 22 32 18 133 135 199 199 115 227 100 101 165 167 71 68 0 54 91 87 215 221 187 64 103 103 79 15 235 10 249 124 63 111 30 15 232 235 27 28 156 154 98 21 15 228 230 238 221 251 220 115 129 239 17 26 141 70 163 209 0 189 189 189 189 189 189 129 246 50 3 108 84 227 47 17 189 255 254 225 195 26 13 183 217 177 239 91 90 46 92 216 176 129 232 179 207 62 248 64 171 37 74 74 210 104 132 66 238 123 73 201 235 175 191 248 34 81 77 205 153 51 167 79 19 21 20 20 20 20 20 112 130 46 85 185 180 180 180 180 180 148 27 87 175 87 171 69 34 162 143 63 62 126 92 173 246 111 226 51 55 121 54 154 58 121 210 108 142 138 226 222 155 205 102 243 145 35 129 27 98 201 50 100 149 74 165 82 169 136 210 210 158 125 86 169 228 194 62 151 203 98 145 203 137 108 182 159 127 246 123 97 78 96 161 144 72 36 18 10 121 60 162 99 199 204 230 226 98 162 154 154 154 154 47 191 124 84 241 139 197 108 134 40 47 47 47 47 47 39 18 139 69 34 62 159 51 4 203 179 181 181 161 193 96 120 48 172 149 203 137 114 114 118 236 136 137 33 98 247 184 64 121 73 165 82 169 84 186 8 57 203 202 202 202 14 29 122 244 170 142 37 202 26 160 169 233 220 57 189 158 200 191 228 185 122 199 142 29 61 122 224 128 127 198 87 85 45 189 226 23 106 8 150 23 203 179 190 254 139 47 210 210 136 156 206 198 198 153 97 234 195 79 54 145 92 40 31 129 64 32 16 8 22 33 175 63 76 35 202 203 51 153 20 10 162 206 206 198 198 245 235 231 186 188 38 58 120 240 192 1 163 113 229 20 63 151 33 246 237 219 183 239 229 151 31 117 77 15 63 217 137 229 143 150 136 76 166 221 187 211 211 137 12 6 131 129 203 55 230 15 134 97 24 134 89 180 220 68 231 206 125 250 105 108 172 63 19 85 169 56 159 217 212 84 91 187 101 11 81 82 146 86 43 145 16 109 220 104 48 196 196 16 85 87 87 85 157 58 181 242 138 159 203 16 108 57 53 53 53 53 53 149 115 73 13 13 167 79 103 100 60 152 105 43 20 68 167 78 157 60 201 181 95 126 3 220 143 130 222 122 171 180 52 55 151 72 167 83 42 5 2 96 251 246 77 155 248 124 192 231 35 226 241 128 241 241 169 169 200 72 224 242 229 182 54 141 6 176 217 156 78 143 7 40 46 126 229 149 220 92 224 252 249 239 191 183 88 252 97 99 93 29 195 4 203 16 15 143 207 150 243 243 243 243 243 243 1 187 221 229 26 29 5 6 7 7 7 111 221 2 124 62 159 47 44 140 83 247 75 47 109 223 126 245 42 144 144 144 144 16 27 203 29 30 206 119 124 62 159 207 231 243 3 32 190 115 231 243 207 111 222 60 251 82 101 207 102 222 124 179 184 120 230 153 205 227 246 76 75 75 78 22 8 30 221 132 231 215 126 54 4 222 254 126 162 144 152 168 209 200 100 68 71 142 28 60 8 0 5 5 57 57 66 225 191 149 152 255 223 205 195 239 103 171 55 95 60 156 182 16 249 87 32 159 207 229 27 60 30 195 76 78 114 101 134 241 31 121 204 197 107 122 218 223 79 88 24 48 49 241 95 28 24 134 97 128 138 138 31 126 72 77 5 106 106 190 251 174 165 5 232 238 238 238 238 238 158 45 177 10 252 191 176 247 255 154 248 207 63 125 125 94 47 32 145 8 133 207 60 3 56 157 61 61 99 99 129 41 114 169 177 84 60 102 222 51 204 5 189 94 38 251 253 119 96 96 96 96 96 114 114 174 218 115 77 189 217 51 226 251 6 24 25 241 122 199 198 128 145 145 241 241 152 24 192 225 184 121 51 44 12 16 137 252 105 213 192 192 221 187 97 97 0 155 145 62 233 240 122 199 198 124 62 255 5 145 203 53 223 86 179 249 132 255 104 241 96 113 181 93 249 61 233 152 97 160 229 58 34 120 82 202 43 134 96 11 186 218 203 33 197 63 105 138 15 33 132 16 66 8 30 254 7 195 86 167 231 229 37 138 50 0 0 0 0 73 69 78 68 174 66 96 130)! !

!DrGIcons class methodsFor: 'accessing' stamp: 'HilaireFernandes 11/26/2005 18:58'!
arc
	^self formFromGraphic:  #(137 80 78 71 13 10 26 10 0 0 0 13 73 72 68 82 0 0 0 48 0 0 0 48 16 6 0 0 0 7 146 37 196 0 0 0 6 98 75 71 68 255 255 255 255 255 255 9 88 247 220 0 0 0 9 112 72 89 115 0 0 0 72 0 0 0 72 0 70 201 107 62 0 0 7 221 73 68 65 84 120 218 237 90 203 79 147 89 20 63 95 203 163 60 74 65 10 5 132 2 37 136 18 27 98 16 17 137 129 21 134 168 33 237 2 67 38 153 132 133 75 137 18 163 9 175 70 94 49 6 212 68 131 11 23 147 204 31 160 49 51 106 52 136 102 76 88 32 105 124 67 38 12 146 182 137 82 138 188 218 210 150 199 157 197 153 147 27 202 212 66 31 130 245 251 109 126 185 95 239 215 246 158 115 238 185 231 113 1 68 136 16 33 66 132 8 17 34 68 136 248 89 193 216 70 254 121 32 217 217 159 255 249 4 238 141 168 240 126 189 47 1 11 194 78 47 124 183 32 204 130 216 174 133 11 66 119 119 119 119 119 55 99 211 211 211 211 211 211 0 209 209 81 81 81 81 0 215 175 223 184 113 227 70 228 41 46 204 59 96 235 232 237 237 237 237 237 101 204 100 50 153 76 38 128 153 153 153 153 153 25 0 133 34 41 41 41 105 167 255 93 248 16 182 51 224 249 243 11 23 46 92 216 250 252 193 193 193 193 193 65 128 145 145 145 145 145 17 0 139 197 98 177 88 0 60 158 149 149 149 21 128 174 174 174 174 174 174 200 59 51 66 174 128 135 15 209 133 28 63 126 251 246 237 219 91 127 111 104 104 104 104 104 8 96 97 97 97 97 97 1 96 121 121 121 121 121 25 96 126 126 126 126 126 158 143 69 248 129 205 166 209 104 52 222 97 229 246 185 186 186 186 186 186 154 177 166 166 166 166 166 166 200 179 124 66 200 118 192 219 183 58 157 78 199 152 82 57 57 57 57 233 127 126 81 81 81 81 81 209 230 231 101 101 101 101 101 101 0 114 185 92 46 151 3 200 100 50 153 76 182 211 98 10 31 66 166 0 173 246 254 253 251 247 253 207 43 47 47 47 47 47 7 136 142 142 142 142 142 222 252 60 33 33 33 33 33 1 96 109 109 109 109 109 13 96 118 118 118 118 118 22 128 162 163 157 22 88 168 17 180 2 222 191 63 117 234 212 41 255 130 169 172 172 172 172 172 4 112 187 221 110 183 27 64 42 149 74 165 82 128 138 138 138 138 138 10 0 133 66 161 80 40 0 146 146 48 234 161 207 237 118 187 221 110 7 88 90 90 90 90 90 218 105 113 133 30 65 135 161 5 5 47 95 190 124 201 199 70 35 242 111 191 197 199 199 199 3 124 248 112 244 232 209 163 0 115 115 115 115 115 115 220 178 227 226 226 226 226 226 184 224 85 42 149 74 165 226 59 192 225 112 56 28 14 238 130 200 37 69 26 2 222 1 207 158 93 186 116 233 18 99 50 25 70 45 132 215 175 145 191 126 213 104 52 26 128 213 213 213 213 213 85 0 137 68 34 145 72 184 64 73 240 153 153 153 153 153 153 0 249 249 249 249 249 249 0 3 3 3 3 3 3 130 144 157 157 157 157 157 13 144 155 155 155 155 155 11 208 214 214 214 214 214 230 43 17 251 113 93 83 192 59 64 165 250 248 241 227 71 62 254 242 5 249 159 127 144 237 246 140 140 140 12 128 152 24 28 199 198 198 198 198 198 114 11 79 75 75 75 75 75 3 200 201 201 201 201 201 1 232 232 232 232 232 232 224 2 238 233 233 233 233 233 137 188 204 215 27 1 43 32 53 245 211 167 79 159 248 120 106 10 217 102 67 139 246 120 240 144 149 203 209 226 211 211 211 211 211 211 57 99 98 21 249 2 246 135 128 93 144 92 254 229 11 89 61 0 128 197 130 188 178 162 84 42 149 220 229 36 38 38 38 38 38 2 100 101 101 101 101 101 137 130 247 70 192 10 144 201 22 23 23 23 249 216 102 67 94 94 198 195 50 42 10 139 104 164 0 131 193 96 48 24 68 193 123 35 96 23 36 149 98 141 134 143 145 61 30 140 126 40 206 39 159 31 24 182 123 184 110 117 254 238 49 132 128 119 192 218 26 10 152 150 156 150 134 28 23 135 241 58 237 128 224 50 89 18 148 63 14 116 254 206 35 96 5 184 92 152 48 209 146 114 115 145 21 10 12 75 5 65 16 4 129 43 66 196 255 35 96 5 44 46 98 226 68 208 104 144 51 50 176 116 192 24 99 140 241 196 43 50 17 124 47 59 96 5 204 206 98 226 68 72 76 68 214 106 177 145 34 149 98 9 33 50 203 200 161 75 252 2 86 192 231 207 251 247 239 223 191 249 57 61 83 169 176 42 74 37 133 31 11 190 202 228 161 71 208 135 146 195 33 151 203 229 140 197 199 227 225 235 114 225 243 63 255 196 195 247 143 63 234 235 235 235 1 10 11 11 11 11 11 1 218 219 219 219 219 219 119 223 97 184 17 129 247 178 173 86 171 213 106 229 81 96 127 127 127 127 127 191 239 245 6 93 13 253 251 239 99 199 142 29 227 99 42 49 87 86 186 92 46 23 64 81 209 232 232 232 40 239 116 125 123 193 63 94 77 167 174 174 174 174 174 142 177 209 81 92 231 212 212 212 212 212 20 47 62 250 67 208 241 201 161 67 79 158 60 121 34 8 40 58 46 192 244 116 100 157 14 107 70 191 255 142 59 96 35 118 171 192 201 98 253 255 63 179 217 108 54 155 249 89 71 85 94 74 64 253 33 100 13 153 87 175 106 107 107 107 1 36 255 125 35 133 158 197 197 200 87 175 62 120 240 224 1 192 197 139 231 206 157 59 231 123 97 205 205 205 205 205 205 187 85 49 155 97 52 26 141 70 35 143 250 168 108 78 229 117 127 8 153 2 142 28 121 252 248 241 99 65 248 252 121 239 222 189 123 125 207 235 235 187 117 235 214 173 205 207 245 122 189 94 175 231 77 248 157 197 246 13 96 124 124 124 124 124 156 91 62 149 219 253 33 228 183 34 94 188 104 104 104 104 216 254 123 30 143 199 227 241 0 196 196 196 196 80 9 251 123 224 209 163 43 87 174 92 217 250 25 116 228 8 246 57 188 81 90 90 90 90 90 202 19 79 90 143 63 132 92 1 191 252 210 215 215 215 39 8 119 239 234 116 58 221 214 223 171 175 223 104 65 225 198 235 215 122 189 94 207 88 109 173 193 96 48 248 159 95 91 123 240 224 193 131 0 11 11 27 123 217 90 173 86 171 213 114 223 79 13 40 138 134 176 175 225 91 177 97 187 152 53 51 131 183 27 182 138 95 127 157 152 152 152 0 104 105 185 115 231 206 29 128 191 254 58 121 242 228 201 208 157 5 70 227 153 51 103 206 48 230 112 236 217 179 103 15 99 37 37 247 238 221 187 231 255 189 211 167 75 74 74 74 0 44 22 44 173 172 175 175 175 175 175 3 20 23 23 23 23 23 115 87 147 156 156 156 156 156 204 203 240 91 237 101 239 178 187 161 190 223 52 153 14 28 56 112 0 96 122 26 23 62 55 87 80 80 80 0 112 226 196 181 107 215 174 9 194 211 167 151 47 95 190 204 24 53 138 50 50 198 198 198 198 0 178 178 222 189 123 247 206 247 239 140 141 33 15 14 34 191 121 147 154 154 154 10 96 181 238 219 183 111 31 128 217 140 209 141 211 233 116 58 157 220 197 164 164 164 164 164 164 240 142 30 141 41 26 162 34 36 125 222 218 218 218 218 218 186 57 31 8 147 2 252 11 254 252 121 108 220 220 188 105 179 81 47 1 0 192 237 70 166 242 54 193 123 188 93 56 157 200 212 185 163 203 3 164 28 139 5 21 59 63 159 151 151 151 199 51 120 202 95 200 242 233 214 134 90 173 86 171 213 0 5 5 104 8 212 104 194 222 53 99 241 241 88 150 111 105 105 105 105 105 241 157 136 125 119 5 84 85 85 85 85 85 1 172 172 96 63 97 120 120 120 120 120 24 224 236 89 180 188 195 135 113 193 106 53 250 210 236 108 124 79 169 68 166 139 186 177 177 200 235 235 200 148 129 83 147 200 106 69 54 153 144 39 38 144 205 102 180 204 175 95 177 217 111 183 163 32 221 238 141 190 157 92 9 9 158 174 201 40 149 104 56 121 121 168 168 96 27 77 223 77 1 53 53 53 53 53 53 60 69 39 223 73 241 51 221 134 166 123 67 106 53 250 80 165 18 21 146 144 128 28 19 131 174 64 34 193 121 36 160 213 85 20 144 219 141 205 127 151 11 227 113 167 19 125 51 250 126 128 165 37 100 250 93 239 219 26 222 97 36 89 50 29 178 222 151 7 130 69 216 206 128 198 198 198 198 198 70 94 27 33 11 162 219 16 116 237 132 158 219 108 232 138 72 160 4 242 169 228 131 105 231 16 211 124 239 247 8 36 96 98 50 0 239 142 29 9 156 124 249 183 175 193 132 14 97 107 149 80 84 64 32 11 162 123 64 219 45 202 161 229 49 70 190 217 229 194 90 19 237 24 234 59 144 34 72 224 164 96 186 22 67 150 78 130 239 236 236 236 236 236 220 237 197 65 17 34 68 136 16 17 137 248 23 96 111 227 207 148 31 36 213 0 0 0 0 73 69 78 68 174 66 96 130)
! !

!DrGIcons class methodsFor: 'accessing' stamp: 'HilaireFernandes 11/26/2005 18:58'!
axialSymmetry
	^self formFromGraphic: #(137 80 78 71 13 10 26 10 0 0 0 13 73 72 68 82 0 0 0 48 0 0 0 48 16 6 0 0 0 7 146 37 196 0 0 0 6 98 75 71 68 255 255 255 255 255 255 9 88 247 220 0 0 0 9 112 72 89 115 0 0 0 72 0 0 0 72 0 70 201 107 62 0 0 2 63 73 68 65 84 120 218 237 154 63 170 194 48 28 199 211 135 139 131 155 139 32 20 133 174 122 0 111 32 226 9 164 8 221 68 236 238 1 220 123 128 130 20 79 32 226 228 234 1 116 21 132 130 224 226 230 224 82 200 27 106 120 188 188 23 211 104 106 147 248 251 44 90 11 53 126 63 249 31 17 2 0 0 0 0 0 0 0 64 13 48 254 253 154 31 95 178 31 120 187 249 190 239 231 95 240 124 208 181 220 8 161 205 166 209 104 52 48 62 28 28 199 113 48 86 83 4 93 179 233 242 241 238 203 71 90 11 88 44 46 151 203 5 161 56 78 146 36 65 232 116 90 175 215 107 93 90 4 171 124 150 85 116 201 132 25 14 43 149 74 5 99 53 91 4 171 134 191 191 230 75 199 117 93 215 117 127 10 94 180 8 207 243 60 207 99 125 143 65 2 232 224 139 22 65 7 255 191 8 3 91 130 234 34 82 68 131 7 17 146 16 13 88 195 224 105 212 18 193 15 254 241 152 161 49 197 136 200 62 191 207 54 102 24 192 123 68 100 15 246 99 130 167 201 71 132 188 224 65 196 157 108 34 216 65 125 108 176 162 136 138 72 63 101 183 136 201 100 60 30 141 32 120 97 120 34 82 84 221 226 48 8 86 240 228 93 209 91 28 134 195 14 94 141 5 157 241 252 29 92 213 90 208 25 11 127 165 10 34 94 164 223 159 78 59 29 140 219 237 94 175 217 124 126 239 5 68 8 66 130 127 44 64 28 16 113 135 4 74 7 77 238 211 193 203 18 64 208 93 132 244 127 69 16 104 17 132 221 110 181 58 30 229 157 181 70 81 20 69 145 101 145 224 233 235 249 252 122 189 94 45 75 213 51 107 105 2 226 120 191 63 159 139 250 25 250 138 16 22 64 47 237 101 215 232 87 209 77 68 102 1 36 248 48 12 195 48 180 44 209 61 22 219 110 181 106 53 118 215 36 27 93 68 8 215 92 90 4 43 80 86 151 68 68 16 150 203 217 108 187 205 191 5 177 68 144 107 50 88 15 6 213 106 181 138 144 109 151 74 165 18 66 245 122 183 219 237 34 84 46 7 65 16 200 47 231 211 15 228 213 100 34 128 116 81 244 236 167 168 174 75 85 17 194 176 230 247 116 208 170 110 7 235 62 125 69 116 224 188 224 65 132 36 76 61 226 123 86 196 219 10 104 106 240 52 89 69 176 238 231 142 41 65 243 224 137 120 123 240 159 122 214 90 120 240 64 10 4 15 0 0 0 0 134 241 13 162 86 226 137 186 18 79 185 0 0 0 0 73 69 78 68 174 66 96 130)! !

!DrGIcons class methodsFor: 'accessing' stamp: 'HilaireFernandes 6/29/2011 08:22'!
blankFigure
	^self formFromGraphic: #(137 80 78 71 13 10 26 10 0 0 0 13 73 72 68 82 0 0 0 48 0 0 0 48 8 6 0 0 0 87 2 249 135 0 0 0 1 115 82 71 66 0 174 206 28 233 0 0 0 6 98 75 71 68 0 255 0 255 0 255 160 189 167 147 0 0 0 9 112 72 89 115 0 0 11 19 0 0 11 19 1 0 154 156 24 0 0 0 7 116 73 77 69 7 219 6 29 6 15 22 238 223 233 130 0 0 7 58 73 68 65 84 104 222 237 153 91 76 148 233 25 199 127 223 105 142 200 97 0 135 81 131 39 96 100 55 210 210 208 44 106 54 81 183 222 184 132 52 110 77 140 214 100 107 44 171 92 88 172 122 177 166 27 165 224 213 154 232 133 9 89 111 32 219 84 47 188 88 143 27 21 131 46 198 83 151 18 87 84 16 28 79 227 238 202 65 164 226 56 51 31 243 125 111 47 102 24 134 173 117 65 7 210 52 60 201 155 239 155 153 204 188 255 255 243 252 159 231 125 158 111 96 202 166 108 202 166 108 202 166 108 242 76 196 86 210 76 154 68 224 236 221 187 23 128 237 219 183 39 109 111 117 178 128 231 230 230 98 24 17 158 63 31 196 229 114 209 223 223 207 255 26 129 184 52 100 89 38 47 47 143 170 170 42 102 207 158 141 97 24 8 97 98 154 42 86 171 149 37 75 22 115 226 196 73 145 140 40 72 111 11 120 250 244 233 0 172 94 189 154 146 146 18 102 204 240 32 68 148 139 105 154 152 166 136 93 77 132 16 152 166 65 111 111 47 155 54 109 78 138 140 222 40 2 57 57 57 98 207 158 61 76 155 150 130 44 203 168 170 138 205 102 143 178 18 195 75 196 136 68 151 36 17 123 45 145 157 61 157 252 252 124 186 186 186 222 58 236 242 155 120 190 177 177 17 159 207 135 207 119 143 193 193 23 132 66 33 194 225 16 145 200 16 134 97 96 24 70 130 247 197 40 66 146 20 13 222 134 13 127 32 25 21 105 188 4 196 173 91 183 72 77 157 134 166 105 60 126 252 152 135 15 31 18 12 134 208 245 33 34 17 3 195 136 32 132 153 112 53 98 210 17 241 232 0 20 20 20 144 153 153 57 169 17 16 13 13 13 216 108 54 116 125 136 178 178 50 158 61 235 39 20 10 161 235 97 194 225 48 145 72 4 195 48 137 68 12 76 83 96 24 6 32 226 250 143 255 144 136 202 127 219 182 109 111 29 133 49 19 240 120 60 148 148 148 32 203 50 178 44 99 179 217 8 133 194 232 186 78 48 24 149 144 174 135 19 228 147 40 161 145 92 16 194 140 203 200 235 245 190 117 20 198 90 5 196 145 35 71 88 180 104 17 23 47 54 211 216 216 72 111 111 31 233 233 233 184 92 46 114 115 115 113 58 157 88 44 22 172 86 43 86 171 5 144 144 101 9 144 226 250 143 230 128 132 36 17 187 74 116 116 116 240 233 167 59 223 184 34 141 185 10 157 57 115 134 99 199 142 17 12 6 17 66 144 149 149 69 90 90 26 170 170 34 203 50 146 20 5 42 203 114 28 139 36 69 9 68 43 16 152 230 136 223 36 41 250 157 119 222 121 151 172 172 44 250 250 250 38 86 66 247 238 221 99 254 252 249 228 229 229 145 159 159 143 219 237 198 102 179 225 118 187 81 20 5 77 83 177 90 45 88 44 26 170 170 162 105 26 138 162 162 170 42 138 162 198 164 39 161 40 50 138 34 199 165 40 66 33 254 188 101 203 27 231 194 88 9 72 29 29 29 164 165 165 49 119 238 28 156 78 7 110 183 27 171 213 138 221 110 199 225 112 96 181 218 120 248 240 17 131 131 47 80 213 40 161 68 160 170 170 161 170 35 164 84 85 193 8 135 152 125 226 28 25 146 138 215 235 157 216 131 172 187 187 155 66 89 230 95 233 25 56 28 78 12 195 32 51 211 133 170 170 124 255 253 15 236 219 183 15 93 215 217 188 121 51 30 143 39 174 241 104 226 14 159 204 35 78 190 113 35 141 166 166 34 254 113 237 67 86 217 30 177 113 163 198 142 29 59 198 221 94 140 153 128 97 24 4 114 114 248 213 201 147 180 125 240 1 170 203 21 43 137 2 167 211 137 215 235 165 184 248 151 20 21 21 197 115 34 74 128 56 129 39 79 94 208 220 92 192 185 115 110 180 161 32 107 3 245 252 46 120 159 182 208 70 188 222 66 10 11 11 105 111 111 159 184 86 34 50 52 132 235 254 125 10 30 251 249 193 227 129 88 226 206 156 57 147 161 161 33 190 248 226 32 69 69 191 32 35 35 157 96 48 136 205 102 67 211 44 124 243 77 58 95 127 157 205 131 7 14 138 139 7 216 178 229 1 165 246 127 146 251 183 211 52 149 252 154 223 150 189 196 52 237 236 218 181 139 53 107 214 140 43 10 227 34 160 88 173 4 150 44 97 65 237 30 148 154 26 250 23 47 198 52 13 108 54 27 5 5 5 44 88 176 128 204 204 76 122 123 123 169 175 175 39 28 246 242 221 119 21 204 153 147 197 202 149 125 188 247 94 7 169 169 42 226 233 83 22 174 223 78 79 89 25 5 197 197 4 148 104 130 231 228 184 249 236 179 191 80 83 83 59 102 18 227 106 37 254 126 232 16 189 229 229 244 127 252 49 169 61 61 200 64 74 202 52 82 83 83 41 44 44 100 96 96 0 128 214 214 86 78 159 62 141 221 222 67 93 93 128 186 186 7 148 151 7 200 206 118 96 177 104 104 167 78 97 166 164 224 95 189 154 240 188 121 40 138 26 91 10 43 86 172 32 59 59 107 194 218 105 225 247 251 121 242 237 183 204 107 108 36 48 119 46 47 203 203 1 208 117 61 86 50 21 12 35 194 249 243 23 48 77 147 165 75 151 198 115 33 162 235 56 91 91 153 94 87 199 163 247 223 39 252 251 117 241 19 123 248 160 51 77 147 206 206 78 54 108 216 48 38 124 227 38 224 243 249 112 216 237 240 252 57 174 149 43 209 23 46 228 217 206 157 232 25 25 72 138 50 42 105 163 247 102 124 54 120 122 215 199 187 159 127 78 160 168 136 222 53 107 48 237 246 248 140 48 12 222 48 162 109 72 69 69 5 183 111 223 150 146 42 33 128 190 190 62 236 14 7 90 86 22 61 91 183 98 185 123 23 247 186 117 184 14 31 38 52 248 28 69 81 98 117 94 141 221 107 40 161 48 153 245 13 20 239 216 142 229 199 31 9 174 248 13 74 106 42 138 162 196 150 138 44 43 200 178 18 63 232 106 107 107 198 116 184 141 155 192 241 227 199 227 39 173 99 237 90 2 7 15 162 151 151 147 242 229 151 228 253 177 2 75 91 27 154 166 161 170 202 8 137 151 47 113 158 61 203 192 159 170 120 244 213 87 24 222 5 177 207 149 81 135 93 116 69 137 120 60 51 104 104 168 255 89 18 227 110 160 102 205 154 37 124 62 95 194 224 98 16 137 24 60 185 122 149 252 234 106 228 142 14 244 178 50 132 197 130 94 82 130 118 245 42 182 83 167 120 89 94 206 227 138 10 108 30 79 66 183 58 188 70 58 216 97 9 133 195 97 90 90 90 168 170 170 122 45 206 55 233 0 69 87 87 87 124 88 79 4 18 238 237 193 121 232 48 242 253 251 40 29 29 16 8 16 252 232 35 2 171 86 33 92 174 81 96 135 129 190 138 76 115 243 69 106 107 107 69 79 79 207 95 129 221 201 30 234 197 217 179 103 89 190 124 121 60 241 18 151 16 130 72 36 130 20 10 18 209 44 9 195 189 249 95 61 63 76 166 189 189 157 202 202 74 186 187 187 171 99 123 237 158 144 167 18 165 165 165 226 242 229 203 241 106 147 24 137 97 82 163 201 25 175 241 184 73 91 219 13 62 249 100 83 34 240 68 75 122 4 98 149 82 140 42 153 81 208 230 107 129 38 122 253 217 179 1 90 91 91 89 191 126 125 245 207 236 53 49 4 154 154 154 88 182 108 217 168 55 127 10 244 85 100 58 59 59 185 126 253 58 149 149 149 213 227 216 111 119 210 159 204 157 63 127 254 63 8 68 167 177 209 254 25 30 33 143 30 61 202 129 3 7 104 105 105 169 30 231 86 19 18 129 81 50 250 169 13 123 251 230 205 155 92 184 112 129 173 91 183 38 21 116 210 158 141 190 42 10 126 191 159 75 151 46 177 127 255 126 174 93 187 86 157 108 192 73 125 188 94 90 90 42 174 92 185 130 223 239 231 206 157 59 52 55 55 83 83 83 83 61 145 128 147 253 255 192 238 137 246 240 100 252 193 177 123 178 192 78 217 148 77 217 255 161 253 27 58 98 17 74 10 213 30 16 0 0 0 0 73 69 78 68 174 66 96 130)! !

!DrGIcons class methodsFor: 'accessing' stamp: 'HilaireFernandes 11/26/2005 18:58'!
centralSymmetry
	^self formFromGraphic: #(137 80 78 71 13 10 26 10 0 0 0 13 73 72 68 82 0 0 0 48 0 0 0 48 16 6 0 0 0 7 146 37 196 0 0 0 6 98 75 71 68 255 255 255 255 255 255 9 88 247 220 0 0 0 9 112 72 89 115 0 0 0 72 0 0 0 72 0 70 201 107 62 0 0 2 3 73 68 65 84 120 218 237 154 209 141 194 48 12 134 233 233 150 224 9 137 1 96 0 86 64 44 209 55 132 24 128 1 24 0 33 222 186 4 98 5 6 128 1 144 120 98 140 242 148 67 245 93 206 73 157 196 105 251 127 47 149 64 77 157 223 142 237 90 29 141 0 0 0 0 16 128 186 110 94 121 190 180 77 238 7 238 130 83 190 181 77 239 6 54 129 139 66 186 178 120 129 97 224 27 225 238 142 65 10 82 6 14 32 148 101 89 150 37 141 120 121 170 1 12 84 248 191 29 65 187 28 238 10 188 9 35 188 187 35 112 180 254 197 53 146 77 138 242 79 93 168 1 34 138 162 121 98 140 224 168 25 129 176 167 20 191 154 161 78 119 138 22 39 172 155 240 116 223 106 248 26 16 202 96 127 199 74 133 207 204 17 210 7 182 189 95 126 178 252 132 77 165 71 54 15 226 4 110 239 128 237 118 179 89 175 67 11 207 235 18 168 90 211 46 32 22 241 102 50 105 236 254 109 143 176 13 77 37 124 87 225 245 105 233 128 248 194 119 123 38 19 77 159 248 197 164 219 51 153 104 207 75 223 207 166 158 201 104 233 195 28 145 92 114 124 232 153 12 183 158 235 126 229 250 244 100 22 20 106 38 147 78 120 71 114 121 179 147 206 100 220 215 107 167 135 162 0 177 8 59 10 176 237 199 117 159 217 204 116 252 13 90 173 118 187 197 162 174 231 243 229 114 58 229 239 139 59 147 233 28 237 35 194 8 239 235 0 67 207 133 253 129 249 46 200 94 100 140 160 147 201 108 54 30 127 126 63 159 247 251 235 245 115 223 243 121 191 191 94 238 6 153 153 204 225 112 60 158 78 159 226 90 85 85 85 85 218 221 88 120 90 111 200 230 0 138 113 192 237 118 185 60 30 253 19 80 138 184 13 245 141 112 208 196 217 1 52 7 35 162 195 192 58 128 230 96 223 98 104 82 148 41 198 218 27 206 13 231 8 166 142 176 9 106 75 73 92 177 30 42 222 2 112 145 76 139 46 109 63 145 186 154 136 63 79 167 17 111 4 238 123 251 168 134 137 104 122 53 255 15 229 5 42 57 3 31 9 232 1 225 51 1 66 43 17 247 123 25 0 0 0 0 0 37 222 28 134 87 22 39 202 104 13 0 0 0 0 73 69 78 68 174 66 96 130)  
! !

!DrGIcons class methodsFor: 'accessing' stamp: 'HilaireFernandes 11/26/2005 18:58'!
circle
	^self formFromGraphic: #(137 80 78 71 13 10 26 10 0 0 0 13 73 72 68 82 0 0 0 48 0 0 0 48 16 6 0 0 0 7 146 37 196 0 0 0 6 98 75 71 68 255 255 255 255 255 255 9 88 247 220 0 0 0 9 112 72 89 115 0 0 0 72 0 0 0 72 0 70 201 107 62 0 0 11 6 73 68 65 84 120 218 237 90 89 76 84 247 23 62 195 62 3 195 54 80 96 216 226 64 21 172 136 49 77 80 145 64 92 74 77 120 16 77 109 40 47 188 25 18 86 31 36 225 161 66 99 2 24 21 100 107 66 82 19 107 31 140 90 104 19 105 155 136 177 46 149 132 88 75 193 34 9 160 172 14 21 134 25 134 1 134 97 185 125 56 249 114 203 181 83 96 24 168 252 255 243 189 124 222 59 227 112 239 57 191 223 89 190 243 35 114 192 1 7 28 112 192 1 7 28 112 192 129 77 135 236 191 126 0 107 184 119 239 236 217 179 103 5 33 40 168 171 171 171 139 72 165 122 245 234 213 43 34 165 114 116 116 116 148 200 195 195 104 52 26 137 156 157 231 231 231 231 137 22 23 93 93 93 93 137 204 102 111 111 111 111 34 163 49 40 40 40 136 72 167 219 182 109 219 54 162 209 209 216 216 216 88 162 163 71 47 94 188 120 81 246 206 190 247 166 227 219 111 175 92 185 114 69 16 158 63 79 75 75 75 19 132 217 89 31 31 31 31 65 16 4 34 162 213 243 210 210 234 190 55 61 173 84 42 149 130 240 219 111 169 169 169 169 130 112 243 230 229 203 151 47 243 167 255 87 232 232 72 79 79 79 95 217 96 90 45 115 107 43 243 173 91 204 95 126 201 220 208 192 220 212 196 252 236 25 179 193 192 188 176 176 156 173 253 157 182 182 99 199 142 29 219 124 71 108 218 86 108 110 62 127 254 252 121 65 72 72 184 122 245 234 85 34 149 234 229 203 151 47 197 207 77 38 230 158 30 230 246 118 230 190 62 230 241 241 144 144 144 16 162 249 249 128 128 128 0 162 217 89 165 82 169 36 178 88 20 10 133 130 72 46 159 154 154 154 34 242 241 153 156 156 156 36 10 14 214 233 116 58 162 184 184 177 177 177 49 162 152 24 254 157 200 72 102 87 215 229 207 231 228 196 172 213 134 134 134 134 18 61 124 152 153 153 153 73 148 145 113 225 194 133 11 27 23 178 54 220 1 247 239 23 22 22 22 10 66 82 82 109 109 109 173 24 179 129 174 46 230 159 127 102 238 238 246 244 244 244 36 210 235 53 26 141 134 200 100 98 195 91 44 46 46 46 46 68 50 153 76 38 147 17 185 184 240 181 171 43 199 126 92 227 115 65 224 213 237 236 108 50 153 76 68 65 65 236 240 253 251 153 19 19 205 102 179 153 232 189 247 254 217 17 139 139 252 123 141 141 236 136 79 63 189 118 237 218 53 251 59 98 195 28 0 195 167 164 84 86 86 86 138 247 167 166 152 31 63 102 126 244 136 121 96 96 215 174 93 187 136 38 39 35 35 35 35 255 110 64 103 103 103 103 34 119 119 119 119 119 119 34 133 130 87 188 167 39 59 10 44 151 203 229 114 185 248 253 197 197 197 197 197 69 162 217 217 217 217 217 89 162 233 233 233 233 233 105 145 119 236 120 250 244 233 83 162 244 116 78 242 97 97 252 28 10 197 242 247 48 26 153 155 154 50 50 50 50 136 102 103 19 19 19 19 137 114 114 114 114 114 114 214 239 16 187 59 0 161 38 53 181 180 180 180 84 92 241 35 35 252 249 79 63 49 183 181 169 84 42 21 145 193 16 23 23 23 71 100 50 177 97 177 130 97 104 31 31 31 31 31 31 34 95 95 95 95 95 95 241 26 134 119 115 115 115 115 115 179 110 144 218 218 218 218 218 90 65 176 88 44 22 139 69 116 192 228 36 135 42 23 151 158 158 158 30 162 228 228 7 15 30 60 32 218 191 223 96 48 24 136 188 188 248 255 47 46 50 247 245 177 99 155 155 179 178 178 178 136 220 221 227 227 227 227 137 242 242 242 242 242 242 108 119 132 221 29 48 62 174 209 104 52 130 128 24 143 21 127 243 38 243 227 199 225 225 225 225 68 19 19 187 119 239 222 77 180 176 192 43 29 161 4 6 14 12 12 12 12 12 36 10 8 224 152 175 84 114 204 183 215 202 131 99 166 166 56 119 232 245 92 222 126 252 113 99 99 99 35 81 98 226 208 208 208 16 145 179 51 127 127 102 134 185 165 133 159 231 215 95 79 159 62 125 154 40 36 132 67 164 173 207 101 55 7 160 170 137 139 107 106 226 138 132 241 227 143 204 223 125 199 43 126 116 116 223 190 125 251 68 195 99 165 171 84 252 121 80 16 215 239 184 206 207 207 207 207 207 223 248 186 189 186 186 186 186 186 90 16 12 6 118 68 86 86 67 67 67 3 81 120 248 248 248 248 184 184 19 116 58 230 111 190 217 179 103 207 30 162 169 169 227 199 143 31 39 42 41 41 41 41 41 89 251 115 58 173 247 193 81 199 75 13 143 228 138 24 143 80 35 53 60 12 30 25 201 177 31 43 106 179 12 15 32 148 4 6 242 14 189 119 239 212 169 83 167 222 254 158 74 197 28 31 223 222 222 222 78 100 52 246 245 245 245 17 85 84 84 84 84 84 172 189 140 93 247 11 162 129 250 224 131 59 119 238 220 17 203 201 175 191 102 254 229 23 78 174 6 67 68 68 68 132 24 106 130 131 131 131 131 131 69 195 23 23 23 23 23 23 191 123 29 234 195 135 71 142 28 57 34 8 73 73 45 45 45 45 226 78 24 24 96 254 234 171 152 152 152 24 34 139 37 45 45 45 141 40 34 130 223 115 181 185 97 221 59 32 42 234 209 35 172 114 34 177 142 71 57 137 170 6 201 21 49 30 43 223 207 207 207 207 207 239 191 54 179 117 252 241 199 201 147 39 79 18 25 141 44 113 32 39 160 106 210 104 88 34 209 233 184 239 48 26 89 34 89 45 108 118 0 180 26 15 15 174 38 0 52 80 168 227 81 78 34 228 32 185 34 198 103 103 103 103 103 103 175 119 229 111 92 7 139 231 235 236 76 78 78 78 22 239 195 17 161 161 115 115 115 115 68 110 110 189 189 189 189 68 122 189 94 175 215 19 213 212 212 212 212 212 172 252 92 54 59 0 34 25 48 58 202 140 206 213 100 226 16 131 186 28 43 31 85 205 102 199 248 245 98 112 240 208 161 67 135 222 190 143 157 32 151 191 126 253 250 181 88 222 206 204 204 204 160 114 250 55 216 236 0 168 147 64 127 63 51 36 3 139 133 99 61 26 40 212 241 40 39 183 26 62 251 172 176 176 176 80 38 27 28 100 85 21 59 32 32 128 217 211 147 251 7 105 195 183 18 108 118 0 100 97 96 120 152 25 90 141 181 134 202 94 117 252 127 133 63 255 220 185 115 231 78 241 218 219 155 217 213 149 87 252 252 60 55 158 112 0 250 13 107 191 103 179 3 160 199 3 227 227 204 16 201 160 205 160 99 133 35 182 58 244 250 168 168 168 40 241 218 221 157 217 201 137 115 1 28 96 54 179 214 132 107 107 112 177 245 65 164 162 26 182 36 212 73 148 155 112 0 66 209 218 176 214 228 186 218 239 219 190 3 33 14 2 60 135 32 90 90 90 90 90 90 18 121 110 142 29 2 77 202 26 108 222 1 152 64 225 149 3 3 153 33 11 99 7 120 120 120 120 120 120 216 26 122 240 253 149 216 214 239 175 29 210 208 107 54 51 47 44 112 177 1 7 192 240 184 182 187 3 48 250 195 43 65 103 135 30 47 149 141 255 87 32 45 62 16 134 231 230 150 239 112 24 30 101 184 53 216 236 0 204 92 1 141 134 25 131 16 252 225 149 182 224 86 67 112 240 139 23 47 94 136 215 111 222 48 155 205 203 171 59 39 39 39 39 39 39 113 33 90 131 205 14 192 176 27 128 124 139 9 20 6 33 208 227 87 170 6 222 117 220 186 197 207 175 86 119 118 118 118 138 247 7 7 153 103 102 184 204 134 225 209 255 224 218 26 108 118 128 86 203 26 136 20 184 135 9 20 202 49 206 1 248 214 214 115 132 84 114 65 147 213 219 203 60 61 237 239 239 239 47 26 28 69 7 28 97 13 54 71 231 143 62 186 116 233 210 37 153 12 167 12 20 10 78 190 200 5 24 253 125 242 201 242 142 121 99 176 241 125 197 142 29 119 239 222 189 43 94 163 241 28 26 226 34 99 106 138 29 224 225 177 188 248 64 53 104 13 235 22 227 60 61 217 240 48 129 92 206 140 153 235 86 71 123 251 137 19 39 78 8 130 66 49 49 49 49 33 222 231 211 23 68 19 19 203 71 168 152 208 161 252 94 169 250 91 183 3 172 65 173 254 231 251 101 101 101 101 101 101 111 223 175 170 170 170 170 170 18 4 240 230 152 215 58 126 248 161 180 180 180 84 16 226 227 121 66 6 32 1 35 15 152 76 44 63 35 244 72 103 213 43 97 211 11 68 157 142 69 171 242 242 242 242 242 114 65 64 195 210 215 199 131 13 108 89 56 162 160 160 160 160 160 96 243 165 11 28 159 1 176 250 91 90 152 135 135 89 146 48 155 209 241 115 200 129 228 178 218 206 223 78 59 96 245 6 74 77 189 125 251 246 109 162 145 17 54 120 127 127 127 127 127 63 209 200 200 200 200 200 8 145 193 192 162 22 134 232 155 141 161 161 189 123 247 238 21 4 127 255 129 1 12 93 136 136 248 96 24 81 71 7 107 93 147 147 92 5 98 229 123 121 121 121 121 121 137 162 99 110 110 110 110 110 238 38 12 100 214 138 163 71 181 90 173 150 40 47 239 198 141 27 55 136 148 202 238 238 238 110 209 224 136 161 43 37 47 123 1 161 70 167 139 140 140 140 20 132 176 176 103 207 16 223 137 136 158 60 97 190 127 159 171 25 131 129 39 124 136 249 8 53 144 217 177 3 86 11 59 133 160 181 199 236 247 223 231 228 45 8 173 173 173 173 68 215 175 71 71 71 71 19 117 116 112 121 107 155 118 180 122 32 185 198 199 159 59 119 238 220 219 159 195 240 223 127 207 60 50 146 144 144 144 64 52 51 195 161 5 11 5 131 37 12 154 214 122 76 101 195 39 81 159 127 126 224 192 129 3 68 95 124 241 228 9 94 138 136 136 207 106 138 64 173 172 215 243 138 122 254 60 37 37 37 133 104 104 232 240 225 195 135 137 50 51 207 156 57 115 102 237 185 0 13 20 234 120 148 147 210 170 6 255 70 168 193 138 135 225 141 70 14 45 168 235 177 226 49 211 182 245 84 196 134 37 225 186 186 186 186 186 58 162 158 30 62 248 68 196 14 184 126 157 147 149 78 199 101 42 78 25 64 174 240 243 227 198 45 41 169 185 185 185 153 136 136 249 224 193 216 216 216 88 65 128 30 15 89 24 234 36 68 50 104 53 144 12 212 234 191 55 128 111 3 85 13 146 43 98 60 66 13 86 60 12 143 25 182 90 173 86 171 213 162 35 108 197 6 237 0 153 172 190 190 190 190 190 94 16 6 6 56 153 117 117 113 67 166 213 114 14 56 120 144 147 111 90 26 159 187 129 172 129 17 159 180 123 252 183 110 114 53 64 231 138 6 10 113 30 229 36 170 26 36 87 105 93 47 53 60 14 21 172 247 100 220 58 119 128 245 63 140 83 2 99 99 172 13 45 44 44 44 44 44 136 73 171 173 109 251 246 237 219 137 126 255 157 87 112 116 52 151 167 31 126 200 106 106 68 4 127 31 14 193 42 195 4 10 249 1 82 47 154 62 168 147 16 201 160 213 64 50 64 231 138 6 10 117 252 220 28 39 125 188 17 170 26 233 129 49 172 248 119 246 108 104 101 101 101 101 101 165 32 32 244 12 15 15 15 15 15 139 19 34 233 17 68 172 52 56 10 125 65 68 4 139 121 1 1 236 16 79 79 102 55 55 30 253 97 2 5 217 23 122 60 100 97 168 147 16 201 160 213 64 50 192 223 69 25 9 233 0 51 107 105 114 45 42 42 42 42 42 178 127 63 98 247 28 128 17 28 84 80 200 209 120 65 188 80 88 88 88 88 88 152 24 91 113 95 58 192 48 153 248 119 222 188 89 62 115 5 75 39 81 82 192 192 96 185 156 23 128 116 98 135 5 129 80 227 237 205 243 142 245 134 152 77 119 0 98 38 26 18 0 199 199 113 244 16 91 25 231 110 164 114 53 12 106 237 120 57 118 148 116 244 135 255 39 149 133 81 214 98 33 72 37 3 116 174 171 109 160 236 133 13 251 67 144 18 208 96 97 197 21 22 242 241 14 91 127 23 142 66 78 1 75 39 80 24 132 192 17 152 204 129 183 250 233 12 7 28 112 192 1 59 224 47 137 64 226 59 103 178 13 148 0 0 0 0 73 69 78 68 174 66 96 130) ! !

!DrGIcons class methodsFor: 'form' stamp: 'HilaireFernandes 10/29/2010 10:57'!
colorForm: aColor
	^ self iconWithBackground: ((ColorForm extent: self menuExtent depth: 16) fillWithColor: aColor)! !

!DrGIcons class methodsFor: 'accessing' stamp: 'HilaireFernandes 11/26/2005 18:59'!
coordinates
	^self formFromGraphic:  #(137 80 78 71 13 10 26 10 0 0 0 13 73 72 68 82 0 0 0 48 0 0 0 48 16 6 0 0 0 7 146 37 196 0 0 0 6 98 75 71 68 255 255 255 255 255 255 9 88 247 220 0 0 0 9 112 72 89 115 0 0 0 72 0 0 0 72 0 70 201 107 62 0 0 2 14 73 68 65 84 120 218 237 154 193 21 194 32 12 134 169 207 17 220 194 33 28 192 147 87 174 234 115 130 78 209 21 60 115 245 228 0 14 225 22 238 80 79 60 159 244 97 8 80 146 216 124 151 190 182 208 134 252 16 82 168 49 138 162 40 82 25 71 106 11 164 211 229 85 139 57 190 203 124 222 114 65 58 44 181 199 171 16 141 208 16 84 202 138 218 128 165 179 198 21 215 30 207 12 21 164 20 13 65 196 168 0 196 44 116 14 8 219 65 151 54 35 71 128 55 84 106 158 207 175 3 77 4 216 239 251 126 183 227 103 104 26 222 238 240 200 151 232 8 240 66 200 22 196 195 119 233 100 98 0 228 240 211 105 24 94 47 106 179 167 28 14 198 60 159 232 230 119 214 90 107 237 56 58 231 156 115 211 243 230 13 9 123 254 111 65 184 142 140 88 40 250 28 189 163 125 141 240 156 12 92 200 97 96 240 79 187 4 11 129 107 40 23 210 29 31 30 133 56 158 107 150 145 234 232 105 121 33 142 135 26 78 13 236 120 239 232 219 205 152 237 86 124 232 9 27 78 13 236 120 127 238 5 136 221 23 38 4 95 67 99 142 21 62 2 184 206 1 31 32 199 66 35 64 24 124 13 135 70 0 181 125 149 224 215 144 203 229 124 62 30 211 71 128 112 228 53 132 155 0 11 221 15 224 195 194 246 3 248 161 91 146 196 8 21 0 10 133 165 161 178 221 234 47 82 128 212 252 191 118 185 176 124 44 4 66 247 75 159 239 175 215 19 98 230 57 32 102 48 214 81 212 142 79 109 23 158 70 33 40 52 184 246 36 30 62 31 235 152 210 250 100 96 99 101 110 108 205 189 159 122 125 46 187 96 26 127 7 104 250 90 153 90 217 66 173 45 80 40 132 64 107 65 216 228 161 156 153 230 0 108 79 175 53 249 65 73 2 244 30 168 126 234 28 150 46 144 208 239 0 170 80 134 125 47 44 196 58 183 98 89 121 46 180 178 59 62 114 34 2 148 230 231 165 121 251 191 220 135 253 41 52 4 229 115 189 246 253 102 211 234 109 112 71 46 84 112 174 15 171 250 196 126 56 187 223 135 225 241 96 243 123 122 232 208 250 107 31 202 55 127 31 130 160 95 45 153 254 253 45 110 77 68 81 242 120 3 185 219 205 252 208 44 95 233 0 0 0 0 73 69 78 68 174 66 96 130)
! !

!DrGIcons class methodsFor: 'accessing' stamp: 'HilaireFernandes 3/22/2006 10:37'!
createAndMove
	^self formFromGraphic: #(137 80 78 71 13 10 26 10 0 0 0 13 73 72 68 82 0 0 0 24 0 0 0 24 8 6 0 0 0 224 119 61 248 0 0 0 9 112 72 89 115 0 0 11 18 0 0 11 18 1 210 221 126 252 0 0 3 152 73 68 65 84 72 199 181 150 111 76 155 85 20 198 127 183 45 157 64 9 89 72 216 194 212 18 144 66 178 4 54 162 4 170 147 15 68 81 214 26 23 18 135 113 115 24 13 146 169 153 201 18 12 81 99 244 203 20 17 93 92 182 15 203 134 154 105 130 75 220 146 206 40 206 178 17 54 33 197 132 145 77 201 168 150 117 98 97 64 11 133 240 167 127 222 30 63 16 154 17 186 63 17 56 95 222 188 231 189 247 62 247 156 231 62 247 121 149 136 8 235 24 58 214 57 12 171 153 124 163 194 188 236 61 219 233 93 251 10 178 157 222 132 11 175 25 64 83 83 19 74 41 46 5 22 18 15 144 251 8 64 18 13 181 150 149 198 243 214 178 210 132 115 87 112 160 148 90 2 6 192 110 183 211 213 241 13 67 158 97 148 82 136 8 74 41 182 109 43 162 175 239 10 197 197 197 0 164 164 154 120 220 90 22 95 231 242 111 221 43 43 0 228 187 83 159 201 192 181 159 226 59 3 228 214 205 11 210 223 115 58 158 123 181 246 121 25 245 58 197 102 179 73 67 67 131 136 136 52 54 54 138 136 72 77 77 141 12 14 14 198 43 90 193 65 118 246 131 232 245 58 234 234 234 226 185 152 166 33 154 6 128 217 108 102 239 238 103 24 241 141 210 217 217 73 85 85 21 0 149 149 149 0 212 215 215 223 153 100 17 225 248 241 54 50 50 54 242 70 221 206 120 187 208 4 137 45 2 60 105 221 202 35 57 89 56 157 61 24 141 70 154 155 155 201 207 51 83 94 94 14 16 127 222 241 20 181 126 253 3 63 183 119 145 102 50 113 244 139 247 151 241 1 80 251 226 179 104 225 8 151 187 175 16 12 6 41 45 202 162 227 220 49 242 243 22 53 225 114 185 238 14 32 34 156 56 121 142 52 83 10 59 118 60 122 251 7 0 242 114 178 184 229 27 227 204 143 93 188 190 207 198 158 93 21 124 220 210 138 165 160 16 151 203 69 110 110 110 98 0 165 20 118 187 29 0 103 71 7 215 7 135 216 148 153 193 168 247 34 74 167 72 51 165 114 179 255 12 177 72 20 145 197 118 29 124 173 26 223 200 56 191 116 184 112 56 28 148 148 148 36 22 154 82 138 192 120 47 219 139 30 142 131 52 127 126 2 4 174 94 29 68 161 103 120 100 10 148 14 68 72 75 125 128 191 46 181 226 247 7 57 223 217 139 165 160 16 0 191 223 127 119 37 191 125 224 21 172 37 249 216 237 118 206 58 156 40 93 18 55 134 103 209 27 146 233 31 248 7 141 36 46 254 254 47 161 185 8 11 179 33 66 209 8 223 158 189 64 75 75 11 110 183 155 64 32 64 32 16 160 189 189 125 249 101 183 36 30 255 88 47 213 187 42 49 110 48 208 221 221 141 94 103 36 197 148 206 184 127 146 212 180 141 16 213 8 71 21 198 13 41 76 79 5 112 252 218 131 165 160 144 218 125 47 175 216 249 146 208 12 183 147 123 244 200 135 236 174 174 226 233 138 39 120 110 231 83 8 26 254 137 49 98 209 116 82 146 147 153 14 78 176 48 59 201 223 158 25 68 9 159 28 105 227 94 118 178 172 69 251 223 252 128 119 222 253 148 172 45 155 144 152 70 76 34 20 109 125 136 140 140 116 54 111 206 36 18 137 226 243 186 65 193 87 109 231 177 217 108 247 188 12 85 34 71 203 201 201 225 212 201 67 88 44 102 36 166 161 133 66 12 251 102 144 249 49 194 161 57 220 67 195 28 250 242 123 174 187 189 255 207 209 60 30 15 47 188 116 0 127 96 10 4 20 138 45 153 201 40 3 204 204 206 163 79 210 175 222 50 183 23 63 70 56 28 1 5 162 19 68 52 230 230 34 204 135 163 24 12 134 248 209 92 149 101 94 27 112 19 156 206 68 162 81 36 18 101 98 114 134 254 63 61 8 178 54 166 239 243 141 177 255 173 143 64 167 136 41 56 248 222 97 14 31 59 77 223 31 19 56 28 142 251 2 80 235 253 219 242 31 185 225 254 15 195 173 162 18 0 0 0 0 73 69 78 68 174 66 96 130)! !

!DrGIcons class methodsFor: 'accessing' stamp: 'HilaireFernandes 6/29/2011 08:03'!
createMultiple
	^ self formFromGraphic: #(137 80 78 71 13 10 26 10 0 0 0 13 73 72 68 82 0 0 0 48 0 0 0 48 8 6 0 0 0 87 2 249 135 0 0 0 1 115 82 71 66 0 174 206 28 233 0 0 0 6 98 75 71 68 0 255 0 255 0 255 160 189 167 147 0 0 0 9 112 72 89 115 0 0 11 19 0 0 11 19 1 0 154 156 24 0 0 0 7 116 73 77 69 7 219 6 29 6 2 11 56 119 251 22 0 0 1 11 73 68 65 84 104 222 237 151 205 13 131 48 12 133 177 197 0 28 122 232 28 172 210 57 89 165 179 176 129 123 105 37 14 129 36 142 159 67 42 127 18 226 128 228 23 255 17 189 105 10 130 32 8 130 123 33 223 167 8 70 11 40 98 87 49 163 5 42 226 145 38 0 23 86 91 156 70 39 149 140 92 21 143 209 2 6 73 209 225 109 178 3 85 2 25 168 224 251 81 163 58 129 102 129 198 113 202 22 136 26 22 151 12 58 32 21 5 82 47 49 106 124 36 83 208 162 238 50 90 64 185 15 169 238 138 166 66 169 223 168 100 222 154 248 214 177 125 4 46 22 23 162 33 29 146 128 221 150 230 2 207 199 130 61 60 92 32 8 130 160 187 191 69 56 50 87 127 171 97 6 29 152 188 90 204 14 213 135 218 79 118 222 135 22 255 0 79 160 139 253 100 167 202 195 58 64 160 185 87 249 219 158 29 240 182 159 240 17 50 247 183 94 55 174 151 253 116 93 220 191 73 98 28 194 126 118 186 7 32 44 239 215 233 183 125 221 238 159 192 89 18 251 186 141 209 129 84 18 191 195 15 199 213 56 13 203 7 27 12 132 199 176 214 170 20 0 0 0 0 73 69 78 68 174 66 96 130)! !

!DrGIcons class methodsFor: 'accessing' stamp: 'HilaireFernandes 11/26/2005 18:59'!
distance
		^self formFromGraphic:  #(137 80 78 71 13 10 26 10 0 0 0 13 73 72 68 82 0 0 0 48 0 0 0 48 16 6 0 0 0 7 146 37 196 0 0 0 6 98 75 71 68 255 255 255 255 255 255 9 88 247 220 0 0 0 9 112 72 89 115 0 0 0 72 0 0 0 72 0 70 201 107 62 0 0 1 212 73 68 65 84 120 218 237 153 59 170 2 49 24 133 227 221 128 133 165 157 149 224 62 132 108 69 16 23 32 86 174 64 132 244 46 65 16 2 22 130 59 176 180 16 5 221 132 221 88 92 66 32 26 50 143 252 127 226 120 190 102 8 153 201 227 156 147 76 116 132 0 32 62 69 145 122 4 64 8 1 35 178 1 70 100 2 140 96 194 8 237 94 221 122 64 140 207 0 183 30 68 166 106 226 91 99 68 217 137 134 146 73 213 63 245 243 201 169 187 212 115 19 142 207 136 63 174 142 104 48 66 117 58 113 219 53 237 209 27 241 165 6 80 9 111 219 223 239 215 235 201 196 237 47 62 145 39 16 18 198 55 145 178 66 114 11 111 25 143 167 83 165 170 142 183 53 80 111 5 70 120 123 56 48 229 251 253 112 232 247 237 53 246 120 50 223 130 248 19 175 148 82 74 217 196 239 118 231 243 124 110 235 255 141 112 199 215 58 210 37 254 115 255 118 5 28 143 155 77 175 87 20 171 213 98 49 24 52 95 17 153 173 0 254 196 155 242 112 56 26 109 183 118 5 152 251 77 249 118 123 60 158 79 33 78 167 203 165 219 21 98 54 91 46 175 87 115 223 215 191 19 242 217 227 221 107 40 241 82 74 41 101 253 241 39 118 46 221 169 198 36 222 71 40 241 70 120 173 181 214 250 189 156 82 213 82 194 80 183 207 149 248 47 91 1 72 124 34 144 120 31 1 231 154 38 22 137 143 54 209 106 245 72 124 89 42 58 233 75 180 59 64 36 158 24 95 226 233 62 180 180 37 241 46 13 157 197 30 159 9 72 124 93 152 191 7 132 159 71 226 163 80 53 81 191 147 120 23 98 231 195 95 200 144 120 22 222 79 77 191 154 120 23 246 255 130 144 248 36 32 241 62 200 223 1 72 124 18 144 248 178 68 255 29 128 196 103 1 18 207 206 103 97 32 60 59 33 35 32 60 19 62 35 32 60 51 62 35 124 245 16 158 136 144 208 16 158 9 8 159 9 16 30 0 0 146 243 2 254 23 48 217 74 149 99 100 0 0 0 0 73 69 78 68 174 66 96 130)! !

!DrGIcons class methodsFor: 'accessing' stamp: 'HilaireFernandes 2/16/2011 21:30'!
drgeo
	^self formFromGraphic: #(137 80 78 71 13 10 26 10 0 0 0 13 73 72 68 82 0 0 0 48 0 0 0 48 8 6 0 0 0 87 2 249 135 0 0 0 4 103 65 77 65 0 0 177 143 11 252 97 5 0 0 0 9 112 72 89 115 0 0 11 18 0 0 11 18 1 210 221 126 252 0 0 15 190 73 68 65 84 120 156 197 154 121 112 84 215 149 135 191 183 244 91 122 111 181 164 110 9 9 33 9 1 66 8 4 2 140 88 108 179 36 142 9 137 99 187 176 157 196 241 56 51 21 87 202 51 169 76 77 198 197 76 37 169 33 227 170 120 166 226 144 154 137 29 39 241 16 39 94 98 108 12 56 142 67 8 198 118 48 216 236 59 2 1 18 32 9 16 218 90 173 222 212 203 123 239 206 31 54 158 33 56 11 72 73 78 85 255 115 251 245 185 231 123 231 174 231 215 210 219 111 191 205 95 202 36 73 66 8 49 166 62 229 49 245 118 197 169 44 35 73 210 85 109 3 3 3 47 175 89 179 70 60 251 236 179 226 119 191 27 85 95 99 230 233 3 51 12 131 117 235 214 137 129 129 129 255 184 210 22 139 197 30 90 189 122 245 202 154 154 26 118 236 216 65 58 157 30 179 254 198 20 96 104 104 104 217 195 15 63 44 18 137 4 61 61 61 171 174 180 75 146 116 246 145 71 30 225 142 59 238 144 238 187 239 190 107 178 51 26 147 198 114 14 164 82 105 246 236 217 45 150 46 93 58 118 17 254 17 27 83 128 63 102 87 230 134 109 219 99 230 83 29 43 71 142 99 115 248 240 33 49 156 28 6 91 194 116 123 152 51 103 182 164 40 10 153 76 134 231 158 127 94 188 245 214 54 220 30 131 150 121 45 124 229 31 190 42 229 114 249 81 247 59 102 0 91 223 252 149 56 223 121 138 162 168 7 9 232 61 53 140 227 228 69 115 243 28 233 133 23 158 23 200 25 30 249 198 23 41 20 70 120 111 199 17 182 108 217 44 22 47 94 54 234 161 54 38 0 137 196 48 71 142 237 99 197 93 243 240 251 189 12 14 119 17 44 86 217 246 214 111 104 63 123 86 8 53 203 223 125 233 30 188 94 131 193 161 62 220 126 137 29 219 14 51 127 254 34 116 93 255 235 2 216 142 205 211 107 127 40 86 126 110 9 21 21 149 4 253 81 122 7 130 148 132 135 112 187 138 9 133 34 52 53 53 16 10 68 240 185 67 248 60 165 100 115 67 84 84 247 144 76 199 133 174 71 70 149 133 81 3 12 13 197 246 103 243 9 38 84 213 16 240 149 18 10 148 81 26 174 32 51 146 164 121 154 138 132 140 170 105 152 186 15 69 150 241 184 253 24 70 128 68 60 69 62 95 24 109 247 163 223 7 18 137 120 179 199 39 97 152 94 28 75 102 237 211 63 229 209 213 107 200 36 5 165 197 213 148 22 87 17 242 69 233 60 123 158 239 124 231 113 122 123 123 240 184 3 116 180 95 36 54 216 255 215 5 144 101 153 211 29 199 168 170 141 160 185 12 158 249 201 207 57 223 217 205 236 57 77 168 170 130 109 229 25 201 166 200 230 82 180 159 109 39 59 146 231 229 117 175 18 240 134 48 52 157 87 54 108 36 147 25 221 174 172 60 248 224 131 55 252 99 65 129 45 219 94 93 221 178 104 42 245 19 231 50 103 206 92 74 35 197 188 244 194 47 216 249 222 46 12 211 69 32 164 33 132 32 224 47 98 40 150 68 215 76 76 211 205 196 73 21 188 185 109 39 118 65 172 174 175 159 250 173 27 141 97 84 25 104 61 121 76 248 195 42 53 85 147 81 93 6 251 247 29 230 107 95 253 38 51 102 204 162 162 188 154 199 30 93 67 54 155 37 147 25 225 187 255 249 125 118 108 223 195 96 127 146 213 223 252 30 207 252 104 3 119 175 252 24 191 250 245 107 116 117 119 222 240 17 245 134 1 6 7 251 247 191 185 99 19 45 45 141 4 253 227 216 250 235 173 124 235 223 254 147 39 159 120 138 229 203 151 179 252 246 219 201 100 114 36 18 73 98 177 1 186 47 92 226 177 199 190 205 67 15 125 137 117 63 127 145 25 211 231 241 214 27 187 185 255 75 159 96 205 154 199 233 234 238 186 33 136 27 2 72 166 226 53 235 54 172 109 110 154 91 73 113 113 5 169 68 150 239 125 247 199 124 237 159 30 161 182 182 150 72 36 66 69 197 56 34 145 82 250 251 7 104 107 59 69 73 113 9 146 36 225 114 185 48 12 131 7 30 120 128 248 80 150 138 242 18 90 22 215 242 244 218 167 200 140 100 254 252 0 154 230 226 173 237 91 59 92 222 20 53 19 234 8 122 163 108 221 186 157 137 181 147 249 228 39 111 39 28 14 19 10 133 48 12 147 124 161 64 105 36 202 241 99 167 184 233 166 22 12 195 32 16 8 16 14 135 41 47 31 199 212 250 6 78 28 59 199 188 69 83 8 150 200 60 241 196 127 95 119 22 174 27 224 229 13 47 138 87 127 241 10 179 230 214 81 30 153 8 142 139 159 173 93 199 163 143 254 59 62 159 31 195 48 144 36 137 92 46 203 224 224 32 37 225 0 71 14 182 82 59 177 26 89 150 49 12 3 77 211 48 12 131 69 139 22 176 127 207 73 76 211 203 226 143 205 160 111 176 147 159 61 251 19 49 50 50 50 246 0 66 8 90 219 246 139 141 27 214 83 26 13 209 48 105 54 94 51 196 249 206 46 202 202 162 84 85 85 161 40 202 135 207 14 12 12 208 123 185 151 108 161 64 34 153 38 28 46 66 150 101 84 85 69 150 101 16 48 99 70 19 3 125 41 36 225 34 90 30 102 249 157 11 56 120 104 47 27 55 109 20 150 101 141 45 192 185 238 54 241 131 39 159 194 31 86 48 53 55 145 112 13 178 164 176 119 247 126 150 44 94 138 36 73 31 94 84 10 133 60 201 100 28 203 42 48 146 78 146 207 229 113 187 61 255 23 60 32 16 148 150 70 168 170 170 226 232 225 211 128 196 64 255 48 94 159 206 177 214 3 236 220 185 243 79 26 78 127 210 81 226 248 201 195 226 103 207 253 152 72 149 65 121 69 148 142 214 4 150 237 128 237 226 208 129 19 124 243 235 247 34 132 192 113 28 132 16 228 243 121 82 169 12 85 85 227 144 85 25 213 165 162 200 239 195 9 33 176 109 27 73 146 49 116 131 146 146 98 50 169 12 46 69 99 233 178 22 22 46 152 205 192 64 140 231 159 217 136 162 200 98 193 130 133 210 21 232 235 6 176 109 155 182 211 173 226 165 151 158 163 110 106 49 51 231 214 81 20 8 83 94 122 129 108 62 69 62 165 99 21 4 211 166 53 126 152 129 124 62 143 16 176 119 239 30 102 54 55 224 80 0 28 250 250 251 137 68 202 177 44 11 85 85 1 129 36 73 168 138 11 89 5 144 240 121 252 20 5 13 12 175 139 79 221 221 194 218 39 255 135 116 38 35 150 44 94 34 105 154 246 145 49 254 193 33 116 172 245 144 120 238 249 31 17 173 210 113 123 188 76 159 58 143 186 154 102 22 222 114 19 182 101 113 230 76 59 19 39 77 198 52 77 36 73 66 81 20 10 133 2 153 145 12 91 182 108 229 166 249 179 0 135 233 51 235 89 255 202 6 146 137 228 7 128 2 203 178 24 201 102 201 91 5 210 153 4 35 185 36 8 240 152 1 2 222 16 182 200 17 42 50 121 251 157 45 108 125 99 235 239 29 78 191 23 96 199 187 111 138 23 95 121 154 69 159 168 100 114 253 120 134 250 51 56 182 66 192 87 66 105 184 26 143 17 36 49 148 164 172 52 74 42 149 226 232 209 163 220 115 207 61 180 181 181 241 206 246 237 88 86 142 169 51 170 25 78 12 240 241 21 115 217 254 219 55 233 237 189 76 79 79 15 171 86 173 98 197 138 21 28 60 120 128 216 96 12 73 182 201 21 178 8 69 66 150 20 52 151 155 142 211 125 4 194 6 183 221 209 200 142 247 182 208 221 221 253 145 16 215 0 20 10 121 54 189 182 78 28 108 219 204 188 197 149 212 55 52 82 63 165 1 85 85 57 116 240 48 8 129 199 244 35 112 200 228 147 216 142 77 33 95 160 189 189 157 214 214 86 250 250 250 232 238 190 64 221 212 241 56 100 192 17 24 110 193 248 154 16 175 108 122 133 254 254 126 186 187 187 57 115 230 12 221 221 231 105 59 121 130 170 218 32 154 170 99 104 110 4 14 167 218 58 56 123 234 34 159 88 177 128 170 154 50 238 252 252 124 214 111 252 57 61 61 61 215 64 92 5 80 176 10 60 189 246 9 209 126 105 39 141 179 163 76 158 84 71 69 201 20 38 84 77 165 121 238 52 246 238 58 74 247 133 110 4 18 233 108 156 100 58 6 64 42 157 194 239 247 179 106 213 42 234 167 76 225 229 151 215 209 114 235 36 28 199 198 235 45 194 208 220 124 234 222 57 188 177 237 151 60 246 237 199 184 239 222 123 89 189 122 53 170 11 178 133 20 142 148 195 227 46 198 212 253 168 170 78 95 207 0 247 127 241 110 22 45 88 74 101 100 18 227 43 163 76 152 236 97 195 166 245 215 148 100 174 154 196 235 215 191 40 146 118 59 51 230 149 227 117 7 9 7 203 81 93 42 249 124 134 250 134 137 8 73 70 8 9 93 51 0 135 241 53 126 158 252 206 235 220 124 203 34 92 170 78 67 67 61 63 248 225 147 84 214 249 137 142 243 226 243 132 9 249 163 200 178 139 84 54 206 3 95 105 97 199 214 147 124 229 31 191 76 168 168 152 108 46 69 253 172 82 60 70 0 83 247 160 200 50 154 230 225 158 123 239 71 56 14 201 116 236 253 139 144 203 197 244 230 36 103 78 238 230 192 129 253 98 214 172 102 233 26 128 100 50 201 59 239 190 193 226 79 87 162 202 26 225 96 20 77 115 19 139 95 194 113 44 82 153 4 213 213 69 84 84 148 227 53 131 120 205 34 74 162 33 166 207 169 228 27 95 255 87 162 101 101 216 118 158 120 162 159 7 255 126 41 94 51 68 192 27 65 215 61 120 29 27 191 89 132 215 237 97 254 178 90 102 206 175 70 20 100 10 86 129 112 177 151 128 47 140 101 23 176 108 11 231 131 143 64 96 59 5 100 73 33 232 141 16 10 148 51 189 185 138 221 123 223 99 214 172 230 107 51 240 250 230 95 10 20 11 211 212 48 13 15 33 127 57 125 125 125 28 62 216 78 124 48 73 73 196 195 148 25 229 152 151 79 98 232 110 12 205 67 145 63 202 231 31 184 157 115 183 116 209 121 238 18 142 99 83 82 54 137 178 200 56 74 139 39 96 232 110 36 73 193 208 60 4 252 17 198 69 39 227 73 244 225 56 22 8 9 7 7 151 162 97 26 33 92 170 142 109 219 228 11 89 28 225 32 9 129 101 91 184 20 9 151 203 36 28 40 103 114 125 140 55 94 223 128 166 171 228 115 214 213 0 166 105 98 24 42 170 75 37 228 47 101 120 168 192 150 215 222 197 146 147 216 142 197 233 119 50 156 57 211 197 237 159 206 35 112 40 43 174 195 52 3 184 221 1 60 102 136 9 85 227 209 92 110 76 195 139 91 15 18 240 149 224 82 117 44 187 128 162 168 132 2 81 52 85 39 28 24 135 35 44 16 144 207 103 201 21 70 80 85 29 67 243 96 57 22 78 54 129 229 228 177 109 11 77 51 49 52 3 85 85 241 184 67 20 23 149 82 93 27 229 192 129 253 162 113 90 147 116 21 64 98 56 137 207 111 34 41 10 126 119 128 211 39 46 82 85 87 196 197 254 126 210 217 56 193 74 139 243 29 113 158 95 59 196 93 159 205 195 100 7 159 183 20 77 53 241 22 133 40 149 107 112 169 26 178 226 66 145 20 84 151 134 101 229 113 28 231 195 55 173 233 110 28 97 35 75 42 142 16 8 134 176 133 131 97 122 81 20 141 124 46 69 38 23 7 36 242 35 130 88 127 22 143 225 163 178 122 60 1 191 137 169 123 40 41 45 230 216 177 86 26 167 53 93 157 129 202 241 21 28 57 97 145 31 177 73 141 12 51 117 70 45 133 124 41 53 137 74 186 46 158 97 40 217 139 219 127 153 115 199 6 217 188 105 15 238 47 24 84 68 45 130 254 40 94 79 24 175 59 136 34 43 216 142 131 35 108 28 219 193 178 10 128 132 36 129 229 88 104 170 129 208 222 63 110 56 194 193 208 60 168 138 142 162 184 72 166 7 137 13 95 192 118 108 14 239 237 228 213 151 222 193 165 23 48 12 29 69 241 243 248 154 213 104 186 135 112 36 192 64 247 71 172 66 139 111 89 34 253 242 245 95 136 190 139 105 66 69 189 8 100 124 158 16 19 42 166 48 46 90 67 54 55 76 239 80 55 126 207 78 14 238 188 196 166 23 119 178 98 229 8 197 69 113 16 224 53 3 8 225 0 14 194 118 200 217 105 114 185 17 76 211 7 226 253 253 69 145 21 100 69 35 22 235 37 224 15 160 107 30 84 59 207 229 254 14 122 135 58 25 138 15 243 211 239 111 99 40 62 72 203 109 81 252 65 31 193 96 136 34 99 26 219 183 111 231 230 37 179 25 232 141 49 173 113 225 135 0 87 93 234 39 77 154 180 250 221 119 246 97 57 185 247 87 0 213 6 217 70 83 117 36 20 76 221 131 207 231 69 210 83 156 56 210 77 50 110 17 40 145 65 42 224 119 135 209 117 47 146 36 113 255 231 254 150 201 13 227 9 250 253 104 46 15 0 142 83 192 113 108 218 78 180 241 95 223 123 154 230 217 141 24 186 78 127 188 139 243 61 173 92 188 216 195 250 103 118 145 41 12 210 178 180 146 198 198 70 38 215 204 196 173 22 51 60 144 163 166 118 2 46 67 162 163 189 11 73 120 168 174 170 253 214 53 251 64 245 132 90 233 225 47 127 77 236 218 189 147 183 55 191 73 160 232 28 205 45 117 212 84 151 129 44 129 16 120 61 62 154 166 207 196 182 45 222 219 210 129 16 54 115 23 230 112 235 1 170 132 141 238 242 210 213 121 129 179 29 29 212 213 54 124 112 200 147 145 101 21 129 141 131 197 205 183 206 70 55 117 6 19 61 92 236 111 231 210 165 11 108 122 225 8 137 100 156 150 101 101 204 152 57 139 178 226 10 76 181 136 189 191 221 69 105 89 41 101 21 197 244 246 118 147 24 78 65 110 240 195 152 255 64 121 93 208 218 122 92 188 190 249 53 186 187 187 144 100 9 69 181 8 150 232 148 149 149 81 18 213 233 186 124 154 158 115 89 132 80 88 182 98 26 149 101 53 76 155 188 0 43 11 166 199 141 215 27 196 165 104 200 146 140 237 88 228 173 44 249 124 150 108 46 203 112 242 34 221 151 219 56 222 122 138 151 215 238 34 84 38 104 106 25 71 99 125 19 101 145 74 188 238 32 154 230 166 80 200 35 1 233 76 130 195 135 218 249 205 171 135 120 242 251 79 73 178 236 186 122 14 92 107 18 13 13 141 210 212 134 6 226 241 248 67 185 92 238 71 217 124 146 67 39 222 230 252 217 78 142 236 141 209 219 155 98 194 132 9 156 57 221 201 209 125 23 144 231 202 168 170 70 237 248 70 28 241 254 14 238 168 54 32 0 137 92 110 132 212 72 140 193 225 75 244 15 118 209 215 219 71 219 225 94 170 167 20 19 12 107 4 180 137 36 7 85 130 94 153 34 159 31 73 40 8 199 102 56 49 196 169 182 243 28 220 125 142 47 124 254 111 184 18 252 31 201 192 71 91 193 202 243 238 190 215 196 238 67 111 97 137 97 122 58 211 104 138 73 34 230 112 243 109 117 212 77 29 71 36 92 137 223 19 38 224 45 65 81 85 28 225 224 56 130 145 108 156 120 178 143 193 120 15 185 92 158 3 187 206 81 200 73 248 3 38 201 68 22 85 242 113 238 204 101 116 93 67 150 32 149 30 193 178 44 92 46 149 121 243 230 179 248 214 37 68 35 229 210 255 87 58 175 187 184 235 82 53 230 54 45 151 78 159 232 17 182 217 73 180 60 71 174 144 196 74 123 216 179 253 44 151 47 197 185 121 153 77 38 59 204 96 226 50 178 4 249 194 8 185 92 150 244 72 146 244 72 134 108 82 176 99 107 7 77 179 39 162 7 227 12 14 159 70 24 26 29 199 99 124 124 201 29 44 95 190 252 247 86 172 127 87 166 189 161 210 162 203 229 162 166 122 162 185 103 207 161 133 53 181 81 130 193 0 197 37 126 130 69 62 142 236 189 72 58 157 39 24 214 201 230 18 196 147 49 6 99 3 244 246 14 114 169 107 136 211 199 6 216 183 163 155 207 172 188 149 105 77 85 120 124 110 116 197 71 251 113 155 7 62 251 101 22 204 95 112 93 229 246 81 105 100 27 54 190 36 50 162 131 241 117 6 2 5 89 150 200 166 37 54 189 176 15 69 145 168 158 88 130 166 43 196 135 210 12 197 210 164 147 57 202 43 162 220 185 114 41 69 37 6 3 177 139 216 78 158 205 27 143 51 109 226 66 238 252 204 221 18 92 159 92 48 42 128 129 129 190 216 19 63 124 60 180 228 211 53 184 61 26 18 50 110 211 139 199 12 112 186 237 2 71 15 119 0 18 37 197 65 42 171 34 140 175 42 197 244 184 209 84 157 120 178 151 161 225 62 142 29 184 128 87 169 227 174 59 87 74 138 114 253 114 197 168 4 142 226 226 210 162 241 21 53 98 224 114 134 154 41 30 116 197 196 165 234 232 46 55 51 154 38 211 208 56 17 69 86 81 85 23 134 230 121 191 106 129 192 177 115 8 97 227 20 100 186 59 82 252 203 63 223 117 67 193 143 26 0 224 182 219 150 211 51 116 136 210 34 15 170 75 67 150 21 188 70 8 73 82 80 21 23 146 228 66 150 174 20 188 28 28 225 96 21 242 120 204 52 105 93 48 119 246 34 52 237 198 117 178 191 168 78 124 181 93 89 77 70 39 84 142 153 204 122 253 54 54 98 254 159 229 223 42 127 73 251 95 108 18 182 185 23 127 126 182 0 0 0 0 73 69 78 68 174 66 96 130)! !

!DrGIcons class methodsFor: 'accessing' stamp: 'HilaireFernandes 3/22/2006 10:35'!
editGroup
	^self formFromGraphic: #(137 80 78 71 13 10 26 10 0 0 0 13 73 72 68 82 0 0 0 24 0 0 0 24 8 2 0 0 0 111 21 170 175 0 0 0 9 112 72 89 115 0 0 11 19 0 0 11 19 1 0 154 156 24 0 0 1 41 73 68 65 84 56 203 173 149 49 110 132 48 16 69 191 131 197 5 182 163 247 17 246 12 72 78 144 184 4 72 73 15 21 169 178 29 37 82 34 238 177 65 162 65 162 206 17 124 3 76 68 109 26 82 16 33 194 238 26 47 97 42 100 164 55 223 255 15 3 25 134 1 123 20 5 240 248 252 70 8 217 140 24 134 225 243 227 149 2 32 132 156 223 147 205 160 167 151 19 128 7 236 84 116 122 10 195 176 105 154 190 239 167 19 203 178 28 199 201 243 252 62 144 148 50 77 83 198 216 116 34 132 136 162 232 110 69 74 169 57 5 0 99 108 18 24 4 129 148 82 163 151 26 54 108 219 86 175 215 212 108 189 222 61 83 187 9 170 170 202 247 125 0 156 115 206 57 0 207 243 202 178 92 55 123 81 89 150 93 53 197 117 221 21 69 182 109 11 33 204 77 185 162 104 252 62 186 174 139 227 88 41 245 223 201 174 235 122 254 98 244 69 83 66 8 74 233 31 208 33 57 126 159 190 12 59 79 13 40 165 243 157 241 139 52 103 21 69 177 98 246 33 57 238 54 71 115 214 34 196 75 83 116 115 52 191 221 101 136 11 83 110 130 22 30 45 66 52 189 154 121 106 58 69 35 101 92 189 219 150 63 0 178 215 239 232 7 171 231 143 35 141 123 83 50 0 0 0 0 73 69 78 68 174 66 96 130)! !

!DrGIcons class methodsFor: 'accessing' stamp: 'HilaireFernandes 11/26/2005 18:59'!
eraser
	^self formFromGraphic:  #(137 80 78 71 13 10 26 10 0 0 0 13 73 72 68 82 0 0 0 48 0 0 0 48 16 6 0 0 0 7 146 37 196 0 0 0 6 98 75 71 68 255 255 255 255 255 255 9 88 247 220 0 0 0 9 112 72 89 115 0 0 0 72 0 0 0 72 0 70 201 107 62 0 0 9 239 73 68 65 84 120 218 237 90 91 76 92 85 27 93 231 54 55 152 2 214 182 64 165 23 27 72 109 129 137 144 106 133 146 42 84 45 137 6 155 246 129 55 83 245 193 7 180 49 26 77 154 152 244 197 24 19 125 52 49 233 19 154 152 38 53 40 15 42 38 26 172 166 25 40 49 150 166 173 45 246 70 129 82 58 148 12 12 157 203 185 109 31 190 255 251 247 204 80 34 197 78 161 237 172 4 14 251 156 189 55 103 214 218 123 125 223 222 123 128 60 242 200 35 143 60 242 200 35 143 60 242 200 35 143 60 242 120 152 160 44 245 11 204 135 221 187 119 239 126 249 101 33 250 250 250 250 126 255 125 169 223 6 136 70 163 209 104 84 185 235 124 45 59 1 246 236 217 179 103 223 62 33 82 41 211 92 181 10 168 174 174 174 110 110 6 132 160 31 134 101 89 150 227 0 66 184 174 101 1 182 237 186 174 11 56 142 109 11 1 40 138 108 67 127 187 174 227 200 246 174 75 189 101 223 87 20 69 81 20 192 113 132 72 165 128 174 174 163 71 63 252 48 119 2 168 75 77 56 163 181 181 181 181 181 85 136 222 222 222 222 159 127 6 170 171 107 106 154 155 1 215 101 98 29 199 113 0 203 178 109 38 222 113 232 190 16 146 120 215 117 28 219 166 50 61 183 109 203 146 253 240 53 155 120 134 16 66 8 1 12 14 254 249 231 79 63 229 254 115 47 185 0 7 14 28 56 112 224 128 16 225 112 56 28 14 3 29 29 111 191 221 217 121 59 98 120 36 19 65 92 6 84 85 85 1 85 165 146 162 168 170 166 205 157 49 11 197 216 216 216 216 153 51 192 35 143 148 148 148 151 231 254 243 47 185 0 157 157 157 157 157 157 192 59 239 188 247 222 215 95 203 251 76 164 162 48 181 124 63 243 135 159 26 134 97 80 125 178 16 85 101 97 52 77 215 101 125 22 48 27 60 51 38 39 35 145 75 151 128 141 27 55 109 218 182 237 1 22 160 184 184 184 184 184 88 136 215 94 123 253 245 79 62 1 52 77 81 104 84 51 129 153 126 75 116 166 11 65 53 52 77 211 52 77 122 186 166 209 125 238 135 173 198 117 133 112 221 249 223 103 116 116 100 164 175 15 168 168 168 168 168 173 5 10 11 11 10 124 190 220 243 160 223 107 226 219 219 219 219 219 219 133 16 66 81 54 108 0 86 174 124 244 209 245 235 165 151 43 10 224 186 146 64 121 117 93 46 3 233 22 67 191 229 172 96 233 92 151 37 35 226 89 80 110 37 132 162 0 211 211 211 211 99 99 192 173 91 241 120 44 6 212 214 214 214 110 221 10 152 166 101 217 118 238 249 184 103 51 160 163 163 163 163 163 67 136 158 158 158 158 158 30 250 160 207 62 75 65 147 9 33 106 136 182 116 139 73 31 251 60 19 88 24 38 54 187 44 99 2 205 16 69 81 20 77 227 62 164 48 103 207 158 57 211 211 3 212 215 215 215 191 248 34 160 105 186 174 40 148 101 89 214 3 36 192 241 227 199 143 159 58 5 116 116 188 245 214 225 195 233 132 18 45 76 180 28 201 154 150 62 162 165 167 83 59 69 33 193 164 101 105 26 181 167 145 205 245 185 63 182 42 77 211 117 77 3 70 70 174 94 253 227 15 96 211 166 202 202 167 159 6 130 193 96 176 168 8 72 36 18 9 203 2 226 241 120 124 102 230 1 16 32 20 10 133 154 155 133 216 177 163 169 169 189 29 240 251 3 129 96 48 189 70 118 72 36 33 152 64 77 35 33 184 156 253 92 8 114 255 249 130 43 11 195 87 33 92 55 149 2 70 71 71 71 79 159 6 182 108 121 226 137 237 219 1 211 52 77 74 79 169 159 104 52 26 189 113 227 62 22 96 255 254 253 251 247 239 23 162 170 170 170 170 174 14 40 45 45 43 123 252 241 219 213 204 14 182 50 185 164 17 76 115 68 215 117 157 5 97 43 161 214 50 216 210 58 128 130 45 7 101 238 207 227 209 117 93 7 6 7 7 7 127 252 17 216 182 237 169 167 218 218 50 179 38 0 176 109 219 118 93 224 216 177 95 127 253 226 11 160 178 178 178 178 178 242 62 20 224 220 185 243 231 35 17 160 166 38 20 106 105 89 192 139 252 143 0 182 34 190 74 194 217 162 50 45 137 5 20 130 131 181 140 10 220 175 174 3 215 174 141 141 157 61 11 20 20 20 20 20 21 1 171 87 175 94 189 110 29 165 159 0 144 74 165 82 150 37 189 127 235 214 234 234 93 187 128 129 129 129 129 129 129 187 191 2 206 153 0 85 85 85 85 79 62 41 68 75 75 75 203 171 175 74 11 89 240 11 169 183 15 194 210 146 200 195 179 211 80 77 203 204 154 120 228 235 58 89 207 185 115 231 206 253 242 11 80 87 87 87 183 107 151 252 127 108 57 241 120 60 110 154 192 55 223 28 61 250 193 7 192 198 141 27 54 20 20 0 109 109 109 109 109 109 139 89 210 45 12 119 45 13 13 133 66 161 29 59 132 104 104 104 108 220 183 239 118 94 255 111 200 76 19 21 133 214 5 236 236 186 78 22 98 219 182 109 219 153 158 158 30 140 53 205 48 12 67 182 159 152 24 31 63 127 30 88 183 110 253 250 80 136 250 241 120 0 219 182 44 215 5 18 137 100 50 149 154 155 245 196 98 177 88 52 10 116 119 119 119 119 119 47 227 25 208 212 212 212 244 202 43 66 212 214 134 66 205 205 192 99 143 85 84 108 222 44 159 115 126 47 61 250 118 59 48 115 133 72 183 154 244 167 60 163 116 157 102 134 174 27 70 250 125 190 222 184 49 49 113 225 2 112 226 196 137 19 93 93 64 77 77 77 77 99 163 220 196 75 165 76 211 178 164 245 124 251 109 87 215 193 131 192 206 157 59 119 238 220 41 137 207 245 12 88 180 178 188 146 205 213 139 253 87 172 89 179 102 77 105 41 176 121 243 230 205 47 188 64 155 123 13 13 68 184 227 208 194 43 149 2 174 95 31 31 31 26 2 188 94 143 103 108 12 56 114 228 200 145 175 190 146 196 231 122 6 252 103 11 234 237 125 243 205 131 7 129 217 217 120 60 30 151 158 204 150 192 150 33 199 62 239 241 240 182 175 101 153 102 122 90 73 121 188 162 144 197 112 62 147 93 214 52 42 235 58 197 4 85 165 25 243 238 187 199 143 255 240 3 112 242 228 153 51 167 79 3 207 61 215 210 82 83 35 131 173 244 252 217 217 217 89 224 242 229 75 151 194 97 224 228 201 147 39 143 29 155 75 252 161 67 135 14 29 58 36 4 93 239 190 16 119 44 0 143 252 149 43 11 11 131 65 242 202 217 89 73 168 105 90 86 50 9 88 150 227 88 150 36 84 85 105 133 169 235 114 43 64 8 18 136 77 201 178 136 88 195 0 52 77 8 190 79 2 8 145 105 70 170 74 94 79 105 231 244 116 42 101 154 192 216 216 196 196 245 235 178 150 207 231 245 122 60 210 202 226 241 68 194 52 129 161 161 191 255 254 237 183 249 137 231 114 174 136 95 180 0 140 206 206 189 123 223 120 99 238 72 54 12 195 208 117 32 16 8 4 252 126 34 138 151 246 142 3 24 6 237 78 186 46 29 164 56 14 187 120 230 246 155 16 142 147 190 48 50 12 175 151 219 1 233 231 3 212 239 151 95 14 14 246 247 3 229 229 229 229 229 229 64 36 50 57 57 57 73 239 99 24 178 221 236 108 44 54 53 5 248 253 62 159 162 200 44 39 155 248 92 91 15 99 209 65 152 211 62 30 149 193 96 48 88 88 40 137 103 139 160 92 132 136 231 116 146 218 19 49 154 70 130 249 124 62 159 199 3 148 148 172 88 177 98 5 80 82 82 84 84 84 4 20 23 83 57 24 244 251 3 1 96 197 138 194 194 64 0 8 4 168 126 32 16 8 4 2 64 56 124 241 226 249 243 192 199 31 63 255 252 222 189 233 66 146 101 205 204 204 204 196 227 192 240 240 240 240 224 32 189 175 105 206 13 182 217 214 147 107 1 22 60 3 216 122 250 251 223 127 255 163 143 0 219 166 244 205 113 40 157 99 41 88 16 199 81 85 215 165 160 103 154 116 196 7 0 94 111 230 74 214 182 229 238 165 109 147 69 80 44 201 76 43 29 135 44 200 227 161 25 164 170 126 191 174 3 87 174 68 163 55 111 210 127 86 20 160 162 98 237 218 244 131 148 100 210 52 77 19 152 154 154 154 154 156 36 65 98 177 249 137 191 87 214 115 199 2 52 54 110 223 254 204 51 64 48 232 245 22 20 0 138 226 243 21 22 206 221 26 96 200 147 43 69 73 183 18 0 184 93 125 18 13 176 109 211 164 35 71 110 79 87 185 25 199 91 20 36 208 247 223 247 247 247 246 2 189 189 159 125 246 249 231 128 235 70 163 177 88 186 0 228 249 87 174 92 190 124 226 4 16 14 135 195 223 125 55 63 241 247 202 122 238 88 128 210 82 211 36 34 2 1 159 15 112 28 191 223 48 36 65 138 66 158 205 87 41 4 121 53 91 145 244 122 185 128 250 255 203 232 228 245 148 5 81 16 230 45 6 106 153 254 70 20 188 7 6 134 135 47 92 0 20 133 118 49 45 107 122 122 118 86 214 26 31 191 118 237 175 191 128 181 107 43 42 54 110 4 218 218 86 175 94 14 196 51 22 28 3 182 108 41 45 173 168 32 66 57 173 164 223 68 168 227 144 199 243 209 158 116 123 143 199 235 5 92 151 60 219 117 41 109 116 93 143 71 215 1 199 49 12 143 71 182 119 28 154 31 150 69 177 193 113 232 234 186 30 15 157 245 82 48 22 194 239 247 122 129 161 161 241 241 209 81 192 52 147 201 68 2 152 152 32 171 97 68 34 145 200 197 139 128 101 165 82 67 67 75 239 249 217 88 240 12 176 109 202 227 109 155 22 48 201 228 212 212 244 52 61 35 139 161 145 234 186 84 143 13 71 211 232 32 132 211 80 222 163 231 153 160 170 180 53 144 157 231 243 186 129 251 179 109 90 185 242 174 102 60 78 49 163 189 189 190 190 161 1 184 121 115 98 226 198 13 96 102 38 22 75 183 160 178 178 178 50 85 93 62 158 191 104 1 70 70 110 222 140 68 104 223 60 145 144 249 59 125 101 132 238 155 38 144 76 38 147 201 36 221 231 44 136 199 149 16 128 174 123 60 134 33 133 224 249 194 66 102 30 35 146 240 188 14 224 5 89 122 12 73 36 40 200 143 143 15 15 95 189 10 148 148 148 148 148 148 200 231 255 70 252 82 89 207 29 11 224 243 145 149 156 58 53 58 58 50 2 172 90 101 24 100 45 252 61 155 204 19 42 199 225 163 196 116 138 129 84 138 206 90 169 13 45 184 210 219 101 7 95 69 161 172 135 173 143 131 176 215 75 11 172 91 183 82 169 68 2 8 4 10 11 11 10 128 151 94 58 124 248 211 79 229 23 169 150 43 241 140 59 126 129 229 190 7 180 80 226 115 189 197 144 51 100 239 14 222 239 229 251 22 75 77 220 67 75 124 54 150 154 200 135 150 248 60 242 200 35 143 60 242 200 35 143 60 242 88 60 254 1 118 124 0 146 139 47 157 2 0 0 0 0 73 69 78 68 174 66 96 130)! !

!DrGIcons class methodsFor: 'form' stamp: 'HilaireFernandes 11/27/2005 19:46'!
formFromGraphic: anArray
	^ self formFromPNG: anArray 
! !

!DrGIcons class methodsFor: 'form' stamp: 'HilaireFernandes 1/6/2012 22:42'!
formFromPNG: anArray
	^(PNGReadWriter createAFormFrom: anArray asByteArray ) first
! !

!DrGIcons class methodsFor: 'accessing' stamp: 'HilaireFernandes 11/26/2005 19:02'!
freePoint
	^self formFromGraphic: #(137 80 78 71 13 10 26 10 0 0 0 13 73 72 68 82 0 0 0 48 0 0 0 48 16 6 0 0 0 7 146 37 196 0 0 0 6 98 75 71 68 255 255 255 255 255 255 9 88 247 220 0 0 0 9 112 72 89 115 0 0 0 72 0 0 0 72 0 70 201 107 62 0 0 0 174 73 68 65 84 120 218 237 215 65 10 131 48 16 5 80 45 61 128 11 23 222 255 162 118 81 178 104 48 104 19 219 76 224 189 141 8 46 134 255 157 160 211 4 0 0 0 0 0 0 192 177 125 255 188 142 231 209 123 128 58 227 6 158 123 246 30 224 88 41 224 121 238 61 217 221 6 219 128 111 139 201 159 143 183 57 131 21 112 85 10 58 21 147 223 199 17 180 128 214 160 74 193 219 128 31 41 29 53 241 55 32 220 64 111 173 111 106 252 224 147 96 27 112 53 248 82 160 227 28 61 73 176 2 206 212 6 31 191 136 32 242 63 219 82 96 103 159 151 241 63 63 7 213 26 188 34 110 34 216 191 219 182 117 93 22 193 3 0 0 0 0 64 133 23 160 158 76 100 195 246 158 5 0 0 0 0 73 69 78 68 174 66 96 130)! !

!DrGIcons class methodsFor: 'accessing' stamp: 'HilaireFernandes 8/3/2010 14:05'!
grid
	^ self formFromGraphic: #(137 80 78 71 13 10 26 10 0 0 0 13 73 72 68 82 0 0 0 48 0 0 0 48 8 6 0 0 0 87 2 249 135 0 0 0 1 115 82 71 66 0 174 206 28 233 0 0 0 6 98 75 71 68 0 255 0 255 0 255 160 189 167 147 0 0 0 9 112 72 89 115 0 0 11 19 0 0 11 19 1 0 154 156 24 0 0 0 7 116 73 77 69 7 218 8 3 11 29 16 179 58 226 95 0 0 0 90 73 68 65 84 104 222 237 213 177 9 0 49 12 4 65 157 251 239 89 46 193 153 145 96 20 127 50 152 251 77 119 215 230 59 181 252 0 198 1 146 116 146 231 48 166 124 183 254 5 226 47 4 0 0 0 0 160 196 74 108 3 0 0 0 0 74 172 196 54 0 0 0 0 0 160 196 74 108 3 0 0 0 0 74 172 196 54 240 241 46 99 58 68 173 164 85 224 244 0 0 0 0 73 69 78 68 174 66 96 130)! !

!DrGIcons class methodsFor: 'accessing' stamp: 'HilaireFernandes 8/3/2010 13:44'!
gridMagnet
	^ self formFromGraphic:  #(137 80 78 71 13 10 26 10 0 0 0 13 73 72 68 82 0 0 0 48 0 0 0 48 8 6 0 0 0 87 2 249 135 0 0 0 1 115 82 71 66 0 174 206 28 233 0 0 0 6 98 75 71 68 0 255 0 255 0 255 160 189 167 147 0 0 0 9 112 72 89 115 0 0 11 19 0 0 11 19 1 0 154 156 24 0 0 0 7 116 73 77 69 7 218 8 3 11 34 36 202 80 60 214 0 0 1 137 73 68 65 84 104 222 237 88 61 75 196 64 16 125 147 28 34 34 71 176 241 42 43 197 66 172 82 137 156 133 189 165 246 254 13 91 193 191 97 47 88 9 150 22 126 112 26 56 27 177 136 41 4 17 221 59 44 2 73 33 98 110 45 206 156 146 160 249 48 159 231 76 19 216 12 179 243 102 119 242 222 132 164 148 168 179 41 168 185 49 128 202 1 32 34 73 68 145 141 81 21 191 218 159 0 241 87 136 1 48 0 6 192 0 254 98 141 172 2 9 33 164 227 56 208 52 173 235 175 189 29 237 234 0 240 110 139 68 177 220 27 129 165 253 83 42 148 137 29 199 41 134 184 2 251 134 136 204 127 41 165 164 168 64 223 253 44 203 146 0 208 19 207 35 31 185 185 149 58 81 109 117 14 0 176 124 104 208 111 251 134 174 80 84 226 73 253 178 182 224 190 153 245 128 95 121 227 170 51 90 91 24 12 146 85 93 249 186 209 246 197 3 0 224 100 101 81 2 192 122 199 164 202 107 161 179 86 43 148 140 253 89 132 141 126 159 152 7 242 182 182 16 212 22 130 38 102 167 153 137 75 181 169 249 153 122 3 80 154 147 60 19 215 87 204 101 205 196 121 251 53 170 84 205 219 237 53 9 0 238 245 83 57 114 26 0 84 85 237 122 158 167 3 128 119 188 135 56 114 250 245 254 101 40 191 31 237 225 179 231 142 222 253 196 192 185 244 64 81 146 58 151 19 240 147 55 46 207 117 127 173 185 115 144 58 158 29 83 8 242 72 25 148 211 166 121 151 90 78 199 189 247 99 117 2 153 141 148 101 249 241 223 105 150 211 12 128 1 48 0 6 48 94 0 120 38 102 38 254 103 61 240 1 57 113 219 42 89 173 167 92 0 0 0 0 73 69 78 68 174 66 96 130)! !

!DrGIcons class methodsFor: 'accessing' stamp: 'HilaireFernandes 3/22/2006 10:35'!
group
	^self formFromGraphic: #(137 80 78 71 13 10 26 10 0 0 0 13 73 72 68 82 0 0 0 24 0 0 0 24 8 2 0 0 0 111 21 170 175 0 0 0 9 112 72 89 115 0 0 11 19 0 0 11 19 1 0 154 156 24 0 0 1 13 73 68 65 84 56 203 173 85 65 106 133 48 20 156 212 224 33 220 231 44 66 90 193 75 40 180 123 93 181 171 118 231 82 104 241 30 173 224 70 240 28 185 129 17 92 199 77 186 248 16 196 95 53 166 153 85 120 129 121 147 153 199 11 209 90 195 7 40 128 199 231 119 66 136 51 133 214 250 231 235 141 2 32 132 124 127 190 58 19 61 189 124 0 120 128 39 80 115 202 243 124 28 199 101 89 76 37 8 130 40 138 154 166 185 70 36 165 172 170 138 49 102 42 66 136 162 40 46 43 82 74 173 89 0 48 198 140 192 44 203 164 148 7 122 169 101 195 105 154 142 245 218 154 125 172 215 103 106 187 68 125 223 167 105 10 128 115 206 57 7 144 36 73 215 117 231 102 111 80 215 245 159 166 196 113 124 162 40 12 67 33 132 189 41 187 138 230 121 46 203 82 41 245 223 201 30 134 97 125 113 243 229 0 66 8 74 233 185 71 123 48 13 40 165 235 157 113 153 168 109 219 107 241 123 155 163 77 136 247 166 216 206 209 125 136 27 83 108 137 54 33 186 63 205 101 142 180 214 183 189 235 188 252 1 16 95 223 209 47 26 227 127 226 181 183 68 199 0 0 0 0 73 69 78 68 174 66 96 130)! !

!DrGIcons class methodsFor: 'form' stamp: 'HilaireFernandes 10/29/2010 10:55'!
iconWithBackground: morphOrForm
	| form imageForm|
	morphOrForm isMorph ifTrue: [imageForm := morphOrForm imageForm] ifFalse: [imageForm := morphOrForm].
	form := (FormCanvas extent: morphOrForm extent + (self menuInset * 2)) fillColor: Color white.
	form paintImage: imageForm at: self menuInset @ self menuInset.
	^ form form! !

!DrGIcons class methodsFor: 'accessing' stamp: 'HilaireFernandes 11/26/2005 19:03'!
intersection
	^ self formFromGraphic: #(137 80 78 71 13 10 26 10 0 0 0 13 73 72 68 82 0 0 0 48 0 0 0 48 16 6 0 0 0 7 146 37 196 0 0 0 6 98 75 71 68 255 255 255 255 255 255 9 88 247 220 0 0 0 9 112 72 89 115 0 0 0 72 0 0 0 72 0 70 201 107 62 0 0 11 48 73 68 65 84 120 218 237 91 91 76 84 231 22 94 123 102 24 174 51 80 112 0 47 132 80 165 197 20 4 163 216 24 19 53 68 171 181 1 155 244 73 49 177 16 105 26 193 32 134 6 212 86 130 137 138 66 50 76 100 30 154 84 211 135 19 209 135 182 52 37 16 155 166 218 23 99 172 161 42 180 160 92 172 145 84 164 168 204 48 195 48 215 125 30 214 249 206 150 61 103 156 97 152 1 60 229 123 217 217 123 246 236 203 250 254 127 253 107 125 107 109 162 69 252 147 32 138 243 253 4 11 13 138 249 126 128 127 58 22 9 152 103 44 18 48 207 152 49 1 54 91 101 101 101 101 160 190 28 231 201 207 247 117 124 17 62 241 211 79 25 25 25 25 162 216 223 159 153 153 153 41 138 179 35 98 209 240 128 16 232 137 31 127 172 209 104 52 162 184 111 223 146 37 75 150 16 165 167 171 84 42 21 209 138 21 187 118 237 218 69 20 29 109 48 24 12 130 143 235 249 50 184 224 243 254 63 252 112 246 236 217 179 162 168 213 62 125 250 244 41 81 92 220 179 103 207 158 17 69 69 77 76 76 76 16 69 68 76 77 77 77 17 41 20 110 183 219 77 228 241 40 149 74 37 145 211 25 21 21 21 69 100 179 105 52 26 13 145 197 146 152 152 152 72 100 54 167 166 166 166 18 21 21 213 212 212 212 8 1 191 247 130 33 32 180 68 120 255 254 203 47 71 142 28 57 34 138 201 201 131 131 131 131 146 129 67 13 135 35 50 50 50 146 104 100 36 35 35 35 131 168 160 224 85 207 27 126 4 124 227 61 123 246 236 217 179 71 20 91 91 91 91 91 91 5 97 118 68 0 210 204 24 24 216 177 99 199 14 239 51 94 188 224 17 254 244 169 203 229 114 17 141 141 241 214 100 242 120 60 30 34 149 138 207 211 106 21 10 133 130 40 37 133 159 35 53 53 34 34 34 130 40 42 138 143 3 10 31 171 222 208 80 86 86 86 22 209 123 239 205 45 33 126 111 36 55 124 176 68 124 243 141 193 96 48 136 226 71 31 85 86 86 86 122 223 167 167 103 251 246 237 219 137 254 250 203 233 116 58 137 6 7 29 14 135 131 232 201 19 222 55 153 216 165 136 34 111 93 174 232 232 232 104 34 183 91 173 86 171 137 212 106 187 221 110 39 138 142 230 153 19 31 63 57 57 57 73 148 158 206 251 105 105 76 136 78 199 207 167 82 9 194 203 102 6 49 47 94 104 181 90 45 209 253 251 235 214 173 91 71 180 111 95 109 109 109 109 248 8 9 122 6 248 34 226 235 175 217 71 203 241 224 1 19 242 214 91 29 29 29 29 222 191 55 55 111 220 184 113 35 209 240 48 143 88 155 77 167 211 233 136 236 118 54 136 219 205 62 30 80 169 96 72 222 42 149 252 187 32 176 97 69 81 20 69 145 72 161 96 98 52 154 177 177 177 49 162 204 76 147 201 100 34 122 231 29 190 79 124 252 244 235 130 8 183 155 127 191 125 59 47 47 47 143 168 184 248 204 153 51 103 66 79 196 140 47 232 143 8 62 43 240 40 231 243 207 243 243 243 243 137 158 63 95 182 108 217 50 162 169 169 164 164 164 164 151 13 200 134 128 161 35 35 217 135 199 198 198 198 198 198 74 251 216 226 124 143 135 93 148 221 206 4 96 59 53 197 51 34 37 229 241 227 199 143 137 222 125 247 249 243 231 207 137 116 58 38 66 173 158 238 178 108 54 190 206 173 91 185 185 185 185 68 101 101 77 77 77 77 161 35 34 232 11 189 154 136 192 175 243 233 167 197 197 197 197 68 118 59 187 18 32 42 138 163 25 181 154 143 107 52 236 122 180 90 158 17 209 209 236 130 14 29 58 116 232 208 33 255 6 57 127 254 252 249 243 231 69 17 4 88 173 86 171 213 74 36 8 28 101 173 94 253 240 225 195 135 68 89 89 188 230 96 237 240 120 120 32 140 140 48 17 191 253 182 126 253 250 245 68 85 85 13 13 13 13 179 39 98 214 23 144 19 113 231 206 222 189 123 247 138 98 94 222 165 75 151 46 249 255 127 73 73 73 73 73 137 52 194 49 130 221 110 54 68 106 42 135 143 216 86 84 84 84 84 84 132 110 4 234 245 122 189 94 47 138 102 51 207 132 188 188 158 158 158 30 162 183 223 102 130 20 10 118 105 14 7 19 112 231 14 175 37 143 30 109 218 180 105 19 209 241 227 199 143 31 63 30 252 243 132 236 69 174 95 175 174 174 174 22 197 21 43 186 187 187 187 137 86 173 186 122 245 234 213 192 255 95 89 201 139 179 201 196 62 26 68 164 167 167 167 167 167 19 213 213 213 213 213 213 133 63 58 105 108 60 125 250 244 105 81 220 188 249 214 173 91 183 136 146 146 108 54 155 141 200 227 225 223 39 38 120 96 92 191 206 65 135 203 181 118 237 218 181 68 53 53 193 229 23 33 211 130 130 53 60 96 48 24 12 6 131 228 171 225 195 205 102 179 217 108 38 106 108 108 108 108 108 12 127 6 253 217 103 199 142 29 59 38 8 191 255 206 62 95 14 141 134 215 138 244 244 39 79 158 60 33 178 90 71 71 71 71 137 154 155 155 155 155 155 103 254 124 179 38 0 9 84 160 134 47 43 43 43 43 43 243 62 206 18 7 209 197 139 201 201 201 201 210 12 152 156 228 112 210 102 227 145 56 87 40 45 173 175 175 175 23 132 251 247 87 173 90 181 202 59 127 64 190 161 84 14 13 13 13 17 141 143 143 143 143 143 207 252 62 179 38 0 153 171 255 23 42 45 45 45 37 234 235 235 235 235 235 147 142 115 248 74 244 232 17 39 88 195 195 28 166 26 141 124 28 139 111 76 76 76 76 76 76 152 173 254 63 240 193 7 70 163 209 40 8 54 27 162 44 62 142 168 73 167 227 181 3 51 181 169 169 169 169 169 41 240 153 16 52 1 208 106 252 73 6 88 100 123 123 123 123 123 123 137 214 172 89 179 102 205 26 162 252 124 14 63 223 120 227 192 129 3 7 136 254 245 47 142 211 229 68 212 215 179 75 170 174 174 174 174 174 158 63 201 96 120 120 229 202 149 43 95 50 220 127 22 231 196 68 142 146 148 74 118 69 22 139 197 98 177 4 126 221 160 9 128 72 6 252 250 235 182 109 219 182 17 157 56 193 134 45 43 219 191 127 255 126 41 49 202 206 206 206 206 206 38 234 234 234 234 234 234 34 218 178 101 203 150 45 91 136 250 251 251 251 251 251 137 56 129 19 4 95 68 204 76 125 13 61 222 127 95 175 215 235 189 7 0 102 130 74 197 193 3 194 219 64 215 132 160 9 128 58 9 64 171 153 152 136 143 143 143 39 114 185 166 39 80 9 9 9 9 9 9 68 133 133 133 133 133 133 68 3 3 3 3 3 3 68 237 237 237 237 237 237 82 30 177 208 137 120 246 44 37 37 37 69 114 69 90 45 18 56 94 171 144 103 96 27 54 2 32 11 3 16 201 60 158 184 184 184 56 105 228 35 97 210 233 88 90 56 118 140 163 140 182 182 182 182 182 54 239 76 122 161 19 97 54 51 1 64 76 12 191 167 82 201 174 210 233 100 237 42 236 4 200 125 63 212 73 232 241 208 102 176 120 130 8 57 124 73 26 11 149 8 171 149 227 127 201 14 60 211 5 129 159 11 9 36 194 233 150 150 150 150 150 22 223 207 23 52 1 40 132 0 144 133 161 78 202 181 27 127 146 193 235 66 196 135 31 202 19 46 94 132 69 145 7 32 242 23 151 11 30 129 247 67 78 0 42 80 0 244 120 200 194 136 227 161 229 4 138 133 78 196 247 223 159 59 119 238 156 116 125 135 131 9 192 154 7 17 17 51 33 108 4 56 157 60 178 1 36 38 208 227 65 0 92 209 76 177 80 137 208 104 56 238 7 172 86 16 192 239 15 128 8 127 8 154 0 155 141 85 73 0 21 40 20 66 192 188 191 17 240 186 17 161 213 142 140 140 140 72 251 38 19 223 207 233 156 62 211 17 132 132 141 0 20 187 1 200 183 168 64 41 149 92 209 194 98 52 91 44 20 34 18 18 88 3 2 70 71 217 213 56 157 92 159 248 175 97 21 88 156 95 77 68 208 4 152 76 211 195 49 0 165 63 84 160 64 0 244 248 215 149 136 159 127 174 173 173 173 149 254 103 183 243 204 70 9 213 110 231 104 15 6 199 26 224 207 21 5 77 192 238 221 92 43 69 151 129 164 145 176 47 68 233 15 34 90 160 113 241 66 37 162 160 160 161 161 161 65 82 123 145 120 254 253 55 27 220 110 231 25 128 232 15 4 32 47 8 57 1 0 218 59 0 20 187 81 115 77 77 29 30 30 30 150 82 244 80 99 110 136 240 254 125 96 128 93 236 228 228 244 18 170 188 54 29 54 23 4 248 234 171 65 177 27 53 87 148 254 80 129 154 27 34 164 138 157 47 34 184 107 67 110 104 81 252 246 219 87 63 231 159 127 50 1 14 7 175 133 48 52 124 127 68 4 187 98 127 97 120 200 212 197 31 127 228 145 244 230 155 44 55 35 242 65 41 239 143 63 152 144 190 190 13 27 54 108 32 58 113 226 228 201 147 39 195 173 110 250 238 200 243 215 197 225 11 122 61 119 111 60 120 192 25 241 212 20 111 97 112 108 147 147 185 174 225 175 146 23 178 138 24 26 154 208 87 131 53 1 93 6 40 118 231 230 114 229 12 165 191 153 25 114 166 91 223 215 195 140 152 233 123 14 13 241 136 134 225 49 242 33 185 204 180 126 17 242 246 116 52 52 161 175 6 64 152 138 98 55 106 174 23 47 214 213 213 213 249 51 24 12 53 211 173 239 235 193 53 125 247 93 115 115 115 115 224 239 103 179 45 93 186 116 233 203 239 197 218 23 164 7 172 117 216 159 115 2 208 73 134 134 38 244 213 160 189 3 133 12 20 187 55 111 190 121 243 230 77 162 246 246 131 7 15 30 12 151 132 32 17 3 195 127 242 73 90 90 90 26 81 78 78 103 103 103 103 224 87 250 234 171 203 151 47 95 246 238 87 66 233 20 97 55 154 11 252 173 121 97 251 64 3 157 100 104 104 66 95 13 214 4 172 17 216 102 101 113 105 179 187 187 168 168 168 72 20 59 59 171 170 170 170 102 75 136 52 19 16 199 127 241 5 90 21 239 222 189 123 55 248 38 130 11 23 46 92 184 112 65 42 69 202 219 106 16 118 251 171 101 207 89 137 79 175 103 3 232 116 247 238 221 187 71 180 122 53 71 17 232 50 0 124 53 207 162 16 2 61 126 114 146 125 240 238 221 172 78 66 36 131 86 3 201 64 158 185 2 129 26 30 53 107 249 98 205 225 43 81 77 13 207 108 52 140 97 6 32 250 89 178 132 159 179 170 170 170 170 170 202 219 53 206 121 141 245 212 169 83 167 78 157 18 197 168 40 94 140 209 222 1 49 15 37 62 184 42 127 196 4 10 100 174 72 160 10 10 174 93 187 118 205 247 249 185 185 60 115 151 47 95 190 124 249 114 162 142 14 206 27 248 67 21 239 38 228 47 191 228 227 24 249 144 225 125 25 126 222 8 0 206 158 229 162 62 250 106 208 222 129 46 3 20 187 65 8 74 127 168 64 161 16 2 61 30 178 48 212 73 136 100 208 106 32 25 140 141 241 255 206 157 187 113 227 198 13 239 231 66 179 64 78 78 78 78 78 14 81 119 55 15 20 16 226 114 93 185 114 229 10 209 236 218 242 23 0 1 0 138 215 232 171 129 79 69 151 1 138 221 168 185 162 244 135 10 20 10 33 208 227 33 11 187 92 60 2 29 14 14 7 161 213 64 50 144 55 255 34 126 71 229 14 134 223 186 117 235 214 173 91 137 110 222 228 96 97 231 206 157 59 119 238 36 186 125 251 232 209 163 71 103 79 132 202 191 137 194 139 195 135 15 31 62 124 88 122 80 244 213 88 44 236 83 17 214 89 44 60 181 161 173 200 11 30 190 68 47 121 134 170 86 243 76 130 193 17 70 202 227 120 142 94 4 33 49 49 49 49 49 81 20 209 60 16 23 23 23 23 23 39 138 151 47 191 156 71 72 223 71 176 179 146 36 14 127 68 204 251 12 240 7 204 16 121 183 1 22 59 196 219 114 245 17 134 135 54 35 47 145 194 240 216 202 7 130 28 193 126 168 146 153 217 223 223 223 255 26 19 224 11 70 163 209 104 52 138 162 220 240 114 2 228 68 148 151 151 151 151 151 7 47 129 204 236 67 149 133 243 49 224 255 29 96 120 95 251 139 152 35 44 26 254 53 199 191 1 200 20 199 16 138 17 10 59 0 0 0 0 73 69 78 68 174 66 96 130)! !

!DrGIcons class methodsFor: 'accessing' stamp: 'HilaireFernandes 8/3/2011 09:49'!
keep
	^self formFromGraphic: #(137 80 78 71 13 10 26 10 0 0 0 13 73 72 68 82 0 0 0 48 0 0 0 48 8 6 0 0 0 87 2 249 135 0 0 0 1 115 82 71 66 0 174 206 28 233 0 0 0 6 98 75 71 68 0 255 0 255 0 255 160 189 167 147 0 0 0 9 112 72 89 115 0 0 11 19 0 0 11 19 1 0 154 156 24 0 0 0 7 116 73 77 69 7 219 8 3 7 48 6 165 130 39 32 0 0 10 122 73 68 65 84 104 222 237 153 123 108 92 85 126 199 63 231 220 153 59 47 143 103 252 72 108 39 118 220 96 231 229 216 236 6 26 18 2 41 109 104 96 11 37 2 90 88 22 181 162 149 90 169 42 143 86 72 84 244 189 84 229 31 170 106 89 118 23 45 21 15 117 89 161 21 219 64 23 136 132 182 11 137 67 30 198 36 177 157 196 118 136 99 27 59 30 63 102 226 177 199 243 186 119 230 222 123 250 199 157 216 134 93 90 59 36 20 85 249 74 71 231 92 221 115 175 126 223 223 235 254 206 239 10 150 7 177 104 173 248 10 64 252 111 27 206 130 79 65 117 123 40 20 77 249 253 193 235 76 51 144 243 120 82 246 236 108 66 65 174 27 50 79 129 245 149 35 208 12 190 191 130 77 183 55 52 220 81 190 97 195 206 76 69 69 149 174 105 254 50 159 175 204 200 231 243 230 220 220 120 126 106 106 56 27 139 157 24 136 199 223 249 61 24 251 202 16 120 30 86 174 175 175 223 85 219 214 246 96 195 154 53 191 25 168 171 43 147 32 148 109 131 174 131 109 43 164 68 21 10 142 25 139 165 166 7 7 255 43 61 50 242 250 177 145 145 35 127 12 147 255 167 4 94 133 186 173 215 92 243 200 234 235 175 127 80 111 104 88 37 53 77 39 157 134 98 209 221 224 56 32 4 72 9 161 16 68 163 202 114 28 149 75 38 135 82 111 191 253 179 131 163 163 223 249 35 136 125 89 4 180 197 23 123 32 124 95 67 195 67 141 219 183 255 89 160 169 169 94 228 114 30 230 230 192 178 64 41 119 44 134 101 65 177 40 164 215 43 124 107 214 84 4 34 145 141 101 62 159 186 229 252 249 193 159 66 234 75 37 112 31 104 127 174 105 183 55 93 119 221 227 145 166 166 102 230 230 36 134 1 126 63 4 131 16 14 187 90 119 28 119 128 107 9 77 3 33 16 182 45 60 141 141 254 96 117 245 250 96 60 30 216 50 61 221 253 22 228 190 52 2 223 134 234 77 235 214 61 82 179 109 219 78 41 132 78 161 0 145 8 118 40 84 52 149 74 57 134 145 23 62 159 146 213 213 26 66 128 105 138 121 139 232 58 120 60 8 165 132 103 245 234 178 160 215 187 198 201 100 6 94 154 158 30 0 236 75 201 126 75 133 231 226 34 84 94 254 245 200 218 181 59 101 40 228 39 147 129 186 58 85 72 38 179 163 61 61 135 6 98 177 247 2 201 100 188 174 162 98 195 234 182 182 61 254 13 27 214 123 124 62 157 169 41 215 141 12 195 181 18 32 12 67 248 175 189 182 54 52 59 123 239 99 103 207 118 61 7 3 0 175 131 118 11 172 176 161 62 9 99 173 151 41 216 47 18 208 107 66 161 173 161 166 166 58 233 245 74 85 85 133 157 78 91 201 206 206 125 51 189 189 207 236 207 229 250 247 64 241 92 34 17 152 155 153 249 224 154 153 153 191 168 220 177 227 54 17 137 72 82 41 151 64 62 239 6 181 109 35 170 170 188 181 45 45 55 223 223 210 242 141 135 251 250 114 85 161 80 139 30 137 220 224 105 106 218 158 235 235 139 38 167 167 159 228 114 18 216 11 85 117 171 86 125 205 91 95 31 192 178 80 154 102 39 142 28 233 57 126 226 196 139 119 219 118 23 160 190 227 238 79 255 99 34 241 243 223 237 236 44 115 234 235 235 171 27 26 90 132 101 73 114 57 200 231 81 166 137 8 135 193 178 136 52 55 215 110 222 184 241 17 61 26 189 211 211 214 182 78 108 218 180 198 209 52 235 194 203 47 191 254 202 244 244 217 203 234 66 6 4 236 138 138 106 41 132 80 193 160 178 19 9 51 55 57 185 175 203 182 15 126 182 100 120 10 156 177 76 230 253 63 56 125 122 109 69 85 213 223 120 106 107 35 76 76 184 241 96 154 110 80 43 133 39 24 148 225 251 239 111 82 154 182 78 248 253 66 248 253 42 215 217 57 53 58 59 251 142 23 102 47 23 1 9 16 2 175 45 165 215 17 110 108 25 195 195 99 185 120 252 200 83 80 248 85 15 189 4 51 51 39 79 182 171 145 145 243 132 66 174 235 248 124 96 219 110 134 10 4 16 150 37 132 174 75 169 148 16 166 137 74 165 84 166 183 183 47 62 49 209 249 111 80 188 24 204 81 63 27 87 148 177 85 150 100 209 4 190 101 91 160 25 18 86 44 118 216 60 124 184 205 145 114 58 113 242 228 235 231 19 137 19 191 234 129 65 136 84 193 6 35 26 221 161 138 69 93 8 161 8 135 5 134 225 10 111 154 8 175 23 188 94 112 28 129 166 129 223 143 49 60 108 166 123 123 143 60 95 48 140 230 42 190 181 54 202 125 245 97 118 215 53 172 42 179 109 155 137 216 84 194 171 81 166 9 2 189 113 190 127 248 60 143 46 153 64 43 36 255 179 183 247 123 190 193 193 83 159 120 189 115 31 164 211 31 253 45 92 184 184 105 63 148 109 134 245 254 218 218 237 162 177 241 6 79 115 243 13 129 134 134 70 185 122 181 31 81 50 155 174 187 31 58 203 66 41 229 146 80 10 28 7 5 196 123 186 46 252 96 166 111 119 203 215 121 114 91 164 220 119 253 238 221 108 191 227 14 242 217 44 63 121 230 25 34 229 193 21 235 182 108 97 232 212 41 202 83 115 77 203 78 163 119 195 56 134 241 26 134 33 1 103 177 155 69 96 167 83 89 249 168 190 107 215 54 237 182 219 42 132 166 9 146 73 200 100 22 74 12 41 93 129 139 69 55 43 133 195 238 135 78 215 49 38 199 233 254 248 112 195 230 123 111 106 184 241 158 61 180 238 216 193 248 208 16 255 241 236 179 204 77 79 115 239 163 143 178 101 215 46 132 16 60 251 240 195 28 25 248 240 228 178 9 44 130 243 217 235 126 232 160 88 36 55 58 58 178 242 232 209 59 245 214 214 90 161 105 30 33 165 64 136 133 50 67 8 215 141 50 25 84 101 37 66 215 193 239 167 48 50 200 141 143 255 41 43 239 218 73 54 149 226 71 255 244 247 244 30 61 198 3 79 60 193 245 187 119 35 176 32 125 12 50 93 196 71 134 72 230 56 121 73 181 208 231 225 13 48 94 40 20 206 5 70 71 59 42 187 186 134 69 177 136 208 180 74 189 186 58 36 162 81 65 161 176 80 228 129 75 38 26 69 4 131 168 120 156 76 172 15 255 111 175 32 192 41 94 253 231 167 233 63 118 142 199 95 120 129 13 91 183 34 242 103 33 254 19 200 157 165 224 132 217 251 114 59 199 39 248 118 222 34 126 217 8 92 196 33 200 191 168 212 153 77 177 216 71 7 134 63 174 44 76 12 183 148 235 62 221 87 187 10 225 247 131 109 33 28 199 37 16 14 35 194 101 100 207 157 38 31 236 32 178 46 137 70 134 55 127 124 138 223 248 253 7 184 238 214 91 97 122 31 164 59 92 49 180 32 147 241 0 251 223 254 176 208 17 227 113 165 126 201 19 150 236 66 255 115 222 21 136 189 245 230 191 54 68 204 123 102 242 89 236 49 31 109 90 145 202 112 13 190 154 58 48 13 200 230 192 52 113 76 147 66 106 128 178 27 43 209 67 85 204 165 10 92 152 152 97 251 157 119 130 157 134 194 56 104 17 16 26 8 141 225 115 179 204 26 244 219 206 210 79 120 203 38 240 181 26 158 104 140 114 79 203 246 237 252 201 211 79 227 215 138 100 143 255 156 236 161 15 41 14 86 227 175 220 128 167 102 21 132 130 88 137 73 148 158 36 208 84 79 60 110 242 234 119 223 163 126 253 122 42 107 107 33 63 8 90 216 21 30 13 132 228 227 158 110 166 151 225 255 151 68 224 215 162 220 189 162 190 158 199 158 123 14 145 122 31 210 125 148 55 73 10 117 53 228 123 211 100 79 31 195 123 174 14 255 154 54 138 83 131 196 203 114 188 253 195 118 186 14 245 211 122 243 77 60 244 216 95 186 47 42 76 129 86 182 64 64 11 50 208 221 199 180 113 101 9 4 117 141 234 45 187 118 33 164 0 99 196 213 162 244 160 71 61 232 55 175 34 187 209 96 224 167 61 152 191 232 34 254 201 24 29 229 89 54 239 190 133 191 123 237 31 168 105 108 116 223 98 198 192 201 185 4 144 32 36 249 66 136 153 120 156 68 150 163 87 138 64 8 228 77 66 58 129 178 72 4 236 76 73 131 30 16 30 28 37 121 235 71 237 252 226 141 195 212 172 169 227 218 109 191 78 219 35 15 112 215 45 55 151 14 66 38 20 38 160 152 4 107 22 180 208 188 235 128 100 176 119 4 203 33 55 149 161 243 10 17 208 154 65 248 52 225 120 188 62 31 40 11 60 229 37 2 26 123 127 184 143 51 221 35 60 249 202 43 212 175 223 8 202 6 59 5 230 8 216 179 224 24 11 2 203 64 41 112 101 201 2 26 93 237 31 49 153 225 144 163 230 235 164 203 77 64 249 65 32 193 227 213 245 18 39 55 8 135 251 207 211 117 184 159 191 254 247 31 19 174 172 112 93 171 48 233 30 188 46 10 42 131 37 127 151 139 4 119 103 219 145 28 123 255 16 227 25 14 92 242 137 108 9 13 12 89 170 24 52 175 223 239 10 87 114 131 131 251 58 217 253 135 15 17 174 172 132 66 28 172 36 72 255 167 5 254 140 208 139 215 159 244 15 97 100 179 140 207 177 255 146 202 233 37 90 192 41 17 240 186 22 144 32 67 152 5 73 255 241 62 118 220 181 199 221 98 205 184 46 162 5 92 18 154 191 68 102 241 236 115 135 208 65 234 116 188 123 128 76 129 79 226 89 62 188 146 22 48 1 60 2 143 230 241 148 122 67 1 142 183 119 114 195 55 126 7 95 32 224 6 168 244 254 146 134 221 18 99 209 60 191 22 204 38 102 57 252 214 59 156 185 192 203 151 210 111 93 134 5 156 66 169 157 160 205 87 33 210 71 199 187 7 249 173 111 126 211 213 190 147 255 180 134 165 15 164 14 226 226 92 26 210 11 194 139 227 104 188 246 47 223 165 88 180 204 143 147 188 242 133 186 18 75 128 41 5 218 124 75 68 72 226 177 11 148 87 85 83 177 114 165 91 26 72 189 116 123 65 195 159 94 151 134 144 140 13 12 177 247 251 207 211 123 180 195 126 111 136 111 101 205 75 235 173 46 199 133 10 82 40 239 194 165 198 169 163 39 184 245 193 7 93 237 43 27 132 247 115 4 22 243 233 114 232 84 47 251 94 124 137 211 71 142 96 88 36 14 141 242 240 240 44 111 126 225 190 208 82 44 160 73 225 89 236 125 10 193 218 214 86 55 199 11 207 188 118 23 250 86 174 208 177 193 33 186 15 180 211 125 224 0 163 103 206 160 192 58 61 197 247 142 79 240 84 193 254 98 45 200 229 16 40 104 40 109 113 86 93 219 218 182 112 254 17 222 121 2 150 229 48 216 211 67 247 129 118 122 14 30 100 122 124 156 124 145 100 44 205 193 243 115 252 44 54 199 187 185 226 229 109 108 45 41 136 165 230 241 2 164 103 102 112 108 135 181 155 55 3 2 195 176 25 58 121 146 129 19 93 156 235 238 102 248 244 105 12 163 96 78 100 69 207 88 74 236 31 75 243 102 50 183 252 20 121 217 93 40 95 80 169 162 67 238 131 55 222 8 122 117 157 209 51 103 24 232 234 98 236 236 89 114 69 39 157 200 136 51 147 25 209 51 153 161 61 158 227 93 219 81 23 174 244 159 168 101 54 89 181 93 13 229 106 231 166 21 234 246 160 151 138 164 193 249 169 140 236 139 103 56 58 99 56 199 64 77 2 217 175 212 63 178 207 108 143 128 92 235 86 166 106 14 84 28 212 133 207 233 64 95 197 85 92 197 85 92 197 85 92 197 255 123 252 55 116 190 108 56 154 203 241 187 0 0 0 0 73 69 78 68 174 66 96 130)! !

!DrGIcons class methodsFor: 'accessing' stamp: 'HilaireFernandes 11/26/2005 19:03'!
line
	^self formFromGraphic: #(137 80 78 71 13 10 26 10 0 0 0 13 73 72 68 82 0 0 0 48 0 0 0 48 16 6 0 0 0 7 146 37 196 0 0 0 6 98 75 71 68 255 255 255 255 255 255 9 88 247 220 0 0 0 9 112 72 89 115 0 0 0 72 0 0 0 72 0 70 201 107 62 0 0 1 149 73 68 65 84 120 218 237 155 61 174 130 64 20 133 103 172 97 17 20 108 107 98 50 157 49 178 25 151 96 193 10 168 220 137 149 137 37 189 13 201 125 5 121 49 206 123 58 255 204 85 206 215 65 32 145 243 225 56 3 71 33 0 47 136 250 190 239 137 74 127 142 213 113 62 55 77 211 16 93 46 109 219 182 68 16 81 8 83 196 253 222 117 93 7 17 185 216 152 59 78 167 113 28 71 33 174 215 105 154 38 33 110 183 97 24 6 33 32 34 51 74 41 165 212 35 224 237 182 174 235 154 40 238 27 241 123 28 196 189 196 12 62 141 8 51 120 8 176 226 42 98 222 178 5 12 1 193 216 68 204 216 68 32 248 104 222 139 128 128 197 136 21 161 181 214 90 63 206 55 183 129 35 177 67 17 130 79 74 232 80 4 17 38 27 191 195 93 131 147 210 220 51 79 95 133 56 30 171 170 170 74 95 54 31 60 5 216 144 242 249 14 151 114 158 190 98 101 157 136 215 179 28 219 143 109 154 149 53 248 131 45 120 136 200 132 111 240 16 145 136 216 224 243 60 244 91 33 190 11 172 60 15 253 86 200 225 176 223 239 118 225 43 91 136 96 66 168 8 188 42 77 12 68 48 1 67 19 19 32 130 9 16 193 4 136 96 2 68 48 1 34 152 0 17 76 128 8 38 64 4 19 150 17 129 218 141 21 84 46 153 128 202 37 19 80 185 100 66 234 202 37 122 78 129 152 34 158 131 68 193 108 49 254 15 114 57 17 50 244 196 239 36 246 142 150 222 121 38 110 198 125 42 241 149 203 208 5 29 4 56 129 202 101 102 80 185 44 14 154 126 133 136 13 222 85 4 90 28 22 82 255 149 202 20 81 250 250 216 18 219 244 3 31 206 15 224 42 63 94 246 95 66 123 0 0 0 0 73 69 78 68 174 66 96 130)! !

!DrGIcons class methodsFor: 'accessing' stamp: 'HilaireFernandes 11/28/2005 09:19'!
locus
	^self formFromGraphic: #(137 80 78 71 13 10 26 10 0 0 0 13 73 72 68 82 0 0 0 48 0 0 0 48 8 6 0 0 0 87 2 249 135 0 0 0 6 98 75 71 68 0 99 0 99 0 99 185 251 243 121 0 0 0 9 112 72 89 115 0 0 11 19 0 0 11 19 1 0 154 156 24 0 0 0 7 116 73 77 69 7 213 11 28 8 18 45 208 0 131 20 0 0 0 29 116 69 88 116 67 111 109 109 101 110 116 0 67 114 101 97 116 101 100 32 119 105 116 104 32 84 104 101 32 71 73 77 80 239 100 37 110 0 0 3 87 73 68 65 84 104 222 237 152 201 139 19 65 20 198 127 17 241 232 93 196 187 127 136 224 217 131 94 20 81 90 16 65 16 84 80 65 208 163 140 163 40 110 12 58 174 131 251 46 138 184 226 134 131 136 136 3 234 136 224 174 227 160 102 50 49 147 69 103 62 15 121 5 25 237 116 58 109 119 18 49 15 138 110 146 234 170 239 171 122 239 213 247 10 218 214 182 182 181 173 109 255 189 149 64 5 208 191 128 53 7 26 174 192 154 114 4 42 59 141 1 63 128 97 96 170 245 9 48 85 142 149 132 141 128 198 128 209 50 86 138 246 44 185 73 243 32 25 112 128 9 246 71 10 248 9 100 170 19 169 36 158 74 98 181 221 98 230 129 172 97 201 216 123 190 218 164 251 65 211 129 41 192 100 96 146 177 159 60 190 91 42 41 2 35 32 183 218 57 96 0 120 14 204 143 50 199 102 208 19 208 183 50 216 106 45 54 203 130 190 130 222 128 238 213 51 182 231 121 129 157 15 39 76 224 59 40 11 26 0 61 14 49 166 47 94 247 227 239 79 31 191 31 215 106 145 15 3 126 8 244 30 116 57 0 124 32 190 106 127 250 116 86 156 36 178 230 158 175 171 0 15 129 167 246 54 213 2 31 213 149 178 160 180 249 251 77 159 239 35 129 174 145 239 199 181 231 21 239 215 35 16 248 6 122 7 186 20 2 124 28 246 199 106 219 224 42 130 62 131 186 234 32 145 1 13 90 134 75 18 116 53 2 227 38 115 169 239 92 29 4 6 65 253 160 3 62 227 37 174 71 126 159 44 109 250 228 33 104 93 72 18 239 65 189 173 0 222 243 60 165 65 35 150 73 14 133 32 240 25 244 10 116 173 34 123 53 13 60 101 209 167 2 232 11 232 10 104 118 13 18 3 160 23 21 46 215 16 240 65 129 150 181 64 30 2 221 5 45 8 32 48 104 7 214 19 219 173 166 131 119 170 177 104 113 112 31 180 40 128 192 71 115 159 7 160 93 205 216 1 63 203 27 129 140 9 176 133 1 4 222 153 251 220 1 109 142 97 7 38 196 65 192 105 220 81 211 232 185 128 190 211 32 85 50 141 63 6 116 119 119 167 154 78 192 85 113 133 242 233 202 113 72 5 173 108 209 14 147 137 173 224 62 148 129 43 103 210 162 51 132 91 244 130 110 129 246 180 74 29 238 2 248 118 29 121 253 1 232 60 104 101 179 73 20 44 128 7 65 61 33 179 138 231 121 234 179 128 63 80 35 232 27 146 129 50 160 62 208 234 58 118 224 145 165 220 11 160 245 205 34 224 192 247 71 212 52 55 64 87 65 251 154 225 74 5 171 97 95 131 142 68 208 52 174 223 101 208 41 59 212 230 52 138 132 3 255 22 116 210 71 207 215 123 178 159 6 29 179 131 109 73 146 36 138 166 58 211 160 151 160 19 49 20 35 174 127 15 232 32 104 27 104 45 104 94 156 68 138 166 54 179 160 79 166 99 58 99 4 239 158 59 173 170 219 14 218 8 90 243 183 36 138 160 60 232 187 73 229 103 230 175 113 214 174 126 223 111 2 109 5 109 1 117 88 219 16 68 166 4 42 89 62 47 84 128 30 178 21 239 3 157 177 52 153 68 225 237 55 206 10 75 175 157 70 100 7 104 183 37 140 75 38 219 31 57 60 14 120 206 238 104 62 128 158 218 85 71 23 104 113 3 110 13 170 141 55 23 180 220 200 116 152 123 237 5 29 5 157 117 184 210 160 143 182 210 23 45 144 150 129 102 132 188 108 74 186 226 115 54 19 52 11 52 31 180 20 180 42 202 221 105 210 53 108 34 243 52 10 252 95 205 27 246 110 180 209 250 189 46 28 173 2 58 18 174 86 5 223 168 4 210 182 168 246 11 175 92 10 76 202 252 154 245 0 0 0 0 73 69 78 68 174 66 96 130)! !

!DrGIcons class methodsFor: 'accessing' stamp: 'HilaireFernandes 7/25/2008 09:20'!
macroBuild
	^ self formFromGraphic: #(
			137
			80
			78
			71
			13
			10
			26
			10
			0
			0
			0
			13
			73
			72
			68
			82
			0
			0
			0
			48
			0
			0
			0
			48
			16
			6
			0
			0
			0
			7
			146
			37
			196
			0
			0
			0
			6
			98
			75
			71
			68
			255
			255
			255
			255
			255
			255
			9
			88
			247
			220
			0
			0
			0
			9
			112
			72
			89
			115
			0
			0
			0
			72
			0
			0
			0
			72
			0
			70
			201
			107
			62
			0
			0
			16
			223
			73
			68
			65
			84
			120
			218
			237
			90
			123
			80
			84
			231
			21
			255
			221
			221
			187
			187
			44
			203
			99
			129
			229
			177
			203
			35
			192
			138
			32
			10
			10
			40
			104
			89
			17
			43
			160
			213
			196
			198
			154
			76
			195
			24
			107
			211
			214
			209
			38
			182
			206
			244
			49
			157
			206
			116
			226
			180
			51
			109
			30
			181
			173
			147
			56
			182
			102
			106
			219
			212
			8
			141
			160
			81
			51
			2
			106
			70
			81
			20
			34
			6
			80
			43
			6
			20
			17
			129
			240
			208
			229
			177
			60
			118
			97
			95
			95
			255
			56
			249
			230
			2
			134
			65
			35
			15
			147
			122
			102
			244
			114
			247
			222
			251
			221
			239
			59
			231
			252
			126
			231
			247
			157
			93
			224
			137
			61
			177
			39
			54
			105
			198
			216
			232
			227
			215
			197
			166
			110
			93
			194
			228
			78
			112
			220
			215
			76
			210
			123
			166
			219
			166
			126
			93
			147
			236
			152
			175
			75
			32
			166
			111
			29
			83
			228
			144
			175
			106
			32
			166
			127
			222
			83
			236
			136
			175
			74
			32
			102
			110
			158
			178
			169
			93
			216
			227
			226
			224
			199
			119
			254
			211
			236
			160
			241
			50
			109
			166
			2
			53
			243
			243
			153
			98
			4
			60
			234
			194
			30
			85
			246
			61
			236
			243
			211
			159
			8
			211
			28
			128
			241
			22
			58
			214
			81
			147
			173
			183
			39
			26
			127
			230
			168
			242
			49
			225
			232
			169
			42
			130
			143
			191
			8
			152
			33
			4
			112
			155
			233
			29
			243
			76
			191
			127
			198
			2
			48
			243
			11
			127
			92
			230
			35
			206
			204
			107
			39
			170
			1
			51
			53
			143
			39
			54
			198
			166
			91
			5
			61
			177
			255
			59
			155
			49
			8
			38
			38
			38
			38
			38
			37
			49
			22
			26
			26
			30
			30
			29
			13
			248
			248
			248
			248
			4
			4
			0
			173
			173
			45
			45
			245
			245
			0
			99
			110
			183
			195
			1
			148
			151
			151
			151
			151
			151
			79
			76
			21
			161
			161
			161
			161
			161
			161
			140
			133
			132
			232
			245
			97
			97
			244
			188
			32
			0
			22
			139
			197
			210
			219
			11
			180
			180
			180
			180
			52
			54
			2
			169
			169
			169
			169
			105
			105
			192
			249
			243
			231
			207
			151
			149
			205
			60
			5
			77
			123
			13
			8
			15
			15
			15
			15
			15
			103
			108
			233
			210
			101
			203
			86
			175
			6
			102
			205
			154
			61
			59
			49
			17
			144
			203
			229
			114
			185
			28
			232
			239
			239
			239
			79
			77
			5
			246
			237
			251
			219
			223
			94
			127
			125
			226
			241
			182
			108
			217
			178
			101
			203
			22
			198
			42
			42
			202
			203
			47
			93
			2
			182
			110
			125
			229
			149
			29
			59
			128
			190
			62
			114
			188
			203
			229
			118
			59
			157
			64
			71
			71
			123
			123
			107
			43
			240
			175
			127
			237
			219
			183
			115
			231
			248
			227
			205
			157
			59
			119
			238
			220
			185
			140
			13
			13
			13
			13
			13
			13
			241
			115
			224
			216
			177
			99
			199
			142
			29
			155
			252
			128
			125
			105
			21
			180
			113
			227
			198
			141
			27
			55
			50
			22
			31
			31
			31
			31
			31
			207
			88
			112
			112
			72
			136
			193
			192
			216
			143
			126
			180
			121
			243
			214
			173
			227
			115
			111
			76
			76
			76
			76
			124
			60
			112
			243
			102
			67
			67
			93
			29
			80
			87
			87
			91
			91
			83
			3
			244
			247
			247
			245
			89
			44
			128
			74
			165
			84
			170
			213
			192
			130
			5
			201
			201
			139
			23
			3
			217
			217
			217
			217
			217
			217
			227
			143
			215
			208
			208
			208
			208
			208
			0
			196
			198
			198
			197
			37
			36
			72
			227
			12
			14
			14
			14
			14
			12
			0
			78
			167
			195
			225
			116
			2
			90
			173
			175
			175
			159
			31
			32
			147
			201
			100
			10
			197
			253
			227
			228
			230
			230
			230
			230
			230
			50
			102
			181
			90
			173
			86
			43
			33
			37
			35
			3
			80
			171
			53
			26
			63
			63
			32
			41
			41
			41
			41
			57
			153
			177
			184
			184
			184
			184
			184
			56
			198
			114
			114
			114
			114
			114
			114
			30
			189
			198
			60
			112
			68
			55
			111
			222
			188
			121
			243
			102
			198
			138
			139
			139
			139
			139
			139
			129
			144
			144
			144
			144
			208
			80
			96
			237
			218
			103
			159
			125
			233
			37
			192
			108
			238
			238
			238
			236
			4
			14
			30
			204
			207
			223
			187
			23
			48
			153
			76
			166
			244
			116
			160
			176
			176
			160
			160
			160
			64
			202
			156
			196
			196
			249
			243
			147
			146
			24
			211
			106
			253
			252
			12
			6
			96
			217
			178
			204
			204
			85
			171
			0
			15
			15
			15
			15
			79
			79
			114
			144
			76
			6
			168
			84
			42
			149
			90
			13
			92
			187
			246
			223
			255
			86
			86
			2
			183
			110
			221
			186
			85
			87
			7
			216
			237
			195
			195
			195
			195
			192
			189
			123
			247
			238
			181
			183
			3
			203
			151
			47
			95
			190
			102
			13
			16
			27
			59
			103
			206
			252
			249
			0
			207
			92
			128
			74
			48
			71
			86
			91
			219
			103
			159
			181
			180
			0
			133
			133
			7
			15
			190
			243
			14
			160
			84
			42
			149
			226
			8
			252
			107
			181
			126
			126
			58
			29
			176
			102
			205
			51
			207
			188
			248
			34
			160
			215
			235
			245
			17
			17
			128
			74
			229
			225
			225
			225
			1
			200
			229
			52
			47
			171
			213
			102
			179
			90
			129
			63
			252
			225
			183
			191
			253
			201
			79
			128
			89
			179
			98
			98
			140
			70
			160
			172
			172
			172
			236
			203
			80
			218
			132
			15
			240
			204
			56
			119
			238
			220
			185
			243
			231
			1
			157
			46
			56
			56
			52
			20
			216
			182
			109
			219
			182
			29
			59
			128
			254
			254
			129
			1
			139
			5
			176
			90
			41
			227
			42
			43
			47
			94
			44
			45
			5
			44
			150
			158
			158
			123
			247
			0
			187
			221
			225
			176
			217
			128
			206
			206
			206
			206
			206
			78
			192
			233
			116
			58
			157
			78
			32
			51
			147
			28
			183
			112
			225
			162
			69
			25
			25
			128
			219
			237
			118
			187
			221
			128
			211
			233
			114
			57
			157
			128
			32
			48
			230
			118
			3
			114
			185
			66
			161
			84
			2
			54
			155
			213
			58
			56
			8
			244
			246
			90
			44
			221
			221
			128
			167
			39
			33
			69
			167
			11
			10
			10
			14
			6
			6
			7
			109
			182
			193
			65
			226
			126
			64
			10
			164
			76
			38
			151
			203
			100
			128
			66
			161
			80
			120
			120
			0
			102
			179
			217
			220
			222
			14
			216
			108
			116
			191
			70
			163
			209
			120
			123
			3
			190
			190
			62
			62
			90
			45
			48
			60
			76
			1
			230
			243
			49
			155
			205
			230
			206
			78
			224
			192
			129
			127
			255
			123
			247
			110
			96
			222
			188
			196
			196
			148
			20
			96
			201
			146
			37
			75
			50
			51
			129
			252
			252
			247
			222
			219
			179
			7
			184
			117
			171
			177
			177
			161
			97
			10
			2
			144
			145
			145
			145
			145
			145
			193
			88
			86
			214
			202
			149
			185
			185
			64
			73
			73
			81
			81
			97
			33
			81
			200
			167
			159
			2
			155
			54
			109
			218
			180
			125
			59
			16
			20
			20
			28
			172
			215
			3
			130
			32
			8
			124
			193
			74
			37
			32
			138
			162
			40
			138
			128
			82
			169
			82
			121
			120
			144
			99
			228
			114
			224
			220
			185
			179
			103
			139
			139
			129
			207
			62
			107
			109
			109
			106
			2
			122
			123
			187
			187
			239
			222
			5
			26
			26
			110
			222
			188
			126
			29
			80
			171
			61
			61
			61
			60
			104
			28
			181
			26
			112
			185
			92
			46
			0
			240
			240
			208
			104
			84
			42
			64
			163
			161
			204
			116
			185
			156
			78
			135
			3
			24
			24
			24
			28
			164
			0
			0
			50
			25
			57
			210
			102
			3
			100
			50
			65
			96
			12
			240
			242
			242
			242
			242
			241
			1
			162
			163
			141
			198
			57
			115
			0
			133
			66
			169
			244
			244
			4
			210
			211
			77
			166
			236
			108
			64
			173
			86
			171
			61
			61
			1
			171
			149
			2
			45
			8
			130
			32
			8
			64
			93
			93
			93
			221
			213
			171
			192
			225
			195
			5
			5
			127
			255
			59
			176
			115
			231
			174
			93
			121
			121
			20
			56
			31
			31
			224
			205
			55
			95
			123
			237
			167
			63
			5
			210
			210
			82
			83
			83
			83
			129
			210
			210
			179
			103
			203
			203
			249
			58
			38
			14
			200
			132
			55
			164
			167
			167
			167
			167
			167
			51
			246
			253
			239
			255
			240
			135
			191
			254
			53
			101
			176
			219
			13
			156
			60
			121
			226
			196
			161
			67
			128
			40
			202
			229
			162
			8
			36
			39
			47
			92
			104
			50
			1
			14
			135
			195
			97
			183
			143
			120
			193
			231
			11
			113
			58
			201
			133
			127
			252
			227
			235
			175
			255
			226
			23
			192
			240
			48
			81
			197
			142
			29
			191
			251
			221
			91
			111
			1
			158
			158
			106
			181
			151
			23
			57
			194
			195
			3
			8
			8
			208
			233
			244
			122
			10
			156
			66
			1
			132
			134
			234
			245
			58
			29
			16
			23
			23
			27
			27
			25
			9
			248
			248
			120
			123
			139
			34
			64
			249
			14
			220
			186
			213
			208
			112
			247
			46
			208
			222
			126
			247
			110
			119
			55
			48
			56
			56
			48
			96
			181
			2
			29
			29
			132
			60
			179
			153
			40
			75
			20
			229
			114
			165
			18
			200
			203
			59
			112
			96
			239
			94
			160
			186
			186
			170
			170
			188
			28
			120
			238
			185
			231
			159
			255
			193
			15
			168
			246
			124
			227
			27
			0
			99
			132
			37
			62
			255
			75
			151
			42
			43
			203
			202
			36
			53
			101
			183
			219
			237
			195
			195
			132
			252
			254
			126
			105
			189
			27
			55
			110
			218
			244
			227
			31
			3
			251
			247
			191
			251
			238
			95
			255
			58
			113
			32
			38
			84
			65
			114
			185
			40
			170
			84
			64
			113
			113
			81
			81
			65
			1
			16
			20
			20
			20
			20
			30
			78
			153
			44
			138
			64
			124
			252
			220
			185
			201
			201
			128
			151
			151
			183
			183
			143
			143
			196
			209
			78
			39
			101
			166
			203
			69
			148
			35
			8
			130
			32
			151
			3
			105
			105
			139
			23
			103
			102
			2
			151
			47
			87
			87
			151
			151
			3
			71
			143
			30
			57
			178
			127
			63
			113
			181
			82
			73
			114
			242
			169
			167
			40
			51
			159
			126
			154
			206
			195
			194
			164
			226
			201
			19
			192
			110
			119
			56
			104
			134
			84
			8
			25
			147
			201
			24
			3
			148
			74
			133
			66
			161
			0
			218
			219
			45
			150
			158
			30
			224
			234
			213
			154
			154
			202
			74
			160
			162
			162
			188
			252
			204
			25
			66
			164
			92
			14
			220
			188
			121
			227
			198
			181
			107
			192
			188
			121
			9
			9
			201
			201
			64
			112
			176
			193
			16
			30
			46
			173
			155
			49
			26
			87
			165
			82
			169
			84
			42
			32
			43
			43
			39
			231
			217
			103
			169
			118
			168
			84
			210
			125
			78
			167
			211
			105
			183
			75
			181
			199
			219
			155
			252
			48
			48
			208
			223
			223
			221
			253
			8
			20
			20
			18
			18
			18
			18
			18
			194
			88
			88
			24
			233
			244
			13
			27
			190
			247
			189
			237
			219
			73
			38
			246
			244
			80
			113
			172
			174
			6
			174
			92
			169
			169
			249
			248
			99
			130
			180
			74
			69
			106
			195
			215
			151
			138
			98
			82
			18
			16
			21
			21
			21
			21
			23
			71
			84
			34
			138
			128
			40
			114
			78
			167
			204
			113
			185
			24
			115
			185
			168
			102
			116
			119
			3
			21
			21
			23
			46
			20
			21
			1
			55
			110
			220
			184
			209
			208
			64
			84
			50
			178
			168
			138
			162
			40
			42
			149
			128
			32
			200
			100
			110
			55
			101
			180
			66
			65
			212
			97
			179
			1
			196
			250
			68
			67
			140
			73
			212
			104
			50
			45
			91
			182
			114
			37
			16
			31
			63
			103
			78
			98
			34
			33
			213
			229
			146
			138
			189
			219
			237
			118
			187
			92
			128
			195
			97
			183
			219
			237
			180
			30
			165
			18
			184
			115
			231
			246
			237
			250
			122
			224
			244
			233
			51
			103
			142
			31
			31
			73
			81
			244
			207
			100
			90
			186
			52
			43
			11
			136
			143
			143
			143
			95
			184
			16
			104
			111
			239
			236
			108
			106
			2
			222
			121
			103
			207
			158
			55
			222
			32
			164
			216
			237
			227
			35
			224
			190
			11
			92
			246
			53
			53
			53
			53
			53
			53
			17
			247
			202
			100
			18
			39
			207
			158
			29
			19
			147
			152
			8
			168
			84
			106
			181
			70
			3
			132
			133
			69
			68
			68
			69
			17
			50
			66
			67
			129
			210
			210
			211
			167
			143
			29
			163
			0
			85
			85
			1
			171
			87
			175
			89
			243
			194
			11
			64
			120
			56
			221
			199
			152
			219
			77
			234
			132
			215
			6
			10
			156
			90
			237
			233
			169
			209
			144
			26
			82
			171
			73
			117
			200
			229
			18
			165
			13
			15
			19
			228
			121
			77
			81
			171
			169
			6
			48
			70
			52
			100
			179
			89
			173
			67
			67
			228
			126
			185
			92
			66
			6
			47
			194
			110
			183
			203
			69
			69
			158
			144
			57
			52
			52
			52
			100
			179
			73
			1
			118
			185
			168
			248
			51
			70
			185
			207
			85
			217
			63
			255
			249
			143
			127
			236
			220
			73
			251
			129
			133
			11
			129
			148
			148
			69
			139
			150
			46
			5
			46
			95
			190
			124
			185
			188
			28
			104
			107
			107
			109
			109
			110
			6
			250
			251
			9
			113
			109
			109
			109
			109
			183
			111
			3
			222
			222
			180
			177
			124
			104
			10
			58
			117
			234
			212
			169
			83
			167
			4
			129
			111
			72
			222
			124
			243
			207
			127
			62
			120
			16
			248
			217
			207
			182
			111
			127
			225
			5
			160
			166
			166
			166
			166
			162
			2
			120
			235
			173
			61
			123
			242
			243
			129
			150
			150
			230
			230
			230
			102
			138
			180
			195
			65
			16
			212
			106
			137
			27
			7
			7
			129
			136
			136
			200
			200
			89
			179
			200
			241
			148
			177
			68
			1
			54
			27
			45
			188
			182
			246
			218
			181
			170
			42
			224
			211
			79
			105
			63
			96
			54
			155
			205
			29
			29
			128
			32
			144
			35
			77
			166
			244
			244
			172
			44
			96
			206
			156
			121
			243
			82
			82
			0
			133
			66
			20
			21
			10
			192
			98
			33
			23
			115
			135
			113
			213
			35
			151
			211
			245
			214
			214
			59
			119
			26
			27
			129
			43
			87
			174
			94
			173
			172
			4
			46
			94
			172
			168
			56
			125
			154
			50
			29
			0
			22
			45
			74
			77
			93
			186
			20
			200
			204
			92
			177
			226
			153
			103
			40
			160
			35
			145
			192
			229
			107
			114
			114
			114
			178
			201
			4
			52
			55
			55
			55
			223
			186
			37
			81
			147
			94
			31
			18
			18
			21
			5
			204
			154
			101
			52
			38
			36
			0
			255
			249
			79
			94
			222
			158
			61
			192
			119
			191
			155
			155
			187
			101
			11
			80
			84
			116
			252
			120
			97
			225
			196
			20
			116
			95
			0
			56
			2
			218
			218
			218
			218
			218
			218
			36
			142
			251
			230
			55
			87
			172
			88
			189
			154
			168
			68
			46
			39
			71
			80
			166
			1
			46
			151
			164
			110
			62
			250
			232
			228
			201
			163
			71
			129
			236
			236
			149
			43
			215
			173
			3
			2
			3
			3
			3
			131
			131
			137
			186
			250
			250
			128
			174
			46
			146
			117
			101
			101
			165
			165
			197
			197
			36
			99
			251
			250
			128
			164
			164
			228
			228
			37
			75
			136
			194
			252
			253
			129
			161
			33
			170
			37
			71
			143
			126
			240
			193
			190
			125
			36
			63
			205
			102
			96
			217
			178
			229
			203
			215
			174
			165
			140
			118
			58
			9
			65
			74
			165
			116
			180
			88
			122
			122
			122
			122
			128
			195
			135
			63
			248
			224
			221
			119
			105
			188
			128
			0
			96
			195
			134
			23
			95
			124
			229
			21
			82
			81
			26
			13
			112
			254
			124
			105
			105
			73
			9
			80
			83
			243
			201
			39
			23
			46
			72
			28
			207
			29
			239
			112
			80
			237
			138
			138
			50
			26
			99
			99
			165
			117
			37
			36
			36
			36
			52
			54
			74
			129
			230
			148
			60
			182
			38
			60
			168
			77
			136
			0
			128
			56
			154
			23
			87
			0
			32
			14
			166
			162
			202
			101
			167
			74
			165
			82
			41
			149
			64
			87
			87
			87
			87
			103
			39
			112
			242
			100
			73
			201
			161
			67
			64
			99
			35
			201
			85
			95
			95
			63
			63
			127
			127
			192
			229
			114
			56
			28
			14
			32
			50
			50
			58
			218
			104
			4
			230
			205
			155
			55
			47
			57
			89
			82
			61
			60
			80
			50
			25
			169
			43
			139
			197
			98
			89
			183
			142
			84
			72
			105
			41
			144
			157
			157
			147
			243
			220
			115
			0
			165
			2
			81
			5
			151
			171
			74
			37
			112
			249
			242
			149
			43
			151
			46
			73
			153
			186
			122
			245
			211
			79
			63
			255
			60
			224
			235
			235
			235
			171
			211
			209
			248
			189
			189
			192
			252
			249
			164
			118
			46
			95
			38
			68
			31
			63
			254
			225
			135
			249
			249
			64
			79
			79
			111
			175
			217
			12
			52
			53
			53
			54
			222
			188
			73
			136
			112
			58
			1
			131
			193
			96
			8
			11
			35
			255
			28
			57
			2
			172
			93
			251
			237
			111
			111
			216
			48
			122
			227
			71
			148
			39
			147
			61
			204
			110
			96
			66
			4
			48
			54
			186
			107
			238
			112
			16
			213
			8
			2
			233
			107
			190
			84
			190
			41
			247
			243
			243
			243
			11
			12
			36
			121
			185
			123
			55
			181
			10
			174
			95
			151
			40
			166
			178
			146
			28
			249
			210
			75
			155
			55
			191
			255
			62
			201
			79
			141
			134
			118
			182
			189
			189
			18
			231
			187
			92
			180
			176
			192
			192
			160
			32
			189
			30
			184
			113
			163
			190
			254
			218
			53
			137
			178
			56
			215
			139
			34
			169
			49
			183
			91
			16
			220
			110
			224
			234
			213
			234
			234
			139
			23
			1
			157
			78
			167
			11
			12
			4
			22
			44
			88
			176
			32
			53
			21
			168
			175
			175
			175
			175
			173
			149
			184
			95
			173
			86
			171
			189
			189
			169
			120
			38
			39
			3
			167
			78
			157
			56
			113
			248
			48
			240
			242
			203
			219
			182
			253
			234
			87
			128
			40
			82
			109
			10
			15
			15
			13
			141
			136
			160
			158
			146
			203
			5
			236
			220
			249
			198
			27
			175
			190
			42
			213
			42
			155
			141
			118
			198
			220
			184
			106
			156
			68
			4
			124
			177
			113
			21
			64
			81
			167
			64
			240
			0
			120
			123
			3
			70
			163
			209
			56
			107
			22
			81
			131
			66
			1
			164
			165
			165
			165
			101
			102
			74
			250
			89
			162
			14
			42
			170
			60
			211
			120
			6
			57
			28
			14
			7
			237
			132
			69
			81
			38
			3
			60
			61
			61
			61
			125
			124
			168
			200
			81
			209
			235
			235
			235
			235
			147
			174
			43
			149
			164
			142
			184
			40
			168
			171
			171
			171
			187
			114
			5
			208
			106
			253
			253
			253
			253
			1
			167
			147
			28
			200
			119
			184
			110
			55
			117
			91
			249
			121
			104
			104
			120
			120
			68
			4
			144
			144
			48
			127
			254
			194
			133
			64
			79
			79
			119
			119
			87
			151
			244
			220
			192
			0
			245
			152
			134
			134
			236
			118
			155
			77
			218
			31
			140
			53
			198
			232
			254
			71
			70
			0
			111
			29
			140
			118
			185
			116
			36
			45
			67
			231
			132
			18
			250
			156
			23
			69
			126
			116
			56
			168
			25
			198
			101
			163
			221
			78
			25
			238
			118
			147
			158
			151
			203
			61
			61
			73
			50
			114
			210
			24
			141
			56
			202
			107
			96
			254
			124
			114
			204
			214
			173
			47
			191
			252
			203
			95
			74
			77
			55
			9
			125
			244
			191
			183
			183
			143
			143
			175
			47
			176
			113
			99
			110
			110
			78
			142
			20
			24
			106
			75
			72
			142
			115
			187
			233
			125
			92
			6
			187
			92
			78
			39
			201
			80
			162
			72
			142
			68
			174
			186
			248
			123
			100
			159
			183
			47
			121
			49
			31
			107
			46
			23
			169
			173
			7
			181
			251
			186
			161
			28
			1
			252
			92
			46
			39
			25
			199
			51
			158
			191
			96
			164
			195
			8
			5
			84
			132
			165
			137
			72
			170
			135
			22
			79
			181
			130
			38
			15
			48
			70
			174
			21
			4
			82
			47
			35
			199
			34
			105
			201
			51
			149
			138
			188
			195
			225
			116
			218
			108
			180
			193
			25
			24
			0
			122
			123
			123
			123
			123
			122
			232
			216
			219
			43
			245
			253
			135
			134
			108
			54
			155
			77
			234
			57
			241
			13
			35
			47
			174
			163
			223
			66
			243
			18
			4
			66
			210
			232
			235
			247
			167
			30
			79
			48
			186
			95
			46
			255
			34
			4
			140
			125
			203
			68
			54
			46
			2
			6
			7
			7
			7
			173
			86
			224
			230
			205
			250
			250
			235
			215
			1
			141
			198
			203
			203
			219
			27
			176
			217
			134
			134
			172
			86
			234
			133
			120
			121
			17
			229
			4
			4
			124
			62
			216
			23
			236
			171
			185
			60
			20
			4
			162
			22
			158
			57
			252
			11
			19
			94
			204
			185
			35
			70
			135
			149
			238
			228
			1
			226
			71
			250
			91
			66
			34
			253
			117
			63
			18
			57
			21
			140
			108
			41
			240
			55
			112
			170
			227
			72
			96
			204
			229
			26
			249
			102
			183
			155
			183
			245
			248
			58
			4
			65
			20
			169
			59
			170
			86
			3
			62
			62
			212
			222
			238
			239
			31
			24
			24
			217
			138
			80
			42
			169
			119
			245
			165
			3
			192
			17
			144
			150
			150
			150
			182
			120
			49
			99
			175
			189
			246
			251
			223
			255
			252
			231
			128
			201
			148
			145
			145
			149
			69
			197
			45
			36
			4
			168
			170
			170
			170
			170
			168
			0
			78
			159
			254
			232
			163
			163
			71
			41
			51
			123
			123
			239
			215
			229
			252
			40
			229
			56
			133
			128
			23
			53
			198
			36
			46
			229
			8
			161
			59
			249
			167
			99
			29
			52
			58
			64
			220
			73
			114
			57
			33
			204
			229
			162
			251
			36
			10
			148
			40
			129
			198
			31
			141
			84
			190
			63
			225
			189
			170
			209
			244
			55
			50
			179
			9
			33
			29
			29
			237
			237
			183
			111
			3
			231
			207
			159
			61
			123
			228
			8
			96
			48
			132
			133
			205
			158
			77
			45
			150
			101
			203
			128
			253
			251
			247
			239
			223
			179
			135
			84
			227
			131
			200
			209
			113
			17
			192
			3
			145
			146
			146
			146
			178
			104
			17
			99
			37
			37
			69
			69
			135
			14
			145
			156
			243
			245
			5
			172
			214
			129
			129
			174
			46
			250
			10
			177
			177
			145
			54
			104
			213
			213
			130
			96
			52
			26
			141
			70
			35
			45
			159
			103
			56
			45
			156
			115
			239
			200
			28
			3
			212
			106
			234
			146
			106
			52
			164
			207
			221
			110
			169
			186
			144
			204
			37
			80
			75
			148
			55
			26
			248
			60
			79
			71
			214
			36
			122
			150
			90
			20
			162
			72
			45
			16
			127
			127
			127
			127
			157
			78
			250
			188
			171
			139
			18
			163
			175
			175
			191
			223
			98
			145
			2
			205
			17
			33
			133
			159
			183
			64
			104
			156
			216
			216
			216
			216
			216
			88
			32
			50
			242
			169
			167
			12
			6
			160
			168
			136
			122
			100
			45
			45
			45
			45
			205
			205
			146
			250
			234
			236
			236
			232
			104
			107
			155
			88
			144
			142
			91
			3
			120
			32
			170
			170
			170
			170
			46
			93
			18
			4
			147
			201
			100
			50
			153
			128
			142
			142
			142
			142
			142
			14
			65
			232
			234
			234
			234
			106
			111
			151
			28
			159
			149
			149
			149
			149
			149
			197
			24
			47
			90
			162
			200
			101
			34
			231
			86
			90
			24
			239
			158
			218
			108
			36
			7
			47
			93
			250
			228
			147
			138
			10
			32
			63
			63
			47
			111
			239
			94
			224
			196
			137
			146
			146
			130
			2
			10
			108
			67
			131
			212
			178
			224
			27
			159
			145
			249
			57
			178
			120
			243
			240
			140
			252
			66
			71
			165
			2
			202
			202
			206
			156
			57
			113
			2
			120
			251
			237
			93
			187
			126
			243
			27
			32
			63
			255
			192
			129
			191
			252
			5
			168
			168
			40
			43
			43
			42
			162
			98
			59
			60
			76
			251
			8
			133
			66
			170
			85
			92
			102
			250
			249
			105
			181
			1
			1
			18
			82
			188
			188
			188
			188
			188
			188
			128
			218
			218
			218
			218
			218
			90
			234
			89
			213
			215
			75
			254
			161
			64
			72
			254
			123
			100
			4
			140
			119
			94
			82
			82
			82
			82
			82
			114
			255
			117
			189
			94
			175
			215
			235
			25
			59
			126
			252
			248
			241
			130
			2
			96
			213
			170
			111
			125
			235
			59
			223
			145
			90
			8
			190
			190
			90
			173
			191
			63
			125
			55
			187
			123
			55
			16
			28
			28
			18
			162
			211
			209
			247
			12
			135
			15
			19
			226
			82
			82
			24
			235
			234
			50
			155
			91
			91
			201
			17
			90
			173
			212
			28
			147
			40
			137
			103
			42
			167
			48
			58
			242
			162
			219
			215
			71
			148
			152
			151
			247
			222
			123
			111
			191
			13
			92
			191
			126
			253
			250
			181
			107
			247
			207
			55
			43
			43
			59
			91
			173
			102
			44
			48
			48
			48
			208
			96
			160
			12
			14
			10
			2
			238
			220
			105
			110
			110
			108
			4
			118
			237
			250
			211
			159
			94
			125
			21
			232
			236
			188
			123
			183
			189
			157
			118
			214
			222
			222
			36
			115
			235
			234
			198
			247
			207
			88
			49
			243
			208
			54
			54
			130
			15
			123
			190
			98
			197
			138
			21
			43
			86
			48
			198
			247
			19
			81
			81
			81
			81
			209
			209
			140
			5
			4
			4
			4
			4
			4
			48
			182
			126
			253
			250
			245
			235
			215
			143
			255
			124
			102
			102
			102
			102
			102
			38
			99
			179
			103
			207
			158
			61
			103
			14
			99
			43
			87
			174
			90
			181
			110
			29
			99
			199
			142
			21
			21
			93
			189
			202
			88
			97
			225
			145
			35
			31
			127
			204
			216
			251
			239
			23
			22
			94
			184
			192
			88
			126
			254
			193
			131
			231
			206
			49
			118
			228
			200
			135
			31
			86
			87
			51
			70
			148
			246
			224
			243
			53
			24
			12
			6
			131
			129
			126
			85
			161
			215
			51
			22
			25
			25
			17
			17
			29
			45
			93
			207
			201
			201
			201
			89
			185
			242
			203
			251
			99
			202
			2
			49
			217
			129
			27
			123
			30
			29
			29
			29
			29
			29
			205
			24
			255
			53
			5
			71
			152
			78
			167
			211
			233
			116
			140
			105
			181
			90
			173
			86
			203
			88
			80
			80
			80
			80
			80
			16
			99
			70
			163
			209
			24
			19
			195
			88
			100
			100
			100
			100
			100
			36
			99
			147
			61
			159
			105
			115
			252
			84
			7
			238
			97
			207
			199
			254
			10
			225
			255
			214
			177
			147
			237
			248
			201
			30
			111
			186
			17
			251
			216
			218
			100
			79
			124
			170
			29
			245
			149
			117
			244
			116
			219
			215
			38
			67
			159
			216
			19
			123
			98
			79
			108
			230
			237
			127
			37
			64
			140
			158
			47
			166
			2
			126
			0
			0
			0
			0
			73
			69
			78
			68
			174
			66
			96
			130
		)! !

!DrGIcons class methodsFor: 'accessing' stamp: 'HilaireFernandes 7/25/2008 09:21'!
macroExecute
	^ self formFromGraphic: #(
			137
			80
			78
			71
			13
			10
			26
			10
			0
			0
			0
			13
			73
			72
			68
			82
			0
			0
			0
			48
			0
			0
			0
			48
			16
			6
			0
			0
			0
			7
			146
			37
			196
			0
			0
			0
			6
			98
			75
			71
			68
			255
			255
			255
			255
			255
			255
			9
			88
			247
			220
			0
			0
			0
			9
			112
			72
			89
			115
			0
			0
			0
			72
			0
			0
			0
			72
			0
			70
			201
			107
			62
			0
			0
			20
			29
			73
			68
			65
			84
			120
			218
			237
			91
			123
			80
			84
			231
			221
			126
			14
			123
			99
			129
			133
			133
			93
			46
			203
			45
			192
			202
			93
			80
			64
			193
			148
			5
			73
			92
			208
			106
			98
			99
			77
			166
			161
			9
			53
			73
			107
			77
			147
			180
			206
			52
			189
			206
			180
			201
			52
			51
			109
			46
			181
			109
			154
			56
			182
			185
			181
			77
			141
			210
			168
			49
			106
			70
			192
			152
			49
			70
			16
			34
			6
			188
			20
			17
			4
			87
			4
			228
			230
			2
			187
			44
			123
			191
			188
			223
			31
			191
			239
			157
			131
			90
			199
			92
			208
			164
			223
			199
			111
			70
			215
			179
			231
			236
			57
			231
			253
			93
			158
			223
			243
			62
			239
			43
			48
			103
			115
			54
			103
			115
			54
			103
			55
			219
			202
			202
			202
			202
			202
			202
			24
			139
			137
			137
			137
			137
			137
			97
			236
			203
			126
			31
			110
			194
			151
			253
			2
			183
			202
			162
			163
			163
			163
			181
			90
			198
			188
			94
			159
			207
			235
			5
			178
			178
			114
			115
			211
			211
			1
			169
			52
			40
			72
			161
			0
			154
			154
			142
			28
			57
			122
			84
			184
			229
			254
			248
			210
			2
			112
			239
			189
			53
			53
			43
			86
			48
			118
			225
			130
			201
			116
			241
			34
			48
			49
			97
			54
			91
			173
			64
			72
			136
			74
			165
			80
			0
			126
			191
			207
			23
			8
			0
			193
			193
			74
			165
			92
			14
			56
			157
			14
			135
			219
			13
			200
			229
			50
			153
			92
			14
			120
			60
			46
			215
			204
			243
			81
			81
			26
			141
			90
			13
			248
			124
			94
			47
			99
			128
			74
			21
			26
			170
			82
			1
			117
			117
			251
			246
			125
			240
			129
			32
			132
			133
			169
			84
			33
			33
			140
			189
			246
			218
			187
			239
			254
			250
			215
			128
			223
			79
			85
			176
			113
			227
			183
			191
			253
			194
			11
			64
			110
			110
			78
			78
			110
			46
			112
			228
			200
			71
			31
			53
			53
			221
			186
			64
			220
			180
			7
			61
			244
			208
			35
			143
			212
			212
			48
			54
			60
			60
			54
			214
			223
			15
			152
			76
			38
			83
			95
			31
			160
			80
			40
			149
			50
			25
			16
			27
			155
			152
			168
			86
			3
			69
			69
			165
			165
			89
			89
			64
			70
			70
			94
			94
			98
			34
			16
			8
			8
			2
			99
			128
			84
			42
			8
			18
			9
			224
			245
			6
			2
			62
			31
			224
			241
			120
			189
			110
			55
			32
			8
			64
			80
			16
			160
			80
			200
			100
			50
			153
			248
			60
			159
			207
			239
			15
			4
			128
			64
			128
			177
			64
			0
			168
			171
			219
			182
			237
			216
			49
			160
			183
			183
			163
			99
			120
			24
			184
			112
			161
			167
			103
			96
			0
			120
			237
			181
			253
			251
			127
			241
			11
			128
			49
			198
			4
			129
			2
			233
			245
			2
			63
			251
			217
			186
			117
			155
			55
			139
			129
			14
			9
			9
			9
			9
			13
			5
			38
			39
			39
			38
			38
			39
			111
			94
			64
			102
			253
			198
			121
			121
			11
			22
			228
			230
			50
			150
			158
			158
			155
			27
			31
			15
			76
			78
			90
			44
			22
			11
			176
			106
			85
			117
			117
			73
			9
			160
			84
			170
			213
			148
			193
			228
			80
			198
			128
			64
			0
			240
			249
			2
			1
			143
			7
			8
			4
			252
			126
			191
			159
			28
			77
			142
			165
			239
			1
			186
			142
			49
			65
			16
			4
			64
			34
			97
			76
			38
			3
			130
			130
			40
			80
			140
			9
			130
			223
			15
			72
			36
			130
			64
			223
			3
			82
			41
			48
			54
			54
			56
			232
			112
			0
			111
			189
			181
			121
			115
			125
			61
			240
			228
			147
			155
			54
			221
			119
			159
			248
			59
			110
			140
			5
			2
			130
			0
			120
			189
			110
			183
			199
			3
			188
			250
			234
			51
			207
			28
			56
			0
			92
			186
			116
			241
			226
			232
			40
			224
			116
			58
			157
			78
			231
			236
			7
			66
			58
			91
			55
			90
			184
			176
			160
			32
			63
			159
			177
			236
			236
			194
			194
			228
			100
			192
			104
			92
			179
			102
			225
			66
			202
			104
			159
			15
			176
			88
			236
			246
			203
			151
			129
			137
			9
			155
			109
			116
			20
			240
			251
			201
			209
			62
			31
			7
			3
			126
			236
			241
			80
			38
			123
			189
			228
			112
			198
			24
			3
			4
			129
			28
			47
			8
			65
			65
			220
			97
			0
			32
			145
			72
			165
			244
			13
			55
			186
			78
			38
			35
			168
			106
			106
			58
			112
			224
			236
			89
			64
			46
			87
			42
			37
			18
			192
			229
			10
			4
			166
			167
			1
			175
			215
			227
			113
			185
			0
			175
			215
			229
			242
			249
			0
			192
			235
			245
			251
			1
			198
			130
			130
			4
			1
			8
			4
			168
			242
			110
			150
			227
			103
			61
			0
			163
			163
			99
			99
			35
			35
			128
			209
			168
			215
			207
			159
			15
			216
			108
			14
			199
			228
			36
			224
			112
			120
			60
			102
			51
			96
			181
			78
			77
			77
			79
			3
			140
			249
			124
			140
			1
			19
			19
			163
			163
			118
			59
			208
			217
			249
			201
			39
			35
			35
			192
			229
			203
			151
			46
			77
			77
			1
			86
			235
			248
			184
			211
			9
			168
			213
			26
			141
			74
			69
			16
			225
			243
			1
			26
			77
			116
			116
			100
			36
			32
			151
			43
			20
			50
			25
			112
			249
			242
			232
			232
			196
			4
			160
			84
			134
			134
			42
			20
			128
			66
			161
			80
			72
			165
			228
			88
			191
			31
			240
			120
			60
			30
			159
			15
			232
			235
			235
			233
			185
			116
			9
			48
			24
			238
			185
			39
			51
			19
			112
			58
			109
			54
			155
			13
			112
			187
			109
			54
			183
			27
			240
			120
			28
			14
			159
			15
			216
			189
			251
			245
			215
			219
			219
			129
			168
			168
			232
			232
			136
			8
			192
			100
			58
			119
			110
			112
			240
			230
			247
			130
			89
			11
			128
			195
			97
			183
			59
			157
			64
			120
			120
			92
			156
			32
			0
			253
			253
			151
			46
			153
			76
			192
			217
			179
			109
			109
			67
			67
			64
			103
			103
			107
			235
			208
			16
			48
			53
			53
			49
			225
			114
			1
			145
			145
			106
			181
			90
			13
			100
			100
			100
			101
			165
			167
			3
			205
			205
			29
			29
			221
			221
			226
			128
			199
			198
			134
			135
			45
			22
			241
			254
			125
			125
			189
			189
			195
			195
			159
			253
			189
			162
			162
			98
			98
			84
			42
			198
			10
			11
			13
			134
			228
			100
			192
			233
			156
			156
			116
			58
			129
			237
			219
			255
			252
			231
			99
			199
			168
			82
			100
			50
			160
			191
			191
			183
			119
			124
			92
			16
			232
			243
			102
			187
			125
			22
			173
			172
			172
			172
			204
			96
			96
			44
			58
			58
			38
			70
			163
			97
			236
			254
			251
			191
			255
			125
			163
			145
			49
			189
			62
			55
			55
			62
			158
			49
			163
			113
			245
			234
			146
			146
			47
			143
			119
			199
			196
			232
			116
			26
			13
			99
			149
			149
			107
			215
			46
			92
			200
			88
			86
			86
			65
			65
			66
			194
			87
			103
			30
			48
			11
			3
			164
			137
			77
			126
			126
			126
			126
			94
			30
			99
			21
			21
			119
			222
			89
			94
			254
			213
			25
			96
			118
			118
			78
			78
			86
			22
			99
			139
			23
			47
			89
			82
			80
			240
			213
			121
			175
			57
			155
			179
			57
			155
			179
			57
			155
			179
			47
			77
			11
			202
			207
			207
			207
			47
			40
			96
			44
			33
			33
			41
			41
			45
			13
			8
			15
			15
			15
			215
			104
			128
			193
			193
			129
			129
			238
			110
			154
			104
			121
			189
			64
			115
			115
			115
			115
			115
			243
			141
			249
			120
			66
			66
			66
			66
			66
			2
			99
			113
			113
			58
			93
			98
			162
			56
			179
			181
			90
			173
			86
			139
			5
			24
			24
			24
			24
			48
			153
			128
			226
			226
			226
			226
			146
			18
			160
			169
			169
			169
			169
			177
			241
			214
			139
			111
			87
			219
			172
			205
			3
			62
			173
			37
			37
			37
			37
			37
			37
			49
			86
			86
			182
			116
			233
			202
			149
			192
			188
			121
			25
			25
			249
			249
			128
			68
			34
			145
			72
			36
			128
			205
			102
			179
			21
			23
			3
			111
			188
			241
			215
			191
			62
			247
			220
			141
			239
			183
			97
			195
			134
			13
			27
			54
			48
			214
			210
			210
			220
			124
			252
			56
			240
			232
			163
			143
			63
			254
			244
			211
			192
			212
			20
			57
			222
			239
			167
			25
			237
			200
			200
			240
			240
			224
			32
			240
			143
			127
			188
			241
			198
			166
			77
			215
			191
			95
			110
			110
			110
			110
			110
			46
			99
			46
			151
			203
			229
			114
			241
			99
			96
			223
			190
			125
			251
			246
			237
			155
			253
			128
			5
			125
			222
			31
			214
			212
			212
			212
			212
			212
			48
			150
			147
			147
			147
			147
			147
			195
			88
			108
			108
			92
			92
			124
			60
			99
			223
			251
			222
			250
			245
			143
			62
			122
			125
			186
			151
			158
			158
			158
			158
			147
			3
			244
			244
			244
			246
			118
			117
			1
			93
			93
			103
			206
			156
			56
			1
			216
			108
			83
			83
			86
			43
			160
			80
			200
			229
			74
			37
			176
			112
			97
			97
			225
			146
			37
			64
			101
			101
			101
			101
			101
			229
			245
			239
			215
			219
			219
			219
			219
			219
			11
			100
			102
			102
			101
			229
			229
			137
			247
			177
			219
			237
			246
			233
			105
			82
			71
			125
			62
			64
			173
			142
			136
			136
			140
			4
			130
			130
			130
			130
			102
			138
			120
			220
			170
			171
			171
			171
			171
			171
			25
			115
			56
			28
			14
			135
			131
			42
			165
			188
			156
			102
			218
			145
			145
			64
			65
			65
			65
			65
			97
			33
			99
			89
			89
			89
			89
			89
			89
			140
			85
			85
			85
			85
			85
			85
			125
			113
			90
			251
			169
			35
			186
			126
			253
			250
			245
			235
			215
			51
			86
			95
			95
			95
			95
			95
			15
			196
			197
			197
			197
			37
			36
			0
			171
			87
			223
			115
			207
			195
			15
			3
			102
			243
			196
			196
			232
			40
			176
			99
			71
			109
			237
			43
			175
			0
			6
			131
			193
			80
			90
			10
			236
			218
			181
			115
			231
			206
			157
			98
			230
			228
			231
			47
			88
			80
			80
			192
			152
			90
			29
			25
			25
			31
			15
			44
			93
			90
			81
			177
			98
			5
			16
			28
			28
			28
			28
			18
			66
			14
			34
			181
			83
			161
			80
			42
			129
			142
			142
			127
			255
			187
			181
			21
			56
			127
			254
			252
			249
			174
			46
			192
			227
			113
			187
			221
			110
			224
			242
			229
			203
			151
			135
			135
			129
			59
			238
			184
			227
			142
			85
			171
			128
			204
			204
			236
			236
			5
			11
			0
			158
			185
			0
			9
			125
			188
			178
			134
			134
			46
			93
			26
			24
			0
			118
			237
			218
			177
			227
			213
			87
			1
			185
			92
			46
			151
			206
			168
			127
			181
			58
			50
			82
			171
			5
			86
			173
			186
			251
			238
			7
			31
			4
			116
			58
			157
			46
			57
			25
			80
			40
			130
			131
			131
			131
			1
			137
			132
			222
			203
			225
			112
			58
			29
			14
			224
			119
			191
			251
			205
			111
			126
			248
			67
			96
			222
			188
			244
			116
			189
			30
			104
			108
			108
			108
			252
			60
			144
			118
			195
			31
			240
			204
			56
			114
			228
			200
			145
			166
			38
			64
			171
			141
			141
			77
			72
			0
			158
			120
			226
			137
			39
			158
			126
			26
			176
			217
			166
			167
			173
			86
			146
			34
			166
			167
			129
			214
			214
			99
			199
			14
			31
			6
			172
			214
			201
			201
			203
			151
			73
			245
			116
			58
			129
			209
			209
			209
			209
			209
			81
			192
			231
			243
			249
			124
			62
			160
			162
			130
			28
			183
			104
			209
			226
			197
			229
			229
			36
			126
			145
			42
			234
			247
			251
			124
			128
			32
			144
			220
			38
			145
			144
			168
			230
			116
			58
			28
			118
			59
			96
			177
			88
			173
			19
			19
			64
			72
			8
			85
			138
			86
			27
			19
			19
			27
			11
			216
			237
			78
			167
			221
			46
			138
			116
			60
			144
			65
			65
			18
			73
			80
			16
			73
			14
			193
			193
			128
			217
			108
			54
			15
			15
			147
			200
			102
			183
			3
			161
			161
			180
			110
			16
			17
			17
			30
			174
			86
			3
			110
			55
			5
			152
			191
			143
			217
			108
			54
			143
			142
			2
			219
			182
			253
			243
			159
			155
			55
			3
			243
			231
			231
			231
			23
			21
			1
			183
			223
			126
			251
			237
			21
			21
			64
			109
			237
			91
			111
			109
			217
			2
			156
			63
			111
			50
			245
			246
			222
			132
			0
			148
			151
			151
			151
			151
			151
			51
			102
			52
			46
			95
			94
			93
			13
			52
			52
			212
			213
			237
			218
			69
			16
			114
			246
			44
			176
			110
			221
			186
			117
			27
			55
			2
			49
			49
			177
			177
			58
			29
			105
			145
			124
			192
			114
			57
			32
			149
			74
			165
			82
			41
			137
			104
			193
			193
			228
			24
			137
			132
			22
			62
			234
			235
			129
			75
			151
			6
			7
			251
			250
			0
			139
			101
			98
			98
			108
			12
			232
			237
			237
			233
			233
			236
			4
			148
			202
			144
			144
			224
			96
			186
			143
			82
			201
			181
			82
			32
			56
			152
			196
			183
			208
			80
			202
			76
			191
			159
			86
			184
			166
			167
			237
			118
			10
			0
			73
			209
			110
			183
			219
			237
			116
			146
			236
			204
			24
			16
			22
			22
			22
			22
			30
			14
			164
			165
			233
			245
			217
			217
			128
			76
			38
			151
			135
			132
			0
			165
			165
			6
			67
			101
			37
			160
			84
			42
			149
			33
			33
			128
			195
			65
			129
			230
			234
			107
			87
			87
			87
			215
			233
			211
			192
			238
			221
			59
			119
			190
			254
			58
			176
			105
			211
			139
			47
			110
			223
			78
			129
			11
			15
			7
			94
			120
			225
			217
			103
			127
			244
			35
			160
			164
			164
			184
			184
			184
			24
			56
			124
			248
			163
			143
			154
			155
			249
			56
			110
			28
			144
			27
			94
			80
			90
			90
			90
			90
			90
			202
			216
			67
			15
			125
			247
			187
			191
			252
			37
			101
			112
			32
			0
			188
			255
			254
			129
			3
			239
			188
			3
			72
			165
			18
			137
			84
			10
			20
			22
			46
			90
			100
			48
			0
			94
			175
			215
			203
			245
			123
			64
			28
			136
			207
			71
			46
			252
			253
			239
			159
			123
			238
			39
			63
			1
			220
			110
			130
			138
			167
			159
			126
			230
			153
			151
			94
			2
			66
			66
			148
			202
			176
			48
			114
			68
			112
			48
			160
			209
			104
			181
			58
			157
			168
			126
			38
			36
			232
			116
			90
			45
			144
			149
			149
			153
			153
			146
			2
			132
			135
			171
			84
			82
			41
			173
			17
			0
			192
			249
			243
			189
			189
			99
			99
			192
			240
			240
			216
			216
			196
			4
			96
			183
			79
			79
			59
			28
			192
			200
			8
			85
			158
			217
			76
			144
			37
			149
			74
			36
			114
			57
			176
			125
			251
			182
			109
			175
			188
			2
			180
			183
			183
			181
			53
			55
			3
			247
			222
			123
			223
			125
			143
			60
			66
			189
			231
			107
			95
			35
			25
			60
			16
			16
			223
			255
			248
			241
			214
			214
			198
			70
			145
			77
			121
			60
			30
			143
			219
			77
			149
			111
			179
			137
			227
			173
			169
			89
			183
			238
			7
			63
			0
			182
			110
			125
			243
			205
			191
			252
			229
			198
			129
			184
			33
			11
			146
			72
			164
			82
			133
			2
			168
			175
			175
			171
			219
			185
			147
			180
			159
			164
			36
			202
			100
			169
			20
			200
			201
			201
			205
			45
			44
			4
			194
			194
			84
			170
			240
			112
			17
			163
			125
			62
			202
			76
			191
			159
			32
			71
			16
			104
			1
			164
			164
			100
			201
			146
			138
			10
			224
			228
			201
			246
			246
			230
			102
			96
			239
			222
			61
			123
			182
			110
			37
			172
			150
			203
			137
			78
			222
			118
			27
			101
			230
			93
			119
			209
			113
			98
			162
			216
			60
			121
			2
			120
			60
			94
			47
			189
			33
			53
			66
			198
			130
			130
			24
			163
			37
			75
			153
			12
			24
			30
			182
			90
			39
			39
			129
			211
			167
			79
			156
			104
			109
			5
			90
			90
			154
			155
			63
			252
			144
			42
			82
			34
			1
			122
			122
			206
			157
			235
			232
			0
			230
			207
			207
			203
			43
			44
			4
			98
			99
			227
			227
			147
			146
			196
			113
			211
			42
			4
			245
			34
			133
			2
			48
			26
			171
			170
			238
			185
			135
			122
			135
			66
			33
			94
			231
			243
			249
			124
			30
			143
			216
			123
			84
			42
			242
			195
			244
			180
			205
			54
			49
			241
			5
			32
			40
			46
			46
			46
			46
			46
			142
			177
			196
			68
			226
			233
			15
			60
			240
			157
			239
			108
			220
			72
			52
			113
			114
			146
			154
			99
			123
			59
			112
			234
			212
			137
			19
			31
			127
			76
			37
			173
			80
			16
			219
			136
			136
			160
			166
			88
			80
			0
			164
			166
			166
			166
			102
			101
			17
			148
			72
			165
			128
			84
			202
			49
			157
			50
			199
			239
			103
			204
			239
			167
			158
			49
			49
			1
			180
			180
			28
			61
			90
			87
			7
			156
			59
			119
			238
			92
			111
			47
			65
			201
			204
			166
			42
			149
			74
			165
			114
			57
			45
			204
			4
			2
			148
			209
			50
			25
			65
			135
			211
			9
			16
			234
			19
			12
			49
			38
			66
			163
			193
			176
			116
			233
			242
			229
			64
			78
			78
			118
			118
			126
			62
			85
			170
			223
			47
			54
			251
			64
			32
			16
			240
			251
			105
			61
			193
			227
			161
			241
			200
			229
			192
			197
			139
			23
			46
			116
			119
			3
			135
			14
			125
			248
			225
			254
			253
			51
			33
			138
			254
			24
			12
			101
			101
			70
			35
			144
			147
			147
			147
			179
			104
			17
			48
			60
			60
			58
			218
			215
			7
			188
			250
			234
			150
			45
			207
			63
			79
			149
			226
			241
			92
			191
			2
			174
			57
			193
			105
			95
			95
			95
			95
			95
			95
			31
			97
			111
			80
			144
			136
			201
			25
			25
			233
			233
			249
			249
			180
			182
			27
			26
			10
			36
			38
			38
			39
			167
			166
			82
			101
			36
			36
			0
			135
			15
			31
			58
			180
			111
			31
			5
			168
			173
			13
			88
			185
			114
			213
			170
			251
			239
			7
			146
			146
			232
			58
			198
			2
			1
			98
			39
			188
			55
			80
			224
			148
			74
			90
			131
			13
			14
			14
			14
			86
			42
			137
			117
			208
			154
			48
			65
			154
			219
			77
			37
			207
			123
			138
			82
			73
			61
			128
			47
			105
			58
			157
			14
			135
			203
			69
			238
			167
			165
			70
			202
			96
			222
			132
			3
			1
			90
			51
			230
			149
			233
			114
			185
			92
			78
			167
			24
			96
			191
			159
			154
			63
			95
			129
			227
			172
			236
			239
			127
			255
			219
			223
			54
			109
			162
			249
			192
			162
			69
			64
			81
			209
			226
			197
			101
			101
			192
			201
			147
			39
			79
			54
			55
			3
			67
			67
			131
			131
			253
			253
			128
			205
			70
			21
			55
			52
			52
			52
			116
			225
			2
			160
			82
			209
			196
			242
			51
			67
			208
			193
			131
			7
			15
			30
			60
			40
			8
			124
			66
			242
			194
			11
			127
			252
			227
			142
			29
			192
			143
			127
			188
			113
			227
			253
			247
			3
			39
			78
			156
			56
			209
			210
			2
			188
			244
			210
			150
			45
			181
			181
			192
			192
			64
			127
			127
			127
			63
			69
			218
			235
			165
			18
			84
			171
			9
			27
			237
			118
			32
			57
			57
			37
			101
			222
			60
			114
			60
			101
			44
			65
			128
			211
			73
			3
			63
			115
			166
			163
			163
			173
			13
			56
			123
			150
			230
			3
			102
			179
			217
			60
			50
			2
			8
			2
			57
			210
			96
			40
			45
			53
			26
			129
			236
			236
			249
			243
			139
			138
			0
			153
			76
			42
			149
			201
			0
			171
			149
			92
			204
			29
			198
			89
			143
			68
			66
			231
			7
			7
			47
			94
			52
			153
			128
			83
			167
			78
			159
			110
			109
			5
			142
			29
			107
			105
			57
			116
			136
			50
			29
			0
			22
			47
			46
			46
			46
			43
			3
			42
			42
			150
			45
			187
			251
			110
			10
			232
			204
			74
			224
			244
			181
			176
			176
			176
			208
			96
			0
			250
			251
			251
			251
			207
			159
			23
			161
			73
			167
			139
			139
			75
			77
			5
			230
			205
			211
			235
			243
			242
			128
			127
			253
			107
			251
			246
			45
			91
			128
			111
			125
			171
			186
			122
			195
			6
			160
			174
			110
			255
			254
			93
			187
			110
			12
			65
			215
			4
			128
			87
			192
			208
			208
			208
			208
			208
			144
			136
			113
			119
			222
			185
			108
			217
			202
			149
			4
			37
			18
			9
			57
			130
			47
			106
			251
			253
			34
			187
			249
			224
			131
			247
			223
			223
			187
			23
			168
			172
			92
			190
			124
			205
			26
			218
			143
			19
			27
			75
			208
			53
			53
			5
			140
			143
			19
			173
			107
			108
			60
			124
			184
			190
			158
			104
			236
			212
			20
			80
			80
			80
			88
			120
			251
			237
			4
			97
			81
			81
			128
			203
			69
			189
			100
			239
			222
			119
			223
			125
			227
			13
			162
			159
			102
			51
			176
			116
			233
			29
			119
			172
			94
			77
			25
			237
			243
			81
			5
			201
			229
			226
			167
			213
			58
			57
			57
			57
			9
			236
			222
			253
			238
			187
			111
			190
			73
			247
			211
			104
			128
			7
			30
			120
			240
			193
			199
			31
			39
			22
			21
			26
			10
			52
			53
			29
			62
			220
			208
			0
			156
			56
			241
			201
			39
			71
			143
			138
			24
			207
			29
			239
			245
			82
			239
			74
			77
			213
			235
			51
			51
			197
			113
			229
			229
			229
			229
			153
			76
			98
			160
			57
			36
			95
			221
			19
			62
			173
			221
			176
			2
			0
			194
			104
			222
			92
			1
			128
			48
			152
			154
			42
			167
			157
			10
			133
			66
			33
			151
			3
			227
			227
			227
			227
			163
			163
			192
			251
			239
			55
			52
			188
			243
			14
			96
			50
			17
			93
			141
			136
			136
			140
			140
			138
			2
			252
			126
			175
			215
			235
			5
			82
			82
			210
			210
			244
			122
			96
			254
			252
			249
			243
			11
			11
			69
			214
			195
			3
			21
			20
			68
			236
			202
			106
			181
			90
			215
			172
			33
			22
			114
			248
			48
			80
			89
			89
			85
			117
			239
			189
			0
			165
			2
			65
			5
			167
			171
			114
			57
			112
			242
			228
			169
			83
			199
			143
			139
			153
			186
			114
			229
			93
			119
			221
			119
			31
			16
			17
			17
			17
			161
			213
			210
			253
			45
			22
			96
			193
			2
			98
			59
			39
			79
			82
			69
			239
			223
			255
			222
			123
			181
			181
			180
			139
			195
			108
			6
			250
			250
			76
			166
			158
			30
			113
			113
			62
			62
			62
			62
			62
			49
			145
			252
			179
			103
			15
			176
			122
			245
			55
			190
			241
			192
			3
			87
			78
			252
			8
			242
			104
			81
			255
			115
			7
			224
			234
			10
			160
			29
			9
			226
			121
			175
			151
			160
			70
			16
			136
			95
			243
			161
			242
			73
			121
			100
			100
			100
			100
			116
			52
			209
			203
			205
			155
			73
			42
			232
			236
			20
			33
			166
			181
			149
			28
			249
			240
			195
			235
			215
			191
			253
			54
			209
			207
			208
			80
			154
			217
			90
			44
			34
			230
			251
			253
			52
			176
			232
			232
			152
			24
			157
			14
			56
			119
			174
			187
			187
			163
			67
			132
			44
			142
			245
			82
			41
			177
			177
			64
			64
			16
			2
			1
			224
			244
			233
			246
			246
			99
			199
			0
			173
			86
			171
			141
			142
			6
			22
			46
			92
			184
			176
			184
			24
			232
			238
			238
			238
			62
			115
			70
			196
			126
			165
			82
			169
			84
			169
			168
			121
			22
			22
			2
			7
			15
			30
			56
			176
			123
			55
			240
			216
			99
			79
			60
			241
			243
			159
			3
			82
			41
			245
			166
			164
			164
			132
			132
			228
			100
			210
			148
			252
			126
			96
			211
			166
			231
			159
			127
			234
			41
			177
			87
			57
			157
			52
			51
			230
			198
			89
			227
			44
			86
			192
			127
			54
			206
			2
			40
			234
			20
			8
			30
			0
			149
			10
			208
			235
			245
			250
			121
			243
			8
			26
			100
			50
			160
			164
			164
			164
			164
			162
			66
			228
			207
			34
			116
			80
			83
			229
			153
			198
			51
			200
			235
			165
			109
			41
			124
			219
			73
			72
			72
			72
			72
			120
			56
			53
			57
			106
			122
			83
			83
			83
			83
			226
			121
			185
			156
			216
			17
			39
			5
			93
			93
			93
			93
			167
			78
			1
			106
			117
			84
			84
			84
			20
			237
			47
			242
			251
			197
			25
			110
			32
			64
			106
			43
			63
			78
			72
			72
			74
			74
			78
			166
			125
			77
			139
			22
			209
			134
			172
			241
			113
			241
			119
			211
			211
			164
			49
			185
			92
			30
			143
			211
			41
			206
			15
			174
			54
			198
			232
			250
			47
			92
			1
			92
			58
			184
			210
			229
			226
			39
			113
			25
			58
			230
			251
			118
			120
			61
			204
			108
			142
			94
			47
			137
			97
			156
			54
			122
			60
			148
			225
			129
			0
			241
			121
			137
			36
			36
			132
			40
			35
			7
			141
			43
			43
			142
			242
			26
			88
			176
			128
			28
			243
			232
			163
			143
			61
			246
			211
			159
			138
			162
			155
			88
			125
			244
			183
			74
			21
			30
			30
			17
			1
			212
			212
			84
			87
			87
			85
			137
			129
			33
			89
			66
			116
			92
			32
			64
			207
			227
			52
			216
			239
			247
			249
			136
			134
			18
			68
			242
			74
			228
			172
			139
			63
			135
			239
			65
			226
			205
			252
			106
			243
			251
			137
			109
			125
			90
			187
			70
			13
			229
			21
			192
			143
			37
			18
			162
			113
			60
			227
			249
			3
			102
			58
			140
			170
			128
			154
			176
			248
			34
			34
			235
			161
			193
			243
			45
			85
			226
			14
			55
			154
			105
			18
			123
			153
			121
			47
			162
			150
			60
			83
			169
			201
			123
			189
			62
			159
			211
			73
			19
			156
			233
			105
			192
			98
			177
			88
			38
			39
			233
			211
			98
			17
			117
			127
			151
			203
			233
			116
			58
			69
			205
			137
			79
			24
			121
			115
			189
			242
			41
			244
			94
			180
			211
			78
			36
			21
			87
			219
			204
			132
			224
			91
			26
			37
			18
			137
			228
			63
			85
			192
			213
			79
			185
			145
			93
			183
			2
			236
			118
			187
			221
			225
			0
			122
			122
			186
			187
			59
			59
			129
			208
			208
			176
			48
			149
			10
			112
			58
			93
			46
			135
			131
			180
			144
			176
			48
			130
			28
			141
			230
			127
			111
			246
			31
			230
			213
			156
			30
			10
			130
			184
			227
			140
			6
			66
			11
			38
			188
			153
			115
			71
			92
			25
			86
			186
			146
			7
			136
			127
			210
			191
			197
			74
			164
			127
			93
			91
			137
			28
			10
			102
			74
			10
			252
			9
			28
			234
			120
			37
			48
			198
			119
			232
			241
			223
			112
			89
			143
			143
			67
			16
			164
			82
			82
			71
			149
			74
			32
			60
			156
			228
			109
			155
			109
			122
			122
			166
			20
			33
			151
			147
			118
			245
			185
			3
			192
			43
			160
			164
			164
			164
			100
			201
			18
			198
			158
			125
			246
			183
			191
			125
			242
			73
			192
			96
			40
			47
			55
			26
			169
			185
			197
			197
			1
			109
			109
			109
			109
			45
			45
			192
			161
			67
			31
			124
			176
			119
			47
			101
			166
			197
			114
			45
			47
			231
			159
			98
			142
			83
			8
			120
			83
			227
			123
			61
			105
			144
			51
			115
			147
			127
			123
			181
			131
			174
			12
			16
			119
			146
			68
			66
			21
			230
			247
			211
			117
			34
			4
			138
			144
			64
			247
			191
			178
			82
			249
			252
			132
			107
			85
			87
			194
			223
			204
			204
			166
			10
			25
			25
			25
			30
			190
			112
			1
			104
			106
			250
			232
			163
			61
			123
			128
			248
			248
			196
			196
			140
			12
			146
			88
			150
			46
			5
			182
			110
			221
			186
			117
			203
			22
			98
			141
			159
			134
			142
			94
			183
			2
			120
			32
			138
			138
			138
			138
			22
			47
			102
			172
			161
			161
			174
			238
			157
			119
			136
			206
			69
			68
			0
			14
			199
			244
			244
			248
			56
			45
			33
			154
			76
			52
			65
			107
			111
			23
			4
			189
			94
			175
			215
			235
			105
			248
			60
			195
			105
			224
			28
			123
			103
			230
			24
			160
			84
			146
			74
			26
			26
			74
			252
			60
			16
			16
			187
			11
			209
			92
			42
			106
			17
			242
			174
			44
			124
			158
			167
			51
			123
			18
			253
			150
			36
			10
			169
			148
			36
			144
			168
			168
			168
			40
			173
			86
			252
			126
			124
			156
			18
			99
			106
			202
			102
			179
			90
			197
			64
			243
			138
			16
			195
			207
			37
			16
			186
			79
			102
			102
			102
			102
			102
			38
			144
			146
			114
			219
			109
			241
			241
			64
			93
			29
			105
			100
			3
			3
			3
			3
			253
			253
			34
			251
			26
			29
			29
			25
			25
			26
			186
			49
			33
			189
			110
			15
			224
			129
			104
			107
			107
			107
			59
			126
			92
			16
			12
			6
			131
			193
			96
			0
			70
			70
			70
			70
			70
			70
			4
			97
			124
			124
			124
			124
			120
			88
			116
			188
			209
			104
			52
			26
			141
			140
			241
			166
			37
			149
			114
			154
			200
			177
			149
			6
			198
			213
			83
			167
			147
			232
			224
			241
			227
			159
			124
			210
			210
			2
			212
			214
			110
			223
			254
			202
			43
			192
			129
			3
			13
			13
			59
			119
			82
			96
			123
			123
			69
			201
			130
			79
			124
			102
			230
			231
			204
			230
			205
			195
			51
			115
			65
			71
			161
			0
			26
			27
			63
			252
			240
			192
			1
			224
			229
			151
			95
			124
			241
			87
			191
			2
			106
			107
			183
			109
			251
			211
			159
			128
			150
			150
			198
			198
			186
			58
			106
			182
			110
			183
			184
			69
			145
			247
			42
			78
			51
			35
			35
			213
			106
			141
			70
			172
			148
			176
			176
			176
			176
			176
			48
			224
			204
			153
			51
			103
			206
			156
			33
			205
			170
			187
			91
			244
			15
			5
			66
			244
			223
			23
			174
			128
			235
			29
			55
			52
			52
			52
			52
			52
			92
			123
			94
			167
			211
			233
			116
			58
			198
			246
			239
			223
			191
			127
			231
			78
			96
			197
			138
			175
			127
			253
			155
			223
			20
			37
			132
			136
			8
			181
			58
			42
			138
			214
			102
			55
			111
			6
			98
			99
			227
			226
			180
			90
			90
			103
			216
			189
			155
			42
			174
			168
			136
			177
			241
			113
			179
			121
			112
			144
			28
			161
			86
			139
			226
			152
			8
			73
			87
			238
			154
			230
			21
			198
			155
			238
			212
			20
			65
			226
			246
			237
			111
			189
			245
			242
			203
			64
			103
			103
			103
			103
			71
			199
			181
			239
			107
			52
			86
			86
			42
			149
			140
			69
			71
			71
			71
			199
			199
			83
			6
			199
			196
			0
			23
			47
			246
			247
			155
			76
			192
			139
			47
			254
			225
			15
			79
			61
			69
			155
			143
			135
			135
			105
			102
			173
			82
			17
			205
			237
			234
			186
			190
			127
			174
			38
			51
			159
			217
			174
			142
			224
			103
			61
			94
			182
			108
			217
			178
			101
			203
			24
			227
			243
			137
			212
			212
			212
			212
			180
			52
			198
			52
			26
			141
			70
			163
			97
			108
			237
			218
			181
			107
			215
			174
			189
			254
			239
			43
			42
			42
			42
			42
			42
			24
			203
			200
			200
			200
			200
			206
			102
			108
			249
			242
			21
			43
			214
			172
			97
			108
			223
			190
			186
			186
			211
			167
			25
			219
			181
			107
			207
			158
			143
			63
			102
			236
			237
			183
			119
			237
			58
			122
			148
			177
			218
			218
			29
			59
			142
			28
			97
			108
			207
			158
			247
			222
			107
			111
			103
			140
			32
			237
			211
			191
			111
			124
			124
			124
			124
			124
			60
			237
			170
			208
			233
			24
			75
			73
			73
			78
			78
			75
			19
			207
			87
			85
			85
			85
			45
			95
			254
			249
			253
			113
			211
			2
			49
			219
			129
			187
			250
			56
			45
			45
			45
			45
			45
			141
			49
			190
			155
			130
			87
			152
			86
			171
			213
			106
			181
			140
			169
			213
			106
			181
			90
			45
			254
			231
			59
			189
			94
			175
			79
			79
			103
			44
			37
			37
			37
			37
			37
			133
			177
			217
			126
			159
			91
			230
			248
			155
			29
			184
			207
			122
			124
			245
			46
			132
			255
			183
			142
			157
			109
			199
			207
			246
			253
			110
			117
			197
			126
			101
			109
			182
			95
			252
			102
			59
			234
			191
			214
			209
			183
			218
			254
			207
			100
			232
			156
			205
			217
			156
			205
			217
			151
			111
			255
			3
			80
			244
			143
			206
			188
			145
			181
			235
			0
			0
			0
			0
			73
			69
			78
			68
			174
			66
			96
			130
		)! !

!DrGIcons class methodsFor: 'helpers' stamp: 'HilaireFernandes 10/29/2010 10:22'!
menuExtent
	^ 60@12! !

!DrGIcons class methodsFor: 'helpers' stamp: 'HilaireFernandes 10/29/2010 10:48'!
menuInset
	^ 4! !

!DrGIcons class methodsFor: 'accessing' stamp: 'HilaireFernandes 11/28/2005 08:54'!
middle
	^self formFromGraphic: #(137 80 78 71 13 10 26 10 0 0 0 13 73 72 68 82 0 0 0 48 0 0 0 48 16 6 0 0 0 7 146 37 196 0 0 0 6 98 75 71 68 255 255 255 255 255 255 9 88 247 220 0 0 0 9 112 72 89 115 0 0 0 72 0 0 0 72 0 70 201 107 62 0 0 1 193 73 68 65 84 120 218 237 154 75 170 131 48 20 134 79 46 119 168 139 112 224 182 164 224 172 148 186 153 174 197 81 119 210 81 161 123 232 68 200 29 200 69 140 143 228 228 97 212 252 223 36 40 88 234 255 229 101 56 68 0 0 0 0 56 35 82 142 91 160 242 19 230 103 17 184 41 191 118 143 45 5 44 68 236 23 58 26 158 71 64 44 49 199 29 113 129 166 32 96 138 165 0 243 30 93 215 117 93 215 67 15 85 175 185 84 85 85 85 213 113 123 188 103 212 93 206 114 235 26 252 63 243 2 146 19 98 30 188 218 134 25 1 199 21 192 156 130 76 95 116 58 69 189 94 101 89 150 68 143 71 150 101 89 236 215 222 15 158 23 97 33 198 61 92 136 203 37 207 243 156 232 253 238 186 174 35 250 124 218 182 109 137 190 223 166 105 154 227 246 220 72 44 77 45 250 197 182 23 33 229 243 89 20 69 33 101 63 34 164 212 137 88 159 114 116 109 2 232 130 143 35 34 1 184 193 135 17 145 160 0 215 224 213 158 204 21 113 182 93 144 53 220 15 44 53 184 48 34 84 78 40 230 126 191 221 174 87 251 47 91 223 34 230 73 104 74 178 37 140 8 4 207 198 143 136 105 240 56 51 98 194 21 209 223 69 240 222 209 137 232 25 130 183 91 35 128 22 93 240 126 22 107 96 200 16 124 152 93 19 88 96 57 120 136 8 138 121 240 16 17 8 110 240 16 17 8 221 182 18 34 118 2 68 236 4 156 53 237 132 109 206 154 32 64 139 169 136 254 74 23 48 4 88 195 61 226 48 107 1 155 117 17 124 1 190 10 206 146 67 21 49 14 114 187 74 191 228 153 15 114 59 17 168 231 31 225 218 163 249 101 248 40 79 39 34 151 146 203 245 251 122 32 192 136 105 201 101 127 29 251 127 157 6 251 146 75 44 198 158 112 45 56 3 150 184 6 15 17 158 64 176 145 112 173 244 3 7 231 15 70 90 152 213 40 169 61 128 0 0 0 0 73 69 78 68 174 66 96 130)! !

!DrGIcons class methodsFor: 'accessing' stamp: 'HilaireFernandes 11/26/2005 19:04'!
move
	^self formFromGraphic: #(137 80 78 71 13 10 26 10 0 0 0 13 73 72 68 82 0 0 0 48 0 0 0 48 16 6 0 0 0 7 146 37 196 0 0 0 6 98 75 71 68 255 255 255 255 255 255 9 88 247 220 0 0 0 9 112 72 89 115 0 0 0 72 0 0 0 72 0 70 201 107 62 0 0 1 153 73 68 65 84 120 218 237 155 73 110 195 48 12 69 227 162 183 211 9 180 11 130 220 77 39 208 249 210 77 181 8 81 69 19 39 169 255 109 12 35 137 197 255 105 81 244 144 219 13 0 0 0 0 0 116 16 66 8 33 188 94 214 113 120 99 214 151 111 238 1 115 206 57 231 235 178 54 68 10 238 19 112 58 1 189 1 238 158 16 233 25 255 37 117 224 221 141 215 210 193 158 128 83 140 215 210 181 156 0 26 216 169 139 180 155 210 218 50 184 124 126 74 34 122 245 204 234 101 207 224 41 221 145 150 14 49 67 118 77 132 118 220 98 93 208 168 64 107 172 226 18 63 19 91 194 172 103 132 117 124 106 194 117 132 210 227 215 143 103 109 124 65 173 4 181 4 173 149 128 254 223 121 49 190 160 190 6 212 4 126 22 94 12 163 91 173 241 229 112 219 141 188 51 106 184 223 46 139 162 62 3 164 137 49 198 24 253 117 89 219 241 183 145 181 82 180 94 154 192 47 212 248 247 253 209 45 152 102 205 120 36 130 137 182 193 159 19 5 22 233 53 158 126 31 48 209 54 190 181 15 8 189 247 215 71 141 174 25 127 218 243 137 105 168 17 53 99 164 140 247 154 8 183 119 67 139 177 41 165 148 210 117 209 125 233 241 181 16 187 18 158 21 254 124 62 30 247 251 186 241 173 113 122 227 148 70 237 145 100 175 33 210 120 139 207 205 237 232 255 22 71 129 45 16 60 3 158 163 123 6 140 214 74 239 198 207 198 201 189 102 12 151 32 26 128 155 23 150 22 105 189 96 38 181 88 31 211 206 113 163 165 151 109 17 182 110 231 118 213 193 222 5 237 154 8 171 184 217 254 31 176 123 9 114 255 146 177 187 128 224 3 0 0 0 0 192 198 252 0 102 116 72 96 211 171 62 8 0 0 0 0 73 69 78 68 174 66 96 130)! !

!DrGIcons class methodsFor: 'accessing' stamp: 'HilaireFernandes 3/22/2006 10:38'!
moveObject
	^self formFromGraphic: #(137 80 78 71 13 10 26 10 0 0 0 13 73 72 68 82 0 0 0 24 0 0 0 24 8 2 0 0 0 111 21 170 175 0 0 0 9 112 72 89 115 0 0 11 19 0 0 11 19 1 0 154 156 24 0 0 3 197 73 68 65 84 56 203 205 149 219 107 28 85 28 199 127 51 231 204 206 236 165 201 54 187 221 100 215 166 209 104 188 164 165 193 75 189 165 40 74 109 73 74 8 40 136 79 17 250 32 69 240 81 241 193 23 255 13 125 146 22 4 219 162 105 170 62 72 17 74 235 173 52 77 27 67 211 244 146 205 50 187 217 251 236 206 204 206 204 185 205 241 33 208 37 65 155 60 244 193 207 243 151 15 191 239 57 156 223 81 164 148 240 40 80 225 17 129 119 152 59 250 209 124 143 30 211 176 202 5 56 140 249 150 109 123 246 245 243 199 186 9 185 3 222 154 249 253 212 108 213 44 147 142 207 93 143 175 21 131 175 207 20 15 189 119 233 233 163 103 31 100 182 23 189 48 117 254 199 75 245 82 149 84 155 180 209 98 213 38 45 85 137 89 14 190 255 101 253 153 137 179 185 131 95 110 196 30 86 237 205 183 243 134 210 84 227 70 58 169 149 107 76 195 10 194 32 56 48 46 153 16 153 221 26 235 248 156 213 182 57 163 241 241 149 190 30 45 26 21 133 102 121 238 215 194 232 72 191 142 84 69 129 48 148 52 12 5 97 11 203 117 65 44 230 175 63 76 52 54 182 208 211 151 192 88 134 216 114 156 149 31 102 229 226 232 176 107 94 185 124 225 243 227 51 95 165 250 95 42 212 189 252 106 149 249 235 204 41 255 167 104 242 113 48 98 17 61 136 128 130 72 39 17 58 229 6 15 232 13 59 29 156 243 8 71 8 31 152 248 54 112 37 101 21 226 175 113 97 253 187 104 98 64 244 15 160 38 173 135 24 252 48 17 18 136 134 189 245 218 181 192 94 213 163 46 8 14 0 141 252 13 9 156 122 197 192 94 22 172 189 85 116 226 139 139 157 133 199 162 195 123 244 148 164 5 51 38 4 81 153 8 152 161 140 170 116 201 181 255 220 181 63 199 125 27 0 28 235 55 224 140 179 150 160 14 245 204 77 162 153 79 127 238 77 196 219 54 142 233 160 42 42 230 110 195 206 235 200 17 4 179 0 18 202 225 80 211 24 51 65 208 233 169 201 217 185 159 84 188 75 48 123 235 19 121 247 227 115 42 86 49 82 73 199 226 224 217 190 18 215 211 129 115 183 221 250 187 233 220 33 158 213 114 254 176 91 115 217 108 22 84 24 25 26 12 5 15 185 83 171 86 18 241 216 38 17 37 65 200 67 46 66 139 221 243 220 82 187 99 101 244 215 148 48 225 186 183 3 111 193 37 75 46 185 224 216 246 228 228 59 32 229 129 103 135 1 148 207 62 57 153 78 167 79 126 248 193 233 211 167 186 213 28 171 162 27 8 69 52 137 42 101 47 136 42 54 129 190 92 98 202 164 134 229 93 21 180 14 0 29 167 37 165 20 180 99 54 218 194 107 228 134 246 57 173 154 145 76 155 166 217 157 200 106 220 179 42 69 199 169 169 189 194 229 55 235 244 74 213 191 220 38 215 176 150 233 143 140 247 168 99 0 64 41 221 221 19 167 190 119 228 141 215 87 110 47 77 31 63 86 41 21 78 188 63 29 215 194 238 68 110 179 196 57 37 180 19 207 238 105 212 60 187 115 87 1 13 88 60 20 10 227 62 13 171 8 192 247 188 231 159 27 94 47 87 51 201 56 165 20 243 78 181 92 77 166 50 149 82 177 43 162 196 246 138 101 215 50 141 88 58 154 74 33 22 247 218 235 156 23 4 35 146 10 33 85 0 200 23 10 139 11 243 47 31 28 17 82 18 223 87 85 5 35 124 245 250 197 59 247 215 54 68 202 198 134 236 27 216 79 2 11 33 61 18 77 98 205 64 88 231 196 35 190 37 67 209 110 220 7 128 39 247 13 142 31 126 117 111 110 96 111 54 29 248 68 215 35 24 161 191 230 23 191 249 238 12 167 178 43 2 128 100 102 132 6 118 40 184 162 34 140 12 85 139 168 42 182 42 203 15 46 56 149 76 190 114 232 197 236 64 63 132 52 106 232 102 217 186 121 235 86 97 53 207 55 12 114 199 32 13 48 64 2 33 3 64 7 0 128 167 158 24 66 26 108 191 143 182 32 25 84 204 130 237 56 148 50 35 166 71 227 137 220 208 224 70 175 77 213 254 47 191 200 63 151 156 177 64 253 10 252 215 0 0 0 0 73 69 78 68 174 66 96 130)! !

!DrGIcons class methodsFor: 'accessing' stamp: 'HilaireFernandes 8/3/2011 09:33'!
open
	^self formFromGraphic: #(137 80 78 71 13 10 26 10 0 0 0 13 73 72 68 82 0 0 0 48 0 0 0 48 8 6 0 0 0 87 2 249 135 0 0 0 1 115 82 71 66 0 174 206 28 233 0 0 0 6 98 75 71 68 0 255 0 255 0 255 160 189 167 147 0 0 0 9 112 72 89 115 0 0 11 19 0 0 11 19 1 0 154 156 24 0 0 0 7 116 73 77 69 7 219 8 3 7 31 18 173 68 203 48 0 0 10 42 73 68 65 84 104 222 237 152 105 108 92 215 117 199 127 247 190 121 51 67 206 198 25 110 34 69 73 164 101 203 148 69 201 218 98 73 222 106 179 33 100 201 78 209 216 49 156 184 48 210 162 69 139 194 49 90 7 48 144 124 8 2 39 64 81 59 95 220 54 112 55 55 70 18 183 104 221 196 73 92 27 85 1 215 218 44 137 182 36 139 212 70 106 27 238 228 112 246 125 121 219 205 135 161 40 185 176 3 83 142 0 163 224 31 184 120 247 189 119 31 222 255 127 207 57 247 222 115 96 25 203 88 198 50 150 241 25 32 110 228 155 174 103 241 182 151 252 62 205 109 6 106 5 148 85 109 180 189 43 51 149 147 111 82 224 50 38 224 124 30 5 136 13 79 208 213 217 219 210 19 110 110 124 48 220 28 218 238 243 4 219 45 101 40 211 172 97 91 204 204 79 229 7 19 177 204 113 39 146 27 30 124 158 244 231 69 128 184 229 113 130 27 182 118 63 176 170 171 253 15 123 86 247 108 246 250 181 118 209 80 211 209 108 205 193 82 40 148 116 60 170 90 114 138 165 108 53 57 151 158 58 122 121 108 236 159 223 249 86 241 125 192 188 153 2 52 128 223 249 46 222 137 131 88 31 55 160 239 79 233 221 117 207 166 63 217 178 113 227 115 183 246 174 217 165 135 43 77 5 109 66 79 217 81 153 178 162 34 103 207 136 172 61 35 139 42 38 29 119 209 27 12 251 35 45 193 142 62 175 29 90 229 52 102 230 102 134 107 147 55 211 165 180 157 207 210 208 216 44 159 88 255 123 238 21 242 1 59 158 122 7 99 225 157 220 244 251 45 183 245 63 188 245 185 29 155 119 253 81 120 133 183 163 40 102 68 214 156 21 53 39 143 233 84 177 177 80 56 128 194 81 22 134 83 162 104 39 144 110 75 134 195 145 206 72 91 112 149 127 99 126 54 186 191 54 6 168 155 34 160 107 23 122 192 23 121 106 93 247 250 111 132 221 158 150 192 206 76 108 234 29 18 221 143 178 122 247 158 123 254 124 235 157 155 159 12 180 234 145 148 53 46 12 85 198 84 21 170 78 1 75 85 235 19 171 192 37 221 72 52 16 10 133 162 108 231 208 220 142 30 14 180 174 17 150 203 155 45 207 31 75 95 38 255 113 238 217 228 165 55 224 165 167 106 16 83 160 52 129 71 129 253 105 5 184 6 15 97 61 246 72 199 200 125 235 190 242 117 233 82 127 57 60 113 228 190 149 175 124 248 186 199 136 116 109 222 186 241 15 2 205 222 112 214 156 0 192 82 53 74 118 154 154 83 2 165 240 106 1 2 90 155 242 203 118 71 216 186 99 83 147 101 82 178 224 196 68 222 138 35 188 46 173 101 101 224 193 238 109 145 129 203 137 244 107 156 196 116 187 8 174 14 241 112 79 19 143 119 5 24 232 88 213 233 183 109 155 185 153 249 132 174 225 215 4 13 231 226 252 240 200 20 207 124 42 1 156 196 206 14 36 199 115 229 120 233 246 181 125 161 251 131 123 239 93 221 178 110 179 101 27 170 115 101 135 63 99 77 10 144 24 78 153 188 57 79 205 41 2 138 144 187 131 22 122 85 33 101 166 198 226 241 211 217 116 54 233 247 53 69 90 86 4 214 55 183 251 58 147 92 22 69 59 73 184 41 28 110 109 109 122 164 55 147 14 118 223 202 64 87 144 47 6 66 65 207 182 129 1 118 238 221 75 165 84 226 223 95 124 145 80 176 177 245 182 45 91 136 158 57 67 48 151 95 251 169 45 0 56 53 171 58 155 43 228 114 21 171 208 81 19 121 177 166 171 219 167 148 67 193 142 11 16 88 170 74 222 158 195 80 37 52 225 34 172 119 17 52 187 107 83 99 201 51 151 70 162 127 63 117 41 182 127 222 172 101 130 210 19 185 173 119 229 151 238 216 188 250 217 240 170 53 171 147 206 37 97 187 74 174 176 87 251 242 174 181 238 71 119 110 189 159 29 123 247 210 119 247 221 204 70 163 252 236 165 151 200 167 82 60 250 204 51 108 233 239 71 8 193 75 79 63 205 209 75 239 159 94 138 0 52 95 54 94 49 75 51 110 209 176 222 144 69 42 42 35 4 2 77 106 72 160 224 196 176 168 160 187 220 52 106 97 66 214 45 106 124 52 113 248 240 209 193 151 7 15 85 222 226 100 125 169 12 184 107 33 86 69 125 218 238 172 182 227 169 93 34 220 186 2 225 64 199 173 171 197 221 127 253 199 244 111 253 34 133 76 134 159 124 239 59 156 59 118 130 175 62 247 28 219 6 6 16 88 80 56 1 197 83 196 39 162 164 203 44 77 192 153 203 148 119 108 74 79 59 54 74 115 185 132 92 216 30 148 114 200 91 49 202 42 141 203 229 70 8 65 155 188 93 197 38 139 227 231 79 93 124 121 240 141 202 127 55 197 233 233 238 224 177 181 77 60 214 220 200 54 165 192 87 244 81 78 26 248 219 130 152 90 137 134 80 3 150 53 133 74 254 23 255 241 87 63 101 244 116 138 111 254 227 63 209 185 118 45 148 47 64 250 127 192 46 99 208 70 42 150 32 93 89 162 128 244 191 82 74 245 207 70 13 211 176 220 158 6 221 82 85 20 80 181 227 20 156 57 92 46 23 160 227 145 126 244 90 200 154 137 143 76 79 28 158 190 227 113 63 223 15 183 176 1 160 169 173 141 187 30 122 136 29 123 246 224 106 14 240 243 51 63 69 147 6 72 11 71 214 176 74 195 56 165 30 38 174 228 232 255 218 147 117 242 169 183 161 114 177 78 195 21 34 17 211 113 20 70 174 198 232 146 4 0 78 62 85 187 82 169 84 11 141 193 64 196 118 12 76 85 38 109 78 96 170 50 82 104 0 216 212 80 141 37 215 214 190 47 220 19 122 74 191 47 115 60 202 166 85 219 233 223 253 101 122 183 111 71 88 9 40 143 82 204 158 39 104 207 80 52 219 113 121 189 216 182 134 87 132 40 150 221 36 231 50 236 124 248 97 176 11 96 204 130 22 2 161 129 208 24 187 156 37 91 101 196 118 62 126 83 253 77 2 84 124 38 57 59 155 24 47 52 183 222 25 1 69 222 154 35 103 205 162 80 8 177 224 82 216 196 173 243 162 169 101 165 216 178 103 23 249 109 219 144 181 42 25 239 113 42 83 31 208 40 107 32 116 26 164 70 179 108 36 17 175 224 105 9 80 204 166 144 68 120 229 7 239 210 181 110 29 145 21 43 160 114 5 180 64 157 60 26 8 201 133 225 33 82 75 240 127 0 121 181 147 203 171 120 42 19 79 73 165 59 166 170 146 54 199 235 155 149 0 181 176 137 74 116 26 100 19 142 13 142 52 104 137 180 82 85 101 246 93 56 204 233 185 57 28 25 2 87 19 154 39 204 214 142 181 172 53 220 216 231 230 41 29 153 230 245 239 189 133 191 181 155 63 123 225 133 250 15 141 121 208 252 215 154 222 194 165 161 243 164 170 75 19 112 213 2 248 219 237 185 92 57 53 105 89 230 166 2 113 89 180 147 8 161 33 133 70 163 108 194 77 0 175 19 86 170 228 33 151 204 138 28 22 229 210 17 214 55 134 232 191 117 59 157 225 8 210 29 196 52 97 100 104 146 225 99 23 25 26 188 0 186 198 157 119 223 203 238 215 94 164 125 205 154 250 207 106 51 224 148 235 196 145 32 36 21 195 71 38 30 39 81 226 216 13 9 200 155 57 35 153 75 206 229 42 73 167 234 201 227 147 173 202 39 90 176 13 97 219 69 173 154 78 231 18 209 11 103 179 165 147 99 235 238 218 116 151 111 247 151 190 66 179 227 195 231 109 68 211 116 20 26 191 252 201 65 222 121 227 8 29 107 58 185 243 254 123 249 139 191 253 6 171 215 247 45 68 89 13 140 57 48 211 96 101 65 243 45 186 14 72 174 156 155 192 114 40 207 23 249 224 134 4 156 246 83 187 61 95 56 147 45 164 13 191 236 210 138 105 85 153 205 20 167 38 231 162 99 137 249 228 177 115 251 83 51 83 31 86 228 35 17 231 133 59 30 184 205 119 75 103 7 228 219 64 184 64 104 252 236 31 222 102 116 104 130 111 189 250 42 93 235 122 65 217 96 231 160 50 6 118 22 156 234 53 194 178 97 33 112 229 130 5 52 78 29 60 78 172 200 123 142 90 218 241 123 81 0 207 227 164 190 147 60 123 120 104 223 126 77 121 198 231 115 211 231 11 25 227 120 60 81 157 204 228 139 133 137 55 197 102 208 90 245 136 163 185 61 158 133 163 96 61 8 199 70 166 56 117 100 132 111 255 248 53 2 145 48 84 39 192 136 213 211 141 171 68 101 227 66 192 202 235 136 215 175 182 35 57 241 238 123 204 22 57 176 212 211 168 235 186 190 243 238 104 246 104 190 148 125 250 196 235 36 153 166 246 209 115 188 20 0 82 162 233 94 111 157 220 130 27 28 122 251 3 6 158 250 58 129 72 4 140 56 88 105 144 222 143 18 254 63 164 175 239 143 143 68 169 150 74 204 230 217 255 89 4 192 127 98 159 128 169 143 31 170 28 16 72 137 174 187 221 117 2 210 71 173 106 48 114 242 60 95 253 246 247 235 67 172 204 130 139 124 50 97 132 88 120 38 64 72 6 247 29 160 104 48 30 47 241 254 103 19 240 155 51 203 26 128 75 224 210 92 174 58 9 217 192 201 131 31 112 215 67 123 240 52 52 212 3 84 234 159 76 120 145 248 213 190 32 155 200 114 228 205 183 24 77 242 163 27 73 122 228 167 31 234 24 11 73 180 182 152 141 74 15 131 251 14 241 224 19 79 212 103 223 169 128 240 130 244 92 215 220 32 174 94 23 154 212 65 232 56 142 198 191 253 224 111 48 77 171 118 33 205 171 55 146 145 45 193 2 212 164 64 91 44 4 8 73 124 38 73 176 185 133 112 91 91 253 104 32 221 11 175 175 205 240 71 251 98 209 109 166 47 69 249 249 15 95 230 220 177 65 251 127 163 124 173 84 99 250 38 11 16 134 20 74 191 118 171 113 230 216 135 252 238 147 79 214 103 95 217 32 244 79 32 44 22 151 203 232 153 115 188 253 202 191 112 246 232 81 170 22 137 247 38 121 122 44 203 47 110 52 39 94 146 5 52 41 92 215 123 159 66 208 211 215 87 95 227 133 107 113 118 175 85 107 234 164 103 174 68 25 58 112 144 161 3 7 152 28 29 69 129 117 118 158 191 59 57 199 243 134 77 238 179 36 245 75 17 96 104 40 237 154 5 36 61 125 27 175 173 180 66 95 20 96 89 14 87 134 135 25 58 112 144 225 67 135 72 205 206 82 49 73 207 20 56 52 149 231 87 51 121 246 149 77 98 191 141 170 132 107 41 65 44 53 151 14 80 200 100 112 108 135 158 13 27 0 65 181 106 19 61 125 154 75 31 158 226 242 208 16 99 103 207 82 173 26 181 185 146 24 158 206 137 253 211 5 126 145 46 47 125 137 252 173 187 80 197 80 57 211 161 124 248 141 55 26 117 183 155 201 209 81 46 157 58 197 244 197 139 148 77 167 144 40 138 209 88 81 12 199 138 28 140 151 217 103 59 42 121 147 202 65 55 90 220 213 250 87 5 213 125 235 91 213 238 70 157 112 186 202 212 124 81 158 143 23 57 150 169 58 39 64 197 128 210 231 184 58 45 66 32 123 0 31 168 60 168 56 168 36 75 40 68 45 99 25 203 88 198 50 150 177 140 255 79 248 53 249 132 185 253 79 213 252 73 0 0 0 0 73 69 78 68 174 66 96 130)! !

!DrGIcons class methodsFor: 'accessing' stamp: 'HilaireFernandes 11/26/2005 19:04'!
parallel
	^self formFromGraphic: #(137 80 78 71 13 10 26 10 0 0 0 13 73 72 68 82 0 0 0 48 0 0 0 48 16 6 0 0 0 7 146 37 196 0 0 0 6 98 75 71 68 255 255 255 255 255 255 9 88 247 220 0 0 0 9 112 72 89 115 0 0 0 72 0 0 0 72 0 70 201 107 62 0 0 1 160 73 68 65 84 120 218 237 217 33 142 194 64 20 198 241 233 250 17 40 116 69 19 36 2 138 168 196 115 3 154 106 18 210 27 204 45 240 156 1 13 178 130 130 64 146 112 2 20 2 129 124 136 221 102 67 147 102 187 48 125 125 109 191 191 33 164 1 166 239 151 161 98 148 66 8 49 182 94 39 201 102 67 84 247 58 164 244 197 245 67 243 185 49 203 37 81 154 158 78 199 35 32 178 216 0 14 135 36 217 239 149 186 223 175 215 219 77 41 64 124 231 112 255 224 96 48 157 250 62 209 120 28 4 147 137 82 90 247 251 189 158 82 190 63 28 142 70 74 69 81 16 204 102 14 251 186 234 138 109 7 24 99 140 49 68 231 243 110 151 166 142 131 29 193 84 54 248 162 247 217 142 200 158 17 139 197 106 101 12 17 32 44 7 8 33 189 14 62 255 10 136 138 43 26 60 17 118 4 107 197 131 7 196 79 143 71 28 199 177 237 27 204 127 95 241 224 59 11 177 221 186 174 235 18 93 46 158 231 121 68 118 32 202 127 190 243 16 81 164 181 214 68 118 32 222 31 8 32 62 130 176 55 128 206 65 132 97 24 134 225 239 194 255 7 81 221 13 183 30 34 63 248 207 32 170 11 16 128 224 9 16 66 2 132 144 0 33 36 64 8 9 16 66 2 132 144 0 33 36 64 8 9 16 66 2 132 144 154 9 209 194 51 107 217 16 29 58 179 110 10 4 254 154 88 33 236 159 89 215 61 223 210 213 11 97 255 204 186 232 186 248 120 33 170 59 179 110 220 224 243 85 11 193 119 102 221 248 236 66 240 157 89 183 46 89 15 235 14 7 8 33 1 66 72 128 16 18 32 132 244 46 68 221 235 110 93 101 33 138 174 35 75 253 5 129 193 51 133 193 11 9 131 71 47 61 1 49 101 139 124 116 26 58 40 0 0 0 0 73 69 78 68 174 66 96 130) ! !

!DrGIcons class methodsFor: 'accessing' stamp: 'HilaireFernandes 11/26/2005 19:05'!
perpendicular
	^ self formFromGraphic: #(137 80 78 71 13 10 26 10 0 0 0 13 73 72 68 82 0 0 0 48 0 0 0 48 16 6 0 0 0 7 146 37 196 0 0 0 6 98 75 71 68 255 255 255 255 255 255 9 88 247 220 0 0 0 9 112 72 89 115 0 0 0 72 0 0 0 72 0 70 201 107 62 0 0 2 43 73 68 65 84 120 218 237 154 177 106 194 64 24 199 191 136 171 248 2 5 201 32 244 9 220 83 55 103 87 131 155 80 16 103 151 27 178 244 5 218 221 193 39 112 112 210 209 193 232 208 177 80 40 20 92 74 133 34 25 186 8 215 233 40 124 114 36 198 187 220 93 242 253 22 137 201 93 46 191 63 201 229 114 7 64 16 4 65 16 4 65 16 4 65 16 4 65 84 158 223 223 201 100 50 225 220 116 59 42 199 106 229 251 190 207 249 251 123 187 221 110 115 78 65 220 198 108 182 217 44 22 114 127 53 252 199 124 126 60 30 143 0 159 159 231 243 249 12 112 56 44 151 203 165 237 119 132 125 237 18 226 227 248 245 117 191 7 24 12 24 27 143 47 219 89 187 44 152 36 73 226 121 110 6 97 30 44 62 73 190 190 126 126 0 118 187 205 102 187 189 60 254 34 128 48 12 195 48 228 156 130 184 142 52 241 111 111 235 117 28 123 158 180 2 33 94 182 61 28 54 26 141 6 231 118 246 17 230 206 47 196 63 62 190 188 48 198 185 120 212 220 223 119 187 157 206 127 187 24 99 140 177 12 237 116 51 136 226 207 155 87 124 73 130 16 245 167 253 58 40 30 227 102 16 234 41 92 60 198 141 32 212 99 92 60 198 238 32 212 97 157 120 76 89 131 176 94 60 166 44 65 56 39 30 99 54 136 252 229 157 23 143 41 54 8 124 124 246 242 165 19 143 209 27 68 254 253 165 23 143 81 27 68 254 59 164 114 226 49 166 250 136 202 139 199 20 21 4 137 79 65 87 16 36 254 74 84 5 65 226 111 36 111 16 36 94 49 89 131 32 241 154 145 5 145 38 94 60 154 92 17 239 221 94 133 94 132 248 135 135 209 168 223 7 144 205 185 62 63 127 124 124 127 3 180 90 245 122 189 14 112 119 215 235 245 122 0 79 79 205 102 179 9 16 69 81 20 69 158 117 215 91 187 189 10 189 164 137 23 147 221 178 197 3 211 233 233 116 58 153 190 10 57 214 6 144 117 149 65 16 4 65 16 208 42 14 101 100 237 92 133 120 217 182 43 159 193 173 33 239 91 13 22 239 234 124 132 49 84 189 78 150 101 98 168 48 116 189 199 231 13 194 180 143 194 209 61 128 202 26 132 108 127 105 193 34 117 143 92 211 130 168 140 120 76 154 104 213 35 87 18 47 161 232 79 6 36 94 130 173 223 106 116 243 7 21 204 94 20 55 57 135 172 0 0 0 0 73 69 78 68 174 66 96 130)! !

!DrGIcons class methodsFor: 'accessing' stamp: 'HilaireFernandes 11/26/2005 19:05'!
pointDefinedByCoordinates
	^self formFromGraphic: #(137 80 78 71 13 10 26 10 0 0 0 13 73 72 68 82 0 0 0 48 0 0 0 48 16 6 0 0 0 7 146 37 196 0 0 0 6 98 75 71 68 255 255 255 255 255 255 9 88 247 220 0 0 0 9 112 72 89 115 0 0 0 72 0 0 0 72 0 70 201 107 62 0 0 1 180 73 68 65 84 120 218 237 154 193 205 194 48 12 133 211 95 140 192 22 12 193 0 156 184 230 10 136 9 58 69 87 224 156 43 39 6 96 8 182 96 135 254 39 11 41 81 112 210 66 253 76 252 93 162 170 45 181 223 139 157 138 198 57 195 48 12 173 140 163 116 4 218 233 166 221 150 19 190 155 248 123 237 82 41 88 233 140 55 35 22 194 90 208 92 254 164 3 104 157 85 221 229 54 227 193 48 67 230 98 45 72 24 51 64 152 70 215 128 56 15 185 215 230 202 10 160 64 181 190 231 227 77 160 196 128 221 174 239 183 91 188 64 203 160 184 227 17 151 108 5 144 17 186 13 33 112 255 58 73 2 224 4 63 30 135 225 249 148 14 59 101 191 119 238 241 168 78 191 243 222 123 239 199 49 132 16 66 72 143 23 79 36 158 249 239 13 65 173 140 92 43 122 141 36 52 221 17 31 139 81 215 114 0 2 126 27 151 98 35 234 18 69 161 92 248 120 84 34 60 234 91 70 169 208 233 245 74 132 231 18 151 134 23 158 132 190 94 157 219 108 212 183 158 56 113 105 120 225 233 152 12 200 157 87 102 4 110 160 57 97 149 87 0 234 26 240 130 19 150 171 0 101 224 6 206 85 128 116 124 31 2 47 145 243 249 116 58 28 202 43 64 57 250 18 65 51 160 209 239 1 56 52 246 61 0 15 251 36 41 140 25 32 140 173 1 194 216 26 32 140 181 32 97 204 0 97 108 13 16 198 214 0 97 172 5 125 133 242 78 97 6 124 21 222 136 213 212 27 231 93 143 194 82 113 211 115 210 214 157 49 160 180 199 231 126 56 255 192 182 206 243 122 54 215 130 46 151 190 95 175 151 122 26 63 145 103 58 200 205 4 28 114 27 206 110 183 97 184 223 97 182 167 199 130 210 168 181 199 227 243 243 45 136 219 106 9 186 251 27 119 215 131 97 124 148 127 134 246 108 126 47 30 109 160 0 0 0 0 73 69 78 68 174 66 96 130)! !

!DrGIcons class methodsFor: 'accessing' stamp: 'HilaireFernandes 11/26/2005 19:05'!
polygon
	^self formFromGraphic: #(137 80 78 71 13 10 26 10 0 0 0 13 73 72 68 82 0 0 0 48 0 0 0 48 16 6 0 0 0 7 146 37 196 0 0 0 6 98 75 71 68 255 255 255 255 255 255 9 88 247 220 0 0 0 9 112 72 89 115 0 0 0 72 0 0 0 72 0 70 201 107 62 0 0 9 92 73 68 65 84 120 218 237 91 107 76 155 101 27 190 74 129 210 195 91 218 66 143 208 150 114 42 39 41 200 32 50 220 208 56 141 75 22 77 22 157 51 51 131 4 231 70 68 102 166 176 17 127 204 68 127 105 162 46 152 37 102 25 78 101 33 217 146 109 49 100 51 134 140 48 53 49 147 127 75 212 132 31 99 39 206 231 210 183 156 74 253 113 231 249 234 247 242 189 225 208 246 165 240 113 253 121 211 62 240 230 121 238 235 62 223 79 129 29 236 64 74 212 214 114 28 199 5 131 236 185 217 251 137 219 236 13 72 5 161 192 11 11 149 74 165 18 184 126 61 43 43 43 43 24 28 25 57 122 244 232 81 233 9 137 223 108 193 68 11 98 26 158 153 153 152 152 152 8 228 230 38 37 37 37 1 46 151 94 175 215 3 193 224 236 236 236 172 244 251 220 182 4 8 161 84 202 229 114 57 224 241 168 213 106 53 224 114 89 44 22 11 160 209 208 103 133 66 165 82 169 164 223 215 182 33 64 76 227 51 50 116 58 157 14 40 40 144 201 100 50 192 225 32 193 3 28 199 113 192 159 127 62 120 240 224 1 80 90 74 150 33 53 182 124 12 88 45 152 122 60 9 9 9 9 64 81 17 9 62 35 163 172 172 172 12 248 226 139 187 119 239 222 5 30 62 244 251 253 126 192 231 27 24 24 24 0 252 254 198 198 198 70 233 98 193 150 37 64 76 240 121 121 228 82 94 125 53 53 53 53 21 40 46 166 207 147 147 46 151 203 5 124 248 225 139 47 126 245 85 232 239 103 102 22 23 23 23 1 158 39 34 252 254 225 225 225 97 233 206 177 229 8 16 19 60 203 106 158 121 134 124 121 81 145 195 225 112 0 28 71 130 111 107 123 227 141 239 191 7 2 129 154 154 212 84 96 121 121 255 254 253 251 129 191 255 38 193 243 60 123 83 48 40 101 46 180 101 8 16 19 188 209 72 46 166 164 132 8 72 75 75 75 75 75 3 244 122 147 201 100 2 90 90 106 107 175 94 253 31 7 143 187 117 235 214 173 208 231 193 193 249 249 249 121 128 231 39 39 39 39 165 59 87 204 19 32 38 120 143 135 130 104 117 53 165 145 78 167 213 106 181 2 41 41 54 155 205 6 156 62 253 230 155 237 237 33 141 103 79 185 252 187 239 198 198 66 79 134 161 33 114 69 126 255 227 199 143 31 3 62 95 125 125 125 125 244 109 33 102 9 16 19 124 101 165 70 163 209 0 79 61 165 213 106 181 64 65 1 9 222 110 175 168 168 168 0 154 154 200 213 172 23 127 252 225 243 249 124 128 207 183 188 188 188 12 44 44 76 77 77 77 69 255 156 49 151 134 138 251 120 242 237 89 89 164 249 110 183 193 96 48 0 106 53 249 250 134 134 231 158 251 252 243 144 166 51 8 53 93 108 157 125 207 243 129 64 32 0 0 148 182 70 27 49 67 128 152 224 159 126 154 178 152 138 10 18 124 90 90 118 118 118 54 160 84 146 207 111 106 122 237 181 111 191 141 220 62 88 44 112 58 165 201 134 54 157 0 49 193 187 221 212 42 200 207 39 65 155 205 233 233 233 233 128 86 75 193 245 212 169 87 94 249 230 155 141 107 188 216 250 232 232 210 210 210 18 192 243 79 158 60 121 2 120 189 199 142 29 59 22 12 114 220 133 11 23 46 68 222 38 54 141 0 49 193 239 221 155 156 156 156 12 216 237 164 241 6 3 249 124 151 171 187 187 187 91 38 171 171 187 118 109 126 62 122 193 241 247 223 169 39 84 89 73 193 61 16 160 52 53 90 144 156 0 161 224 19 19 201 197 60 251 44 245 106 152 111 183 90 73 211 83 83 11 11 11 11 129 186 186 150 150 95 127 13 6 35 173 241 98 235 60 79 174 40 24 140 110 93 32 89 22 36 20 124 106 106 74 74 74 10 224 118 211 1 115 114 40 171 97 189 26 135 163 171 171 171 75 38 123 239 189 234 234 207 62 147 106 151 33 12 12 44 44 44 44 0 60 79 45 138 104 33 234 22 32 222 50 160 188 187 172 140 136 176 90 243 242 242 242 128 143 62 122 235 173 142 14 160 174 238 196 9 64 58 141 23 98 98 130 178 33 191 127 124 124 124 60 122 242 137 154 5 8 5 111 52 82 26 201 242 248 146 18 210 120 163 49 35 35 35 3 224 56 42 160 98 5 191 253 230 245 122 189 192 232 40 41 202 240 112 116 6 54 17 183 128 149 174 38 62 62 62 30 40 47 167 118 175 203 69 77 50 139 133 130 236 217 179 111 191 125 253 122 232 255 165 214 248 213 214 167 167 89 16 102 245 65 140 18 176 82 227 89 254 78 189 26 183 155 124 187 217 76 149 235 233 211 221 221 221 221 128 92 30 249 67 69 18 172 48 91 90 162 22 69 164 17 54 1 66 193 179 1 136 221 78 27 207 205 165 180 210 106 37 23 115 230 76 77 205 149 43 128 92 94 91 11 196 158 198 11 43 227 153 25 106 77 248 253 228 146 34 141 13 199 0 161 224 61 30 242 241 121 121 113 113 113 113 64 101 165 217 108 54 3 54 91 65 65 65 1 208 212 116 251 246 237 219 145 63 64 180 49 60 204 230 5 244 156 157 61 126 252 248 241 200 197 130 117 91 128 80 240 102 51 185 24 155 141 158 133 133 228 227 245 122 106 11 159 57 115 248 240 15 63 0 114 249 225 195 64 236 106 188 216 250 95 127 81 12 216 189 155 10 180 197 197 200 90 194 154 45 64 40 120 135 131 130 106 85 21 5 83 54 0 97 249 125 75 11 249 248 237 130 241 113 114 169 62 95 100 99 193 170 22 32 20 124 102 38 5 87 54 196 206 201 161 30 141 201 68 4 52 53 189 254 250 165 75 128 92 126 226 4 16 123 26 189 209 117 175 151 98 193 220 220 220 220 220 92 228 8 16 181 0 161 224 139 138 40 111 207 207 87 40 20 10 32 39 135 122 37 38 147 221 110 183 3 167 78 117 118 118 118 70 110 99 177 134 135 15 217 196 140 231 121 62 114 195 251 21 22 32 20 124 121 57 107 134 81 87 210 237 166 30 141 78 231 116 58 157 255 214 248 67 135 128 216 209 216 72 173 51 244 245 145 230 79 79 211 224 134 231 7 7 7 7 195 21 255 191 44 64 204 199 103 101 177 155 100 148 199 127 242 201 189 123 247 238 1 31 124 176 189 53 158 225 210 37 175 215 235 13 181 161 167 166 40 22 204 205 141 140 140 140 132 255 126 153 80 240 121 121 164 233 101 101 76 243 73 211 63 253 180 183 183 183 119 243 53 114 179 215 247 237 163 186 166 178 146 92 175 197 82 90 90 90 10 168 213 231 207 159 63 191 254 121 193 138 24 80 89 73 233 99 97 97 113 113 113 49 96 50 101 102 102 102 134 203 243 246 65 87 215 244 244 244 52 192 243 51 51 51 51 64 92 28 165 223 27 197 138 24 48 62 78 21 107 110 46 5 155 184 56 114 61 194 10 145 97 179 53 82 170 245 139 23 15 30 84 40 100 255 241 24 83 83 36 31 159 239 209 163 71 143 194 32 128 249 56 246 226 31 127 188 115 231 206 29 64 161 160 193 136 197 210 223 223 223 15 52 55 247 246 86 85 1 106 117 73 73 73 9 160 82 145 165 40 149 61 61 205 205 128 82 121 238 220 185 115 50 89 172 220 187 143 52 132 231 154 157 141 204 237 137 21 62 75 76 128 123 246 80 193 101 50 81 119 211 96 160 167 94 79 62 145 227 232 169 80 80 157 192 110 43 188 251 110 123 123 123 123 232 61 98 26 37 92 103 136 53 139 96 120 225 5 58 239 238 221 116 78 155 173 188 188 188 60 164 136 107 37 96 133 11 18 70 125 38 160 95 126 17 43 193 71 71 71 71 129 67 135 168 2 214 104 232 58 135 193 208 215 215 215 7 156 61 203 130 20 213 17 9 9 87 175 214 214 2 95 127 125 242 228 201 147 193 96 125 125 91 91 91 91 232 109 140 152 104 207 126 195 197 253 251 148 150 150 148 208 77 186 96 112 99 237 234 176 167 252 171 185 156 151 94 34 77 97 105 173 90 77 4 105 181 228 226 52 26 106 79 199 199 83 129 167 80 80 129 215 208 208 209 209 209 177 249 26 47 182 110 177 80 240 61 112 128 206 231 118 123 60 30 15 96 52 222 184 113 227 70 24 22 176 94 8 45 70 72 204 207 63 83 214 32 68 85 21 5 177 204 204 177 177 177 49 64 167 163 3 113 28 205 136 155 155 119 237 218 181 11 48 24 174 93 171 171 3 146 146 136 184 247 223 15 119 199 145 193 226 34 237 211 239 167 231 70 111 79 72 112 247 235 191 177 214 32 253 242 203 52 87 112 56 168 46 209 235 201 66 212 106 170 79 216 173 103 185 156 214 27 27 175 92 161 57 131 52 49 128 225 249 231 201 181 238 217 67 163 85 171 149 20 103 173 177 64 242 107 41 98 22 195 192 8 250 233 39 202 46 172 86 42 253 157 78 34 192 96 32 159 107 181 14 13 13 13 1 58 29 37 7 31 127 76 67 125 165 242 226 197 131 7 1 173 150 110 208 5 2 71 142 28 57 2 52 52 92 190 124 249 114 248 251 23 166 227 161 30 17 197 194 64 96 125 205 58 201 45 96 189 88 205 98 138 139 105 16 148 158 78 49 70 175 167 251 69 70 35 89 6 199 145 37 41 149 244 119 42 21 197 28 133 130 146 6 161 166 174 55 43 99 120 231 29 163 209 104 4 114 114 168 128 93 107 44 136 121 2 196 32 70 140 78 71 4 104 52 244 100 87 27 141 70 150 54 211 247 201 201 44 9 160 160 159 144 64 4 169 213 212 98 80 169 90 91 91 91 67 2 100 89 153 24 1 53 53 68 64 126 62 89 162 197 114 243 230 205 155 219 152 0 49 172 102 49 108 130 231 116 146 197 184 221 68 16 203 206 146 147 201 213 105 52 212 17 72 74 34 31 239 247 83 125 51 49 65 159 91 91 247 237 251 242 203 144 69 84 87 211 247 213 213 235 139 5 219 142 0 49 172 53 248 23 21 17 33 46 23 17 97 179 209 147 227 104 93 163 161 118 60 35 136 253 182 248 254 253 137 137 137 9 32 59 155 234 1 187 189 167 167 167 231 255 208 2 54 138 213 8 98 191 79 48 155 201 149 49 130 180 90 102 49 116 25 65 163 161 236 204 108 238 236 236 236 220 33 32 108 172 213 114 246 238 101 173 26 114 109 7 14 244 247 247 247 75 241 19 143 29 236 96 43 227 31 155 227 168 4 251 87 97 42 0 0 0 0 73 69 78 68 174 66 96 130)! !

!DrGIcons class methodsFor: 'accessing' stamp: 'HilaireFernandes 11/26/2005 19:06'!
property
	^self formFromGraphic: #(137 80 78 71 13 10 26 10 0 0 0 13 73 72 68 82 0 0 0 48 0 0 0 48 16 6 0 0 0 7 146 37 196 0 0 0 6 98 75 71 68 255 255 255 255 255 255 9 88 247 220 0 0 0 9 112 72 89 115 0 0 0 72 0 0 0 72 0 70 201 107 62 0 0 8 232 73 68 65 84 120 218 237 91 71 72 124 189 23 189 118 84 176 160 130 189 128 136 88 113 41 138 5 68 172 8 54 176 131 11 43 130 32 98 69 65 193 130 226 198 133 109 161 27 11 136 136 88 112 225 194 238 194 133 21 43 138 189 96 239 221 247 95 12 135 252 191 153 111 190 153 167 51 234 175 156 77 200 155 188 188 228 156 228 222 228 38 67 244 23 127 241 39 67 129 95 113 142 251 238 6 255 76 40 240 228 241 255 222 148 174 24 35 222 211 211 211 211 211 83 254 93 226 56 142 147 165 220 178 170 111 126 126 126 126 126 158 232 234 234 234 234 234 74 248 87 254 66 72 120 129 53 249 252 252 252 252 252 156 72 71 71 71 71 71 71 118 4 8 63 151 119 57 89 213 63 54 54 54 54 54 70 148 147 147 147 147 147 67 180 176 176 176 176 176 192 95 8 169 5 120 127 127 127 127 127 39 122 122 122 122 122 122 34 122 126 126 126 126 126 230 47 196 239 2 5 5 5 5 5 5 162 203 203 203 203 203 75 162 192 192 192 192 192 64 254 66 240 22 224 230 230 230 230 230 134 165 210 54 84 210 243 95 173 28 242 138 138 138 138 138 138 31 23 66 81 50 133 31 195 79 33 74 94 229 0 12 76 109 109 109 109 109 109 162 190 190 190 190 190 62 34 71 71 71 71 71 71 148 18 239 125 100 46 192 79 35 10 207 219 219 219 219 219 219 137 66 66 66 66 66 66 136 244 244 244 244 244 244 136 172 172 172 172 172 172 136 22 23 23 23 23 23 63 255 93 8 209 219 219 219 219 219 75 228 224 224 224 224 224 32 94 8 153 9 240 211 136 239 232 232 232 232 232 32 114 113 113 113 113 113 33 202 200 200 200 200 200 96 68 39 36 36 36 36 36 176 247 131 131 131 131 131 131 69 133 144 246 187 200 35 133 16 173 173 173 173 173 173 194 181 49 33 62 45 192 79 33 30 196 121 123 123 123 123 123 51 194 177 106 169 170 170 170 170 170 34 218 218 218 218 218 218 34 106 105 105 105 105 105 33 26 30 30 30 30 30 102 245 5 5 5 5 5 5 49 27 46 45 241 226 132 48 51 51 51 51 51 35 114 115 115 115 115 115 19 237 151 242 239 66 60 76 203 245 245 245 245 245 53 81 113 113 113 113 113 49 19 2 56 59 59 59 59 59 99 66 216 216 216 216 216 216 48 33 48 99 6 6 6 6 6 6 136 156 156 156 156 156 156 62 222 94 73 144 90 0 73 27 153 239 38 94 87 87 87 87 87 151 104 104 104 104 104 104 136 200 216 216 216 216 216 152 232 240 240 240 240 240 240 95 58 174 172 172 172 172 76 180 189 189 189 189 189 205 102 8 0 19 242 217 246 74 226 237 195 51 224 187 136 71 190 176 176 176 176 176 144 117 176 161 161 161 161 161 129 17 119 119 119 119 119 119 71 164 170 170 170 170 170 42 190 253 185 185 185 185 185 185 68 221 221 221 221 221 221 68 209 209 209 209 209 209 68 105 105 105 105 105 105 242 35 158 183 0 194 21 126 23 241 192 196 196 196 196 196 4 81 120 120 120 120 120 56 145 173 173 173 173 173 173 180 189 33 202 202 202 202 202 202 18 37 190 174 174 174 174 174 78 118 253 146 153 0 226 240 213 196 11 195 194 194 194 194 194 130 72 77 77 77 77 77 77 114 123 75 74 74 74 74 74 136 186 186 186 186 186 186 228 71 188 220 77 208 119 17 47 252 252 224 224 224 224 224 64 122 1 12 12 12 12 12 12 88 62 53 53 53 53 53 149 255 119 199 199 199 199 199 199 89 76 8 203 88 56 109 105 77 144 212 203 208 175 114 194 210 190 135 229 29 156 49 108 189 164 84 56 154 11 83 38 237 119 177 174 247 247 247 247 247 247 39 42 43 43 43 43 43 35 242 243 243 243 243 243 99 209 82 225 25 32 142 191 15 239 3 240 129 175 38 30 121 119 119 119 119 119 119 182 94 223 223 223 223 223 223 23 141 85 9 167 120 15 192 8 150 150 248 148 148 148 148 148 20 34 103 103 103 103 103 103 230 67 0 8 49 55 55 55 55 55 39 153 71 185 207 0 89 19 15 32 232 37 76 208 206 206 206 206 206 142 248 244 241 241 241 241 241 145 109 184 96 74 96 138 80 15 226 253 226 136 199 190 1 245 32 15 224 249 191 159 27 124 64 0 190 196 203 187 28 4 208 210 210 210 210 210 98 65 48 68 39 197 165 15 15 15 15 15 15 108 21 100 105 105 105 105 105 201 136 134 16 166 166 166 166 166 166 226 137 127 123 123 123 123 123 99 51 16 27 186 204 204 204 204 204 76 182 33 252 103 84 84 134 2 124 23 241 120 222 214 214 214 214 214 198 58 10 66 140 140 140 140 140 140 136 212 213 213 213 213 213 69 83 140 72 56 203 233 233 233 233 233 105 162 139 139 139 139 139 11 102 82 16 43 194 70 79 120 132 31 29 29 29 29 29 49 223 130 13 29 234 195 192 16 23 130 0 62 189 19 254 46 226 211 211 211 211 211 211 69 71 38 108 189 164 85 17 78 248 144 170 168 168 168 168 168 48 31 1 39 139 122 80 239 241 241 241 241 241 177 104 253 24 8 8 97 224 253 95 118 25 202 151 120 13 13 13 13 13 13 70 132 164 29 176 56 220 223 223 223 223 223 179 20 33 139 215 215 215 215 215 87 241 245 246 244 244 244 244 244 176 124 64 64 64 64 64 128 20 60 254 247 207 76 59 56 47 28 73 34 149 55 241 176 161 30 30 30 30 30 30 44 212 48 59 59 59 59 59 75 100 98 98 98 98 98 194 70 50 14 72 190 10 16 220 199 199 199 199 199 135 72 73 73 73 73 73 137 45 71 225 115 66 67 67 67 67 67 137 38 39 39 39 39 39 89 47 127 108 44 8 196 99 131 3 226 49 226 225 36 113 20 40 236 108 191 10 53 53 53 53 53 53 68 123 123 123 123 123 123 68 77 77 77 77 77 77 114 136 5 125 23 241 0 136 199 9 19 108 50 95 194 97 194 224 140 97 42 176 170 130 208 146 80 95 95 95 95 95 79 212 216 216 216 216 216 200 218 27 31 31 31 31 31 207 70 62 32 214 135 254 247 103 216 43 168 16 183 33 144 126 21 241 56 99 189 189 189 189 189 189 229 127 207 7 209 77 28 77 138 3 86 45 248 158 176 32 253 253 253 253 253 253 108 7 14 211 216 217 217 217 217 217 201 174 237 192 100 195 151 192 4 77 77 77 77 77 77 177 222 75 104 54 186 200 113 2 1 56 78 48 114 56 238 244 244 244 244 244 84 52 21 28 120 176 84 96 155 69 83 193 178 143 227 70 71 71 71 71 71 57 78 208 81 150 206 204 204 204 204 204 112 156 192 215 112 156 192 212 240 79 5 235 122 214 15 193 6 137 227 4 49 36 142 19 132 177 57 78 32 60 199 153 155 155 155 155 155 179 242 194 169 32 248 199 113 213 213 213 213 213 213 172 125 72 5 3 147 125 95 112 30 193 113 174 174 174 174 174 174 194 245 241 152 1 80 242 229 229 229 229 229 133 165 242 26 241 118 118 118 118 118 118 108 36 241 5 54 86 24 249 88 247 15 14 14 14 14 14 138 94 48 147 245 170 14 22 3 231 18 97 97 97 97 97 97 162 51 64 238 203 80 225 231 176 189 177 177 177 177 177 177 44 143 13 144 189 189 189 189 189 61 35 94 92 125 63 149 120 228 229 126 32 195 183 193 32 26 35 30 177 153 230 230 230 230 230 102 230 4 63 59 226 17 74 248 46 226 249 242 38 247 88 144 48 241 48 61 32 62 38 38 38 38 38 230 247 35 94 90 200 109 25 42 137 248 200 200 200 200 200 72 230 91 248 2 59 227 239 54 53 146 202 201 76 0 105 99 65 146 136 135 51 250 40 241 184 112 133 235 38 124 137 255 168 239 250 104 185 47 139 5 73 34 94 248 222 14 95 44 45 45 45 45 45 17 21 20 20 20 20 20 176 245 57 110 67 200 138 120 89 9 244 229 78 24 171 26 16 143 139 81 112 174 8 25 240 197 242 242 242 242 242 50 81 84 84 84 84 84 20 235 32 70 60 130 114 95 69 188 172 76 15 111 1 196 1 91 123 156 44 33 142 158 156 156 156 156 156 204 130 100 124 177 178 178 178 178 178 66 20 23 23 23 23 23 199 66 14 63 157 120 228 165 21 228 195 51 0 196 195 25 98 6 20 21 21 21 21 21 17 9 118 198 252 137 95 93 93 93 93 93 37 74 76 76 76 76 76 252 245 136 231 11 222 51 160 188 188 188 188 188 92 148 248 188 188 188 188 188 60 118 178 196 23 107 107 107 107 107 107 68 73 73 73 73 73 73 44 172 251 187 18 207 91 0 216 240 138 138 138 138 138 10 209 107 223 146 14 159 197 97 125 125 125 125 125 157 205 168 63 133 120 222 2 8 31 46 251 250 250 250 250 250 178 152 7 4 64 57 67 67 67 67 67 67 34 107 107 107 107 107 107 209 250 16 213 44 45 45 45 45 45 101 121 220 80 67 216 25 103 189 210 118 88 94 229 96 130 165 125 15 39 104 50 19 64 216 7 232 235 235 235 235 235 139 158 144 33 76 125 114 114 114 114 114 194 46 80 1 8 78 101 103 103 103 103 103 19 109 108 108 108 108 108 16 213 214 214 214 214 214 18 69 68 68 68 68 68 136 238 140 191 250 223 148 146 158 203 10 82 11 0 211 3 192 102 227 82 44 110 29 32 152 134 145 2 65 64 124 126 126 126 126 126 62 209 230 230 230 230 230 38 27 233 187 187 187 187 187 187 68 149 149 149 149 149 149 146 219 35 45 49 178 46 199 23 176 12 255 140 130 50 72 105 197 68 155 166 169 169 169 169 169 201 46 187 226 126 141 48 64 60 86 71 248 99 196 159 13 102 196 62 233 70 56 14 183 17 188 188 188 188 188 188 216 109 101 152 160 145 145 145 145 145 17 150 23 110 192 159 14 25 17 33 237 228 253 75 252 95 252 48 252 15 157 2 91 74 184 1 204 172 0 0 0 0 73 69 78 68 174 66 96 130)! !

!DrGIcons class methodsFor: 'accessing' stamp: 'HilaireFernandes 11/26/2005 19:06'!
ray
	^self formFromGraphic: #(137 80 78 71 13 10 26 10 0 0 0 13 73 72 68 82 0 0 0 48 0 0 0 48 16 6 0 0 0 7 146 37 196 0 0 0 6 98 75 71 68 255 255 255 255 255 255 9 88 247 220 0 0 0 9 112 72 89 115 0 0 0 72 0 0 0 72 0 70 201 107 62 0 0 1 149 73 68 65 84 120 218 237 154 61 14 130 48 24 134 91 103 56 4 3 215 106 76 186 25 35 151 241 8 14 156 128 201 155 56 153 56 178 187 144 212 161 49 198 250 83 104 191 210 79 121 159 173 4 18 120 31 126 90 242 10 1 120 97 76 219 182 173 49 185 207 99 113 28 143 85 85 85 198 156 78 117 93 215 198 64 68 38 92 17 215 107 211 52 13 68 164 98 229 110 56 28 250 190 239 133 56 159 135 97 24 132 184 92 186 174 235 132 128 136 196 40 165 148 82 143 128 215 235 178 44 75 99 226 158 136 251 126 16 247 17 55 120 26 17 110 240 16 224 101 172 8 59 242 5 12 1 193 248 68 88 124 34 16 124 52 223 69 64 192 108 196 138 208 90 107 173 31 199 187 99 48 146 216 87 17 130 39 37 244 85 4 17 46 171 105 187 143 13 78 74 119 139 157 190 10 177 223 23 69 81 228 190 108 62 76 20 224 67 202 231 59 92 74 59 125 197 202 154 136 207 179 28 223 199 150 102 101 13 94 240 5 15 17 137 152 26 60 68 16 17 27 124 154 159 126 11 100 234 2 43 205 79 191 5 178 219 109 183 155 77 248 202 22 34 152 0 17 76 128 8 38 64 4 19 66 69 160 197 65 12 158 8 38 64 4 19 32 130 9 16 193 4 136 96 2 68 48 1 34 152 48 143 8 212 110 188 160 114 201 4 84 46 153 128 202 37 19 168 43 151 232 57 5 226 138 120 14 18 5 179 217 120 31 228 124 34 100 232 129 255 73 236 29 45 39 231 73 220 140 251 85 194 43 151 223 183 251 129 128 81 188 86 46 237 56 247 121 253 13 225 149 75 124 140 137 136 45 156 129 64 98 131 135 8 34 16 108 38 98 155 126 224 199 185 1 85 109 241 171 221 62 127 106 0 0 0 0 73 69 78 68 174 66 96 130)! !

!DrGIcons class methodsFor: 'accessing' stamp: 'HilaireFernandes 6/21/2007 16:23'!
redo
	^ self formFromGraphic: #(137 80 78 71 13 10 26 10 0 0 0 13 73 72 68 82 0 0 0 48 0 0 0 48 8 6 0 0 0 87 2 249 135 0 0 0 6 98 75 71 68 0 255 0 255 0 255 160 189 167 147 0 0 0 9 112 72 89 115 0 0 13 215 0 0 13 215 1 66 40 155 120 0 0 0 7 116 73 77 69 7 215 6 21 14 22 55 133 211 233 232 0 0 8 11 73 68 65 84 104 222 237 152 105 108 84 215 21 199 255 247 222 247 222 204 120 188 225 193 216 198 27 182 176 195 82 108 140 77 73 104 33 64 0 177 101 32 141 147 170 129 46 106 196 180 85 219 168 31 218 170 21 72 84 173 104 20 169 85 104 139 42 58 137 105 17 137 66 128 54 241 184 132 134 134 128 144 193 16 92 28 83 4 49 16 179 120 101 48 227 241 50 51 111 222 123 247 222 126 176 227 96 48 48 4 236 98 201 231 203 147 238 242 206 255 119 238 126 128 49 27 179 49 27 213 70 70 194 137 219 171 217 1 220 0 80 238 243 24 251 31 230 191 233 8 6 203 225 74 200 248 251 234 215 180 239 141 74 0 66 32 126 182 124 187 35 119 252 180 87 87 191 110 123 101 52 142 0 226 181 113 248 241 226 173 142 226 156 121 47 173 169 176 237 113 123 53 117 84 1 0 64 179 254 49 190 61 247 215 246 5 83 203 159 166 140 28 113 123 181 164 81 5 208 99 94 71 99 168 6 79 23 255 208 246 236 151 95 42 165 140 212 185 189 90 246 168 1 0 0 93 244 224 211 208 81 60 145 191 90 93 191 232 229 92 198 104 157 219 171 21 143 26 0 2 10 75 70 113 41 92 131 130 9 37 244 167 43 95 79 177 107 113 71 221 94 109 233 255 253 28 112 123 181 100 0 83 1 20 16 130 41 148 145 153 18 120 204 78 227 39 189 242 220 251 244 116 247 62 80 66 65 41 235 251 18 5 89 142 34 24 134 137 223 191 231 137 116 133 59 126 84 185 62 186 125 196 1 220 94 45 143 50 178 137 17 237 249 130 244 34 115 162 107 178 154 154 152 97 79 113 78 36 201 142 116 36 219 38 160 203 106 131 223 184 0 70 24 8 165 160 132 13 192 164 170 147 97 147 137 216 114 224 7 225 214 96 227 150 119 95 212 55 140 8 128 219 171 77 165 140 252 38 78 75 90 177 168 168 92 157 149 189 88 225 136 66 23 189 48 68 24 166 136 192 148 58 76 17 5 37 12 26 181 131 246 139 39 148 130 17 6 144 62 25 201 44 19 201 44 7 21 213 191 208 207 54 31 127 135 115 241 45 159 199 176 134 5 192 237 213 236 148 145 55 156 90 210 138 167 102 62 103 43 206 92 68 3 230 21 52 71 62 70 84 132 192 136 6 133 104 3 95 149 218 160 82 7 52 102 7 163 234 64 244 9 29 188 12 227 233 120 164 105 83 176 247 212 239 244 234 134 202 90 110 137 21 62 143 209 243 80 1 220 94 45 137 50 114 112 118 225 226 233 75 166 173 179 183 69 207 161 41 124 10 166 212 7 4 43 68 3 163 159 65 216 160 49 59 52 234 128 74 237 80 153 189 127 234 208 33 37 216 105 34 50 181 34 28 186 240 166 89 121 114 219 21 206 197 147 62 143 209 250 80 0 220 94 45 141 50 90 61 111 250 170 156 199 11 86 105 39 3 111 33 204 3 159 71 155 106 183 71 191 95 188 66 85 72 34 0 0 140 41 125 211 8 4 0 1 1 1 35 42 52 18 7 66 24 84 98 71 182 109 22 78 183 30 22 127 61 252 171 0 231 124 161 207 99 156 121 32 0 183 87 203 167 148 84 47 42 121 54 181 56 103 158 114 44 176 3 6 15 221 20 105 21 10 177 129 17 21 92 154 136 138 94 68 69 8 32 22 4 225 0 17 96 84 133 194 84 48 170 220 209 143 74 28 208 168 19 241 108 60 30 79 252 14 142 54 190 35 125 199 43 186 42 215 71 199 221 218 86 185 15 241 132 50 242 207 37 165 207 167 21 100 20 211 15 253 127 130 5 3 10 209 64 37 135 0 135 41 34 136 138 16 162 34 4 33 251 214 30 163 74 159 96 162 66 33 202 192 246 121 55 51 101 4 66 112 76 115 46 195 249 27 199 177 191 118 103 68 74 249 245 161 218 42 247 49 0 223 204 116 229 229 20 101 205 163 251 219 95 134 37 251 196 11 112 68 101 8 134 184 14 46 205 193 167 36 97 253 130 25 24 237 219 46 25 97 32 247 0 80 137 3 101 137 47 160 173 227 170 220 117 228 143 61 220 178 150 250 60 198 137 47 12 224 246 106 113 148 145 87 151 150 172 117 158 233 218 143 168 8 129 128 130 195 132 33 34 144 16 67 44 46 50 32 248 51 8 74 89 255 194 189 179 217 105 34 102 39 174 195 249 150 58 81 89 83 209 201 57 159 231 243 24 231 238 212 62 38 0 74 201 198 226 252 39 156 14 123 28 62 189 86 3 0 144 16 176 164 113 151 62 236 166 19 247 115 136 187 45 59 39 115 97 118 194 90 212 54 30 180 62 56 181 187 141 115 241 85 159 199 184 122 55 109 74 76 91 38 85 126 50 127 106 185 173 54 248 214 144 209 30 28 193 4 100 56 166 97 162 99 26 226 85 23 52 234 132 141 57 192 136 134 70 253 40 174 232 31 13 217 47 81 201 64 89 252 55 112 232 236 94 179 230 220 254 139 156 139 249 62 143 209 113 47 125 177 140 64 201 196 148 92 110 145 8 252 209 11 119 108 148 162 229 160 36 249 25 36 178 12 52 7 63 193 217 198 122 253 70 208 111 232 70 136 134 245 16 181 41 14 237 251 203 55 43 67 1 184 212 73 40 118 150 99 223 169 138 104 253 165 234 58 110 137 37 62 143 209 27 203 236 136 5 160 116 98 106 158 189 211 104 30 178 146 17 21 51 146 86 34 219 86 130 195 231 246 68 78 55 158 32 18 188 150 91 98 23 128 51 0 130 0 34 132 144 179 10 177 221 214 63 77 155 130 233 142 149 216 83 179 69 191 208 90 255 33 183 196 51 62 143 97 196 186 179 220 19 128 41 100 126 250 184 28 165 211 108 26 178 126 78 202 90 24 33 41 183 29 218 160 235 102 168 66 8 249 203 91 163 215 159 149 184 205 178 108 37 152 108 91 128 157 71 126 27 105 234 184 184 155 91 226 187 62 143 33 238 231 108 186 39 128 148 40 77 75 152 132 250 208 127 110 171 43 140 127 18 54 158 140 55 171 55 135 44 203 92 232 243 24 181 177 58 206 179 207 69 150 58 11 21 31 108 138 248 187 91 182 190 251 162 254 243 97 121 208 72 33 211 18 108 41 232 177 252 131 201 137 134 233 9 203 177 187 122 107 196 178 204 23 238 71 252 99 113 139 145 70 191 132 191 188 191 81 247 119 53 111 252 162 226 99 26 1 194 72 83 80 111 207 115 178 20 24 34 60 80 62 78 203 129 191 247 50 186 245 27 23 125 30 163 42 86 135 51 156 110 168 86 18 182 29 216 16 9 71 123 60 149 235 163 111 12 235 163 158 0 117 254 158 171 72 84 211 110 217 117 178 209 28 56 47 36 151 255 142 213 153 148 2 66 87 241 218 191 54 133 67 122 119 249 131 138 143 9 128 91 242 152 63 216 98 58 21 215 160 114 75 68 251 47 145 68 139 213 89 195 245 19 114 251 129 205 61 186 17 89 226 243 24 239 141 84 98 171 206 223 217 162 59 217 96 128 160 217 2 87 98 58 161 20 11 98 139 190 100 187 14 253 161 211 178 204 185 62 143 113 108 36 51 115 117 77 254 70 213 134 4 196 177 228 155 0 90 145 236 76 133 77 137 203 119 123 181 57 49 60 94 27 45 206 75 135 186 211 15 43 128 207 99 116 114 105 236 252 239 229 26 115 82 220 236 129 114 33 45 180 68 79 227 169 210 53 14 202 200 219 110 175 102 187 219 13 25 18 115 125 30 227 242 195 206 130 196 148 23 18 92 110 170 191 240 145 229 144 46 184 180 73 3 151 181 43 145 147 72 117 165 147 130 172 25 19 24 163 59 220 94 77 185 67 16 184 207 99 92 27 142 28 19 139 105 241 85 241 222 194 85 232 108 15 52 47 44 203 91 162 118 154 77 144 68 128 81 138 176 8 160 52 103 153 218 29 238 204 15 134 58 86 20 174 164 123 27 170 120 116 164 146 100 44 230 29 164 138 159 204 93 166 151 70 140 80 254 204 172 133 74 143 184 6 9 14 139 24 8 73 63 74 115 151 170 140 210 180 246 64 211 218 194 149 116 87 67 21 15 61 82 0 0 80 184 138 237 11 246 118 44 13 71 122 93 51 179 23 170 186 236 2 135 1 65 44 116 89 45 40 76 155 205 156 90 82 124 75 199 165 57 159 84 90 127 123 228 0 26 170 184 89 184 138 237 236 10 5 202 110 4 175 103 23 101 63 169 218 85 7 116 217 3 14 19 65 171 25 147 199 151 209 214 246 150 212 204 197 189 167 26 170 248 197 225 6 184 239 228 174 207 99 24 130 203 53 109 29 77 127 254 199 193 29 145 80 64 200 124 199 92 36 43 89 80 136 29 166 140 64 85 84 14 160 115 100 18 197 15 150 86 44 99 10 125 59 115 66 110 122 65 206 244 56 87 82 58 218 59 154 69 77 253 193 144 105 153 89 62 143 209 253 72 3 244 67 168 0 214 41 42 91 14 137 175 72 160 141 91 252 107 247 122 203 142 217 152 141 217 195 177 255 1 144 175 73 88 169 71 12 171 0 0 0 0 73 69 78 68 174 66 96 130)! !

!DrGIcons class methodsFor: 'accessing' stamp: 'HilaireFernandes 11/26/2005 19:06'!
rotation
	^self formFromGraphic: #(137 80 78 71 13 10 26 10 0 0 0 13 73 72 68 82 0 0 0 48 0 0 0 48 16 6 0 0 0 7 146 37 196 0 0 0 6 98 75 71 68 255 255 255 255 255 255 9 88 247 220 0 0 0 9 112 72 89 115 0 0 0 72 0 0 0 72 0 70 201 107 62 0 0 3 12 73 68 65 84 120 218 237 155 189 106 181 64 16 134 85 210 89 88 167 144 64 26 193 194 52 22 66 238 64 66 106 123 187 16 115 1 185 128 128 157 96 237 77 132 220 66 192 54 22 150 129 96 145 43 176 246 43 14 131 100 206 217 111 221 31 221 245 231 105 68 61 174 235 59 179 51 235 142 199 48 14 14 14 180 97 24 254 110 183 143 165 186 3 39 246 35 56 230 106 153 219 144 4 54 77 213 2 168 134 83 0 154 199 98 97 89 61 124 63 134 97 28 1 32 228 126 4 154 155 137 57 96 186 240 105 154 166 105 138 61 254 48 24 39 211 67 7 22 254 178 33 240 44 135 182 221 61 236 66 200 17 126 63 134 160 132 6 209 152 63 85 64 104 127 127 161 75 241 123 128 105 254 29 49 32 56 54 200 110 71 136 232 3 146 5 227 205 25 69 81 20 69 177 121 225 177 0 211 161 9 59 77 120 124 255 97 240 60 207 243 188 113 63 8 130 32 8 54 111 8 121 179 32 62 67 144 67 77 24 134 97 24 142 231 227 56 142 227 88 119 131 48 247 79 254 8 224 187 63 249 250 44 203 178 44 211 125 100 144 28 139 249 66 50 47 47 207 207 79 79 178 132 103 239 71 158 231 121 158 15 131 109 219 182 109 235 102 136 5 12 160 11 96 8 232 63 95 104 162 229 36 210 113 218 136 93 32 4 233 2 14 77 114 158 91 154 176 162 29 89 15 144 172 217 66 146 232 8 144 198 250 13 128 67 146 216 115 139 94 207 204 250 13 0 192 123 68 146 36 73 146 208 158 107 57 3 104 82 146 156 159 40 138 162 40 50 140 182 109 219 182 149 221 58 127 8 210 212 0 242 99 170 239 251 190 239 27 70 215 117 93 215 209 126 77 90 4 164 29 199 91 233 130 204 45 212 252 139 109 150 101 89 150 165 79 104 157 56 2 72 203 197 152 169 231 119 179 154 73 133 49 4 201 50 196 212 223 203 27 202 48 27 114 28 199 113 28 89 173 138 195 153 3 104 194 232 87 72 129 228 235 186 174 235 186 170 123 51 34 57 9 143 194 235 86 156 175 235 186 174 235 49 25 111 22 221 138 243 124 47 98 27 64 77 113 158 92 55 208 107 117 116 113 88 5 22 93 68 27 6 177 197 184 233 60 62 190 190 222 223 15 195 221 221 195 195 237 237 106 95 196 104 197 121 118 202 178 44 203 210 48 78 203 209 242 123 12 194 171 80 139 3 209 226 60 110 135 124 159 203 5 25 246 145 0 30 13 66 99 193 177 231 107 55 2 176 71 159 246 225 236 232 241 85 85 85 85 117 190 127 185 85 210 231 42 227 249 190 239 251 190 55 205 166 105 154 166 145 191 68 64 242 252 175 175 143 143 239 111 13 166 223 243 20 231 105 200 143 245 216 179 241 72 16 141 253 179 35 183 56 207 10 187 65 72 253 155 219 0 210 67 16 20 231 217 66 139 108 104 95 216 1 227 113 220 223 169 119 186 185 9 130 235 107 254 164 172 62 102 41 225 92 40 200 81 96 8 146 160 63 63 77 243 251 123 222 34 24 2 120 127 127 123 251 252 164 59 220 206 12 64 255 103 15 205 147 193 0 144 116 113 8 98 77 198 11 253 71 108 189 96 143 7 129 213 132 214 29 128 103 63 216 195 213 78 46 54 204 50 211 229 131 51 14 225 53 225 16 90 17 243 126 84 124 112 112 240 127 254 1 239 98 79 150 168 109 20 16 0 0 0 0 73 69 78 68 174 66 96 130)! !

!DrGIcons class methodsFor: 'accessing' stamp: 'HilaireFernandes 11/26/2005 19:07'!
scale
	^ self formFromGraphic: #(137 80 78 71 13 10 26 10 0 0 0 13 73 72 68 82 0 0 0 47 0 0 0 47 16 6 0 0 0 35 183 79 239 0 0 0 6 98 75 71 68 255 255 255 255 255 255 9 88 247 220 0 0 0 9 112 72 89 115 0 0 0 72 0 0 0 72 0 70 201 107 62 0 0 2 19 73 68 65 84 120 218 237 155 221 141 194 48 12 128 219 211 45 193 19 18 3 192 0 172 128 88 162 111 8 49 0 3 176 69 150 64 172 192 0 48 0 18 79 140 209 123 138 170 90 117 157 31 39 78 56 127 47 149 142 107 147 124 117 156 186 132 166 81 50 208 247 227 99 211 252 72 119 233 187 25 68 67 126 165 187 86 55 152 216 182 165 206 36 255 65 153 3 143 232 105 134 27 162 169 70 8 21 239 72 215 117 93 215 193 8 167 83 202 151 1 159 18 176 99 60 80 248 244 13 112 237 15 95 191 50 227 219 241 116 55 96 124 125 255 99 37 83 197 118 56 116 106 199 158 143 93 143 194 182 87 93 138 42 117 106 210 169 100 122 134 20 79 238 14 251 230 96 119 225 243 41 170 24 164 133 227 237 83 66 221 132 75 141 147 20 144 234 186 152 88 90 124 172 240 66 35 62 117 7 248 30 243 252 34 90 122 220 226 13 199 139 63 157 142 199 195 129 123 209 204 46 62 93 131 110 57 181 148 194 38 91 123 249 132 151 93 89 22 35 156 111 224 220 149 101 94 15 217 26 138 253 220 181 125 87 177 249 133 51 191 157 228 46 205 83 209 182 227 25 98 251 203 253 74 33 185 7 174 72 225 190 142 107 41 207 149 114 138 73 41 249 174 19 87 216 168 112 111 120 43 201 208 119 54 201 41 71 56 36 174 178 12 125 254 79 78 238 119 41 238 237 165 169 44 197 169 126 0 181 161 194 83 163 187 12 132 240 20 175 51 129 11 162 178 10 223 162 166 204 147 44 213 236 247 231 243 118 219 247 155 205 110 183 90 233 76 129 32 226 195 69 89 225 210 3 43 29 100 183 48 158 74 108 4 47 151 235 245 98 49 252 253 122 189 92 238 247 225 188 247 251 249 252 124 164 135 87 46 108 169 6 139 244 199 227 118 123 189 116 77 128 4 139 215 136 142 131 20 15 75 112 141 96 30 80 241 86 184 49 198 24 3 191 56 160 177 107 128 46 182 211 144 145 11 111 0 38 18 75 61 212 34 252 95 113 22 64 69 174 21 111 83 17 124 126 215 20 53 38 248 199 103 48 194 173 88 56 67 164 7 88 61 54 130 225 209 126 206 187 197 77 97 222 37 171 144 168 112 97 84 112 102 190 244 187 76 69 81 68 248 3 221 179 150 104 213 141 47 93 0 0 0 0 73 69 78 68 174 66 96 130)! !

!DrGIcons class methodsFor: 'helpers' stamp: 'HilaireFernandes 3/29/2010 22:39'!
scaled16: symbol
	^ (self perform: symbol) scaledToSize: 16@16! !

!DrGIcons class methodsFor: 'helpers' stamp: 'HilaireFernandes 3/30/2010 13:29'!
scaled24: symbol
	^ (self perform: symbol) scaledToSize: 24@24! !

!DrGIcons class methodsFor: 'accessing' stamp: 'HilaireFernandes 11/26/2005 19:07'!
script
	^self formFromGraphic: #(137 80 78 71 13 10 26 10 0 0 0 13 73 72 68 82 0 0 0 48 0 0 0 48 16 6 0 0 0 7 146 37 196 0 0 0 6 98 75 71 68 255 255 255 255 255 255 9 88 247 220 0 0 0 9 112 72 89 115 0 0 0 72 0 0 0 72 0 70 201 107 62 0 0 15 228 73 68 65 84 120 218 237 92 87 76 85 89 23 94 32 72 17 112 20 145 50 2 14 14 32 29 212 40 168 1 6 46 96 67 29 49 106 98 124 48 177 36 250 98 124 149 7 19 95 124 241 197 18 77 196 68 52 64 148 166 210 171 151 38 69 229 114 17 27 88 232 92 122 239 229 252 15 123 214 172 195 185 231 220 123 65 248 199 127 242 175 151 29 206 89 187 173 181 87 251 246 185 232 193 60 226 56 248 63 1 0 192 166 77 155 54 109 218 4 48 51 51 51 51 51 3 208 212 212 212 212 212 164 167 183 76 211 113 92 125 125 125 125 125 61 247 127 250 139 198 198 198 198 198 198 56 78 161 80 40 20 10 142 187 116 233 210 165 75 151 240 128 46 249 65 85 95 64 81 81 81 81 81 209 63 45 6 105 154 155 155 155 155 155 91 190 118 122 122 122 122 122 154 20 209 219 219 219 219 219 203 113 114 185 92 46 151 47 157 34 244 151 201 164 150 156 152 216 169 93 110 154 239 122 0 214 174 93 187 118 237 90 0 63 63 63 63 63 63 128 172 172 172 172 172 44 90 221 98 231 145 84 64 96 96 96 96 96 160 250 243 246 246 246 246 246 118 128 236 236 236 236 236 108 245 231 72 159 63 127 254 252 249 179 122 255 210 210 210 210 210 82 128 218 218 218 218 218 90 128 252 252 252 252 252 124 117 190 79 159 62 125 250 244 137 4 142 124 194 121 150 155 152 37 208 223 22 22 22 22 22 22 0 219 183 111 223 190 125 59 64 74 74 74 74 74 10 128 145 145 145 145 145 209 194 21 33 169 128 226 226 226 226 226 98 245 231 111 222 188 121 243 230 13 128 167 167 167 167 167 39 192 236 236 236 236 236 44 128 74 165 82 169 84 0 213 213 213 213 213 213 0 29 29 29 29 29 29 0 111 223 190 125 251 246 45 245 111 109 109 109 109 109 5 240 246 246 246 246 246 166 254 56 95 121 121 121 121 121 57 192 200 200 200 200 200 8 245 243 245 245 245 245 245 5 248 246 237 219 183 111 223 150 94 208 66 11 195 86 168 112 230 162 200 34 112 31 15 30 60 120 16 19 131 10 210 93 17 11 118 65 219 182 109 219 182 109 27 64 93 93 93 93 93 29 192 138 21 43 86 172 88 1 96 96 96 96 96 96 0 176 101 203 150 45 91 182 0 184 184 184 184 184 184 0 108 221 186 117 235 214 173 0 25 25 25 25 25 25 0 198 198 198 198 198 198 180 193 240 240 240 240 240 112 82 132 191 191 191 191 191 63 192 202 149 43 87 174 92 169 174 120 39 39 39 39 39 167 165 19 176 148 75 99 113 134 214 133 132 174 9 159 99 182 228 234 234 234 186 121 51 192 221 187 119 239 222 189 75 163 104 91 151 30 127 121 203 233 91 255 91 190 123 33 243 160 2 196 222 77 76 76 76 76 76 144 171 13 14 14 14 14 14 6 248 248 241 227 199 143 31 233 32 177 224 12 96 106 106 106 106 106 202 92 209 202 149 0 223 191 127 255 222 216 8 112 226 196 137 19 39 78 252 45 110 181 52 214 64 106 113 232 18 164 98 129 144 208 84 209 183 71 68 68 68 68 68 144 239 150 201 100 50 153 140 248 31 61 122 244 232 209 35 128 130 130 130 130 130 2 128 216 216 216 216 216 88 114 97 104 73 56 174 157 157 157 157 157 29 128 82 169 84 42 149 0 61 61 61 61 61 61 0 247 239 223 191 127 255 62 157 60 180 20 119 119 119 119 119 119 214 207 214 86 90 208 82 138 225 43 136 227 0 206 158 61 123 246 236 89 0 150 21 145 133 8 197 106 109 109 109 189 126 61 192 129 3 7 14 68 70 106 159 203 64 59 139 110 244 250 245 235 215 175 95 147 224 208 68 199 199 199 199 199 199 137 239 242 229 203 151 47 95 6 72 72 72 72 72 72 0 184 122 245 234 213 171 87 41 104 99 44 193 141 99 48 70 193 14 15 15 15 15 15 3 236 222 189 123 247 238 221 20 107 34 35 35 35 35 35 1 238 221 187 119 239 222 61 128 175 95 191 126 253 250 21 192 214 214 214 214 214 86 90 208 66 197 224 188 44 25 5 216 179 103 207 158 61 123 0 172 172 172 172 172 172 0 186 186 186 186 186 186 152 34 70 71 217 223 221 221 0 205 205 205 205 205 205 52 78 76 76 76 76 76 204 15 40 64 219 201 23 154 58 198 6 180 128 209 209 209 209 209 81 106 47 94 188 120 241 226 69 128 184 184 184 184 184 56 128 227 199 143 31 63 126 28 96 195 134 13 27 54 108 160 152 129 2 71 114 117 117 117 117 117 5 120 245 234 213 171 87 175 0 214 172 89 179 102 205 26 0 123 123 123 123 123 123 128 243 231 207 159 63 127 30 224 230 205 155 55 111 222 36 147 79 76 76 76 124 250 84 187 75 226 191 231 243 177 172 6 64 95 95 95 95 95 31 96 199 142 29 59 118 236 160 247 3 3 3 3 3 3 204 213 124 251 198 98 97 96 32 192 224 224 224 224 224 32 128 92 46 151 91 88 0 228 229 229 229 229 229 105 213 131 122 161 35 44 196 22 90 200 48 11 224 184 156 156 156 156 156 28 142 99 39 136 227 88 144 229 56 230 99 137 111 118 118 118 118 102 134 227 88 144 91 88 59 61 205 113 44 189 229 56 75 75 75 75 75 75 142 99 10 166 130 10 219 169 169 169 169 201 73 142 155 156 156 156 212 212 50 203 101 235 28 31 103 127 143 141 177 194 108 116 148 227 216 193 162 22 247 137 244 228 201 147 39 79 158 104 47 216 244 53 171 68 187 9 33 95 91 91 91 91 91 27 21 40 125 125 125 125 125 125 0 103 206 156 57 115 230 12 165 111 231 206 157 59 119 238 28 101 77 73 73 73 73 73 73 228 10 196 242 125 77 217 11 246 195 19 138 193 18 199 121 252 248 241 227 199 143 105 126 177 93 161 203 153 157 165 86 56 62 186 36 126 127 190 120 113 124 36 12 210 218 72 77 1 56 113 80 80 80 80 80 208 252 133 107 74 223 248 245 129 135 7 192 245 235 215 175 95 191 78 166 138 5 12 22 54 24 108 249 133 14 199 1 248 248 248 248 248 248 144 15 215 116 12 196 214 115 242 228 201 147 39 79 18 15 174 99 116 116 116 20 235 10 142 163 24 53 59 59 59 203 87 12 191 157 155 35 133 72 205 39 140 21 35 35 35 35 195 195 116 192 22 173 0 134 121 144 15 148 226 195 22 243 253 210 210 210 210 178 50 128 248 248 248 248 248 120 234 31 20 20 20 20 24 8 96 102 102 102 102 102 70 37 189 163 163 163 163 163 35 109 28 11 55 39 39 39 167 223 126 211 108 1 194 117 204 205 1 4 4 4 4 248 251 3 232 233 233 233 233 233 81 154 248 240 225 195 135 177 177 36 112 49 69 162 192 165 20 194 231 19 198 142 236 236 236 236 172 44 182 63 115 115 237 130 151 84 128 112 97 82 2 23 18 166 137 195 195 195 195 67 67 0 83 83 83 83 83 83 116 18 100 50 153 44 44 12 224 216 177 99 199 142 29 163 241 253 253 253 253 119 236 80 47 204 108 109 109 109 237 236 196 179 20 20 128 208 50 1 88 144 94 187 22 192 193 193 193 193 193 129 250 197 199 199 199 199 197 137 11 92 147 107 145 154 71 76 49 139 169 115 36 45 32 56 56 56 248 143 63 212 43 65 77 39 146 227 0 74 74 74 74 74 75 137 23 79 20 198 4 228 83 42 149 202 218 90 218 200 187 119 239 222 189 123 71 227 87 86 86 86 86 84 144 0 16 194 64 254 218 218 218 90 228 7 96 245 68 126 62 241 57 59 59 59 59 59 211 123 44 140 26 27 27 27 155 154 196 179 31 49 215 34 20 52 219 147 186 98 246 238 221 187 119 223 62 202 130 126 200 2 56 14 224 229 203 151 47 11 11 9 106 16 158 0 33 63 182 88 41 34 97 233 30 16 16 16 16 16 64 10 197 231 152 182 10 55 222 215 215 215 215 223 79 130 86 42 149 202 154 26 6 129 188 123 71 233 31 142 231 233 233 233 233 229 197 158 127 255 206 44 200 198 70 125 111 216 95 40 88 97 172 19 238 11 91 12 181 194 254 8 181 172 94 189 122 245 234 213 63 104 1 252 86 74 208 82 38 137 133 10 18 186 160 150 150 150 150 230 102 138 9 88 186 35 168 215 217 217 217 217 217 73 227 123 121 121 121 121 122 82 54 132 227 187 185 185 185 97 133 251 235 175 196 143 65 125 227 198 141 27 29 29 1 86 173 90 181 202 204 76 125 195 124 75 18 10 92 76 176 66 87 197 223 63 63 102 240 229 179 16 18 141 211 28 7 16 18 18 18 18 18 162 14 199 74 241 35 177 60 154 167 225 191 4 190 111 223 190 125 251 247 211 194 93 92 92 92 156 157 105 163 56 31 10 192 198 198 198 198 214 150 149 246 214 214 234 243 33 58 138 253 249 80 7 199 1 24 26 26 26 26 26 206 23 24 0 171 204 39 38 116 59 96 252 119 124 5 136 241 239 223 191 127 255 129 3 204 114 123 123 117 87 128 164 5 20 22 22 22 22 22 138 111 68 147 5 72 229 191 24 148 145 175 173 173 173 173 189 29 32 55 55 55 55 39 135 20 83 84 84 84 36 151 207 63 161 0 0 10 133 66 81 83 67 127 99 191 202 202 202 202 202 74 114 153 72 13 13 13 13 13 13 196 143 174 10 45 15 247 83 87 87 87 199 119 129 133 133 133 133 114 57 213 53 98 39 92 44 40 167 165 165 165 165 165 49 152 218 210 242 7 20 32 20 180 80 49 154 78 12 199 73 99 47 194 60 28 211 77 116 41 211 211 211 211 51 51 12 83 105 105 1 72 78 78 78 78 73 97 165 124 110 46 19 8 223 117 76 78 78 78 78 77 1 152 152 152 152 152 154 18 56 151 152 152 152 152 148 196 92 154 74 69 2 196 182 189 189 189 189 173 141 129 141 37 37 132 229 224 123 172 99 240 38 76 234 224 137 201 105 201 178 32 0 128 208 208 208 208 208 80 118 114 209 165 136 157 124 126 182 48 55 7 224 225 225 225 225 225 161 62 81 71 71 71 7 10 132 227 168 14 120 255 254 253 123 188 87 208 215 167 152 129 10 11 13 13 13 149 201 152 101 153 152 208 252 22 22 22 22 230 230 76 96 238 238 0 235 215 175 95 191 126 61 139 29 94 94 44 29 30 25 33 23 136 45 98 79 88 103 244 247 247 247 15 12 204 183 52 133 130 210 88 49 129 243 229 132 239 35 35 35 35 15 30 100 7 161 187 91 119 5 168 221 7 160 15 47 41 41 41 41 41 33 223 138 207 81 208 82 138 75 76 76 76 76 76 36 212 19 41 38 38 38 230 254 125 128 176 176 176 176 176 48 113 203 146 202 199 197 44 79 44 89 224 243 35 36 129 22 101 99 99 99 99 99 67 48 59 43 211 164 199 215 101 125 124 66 176 240 224 193 131 7 15 29 2 200 204 204 204 204 204 100 177 97 255 254 191 197 173 54 231 130 92 144 84 133 200 127 31 17 17 17 17 30 78 190 22 77 59 53 53 53 245 217 51 226 71 65 8 179 145 244 244 244 244 180 52 230 42 58 58 212 179 145 234 234 234 106 140 5 28 199 124 125 125 61 193 210 232 138 80 240 152 68 32 180 34 38 120 109 88 147 54 216 90 44 155 210 149 212 20 128 46 2 79 42 226 249 98 166 200 47 72 176 31 166 127 167 78 157 58 117 234 20 229 251 136 151 35 255 208 208 208 208 240 48 249 226 188 188 188 188 252 124 134 179 143 143 3 84 84 84 84 148 151 3 60 125 250 244 41 194 202 0 52 14 22 124 13 13 13 13 95 190 16 102 131 176 52 6 93 19 19 19 19 19 19 128 157 59 119 238 220 185 83 51 166 165 201 178 132 177 68 216 239 208 161 67 135 14 31 102 49 69 165 250 1 5 224 128 24 252 112 3 124 141 11 99 128 48 75 224 56 134 255 95 184 64 23 25 24 116 177 80 51 51 51 51 91 181 138 46 54 100 50 153 44 36 132 158 187 187 187 187 123 120 48 75 90 181 138 92 162 141 141 141 141 181 53 192 174 93 187 118 5 4 48 126 83 83 74 91 43 42 42 42 42 43 105 47 161 161 161 161 33 33 148 182 10 215 41 244 229 98 2 215 197 21 62 123 246 236 89 106 42 139 69 98 5 160 20 25 168 15 58 63 184 242 159 107 187 196 230 47 12 65 183 27 55 110 220 184 113 131 96 105 252 27 175 18 133 138 197 171 76 36 103 103 103 231 223 127 23 119 129 28 199 78 246 174 93 12 6 207 206 6 248 242 229 203 151 47 95 8 28 188 112 225 194 133 139 23 73 113 82 49 67 74 208 186 184 150 197 184 30 36 209 44 8 125 121 68 4 193 171 82 216 8 182 152 222 229 230 230 230 242 111 128 48 171 193 160 156 159 159 159 95 80 192 210 204 228 100 245 27 163 170 170 170 170 215 175 233 4 226 85 165 148 171 72 79 79 79 79 79 7 184 118 237 218 181 107 215 0 214 173 91 183 110 221 58 128 59 119 238 220 185 125 155 146 7 149 74 165 234 232 160 254 105 105 105 105 25 25 116 39 205 31 151 227 88 61 82 84 68 87 164 66 129 11 99 195 225 195 135 15 255 249 39 139 69 109 109 63 160 0 20 180 24 188 42 5 69 32 20 160 80 176 188 126 243 102 122 142 10 196 202 23 99 203 149 43 87 174 68 71 147 107 42 44 44 44 124 249 146 197 156 177 49 154 167 185 185 185 185 169 137 185 160 178 50 128 154 154 154 26 165 146 4 123 235 214 173 91 183 111 83 1 120 250 244 233 211 167 79 51 69 88 90 178 32 60 57 201 4 90 92 76 138 69 87 135 152 211 135 15 31 62 32 134 197 113 44 157 118 119 103 224 93 99 163 230 96 140 73 70 114 50 171 131 16 34 89 148 2 164 76 74 42 234 99 235 231 231 231 231 235 75 62 30 159 227 70 177 174 136 138 138 138 138 138 2 136 142 142 142 142 142 102 233 91 121 57 75 27 3 3 213 191 27 178 183 183 183 119 112 96 223 223 56 57 209 115 252 46 9 249 19 18 18 18 226 227 73 112 245 245 245 245 13 13 116 17 196 175 128 1 212 225 114 60 56 72 53 53 53 53 181 181 116 95 193 63 120 154 238 13 22 74 106 117 192 208 208 208 208 208 16 45 124 112 112 112 112 96 64 122 0 49 31 169 41 109 211 214 143 255 94 236 96 72 109 84 232 162 196 198 209 36 34 41 62 177 117 139 173 203 206 206 206 110 195 6 230 138 91 91 73 129 11 174 3 80 195 153 153 153 153 8 175 254 242 139 246 244 13 81 203 188 188 188 188 130 2 58 33 85 85 85 85 85 85 234 253 248 31 183 242 5 139 24 14 62 199 155 57 172 164 133 130 225 7 103 77 89 141 148 194 196 98 139 20 214 37 5 83 3 176 111 68 147 146 72 17 186 146 228 133 140 38 147 210 20 3 54 111 222 188 217 213 149 120 123 122 122 122 248 232 32 150 250 221 221 221 221 221 221 84 64 213 212 212 212 40 20 12 154 248 240 129 230 71 108 6 113 126 49 23 40 60 185 218 224 101 77 217 157 148 98 180 209 98 93 144 228 149 36 154 78 127 127 127 63 255 38 75 24 11 48 104 99 158 45 140 1 198 198 198 198 70 70 244 183 161 161 161 161 129 193 252 139 30 190 162 144 31 199 71 20 20 125 49 255 50 93 216 226 183 156 252 86 184 78 97 43 230 195 209 81 44 164 141 138 138 138 58 122 148 92 144 174 164 22 3 80 224 136 109 32 206 221 219 219 219 219 211 35 110 9 218 79 135 250 137 19 146 38 188 157 47 32 225 231 31 63 11 97 229 126 228 200 145 35 71 143 254 0 22 36 140 242 66 193 105 114 81 98 190 20 93 12 230 255 72 88 47 224 60 101 101 101 101 101 101 212 31 211 70 161 239 255 183 145 100 12 208 4 175 234 114 49 131 124 152 183 187 186 186 186 186 184 144 192 17 126 198 207 88 90 90 90 90 90 91 73 224 8 107 11 113 249 159 149 240 228 47 212 5 73 42 224 197 139 23 47 158 63 103 5 141 149 149 238 193 25 91 20 180 183 183 183 183 151 23 251 230 147 255 139 25 115 115 115 115 115 115 202 239 49 120 227 149 30 126 5 45 134 203 255 140 180 216 44 72 18 11 210 36 96 49 62 169 172 1 113 120 188 219 69 62 196 106 132 124 248 222 205 205 205 205 205 77 60 43 89 182 31 139 254 3 36 89 9 227 197 66 87 87 87 87 103 39 9 92 46 151 203 209 55 35 94 63 55 199 96 229 162 34 114 33 98 249 179 248 92 218 211 192 255 5 90 82 23 4 0 240 252 249 243 231 207 158 49 120 21 79 238 220 28 131 137 221 220 230 99 36 0 4 31 75 249 108 97 140 208 85 224 98 177 229 103 164 37 43 196 80 112 98 112 52 0 125 209 230 232 232 232 136 159 254 113 156 184 207 22 10 78 172 114 149 18 180 182 202 243 223 66 106 117 0 194 182 120 193 161 82 169 84 252 95 9 46 54 239 215 53 182 104 93 176 158 158 158 190 190 238 252 186 208 82 196 22 252 80 236 135 177 32 92 80 106 106 106 106 74 10 83 132 240 35 89 169 147 41 85 105 106 226 215 93 72 203 231 132 22 42 120 93 176 32 95 95 95 95 31 31 29 230 198 161 216 255 138 160 43 62 196 181 241 130 97 185 130 163 152 98 23 59 142 46 193 126 185 200 193 193 193 97 227 70 118 127 209 216 8 144 147 147 147 147 151 71 63 72 97 164 225 87 146 120 243 131 63 130 83 40 20 10 165 114 249 22 252 111 163 186 186 186 186 15 31 232 139 58 132 245 181 145 64 35 255 198 48 247 51 145 186 5 252 7 6 48 156 116 123 179 113 22 0 0 0 0 73 69 78 68 174 66 96 130)! !

!DrGIcons class methodsFor: 'accessing' stamp: 'HilaireFernandes 11/26/2005 19:07'!
segment
	^self formFromGraphic: #(137 80 78 71 13 10 26 10 0 0 0 13 73 72 68 82 0 0 0 48 0 0 0 48 16 6 0 0 0 7 146 37 196 0 0 0 6 98 75 71 68 255 255 255 255 255 255 9 88 247 220 0 0 0 9 112 72 89 115 0 0 0 72 0 0 0 72 0 70 201 107 62 0 0 1 123 73 68 65 84 120 218 237 218 61 174 130 64 20 134 225 195 141 37 44 130 130 109 77 76 166 51 70 54 227 90 168 220 137 149 137 123 176 33 153 91 80 220 128 23 231 15 228 104 222 167 33 152 104 228 251 6 144 120 68 0 0 192 55 114 110 188 197 212 207 58 31 75 224 161 118 105 111 155 11 184 40 182 62 160 79 147 24 88 236 10 167 152 57 43 93 130 16 42 177 128 240 21 109 173 181 214 254 157 49 211 125 100 153 254 202 153 223 18 252 162 194 131 159 110 41 98 44 242 18 20 26 220 243 37 234 122 109 154 166 17 57 159 203 178 44 183 62 108 61 22 190 9 23 197 120 133 23 197 126 95 85 85 37 114 187 245 125 223 139 220 239 93 215 117 34 143 71 219 182 45 103 66 164 185 75 139 255 102 59 20 225 220 229 82 215 117 237 220 112 70 56 71 17 153 124 193 83 196 74 98 131 167 136 133 228 6 111 140 49 198 80 68 182 216 7 172 105 240 20 145 232 116 58 30 15 135 244 39 91 138 80 130 34 148 160 8 37 40 66 9 138 80 130 34 148 160 8 37 40 66 9 138 80 130 34 148 160 8 37 40 66 137 247 20 193 200 165 215 58 69 204 255 35 136 25 161 69 12 123 190 128 41 32 153 175 136 65 236 152 13 162 189 46 34 190 0 230 156 18 77 139 24 7 201 128 217 219 252 31 228 251 138 96 108 124 36 119 69 199 143 225 51 158 46 34 57 35 151 175 95 247 163 128 32 207 35 151 195 254 214 223 235 107 164 143 92 114 51 94 72 238 192 25 18 229 6 79 17 11 33 216 141 228 78 250 225 195 253 2 130 11 51 156 184 61 65 99 0 0 0 0 73 69 78 68 174 66 96 130)! !

!DrGIcons class methodsFor: 'form' stamp: 'HilaireFernandes 10/29/2010 10:38'!
segment: thick color: color dash: dash
	^ self iconWithBackground: (self segmentMorph: thick color: color dash: dash).
! !

!DrGIcons class methodsFor: 'form' stamp: 'HilaireFernandes 10/29/2010 10:38'!
segment: thick color: color dash: dash arrow: symbol
	| morph |
	morph := self segmentMorph: thick color: color dash: dash.
	morph perform: symbol.
	^ self iconWithBackground: morph
! !

!DrGIcons class methodsFor: 'form' stamp: 'HilaireFernandes 11/16/2011 21:39'!
segment: thick color: color mark: mark
	| morph |
	morph := DrGSegmentMarkMorph new.
	morph forcedMark: mark;
		color: color;
		setVertices: {0@0. self menuExtent x@0};
		borderWidth: thick.
	^ self iconWithBackground: morph.
! !

!DrGIcons class methodsFor: 'form' stamp: 'HilaireFernandes 10/29/2010 10:22'!
segmentMorph: thick color: color dash: dash
	^ (LineMorph from: 0@0 to: self menuExtent x @0 color: color width: thick) 
		dashedBorder: dash! !

!DrGIcons class methodsFor: 'accessing' stamp: 'HilaireFernandes 11/26/2005 19:08'!
style
	^self formFromGraphic: #(137 80 78 71 13 10 26 10 0 0 0 13 73 72 68 82 0 0 0 48 0 0 0 48 16 6 0 0 0 7 146 37 196 0 0 0 6 98 75 71 68 255 255 255 255 255 255 9 88 247 220 0 0 0 9 112 72 89 115 0 0 0 72 0 0 0 72 0 70 201 107 62 0 0 12 96 73 68 65 84 120 218 237 155 93 108 83 231 29 198 127 254 78 236 99 39 118 18 199 78 26 39 77 8 36 144 16 88 10 101 172 104 131 169 163 21 157 4 148 125 73 211 54 42 173 163 157 52 169 218 197 180 155 169 149 216 214 74 211 46 182 21 181 218 84 58 84 237 134 177 93 180 97 85 183 174 148 22 10 11 4 200 23 249 78 252 17 219 241 71 142 125 252 253 113 236 93 28 101 233 202 118 81 8 132 66 158 27 203 199 246 123 254 239 243 188 239 255 253 191 207 121 13 107 88 195 26 214 176 134 53 172 97 13 247 35 84 183 246 243 114 121 181 59 112 119 65 245 169 249 212 222 234 45 143 31 223 177 227 201 39 87 174 11 135 15 95 184 112 234 20 124 247 187 175 191 254 203 95 174 92 187 127 252 227 247 190 247 211 159 194 231 62 247 242 203 199 142 173 92 187 3 3 63 252 225 179 207 222 252 239 213 43 23 202 26 110 6 107 2 172 50 214 4 88 101 172 9 176 202 88 19 96 149 113 203 85 208 103 21 133 130 40 46 46 66 58 61 61 61 51 3 26 141 201 100 50 129 32 116 118 118 118 222 185 56 238 187 25 32 203 217 108 54 11 129 192 169 83 167 78 193 182 109 178 28 143 67 48 216 215 215 215 7 201 228 196 196 248 248 157 139 231 190 19 32 18 57 113 226 248 113 248 246 183 119 237 234 238 134 39 158 232 234 170 173 133 245 235 237 118 139 5 36 105 100 100 116 244 206 197 115 223 9 240 200 35 205 205 102 51 124 243 155 143 62 186 125 59 216 237 86 171 201 4 62 95 48 24 137 64 69 69 93 93 93 221 157 139 231 190 177 34 186 186 90 91 235 235 225 39 63 121 250 233 253 251 161 162 66 163 41 151 97 108 108 106 202 227 129 159 255 252 181 215 222 125 23 178 217 66 161 80 184 217 187 124 122 43 226 150 5 248 232 163 35 71 30 127 124 229 136 250 252 231 95 121 229 111 127 131 183 222 26 29 125 251 237 155 111 71 146 98 49 73 130 55 222 248 197 47 126 247 59 248 254 247 31 123 236 161 135 96 227 70 69 136 247 222 251 240 195 203 151 161 175 239 210 165 185 57 40 151 183 109 251 202 87 160 171 107 255 254 3 7 192 231 187 122 245 234 85 200 102 37 73 146 160 178 210 106 173 174 134 230 230 222 222 135 30 90 190 207 209 163 61 61 221 221 55 47 192 61 151 130 100 89 150 101 25 94 121 229 133 23 126 243 27 120 230 153 3 7 118 239 134 141 27 91 90 234 234 96 120 248 250 245 153 25 16 197 68 34 157 6 183 59 155 53 24 160 187 251 192 129 131 7 193 227 185 124 249 210 165 101 226 255 67 148 90 163 209 222 134 154 241 158 19 224 79 127 122 249 229 55 222 128 47 127 121 243 230 150 22 104 107 107 106 178 219 193 239 95 88 136 197 192 227 153 159 95 88 128 15 62 152 154 138 199 97 251 246 35 71 158 125 22 226 241 64 32 16 128 124 62 149 74 167 151 219 83 169 212 106 149 10 108 54 151 203 229 90 249 120 239 25 1 66 33 133 88 163 49 155 149 36 248 210 151 182 110 221 184 17 10 133 76 38 145 128 201 201 217 217 217 89 8 4 98 177 108 22 114 185 150 150 238 110 176 219 55 108 232 232 128 104 84 249 252 147 168 173 109 109 109 109 5 147 201 102 179 217 64 165 42 151 101 121 229 226 190 103 4 56 126 252 165 151 94 125 21 158 124 114 215 174 222 94 104 104 168 171 51 155 33 26 141 68 194 97 152 159 95 88 16 69 56 123 214 239 47 22 97 199 142 31 252 224 200 17 16 69 159 207 235 5 89 254 239 197 87 163 209 233 116 58 101 228 55 55 3 148 203 229 178 82 118 168 87 144 181 207 188 0 191 255 253 139 47 190 250 42 236 222 189 126 125 109 45 180 180 212 215 87 85 65 34 17 143 139 34 76 77 121 189 11 11 240 246 219 231 207 143 143 67 103 231 183 190 245 157 239 64 54 155 72 72 18 132 195 83 83 211 211 55 182 107 179 53 55 187 92 74 10 250 111 194 85 170 79 191 212 254 127 124 102 5 240 122 103 102 188 94 176 88 178 217 88 12 190 250 213 71 31 221 181 11 244 122 189 94 163 129 177 177 137 137 185 57 184 120 241 250 245 249 121 24 31 119 187 195 97 176 219 59 58 58 59 97 102 230 220 185 243 231 33 26 157 155 155 157 133 96 112 124 124 108 12 146 201 104 52 26 5 65 168 171 179 219 149 113 175 20 219 43 75 252 18 238 122 47 40 149 74 36 82 41 56 117 234 181 215 78 158 92 190 30 10 77 77 205 206 194 11 47 28 57 242 141 111 128 205 86 93 109 50 193 216 216 232 232 208 16 120 60 126 127 48 8 217 172 78 103 52 66 44 150 76 102 50 240 206 59 47 189 244 226 139 48 53 117 246 236 153 51 80 42 41 244 118 117 237 219 247 196 19 96 48 152 76 70 35 120 189 87 174 92 185 2 109 109 59 119 238 220 249 241 136 238 147 25 176 84 78 30 63 254 171 95 253 225 15 176 103 143 203 37 8 80 89 153 203 37 18 112 248 240 161 67 123 246 64 83 147 211 89 83 3 145 72 32 224 118 67 52 42 138 241 56 140 143 135 66 233 52 24 12 14 135 211 185 220 174 40 94 188 248 254 251 176 110 157 221 46 8 208 208 96 54 107 181 112 253 250 95 254 114 242 36 232 116 21 21 149 149 80 44 42 158 81 38 19 143 199 227 183 175 159 119 237 12 120 243 205 19 39 254 250 87 216 191 191 171 171 190 30 214 173 107 109 109 110 134 116 58 147 41 22 97 231 206 246 118 167 19 74 165 100 50 28 134 120 92 153 41 147 147 74 170 121 231 157 243 231 135 135 97 243 102 149 202 100 2 151 203 225 176 217 192 225 104 111 111 111 135 206 78 155 77 167 131 124 62 155 77 38 225 253 247 7 7 231 231 111 140 163 88 204 231 243 121 184 107 83 208 210 206 117 165 81 81 225 241 40 139 230 129 3 143 63 14 193 96 44 86 40 192 250 245 46 87 125 61 104 181 122 125 85 21 92 187 54 50 210 223 15 181 181 85 85 38 19 244 247 143 140 184 221 160 215 43 21 139 203 37 203 137 4 236 219 119 248 240 99 143 41 52 170 84 240 222 123 23 46 12 13 65 56 28 143 167 82 224 243 249 124 11 11 240 231 63 255 248 199 207 61 183 242 253 185 109 2 44 46 142 142 62 255 252 202 5 212 216 184 117 235 209 163 176 119 239 222 189 123 247 130 94 111 177 56 157 96 54 231 243 225 48 244 244 108 218 180 121 51 164 82 146 20 139 65 169 84 42 21 139 144 205 230 114 217 44 252 232 71 79 63 253 245 175 67 177 152 74 197 227 48 56 56 51 19 141 130 199 163 148 161 209 232 226 98 36 2 115 115 161 80 34 1 225 112 42 85 44 130 209 104 177 88 44 240 212 83 71 143 62 255 60 24 141 102 179 217 12 237 237 189 189 91 183 130 213 106 183 219 237 55 198 123 232 144 195 113 43 27 180 187 110 13 56 120 112 247 238 238 110 168 170 50 155 13 6 72 165 210 233 108 22 218 218 26 26 204 102 144 229 114 185 84 2 143 103 118 118 108 12 212 106 101 68 59 28 74 217 216 217 217 210 226 116 66 79 79 79 207 150 45 160 209 148 74 217 44 140 140 76 76 76 78 194 249 243 202 12 73 38 75 37 173 22 218 219 29 14 65 128 13 27 122 122 54 109 130 107 215 206 156 57 123 86 17 182 84 130 217 217 193 193 225 225 219 215 223 187 102 13 88 92 140 197 210 105 232 234 234 234 218 180 9 156 206 182 182 222 94 152 159 31 31 191 120 17 212 234 124 94 163 1 189 222 96 88 183 14 76 38 131 65 163 1 131 193 106 181 88 64 16 170 170 26 26 64 20 131 65 143 7 74 165 66 193 239 135 142 142 166 38 155 13 250 250 254 249 207 92 14 58 59 187 187 183 108 129 109 219 218 218 172 86 200 231 115 185 92 14 250 251 99 49 173 22 252 126 191 63 28 94 142 107 233 243 123 94 128 191 255 253 204 153 169 41 104 109 117 185 214 173 131 108 54 147 145 36 104 105 105 106 106 108 132 138 138 170 42 101 209 45 151 139 69 176 88 4 193 104 84 4 49 26 193 96 168 174 126 224 1 240 249 46 93 58 119 14 106 106 20 235 96 195 134 142 142 13 27 224 183 191 253 217 207 154 154 224 244 233 115 231 198 199 193 235 117 187 61 30 152 153 9 4 162 81 184 122 213 235 77 36 224 145 71 14 30 220 191 255 99 4 105 149 29 241 237 194 170 167 32 89 150 229 82 9 244 122 149 74 175 7 181 90 167 171 172 132 82 73 146 252 126 133 248 186 58 208 104 42 43 205 102 80 236 50 208 235 117 58 173 22 4 193 225 104 111 135 133 133 217 217 225 97 48 26 141 70 189 30 26 27 93 174 214 86 208 106 203 229 124 30 202 101 89 206 231 193 233 180 88 202 101 37 21 205 205 193 191 254 53 52 52 55 7 22 139 66 115 127 127 95 223 233 211 203 241 217 108 78 167 195 113 15 11 144 203 229 114 178 172 152 101 209 40 152 76 149 149 58 29 84 84 64 38 3 6 131 217 108 183 67 46 39 138 138 103 147 207 103 50 32 8 141 141 93 93 144 201 68 34 110 55 24 141 229 114 44 6 245 245 181 181 181 181 160 86 235 245 38 19 4 2 162 152 201 192 224 224 228 100 32 0 225 112 161 96 48 128 70 35 8 22 11 180 180 56 157 86 43 216 237 213 213 149 149 80 46 231 114 169 212 114 124 161 144 199 227 245 66 38 147 74 125 252 250 74 97 213 83 208 204 140 199 19 141 130 221 94 95 95 83 3 235 215 183 180 52 52 128 32 84 87 215 213 41 35 87 150 149 156 174 8 98 177 56 28 144 201 136 162 207 167 8 146 76 130 78 103 52 86 87 131 217 108 181 186 92 224 118 79 78 14 12 64 36 18 14 43 213 83 87 87 71 135 178 40 167 211 96 179 233 245 178 12 115 115 110 247 252 60 164 211 202 142 217 235 125 247 221 171 87 111 140 51 153 20 197 88 12 42 43 149 211 19 43 133 85 159 1 199 142 189 254 250 71 31 193 190 125 123 246 124 241 139 74 7 5 1 180 90 133 208 92 46 145 8 133 148 7 34 58 29 20 10 233 244 226 34 20 10 202 117 173 214 100 178 217 150 133 73 38 253 254 161 33 16 4 37 165 212 213 89 173 130 0 165 82 62 47 73 240 225 135 74 202 17 69 133 208 37 135 115 124 60 24 76 167 225 193 7 123 123 183 111 191 49 78 163 209 98 81 82 224 202 98 213 102 128 36 37 18 217 44 20 10 197 98 169 4 106 117 38 19 143 131 86 91 83 211 220 12 197 98 46 151 76 42 126 190 40 42 199 73 148 247 74 46 215 106 43 42 204 102 208 235 109 182 166 38 136 197 102 103 7 6 96 201 54 14 4 22 22 162 81 200 231 139 69 89 86 170 172 84 10 22 22 124 190 249 121 101 231 156 201 64 36 34 203 90 45 36 147 90 173 82 247 111 222 188 252 136 17 108 54 135 163 190 30 76 38 101 159 112 207 8 112 225 194 192 128 199 3 79 61 117 240 224 23 190 176 124 125 105 68 150 203 178 92 44 66 46 39 73 161 16 40 142 61 84 84 212 212 180 180 64 62 175 88 4 201 164 219 125 229 10 132 66 225 240 226 34 84 87 91 173 85 85 48 63 31 10 137 34 148 74 202 147 90 73 74 36 146 73 240 122 131 65 81 4 48 26 5 1 166 166 210 105 131 1 106 107 27 27 107 107 193 106 85 8 95 170 126 30 124 80 41 139 111 23 86 253 84 196 201 147 191 254 245 51 207 192 195 15 63 252 240 142 29 160 211 85 86 86 85 45 167 158 76 70 146 194 97 8 6 23 23 51 25 104 106 114 56 172 86 240 122 125 62 191 31 172 214 170 42 65 128 43 87 6 7 39 38 32 145 40 20 74 37 16 197 120 60 153 92 174 150 174 93 27 30 158 155 131 145 145 185 185 64 0 70 70 102 102 66 161 101 55 116 133 232 188 211 127 208 184 85 123 170 92 222 178 165 163 163 185 25 52 26 131 65 16 148 148 19 143 67 161 144 74 45 46 130 86 171 209 104 52 240 192 3 117 117 130 0 233 116 42 149 76 42 150 130 36 65 127 255 208 208 244 52 132 66 209 168 36 129 32 40 251 131 37 55 117 116 116 98 194 235 133 179 103 175 93 155 153 129 96 48 26 77 38 151 18 213 237 176 215 238 168 0 55 143 166 38 197 157 212 235 5 161 182 86 89 36 211 105 200 229 226 241 64 224 198 239 23 10 197 98 177 8 243 243 129 64 56 12 211 211 94 111 32 160 16 13 96 54 11 130 193 160 216 209 137 4 124 240 193 192 192 228 36 132 195 177 88 50 9 129 64 36 146 76 174 62 225 159 196 170 9 240 181 175 237 221 187 109 27 168 84 202 113 143 124 94 25 241 159 196 146 205 188 176 16 14 139 34 204 206 250 124 225 176 146 58 84 42 229 81 227 200 8 228 114 133 130 44 131 86 171 156 98 80 171 149 87 159 47 20 146 164 187 143 248 37 172 154 0 59 119 110 217 210 214 182 188 177 82 169 212 106 141 230 198 239 229 114 185 92 62 15 94 175 226 209 44 157 74 184 126 125 122 218 239 7 183 59 16 88 92 4 183 59 24 140 197 238 94 162 255 31 86 109 31 112 232 208 115 207 29 59 166 82 201 242 210 206 83 113 57 63 137 104 84 20 37 105 121 68 7 2 145 136 36 193 137 19 167 79 95 190 172 82 125 86 137 95 194 170 111 196 202 229 255 125 206 38 157 206 100 178 89 48 24 244 122 157 78 89 100 227 113 248 199 63 46 94 28 27 91 237 168 215 176 134 53 172 12 254 13 220 222 31 79 192 215 34 195 0 0 0 0 73 69 78 68 174 66 96 130)! !

!DrGIcons class methodsFor: 'accessing' stamp: 'HilaireFernandes 11/6/2011 12:02'!
text 
	^ self formFromGraphic: #(137 80 78 71 13 10 26 10 0 0 0 13 73 72 68 82 0 0 0 48 0 0 0 48 8 6 0 0 0 87 2 249 135 0 0 0 6 98 75 71 68 0 255 0 255 0 255 160 189 167 147 0 0 0 9 112 72 89 115 0 0 11 19 0 0 11 19 1 0 154 156 24 0 0 0 7 116 73 77 69 7 211 10 23 10 27 53 45 18 57 201 0 0 0 29 116 69 88 116 67 111 109 109 101 110 116 0 67 114 101 97 116 101 100 32 119 105 116 104 32 84 104 101 32 71 73 77 80 239 100 37 110 0 0 7 2 73 68 65 84 120 218 213 153 109 108 83 231 21 199 127 247 250 58 14 37 47 34 162 137 26 8 8 200 232 218 32 85 244 3 74 149 130 131 189 9 146 2 91 165 189 181 27 211 84 74 181 181 171 196 246 161 149 234 177 124 25 234 62 172 133 161 118 221 75 187 85 69 41 41 74 211 109 140 81 66 67 92 84 4 10 66 17 162 82 149 208 208 166 149 105 154 242 22 47 241 181 125 95 158 125 240 181 99 59 126 185 118 46 176 61 210 163 35 251 185 47 231 252 159 255 57 231 57 247 192 255 249 144 138 172 203 214 53 210 77 214 67 100 77 71 198 47 114 60 216 241 185 114 229 202 43 77 77 77 239 0 143 2 75 128 133 22 112 243 26 50 32 70 71 71 197 205 30 145 72 68 12 15 15 139 93 187 118 37 141 90 6 84 217 53 34 31 53 92 128 46 68 249 187 105 247 94 195 48 208 52 13 85 85 185 112 225 2 237 237 237 0 119 3 95 0 51 128 89 12 233 114 124 195 177 161 235 58 227 227 227 212 213 213 177 118 237 90 142 30 61 10 48 2 220 5 220 81 108 39 228 255 133 72 162 105 26 0 53 53 53 172 91 183 142 190 190 62 60 30 207 8 208 88 204 136 124 72 43 128 150 78 131 129 129 129 146 148 242 249 124 182 40 22 139 197 56 121 242 36 155 54 109 194 52 77 100 89 102 108 108 140 51 103 78 243 228 147 79 17 14 135 239 6 46 3 145 98 116 202 54 160 108 199 52 77 115 206 52 12 35 231 156 153 153 22 71 142 28 17 66 8 17 139 197 132 174 235 66 8 33 206 158 61 43 186 187 187 147 142 189 218 138 78 210 77 165 144 16 34 195 121 147 191 51 29 90 100 252 47 4 168 170 202 213 171 87 25 26 26 98 120 120 152 99 199 142 161 170 42 205 205 171 232 233 233 1 120 6 88 12 84 228 66 218 214 40 68 161 92 17 103 174 210 153 235 126 191 63 149 14 132 16 236 220 185 147 72 36 130 16 102 10 232 134 134 122 182 108 217 10 176 3 248 11 16 6 180 116 42 217 246 129 82 67 165 105 154 121 147 110 242 54 33 4 209 104 20 73 146 56 119 238 28 147 147 147 68 34 51 76 78 126 197 141 27 55 8 135 195 140 140 140 112 252 248 113 128 135 128 243 192 151 128 94 242 14 148 18 227 139 161 159 88 79 252 121 234 212 41 58 58 58 50 214 15 29 122 139 229 203 55 48 53 53 69 48 88 147 52 192 147 75 95 71 40 148 174 84 66 217 108 165 179 255 159 93 239 236 236 228 189 127 244 113 254 189 126 154 174 125 65 141 167 146 205 223 251 62 211 211 211 128 64 211 244 130 108 177 109 64 130 179 249 209 46 85 130 224 196 137 65 218 218 218 184 62 254 9 202 7 239 242 177 41 209 177 170 41 195 217 237 156 121 230 125 92 152 69 223 190 20 66 208 217 217 201 190 192 179 252 251 119 123 240 40 110 58 86 53 177 120 243 183 211 34 152 189 120 95 50 133 242 163 74 94 180 179 41 213 219 251 54 109 109 109 76 124 50 198 198 134 90 46 199 117 170 214 220 207 235 35 159 149 116 150 42 153 66 133 169 145 143 235 115 101 103 231 67 124 254 241 40 163 67 167 185 254 200 207 240 93 251 146 5 171 191 206 238 31 237 164 247 79 47 219 54 66 41 143 46 133 80 46 46 247 238 221 71 115 115 51 166 219 67 237 234 22 180 186 6 234 151 47 163 223 218 229 111 254 224 135 57 18 224 60 12 16 66 48 48 48 80 132 50 246 35 80 32 16 96 232 216 235 168 103 118 240 233 231 17 54 248 188 92 17 219 121 236 231 79 243 219 103 126 73 48 24 196 235 245 58 67 161 36 18 62 159 175 44 180 179 229 190 125 191 199 239 247 51 29 158 160 38 237 61 7 14 28 0 224 167 207 237 46 232 95 101 25 96 95 206 85 56 251 154 64 32 192 233 127 237 103 209 204 63 249 208 66 223 93 123 15 207 63 255 99 246 60 251 68 73 202 219 164 80 66 153 217 40 148 63 41 21 91 239 235 235 195 239 247 179 184 202 196 152 153 189 227 149 131 31 1 208 242 64 7 193 96 16 16 172 95 191 1 16 5 142 36 182 119 128 57 20 42 236 200 249 175 217 186 117 43 195 239 247 96 76 188 154 66 255 74 229 118 158 251 213 189 244 254 249 101 188 94 111 234 122 211 52 83 211 1 10 149 207 249 164 28 28 12 226 247 251 185 62 249 41 11 114 112 223 255 221 71 82 239 50 77 129 105 26 232 186 225 148 1 185 40 148 139 50 137 5 171 48 207 64 127 219 182 109 121 185 255 198 107 47 100 212 8 134 97 96 24 6 186 174 99 24 134 51 62 224 243 109 44 27 253 96 48 88 144 251 91 30 254 9 66 36 80 55 12 19 93 215 208 52 221 146 218 252 125 160 52 165 69 14 244 191 197 135 31 252 13 35 212 157 66 191 122 69 130 251 111 247 188 146 66 221 52 77 98 177 24 241 120 28 77 139 163 235 6 170 170 222 26 31 200 231 184 99 99 151 240 251 253 212 214 52 50 19 154 125 238 139 251 95 3 192 251 141 239 164 232 18 141 70 137 199 99 68 163 49 52 45 14 72 40 138 226 12 133 202 221 133 195 135 15 211 210 210 194 64 239 95 185 171 225 107 84 47 128 234 21 219 9 116 221 75 239 193 63 160 235 58 154 22 71 85 163 196 227 113 84 85 69 8 129 44 203 200 178 132 44 203 78 100 98 24 28 60 145 133 118 250 119 217 92 232 39 86 187 186 186 24 232 217 203 165 139 49 0 54 60 250 116 10 125 141 69 244 247 247 19 143 199 137 70 163 214 103 21 23 146 4 173 173 173 184 92 46 92 46 151 51 62 208 222 190 177 44 31 0 136 68 84 154 150 46 165 110 81 53 99 151 198 9 116 189 192 137 119 15 81 191 244 30 84 53 66 60 174 161 40 74 26 234 46 20 69 113 198 128 98 201 201 142 252 234 63 58 245 75 86 80 219 188 140 213 107 188 244 255 253 85 106 239 92 201 212 212 20 146 36 161 40 110 20 101 86 105 183 91 65 8 129 219 237 198 227 241 56 69 161 65 219 71 137 108 231 62 63 26 226 190 170 70 182 172 241 242 199 23 119 243 209 248 52 198 165 247 45 101 43 80 20 151 133 182 140 44 187 144 101 137 7 31 92 143 44 75 78 80 40 153 156 188 101 214 190 112 229 218 117 30 219 241 56 7 223 60 192 146 165 203 144 36 137 202 202 74 60 30 15 110 183 146 218 1 73 146 113 185 18 78 43 73 137 26 222 1 39 46 45 108 166 135 94 93 79 100 209 238 131 135 120 233 165 253 212 55 52 82 81 81 65 117 117 181 69 153 10 20 69 70 146 18 220 7 9 73 74 188 51 41 29 58 141 38 178 105 118 132 41 20 129 132 16 86 44 135 61 123 126 67 107 235 3 120 60 21 44 92 88 149 66 91 146 64 150 93 41 96 102 149 23 206 213 196 201 7 37 43 164 82 234 2 211 52 184 124 57 132 16 38 134 97 166 56 158 173 112 182 191 101 3 115 11 11 154 76 154 73 146 140 162 72 8 225 66 81 210 105 40 89 40 219 15 197 243 174 137 157 168 206 74 247 35 7 107 226 114 191 64 148 47 29 246 1 167 107 99 123 210 94 59 53 239 184 120 241 98 201 159 12 243 161 154 116 80 251 50 161 67 40 20 42 127 7 38 38 38 44 35 110 207 8 133 66 132 195 225 204 152 109 115 184 128 95 115 11 58 245 54 103 15 176 217 234 90 42 118 186 148 50 80 67 162 107 222 8 44 184 149 189 227 57 153 20 84 171 83 249 153 213 102 42 218 98 2 112 91 157 193 59 44 171 111 167 1 186 213 102 157 177 122 100 216 49 32 185 19 210 109 84 62 243 136 155 163 79 252 95 25 186 111 33 126 22 55 5 0 0 0 0 73 69 78 68 174 66 96 130)! !

!DrGIcons class methodsFor: 'accessing' stamp: 'HilaireFernandes 11/26/2005 19:08'!
translation
	^ self formFromGraphic: #(137 80 78 71 13 10 26 10 0 0 0 13 73 72 68 82 0 0 0 48 0 0 0 48 16 6 0 0 0 7 146 37 196 0 0 0 6 98 75 71 68 255 255 255 255 255 255 9 88 247 220 0 0 0 9 112 72 89 115 0 0 0 72 0 0 0 72 0 70 201 107 62 0 0 2 82 73 68 65 84 120 218 237 150 191 106 194 80 20 135 175 165 110 250 16 86 66 199 128 139 116 232 35 152 181 72 36 184 20 132 14 78 46 18 16 33 147 62 66 183 46 77 23 31 32 56 185 56 41 100 22 218 161 67 119 151 14 29 110 7 57 4 143 164 73 108 146 147 120 207 183 92 126 33 38 242 157 63 68 8 134 148 10 245 31 160 162 213 178 172 86 75 74 200 219 237 203 203 118 11 169 146 155 151 43 106 17 212 104 218 253 253 237 45 221 251 175 169 5 228 5 238 248 184 247 29 38 35 187 137 80 102 5 129 216 184 29 255 246 246 248 248 250 10 41 191 149 116 225 64 103 7 231 195 195 243 115 183 123 122 253 248 204 14 101 86 208 129 160 147 255 94 73 220 241 137 249 248 152 205 102 179 64 40 100 207 235 118 131 14 15 56 46 64 84 199 71 77 72 246 147 82 24 194 68 67 222 108 122 189 94 47 200 211 105 167 211 233 72 105 219 182 109 219 231 138 82 80 52 128 5 3 88 52 206 97 157 127 30 248 57 10 23 98 179 49 77 211 140 47 62 221 9 136 123 93 1 242 17 15 68 253 94 161 66 124 125 13 6 131 65 184 120 200 147 201 120 60 30 7 215 211 41 68 20 201 159 95 154 207 173 247 247 249 124 62 151 242 230 102 52 26 141 42 21 207 59 172 162 245 122 191 223 239 133 184 187 171 213 106 53 33 86 171 102 179 217 20 162 90 245 125 223 23 226 231 71 215 117 93 8 199 113 28 199 201 235 243 18 10 113 1 159 179 32 30 50 136 159 78 13 195 48 78 87 14 100 220 241 249 76 0 166 196 171 41 174 120 44 246 233 169 221 110 183 169 197 151 152 115 197 39 205 12 162 108 226 191 191 135 195 225 80 74 124 46 151 141 70 163 81 162 66 167 37 30 147 86 33 194 68 239 118 154 166 105 193 9 226 251 253 122 189 94 151 210 178 44 203 178 10 92 136 172 196 99 146 222 31 87 56 22 143 11 64 237 151 92 252 127 73 42 190 240 19 80 22 241 24 22 79 76 148 248 176 213 67 94 136 178 139 199 43 8 139 198 185 48 19 112 233 226 225 190 40 241 185 23 66 21 241 88 44 249 234 81 85 124 210 204 226 89 252 101 137 199 100 94 8 22 31 15 22 143 200 75 124 234 176 120 34 88 60 17 44 158 136 207 207 197 98 177 96 241 228 176 248 130 193 226 11 2 139 87 28 22 79 4 139 39 130 197 19 193 226 137 96 241 68 176 120 34 164 116 93 215 101 241 228 176 248 130 193 226 11 130 106 226 127 1 94 140 135 70 242 90 129 96 0 0 0 0 73 69 78 68 174 66 96 130)! !

!DrGIcons class methodsFor: 'accessing' stamp: 'HilaireFernandes 6/21/2007 16:24'!
undo
	^ self formFromGraphic: #(137 80 78 71 13 10 26 10 0 0 0 13 73 72 68 82 0 0 0 48 0 0 0 48 8 6 0 0 0 87 2 249 135 0 0 0 6 98 75 71 68 0 255 0 255 0 255 160 189 167 147 0 0 0 9 112 72 89 115 0 0 13 215 0 0 13 215 1 66 40 155 120 0 0 0 7 116 73 77 69 7 215 6 21 14 22 33 113 7 92 185 0 0 7 60 73 68 65 84 104 222 237 89 107 108 20 215 21 254 238 157 153 125 122 237 181 177 215 15 176 193 177 113 32 161 113 29 40 33 102 72 8 105 72 34 138 148 168 65 233 35 49 83 210 170 106 165 72 105 127 164 162 74 164 86 85 155 164 74 11 149 104 162 40 73 181 38 52 13 105 107 42 34 145 166 136 38 38 107 48 21 41 208 96 108 48 182 99 27 227 7 182 215 235 245 122 119 102 238 220 219 31 187 80 199 216 142 141 119 183 137 228 35 141 102 164 189 115 231 251 238 61 247 156 239 156 5 22 108 193 22 236 11 109 36 85 19 7 252 120 16 32 127 1 196 34 85 67 44 85 223 161 169 152 180 161 150 124 223 225 41 254 43 32 156 169 222 129 164 19 56 246 134 244 130 39 175 114 87 229 131 117 78 66 8 79 53 1 57 137 46 163 72 146 252 102 78 241 189 95 43 191 227 5 7 145 92 105 57 3 52 73 224 179 36 89 62 186 248 150 237 91 203 239 124 209 1 222 158 182 67 44 39 1 124 177 36 201 245 101 107 127 186 36 119 233 54 69 152 231 0 30 1 148 47 0 129 128 31 149 84 146 143 220 114 207 158 108 79 222 58 42 140 51 128 48 210 26 70 229 121 128 223 44 219 220 117 183 109 174 117 57 50 74 72 28 188 149 246 60 112 67 4 26 106 201 14 155 203 183 231 182 205 251 156 138 205 9 97 52 1 16 147 70 9 72 74 38 129 24 111 106 252 35 206 91 140 157 22 66 180 0 104 5 208 172 106 24 249 191 36 178 99 111 72 191 116 121 203 159 186 117 147 223 69 201 56 192 58 167 89 154 18 128 228 128 25 87 96 198 250 97 140 247 136 232 104 103 44 60 212 98 6 47 159 80 0 243 109 139 177 159 171 26 58 210 66 32 224 135 44 73 242 94 111 209 250 135 43 170 119 57 32 250 0 214 55 195 27 102 226 110 3 72 252 34 196 1 16 39 132 144 49 124 233 239 172 227 212 171 166 197 70 15 89 204 124 86 213 208 156 50 2 1 63 60 146 44 31 42 172 216 182 166 184 114 167 3 172 13 176 130 51 188 193 1 48 64 152 16 194 188 246 124 237 78 156 32 202 114 128 248 48 210 123 152 183 127 244 138 206 204 208 33 139 153 143 205 85 118 144 89 128 47 146 100 185 190 180 234 169 165 190 242 26 69 152 205 0 31 251 140 183 174 2 54 33 196 36 240 19 126 3 177 3 114 5 32 149 160 231 220 107 177 203 231 235 154 44 198 238 85 53 132 146 66 32 224 199 42 42 73 239 175 216 240 171 156 172 130 77 84 232 167 33 68 44 30 109 196 56 32 24 0 1 1 145 56 196 2 16 28 32 18 8 20 0 28 2 102 28 240 213 11 137 93 185 74 66 24 32 52 11 212 181 5 253 109 117 70 215 199 111 117 89 22 83 85 13 253 243 38 208 80 75 130 55 173 254 110 86 238 178 175 19 22 122 21 156 13 64 240 49 64 68 103 185 60 87 253 222 14 66 221 0 205 72 4 190 56 240 107 46 38 12 16 234 130 156 249 77 12 117 253 147 117 156 222 119 133 91 92 85 53 180 207 119 7 54 75 178 237 192 10 245 71 46 135 203 1 99 228 181 249 39 42 34 131 16 119 156 16 177 1 194 128 16 70 124 94 98 131 205 187 3 35 125 167 121 251 169 55 207 115 75 220 170 106 215 197 231 57 159 129 59 168 68 255 177 124 237 14 143 219 91 68 98 67 187 33 120 36 73 213 136 18 39 2 154 32 162 131 16 59 156 190 95 160 245 196 174 200 232 149 142 31 170 26 246 38 35 10 173 164 148 124 88 90 245 104 118 86 254 42 26 27 250 13 196 140 81 232 6 52 37 81 18 231 130 67 113 223 3 33 175 67 211 251 207 13 115 142 98 85 195 248 188 212 168 170 161 153 115 113 123 199 169 183 122 134 186 143 51 103 238 78 80 185 32 137 4 56 32 244 68 248 5 204 72 61 20 187 19 190 210 117 110 74 241 76 210 50 113 192 143 92 42 225 104 65 217 221 229 133 21 15 41 177 161 221 224 230 212 153 88 113 127 21 138 103 11 32 24 132 136 2 60 10 110 13 193 210 207 194 138 157 133 224 163 51 126 75 178 175 128 146 249 4 206 28 222 25 229 140 21 78 23 90 231 164 133 84 13 131 1 63 214 246 181 213 31 102 122 164 170 248 75 63 182 235 193 151 97 233 45 215 141 181 101 62 140 166 250 231 25 183 162 134 108 115 114 197 230 230 206 204 60 155 183 224 203 14 151 239 219 16 86 15 244 208 159 192 141 169 149 132 165 183 192 70 34 240 228 44 181 66 3 109 85 0 62 72 138 152 83 53 140 5 252 184 123 240 210 201 3 166 17 217 84 122 251 15 28 102 168 22 44 246 239 73 206 233 68 44 220 77 132 64 149 30 129 19 128 119 164 191 101 213 64 199 135 223 16 144 215 248 150 126 69 20 221 252 164 211 210 255 5 99 180 110 202 232 198 141 78 120 22 149 56 66 3 109 171 167 35 112 67 21 153 170 193 224 22 182 134 6 154 247 95 60 177 59 170 100 62 14 217 181 97 186 225 93 170 134 51 170 134 122 85 195 239 239 124 12 27 56 99 121 3 29 199 95 255 248 200 179 81 93 95 36 236 217 79 76 125 50 204 78 184 50 75 100 73 198 93 73 47 41 85 13 188 250 113 104 99 195 29 123 90 26 126 29 149 220 91 161 100 60 48 235 93 172 174 193 147 166 30 185 171 181 241 149 136 101 229 66 201 184 239 122 55 50 59 225 200 92 6 33 176 58 101 53 113 117 13 158 142 134 123 159 105 174 127 62 70 108 42 108 153 219 230 178 8 39 45 198 190 117 161 241 229 168 146 241 80 92 27 77 172 40 88 63 20 123 30 4 71 126 74 139 250 245 53 248 173 17 29 254 222 185 250 231 162 156 84 192 238 253 206 92 72 188 195 98 131 23 245 72 39 36 91 233 167 67 164 148 11 51 214 11 74 209 157 242 190 208 250 237 216 103 234 145 71 154 143 190 56 110 154 217 115 203 2 28 135 35 193 11 156 42 203 62 13 78 46 64 44 220 9 16 156 74 75 99 75 213 112 136 153 250 125 45 129 223 133 195 87 26 196 28 68 189 141 16 65 38 71 34 42 251 48 62 218 195 44 134 99 105 235 204 169 26 142 89 150 85 221 218 248 135 160 16 144 102 37 36 40 54 58 60 5 196 154 148 20 137 156 135 72 240 114 20 72 211 14 76 32 113 150 115 177 26 248 108 57 28 23 139 238 155 236 174 92 112 243 210 4 255 95 4 1 47 66 131 29 74 218 9 36 72 124 2 160 122 66 113 60 21 120 59 165 216 191 172 242 126 167 21 61 145 208 67 137 12 235 170 198 96 215 113 19 220 216 171 106 8 166 188 55 58 13 137 254 153 154 4 148 162 54 187 240 102 159 59 43 151 232 35 7 255 183 170 182 50 112 228 163 183 237 61 198 57 126 150 150 230 238 92 123 169 84 194 187 217 249 101 149 75 86 110 116 154 225 131 137 242 20 32 82 22 100 231 70 92 60 249 78 68 88 198 211 170 134 222 207 21 129 128 31 249 84 66 99 97 89 85 81 94 73 165 205 28 59 0 206 6 18 224 189 80 50 182 160 231 66 32 22 29 235 127 111 253 118 188 148 182 246 250 172 75 23 9 251 23 47 95 183 36 167 104 137 108 140 254 249 90 117 71 229 124 200 174 77 232 110 57 30 29 233 107 63 195 45 212 164 165 59 61 199 213 127 192 233 41 90 147 93 84 34 27 225 191 37 20 40 133 228 168 4 145 203 241 201 127 62 24 15 7 123 143 112 11 143 168 26 140 207 29 1 0 65 42 201 22 248 24 8 117 131 72 197 144 236 43 17 30 30 18 221 205 7 99 156 233 47 113 142 159 168 26 102 253 207 78 186 9 52 71 66 151 73 112 96 49 207 240 170 52 26 30 196 112 235 71 227 225 225 222 62 110 225 81 85 195 201 148 55 119 147 224 70 37 84 66 29 128 66 66 208 96 49 188 11 96 159 170 77 159 47 22 108 193 22 108 122 251 47 1 167 63 47 137 31 193 98 0 0 0 0 73 69 78 68 174 66 96 130)! !

!DrGIcons class methodsFor: 'accessing' stamp: 'HilaireFernandes 11/26/2005 19:08'!
vector
	^self formFromGraphic: #(137 80 78 71 13 10 26 10 0 0 0 13 73 72 68 82 0 0 0 48 0 0 0 48 16 6 0 0 0 7 146 37 196 0 0 0 6 98 75 71 68 255 255 255 255 255 255 9 88 247 220 0 0 0 9 112 72 89 115 0 0 0 72 0 0 0 72 0 70 201 107 62 0 0 1 73 73 68 65 84 120 218 237 213 49 142 130 64 20 128 225 113 59 11 14 65 193 77 188 0 23 152 120 3 206 194 13 204 196 130 154 130 202 214 83 88 16 19 75 58 10 27 34 86 47 36 99 8 236 46 227 51 248 127 141 25 108 204 255 230 161 49 0 0 0 0 0 0 111 179 209 254 1 159 226 126 207 178 44 235 123 255 249 249 92 150 101 105 204 110 87 215 117 189 89 188 215 215 13 96 44 244 237 86 85 85 53 156 175 215 174 235 58 99 142 199 166 105 26 99 30 143 52 77 83 99 156 115 206 185 229 6 177 250 1 204 13 46 36 188 144 1 28 14 109 219 182 203 111 192 143 118 160 208 182 219 60 207 243 33 220 220 240 62 107 173 181 246 117 144 255 181 250 13 240 93 46 73 146 36 67 200 177 240 161 95 61 98 245 27 48 102 234 198 11 9 31 106 3 190 102 0 99 255 5 114 211 199 206 18 62 212 6 172 158 132 151 87 207 233 20 199 113 220 247 251 125 20 69 209 48 16 57 203 167 127 227 67 109 192 106 205 13 239 135 157 250 30 19 254 26 254 183 103 120 8 175 100 169 240 62 6 49 33 84 120 31 225 61 239 10 15 15 225 149 16 94 9 225 149 16 94 9 225 149 16 94 9 225 149 16 94 9 225 149 16 94 9 225 149 16 94 9 225 149 244 125 81 20 5 225 213 17 254 195 16 254 67 16 30 0 0 0 175 158 187 58 229 157 66 156 78 5 0 0 0 0 73 69 78 68 174 66 96 130)! !

!DrGIcons class methodsFor: 'accessing' stamp: 'HilaireFernandes 4/2/2006 10:53'!
viewer
^self formFromGraphic: #(137 80 78 71 13 10 26 10 0 0 0 13 73 72 68 82 0 0 0 24 0 0 0 24 8 2 0 0 0 111 21 170 175 0 0 0 6 98 75 71 68 0 255 0 255 0 255 160 189 167 147 0 0 0 9 112 72 89 115 0 0 14 196 0 0 14 196 1 149 43 14 27 0 0 0 7 116 73 77 69 7 214 4 2 8 52 1 149 4 169 0 0 0 0 180 73 68 65 84 56 203 197 85 203 14 195 48 8 35 86 255 187 233 143 199 59 108 107 9 143 164 149 50 205 226 20 192 134 146 208 66 82 86 0 178 8 91 230 40 199 17 158 115 223 227 120 223 218 69 225 115 190 46 79 103 137 62 44 137 172 166 179 92 84 144 90 133 188 107 181 234 92 76 63 202 0 58 165 159 218 184 35 131 62 24 163 114 10 172 229 69 33 86 56 211 216 132 45 56 119 41 136 11 49 201 221 116 90 88 26 158 177 228 92 72 131 52 157 246 122 177 244 137 104 46 173 60 212 72 222 218 169 60 232 58 109 205 223 128 247 200 180 37 79 239 34 226 163 171 24 45 3 100 10 115 244 193 152 46 154 59 229 172 92 35 63 91 108 43 87 237 159 255 34 47 138 168 164 232 254 82 152 242 0 0 0 0 73 69 78 68 174 66 96 130)! !

Object subclass: #DrGLoader
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'DrGeoII-Core-UI'!

DrGLoader class
	instanceVariableNames: 'loader isLoaderRunning process'!

!DrGLoader commentStamp: 'HilaireFernandes 1/6/2012 18:19' prior: 0!
I animate a loader. !

DrGLoader class
	instanceVariableNames: 'loader isLoaderRunning process'!

!DrGLoader class methodsFor: 'loader' stamp: 'HilaireFernandes 1/6/2012 19:00'!
at: integer
	^ loader at: integer! !

!DrGLoader class methodsFor: 'loader' stamp: 'HilaireFernandes 1/6/2012 19:27'!
decorate: form
	|newForm|
	newForm := Form extent: form extent + 10 depth: 32.
	newForm fillWhite.
	form displayOn: newForm at: newForm center - (form extent / 2).
	newForm  borderWidth: 2 fillColor: Color veryLightGray.
	^ newForm! !

!DrGLoader class methodsFor: 'class initialization' stamp: 'HilaireFernandes 1/6/2012 22:44'!
initialize
"self initialize"
	loader := Array new: self loaderSize.
	1 to: self loaderSize do: [:i | |form|
		form := DrGIcons formFromPNG: (self perform: ('loader', i asString, 'Contents') asSymbol).
		loader at: i put: (self decorate: form)]! !

!DrGLoader class methodsFor: 'loader' stamp: 'HilaireFernandes 1/6/2012 23:00'!
loader10Contents
^ #[137 80 78 71 13 10 26 10 0 0 0 13 73 72 68 82 0 0 0 32 0 0 0 32 8 3 0 0 0 68 164 138 198 0 0 3 0 80 76 84 69 255 255 255 58 60 139 249 249 251 208 208 226 236 236 243 242 242 246 217 218 232 154 155 195 176 177 208 246 246 249 234 234 242 251 251 252 170 170 204 160 161 199 231 231 240 199 199 221 180 181 210 239 239 244 191 192 217 226 227 237 105 107 166 124 125 177 131 132 182 153 154 194 186 187 214 240 240 245 114 116 172 140 142 187 67 69 144 58 60 139 216 216 231 211 211 228 223 224 235 91 93 158 130 131 181 81 82 152 116 117 173 194 194 217 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 42 88 37 121 0 0 1 0 116 82 78 83 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 255 1 110 23 0 0 1 31 73 68 65 84 120 94 133 83 203 114 195 32 12 212 64 20 5 225 16 147 224 131 147 118 166 255 255 149 69 22 174 49 56 233 94 44 208 234 181 194 208 128 217 195 49 244 222 48 27 53 90 191 227 185 38 32 187 134 80 60 219 7 219 20 145 121 35 204 203 161 75 225 86 130 218 45 80 6 80 2 115 220 197 106 199 94 234 46 4 183 142 226 92 113 68 163 131 72 10 163 76 9 140 69 148 220 17 163 88 88 134 247 115 62 121 228 191 81 114 222 174 43 113 71 83 201 212 12 142 109 72 78 184 19 215 107 209 126 23 7 199 96 21 161 21 78 1 240 245 173 24 59 97 21 255 19 222 148 240 165 196 214 203 233 99 207 224 198 231 121 95 97 47 220 109 74 105 170 47 68 254 77 152 199 43 165 244 122 52 194 49 207 133 50 100 247 100 115 218 75 105 212 200 178 100 151 165 78 74 207 81 172 27 209 201 132 96 214 117 203 130 22 194 125 88 38 64 162 17 208 90 148 77 213 15 102 197 72 132 74 200 79 110 238 159 100 32 202 125 40 193 49 119 63 14 12 116 53 43 161 252 3 59 156 137 68 169 66 56 120 247 63 116 129 141 0 120 80 3 106 194 91 120 107 155 208 95 137 155 13 250 175 184 235 210 0 0 0 0 73 69 78 68 174 66 96 130]! !

!DrGLoader class methodsFor: 'loader' stamp: 'HilaireFernandes 1/6/2012 23:01'!
loader11Contents
^ #[137 80 78 71 13 10 26 10 0 0 0 13 73 72 68 82 0 0 0 32 0 0 0 32 8 3 0 0 0 68 164 138 198 0 0 3 0 80 76 84 69 255 255 255 58 60 139 249 249 251 208 208 226 236 236 243 242 242 246 217 218 232 154 155 195 176 177 208 246 246 249 234 234 242 251 251 252 170 170 204 160 161 199 231 231 240 199 199 221 180 181 210 239 239 244 191 192 217 226 227 237 105 107 166 124 125 177 131 132 182 153 154 194 186 187 214 240 240 245 114 116 172 140 142 187 67 69 144 58 60 139 216 216 231 211 211 228 223 224 235 91 93 158 130 131 181 81 82 152 116 117 173 194 194 217 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 42 88 37 121 0 0 1 0 116 82 78 83 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 255 1 110 23 0 0 1 33 73 68 65 84 120 94 133 83 219 114 132 32 12 117 8 151 33 184 10 211 125 161 91 118 219 105 255 255 27 23 12 168 68 157 158 23 49 57 147 235 201 192 128 232 134 115 144 93 32 10 122 112 191 197 184 39 40 180 140 80 61 219 71 241 16 1 113 35 196 229 231 16 194 54 2 189 57 84 105 128 8 136 97 239 241 158 218 40 121 23 130 109 173 216 37 142 78 233 23 168 145 18 66 16 179 36 13 52 20 245 147 82 250 155 202 171 54 239 98 182 187 156 178 181 226 191 50 197 40 94 17 134 117 90 234 81 130 112 127 215 200 116 79 208 13 30 21 223 137 62 91 77 129 212 4 201 7 71 24 134 207 59 97 62 12 150 240 63 225 34 133 171 41 182 90 212 85 145 53 34 220 166 62 67 191 78 61 26 51 238 13 89 16 184 169 78 122 99 140 239 138 88 54 17 43 101 206 238 241 59 27 159 181 14 81 150 85 118 89 243 24 115 131 194 253 0 176 206 90 215 214 109 177 10 239 53 211 250 1 52 9 70 245 130 89 11 133 28 167 73 78 157 72 18 96 106 162 181 251 22 26 94 0 110 149 125 56 202 94 2 200 238 112 184 238 35 60 251 211 19 231 39 44 240 202 115 121 254 111 248 165 14 102 123 116 206 202 0 0 0 0 73 69 78 68 174 66 96 130]! !

!DrGLoader class methodsFor: 'loader' stamp: 'HilaireFernandes 1/6/2012 23:01'!
loader12Contents
^ #[137 80 78 71 13 10 26 10 0 0 0 13 73 72 68 82 0 0 0 32 0 0 0 32 8 3 0 0 0 68 164 138 198 0 0 3 0 80 76 84 69 255 255 255 58 60 139 249 249 251 208 208 226 236 236 243 242 242 246 217 218 232 154 155 195 176 177 208 246 246 249 234 234 242 251 251 252 170 170 204 160 161 199 231 231 240 199 199 221 180 181 210 239 239 244 191 192 217 226 227 237 105 107 166 124 125 177 131 132 182 153 154 194 186 187 214 240 240 245 114 116 172 140 142 187 67 69 144 58 60 139 216 216 231 211 211 228 223 224 235 91 93 158 130 131 181 81 82 152 116 117 173 194 194 217 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 42 88 37 121 0 0 1 0 116 82 78 83 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 255 1 110 23 0 0 1 43 73 68 65 84 120 94 133 83 127 111 132 32 12 37 69 184 180 220 169 68 182 139 187 53 102 217 190 255 103 60 126 169 128 154 245 15 17 222 163 45 237 171 104 140 200 136 115 211 42 124 129 8 226 22 90 252 206 63 37 65 17 54 132 145 121 220 9 126 81 173 139 95 254 86 27 97 38 58 228 32 153 95 43 193 127 241 152 165 99 238 51 129 200 150 200 48 196 165 103 118 137 128 235 83 48 250 153 156 251 208 225 231 198 172 5 128 48 57 67 176 169 40 248 233 156 123 118 254 105 127 50 185 52 179 63 55 138 182 167 12 95 158 50 214 105 5 216 110 213 194 123 112 210 226 213 141 238 229 116 185 55 164 218 158 76 245 118 135 85 159 172 45 45 36 19 226 113 75 166 27 156 178 253 79 184 8 97 114 136 61 23 184 74 50 239 123 169 234 8 117 221 186 69 202 165 60 240 130 160 221 39 106 41 165 174 174 196 78 204 153 50 121 120 153 195 181 124 0 161 89 161 151 57 142 199 251 216 86 223 93 131 104 214 118 35 101 225 209 4 169 252 254 60 10 70 213 130 41 26 104 54 201 169 163 36 147 80 19 1 203 39 172 102 99 192 44 123 123 148 253 62 49 32 78 117 15 52 215 163 7 231 35 12 116 133 92 142 255 27 198 51 14 251 200 3 212 240 0 0 0 0 73 69 78 68 174 66 96 130]! !

!DrGLoader class methodsFor: 'loader' stamp: 'HilaireFernandes 1/6/2012 22:54'!
loader1Contents
^  #[137 80 78 71 13 10 26 10 0 0 0 13 73 72 68 82 0 0 0 32 0 0 0 32 8 3 0 0 0 68 164 138 198 0 0 3 0 80 76 84 69 255 255 255 58 60 139 249 249 251 208 208 226 236 236 243 242 242 246 217 218 232 154 155 195 176 177 208 246 246 249 234 234 242 251 251 252 170 170 204 160 161 199 231 231 240 199 199 221 180 181 210 239 239 244 191 192 217 226 227 237 105 107 166 124 125 177 131 132 182 153 154 194 186 187 214 240 240 245 114 116 172 140 142 187 67 69 144 58 60 139 216 216 231 211 211 228 223 224 235 91 93 158 130 131 181 81 82 152 116 117 173 194 194 217 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 42 88 37 121 0 0 1 0 116 82 78 83 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 255 1 110 23 0 0 1 33 73 68 65 84 120 94 117 83 237 118 131 32 12 245 4 41 35 233 166 174 217 58 183 246 253 95 115 129 128 53 160 249 33 38 185 220 124 50 88 241 143 199 199 112 44 183 152 190 238 249 116 89 133 214 255 201 223 123 128 199 216 0 38 230 233 5 0 68 223 82 124 241 111 220 0 30 177 203 225 194 124 173 0 33 136 125 150 119 230 177 0 16 23 115 247 146 143 145 249 79 1 17 81 139 136 153 103 12 225 154 107 159 153 111 222 57 79 37 67 88 16 41 157 111 33 132 89 76 241 174 84 3 173 98 39 73 180 150 114 121 23 136 179 173 73 238 101 51 193 36 136 159 214 111 10 241 115 48 51 32 244 212 84 57 90 245 229 6 175 210 78 7 84 104 112 69 26 6 105 166 202 6 192 51 192 73 8 218 66 244 89 29 169 210 70 4 27 193 142 51 181 221 12 112 69 220 221 208 182 27 210 108 90 161 178 151 182 87 3 164 97 37 210 18 167 254 164 53 160 24 169 142 59 93 44 107 65 218 126 177 67 10 237 237 194 236 6 72 10 16 206 181 95 73 93 84 5 68 196 238 225 12 75 14 168 128 162 180 4 96 142 216 1 214 29 78 18 130 227 39 12 120 230 169 162 155 190 147 127 215 208 13 219 203 127 1 212 0 0 0 0 73 69 78 68 174 66 96 130]! !

!DrGLoader class methodsFor: 'loader' stamp: 'HilaireFernandes 1/6/2012 22:58'!
loader2Contents 
^ #[137 80 78 71 13 10 26 10 0 0 0 13 73 72 68 82 0 0 0 32 0 0 0 32 8 3 0 0 0 68 164 138 198 0 0 3 0 80 76 84 69 255 255 255 58 60 139 249 249 251 208 208 226 236 236 243 242 242 246 217 218 232 154 155 195 176 177 208 246 246 249 234 234 242 251 251 252 170 170 204 160 161 199 231 231 240 199 199 221 180 181 210 239 239 244 191 192 217 226 227 237 105 107 166 124 125 177 131 132 182 153 154 194 186 187 214 240 240 245 114 116 172 140 142 187 67 69 144 58 60 139 216 216 231 211 211 228 223 224 235 91 93 158 130 131 181 81 82 152 116 117 173 194 194 217 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 42 88 37 121 0 0 1 0 116 82 78 83 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 255 1 110 23 0 0 1 35 73 68 65 84 120 94 117 147 209 118 131 48 8 134 123 136 9 35 108 174 158 121 81 187 178 174 221 251 191 227 146 144 180 6 43 23 42 240 9 63 24 15 189 209 249 60 28 94 219 0 249 26 230 57 20 215 217 252 21 223 214 192 93 190 13 16 16 195 19 112 34 119 91 226 29 63 224 1 220 228 226 45 48 34 126 54 224 40 130 91 149 39 68 175 128 191 200 95 167 95 7 243 136 71 5 80 68 135 32 42 87 231 174 165 161 67 28 40 4 26 69 230 236 195 20 35 167 59 255 56 231 66 18 8 167 177 78 116 75 15 236 99 140 85 233 144 8 55 114 167 40 167 39 104 30 127 37 226 215 230 105 29 128 224 186 193 57 122 54 83 82 239 62 211 12 106 246 133 71 56 86 3 147 111 241 7 64 123 192 78 139 109 152 247 68 182 33 122 17 96 90 230 181 79 235 192 210 201 214 181 119 69 75 104 129 86 189 174 189 5 96 97 45 90 251 180 135 4 50 19 49 215 175 152 95 172 199 130 117 253 41 14 185 181 111 253 137 204 7 100 5 82 205 101 123 36 161 52 82 128 54 155 79 54 149 134 10 84 199 22 128 238 70 27 96 89 113 73 16 188 254 133 33 238 101 154 233 73 95 217 63 178 66 14 153 183 139 109 50 0 0 0 0 73 69 78 68 174 66 96 130]! !

!DrGLoader class methodsFor: 'loader' stamp: 'HilaireFernandes 1/6/2012 22:58'!
loader3Contents
^  #[137 80 78 71 13 10 26 10 0 0 0 13 73 72 68 82 0 0 0 32 0 0 0 32 8 3 0 0 0 68 164 138 198 0 0 3 0 80 76 84 69 255 255 255 58 60 139 249 249 251 208 208 226 236 236 243 242 242 246 217 218 232 154 155 195 176 177 208 246 246 249 234 234 242 251 251 252 170 170 204 160 161 199 231 231 240 199 199 221 180 181 210 239 239 244 191 192 217 226 227 237 105 107 166 124 125 177 131 132 182 153 154 194 186 187 214 240 240 245 114 116 172 140 142 187 67 69 144 58 60 139 216 216 231 211 211 228 223 224 235 91 93 158 130 131 181 81 82 152 116 117 173 194 194 217 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 42 88 37 121 0 0 1 0 116 82 78 83 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 255 1 110 23 0 0 1 39 73 68 65 84 120 94 125 146 127 115 194 32 12 134 123 73 105 15 98 153 61 163 119 184 177 121 186 125 255 175 56 32 180 5 170 230 143 86 94 30 243 230 71 187 58 96 154 84 247 60 52 197 167 29 71 155 142 67 123 111 240 187 4 38 62 52 64 143 216 111 192 192 60 181 41 126 16 105 5 46 124 213 45 160 16 79 11 240 193 124 220 87 121 67 4 1 244 149 207 85 107 32 47 196 65 128 35 179 52 113 72 149 146 49 115 66 44 162 6 107 161 103 254 140 231 191 135 247 9 116 198 24 21 10 164 91 158 209 233 210 199 70 188 247 156 61 2 97 234 170 213 61 92 63 182 74 117 76 82 2 191 222 127 141 165 66 202 64 9 12 254 110 155 46 169 62 110 187 32 144 104 128 110 149 77 14 104 238 23 125 5 244 43 224 133 197 94 166 183 53 199 73 64 237 80 91 194 28 22 82 10 174 42 59 76 73 182 209 213 146 131 109 206 105 159 139 0 142 36 105 246 89 126 4 144 72 107 162 188 151 248 71 201 32 95 124 210 33 90 171 197 95 87 149 6 153 4 8 57 221 254 147 132 100 36 128 222 77 62 196 156 12 5 200 135 54 1 84 47 189 3 92 193 117 234 137 71 153 232 77 196 86 170 248 7 162 137 15 60 93 182 179 128 0 0 0 0 73 69 78 68 174 66 96 130]! !

!DrGLoader class methodsFor: 'loader' stamp: 'HilaireFernandes 1/6/2012 22:58'!
loader4Contents
^ #[137 80 78 71 13 10 26 10 0 0 0 13 73 72 68 82 0 0 0 32 0 0 0 32 8 3 0 0 0 68 164 138 198 0 0 3 0 80 76 84 69 255 255 255 58 60 139 249 249 251 208 208 226 236 236 243 242 242 246 217 218 232 154 155 195 176 177 208 246 246 249 234 234 242 251 251 252 170 170 204 160 161 199 231 231 240 199 199 221 180 181 210 239 239 244 191 192 217 226 227 237 105 107 166 124 125 177 131 132 182 153 154 194 186 187 214 240 240 245 114 116 172 140 142 187 67 69 144 58 60 139 216 216 231 211 211 228 223 224 235 91 93 158 130 131 181 81 82 152 116 117 173 194 194 217 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 42 88 37 121 0 0 1 0 116 82 78 83 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 255 1 110 23 0 0 1 32 73 68 65 84 120 94 117 83 203 114 195 32 12 244 136 168 10 82 236 196 49 153 14 77 123 204 255 255 98 1 97 3 198 222 67 76 208 34 173 94 67 11 49 6 134 99 72 250 69 99 48 29 166 189 221 178 175 9 31 186 239 8 192 12 133 48 17 125 246 46 102 230 66 184 210 216 73 9 46 236 74 248 34 50 189 74 100 22 37 192 72 183 230 173 122 19 102 84 130 33 210 36 158 207 108 152 65 19 97 17 68 65 162 71 252 111 94 206 45 241 224 57 188 141 101 192 44 236 114 13 58 150 183 115 238 189 233 75 10 235 178 140 193 252 186 87 101 138 2 42 124 59 247 243 168 223 8 114 147 248 226 198 203 97 47 10 163 24 64 177 35 132 124 19 166 129 51 118 165 53 127 138 223 141 96 207 8 39 33 166 45 196 137 72 233 7 6 218 222 182 33 33 76 195 92 95 248 70 118 168 82 238 70 123 229 161 212 57 245 115 189 0 47 234 52 199 89 15 169 221 214 138 228 190 196 135 234 193 202 54 48 105 118 113 141 111 109 55 114 58 220 172 59 112 52 180 74 176 93 229 183 177 207 235 49 231 232 253 226 148 143 237 8 190 226 13 200 39 59 12 124 102 89 17 83 105 240 15 76 208 14 94 194 124 25 106 0 0 0 0 73 69 78 68 174 66 96 130] ! !

!DrGLoader class methodsFor: 'loader' stamp: 'HilaireFernandes 1/6/2012 22:59'!
loader5Contents
^  #[137 80 78 71 13 10 26 10 0 0 0 13 73 72 68 82 0 0 0 32 0 0 0 32 8 3 0 0 0 68 164 138 198 0 0 3 0 80 76 84 69 255 255 255 58 60 139 249 249 251 208 208 226 236 236 243 242 242 246 217 218 232 154 155 195 176 177 208 246 246 249 234 234 242 251 251 252 170 170 204 160 161 199 231 231 240 199 199 221 180 181 210 239 239 244 191 192 217 226 227 237 105 107 166 124 125 177 131 132 182 153 154 194 186 187 214 240 240 245 114 116 172 140 142 187 67 69 144 58 60 139 216 216 231 211 211 228 223 224 235 91 93 158 130 131 181 81 82 152 116 117 173 194 194 217 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 42 88 37 121 0 0 1 0 116 82 78 83 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 255 1 110 23 0 0 1 32 73 68 65 84 120 94 125 83 219 146 131 48 8 205 128 145 9 177 142 206 246 161 238 54 181 157 157 254 255 47 54 145 172 185 233 242 162 9 39 112 128 131 170 140 217 170 99 147 123 96 134 237 71 215 126 195 75 14 120 226 87 5 136 158 248 209 136 207 58 196 204 156 0 15 196 134 138 119 153 63 64 135 56 182 44 117 40 96 3 88 196 181 120 43 188 45 179 22 192 136 40 69 172 107 116 204 32 133 132 16 160 0 177 223 42 25 136 186 240 179 176 127 27 56 105 136 37 63 2 143 137 136 166 157 223 198 48 79 251 246 238 161 207 218 20 8 100 118 35 186 32 228 141 214 156 31 85 71 111 125 56 139 132 72 14 16 171 0 166 23 235 20 71 131 18 208 95 197 190 119 128 57 3 156 164 232 246 20 39 36 199 86 48 5 9 116 84 148 9 94 13 115 126 241 114 238 62 229 93 138 211 72 25 174 206 185 23 166 62 111 243 92 98 22 88 60 24 127 61 228 39 42 93 106 12 227 182 198 88 27 231 66 119 231 36 130 177 165 96 116 108 155 166 225 88 114 62 230 210 74 50 23 173 105 58 95 203 94 14 255 44 14 52 179 241 87 197 234 233 131 28 229 242 158 90 179 254 31 178 93 14 174 214 254 40 176 0 0 0 0 73 69 78 68 174 66 96 130]! !

!DrGLoader class methodsFor: 'loader' stamp: 'HilaireFernandes 1/6/2012 22:59'!
loader6Contents
^ #[137 80 78 71 13 10 26 10 0 0 0 13 73 72 68 82 0 0 0 32 0 0 0 32 8 3 0 0 0 68 164 138 198 0 0 3 0 80 76 84 69 255 255 255 58 60 139 249 249 251 208 208 226 236 236 243 242 242 246 217 218 232 154 155 195 176 177 208 246 246 249 234 234 242 251 251 252 170 170 204 160 161 199 231 231 240 199 199 221 180 181 210 239 239 244 191 192 217 226 227 237 105 107 166 124 125 177 131 132 182 153 154 194 186 187 214 240 240 245 114 116 172 140 142 187 67 69 144 58 60 139 216 216 231 211 211 228 223 224 235 91 93 158 130 131 181 81 82 152 116 117 173 194 194 217 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 42 88 37 121 0 0 1 0 116 82 78 83 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 255 1 110 23 0 0 1 24 73 68 65 84 120 94 133 83 219 110 195 32 12 69 80 82 217 116 52 153 88 165 116 181 122 153 246 255 191 56 192 52 4 231 178 243 2 196 39 62 190 42 1 68 167 214 193 223 53 162 230 139 180 3 142 115 130 69 16 132 98 169 135 149 46 6 196 74 24 243 99 225 2 222 4 190 75 216 148 0 19 16 135 230 95 142 216 37 221 76 128 119 42 44 20 13 131 230 68 146 11 205 204 20 200 221 152 172 20 35 66 155 202 96 75 242 110 140 47 232 140 49 221 20 159 140 202 61 162 249 126 152 149 73 36 158 236 126 94 116 103 177 41 46 152 135 94 237 69 101 84 131 102 8 130 246 12 171 176 64 120 244 71 198 199 68 128 45 194 134 132 157 36 54 130 60 44 7 166 9 162 11 125 163 169 7 209 192 75 8 215 243 188 74 165 27 85 225 59 132 112 233 106 157 115 63 199 162 98 158 62 138 124 69 202 169 116 157 115 76 237 118 0 206 19 189 210 187 191 134 192 30 192 181 3 243 34 50 217 208 127 174 143 156 189 209 83 237 14 237 145 168 87 187 99 255 67 55 187 187 56 134 232 180 32 52 171 23 232 119 125 133 245 162 239 234 191 245 255 3 169 222 15 73 159 10 166 100 0 0 0 0 73 69 78 68 174 66 96 130]! !

!DrGLoader class methodsFor: 'loader' stamp: 'HilaireFernandes 1/6/2012 22:59'!
loader7Contents
^  #[137 80 78 71 13 10 26 10 0 0 0 13 73 72 68 82 0 0 0 32 0 0 0 32 8 3 0 0 0 68 164 138 198 0 0 3 0 80 76 84 69 255 255 255 58 60 139 249 249 251 208 208 226 236 236 243 242 242 246 217 218 232 154 155 195 176 177 208 246 246 249 234 234 242 251 251 252 170 170 204 160 161 199 231 231 240 199 199 221 180 181 210 239 239 244 191 192 217 226 227 237 105 107 166 124 125 177 131 132 182 153 154 194 186 187 214 240 240 245 114 116 172 140 142 187 67 69 144 58 60 139 216 216 231 211 211 228 223 224 235 91 93 158 130 131 181 81 82 152 116 117 173 194 194 217 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 42 88 37 121 0 0 1 0 116 82 78 83 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 255 1 110 23 0 0 1 22 73 68 65 84 120 94 133 147 235 110 195 32 12 133 17 212 161 152 134 150 201 171 148 174 213 222 255 45 7 216 185 0 65 243 159 224 240 225 203 193 168 198 16 189 58 55 254 175 17 53 47 218 125 135 203 17 0 116 13 32 59 251 7 218 16 17 113 7 150 226 116 33 220 10 240 186 53 200 13 48 128 24 171 179 92 177 207 121 11 224 214 86 156 147 141 168 185 145 28 66 51 153 15 70 17 37 85 132 144 87 32 205 251 37 121 30 112 107 37 197 237 170 202 219 81 31 100 106 26 135 246 72 10 88 137 235 57 105 127 23 39 174 215 108 45 0 108 90 161 88 115 125 96 196 254 7 6 41 244 150 98 80 36 244 3 83 229 8 214 84 126 150 189 186 192 171 181 243 116 84 9 27 97 224 97 173 189 134 93 231 34 251 34 81 167 231 37 37 185 37 228 46 147 206 178 231 235 14 214 134 11 209 156 125 51 91 43 99 225 183 129 49 159 143 81 51 81 201 175 205 111 55 114 5 112 47 122 170 193 208 22 64 61 136 238 234 124 236 25 80 223 244 26 60 28 1 38 162 175 14 40 79 79 0 245 67 230 252 9 175 192 208 224 253 14 245 159 63 14 38 13 186 43 109 220 91 0 0 0 0 73 69 78 68 174 66 96 130]! !

!DrGLoader class methodsFor: 'loader' stamp: 'HilaireFernandes 1/6/2012 22:59'!
loader8Contents
^ #[137 80 78 71 13 10 26 10 0 0 0 13 73 72 68 82 0 0 0 32 0 0 0 32 8 3 0 0 0 68 164 138 198 0 0 3 0 80 76 84 69 255 255 255 58 60 139 249 249 251 208 208 226 236 236 243 242 242 246 217 218 232 154 155 195 176 177 208 246 246 249 234 234 242 251 251 252 170 170 204 160 161 199 231 231 240 199 199 221 180 181 210 239 239 244 191 192 217 226 227 237 105 107 166 124 125 177 131 132 182 153 154 194 186 187 214 240 240 245 114 116 172 140 142 187 67 69 144 58 60 139 216 216 231 211 211 228 223 224 235 91 93 158 130 131 181 81 82 152 116 117 173 194 194 217 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 42 88 37 121 0 0 1 0 116 82 78 83 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 255 1 110 23 0 0 1 32 73 68 65 84 120 94 133 147 221 110 195 32 12 133 17 196 113 49 43 74 180 220 164 155 219 85 83 223 255 21 7 49 249 193 164 154 111 2 246 103 131 15 142 81 70 20 204 185 137 223 18 89 89 232 184 167 249 8 0 121 5 148 200 254 1 93 98 36 218 129 121 217 52 37 252 10 200 90 27 228 6 4 32 26 171 92 185 113 200 231 46 128 95 91 241 190 4 70 43 141 228 18 86 200 156 56 22 81 210 141 8 242 10 74 243 97 78 187 0 180 181 146 234 54 183 202 225 209 30 100 82 141 131 78 73 5 43 113 131 28 218 190 197 201 54 88 49 157 176 185 169 152 109 132 21 251 31 120 115 68 235 86 192 201 192 84 46 112 177 202 200 178 87 15 120 119 206 117 71 149 72 9 99 251 68 220 97 215 57 203 14 79 39 142 120 73 17 120 36 228 179 76 186 200 254 197 220 119 195 208 1 226 107 225 18 81 198 98 169 30 153 39 211 79 83 111 94 136 113 57 58 86 147 57 49 71 1 236 21 47 237 72 58 102 52 2 24 135 216 55 192 47 127 195 10 152 15 188 106 165 6 230 193 108 64 68 124 40 224 197 79 179 3 102 192 159 243 95 120 5 222 154 191 221 186 218 243 7 48 94 14 167 128 25 78 150 0 0 0 0 73 69 78 68 174 66 96 130]! !

!DrGLoader class methodsFor: 'loader' stamp: 'HilaireFernandes 1/6/2012 23:00'!
loader9Contents
^  #[137 80 78 71 13 10 26 10 0 0 0 13 73 72 68 82 0 0 0 32 0 0 0 32 8 3 0 0 0 68 164 138 198 0 0 3 0 80 76 84 69 255 255 255 58 60 139 249 249 251 208 208 226 236 236 243 242 242 246 217 218 232 154 155 195 176 177 208 246 246 249 234 234 242 251 251 252 170 170 204 160 161 199 231 231 240 199 199 221 180 181 210 239 239 244 191 192 217 226 227 237 105 107 166 124 125 177 131 132 182 153 154 194 186 187 214 240 240 245 114 116 172 140 142 187 67 69 144 58 60 139 216 216 231 211 211 228 223 224 235 91 93 158 130 131 181 81 82 152 116 117 173 194 194 217 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 42 88 37 121 0 0 1 0 116 82 78 83 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 255 1 110 23 0 0 1 33 73 68 65 84 120 94 133 147 219 110 195 32 12 64 17 14 137 48 109 218 104 94 37 150 210 85 138 246 255 191 88 136 33 23 147 118 126 9 216 199 247 160 132 32 58 117 44 172 215 136 154 15 210 110 209 111 1 131 86 0 217 178 126 140 12 49 32 174 128 159 47 85 8 91 0 62 75 49 169 1 6 16 135 157 47 87 236 82 222 25 176 165 21 107 179 97 208 220 72 10 161 153 76 142 67 30 74 172 8 77 58 153 220 188 243 241 230 12 46 173 196 184 85 85 201 60 232 205 152 68 227 70 186 196 128 187 225 58 78 90 239 226 224 234 52 139 116 88 212 152 69 87 131 101 249 31 120 147 162 86 247 31 107 86 166 187 195 62 195 126 112 215 41 132 105 171 240 187 170 90 10 33 80 43 6 135 232 51 50 70 243 244 23 231 125 203 140 121 104 222 101 206 19 194 189 75 139 56 17 125 25 0 163 1 158 101 65 51 112 26 231 14 26 162 81 245 93 215 171 103 164 54 63 76 145 145 168 97 192 1 60 234 95 178 37 186 42 6 84 15 208 84 192 55 253 216 2 168 95 0 57 169 11 209 69 45 64 172 85 62 12 160 155 90 1 213 130 63 126 194 5 120 43 250 124 22 143 242 5 159 241 14 245 151 37 96 90 0 0 0 0 73 69 78 68 174 66 96 130]! !

!DrGLoader class methodsFor: 'loader' stamp: 'HilaireFernandes 1/6/2012 18:20'!
loaderSize
	^ 12! !

!DrGLoader class methodsFor: 'accessing' stamp: 'HilaireFernandes 1/8/2012 14:19'!
start
	isLoaderRunning ifTrue: [self stop].
	isLoaderRunning := true.
	process :=[[1 to: self loaderSize do: [:i| | form|
		form := self at: i.
		form displayAt: (World center - form extent).
		(Delay forMilliseconds: 100) wait]] repeat] fork.! !

!DrGLoader class methodsFor: 'accessing' stamp: 'HilaireFernandes 1/8/2012 14:19'!
stop
	isLoaderRunning ifTrue:[
		process terminate.
		isLoaderRunning := false.
		World restoreDisplay]! !

Object subclass: #DrGLocusSample
	instanceVariableNames: 'abscissa pointScreen pointWorld offScreen'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'DrGeoII-Core-Item-View'!

!DrGLocusSample commentStamp: 'HilaireFernandes 4/1/2011 17:34' prior: 0!
I am sample of a locus with: 
- the curvilinear abscissa of the free point
- my screen coordinates
- my world coordinates!

!DrGLocusSample methodsFor: 'accessing' stamp: 'HilaireFernandes 4/2/2011 10:22'!
offScreen 
	^ offScreen ! !

!DrGLocusSample methodsFor: 'accessing' stamp: 'HilaireFernandes 4/2/2011 10:22'!
offScreen: aBoolean
	offScreen  := aBoolean ! !

!DrGLocusSample methodsFor: 'accessing' stamp: 'HilaireFernandes 4/1/2011 17:34'!
pointScreen

	^ pointScreen! !

!DrGLocusSample methodsFor: 'accessing' stamp: 'HilaireFernandes 4/1/2011 17:34'!
pointScreen: anObject

	pointScreen := anObject! !

!DrGLocusSample methodsFor: 'accessing' stamp: 'HilaireFernandes 4/1/2011 17:34'!
pointWorld

	^ pointWorld! !

!DrGLocusSample methodsFor: 'accessing' stamp: 'HilaireFernandes 4/1/2011 17:34'!
pointWorld: anObject

	pointWorld := anObject! !

!DrGLocusSample methodsFor: 'accessing' stamp: 'HilaireFernandes 4/1/2011 17:39'!
x

	^ abscissa! !

!DrGLocusSample methodsFor: 'accessing' stamp: 'HilaireFernandes 4/1/2011 17:38'!
x: anObject

	abscissa := anObject! !

Object subclass: #DrGMacro
	instanceVariableNames: 'nodes title description'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'DrGeoII-Core-Macro'!

!DrGMacro commentStamp: 'HilaireFernandes 7/18/2008 10:52' prior: 0!
A DrGMacro is a macro-construction definition

Instance Variables
	nodes:		<Object>

nodes
	- xxxxx
!

!DrGMacro class methodsFor: 'testing' stamp: 'HilaireFernandes 8/25/2008 10:50'!
isValidWith: inputs and: outputs 
	"check the validity of a macro defined with these input and output items"
	| parents |
	parents := Set new.
	outputs do: [ :item | 
		(inputs includes: item) 
			ifTrue: 
				["item is an input, we don't go to parents, we keep it"
				parents add: item]
			ifFalse: 
				["item isn't an input, we go to its parents"
				item parents ifNotNil: [ parents addAll: item parents]]].
	parents isEmpty ifTrue: [ ^ false ].
	(inputs includesAllOf: parents) ifTrue: [ ^ true ].
	^ self 
		isValidWith: inputs
		and: parents! !

!DrGMacro methodsFor: 'comparing' stamp: 'HilaireFernandes 7/27/2008 12:11'!
= aMacro
	^ nodes size = aMacro nodes size 
		and: 
			[nodes with: aMacro nodes do: [:n1 :n2 |
				n1 = n2 ifFalse: [^false]].
			true]! !

!DrGMacro methodsFor: 'accessing' stamp: 'HilaireFernandes 10/13/2010 19:06'!
addNode: aNode
	nodes add: aNode! !

!DrGMacro methodsFor: 'accessing' stamp: 'HilaireFernandes 10/27/2010 11:19'!
allButInputNodes
	^ nodes reject: [:node | node isInputNode]! !

!DrGMacro methodsFor: 'private' stamp: 'HilaireFernandes 7/31/2008 11:52'!
cleanNodes
	nodes do: [ :node | 
		node itemTemplate: 
			((node itemTemplate copy) 
				parents: nil;
				yourself).
		node itemTemplate releaseActionMap.]! !

!DrGMacro methodsFor: 'accessing' stamp: 'HilaireFernandes 7/31/2008 14:47'!
constructedNodes
	^ nodes select: [:node | node isInputNode not]! !

!DrGMacro methodsFor: 'accessing' stamp: 'HilaireFernandes 7/26/2008 17:10'!
description
	^ description! !

!DrGMacro methodsFor: 'accessing' stamp: 'HilaireFernandes 7/26/2008 17:10'!
description: anObject
	description := anObject! !

!DrGMacro methodsFor: 'accessing' stamp: 'HilaireFernandes 10/16/2010 23:49'!
detectId: anId
	^ nodes detect: [:node | node isId: anId]! !

!DrGMacro methodsFor: 'as yet unclassified' stamp: 'HilaireFernandes 1/8/2012 14:36'!
execute: inputs 
	self cleanNodes.
	self inputNodes 
		with: inputs
		do: [ :node :item | node itemTemplate: item ].
	self allButInputNodes do: [:node | 
		node itemTemplate parents: (node parentNodes collect: 
			[:aNode | aNode itemTemplate]).
		node itemTemplate update].
	^ self constructedNodes collect: [ :n | n itemTemplate ]! !

!DrGMacro methodsFor: 'private' stamp: 'HilaireFernandes 7/27/2008 20:29'!
extractItemsFrom: inputs to: outputs in: domain 
	"extract the items to define the whole macro structure"
	| items parents tmp |
	items := (Set
		withAll: inputs)
		addAll: outputs;
		yourself.
	parents := outputs.
	[ (inputs includesAllOf: parents) | parents isEmpty ] whileFalse: 
		[ tmp := parents.
		parents := Set new.
		tmp do: [ :item | item parents ifNotNil: [ parents addAll: item parents ] ].
		items addAll: parents ].
	^ domain factory ordered: items! !

!DrGMacro methodsFor: 'initialize-release' stamp: 'HilaireFernandes 7/31/2008 11:54'!
from: inputs to: outputs in: domain 
	"build the tree nodes"
	| items |
	items := self 
		extractItemsFrom: inputs
		to: outputs
		in: domain.
	nodes := items collect: 
		[ :item | 
		DrGMacroNode 
			on: item
			from: inputs
			to: outputs ].
	self updateParentNodes.
	self cleanNodes! !

!DrGMacro methodsFor: 'testing' stamp: 'HilaireFernandes 7/27/2008 12:18'!
hash
	^nodes hash! !

!DrGMacro methodsFor: 'accessing' stamp: 'HilaireFernandes 9/27/2010 21:05'!
hiddenNodes
	^ nodes select: [:node | node isHiddenNode]! !

!DrGMacro methodsFor: 'initialize-release' stamp: 'HilaireFernandes 10/13/2010 19:06'!
initialize 
	nodes := OrderedCollection new.! !

!DrGMacro methodsFor: 'accessing' stamp: 'HilaireFernandes 7/29/2008 09:32'!
inputBasicType
"The basic type of the input parameters"
	^self inputNodes collect: [:n | n itemTemplate basicType]! !

!DrGMacro methodsFor: 'accessing' stamp: 'HilaireFernandes 7/26/2008 15:42'!
inputNodes
	^ nodes select: [:node | node isInputNode]! !

!DrGMacro methodsFor: 'private' stamp: 'HilaireFernandes 7/26/2008 21:15'!
itemToNode: item
	^ nodes detect: [:node |node itemTemplate = item]! !

!DrGMacro methodsFor: 'accessing' stamp: 'HilaireFernandes 7/27/2008 12:03'!
nodes
	^nodes! !

!DrGMacro methodsFor: 'accessing' stamp: 'HilaireFernandes 7/26/2008 15:43'!
outputNodes
	^ nodes select: [:node | node isOutputNode]! !

!DrGMacro methodsFor: 'accessing' stamp: 'HilaireFernandes 7/26/2008 17:10'!
title
	^ title! !

!DrGMacro methodsFor: 'accessing' stamp: 'HilaireFernandes 7/26/2008 17:10'!
title: anObject
	title := anObject! !

!DrGMacro methodsFor: 'private' stamp: 'HilaireFernandes 7/31/2008 13:47'!
updateParentNodes
	nodes do: 
		[ :node | 
		node isInputNode | node itemTemplate parents isNil ifFalse: 
			[ node parentNodes: (node itemTemplate parents collect: [ :item | self itemToNode: item ]) ] ]! !

!DrGMacro methodsFor: 'xml writting' stamp: 'HilaireFernandes 10/27/2010 11:50'!
writeAsXmlTo: aNode
	| node |
	self cleanNodes.
	node := XMLElement named: 'macro' attributes: Dictionary new.
	node attributeAt: #name put: title.
	node addElement: ((XMLElement named: #description) addContent: (XMLStringNode string: description)).
	node 
		addElement: self writeInputAsXml;
		addElement: self writeOutputAsXml.
	aNode addElement: node.
	^node
		! !

!DrGMacro methodsFor: 'xml writting' stamp: 'HilaireFernandes 10/10/2010 10:12'!
writeInputAsXml
	| element |
	element := XMLElement named: 'input'.
	self inputNodes do: [:aMacroNode | aMacroNode writeAsXmlTo: element].
	^ element ! !

!DrGMacro methodsFor: 'xml writting' stamp: 'HilaireFernandes 10/27/2010 11:30'!
writeOutputAsXml
	| element |
	element := XMLElement named: 'output'.
	self allButInputNodes do: [:aMacroNode | aMacroNode writeAsXmlTo: element].
	^ element 
! !

Object subclass: #DrGMacroMathItem
	instanceVariableNames: 'basicType id'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'DrGeoII-Core-Macro'!

!DrGMacroMathItem commentStamp: 'HilaireFernandes 10/13/2010 18:16' prior: 0!
A class to represent input math item template when reloading from a XML representation.!

!DrGMacroMathItem methodsFor: 'accessing' stamp: 'HilaireFernandes 10/13/2010 17:34'!
basicType 
	^ basicType ! !

!DrGMacroMathItem methodsFor: 'accessing' stamp: 'HilaireFernandes 10/13/2010 17:35'!
basicType: symbol
	basicType := symbol ! !

!DrGMacroMathItem methodsFor: 'accessing' stamp: 'HilaireFernandes 10/13/2010 17:35'!
id 
	^ id! !

!DrGMacroMathItem methodsFor: 'accessing' stamp: 'HilaireFernandes 10/13/2010 17:41'!
id: aSymbol
	id := aSymbol ! !

!DrGMacroMathItem methodsFor: 'accessing' stamp: 'HilaireFernandes 10/13/2010 17:56'!
parents
	^ #()! !

!DrGMacroMathItem methodsFor: 'accessing' stamp: 'HilaireFernandes 10/13/2010 17:56'!
parents: object
! !

!DrGMacroMathItem methodsFor: 'xml writing' stamp: 'HilaireFernandes 10/16/2010 09:57'!
writeAsXmlTo: aNode
	"return the newly created element"
	| node |
	node :=  XMLElement named: self basicType attributes: Dictionary new.
	node attributeAt: #type put: 'inputMacroMathItem';
		attributeAt: #id put: self id.
	aNode addElement: node.
	^node! !

Object subclass: #DrGMacroNode
	instanceVariableNames: 'itemTemplate'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'DrGeoII-Core-Macro'!

DrGMacroNode subclass: #DrGMacroHiddenNode
	instanceVariableNames: 'parentNodes'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'DrGeoII-Core-Macro'!

!DrGMacroHiddenNode methodsFor: 'comparing' stamp: 'HilaireFernandes 7/31/2008 14:08'!
= aNode
	^ super = aNode and:
		[parentNodes = aNode parentNodes]! !

!DrGMacroHiddenNode methodsFor: 'accessing' stamp: 'HilaireFernandes 7/31/2008 11:46'!
basicType
	^#hiddenMacroNode! !

!DrGMacroHiddenNode methodsFor: 'comparing' stamp: 'HilaireFernandes 7/31/2008 14:08'!
hash
	^(super hash) bitXor: parentNodes hash! !

!DrGMacroHiddenNode methodsFor: 'initialize-release' stamp: 'HilaireFernandes 10/10/2010 10:01'!
initialize 
	super initialize.
	parentNodes := OrderedCollection new.! !

!DrGMacroHiddenNode methodsFor: 'testing' stamp: 'HilaireFernandes 7/26/2008 15:42'!
isHiddenNode
	^true! !

!DrGMacroHiddenNode methodsFor: 'accessing' stamp: 'HilaireFernandes 7/26/2008 21:14'!
parentNodes
	^ parentNodes! !

!DrGMacroHiddenNode methodsFor: 'accessing' stamp: 'HilaireFernandes 7/26/2008 21:14'!
parentNodes: anObject
	parentNodes := anObject! !

!DrGMacroHiddenNode methodsFor: 'xml writting' stamp: 'HilaireFernandes 10/27/2010 11:25'!
statusElement
	^ XMLElement named: 'hidden'.
! !

!DrGMacroHiddenNode methodsFor: 'xml writting' stamp: 'HilaireFernandes 10/27/2010 12:00'!
writeAsXmlTo: aNode
	| element statusElement|
	statusElement := self statusElement.
	aNode addElement: statusElement.
	itemTemplate parents: (parentNodes collect: [:each | each itemTemplate]).
	element := itemTemplate writeAsXmlTo: statusElement.
	^ statusElement ! !

DrGMacroHiddenNode subclass: #DrGMacroOutputNode
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'DrGeoII-Core-Macro'!

!DrGMacroOutputNode methodsFor: 'accessing' stamp: 'HilaireFernandes 7/31/2008 11:47'!
basicType
	^#outputMacroNode! !

!DrGMacroOutputNode methodsFor: 'testing' stamp: 'HilaireFernandes 7/31/2008 15:17'!
isHiddenNode
	^false! !

!DrGMacroOutputNode methodsFor: 'testing' stamp: 'HilaireFernandes 7/26/2008 15:42'!
isOutputNode
	^true! !

!DrGMacroOutputNode methodsFor: 'xml writting' stamp: 'HilaireFernandes 10/27/2010 11:30'!
statusElement
	^ XMLElement named: 'final'.
! !

DrGMacroNode subclass: #DrGMacroInputNode
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'DrGeoII-Core-Macro'!

!DrGMacroInputNode commentStamp: 'HilaireFernandes 7/18/2008 10:53' prior: 0!
A DrGMacroNode is a node in a macro-construction. It represents an abstracted math item in the macro-construction tree.

Instance Variables
!

!DrGMacroInputNode methodsFor: 'comparing' stamp: 'HilaireFernandes 10/17/2010 11:54'!
= aNode
"this is what we expect for input item of the macro"
	^ self basicType = aNode basicType 
		and: [itemTemplate basicType = aNode itemTemplate basicType]
		! !

!DrGMacroInputNode methodsFor: 'accessing' stamp: 'HilaireFernandes 7/31/2008 11:47'!
basicType
	^#inputMacroNode! !

!DrGMacroInputNode methodsFor: 'testing' stamp: 'HilaireFernandes 10/17/2010 11:55'!
hash
	^ itemTemplate basicType hash bitXor: self basicType hash! !

!DrGMacroInputNode methodsFor: 'testing' stamp: 'HilaireFernandes 7/26/2008 15:42'!
isInputNode
	^true! !

!DrGMacroNode class methodsFor: 'as yet unclassified' stamp: 'HilaireFernandes 7/31/2008 14:09'!
on: item from: inputs to: outputs
"return the appropriate nodes depending on the item is an input, output or intermediate item"
	| class |
	class := DrGMacroHiddenNode.
	(inputs includes: item) ifTrue: [class := DrGMacroInputNode].
	(outputs includes: item) ifTrue: [class := DrGMacroOutputNode].
	^ class new itemTemplate: item! !

!DrGMacroNode methodsFor: 'comparing' stamp: 'HilaireFernandes 7/24/2010 19:18'!
= aNode
"this is what we expect for input item of the macro"
	^ self basicType = aNode basicType 
		and: [itemTemplate basicType = aNode itemTemplate basicType
			and: [itemTemplate  nodeType = aNode itemTemplate nodeType]]
		! !

!DrGMacroNode methodsFor: 'accessing' stamp: 'HilaireFernandes 7/31/2008 11:49'!
basicType
	^#macroNode! !

!DrGMacroNode methodsFor: 'testing' stamp: 'HilaireFernandes 7/31/2008 14:05'!
hash
	^ itemTemplate basicType hash bitXor: 
		(itemTemplate nodeType hash bitXor: self basicType hash)! !

!DrGMacroNode methodsFor: 'testing' stamp: 'HilaireFernandes 7/26/2008 15:41'!
isHiddenNode
	^false! !

!DrGMacroNode methodsFor: 'testing' stamp: 'HilaireFernandes 10/16/2010 23:39'!
isId: anID
	^ itemTemplate id = anID ! !

!DrGMacroNode methodsFor: 'testing' stamp: 'HilaireFernandes 7/26/2008 15:41'!
isInputNode
	^false! !

!DrGMacroNode methodsFor: 'testing' stamp: 'HilaireFernandes 7/26/2008 15:41'!
isOutputNode
	^false! !

!DrGMacroNode methodsFor: 'accessing' stamp: 'HilaireFernandes 7/26/2008 21:08'!
itemTemplate
	^itemTemplate! !

!DrGMacroNode methodsFor: 'accessing' stamp: 'HilaireFernandes 7/26/2008 17:57'!
itemTemplate: anItem
	itemTemplate := anItem! !

!DrGMacroNode methodsFor: 'xml writting' stamp: 'HilaireFernandes 10/10/2010 10:13'!
writeAsXmlTo: aNode
	^ itemTemplate writeAsXmlTo: aNode! !

Object subclass: #DrGMathItemBuilder
	instanceVariableNames: 'point actionMap app'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'DrGeoII-Core-Builder'!

!DrGMathItemBuilder commentStamp: '<historical>' prior: 0!
This is the root class of the MathBuilder hierarchy responsible to create new mathematic items.
A builder instance is specialized to build one type of object: free point, constrained point, line, segment, transformed mathItem...
To effectively build a new mathItem, the builder need to be fed with mathItem with 
the add: message.
At any time it is possible to check wheter the build process is terminated or not with the readyToBuild message.
Related patterns: BUILD, TEMPLATE METHOD!

DrGMathItemBuilder subclass: #DrGAngleBuilder
	instanceVariableNames: 'pointA pointB pointC vectorA vectorB'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'DrGeoII-Core-Builder'!

!DrGAngleBuilder class methodsFor: 'documentation' stamp: 'HilaireFernandes 3/13/2011 18:32'!
description 
	^ 'Angle defined by three points or two vectors.' translated! !

!DrGAngleBuilder class methodsFor: 'documentation' stamp: 'HilaireFernandes 3/13/2011 18:31'!
title
	^ 'Angle' translated! !

!DrGAngleBuilder methodsFor: 'private' stamp: 'HilaireFernandes 2/4/2006 10:59'!
addItem: aMathItemCollection at: aPoint
	super addItem: aMathItemCollection at: aPoint.
	aMathItemCollection first isPointItem
		ifTrue:
			[pointA ifNil: 
				[pointA := aMathItemCollection first.
				^self].
			pointB ifNil:
				[pointB := aMathItemCollection first.
				^self].
			pointC := aMathItemCollection first]
		ifFalse:
			[vectorA ifNil:
				[vectorA := aMathItemCollection first.
				^self].
			vectorB := aMathItemCollection first]! !

!DrGAngleBuilder methodsFor: 'accessing' stamp: 'HilaireFernandes 8/6/2008 13:52'!
arguments
	^pointA 
		ifNotNil: [{pointA . pointB . pointC. point}]
		ifNil: [{vectorA . vectorB. point}]! !

!DrGAngleBuilder methodsFor: 'testing' stamp: 'HilaireFernandes 7/24/2010 19:10'!
isWanted: aMathItemCollection
	^aMathItemCollection notEmpty
		and: 
			[(aMathItemCollection first isPointItem
				and: [aMathItemCollection first ~= pointA
					and: [aMathItemCollection first ~= pointB
						and: [aMathItemCollection first ~= pointC]]]	)
			or: 	[aMathItemCollection first isVectorItem 
				and: [aMathItemCollection first ~= vectorA
					and: [aMathItemCollection first ~= vectorB]]]]! !

!DrGAngleBuilder methodsFor: 'constant' stamp: 'HilaireFernandes 10/16/2010 15:53'!
mathItemClass
	^pointA 
		ifNotNil: [DrGAngle3ptsItem]
		ifNil: [DrGAngleVectorsItem]! !

!DrGAngleBuilder methodsFor: 'xml' stamp: 'HilaireFernandes 10/25/2010 22:52'!
postProcess: item from: node
	super postProcess: item from: node.
	(node attributeAt: #type) asSymbol == #vectors ifTrue: [
		self parseValuePosition: node of: item]! !

!DrGAngleBuilder methodsFor: 'testing' stamp: 'HilaireFernandes 7/24/2010 19:11'!
readyToBuild
	^ (pointA notNil and: [pointB notNil and: [pointC notNil]])
		or: [vectorA notNil and: [vectorB notNil]]
			! !

!DrGAngleBuilder methodsFor: 'updating' stamp: 'HilaireFernandes 2/4/2006 11:02'!
reset
	super reset.
	pointA := pointB := pointC := vectorA := vectorB := nil! !

DrGMathItemBuilder subclass: #DrGArcBuilder
	instanceVariableNames: 'pointA pointB pointC'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'DrGeoII-Core-Builder'!

!DrGArcBuilder class methodsFor: 'documentation' stamp: 'HilaireFernandes 3/13/2011 18:33'!
description 
	^  'Arc-circle defined by three points.' translated! !

!DrGArcBuilder class methodsFor: 'documentation' stamp: 'HilaireFernandes 3/13/2011 18:32'!
title
	^ 'Arc' translated! !

!DrGArcBuilder methodsFor: 'private' stamp: 'HilaireFernandes 1/16/2006 20:45'!
addItem: aMathItemCollection at: aPoint
	super addItem: aMathItemCollection  at: aPoint.
	pointA ifNil: 
		[pointA := aMathItemCollection first.
		^ self].
	pointB ifNil: 
		[pointB := aMathItemCollection first.
		^self].
	pointC ifNil: [pointC := aMathItemCollection first].
	! !

!DrGArcBuilder methodsFor: 'accessing' stamp: 'HilaireFernandes 1/16/2006 20:46'!
arguments
	^{pointA . pointB . pointC}! !

!DrGArcBuilder methodsFor: 'testing' stamp: 'HilaireFernandes 7/24/2010 19:34'!
isWanted: aMathItemCollection 
	^aMathItemCollection notEmpty
		and: [aMathItemCollection first isPointItem 
			and: [aMathItemCollection first ~= pointA 
				and: [aMathItemCollection first ~= pointB
					and: [aMathItemCollection first ~= pointC]]]]! !

!DrGArcBuilder methodsFor: 'constant' stamp: 'HilaireFernandes 10/16/2010 15:54'!
mathItemClass
	^DrGArc3ptsItem! !

!DrGArcBuilder methodsFor: 'testing' stamp: 'HilaireFernandes 7/24/2010 19:11'!
readyToBuild
	^pointA notNil and: [pointB notNil and: [pointC notNil]]! !

!DrGArcBuilder methodsFor: 'updating' stamp: 'HilaireFernandes 1/16/2006 20:52'!
reset
	super reset.
	pointA := pointB := pointC := nil! !

DrGMathItemBuilder subclass: #DrGDummyBuilder
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'DrGeoII-Core-Builder'!

!DrGDummyBuilder commentStamp: '<historical>' prior: 0!
A Builder to build nothing. Used in the select mode!

!DrGDummyBuilder methodsFor: 'private' stamp: 'HilaireFernandes 1/6/2006 10:01'!
addItem: aMathItemCollection at: aPoint
	super addItem: aMathItemCollection  at: aPoint! !

!DrGDummyBuilder methodsFor: 'testing' stamp: 'HilaireFernandes 1/6/2006 10:01'!
isWanted: aMathItemCollection
	^true! !

!DrGDummyBuilder methodsFor: 'testing' stamp: 'HilaireFernandes 12/27/2005 11:22'!
readyToBuild
	^false! !

DrGMathItemBuilder subclass: #DrGFlyPointBuilder
	instanceVariableNames: 'curveItemA curveItemB'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'DrGeoII-Core-Builder'!

!DrGFlyPointBuilder commentStamp: '<historical>' prior: 0!
To build free point and intersection point, it is also used to construct on the fly those point categories.!

!DrGFlyPointBuilder class methodsFor: 'documentation' stamp: 'HilaireFernandes 3/13/2011 18:40'!
description 
	^ 'Click to create a point: free point on the background, on a curve or on two curves intersection.' translated! !

!DrGFlyPointBuilder class methodsFor: 'documentation' stamp: 'HilaireFernandes 3/13/2011 18:40'!
title 
	^ 'Point' translated! !

!DrGFlyPointBuilder methodsFor: 'private' stamp: 'HilaireFernandes 10/25/2010 11:41'!
addItem: aMathItemCollection at: aPoint
"The behavior is a bit different as we send in the collection all the parent in one shot: 
to create the interesection the user clicks on two curves at the same time"
	super addItem: aMathItemCollection at: aPoint.
	aMathItemCollection ifNotEmpty: 
		[curveItemA := aMathItemCollection first].
	aMathItemCollection size >= 2 ifTrue:
		[curveItemB := aMathItemCollection second]! !

!DrGFlyPointBuilder methodsFor: 'accessing' stamp: 'HilaireFernandes 1/6/2006 10:38'!
arguments
	(curveItemA notNil and: [curveItemB notNil])
		ifTrue: [^{curveItemA . curveItemB . point}].
	curveItemA ifNotNil: [^{curveItemA . point}].
	^{point}
			! !

!DrGFlyPointBuilder methodsFor: 'testing' stamp: 'HilaireFernandes 3/21/2010 20:33'!
isCurveIntersectionValid: aCollection
	| item state |
	state := self saveState.
	self restoreState: aCollection.
	[item := self buildItem]
		on: Error 
		do: [self restoreState: state.
			^false].
	self restoreState: state.
	^ item anyOne exist
! !

!DrGFlyPointBuilder methodsFor: 'testing' stamp: 'HilaireFernandes 8/23/2008 15:54'!
isWanted: aMathItemCollection
	^self isWanted: aMathItemCollection  at: 0@0! !

!DrGFlyPointBuilder methodsFor: 'testing' stamp: 'HilaireFernandes 7/24/2010 19:15'!
isWanted: aMathItemCollection at: aPoint
	"click on the background?"
	aMathItemCollection ifEmpty: [^true].
	"click on two curves?"
	(aMathItemCollection size >= 2
		and: [aMathItemCollection first isCurveItem
			and: [aMathItemCollection second isCurveItem]])
	ifTrue: [^ self isCurveIntersectionValid: {aMathItemCollection first. aMathItemCollection second. aPoint}].
	"click on one curve?"
	^aMathItemCollection first isCurveItem		! !

!DrGFlyPointBuilder methodsFor: 'xml' stamp: 'HilaireFernandes 11/1/2011 18:23'!
itemInstanceFrom: node
	self shouldNotImplement! !

!DrGFlyPointBuilder methodsFor: 'constant' stamp: 'HilaireFernandes 10/16/2010 16:13'!
mathItemClass
	(curveItemA notNil and: [curveItemB notNil]) 
		ifTrue: [^DrGPointIntersectionItem].
	curveItemA ifNotNil: [^DrGPointOncurveItem].
	point ifNotNil: [^DrGPointFreeItem].
	self error: 'Should not be there!!'! !

!DrGFlyPointBuilder methodsFor: 'testing' stamp: 'HilaireFernandes 1/6/2006 10:32'!
readyToBuild
	^ curveItemA notNil or: [point notNil]! !

!DrGFlyPointBuilder methodsFor: 'updating' stamp: 'HilaireFernandes 1/8/2006 19:50'!
reset
	super reset.
	curveItemA := curveItemB := nil! !

!DrGFlyPointBuilder methodsFor: 'private' stamp: 'HilaireFernandes 8/25/2008 14:05'!
restoreState: anArray
	curveItemA := anArray first.
	curveItemB := anArray second.
	point := anArray third! !

!DrGFlyPointBuilder methodsFor: 'private' stamp: 'HilaireFernandes 8/25/2008 14:04'!
saveState
	^Array with: curveItemA with: curveItemB with: point! !

DrGMathItemBuilder subclass: #DrGFreePointBuilder
	instanceVariableNames: 'curveItem'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'DrGeoII-Core-Builder'!

!DrGFreePointBuilder methodsFor: 'private' stamp: 'HilaireFernandes 1/6/2006 10:13'!
addItem: aMathItemCollection at: aPoint
	super addItem: aMathItemCollection  at: aPoint.
	aMathItemCollection 
		ifNotEmpty: [curveItem := aMathItemCollection first].! !

!DrGFreePointBuilder methodsFor: 'accessing' stamp: 'HilaireFernandes 12/19/2005 22:29'!
arguments
	curveItem ifNotNil: 	
		[^ OrderedCollection new 
			add: curveItem; 
			add: point; 
			yourself].
	^ OrderedCollection new add: point; yourself! !

!DrGFreePointBuilder methodsFor: 'testing' stamp: 'HilaireFernandes 1/6/2006 10:13'!
isWanted: aMathItemCollection
	"the user clicked on the background: we can build a free point" 
	aMathItemCollection ifEmpty: [^true].
	^aMathItemCollection first isCurveItem.! !

!DrGFreePointBuilder methodsFor: 'xml' stamp: 'HilaireFernandes 11/5/2011 15:45'!
itemInstanceFrom: node
	"works both for free point and free point on a curve"
	self addItem: (self parentsById: node) at: 0@0.
	^ self getItem first.
! !

!DrGFreePointBuilder methodsFor: 'constant' stamp: 'HilaireFernandes 10/16/2010 16:13'!
mathItemClass
	curveItem ifNotNil: [^DrGPointOncurveItem].
	point ifNotNil: [^DrGPointFreeItem].
	self error: 'Should not be there!!'! !

!DrGFreePointBuilder methodsFor: 'xml' stamp: 'HilaireFernandes 10/25/2010 22:44'!
postProcess: item from: node
	super postProcess: item from: node.
	(node attributeAt: #type) asSymbol == #Free
		ifFalse: [
			item setCurveAbscissa: (node elementAt: #value) contentString asNumber] 
		ifTrue: [ |x y|
			x := (node elementAt: #x) contentString asNumber.
			y := (node elementAt: #y) contentString asNumber.
			item point: x@y]
! !

!DrGFreePointBuilder methodsFor: 'testing' stamp: 'HilaireFernandes 12/19/2005 14:54'!
readyToBuild
	^ curveItem ~= nil or: [point ~= nil]! !

!DrGFreePointBuilder methodsFor: 'updating' stamp: 'HilaireFernandes 1/8/2006 19:49'!
reset
	super reset.
	curveItem := nil.! !

DrGMathItemBuilder subclass: #DrGLocusBuilder
	instanceVariableNames: 'freePoint constrainedPoint'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'DrGeoII-Core-Builder'!

!DrGLocusBuilder class methodsFor: 'documentation' stamp: 'HilaireFernandes 3/13/2011 20:45'!
description
	^ 'Locus defined by a free point on a curve and a relative point.' translated! !

!DrGLocusBuilder class methodsFor: 'documentation' stamp: 'HilaireFernandes 3/13/2011 20:44'!
title 
	^ 'Locus' translated.! !

!DrGLocusBuilder methodsFor: 'private' stamp: 'jlrr 11/13/2007 12:05'!
addItem: aMathItemCollection at: aPoint 
	super addItem: aMathItemCollection at: aPoint.
	aMathItemCollection first isPointItemOnCurve
		ifTrue: [freePoint
				ifNil: [freePoint := aMathItemCollection first.
					^ self].
			constrainedPoint
				ifNil: [constrainedPoint := aMathItemCollection first]]
		ifFalse: [constrainedPoint
				ifNil: [constrainedPoint := aMathItemCollection first.
					^ self].
			freePoint
				ifNil: [freePoint := aMathItemCollection first]]! !

!DrGLocusBuilder methodsFor: 'accessing' stamp: 'jlrr 11/13/2007 12:05'!
arguments
	^ {freePoint. constrainedPoint}! !

!DrGLocusBuilder methodsFor: 'testing' stamp: 'HilaireFernandes 7/24/2010 19:18'!
isWanted: aMathItemCollection 
	^ aMathItemCollection notEmpty
		and: [(aMathItemCollection first isPointItemOnCurve
					and: [aMathItemCollection first ~= freePoint
						and: [aMathItemCollection first ~= constrainedPoint]])
				or: [aMathItemCollection first isConstrainedPointItem 
						and: [aMathItemCollection first ~= freePoint
							and: [aMathItemCollection first ~= constrainedPoint]]]]! !

!DrGLocusBuilder methodsFor: 'constant' stamp: 'HilaireFernandes 10/16/2010 16:08'!
mathItemClass
	^ DrGLocus2ptsItem! !

!DrGLocusBuilder methodsFor: 'testing' stamp: 'jlrr 11/13/2007 12:05'!
readyToBuild
	^ freePoint notNil
		and: [constrainedPoint notNil]! !

!DrGLocusBuilder methodsFor: 'updating' stamp: 'jlrr 11/13/2007 12:06'!
reset
	super reset.
	freePoint := constrainedPoint := nil! !

DrGMathItemBuilder subclass: #DrGMacroBuilder
	instanceVariableNames: 'wizard input output description title'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'DrGeoII-Core-Builder'!

!DrGMacroBuilder commentStamp: 'HilaireFernandes 7/18/2008 10:57' prior: 0!
A DrGMacroBuilder is builder to construct a macro-construction.
The builder needs an appropriate list of:
 - input & output mathItems selected in the drawing area
 - a macro-construction name & description

The builder checks the validity of the macro-construction, ie it checks if the outputs
items are deductible from the input items

Instance Variables
!

!DrGMacroBuilder class methodsFor: 'documentation' stamp: 'HilaireFernandes 3/13/2011 20:46'!
description 
	^ 'Construct a macro-construction with input and output items.' translated! !

!DrGMacroBuilder class methodsFor: 'documentation' stamp: 'HilaireFernandes 3/13/2011 20:46'!
title
	^ 'Build macro' translated.! !

!DrGMacroBuilder methodsFor: 'private' stamp: 'HilaireFernandes 7/24/2008 23:33'!
addItem: aMathItemCollection at: aPoint
	super addItem: aMathItemCollection at: aPoint.
	self isInputMode ifTrue: [self addToggleItem: aMathItemCollection in: input].
	self isOutputMode ifTrue: [self addToggleItem: aMathItemCollection in: output]! !

!DrGMacroBuilder methodsFor: 'callback' stamp: 'HilaireFernandes 7/27/2008 20:16'!
apply
	"the user clicked the apply button, do we have all the needed information
	to instantiate a new macro construction?"
	self isMacroValid ifTrue: 
		[ "we can instantiate a macro"
		self triggerEvent: #readyToBuild.
		^self].
	self isLogicValid ifFalse: 
		[self inform: 'Input and output items don''t match' translated].
	self isDescriptionValid ifFalse: 
		[self inform: 'Please, enter a proper title and description' translated]! !

!DrGMacroBuilder methodsFor: 'accessing' stamp: 'HilaireFernandes 7/20/2008 18:11'!
arguments
	self shouldNotImplement! !

!DrGMacroBuilder methodsFor: 'building' stamp: 'HilaireFernandes 7/27/2008 20:28'!
buildItem
	self shouldNotImplement! !

!DrGMacroBuilder methodsFor: 'building' stamp: 'HilaireFernandes 7/31/2008 11:54'!
buildItem: domain 
	| macro |
	macro := DrGMacro new
		title: title; description: description.
	macro 	from: input to: output 	in: domain.
	^ macro! !

!DrGMacroBuilder methodsFor: 'callback' stamp: 'HilaireFernandes 7/25/2008 19:34'!
cancel
	"the user withdrawn the macro-construction creation"
	self triggerEvent: #cancel! !

!DrGMacroBuilder methodsFor: 'constant' stamp: 'HilaireFernandes 7/22/2008 12:48'!
defaultDescription
	^'Write down your description there.' asText! !

!DrGMacroBuilder methodsFor: 'constant' stamp: 'HilaireFernandes 7/22/2008 12:48'!
defaultTitle
	^'Macro name' asText! !

!DrGMacroBuilder methodsFor: 'accessing' stamp: 'HilaireFernandes 7/22/2008 12:50'!
description
	^description ! !

!DrGMacroBuilder methodsFor: 'accessing' stamp: 'HilaireFernandes 7/22/2008 12:50'!
description: string
	description := string! !

!DrGMacroBuilder methodsFor: 'building' stamp: 'HilaireFernandes 7/27/2008 20:27'!
getItem
	self shouldNotImplement! !

!DrGMacroBuilder methodsFor: 'building' stamp: 'HilaireFernandes 7/27/2008 20:27'!
getItem: domain
	self isMacroValid ifTrue:
		[^self buildItem: domain].
	^nil! !

!DrGMacroBuilder methodsFor: 'initialize-release' stamp: 'HilaireFernandes 7/28/2008 11:57'!
initialize
	self reset.
	wizard := (DrGWizardMacroBuild builder: self) openInWorld.
	self registerEvents ! !

!DrGMacroBuilder methodsFor: 'accessing' stamp: 'HilaireFernandes 7/22/2008 12:54'!
input
	^input! !

!DrGMacroBuilder methodsFor: 'accessing' stamp: 'HilaireFernandes 7/25/2008 13:07'!
inputName
	^ self itemsName: input! !

!DrGMacroBuilder methodsFor: 'testing' stamp: 'HilaireFernandes 7/22/2008 12:56'!
isDescriptionValid
	^ ((title = self defaultTitle) | (description = self defaultDescription)) not! !

!DrGMacroBuilder methodsFor: 'testing' stamp: 'HilaireFernandes 7/24/2008 23:31'!
isInputMode
	^ wizard isInputPage! !

!DrGMacroBuilder methodsFor: 'testing' stamp: 'HilaireFernandes 8/25/2008 10:46'!
isLogicValid
	"check wheter a macro can be deduced from the input/output item collections"
	^ DrGMacro 
		isValidWith: input
		and: output! !

!DrGMacroBuilder methodsFor: 'testing' stamp: 'HilaireFernandes 7/27/2008 20:17'!
isMacroValid 
	^self isLogicValid & self isDescriptionValid ! !

!DrGMacroBuilder methodsFor: 'testing' stamp: 'HilaireFernandes 7/24/2008 23:31'!
isOutputMode
	^ wizard isOutputPage! !

!DrGMacroBuilder methodsFor: 'testing' stamp: 'HilaireFernandes 7/25/2008 10:21'!
isWanted: aMathItemCollection 
	^ aMathItemCollection notEmpty and: 
		[ self isInputMode &  (output includes: aMathItemCollection first) not or: [ self isOutputMode & (input includes: aMathItemCollection first) not ] ]! !

!DrGMacroBuilder methodsFor: 'accessing' stamp: 'HilaireFernandes 7/25/2008 13:00'!
itemsName: aMathItemCollection
	^aMathItemCollection collect: [:mathItem | mathItem descriptiveName]! !

!DrGMacroBuilder methodsFor: 'constant' stamp: 'HilaireFernandes 7/20/2008 18:09'!
mathItemClass
	self shouldNotImplement! !

!DrGMacroBuilder methodsFor: 'accessing' stamp: 'HilaireFernandes 7/22/2008 12:54'!
output
	^output ! !

!DrGMacroBuilder methodsFor: 'accessing' stamp: 'HilaireFernandes 7/25/2008 13:07'!
outputName
	^ self itemsName: output! !

!DrGMacroBuilder methodsFor: 'callback' stamp: 'HilaireFernandes 7/28/2008 12:11'!
page: integer 
	"we receive notification from the wizard the user change the wizard
	current page"
	| newState |
	(integer = 1 or: [ integer = 4 ]) ifTrue: [ newState := DrGMacroToolStateNeutral ].
	integer = 2 ifTrue: [ newState := DrGMacroBuildToolStateInput ].
	integer = 3 ifTrue: [ newState := DrGMacroBuildToolStateOutput ].
	self 
		triggerEvent: #changedState
		with: newState! !

!DrGMacroBuilder methodsFor: 'testing' stamp: 'HilaireFernandes 7/22/2008 15:09'!
readyToBuild
"Only when the user clicks on the wizard 'apply' button,
the system checks wheter the macro can be instiated"
	^false! !

!DrGMacroBuilder methodsFor: 'initialize-release' stamp: 'HilaireFernandes 7/20/2008 10:28'!
registerEvents
	wizard when: #page send: #page: to: self.
	wizard when: #apply send: #apply to: self.
	wizard when: #cancel send: #cancel to: self.! !

!DrGMacroBuilder methodsFor: 'initialize-release' stamp: 'HilaireFernandes 7/28/2008 09:57'!
release
	self removeEvents.
	wizard delete.
	wizard := nil! !

!DrGMacroBuilder methodsFor: 'initialize-release' stamp: 'HilaireFernandes 7/20/2008 10:30'!
removeEvents
	wizard removeActionsWithReceiver: self! !

!DrGMacroBuilder methodsFor: 'updating' stamp: 'HilaireFernandes 8/4/2008 09:44'!
reset
	super reset.
	input := OrderedCollection new.
	output := OrderedCollection new.
	title := self defaultTitle.
	description := self defaultDescription.
	self changed: #inputName.
	self changed: #ouputName.
	self changed: #title.
	self changed: #description! !

!DrGMacroBuilder methodsFor: 'accessing' stamp: 'HilaireFernandes 7/22/2008 12:49'!
title
	^title! !

!DrGMacroBuilder methodsFor: 'accessing' stamp: 'HilaireFernandes 7/22/2008 12:49'!
title: string
	title := string! !

DrGMathItemBuilder subclass: #DrGMacroPlayer
	instanceVariableNames: 'wizard selectedMacro input'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'DrGeoII-Core-Builder'!

!DrGMacroPlayer commentStamp: 'HilaireFernandes 1/8/2012 12:51' prior: 0!
A DrGMacroPlayer is a builder used to play a macro-construction against a user provided list of input items.
It chekcs the validity of the input items against the macro-construction definition.
The input parameters type must be user selected in the same other as they appear in the macri inputNodes definition.
Instance Variables
!

!DrGMacroPlayer class methodsFor: 'documentation' stamp: 'HilaireFernandes 3/13/2011 20:47'!
description 
	^'Execute a pre-built macro-construction on selected input items.' translated! !

!DrGMacroPlayer class methodsFor: 'documentation' stamp: 'HilaireFernandes 3/13/2011 20:47'!
title
	^ 'Execute Macro' translated! !

!DrGMacroPlayer methodsFor: 'private' stamp: 'HilaireFernandes 7/29/2008 09:50'!
addItem: aMathItemCollection at: aPoint
	super addItem: aMathItemCollection at: aPoint.
	self addToggleItem: aMathItemCollection in: input! !

!DrGMacroPlayer methodsFor: 'building' stamp: 'HilaireFernandes 7/31/2008 13:14'!
buildItem
	^ selectedMacro execute: input! !

!DrGMacroPlayer methodsFor: 'callback' stamp: 'HilaireFernandes 7/28/2008 12:37'!
cancel
	self triggerEvent: #cancel! !

!DrGMacroPlayer methodsFor: 'accessing' stamp: 'HilaireFernandes 7/31/2008 14:49'!
constructedNodes
	^ selectedMacro constructedNodes ! !

!DrGMacroPlayer methodsFor: 'accessing' stamp: 'HilaireFernandes 7/28/2008 12:35'!
description
	^ selectedMacro ifNotNil: [selectedMacro description] ifNil: [^nil]! !

!DrGMacroPlayer methodsFor: 'accessing' stamp: 'HilaireFernandes 7/28/2008 12:50'!
description: text 
	selectedMacro ifNotNil: [selectedMacro description: text]! !

!DrGMacroPlayer methodsFor: 'initialize-release' stamp: 'HilaireFernandes 7/29/2008 09:43'!
initialize
	wizard := (DrGWizardMacroPlay builder: self) openInWorld.
	input := OrderedCollection new.
	self registerEvents! !

!DrGMacroPlayer methodsFor: 'accessing' stamp: 'HilaireFernandes 7/28/2008 12:13'!
input
	^input! !

!DrGMacroPlayer methodsFor: 'accessing' stamp: 'HilaireFernandes 7/29/2008 09:41'!
inputBasicType
	^input collect: [:i | i basicType]! !

!DrGMacroPlayer methodsFor: 'testing' stamp: 'HilaireFernandes 1/8/2012 12:59'!
isWanted: aMathItemCollection
	selectedMacro isNil ifTrue: [^false].
	^aMathItemCollection notEmpty and: 
		[(selectedMacro inputBasicType at: input size + 1 ) == aMathItemCollection first basicType]! !

!DrGMacroPlayer methodsFor: 'callback' stamp: 'HilaireFernandes 7/29/2008 10:14'!
page: integer 
	"we receive notification from the wizard the user change the wizard
	current page"
	| newState |
	integer = 1 
		ifTrue: [ newState := DrGMacroToolStateNeutral ]
		ifFalse: [ newState := DrGMacroPlayToolStateSelect].
	self 
		triggerEvent: #changedState
		with: newState! !

!DrGMacroPlayer methodsFor: 'testing' stamp: 'HilaireFernandes 10/29/2010 09:35'!
readyToBuild
	^ self inputBasicType = selectedMacro inputBasicType! !

!DrGMacroPlayer methodsFor: 'initialize-release' stamp: 'HilaireFernandes 7/28/2008 10:45'!
registerEvents
	wizard when: #page send: #page: to: self.
	wizard when: #cancel send: #cancel to: self.! !

!DrGMacroPlayer methodsFor: 'initialize-release' stamp: 'HilaireFernandes 7/28/2008 10:46'!
release
	self removeEvents.
	wizard delete.
	wizard := nil! !

!DrGMacroPlayer methodsFor: 'initialize-release' stamp: 'HilaireFernandes 7/28/2008 10:46'!
removeEvents
	wizard removeActionsWithReceiver: self! !

!DrGMacroPlayer methodsFor: 'updating' stamp: 'HilaireFernandes 8/4/2008 09:50'!
reset
	super reset.
	input := OrderedCollection new.
	selectedMacro := nil.
	self changed: #description.
	wizard changed: #selectedMacro! !

!DrGMacroPlayer methodsFor: 'accessing' stamp: 'HilaireFernandes 7/28/2008 12:34'!
selectedMacro
	^selectedMacro ! !

!DrGMacroPlayer methodsFor: 'accessing' stamp: 'HilaireFernandes 1/8/2012 12:55'!
selectedMacro: aMacro
	"Some cleaning are needed"
	selectedMacro := aMacro.
	input := OrderedCollection new.
	self triggerEvent: #changedMacro.! !

!DrGMathItemBuilder class methodsFor: 'xml' stamp: 'HilaireFernandes 11/8/2010 22:02'!
basicItemFrom: node
"create a basic item (unitialised) from this node and add it to the domain, return an item partially initialized. Used for macro node"
	| builder |
	builder := self new.
	^ builder basicItemFrom: node! !

!DrGMathItemBuilder class methodsFor: 'documentation' stamp: 'HilaireFernandes 3/13/2011 18:29'!
description
	"A description how to use the tool"
	self subclassResponsibility! !

!DrGMathItemBuilder class methodsFor: 'xml' stamp: 'HilaireFernandes 10/25/2010 08:25'!
itemFrom: node in: aDrgeoApp
"create an item from this node and add it to the domain, return a couple {item. boolean}"
	| builder |
	builder := self new app: aDrgeoApp; yourself.
	^ builder itemFrom: node! !

!DrGMathItemBuilder class methodsFor: 'documentation' stamp: 'HilaireFernandes 3/13/2011 18:29'!
title
	"One or two words describing the builder"
	self subclassResponsibility ! !

!DrGMathItemBuilder methodsFor: 'event' stamp: 'HilaireFernandes 3/22/2006 12:02'!
actionMap
	actionMap ifNil: [^self createActionMap].
	^ actionMap ! !

!DrGMathItemBuilder methodsFor: 'building' stamp: 'HilaireFernandes 8/23/2008 15:51'!
add: aMathItemCollection at: aPoint
"Add a math item in the selection list of the builder,
aPoint is the position where the action took place
Return true if mathItem accepted"
	(self isWanted: aMathItemCollection at: aPoint) ifTrue:
		[self addItem: aMathItemCollection at: aPoint.
		"Are we done? If so notify our dependent"
		self readyToBuild ifTrue: 
			[self triggerEvent: #readyToBuild].
		^true].
	^false
! !

!DrGMathItemBuilder methodsFor: 'private' stamp: 'HilaireFernandes 1/6/2006 09:46'!
addItem: aMathItemCollection at: aPoint
	point := aPoint.! !

!DrGMathItemBuilder methodsFor: 'private' stamp: 'HilaireFernandes 7/29/2008 09:48'!
addToggleItem: aMathItemCollection in: aCollection
	(aCollection includes: aMathItemCollection first) 
		ifTrue: [aCollection remove: aMathItemCollection first ]
		ifFalse: [aCollection add: aMathItemCollection first]! !

!DrGMathItemBuilder methodsFor: 'accessing' stamp: 'HilaireFernandes 10/25/2010 22:43'!
app: aDrgeoApp
	app := aDrgeoApp ! !

!DrGMathItemBuilder methodsFor: 'accessing' stamp: 'HilaireFernandes 12/19/2005 20:39'!
arguments
"an argument list to give to the MathItem new method"
	self subclassResponsibility ! !

!DrGMathItemBuilder methodsFor: 'xml' stamp: 'HilaireFernandes 10/27/2010 10:52'!
basicItemFrom: node
| type itemClass item|
	"build the item class name from the node description"
	type := ((node  attributeAt: #type) copyWithoutAll: '_') capitalized.
	itemClass := self class environment 
		classNamed: ('DrG' , node tag capitalized , type , 'Item') asSymbol.
	item := itemClass new yourself.
	self postProcess: item from: node.
	^ item! !

!DrGMathItemBuilder methodsFor: 'building' stamp: 'HilaireFernandes 3/5/2006 14:48'!
buildItem
	^ {self mathItemClass newWith: self arguments}! !

!DrGMathItemBuilder methodsFor: 'accessing' stamp: 'HilaireFernandes 10/25/2010 08:18'!
domain
	^ app domain! !

!DrGMathItemBuilder methodsFor: 'accessing' stamp: 'HilaireFernandes 10/25/2010 08:18'!
factory
	^ self domain factory! !

!DrGMathItemBuilder methodsFor: 'building' stamp: 'HilaireFernandes 12/6/2005 15:10'!
getItem
	"return an instantiated mathItem or nil if not possible"
	self readyToBuild ifTrue:
		[^ self buildItem].
	^ nil.! !

!DrGMathItemBuilder methodsFor: 'testing' stamp: 'HilaireFernandes 1/6/2006 09:46'!
isWanted: aMathItemCollection
"Check if the builder is interested by aMathItem"
	self subclassResponsibility ! !

!DrGMathItemBuilder methodsFor: 'testing' stamp: 'HilaireFernandes 8/23/2008 15:52'!
isWanted: aMathItemCollection at: aPoint
"By default we don't need the point position of the click event'"
	^ self isWanted: aMathItemCollection ! !

!DrGMathItemBuilder methodsFor: 'xml' stamp: 'HilaireFernandes 10/28/2011 18:01'!
itemFrom: node
	|item|
	item := self itemInstanceFrom: node.
	self postProcess: item from: node.
	^ {self domain createFromMathItemNoStackNoFactoryCheck: item. true}! !

!DrGMathItemBuilder methodsFor: 'xml' stamp: 'HilaireFernandes 11/5/2011 15:45'!
itemInstanceFrom: node
"build an item from a XML description"
	(self parentsById: node) do: [:each |	self addItem: {each} at: 0@0].
	^ self getItem first.
! !

!DrGMathItemBuilder methodsFor: 'constant' stamp: 'HilaireFernandes 12/6/2005 16:27'!
mathItemClass
"Answer the MathItem class for this builder (i.e. self FreePointItem,...)"
	self subclassResponsibility! !

!DrGMathItemBuilder methodsFor: 'xml' stamp: 'HilaireFernandes 10/25/2010 08:19'!
parentsById: node
"return a collection of the item parent given theirs id in the attribute
elements <parent id=xxx>"
	|collection|
	collection := OrderedCollection new.
	node elementsDo: [:each | 
		(each tag = #parent) ifTrue: [
			collection add: (self factory findInPoolById: (each attributeAt: 'ref') asSymbol)]].
	^collection ! !

!DrGMathItemBuilder methodsFor: 'xml' stamp: 'HilaireFernandes 11/5/2010 22:42'!
parseValuePosition: node
	| x y |
	x := (node elementAt: #x) contentString asNumber.
	y := (node elementAt: #y) contentString asNumber.
	^ x@y.! !

!DrGMathItemBuilder methodsFor: 'xml' stamp: 'HilaireFernandes 11/5/2010 22:43'!
parseValuePosition: node of: item
	item position: (self parseValuePosition: node)! !

!DrGMathItemBuilder methodsFor: 'xml' stamp: 'HilaireFernandes 11/6/2011 22:17'!
postProcess: item from: node
	item id: (node attributeAt: 'id') asSymbol.
	(item respondsTo: #isLocked) ifTrue: [
		(node attributeAt: #locked) =  'True'
			ifTrue: [	item locked: true]
			ifFalse: [item locked: false]]! !

!DrGMathItemBuilder methodsFor: 'testing' stamp: 'HilaireFernandes 12/6/2005 16:40'!
readyToBuild
	"Can the builder build the math item now?"
	self subclassResponsibility ! !

!DrGMathItemBuilder methodsFor: 'event' stamp: 'HilaireFernandes 3/22/2006 16:54'!
releaseActionMap
	actionMap := nil! !

!DrGMathItemBuilder methodsFor: 'updating' stamp: 'HilaireFernandes 1/8/2006 19:46'!
reset
	point := nil.! !

!DrGMathItemBuilder methodsFor: 'event' stamp: 'HilaireFernandes 3/22/2006 11:53'!
updateableActionMap
	actionMap ifNil:
		[actionMap := self createActionMap].
	^actionMap! !

DrGMathItemBuilder subclass: #DrGMiddleBuilder
	instanceVariableNames: 'pointA pointB segment'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'DrGeoII-Core-Builder'!

!DrGMiddleBuilder class methodsFor: 'documentation' stamp: 'HilaireFernandes 3/13/2011 20:48'!
description 
	^ 'The midpoint of a segment or between two points.' translated! !

!DrGMiddleBuilder class methodsFor: 'documentation' stamp: 'HilaireFernandes 3/13/2011 20:48'!
title 
	^ 'Middle' translated. ! !

!DrGMiddleBuilder methodsFor: 'private' stamp: 'HilaireFernandes 2/4/2006 10:57'!
addItem: aMathItemCollection at: aPoint
	super addItem: aMathItemCollection at: aPoint.
	aMathItemCollection first isPointItem 
		ifTrue:
			[pointA ifNil: 
				[pointA := aMathItemCollection first.
				^self].
			pointB ifNil:
				[pointB := aMathItemCollection  first.
				^self]]
		ifFalse: [segment := aMathItemCollection  first]! !

!DrGMiddleBuilder methodsFor: 'accessing' stamp: 'HilaireFernandes 1/2/2006 22:29'!
arguments
	segment ifNotNil: [^{segment}].
	^{pointA . pointB}! !

!DrGMiddleBuilder methodsFor: 'testing' stamp: 'HilaireFernandes 7/24/2010 19:23'!
isWanted: aMathItemCollection
	^ aMathItemCollection  notEmpty
		and:	[
			(aMathItemCollection first isPointItem 
				and: [(pointA isNil or: [pointB isNil])
					and: [aMathItemCollection first ~= pointA
						and: [aMathItemCollection first ~= pointB]]])
			or: [aMathItemCollection first isSegmentItem 
				and: [segment isNil]]]! !

!DrGMiddleBuilder methodsFor: 'constant' stamp: 'HilaireFernandes 10/16/2010 16:13'!
mathItemClass
	segment ifNotNil: [^DrGPointMiddlesegmentItem].
	^DrGPointMiddle2ptsItem! !

!DrGMiddleBuilder methodsFor: 'testing' stamp: 'HilaireFernandes 1/2/2006 22:52'!
readyToBuild
	^ segment notNil or: [pointA notNil and: [pointB notNil]]! !

!DrGMiddleBuilder methodsFor: 'updating' stamp: 'HilaireFernandes 1/8/2006 19:49'!
reset
	super reset.
	pointA := pointB := segment := nil! !

DrGMathItemBuilder subclass: #DrGParallelBuilder
	instanceVariableNames: 'pointA direction'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'DrGeoII-Core-Builder'!

!DrGParallelBuilder commentStamp: '<historical>' prior: 0!
pointA can accept a PointItem instance or just a Point instance.!

!DrGParallelBuilder class methodsFor: 'documentation' stamp: 'HilaireFernandes 3/13/2011 20:50'!
description 
	^ 'Line passing through one point and parallel to a line, half-line, etc.' translated! !

!DrGParallelBuilder class methodsFor: 'documentation' stamp: 'HilaireFernandes 3/13/2011 20:50'!
title 
	^ 'Parallel' translated! !

!DrGParallelBuilder methodsFor: 'private' stamp: 'HilaireFernandes 4/10/2010 12:31'!
addItem: aMathItemCollection at: aPoint
	super addItem: aMathItemCollection  at: aPoint.
	aMathItemCollection first isPointItem 
		ifTrue: [pointA := aMathItemCollection first].
	aMathItemCollection first isDirectionItem 
		ifTrue:[direction := aMathItemCollection first].
	! !

!DrGParallelBuilder methodsFor: 'accessing' stamp: 'HilaireFernandes 1/2/2006 19:36'!
arguments
	^{direction . pointA}! !

!DrGParallelBuilder methodsFor: 'testing' stamp: 'HilaireFernandes 6/15/2010 22:37'!
isWanted: aMathItemCollection
	aMathItemCollection ifEmpty: [^false].
	^ (aMathItemCollection first isPointItem and: [pointA isNil])
		or: [aMathItemCollection first isDirectionItem and: [direction isNil]] 
! !

!DrGParallelBuilder methodsFor: 'constant' stamp: 'HilaireFernandes 10/16/2010 16:04'!
mathItemClass
	^DrGLineParallelItem! !

!DrGParallelBuilder methodsFor: 'testing' stamp: 'HilaireFernandes 1/2/2006 16:09'!
readyToBuild
	^pointA notNil and: [direction notNil]! !

!DrGParallelBuilder methodsFor: 'updating' stamp: 'HilaireFernandes 1/8/2006 19:48'!
reset
	super reset.
	pointA := direction := nil! !

DrGParallelBuilder subclass: #DrGPerpendicularBuilder
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'DrGeoII-Core-Builder'!

!DrGPerpendicularBuilder class methodsFor: 'documentation' stamp: 'HilaireFernandes 3/13/2011 20:51'!
description 
	^ 'Line passing through one point and orthogonal to a line, half-line, etc.' translated! !

!DrGPerpendicularBuilder class methodsFor: 'documentation' stamp: 'HilaireFernandes 3/13/2011 20:51'!
title 
	^ 'Perpendicular' translated! !

!DrGPerpendicularBuilder methodsFor: 'constant' stamp: 'HilaireFernandes 10/27/2010 13:13'!
mathItemClass
	^DrGLinePerpendicularItem! !

DrGMathItemBuilder subclass: #DrGPointByCoordinatesBuilder
	instanceVariableNames: 'valueX valueY script'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'DrGeoII-Core-Builder'!

!DrGPointByCoordinatesBuilder class methodsFor: 'documentation' stamp: 'HilaireFernandes 3/13/2011 22:38'!
description
	^ 'Point given its coordinates: select two numbers or a point coordinates (@).' translated! !

!DrGPointByCoordinatesBuilder class methodsFor: 'documentation' stamp: 'HilaireFernandes 3/13/2011 20:52'!
title 
	^ 'Coordinates' translated! !

!DrGPointByCoordinatesBuilder methodsFor: 'private' stamp: 'HilaireFernandes 8/6/2008 13:56'!
addItem: aMathItemCollection at: aPoint
	super addItem: aMathItemCollection at: aPoint.
	aMathItemCollection first isScriptItem 
		ifTrue: 
			[script := aMathItemCollection first]
		ifFalse: 
			[valueX 
				ifNil: [valueX := aMathItemCollection first.]
				ifNotNil: [valueY ifNil: [valueY := aMathItemCollection first]]]! !

!DrGPointByCoordinatesBuilder methodsFor: 'accessing' stamp: 'HilaireFernandes 8/6/2008 13:52'!
arguments
	^script 
		ifNotNil: [{script}]
		ifNil: [{valueX . valueY}]! !

!DrGPointByCoordinatesBuilder methodsFor: 'testing' stamp: 'HilaireFernandes 7/24/2010 19:23'!
isWanted: aMathItemCollection
	^ aMathItemCollection notEmpty
		and: 
			[(aMathItemCollection first isScriptItem
				and: [aMathItemCollection first ~= script])			
			or: [aMathItemCollection first isValueItem
				and: [aMathItemCollection first ~= valueX
					and: [aMathItemCollection first ~= valueY]]]]
			! !

!DrGPointByCoordinatesBuilder methodsFor: 'constant' stamp: 'HilaireFernandes 11/5/2010 21:49'!
mathItemClass
	^script 
		ifNotNil: [DrGPointScriptItem ]
		ifNil: [DrGPointCoordinateItem]! !

!DrGPointByCoordinatesBuilder methodsFor: 'testing' stamp: 'HilaireFernandes 8/6/2008 13:53'!
readyToBuild
	^ (valueX notNil and: [valueY notNil])
		or: [script notNil]! !

!DrGPointByCoordinatesBuilder methodsFor: 'updating' stamp: 'HilaireFernandes 8/6/2008 13:53'!
reset
	super reset.
	script := valueX := valueY := nil! !

DrGMathItemBuilder subclass: #DrGPointIntersectionBuilder
	instanceVariableNames: 'curveItemA curveItemB'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'DrGeoII-Core-Builder'!

!DrGPointIntersectionBuilder commentStamp: 'HilaireFernandes 10/30/2011 16:35' prior: 0!
The build the intersection between two curves. One or two output may be created!

!DrGPointIntersectionBuilder class methodsFor: 'documentation' stamp: 'HilaireFernandes 10/30/2011 16:37'!
description
	^'Intersection(s) of two curves.' translated! !

!DrGPointIntersectionBuilder class methodsFor: 'documentation' stamp: 'HilaireFernandes 10/30/2011 17:31'!
title
	^ 'Intersection' translated! !

!DrGPointIntersectionBuilder methodsFor: 'private' stamp: 'HilaireFernandes 10/30/2011 17:02'!
addItem: aMathItemCollection at: aPoint
	super addItem: aMathItemCollection at: aPoint.
	curveItemA ifNil: [
		curveItemA := aMathItemCollection first.
		^self].
	curveItemB ifNil: [curveItemB := aMathItemCollection first]
! !

!DrGPointIntersectionBuilder methodsFor: 'accessing' stamp: 'HilaireFernandes 10/30/2011 17:08'!
arguments 
	^{curveItemA . curveItemB.  point}! !

!DrGPointIntersectionBuilder methodsFor: 'updating' stamp: 'HilaireFernandes 10/30/2011 17:57'!
buildItem 
	^ self isArcSelected
		ifFalse: [super buildItem] 
		ifTrue: [{
			self mathItemClass newWith: self arguments k: 1.
			self mathItemClass newWith: self arguments k: -1}]! !

!DrGPointIntersectionBuilder methodsFor: 'testing' stamp: 'HilaireFernandes 10/30/2011 17:57'!
isArcSelected
	"does the user selected two arcs ?"
	^ (curveItemA isCircleItem or: [curveItemA isArcItem]) or: [
		curveItemB isCircleItem or: [curveItemB isArcItem]]! !

!DrGPointIntersectionBuilder methodsFor: 'testing' stamp: 'HilaireFernandes 10/30/2011 17:04'!
isWanted: aMathItemCollection 
	aMathItemCollection ifEmpty: [^false].
	^ aMathItemCollection first isCurveItem and: [
		curveItemA isNil or: [curveItemB isNil]]! !

!DrGPointIntersectionBuilder methodsFor: 'xml' stamp: 'HilaireFernandes 11/1/2011 18:33'!
itemInstanceFrom: node
	"We only want intersection point"
	|arguments k|
	arguments := self parentsById: node.
	"We by-pass the builder because it requires the user mouse position at intersection request "
	k :=  (node attributeAt: 'extra') asInteger = 0 ifTrue: [-1] ifFalse: [1].
	^self mathItemClass newWith: arguments k: k.! !

!DrGPointIntersectionBuilder methodsFor: 'constant' stamp: 'HilaireFernandes 10/30/2011 17:06'!
mathItemClass 
	^ DrGPointIntersectionItem! !

!DrGPointIntersectionBuilder methodsFor: 'as yet unclassified' stamp: 'HilaireFernandes 1/8/2012 15:26'!
postProcess: item from: node
	|k|
	super postProcess: item from: node.
	k :=  (node attributeAt: 'extra') asInteger = 0 ifTrue: [-1] ifFalse: [1].
	item k: k! !

!DrGPointIntersectionBuilder methodsFor: 'testing' stamp: 'HilaireFernandes 10/30/2011 17:06'!
readyToBuild 
	^ curveItemA notNil & curveItemB notNil ! !

!DrGPointIntersectionBuilder methodsFor: 'updating' stamp: 'HilaireFernandes 10/30/2011 17:07'!
reset
	super reset.
	curveItemA := curveItemB := nil! !

DrGMathItemBuilder subclass: #DrGPolygonBuilder
	instanceVariableNames: 'points'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'DrGeoII-Core-Builder'!

!DrGPolygonBuilder class methodsFor: 'documentation' stamp: 'HilaireFernandes 3/13/2011 22:39'!
description
	^ 'Polygon by n points: last point must be the initial point to terminate.' translated! !

!DrGPolygonBuilder class methodsFor: 'documentation' stamp: 'HilaireFernandes 3/13/2011 20:53'!
title 
	^ 'Polygon' translated! !

!DrGPolygonBuilder methodsFor: 'private' stamp: 'HilaireFernandes 2/6/2006 20:27'!
addItem: aMathItemCollection at: aPoint
	super addItem: aMathItemCollection at: aPoint.
	points add: aMathItemCollection first! !

!DrGPolygonBuilder methodsFor: 'accessing' stamp: 'HilaireFernandes 2/6/2006 21:37'!
arguments
	^points allButLast! !

!DrGPolygonBuilder methodsFor: 'initialize-release' stamp: 'HilaireFernandes 2/6/2006 20:40'!
initialize
	points := OrderedCollection new.
! !

!DrGPolygonBuilder methodsFor: 'testing' stamp: 'HilaireFernandes 7/24/2010 19:26'!
isWanted: aMathItemCollection
	^aMathItemCollection notEmpty
		and: [aMathItemCollection first isPointItem
			and: [points isEmpty
				or: [points first = aMathItemCollection first
					or: [(points allButFirst includes: aMathItemCollection first) not]]]]
			! !

!DrGPolygonBuilder methodsFor: 'constant' stamp: 'HilaireFernandes 10/16/2010 16:16'!
mathItemClass
	^DrGPolygonNptsItem! !

!DrGPolygonBuilder methodsFor: 'xml' stamp: 'HilaireFernandes 10/25/2010 11:16'!
parentsById: node
	|arguments|
	"To instantiate a polygon we need the first summit to be duplucated at the end of the point collection"
	arguments := super parentsById: node.
	arguments addLast: arguments first.
	^ arguments ! !

!DrGPolygonBuilder methodsFor: 'testing' stamp: 'HilaireFernandes 2/6/2006 21:45'!
readyToBuild
	^ points size > 2 and: [points first = points last]! !

!DrGPolygonBuilder methodsFor: 'updating' stamp: 'HilaireFernandes 2/6/2006 20:42'!
reset
	super reset.
	points := OrderedCollection new! !

DrGMathItemBuilder subclass: #DrGScriptPlayer
	instanceVariableNames: 'wizard selectedMethod input'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'DrGeoII-Core-Builder'!

!DrGScriptPlayer class methodsFor: 'xml' stamp: 'HilaireFernandes 11/5/2010 22:50'!
basicItemFrom: node
"create a basic item (unitialised) from this node and add it to the domain, return an item partially initialized. Use for macro node"
	| builder |
	builder := self basicNew basicInitialize.
	^ builder basicItemFrom: node! !

!DrGScriptPlayer class methodsFor: 'documentation' stamp: 'HilaireFernandes 3/13/2011 20:58'!
description
	^ 'Insert a Smalltalk script in the drawing area.' translated! !

!DrGScriptPlayer class methodsFor: 'xml' stamp: 'HilaireFernandes 11/5/2010 22:48'!
itemFrom: node in: aDrgeoApp
"create an item from this node and add it to the domain, return a couple {item. boolean}"
	| builder |
	builder := self basicNew basicInitialize; app: aDrgeoApp; yourself.
	^ builder itemFrom: node! !

!DrGScriptPlayer class methodsFor: 'documentation' stamp: 'HilaireFernandes 3/13/2011 20:58'!
title
	^ 'Use a script' translated! !

!DrGScriptPlayer methodsFor: 'private' stamp: 'HilaireFernandes 8/4/2008 14:39'!
addItem: aMathItemCollection at: aPoint
"Take the point position only when the user clic somewhere in the background"
	aMathItemCollection ifEmpty: 
		[point := aPoint.
		^self].
	input size < selectedMethod numArgs 
		ifTrue: [self addToggleItem: aMathItemCollection in: input]
		ifFalse: [(input includes: aMathItemCollection first) 
			ifTrue: [input remove: aMathItemCollection first]]! !

!DrGScriptPlayer methodsFor: 'accessing' stamp: 'HilaireFernandes 8/23/2008 22:38'!
arguments
	^ (OrderedCollection withAll: input)
		add: selectedMethod selector;
		add: point;
		yourself! !

!DrGScriptPlayer methodsFor: 'initialize-release' stamp: 'HilaireFernandes 11/5/2010 22:48'!
basicInitialize
	input := OrderedCollection new.! !

!DrGScriptPlayer methodsFor: 'callback' stamp: 'HilaireFernandes 8/2/2008 21:43'!
cancel
	self triggerEvent: #cancel! !

!DrGScriptPlayer methodsFor: 'accessing' stamp: 'HilaireFernandes 8/23/2008 22:24'!
description
	| comment |
	^ selectedMethod 
		ifNil: [nil]
		ifNotNil: 
			[comment := DrGeoUserScripts firstCommentAt: selectedMethod selector.
			comment isEmpty 
				ifTrue: ['no comment, write one at the beginning of the script source code' translated]
				ifFalse: [comment]]! !

!DrGScriptPlayer methodsFor: 'accessing' stamp: 'HilaireFernandes 8/4/2008 10:08'!
description: aText 
	! !

!DrGScriptPlayer methodsFor: 'initialize-release' stamp: 'HilaireFernandes 8/2/2008 21:44'!
initialize
	wizard := (DrGWizardScript builder: self) openInWorld.
	input := OrderedCollection new.
	self registerEvents! !

!DrGScriptPlayer methodsFor: 'accessing' stamp: 'HilaireFernandes 8/2/2008 21:45'!
input
	^input! !

!DrGScriptPlayer methodsFor: 'testing' stamp: 'HilaireFernandes 8/4/2008 14:50'!
isWanted: aMathItemCollection 
	"We are not yet performing type checking for script, we are even interested
in empty aMathItemCollection, (ie: just get the clic position in the background)"
	selectedMethod ifNil: [^false].
	aMathItemCollection ifEmpty: [ ^ true ].
	^ input size = selectedMethod numArgs 
		ifTrue: 
			[ (input includes: aMathItemCollection first) 
				ifTrue: [ true ]
				ifFalse: [ false ] ]
		ifFalse: [ true ]! !

!DrGScriptPlayer methodsFor: 'xml' stamp: 'Toto 11/13/2010 16:11'!
itemInstanceFrom: node
	|selector method|
	selector := Author 
		useAuthor:  'DrGeoUser' 
		during: [DrGeoUserScripts compile: (node elementAt: #code) contentString
			classified: 'scripts'].
	self selectedMethod: (DrGeoUserScripts compiledMethodAt: selector).
	self add: #() at: 0@0.
	(self parentsById: node) do: [:each |	self addItem: {each} at: 0@0].
	^ self getItem first.! !

!DrGScriptPlayer methodsFor: 'testing' stamp: 'HilaireFernandes 11/5/2010 21:51'!
mathItemClass
	^DrGValueScriptItem ! !

!DrGScriptPlayer methodsFor: 'callback' stamp: 'HilaireFernandes 8/2/2008 21:48'!
page: integer 
	"we receive notification from the wizard the user change the wizard
	current page"
	| newState |
	integer = 1 
		ifTrue: [ newState := DrGMacroToolStateNeutral ]
		ifFalse: [ newState := DrGMacroPlayToolStateSelect].
	self 
		triggerEvent: #changedState
		with: newState! !

!DrGScriptPlayer methodsFor: 'xml' stamp: 'HilaireFernandes 11/5/2010 22:21'!
postProcess: item from: node
	super postProcess: item from: node.
	self parseValuePosition: node of: item! !

!DrGScriptPlayer methodsFor: 'testing' stamp: 'HilaireFernandes 8/4/2008 10:29'!
readyToBuild
	selectedMethod  ifNil: [^false].
	^ input size = selectedMethod numArgs and: [ point isNil not ]! !

!DrGScriptPlayer methodsFor: 'initialize-release' stamp: 'HilaireFernandes 8/2/2008 21:50'!
registerEvents
	wizard when: #page send: #page: to: self.
	wizard when: #cancel send: #cancel to: self.! !

!DrGScriptPlayer methodsFor: 'dependents access' stamp: 'HilaireFernandes 8/2/2008 21:51'!
release
	self removeEvents.
	wizard delete.
	wizard := nil! !

!DrGScriptPlayer methodsFor: 'initialize-release' stamp: 'HilaireFernandes 8/2/2008 21:51'!
removeEvents
	wizard removeActionsWithReceiver: self! !

!DrGScriptPlayer methodsFor: 'testing' stamp: 'HilaireFernandes 8/4/2008 09:44'!
reset
	super reset.
	input := OrderedCollection new.! !

!DrGScriptPlayer methodsFor: 'accessing' stamp: 'HilaireFernandes 8/4/2008 10:00'!
selectedMethod
	^selectedMethod ! !

!DrGScriptPlayer methodsFor: 'accessing' stamp: 'HilaireFernandes 8/4/2008 10:03'!
selectedMethod: aMethod
	selectedMethod := aMethod .
	input := OrderedCollection new.
	self triggerEvent: #changedScript.! !

DrGMathItemBuilder subclass: #DrGSegmentBuilder
	instanceVariableNames: 'pointA pointB'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'DrGeoII-Core-Builder'!

DrGSegmentBuilder subclass: #DrGCircleBuilder
	instanceVariableNames: 'radius segment'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'DrGeoII-Core-Builder'!

!DrGCircleBuilder class methodsFor: 'documentation' stamp: 'HilaireFernandes 3/13/2011 18:34'!
description 
	^ 'Circle defined by its center and a point or by its center and radius, a value.' translated! !

!DrGCircleBuilder class methodsFor: 'documentation' stamp: 'HilaireFernandes 3/13/2011 18:33'!
title
	^ 'Circle' translated ! !

!DrGCircleBuilder methodsFor: 'private' stamp: 'HilaireFernandes 10/28/2011 16:11'!
addItem: aMathItemCollection at: aPoint 
	|item|
	point := aPoint.
	(item := aMathItemCollection first) isSegmentItem ifTrue: [
		^ segment ifNil: [segment := item]].
	item isValueItem ifTrue: [
		^ radius ifNil: [radius := item]].
	^ pointA 
		ifNil: [pointA := item]
		ifNotNil: [pointB := item]! !

!DrGCircleBuilder methodsFor: 'accessing' stamp: 'HilaireFernandes 11/1/2011 19:03'!
arguments
	pointB
		ifNil: [
			radius ifNotNil: [ ^{pointA. radius}].
			segment ifNotNil: [ ^ {pointA. segment}]]
		ifNotNil: [^{pointA. pointB}]! !

!DrGCircleBuilder methodsFor: 'testing' stamp: 'HilaireFernandes 10/28/2011 16:26'!
isWanted: aMathItemCollection 
	^ aMathItemCollection notEmpty
		and: [(aMathItemCollection first isPointItem
				and: [aMathItemCollection first ~= pointA
					and: [aMathItemCollection first ~= pointB]])
			or: [(aMathItemCollection first isValueItem
					and: [pointA isNil not
						and: [aMathItemCollection first ~= radius]])
				or: [aMathItemCollection first isSegmentItem 
					and: [pointA isNil not
						and: [aMathItemCollection first ~= segment]]]]]! !

!DrGCircleBuilder methodsFor: 'constant' stamp: 'HilaireFernandes 10/28/2011 16:18'!
mathItemClass
	^ radius
		ifNil: [
			segment 
				ifNil: [DrGCircle2ptsItem]
				ifNotNil: [DrGCircleSegmentItem]]
		ifNotNil: [DrGCircleRadiusItem]! !

!DrGCircleBuilder methodsFor: 'testing' stamp: 'HilaireFernandes 10/28/2011 16:26'!
readyToBuild
	^ (radius notNil and: [pointA notNil])
		or: [(pointA notNil and: [pointB notNil])
			or: [pointA notNil and: [segment notNil]]]! !

!DrGCircleBuilder methodsFor: 'updating' stamp: 'HilaireFernandes 10/28/2011 16:06'!
reset
	super reset.
	radius := nil.
	segment := nil.! !

DrGSegmentBuilder subclass: #DrGLineBuilder
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'DrGeoII-Core-Builder'!

!DrGLineBuilder class methodsFor: 'documentation' stamp: 'HilaireFernandes 3/13/2011 18:42'!
description 
	^ 'Line defined by two points' translated! !

!DrGLineBuilder class methodsFor: 'documentation' stamp: 'HilaireFernandes 3/13/2011 18:41'!
title
	^ 'Line' translated! !

!DrGLineBuilder methodsFor: 'constant' stamp: 'HilaireFernandes 10/16/2010 16:03'!
mathItemClass
	^DrGLine2ptsItem! !

DrGSegmentBuilder subclass: #DrGRayBuilder
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'DrGeoII-Core-Builder'!

!DrGRayBuilder class methodsFor: 'documentation' stamp: 'HilaireFernandes 3/13/2011 20:55'!
description 
	^ 'Half-Line defined by two points, the first selected point is the origin.' translated! !

!DrGRayBuilder class methodsFor: 'documentation' stamp: 'HilaireFernandes 3/13/2011 20:55'!
title
	^'Ray' translated ! !

!DrGRayBuilder methodsFor: 'constant' stamp: 'HilaireFernandes 10/16/2010 16:18'!
mathItemClass
	^DrGRay2ptsItem! !

!DrGSegmentBuilder class methodsFor: 'documentation' stamp: 'HilaireFernandes 3/13/2011 20:59'!
description
	^ 'Segment defined by two points.' translated! !

!DrGSegmentBuilder class methodsFor: 'documentation' stamp: 'HilaireFernandes 3/13/2011 20:59'!
title
	^ 'Segment' translated! !

!DrGSegmentBuilder methodsFor: 'private' stamp: 'HilaireFernandes 1/6/2006 10:04'!
addItem: aMathItemCollection at: aPoint
	super addItem: aMathItemCollection  at: aPoint.
	pointA ifNil: 
		[pointA := aMathItemCollection first.
		^ self].
	pointB ifNil: [pointB := aMathItemCollection first]
	! !

!DrGSegmentBuilder methodsFor: 'accessing' stamp: 'HilaireFernandes 12/21/2005 16:13'!
arguments
	^{pointA . pointB}! !

!DrGSegmentBuilder methodsFor: 'testing' stamp: 'HilaireFernandes 7/24/2010 19:28'!
isWanted: aMathItemCollection
	^aMathItemCollection notEmpty
		and: [aMathItemCollection first isPointItem
			and: [aMathItemCollection first ~= pointA 
				and: [aMathItemCollection first ~= pointB]]]! !

!DrGSegmentBuilder methodsFor: 'constant' stamp: 'HilaireFernandes 10/16/2010 16:21'!
mathItemClass
	^DrGSegment2ptsItem! !

!DrGSegmentBuilder methodsFor: 'testing' stamp: 'HilaireFernandes 12/21/2005 16:21'!
readyToBuild
	^ pointA notNil and: [pointB notNil]! !

!DrGSegmentBuilder methodsFor: 'updating' stamp: 'HilaireFernandes 1/8/2006 19:46'!
reset
	super reset.
	pointA := pointB := nil! !

DrGSegmentBuilder subclass: #DrGVectorBuilder
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'DrGeoII-Core-Builder'!

!DrGVectorBuilder class methodsFor: 'documentation' stamp: 'HilaireFernandes 3/13/2011 21:03'!
description 
	^'Vector defined by two points.' translated! !

!DrGVectorBuilder class methodsFor: 'documentation' stamp: 'HilaireFernandes 3/13/2011 21:02'!
title
	^'Vector' translated ! !

!DrGVectorBuilder methodsFor: 'constant' stamp: 'HilaireFernandes 10/16/2010 16:42'!
mathItemClass
	^DrGVector2ptsItem! !

DrGMathItemBuilder subclass: #DrGTextBuilder
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'DrGeoII-Core-Builder'!

!DrGTextBuilder class methodsFor: 'documentation' stamp: 'HilaireFernandes 11/5/2011 18:13'!
description 
	^ 'Free text you can edit and move around.' translated! !

!DrGTextBuilder class methodsFor: 'documentation' stamp: 'HilaireFernandes 11/5/2011 17:35'!
title
	^ 'Text' translated! !

!DrGTextBuilder methodsFor: 'accessing' stamp: 'HilaireFernandes 11/5/2011 15:54'!
arguments
	^ {point}! !

!DrGTextBuilder methodsFor: 'testing' stamp: 'HilaireFernandes 11/5/2011 18:04'!
isWanted: aMathItemCollection
	^ aMathItemCollection isEmpty! !

!DrGTextBuilder methodsFor: 'xml' stamp: 'HilaireFernandes 11/5/2011 15:54'!
itemInstanceFrom: node
	self addItem: #() at: 0@0.
	^ self getItem first.
! !

!DrGTextBuilder methodsFor: 'constant' stamp: 'HilaireFernandes 11/5/2011 15:41'!
mathItemClass 
	^ DrGTextItem! !

!DrGTextBuilder methodsFor: 'xml' stamp: 'HilaireFernandes 11/5/2011 15:52'!
postProcess:  item from: node
	super postProcess: item from: node.
	self parseValuePosition: node of: item.
	item text: (node elementAt: #contents) contentString! !

!DrGTextBuilder methodsFor: 'testing' stamp: 'HilaireFernandes 11/5/2011 15:53'!
readyToBuild 
	^ point notNil! !

DrGMathItemBuilder subclass: #DrGTransformationBuilder
	instanceVariableNames: 'mathItem'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'DrGeoII-Core-Builder'!

!DrGTransformationBuilder commentStamp: '<historical>' prior: 0!
Transformation builder can handle compositeItem, we need to specialize
its behavior.!

DrGTransformationBuilder subclass: #DrGReflectionBuilder
	instanceVariableNames: 'axe'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'DrGeoII-Core-Builder'!

!DrGReflectionBuilder class methodsFor: 'documentation' stamp: 'HilaireFernandes 3/13/2011 22:39'!
description
	^ 'Reflexion: select a line and a geometric object. Reflexion axe is the first selected line.' translated! !

!DrGReflectionBuilder class methodsFor: 'documentation' stamp: 'HilaireFernandes 3/13/2011 20:56'!
title
	^ 'Reflection' translated! !

!DrGReflectionBuilder methodsFor: 'private' stamp: 'HilaireFernandes 1/6/2006 10:04'!
addItem: aMathItemCollection at: aPoint
	super addItem: aMathItemCollection  at: aPoint.
	(aMathItemCollection first isLineItem and: [axe isNil])
		ifTrue: [axe := aMathItemCollection first]
		ifFalse: [mathItem := aMathItemCollection first]! !

!DrGReflectionBuilder methodsFor: 'accessing' stamp: 'HilaireFernandes 3/5/2006 16:58'!
arguments: aMathItem
	^{aMathItem . axe}! !

!DrGReflectionBuilder methodsFor: 'testing' stamp: 'HilaireFernandes 2/7/2006 17:30'!
isWanted: aMathItemCollection 
	^ aMathItemCollection notEmpty 
		and:
			[(aMathItemCollection first isLineItem and: [axe isNil])
				or: [aMathItemCollection first isGeometricItem and: [mathItem isNil]]]! !

!DrGReflectionBuilder methodsFor: 'constant' stamp: 'HilaireFernandes 10/16/2010 15:56'!
mathItemClass: aMathItem
	|className|
	className := 'DrG', aMathItem basicType capitalized, 'ReflexionItem'.
	^(self class environment classNamed: className)! !

!DrGReflectionBuilder methodsFor: 'testing' stamp: 'HilaireFernandes 1/3/2006 11:32'!
readyToBuild
	^axe notNil and: [mathItem notNil]! !

!DrGReflectionBuilder methodsFor: 'updating' stamp: 'HilaireFernandes 1/8/2006 19:47'!
reset
	super reset.
	axe := mathItem := nil! !

DrGTransformationBuilder subclass: #DrGRotationBuilder
	instanceVariableNames: 'center value'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'DrGeoII-Core-Builder'!

DrGRotationBuilder subclass: #DrGHomothetyBuilder
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'DrGeoII-Core-Builder'!

!DrGHomothetyBuilder class methodsFor: 'documentation' stamp: 'HilaireFernandes 3/13/2011 22:40'!
description
	^ 'Homothety: select a center, a value and a geometric object. The first selected point is the homothety center.' translated! !

!DrGHomothetyBuilder class methodsFor: 'documentation' stamp: 'HilaireFernandes 3/13/2011 21:23'!
title 
	^ 'Homothety (scale)' translated! !

!DrGHomothetyBuilder methodsFor: 'constant' stamp: 'HilaireFernandes 10/16/2010 15:56'!
mathItemClass: aMathItem
	|className|
	className := 'DrG', aMathItem basicType capitalized, 'HomothetyItem'.
	^(self class environment classNamed: className)! !

!DrGRotationBuilder class methodsFor: 'documentation' stamp: 'HilaireFernandes 3/13/2011 22:40'!
description
	^ 'Rotation: select a point, a value and a geometric object. The first selected point is the rotation center.' translated! !

!DrGRotationBuilder class methodsFor: 'documentation' stamp: 'HilaireFernandes 3/13/2011 20:57'!
title 
	^ 'Rotation' translated! !

!DrGRotationBuilder methodsFor: 'private' stamp: 'HilaireFernandes 2/7/2006 17:28'!
addItem: aMathItemCollection at: aPoint
	super addItem: aMathItemCollection at: aPoint.
	(aMathItemCollection first isPointItem and: [center isNil])
		ifTrue: [center := aMathItemCollection first. ^self].
	(aMathItemCollection first isValueItem and: [value isNil])
		ifTrue: [value := aMathItemCollection first. ^self].
	mathItem := aMathItemCollection first! !

!DrGRotationBuilder methodsFor: 'accessing' stamp: 'HilaireFernandes 3/5/2006 15:22'!
arguments: aMathItem
	^{aMathItem . center . value}! !

!DrGRotationBuilder methodsFor: 'testing' stamp: 'CedrickBeler 4/23/2010 16:54'!
isWanted: aMathItemCollection
	^ aMathItemCollection notEmpty 
		and: [(aMathItemCollection first isPointItem and: [center isNil])
				or: [(aMathItemCollection first isValueItem and: [value isNil])
				or: [aMathItemCollection first isGeometricItem and: [mathItem isNil]]]]! !

!DrGRotationBuilder methodsFor: 'constant' stamp: 'HilaireFernandes 10/16/2010 15:56'!
mathItemClass: aMathItem
	|className|
	className := 'DrG', aMathItem basicType capitalized, 'RotationItem'.
	^(self class environment classNamed: className)! !

!DrGRotationBuilder methodsFor: 'testing' stamp: 'HilaireFernandes 7/24/2010 19:27'!
readyToBuild
	^center notNil and: [value notNil and: [mathItem notNil]]! !

!DrGRotationBuilder methodsFor: 'updating' stamp: 'HilaireFernandes 2/7/2006 17:46'!
reset
	super reset.
	center := value := mathItem := nil! !

DrGTransformationBuilder subclass: #DrGSymmetryBuilder
	instanceVariableNames: 'center'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'DrGeoII-Core-Builder'!

!DrGSymmetryBuilder class methodsFor: 'documentation' stamp: 'HilaireFernandes 3/13/2011 22:41'!
description
	^ 'Symmetry: select a point and a geometric object. The first selected point is the symmetry center.' translated! !

!DrGSymmetryBuilder class methodsFor: 'documentation' stamp: 'HilaireFernandes 3/13/2011 21:00'!
title
	^'Symmetry' translated ! !

!DrGSymmetryBuilder methodsFor: 'private' stamp: 'HilaireFernandes 1/6/2006 10:06'!
addItem: aMathItemCollection at: aPoint
	super addItem: aMathItemCollection at: aPoint.
	(aMathItemCollection first isPointItem and: [center isNil])
		ifTrue: [center := aMathItemCollection first]
		ifFalse: [mathItem := aMathItemCollection first]! !

!DrGSymmetryBuilder methodsFor: 'accessing' stamp: 'HilaireFernandes 3/5/2006 15:23'!
arguments: aMathItem
	^{aMathItem . center}! !

!DrGSymmetryBuilder methodsFor: 'testing' stamp: 'HilaireFernandes 1/6/2006 10:06'!
isWanted: aMathItemCollection 
	^ aMathItemCollection notEmpty
		and:
			[(aMathItemCollection first isPointItem and: [center isNil])
				or:
				[aMathItemCollection first isGeometricItem and: [mathItem isNil]]]! !

!DrGSymmetryBuilder methodsFor: 'constant' stamp: 'HilaireFernandes 10/16/2010 15:56'!
mathItemClass: aMathItem
	|className|
	className := 'DrG', aMathItem basicType capitalized, 'SymmetryItem'.
	^(self class environment classNamed: className)! !

!DrGSymmetryBuilder methodsFor: 'testing' stamp: 'HilaireFernandes 1/3/2006 16:53'!
readyToBuild
	^center notNil and: [mathItem notNil]! !

!DrGSymmetryBuilder methodsFor: 'updating' stamp: 'HilaireFernandes 1/8/2006 19:47'!
reset
	super reset.
	mathItem := center := nil! !

!DrGTransformationBuilder methodsFor: 'accessing' stamp: 'HilaireFernandes 3/5/2006 15:39'!
arguments
	^self arguments: mathItem! !

!DrGTransformationBuilder methodsFor: 'building' stamp: 'HilaireFernandes 3/5/2006 15:37'!
buildItem
	mathItem class = DrGCompositeItem 
		ifTrue:
			[^ mathItem subItems collect: [:aMathItem|
					(self mathItemClass: aMathItem) 
						newWith: (self arguments: aMathItem)]]
		ifFalse:
			[^super buildItem]! !

!DrGTransformationBuilder methodsFor: 'constant' stamp: 'HilaireFernandes 3/5/2006 15:39'!
mathItemClass
	^ self mathItemClass: mathItem! !

!DrGTransformationBuilder methodsFor: 'xml' stamp: 'HilaireFernandes 10/25/2010 11:10'!
parentsById: node
	^ (super parentsById: node) reverse! !

DrGTransformationBuilder subclass: #DrGTranslationBuilder
	instanceVariableNames: 'vector'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'DrGeoII-Core-Builder'!

!DrGTranslationBuilder commentStamp: '<historical>' prior: 0!
curveItem : segment, vector, circle, arc!

!DrGTranslationBuilder class methodsFor: 'documentation' stamp: 'HilaireFernandes 3/13/2011 22:41'!
description
	^ 'Translation: select a vector and a geometric object.' translated! !

!DrGTranslationBuilder class methodsFor: 'documentation' stamp: 'HilaireFernandes 3/13/2011 21:01'!
title
	^ 'Translation' translated! !

!DrGTranslationBuilder methodsFor: 'private' stamp: 'HilaireFernandes 1/6/2006 10:06'!
addItem: aMathItemCollection at: aPoint
	super addItem: aMathItemCollection at: aPoint.
	(aMathItemCollection first isVectorItem and: [vector isNil])
		ifTrue: [vector := aMathItemCollection first]
		ifFalse: [mathItem := aMathItemCollection first]! !

!DrGTranslationBuilder methodsFor: 'accessing' stamp: 'HilaireFernandes 3/5/2006 15:23'!
arguments: aMathItem
	^{aMathItem . vector}! !

!DrGTranslationBuilder methodsFor: 'testing' stamp: 'HilaireFernandes 7/24/2010 19:30'!
isWanted: aMathItemCollection 
	^ aMathItemCollection notEmpty 
		and:
			[(aMathItemCollection first isVectorItem and: [vector isNil])
				or:
					[aMathItemCollection first isGeometricItem 
						and: [aMathItemCollection first isVectorItem not 
							and: [mathItem isNil]]]]! !

!DrGTranslationBuilder methodsFor: 'constant' stamp: 'HilaireFernandes 10/16/2010 15:56'!
mathItemClass: aMathItem
	|className|
	className := 'DrG', aMathItem basicType capitalized, 'TranslationItem'.
	^(self class environment classNamed: className)! !

!DrGTranslationBuilder methodsFor: 'testing' stamp: 'HilaireFernandes 1/4/2006 06:40'!
readyToBuild
	^vector notNil and: [mathItem notNil]! !

!DrGTranslationBuilder methodsFor: 'updating' stamp: 'HilaireFernandes 1/8/2006 19:47'!
reset
	super reset.
	vector := mathItem := nil.! !

DrGMathItemBuilder subclass: #DrGValueBuilder
	instanceVariableNames: 'curveItem pointA pointB'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'DrGeoII-Core-Builder'!

!DrGValueBuilder class methodsFor: 'documentation' stamp: 'HilaireFernandes 3/13/2011 18:36'!
description 
	^ 'Distance between objects, curve length, free value' translated! !

!DrGValueBuilder class methodsFor: 'documentation' stamp: 'HilaireFernandes 3/13/2011 18:36'!
title
	^ 'Distance, length, value' translated! !

!DrGValueBuilder methodsFor: 'private' stamp: 'HilaireFernandes 2/2/2006 14:04'!
addItem: aMathItemCollection at: aPoint
	super addItem: aMathItemCollection at: aPoint.
	aMathItemCollection ifEmpty: [^self].
	aMathItemCollection first isPointItem 
		ifTrue: 
			[pointA ifNil:
				[pointA := aMathItemCollection first.
				^self].
			pointB ifNil:
				[pointB := aMathItemCollection first.
				^self]]
		ifFalse: [curveItem := aMathItemCollection first]! !

!DrGValueBuilder methodsFor: 'accessing' stamp: 'HilaireFernandes 8/3/2008 12:05'!
arguments
	(pointA notNil and: [curveItem notNil]) 
		ifTrue: [^{pointA . curveItem. point}].
	curveItem ifNotNil: [^{curveItem. point}].
	pointB ifNotNil: [^{pointA . pointB. point}].
	^{point}! !

!DrGValueBuilder methodsFor: 'testing' stamp: 'CedrickBeler 4/23/2010 16:46'!
isWanted: aMathItemCollection 
	aMathItemCollection
		ifEmpty: [^ true].
	aMathItemCollection first isPointItem
		ifTrue: [^ true].
	((aMathItemCollection first isLineItem
				or: [aMathItemCollection first isSegmentItem
				or: [aMathItemCollection first isVectorItem
				or: [aMathItemCollection first isCircleItem
				or: [aMathItemCollection first isArcItem]]]])
			and: [pointA isNil])
		ifTrue: [^ true].
	(pointA notNil
			and: [aMathItemCollection first isLineItem
					or: [aMathItemCollection first isCircleItem]])
		ifTrue: [^ true].
	^ false! !

!DrGValueBuilder methodsFor: 'xml' stamp: 'HilaireFernandes 10/25/2010 11:26'!
itemInstanceFrom: node
	(self parentsById: node) do: [:each |	self addItem: {each} at: 0@0].
	"give a chance to a Free value (ValueValueItem) to be instantiated, otherwise it does not hurt"
	self addItem: #() at: 0@0.
	^ self getItem first.
! !

!DrGValueBuilder methodsFor: 'constant' stamp: 'HilaireFernandes 10/16/2010 16:41'!
mathItemClass
	pointA
		ifNotNil: [curveItem
				ifNotNil: [curveItem isLineItem
						ifTrue: [^ DrGValueDistanceptlineItem]]].
	curveItem
		ifNotNil: [curveItem isLineItem
				ifTrue: [^ DrGValueSlopeItem].
			curveItem isSegmentItem
				ifTrue: [^ DrGValueSegmentlengthItem].
			curveItem isCircleItem
				ifTrue: [^ DrGValueCircleperimeterItem].
			curveItem isVectorItem
				ifTrue: [^ DrGValueVectornormItem].
			curveItem isArcItem
				ifTrue: [^ DrGValueArclengthItem]].
	pointB
		ifNotNil: [^ DrGValueDistance2ptsItem].
	^ DrGValueValueItem! !

!DrGValueBuilder methodsFor: 'xml' stamp: 'HilaireFernandes 10/25/2010 22:44'!
postProcess: item from: node
	super postProcess: item from: node.
	self parseValuePosition: node of: item.
	(node attributeAt: #type) asSymbol == #value 
		ifTrue: [item valueItem: (node elementAt: #value) contentString asNumber]! !

!DrGValueBuilder methodsFor: 'testing' stamp: 'HilaireFernandes 7/24/2010 19:32'!
readyToBuild
	^(point notNil 
		and: [curveItem isNil
			and: [pointA isNil
				and: [pointB isNil]]])
		or: [pointB notNil
			or: [curveItem notNil]]! !

!DrGValueBuilder methodsFor: 'updating' stamp: 'HilaireFernandes 2/2/2006 14:00'!
reset
	super reset.
	pointA := pointB := curveItem := nil! !

Object subclass: #DrGMathItemCostume
	instanceVariableNames: 'mathItem morph textMorph textPositionDelta drawable exist style'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'DrGeoII-Core-Item-View'!

!DrGMathItemCostume commentStamp: 'HilaireFernandes 7/20/2011 12:46' prior: 0!
This is the root of the costumue hierarchy, a simple Adapter to morph.
A costume is a visual representation of a mathItem. The user interact with the mathItem through its costume.
 mathItem 		- a costume need to know its model
 dirty 			- boolean, true if it need udpate (repositionned) (deprecated)
 morph 			- this the adaptee, the graphic representation
 textMorph     - to display the object name, may be nil
 drawable 		- a costume need to know where it is printed
 exist 		   - does the costume exist (ie when its mathItem does or not exist, we need it to keep track when the mathItem exist changed)
!

DrGMathItemCostume subclass: #DrGCompositeCostume
	instanceVariableNames: 'subCostumes'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'DrGeoII-Core-Item-View'!

!DrGCompositeCostume commentStamp: '<historical>' prior: 0!
I am a costume composed of several costumes!

!DrGCompositeCostume class methodsFor: 'as yet unclassified' stamp: 'HilaireFernandes 3/2/2006 23:46'!
for: aMathItem in: aDrawable with: theSubCostumes
	^ self basicNew 
		drawable: aDrawable;
		mathItem: aMathItem;
		subCostumes: theSubCostumes;
		initialize! !

!DrGCompositeCostume methodsFor: 'updating' stamp: 'HilaireFernandes 4/19/2010 13:00'!
addCostume: aCostume
	mathItem subItems: (mathItem subItems add: aCostume mathItem; yourself).
	drawable app remove: aCostume.
	subCostumes add: aCostume.

	! !

!DrGCompositeCostume methodsFor: 'testing' stamp: 'HilaireFernandes 3/2/2006 22:48'!
basicType
	^#composite! !

!DrGCompositeCostume methodsFor: 'updating' stamp: 'HilaireFernandes 3/3/2006 11:16'!
changed
	subCostumes do: [:costume |
		costume changed]! !

!DrGCompositeCostume methodsFor: 'updating' stamp: 'HilaireFernandes 3/3/2006 12:03'!
color: aColor
	subCostumes do: [:aCostume |	
		aCostume style color: aColor]! !

!DrGCompositeCostume methodsFor: 'testing' stamp: 'HilaireFernandes 3/2/2006 22:58'!
containsPoint: aPoint
	subCostumes do: [:costume|
		(costume containsPoint: aPoint) ifTrue:
			[^true]].
	^false! !

!DrGCompositeCostume methodsFor: 'updating' stamp: 'HilaireFernandes 3/3/2006 14:57'!
dash: anObject
	subCostumes do: [:aCostume|
		aCostume isCurveCostume ifTrue:
			[aCostume dash: anObject]]! !

!DrGCompositeCostume methodsFor: 'as yet unclassified' stamp: 'HilaireFernandes 4/19/2010 13:01'!
disband
"Disband this composite item"
	drawable app ungroupCostumes: mathItem! !

!DrGCompositeCostume methodsFor: 'updating' stamp: 'HilaireFernandes 3/2/2006 22:43'!
forceVisible: aBoolean
	subCostumes do: [:costume |
		costume forceVisible: aBoolean]! !

!DrGCompositeCostume methodsFor: 'initialize-release' stamp: 'HilaireFernandes 1/23/2008 12:15'!
initialize
	exist := true.
	self style: (self styleClass newWith: self).
! !

!DrGCompositeCostume methodsFor: 'updating' stamp: 'HilaireFernandes 3/3/2006 15:09'!
makeBackArrow
	subCostumes do: [:aCostume|
		aCostume isFinitCurveCostume ifTrue:
			[aCostume makeBackArrow]]! !

!DrGCompositeCostume methodsFor: 'updating' stamp: 'HilaireFernandes 3/3/2006 15:08'!
makeBothArrows
	subCostumes do: [:aCostume|
		aCostume isFinitCurveCostume ifTrue:
			[aCostume makeBothArrows]]! !

!DrGCompositeCostume methodsFor: 'updating' stamp: 'HilaireFernandes 3/3/2006 15:07'!
makeForwardArrow
	subCostumes do: [:aCostume|
		aCostume isFinitCurveCostume ifTrue:
			[aCostume makeForwardArrow]]! !

!DrGCompositeCostume methodsFor: 'updating' stamp: 'HilaireFernandes 3/3/2006 15:10'!
makeNoArrows
	subCostumes do: [:aCostume|
		aCostume isFinitCurveCostume ifTrue:
			[aCostume makeNoArrows]]! !

!DrGCompositeCostume methodsFor: 'updating' stamp: 'HilaireFernandes 5/8/2011 09:15'!
redraw
	super redraw ifFalse: [^ self].
	subCostumes
		do: [:costume | costume redraw]! !

!DrGCompositeCostume methodsFor: 'updating' stamp: 'HilaireFernandes 4/19/2010 13:01'!
removeCostume: aCostume
	mathItem subItems: (mathItem subItems copyWithout: aCostume mathItem).
	drawable app add: aCostume.
	subCostumes remove: aCostume.! !

!DrGCompositeCostume methodsFor: 'updating' stamp: 'HilaireFernandes 3/3/2006 14:51'!
shape: anObject
	subCostumes do: [:aCostume|
		aCostume isPointCostume ifTrue:
			[aCostume style shape: anObject]]! !

!DrGCompositeCostume methodsFor: 'updating' stamp: 'HilaireFernandes 3/2/2006 18:06'!
startBlinking
	subCostumes do: [:costume |
		costume startBlinking ]! !

!DrGCompositeCostume methodsFor: 'updating' stamp: 'HilaireFernandes 3/2/2006 18:06'!
stopBlinking
	subCostumes do: [:costume |
		costume stopBlinking ]! !

!DrGCompositeCostume methodsFor: 'style' stamp: 'HilaireFernandes 3/2/2006 23:42'!
styleClass
	^DrGCompositeStyle! !

!DrGCompositeCostume methodsFor: 'accessing' stamp: 'HilaireFernandes 3/2/2006 19:02'!
subCostumes
	^subCostumes ! !

!DrGCompositeCostume methodsFor: 'accessing' stamp: 'HilaireFernandes 3/2/2006 19:03'!
subCostumes: theSubCostumes
	subCostumes := theSubCostumes ! !

!DrGCompositeCostume methodsFor: 'updating' stamp: 'HilaireFernandes 3/2/2006 22:42'!
updateToInvisible
	subCostumes do: [:costume |
		costume updateToInvisible]! !

!DrGCompositeCostume methodsFor: 'updating' stamp: 'HilaireFernandes 3/2/2006 22:42'!
updateToVisible
	subCostumes do: [:costume |
		costume updateToVisible]! !

!DrGCompositeCostume methodsFor: 'updating' stamp: 'HilaireFernandes 3/25/2006 15:52'!
updatedMathItem
	subCostumes do: [:costume |
		costume update]! !

!DrGCompositeCostume methodsFor: 'updating' stamp: 'HilaireFernandes 3/21/2006 16:27'!
width: anInteger
	subCostumes do: [:aCostume|
		aCostume isCurveCostume ifTrue:
			[aCostume width: anInteger]]! !

DrGMathItemCostume subclass: #DrGCurveCostume
	instanceVariableNames: 'anchorAbscissa'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'DrGeoII-Core-Item-View'!

DrGCurveCostume subclass: #DrGArcCostume
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'DrGeoII-Core-Item-View'!

!DrGArcCostume methodsFor: 'testing' stamp: 'HilaireFernandes 1/17/2006 08:58'!
basicType
	^#arc! !

!DrGArcCostume methodsFor: 'initialize-release' stamp: 'HilaireFernandes 1/24/2006 23:02'!
instantiateMorph
	morph := DrGArcMorph new! !

!DrGArcCostume methodsFor: 'updating' stamp: 'HilaireFernandes 5/8/2011 09:15'!
redraw
	super redraw ifFalse: [^ self].
	mathItem center
		ifNil: [morph class = DrGSegmentMorph
				ifTrue: [self redrawAsSegmentOrRay]
			"mutate the morph to DrGSegment and draw it"]
		ifNotNil: [morph class = DrGArcMorph
				ifTrue: [self redrawAsArc]
			"mutate the morph to an DrGArcMorph and draw it"]! !

!DrGArcCostume methodsFor: 'updating' stamp: 'HilaireFernandes 3/21/2006 11:06'!
redrawAsArc
	morph 
		center: (drawable worldToMorph: mathItem center)
		radius: (drawable worldToPixel: mathItem radius)
		origin: mathItem origin negated
		length: mathItem length negated
! !

!DrGArcCostume methodsFor: 'updating' stamp: 'HilaireFernandes 3/21/2006 11:22'!
redrawAsSegmentOrRay! !

!DrGArcCostume methodsFor: 'style' stamp: 'HilaireFernandes 11/16/2011 11:00'!
styleClass
	^DrGArcCostumeStyle! !

DrGCurveCostume subclass: #DrGCircleCostume
	instanceVariableNames: 'lineMorph'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'DrGeoII-Core-Item-View'!

!DrGCircleCostume commentStamp: 'HilaireFernandes 6/21/2007 17:30' prior: 0!
when Circle goes infinit replace it by a lineMorph!

!DrGCircleCostume methodsFor: 'testing' stamp: 'HilaireFernandes 12/31/2005 10:04'!
basicType
	^#circle! !

!DrGCircleCostume methodsFor: 'initialize-release' stamp: 'HilaireFernandes 1/24/2006 23:02'!
instantiateMorph
	morph := DrGCircleMorph new! !

!DrGCircleCostume methodsFor: 'updating' stamp: 'HilaireFernandes 5/8/2011 09:16'!
redraw
	super redraw ifFalse: [^ self].
	mathItem radius > 100
		ifTrue: [self redrawAsLine]
		ifFalse: [self redrawAsCircle]! !

!DrGCircleCostume methodsFor: 'updating' stamp: 'HilaireFernandes 6/21/2007 17:37'!
redrawAsCircle
	lineMorph ifNotNil: [
		drawable removeMorph: lineMorph.
		drawable addMorph: morph.
		lineMorph := nil].
	morph
		center: (drawable worldToMorph: mathItem center)
		radius: (drawable worldToPixel: mathItem radius)
! !

!DrGCircleCostume methodsFor: 'updating' stamp: 'HilaireFernandes 6/21/2007 18:08'!
redrawAsLine
	| p1 p2 |
	lineMorph
		ifNil: [lineMorph := DrGLineMorph new costume: self.
			lineMorph color: morph color;
				 borderWidth: morph borderWidth.
			drawable removeMorph: morph.
			drawable addMorph: lineMorph].
	p1 := mathItem
				closestPointTo: (drawable morphToWorld: ActiveHand position).
	p2 := mathItem pointAt: (mathItem abscissaOf: p1)
					+ 0.001.
	lineMorph
		from: (drawable worldToMorph: p1)
		direction: (drawable vectorToMorph: p2 - p1)! !

!DrGCircleCostume methodsFor: 'style' stamp: 'HilaireFernandes 11/16/2011 10:57'!
styleClass
	^DrGFinitFilledCostumeStyle! !

!DrGCurveCostume methodsFor: 'as yet unclassified' stamp: 'HilaireFernandes 10/31/2010 22:09'!
anchorAbscissa: number
	anchorAbscissa := number ! !

!DrGCurveCostume methodsFor: 'layout' stamp: 'HilaireFernandes 3/12/2006 22:32'!
constraintTextPosition: aPoint
|center vector|
	center := mathItem closestPointTo: (drawable morphToWorld: aPoint).
	anchorAbscissa := mathItem abscissaOf: center.
	center := drawable worldToMorph: center.
	vector := aPoint - center.
	^textPositionDelta := self computeTextPositionDelta: vector.
! !

!DrGCurveCostume methodsFor: 'initialize-release' stamp: 'HilaireFernandes 3/12/2006 22:38'!
initialize
	super initialize.
	anchorAbscissa := 0.5! !

!DrGCurveCostume methodsFor: 'testing' stamp: 'HilaireFernandes 3/21/2010 20:51'!
isCurveCostume
	^ true! !

!DrGCurveCostume methodsFor: 'layout' stamp: 'HilaireFernandes 3/12/2006 22:32'!
textPosition
	^(drawable worldToMorph: (mathItem pointAt: anchorAbscissa)) + textPositionDelta! !

!DrGCurveCostume methodsFor: 'xml writing' stamp: 'HilaireFernandes 10/31/2010 22:28'!
writeAttributesTo: aNode
	aNode attributeAt: #anchor put: anchorAbscissa asString.
	super writeAttributesTo: aNode.
! !

DrGCurveCostume subclass: #DrGLineCostume
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'DrGeoII-Core-Item-View'!

!DrGLineCostume methodsFor: 'testing' stamp: 'HilaireFernandes 12/31/2005 10:04'!
basicType
	^#line! !

!DrGLineCostume methodsFor: 'initialize-release' stamp: 'HilaireFernandes 1/24/2006 23:02'!
instantiateMorph
	morph := DrGLineMorph new! !

!DrGLineCostume methodsFor: 'updating' stamp: 'HilaireFernandes 10/28/2011 17:15'!
redraw
	super redraw ifFalse: [^ self].
	morph
		from: (drawable worldToMorph: mathItem origin)
		direction: (drawable vectorToMorph: mathItem direction).! !

!DrGLineCostume methodsFor: 'style' stamp: 'HilaireFernandes 1/24/2006 23:02'!
styleClass
	^DrGCurveCostumeStyle! !

DrGCurveCostume subclass: #DrGLocusCostume
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'DrGeoII-Core-Item-View'!

!DrGLocusCostume methodsFor: 'testing' stamp: 'jlrr 11/13/2007 12:15'!
basicType
	^ #locus! !

!DrGLocusCostume methodsFor: 'initialize-release' stamp: 'jlrr 11/13/2007 12:16'!
instantiateMorph
	morph := DrGLocusMorph new.
	morph color: Color red.
	morph borderColor: Color orange.
	morph width: 2! !

!DrGLocusCostume methodsFor: 'updating' stamp: 'HilaireFernandes 5/8/2011 09:16'!
redraw
	super redraw ifFalse: [^ self].
	morph free: mathItem freePoint constrained: mathItem constrainedPoint! !

!DrGLocusCostume methodsFor: 'style' stamp: 'jlrr 11/13/2007 12:16'!
styleClass
	^ DrGFinitCurveCostumeStyle! !

DrGCurveCostume subclass: #DrGPolygonCostume
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'DrGeoII-Core-Item-View'!

!DrGPolygonCostume methodsFor: 'testing' stamp: 'HilaireFernandes 2/6/2006 21:26'!
basicType
	^#polygon! !

!DrGPolygonCostume methodsFor: 'layout' stamp: 'HilaireFernandes 2/14/2011 22:12'!
constraintTextPosition: aPoint
|center vector answer|
	answer := mathItem closestPointInfoTo: (drawable morphToWorld: aPoint).
	center := answer first.
	anchorAbscissa := answer second.
	center := drawable worldToMorph: center.
	vector := aPoint - center.
	^textPositionDelta := self computeTextPositionDelta: vector.
! !

!DrGPolygonCostume methodsFor: 'initialize-release' stamp: 'HilaireFernandes 2/6/2006 21:36'!
instantiateMorph
	morph := DrGPolygoneMorph new.! !

!DrGPolygonCostume methodsFor: 'testing' stamp: 'HilaireFernandes 3/21/2010 20:52'!
isCurveCostume
	^ false! !

!DrGPolygonCostume methodsFor: 'updating' stamp: 'HilaireFernandes 5/8/2011 09:16'!
redraw
	| vertices |
	super redraw 
		ifFalse: [^ self].
	vertices := OrderedCollection new.
	mathItem points
		do: [:point | vertices
				add: (drawable worldToMorph: point)].
	morph setVertices: vertices! !

!DrGPolygonCostume methodsFor: 'style' stamp: 'HilaireFernandes 2/6/2006 21:27'!
styleClass
	^DrGCurveCostumeStyle! !

DrGCurveCostume subclass: #DrGRayCostume
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'DrGeoII-Core-Item-View'!

!DrGRayCostume methodsFor: 'testing' stamp: 'HilaireFernandes 12/31/2005 10:04'!
basicType
	^#ray! !

!DrGRayCostume methodsFor: 'initialize-release' stamp: 'HilaireFernandes 1/24/2006 23:03'!
instantiateMorph
	morph := DrGRayMorph new! !

!DrGRayCostume methodsFor: 'updating' stamp: 'HilaireFernandes 5/8/2011 09:17'!
redraw
	super redraw 
		ifFalse: [^ self].
	morph
		from: (drawable worldToMorph: mathItem origin)
		direction: (drawable vectorToMorph: mathItem direction)! !

!DrGRayCostume methodsFor: 'style' stamp: 'HilaireFernandes 1/24/2006 23:02'!
styleClass
	^DrGCurveCostumeStyle! !

DrGCurveCostume subclass: #DrGSegmentCostume
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'DrGeoII-Core-Item-View'!

!DrGSegmentCostume methodsFor: 'testing' stamp: 'HilaireFernandes 12/31/2005 10:04'!
basicType
	^#segment! !

!DrGSegmentCostume methodsFor: 'initialize-release' stamp: 'HilaireFernandes 11/16/2011 19:57'!
instantiateMorph
	morph := DrGSegmentMarkMorph new! !

!DrGSegmentCostume methodsFor: 'updating' stamp: 'HilaireFernandes 5/8/2011 09:17'!
redraw
	super redraw 
		ifFalse: [^ self].
	morph
		from: (drawable worldToMorph: mathItem origin)
		to: (drawable worldToMorph: mathItem origin + mathItem direction)! !

!DrGSegmentCostume methodsFor: 'style' stamp: 'HilaireFernandes 11/16/2011 18:25'!
styleClass
	^DrGSegmentCostumeStyle! !

DrGSegmentCostume subclass: #DrGVectorCostume
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'DrGeoII-Core-Item-View'!

!DrGVectorCostume methodsFor: 'testing' stamp: 'HilaireFernandes 1/2/2006 09:12'!
basicType
	^#vector! !

!DrGVectorCostume methodsFor: 'initialize-release' stamp: 'HilaireFernandes 1/24/2006 23:03'!
instantiateMorph
	morph := DrGVectorMorph new! !

!DrGVectorCostume methodsFor: 'style' stamp: 'HilaireFernandes 1/24/2006 23:02'!
styleClass
	^DrGCurveCostumeStyle! !

DrGMathItemCostume subclass: #DrGFlyingCostume
	instanceVariableNames: 'costume freePointA freePointB movingPoint radius'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'DrGeoII-Core-Item-View'!

!DrGFlyingCostume commentStamp: '<historical>' prior: 0!
A flying costume is an intermediate costume used to display the on construction mathItem (line, ray, segment, circle, vector, arc).

- costume is the underneath costume
- mathItem is the resulting mathItem
- freePointA, B, C are used for the intermediate construction of mathItem

In fact costume rely on mathItem instance, in turn mathItem need the freePointItem instance to be fully workable!

!DrGFlyingCostume class methodsFor: 'as yet unclassified' stamp: 'HilaireFernandes 1/9/2006 17:58'!
with: aBuilder in: aDrawable
	^self basicNew 
		mathItem: aBuilder mathItemClass new;
		drawable: aDrawable;
		initialize.! !

!DrGFlyingCostume methodsFor: 'accessing' stamp: 'HilaireFernandes 10/16/2010 16:10'!
add2parents: aCostume 
	freePointA
		ifNil: ["Update mathItem"
			freePointA := aCostume mathItem.
			freePointA isValueItem
				ifTrue: [
						radius
		ifNil: [radius := aCostume mathItem.
			^ self]]
					
				ifFalse: [movingPoint := DrGPointFreeItem new point: freePointA point].
			mathItem := mathItem class newWith: {freePointA. movingPoint}.
			costume mathItem: mathItem.
			self drawable addMorph: costume morph.
			self redraw]! !

!DrGFlyingCostume methodsFor: 'accessing' stamp: 'HilaireFernandes 10/16/2010 16:10'!
add3parents: aCostume 
	freePointA ifNil: 
			[freePointA := aCostume mathItem.
			^self].
	freePointB 
		ifNil: 
			[freePointB := aCostume mathItem.
			movingPoint := DrGPointFreeItem new point: freePointB point + (0.1 @ 0.1).
			mathItem := mathItem class newWith: {
								freePointA.
								freePointB.
								movingPoint
							}.
			costume mathItem: mathItem.
			self drawable addMorph: costume morph.
			self redraw]
		ifNotNil:
			[movingPoint := DrGPointFreeItem new 
									point: aCostume mathItem point + (0.1 @ 0.1).
			mathItem parents: (mathItem parents allButLast asOrderedCollection 
										add: aCostume mathItem;
										add: movingPoint;
										yourself).
			self redraw]! !

!DrGFlyingCostume methodsFor: 'accessing' stamp: 'HilaireFernandes 1/17/2006 16:40'!
add: aCostume
	mathItem parentQty = 2 ifTrue: [self add2parents: aCostume].
	mathItem parentQty = 3 ifTrue: [self add3parents: aCostume].! !

!DrGFlyingCostume methodsFor: 'testing' stamp: 'HilaireFernandes 1/9/2006 14:26'!
containsPoint: aPoint
	^costume containsPoint: aPoint! !

!DrGFlyingCostume methodsFor: 'initialize-release' stamp: 'HilaireFernandes 11/16/2011 12:00'!
initialize
	|costumeClass|
	costumeClass := 'DrG', mathItem basicType capitalized, 'Costume'.
	"We don't want initialize to take place in the costume"
	costume := (self class environment classNamed: costumeClass) basicNew.
	costume	drawable: self drawable; exist: true;
		instantiateMorph.
	costume morph costume: costume.
	style := costume styleClass basicNew.
	costume style: style.
	style costume: costume;
		color: Color lightRed ;
		isHidden: false.
! !

!DrGFlyingCostume methodsFor: 'testing' stamp: 'HilaireFernandes 1/9/2006 14:27'!
isArcCostume
	^costume isArcCostume! !

!DrGFlyingCostume methodsFor: 'testing' stamp: 'HilaireFernandes 1/9/2006 14:27'!
isCircleCostume
	^costume isCircleCostume! !

!DrGFlyingCostume methodsFor: 'testing' stamp: 'HilaireFernandes 1/9/2006 14:27'!
isCurveCostume
	^costume isCurveCostume! !

!DrGFlyingCostume methodsFor: 'testing' stamp: 'HilaireFernandes 1/9/2006 14:27'!
isLineCostume
	^costume isLineCostume! !

!DrGFlyingCostume methodsFor: 'testing' stamp: 'HilaireFernandes 1/9/2006 14:28'!
isLocusCostume
	^costume isLocusCostume! !

!DrGFlyingCostume methodsFor: 'testing' stamp: 'HilaireFernandes 1/9/2006 14:28'!
isPointCostume
	^costume isPointCostume! !

!DrGFlyingCostume methodsFor: 'testing' stamp: 'HilaireFernandes 1/9/2006 14:28'!
isRayCostume
	^costume isRayCostume! !

!DrGFlyingCostume methodsFor: 'testing' stamp: 'HilaireFernandes 1/9/2006 14:28'!
isSegmentCostume
	^costume isSegmentCostume! !

!DrGFlyingCostume methodsFor: 'testing' stamp: 'HilaireFernandes 1/9/2006 14:29'!
isVectorCostume
	^costume isVectorCostume! !

!DrGFlyingCostume methodsFor: 'accessing' stamp: 'HilaireFernandes 1/9/2006 15:39'!
mathItem: aMathItem
	mathItem := aMathItem.! !

!DrGFlyingCostume methodsFor: 'accessing' stamp: 'HilaireFernandes 1/9/2006 14:56'!
morph
	^costume morph! !

!DrGFlyingCostume methodsFor: 'accessing' stamp: 'HilaireFernandes 1/9/2006 14:56'!
morph: aMorph
	costume morph: morph! !

!DrGFlyingCostume methodsFor: 'updating' stamp: 'HilaireFernandes 7/19/2011 19:16'!
moveAt: aPoint
	movingPoint ifNotNil: [
		movingPoint moveAt: aPoint.
		mathItem update.
		self update]! !

!DrGFlyingCostume methodsFor: 'updating' stamp: 'HilaireFernandes 7/19/2011 19:18'!
redraw
	costume redraw.
	self morph changed! !

!DrGFlyingCostume methodsFor: 'initialize-release' stamp: 'HilaireFernandes 8/18/2008 12:50'!
release
	costume release.
	super release! !

!DrGFlyingCostume methodsFor: 'updating' stamp: 'HilaireFernandes 7/19/2011 19:17'!
update
	mathItem exist
		ifTrue: [
			costume exist ifFalse: [costume updateToVisible].
			self redraw]
		ifFalse: [costume exist ifTrue: [costume updateToInvisible]]! !

!DrGMathItemCostume class methodsFor: 'as yet unclassified' stamp: 'HilaireFernandes 10/25/2010 23:31'!
basicNew: mathItem
	^ self basicNew ! !

!DrGMathItemCostume class methodsFor: 'instance creation' stamp: 'HilaireFernandes 10/25/2010 23:20'!
costumeClassFor: mathItem
	|className |
	className := 'DrG', mathItem basicType capitalized , 'Costume'.
	^ self environment classNamed: className! !

!DrGMathItemCostume class methodsFor: 'instance creation' stamp: 'HilaireFernandes 10/25/2010 23:33'!
for: aMathItem in: aDrawable
	^ ((self costumeClassFor: aMathItem) 
		basicNew: aMathItem)
		drawable: aDrawable;
		mathItem: aMathItem;
		initialize.! !

!DrGMathItemCostume methodsFor: 'accessing' stamp: 'HilaireFernandes 8/5/2011 15:21'!
app
	^ self drawable app! !

!DrGMathItemCostume methodsFor: 'testing' stamp: 'HilaireFernandes 12/30/2005 11:37'!
basicType
	super basicType ! !

!DrGMathItemCostume methodsFor: 'updating' stamp: 'HilaireFernandes 3/3/2006 11:14'!
changed
	morph changed! !

!DrGMathItemCostume methodsFor: 'accessing' stamp: 'HilaireFernandes 3/3/2006 12:00'!
color: aColor
	morph color: aColor ! !

!DrGMathItemCostume methodsFor: 'layout' stamp: 'HilaireFernandes 10/31/2010 15:38'!
computeTextPositionDelta: aVector
|distance|
	textMorph ifNil: [^0@0].
	aVector = (0@0) 
		ifFalse:
			[distance := aVector r.
			distance > 40 ifTrue: [distance := 40].
			^(aVector / aVector r) * distance +
				(textMorph  fullBounds origin - textMorph fullBounds center)]
		ifTrue:
			[^textMorph  fullBounds origin - textMorph fullBounds center].
	! !

!DrGMathItemCostume methodsFor: 'layout' stamp: 'HilaireFernandes 2/13/2006 12:20'!
constraintTextPosition: aPoint
"Force the text arround the mathItemCostume to a position"
	self subclassResponsibility ! !

!DrGMathItemCostume methodsFor: 'testing' stamp: 'HilaireFernandes 3/21/2006 11:20'!
containsPoint: aPoint
	^exist and: [morph containsPoint: aPoint]! !

!DrGMathItemCostume methodsFor: 'updating' stamp: 'HilaireFernandes 3/3/2006 14:45'!
dash: aDashMotif
	morph dashedBorder: aDashMotif! !

!DrGMathItemCostume methodsFor: 'accessing' stamp: 'HilaireFernandes 5/13/2010 10:47'!
domain
	^drawable drgeoPresenter domain! !

!DrGMathItemCostume methodsFor: 'accessing' stamp: 'HilaireFernandes 12/15/2005 18:28'!
drawable
	^drawable! !

!DrGMathItemCostume methodsFor: 'accessing' stamp: 'HilaireFernandes 12/15/2005 18:28'!
drawable: aDrawable
	drawable := aDrawable ! !

!DrGMathItemCostume methodsFor: 'updating' stamp: 'HilaireFernandes 10/27/2011 10:27'!
editMyProperty 
	self subclassResponsibility ! !

!DrGMathItemCostume methodsFor: 'accessing' stamp: 'HilaireFernandes 1/22/2006 01:26'!
exist
	^exist! !

!DrGMathItemCostume methodsFor: 'accessing' stamp: 'HilaireFernandes 1/22/2006 01:26'!
exist: aBoolean
	exist := aBoolean ! !

!DrGMathItemCostume methodsFor: 'updating' stamp: 'HilaireFernandes 1/24/2008 11:07'!
forceVisible: aBoolean 
	(exist not or: [style isNotHidden]) ifTrue: [^false].
	aBoolean 
		ifTrue: 
			[drawable addMorph: morph.
			self redraw.
			morph color: DrGCostumeStyle hiddenColor.
			textMorph ifNotNil:
				[drawable addMorph: textMorph.
				textMorph position: self textPosition]]
		ifFalse: 
			[drawable removeMorph: morph.
			morph color: style color.
			textMorph ifNotNil:
				[drawable removeMorph: textMorph]].
	^true! !

!DrGMathItemCostume methodsFor: 'initialize-release' stamp: 'HilaireFernandes 6/4/2011 16:19'!
initialize
	"costume should update itself regarding its model (mathItem)"
	self instantiateMorph.
	textPositionDelta := 7 @ 7.
	self morph costume: self.
	exist := mathItem exist.
	exist
		ifTrue: [drawable addMorphBack: self morph].
	self
		style: (self styleClass newWith: self).
	self redraw! !

!DrGMathItemCostume methodsFor: 'initialize-release' stamp: 'HilaireFernandes 12/15/2005 16:01'!
instantiateMorph
	self subclassResponsibility ! !

!DrGMathItemCostume methodsFor: 'testing' stamp: 'HilaireFernandes 2/4/2006 11:57'!
isAngleCostume
	^self basicType = #angle! !

!DrGMathItemCostume methodsFor: 'testing' stamp: 'HilaireFernandes 1/7/2006 09:52'!
isArcCostume
	^self basicType = #arc! !

!DrGMathItemCostume methodsFor: 'testing' stamp: 'HilaireFernandes 1/7/2006 09:42'!
isCircleCostume
	^self basicType = #circle! !

!DrGMathItemCostume methodsFor: 'testing' stamp: 'HilaireFernandes 3/2/2006 22:47'!
isCompositeCostume
	^self basicType = #composite! !

!DrGMathItemCostume methodsFor: 'testing' stamp: 'HilaireFernandes 3/21/2010 20:50'!
isCurveCostume
	^false! !

!DrGMathItemCostume methodsFor: 'testing' stamp: 'HilaireFernandes 7/24/2010 19:34'!
isFinitCurveCostume
	^self isArcCostume 
		or: [self isCircleCostume
			or: [self isSegmentCostume]]
! !

!DrGMathItemCostume methodsFor: 'testing' stamp: 'HilaireFernandes 7/20/2011 12:54'!
isForceVisible
	^ drawable drgeoPresenter isForceVisible ! !

!DrGMathItemCostume methodsFor: 'testing' stamp: 'HilaireFernandes 3/21/2010 20:45'!
isLineCostume
	^self basicType = #line! !

!DrGMathItemCostume methodsFor: 'testing' stamp: 'HilaireFernandes 11/6/2011 21:34'!
isLockable
	^ mathItem respondsTo: #isLocked! !

!DrGMathItemCostume methodsFor: 'testing' stamp: 'HilaireFernandes 11/6/2011 21:38'!
isLocked
	^ mathItem isLocked! !

!DrGMathItemCostume methodsFor: 'testing' stamp: 'HilaireFernandes 1/7/2006 09:52'!
isLocusCostume
	^self basicType = #locus! !

!DrGMathItemCostume methodsFor: 'testing' stamp: 'HilaireFernandes 1/7/2006 09:43'!
isPointCostume
	^self basicType = #point! !

!DrGMathItemCostume methodsFor: 'testing' stamp: 'HilaireFernandes 1/7/2006 09:43'!
isRayCostume
	^self basicType = #ray! !

!DrGMathItemCostume methodsFor: 'testing' stamp: 'HilaireFernandes 3/21/2010 20:45'!
isSegmentCostume
	^self basicType = #segment! !

!DrGMathItemCostume methodsFor: 'testing' stamp: 'HilaireFernandes 2/4/2006 11:57'!
isValueCostume
	^self basicType = #value or: [self basicType = #angle]! !

!DrGMathItemCostume methodsFor: 'testing' stamp: 'HilaireFernandes 1/7/2006 09:43'!
isVectorCostume
	^self basicType = #vector! !

!DrGMathItemCostume methodsFor: 'updating' stamp: 'HilaireFernandes 3/3/2006 15:04'!
makeBackArrow
	morph makeBackArrow! !

!DrGMathItemCostume methodsFor: 'updating' stamp: 'HilaireFernandes 3/3/2006 15:04'!
makeBothArrows
	morph makeBothArrows! !

!DrGMathItemCostume methodsFor: 'updating' stamp: 'HilaireFernandes 3/3/2006 15:04'!
makeForwardArrow
	morph makeForwardArrow! !

!DrGMathItemCostume methodsFor: 'updating' stamp: 'HilaireFernandes 3/3/2006 15:09'!
makeNoArrows
	morph makeNoArrows! !

!DrGMathItemCostume methodsFor: 'accessing' stamp: 'HilaireFernandes 12/13/2005 17:09'!
mathItem
	^mathItem ! !

!DrGMathItemCostume methodsFor: 'accessing' stamp: 'HilaireFernandes 12/14/2005 16:19'!
mathItem: aMathItem
	self removeEvents.
	mathItem := aMathItem.
	self registerEvents.! !

!DrGMathItemCostume methodsFor: 'accessing' stamp: 'HilaireFernandes 12/15/2005 14:40'!
morph
	^morph ! !

!DrGMathItemCostume methodsFor: 'accessing' stamp: 'HilaireFernandes 12/15/2005 14:40'!
morph: aMorph
	morph := aMorph ! !

!DrGMathItemCostume methodsFor: 'accessing' stamp: 'HilaireFernandes 1/19/2008 21:53'!
name
	^textMorph ifNotNil: [textMorph contents asString]
		ifNil: ['']! !

!DrGMathItemCostume methodsFor: 'scripting' stamp: 'HilaireFernandes 4/2/2006 11:13'!
openViewer
	morph ifNotNil: [morph beViewed]! !

!DrGMathItemCostume methodsFor: 'initialize-release' stamp: 'HilaireFernandes 6/4/2011 16:18'!
reInitialize
	"we recover from an undo operation, so let's reInitialize some of my
	aspect "
	self registerEvents.
	exist
		ifTrue: [drawable addMorphBack: self morph].
	self redraw.
	self textMorph: mathItem name.
	self forceVisible: false! !

!DrGMathItemCostume methodsFor: 'updating' stamp: 'HilaireFernandes 7/20/2011 13:35'!
redraw
	(exist and: [style isNotHidden or: [self isForceVisible]]) ifFalse: [^ false].
	textMorph
		ifNotNil: [textMorph position: self textPosition].
	^ true! !

!DrGMathItemCostume methodsFor: 'initialize-release' stamp: 'HilaireFernandes 10/26/2011 23:47'!
registerEvents
	mathItem
		ifNotNil: [
			mathItem
				when: #deletedMathItem
				send: #deleteCostume:
				to: drawable app
				with: self.
			mathItem 
				when: #updateMyView
				send: #updateAllMathItems
				to: self domain]! !

!DrGMathItemCostume methodsFor: 'initialize-release' stamp: 'HilaireFernandes 9/20/2010 22:55'!
release
	super release.
	self drawable removeMorph: self morph.
	self removeTextMorph.
	self removeEvents.! !

!DrGMathItemCostume methodsFor: 'initialize-release' stamp: 'HilaireFernandes 12/14/2005 15:35'!
removeEvents
	mathItem ifNotNil: 
		[mathItem removeActionsWithReceiver: self]! !

!DrGMathItemCostume methodsFor: 'initialize-release' stamp: 'HilaireFernandes 9/20/2010 22:51'!
removeTextMorph
	textMorph ifNotNil:
		[drawable removeMorph: textMorph.
		textMorph := nil]! !

!DrGMathItemCostume methodsFor: 'updating' stamp: 'HilaireFernandes 3/2/2006 18:05'!
startBlinking
	morph startBlinking! !

!DrGMathItemCostume methodsFor: 'updating' stamp: 'HilaireFernandes 3/2/2006 18:05'!
stopBlinking
	morph stopBlinking! !

!DrGMathItemCostume methodsFor: 'accessing' stamp: 'HilaireFernandes 1/19/2006 16:19'!
style
	^style! !

!DrGMathItemCostume methodsFor: 'accessing' stamp: 'HilaireFernandes 1/19/2006 16:19'!
style: aStyle
	style := aStyle ! !

!DrGMathItemCostume methodsFor: 'style' stamp: 'HilaireFernandes 1/24/2006 23:02'!
styleClass
	^DrGCostumeStyle! !

!DrGMathItemCostume methodsFor: 'accessing' stamp: 'HilaireFernandes 1/25/2006 10:32'!
textMorph
	^textMorph! !

!DrGMathItemCostume methodsFor: 'accessing' stamp: 'HilaireFernandes 9/21/2010 21:19'!
textMorph: aString
	aString ifNil: [^self].
	mathItem name: aString.
	aString size = 0
		ifTrue: 
			[textMorph ifNotNil: [self removeTextMorph]]
		ifFalse:
			[textMorph 
				ifNil: 
					[textMorph := DrGLabelMorph new contents: aString.
					textMorph costume: self.
					exist ifTrue: [drawable addMorph: textMorph.
					textMorph position: self textPosition]]
				ifNotNil: 
					[textMorph contents: aString]]! !

!DrGMathItemCostume methodsFor: 'layout' stamp: 'HilaireFernandes 1/25/2006 14:40'!
textPosition
"get an up to date text position"
	self subclassResponsibility! !

!DrGMathItemCostume methodsFor: 'accessing' stamp: 'HilaireFernandes 10/31/2010 22:05'!
textPositionDelta: aPoint
	textPositionDelta := aPoint! !

!DrGMathItemCostume methodsFor: 'updating' stamp: 'HilaireFernandes 11/6/2011 21:40'!
toggleLocked
	mathItem locked: mathItem isLocked not! !

!DrGMathItemCostume methodsFor: 'updating' stamp: 'HilaireFernandes 11/1/2010 20:21'!
unhide
 "make this costume visible in case it was hidden"
	style isHidden ifTrue: [
		self forceVisible: true.
		style isHidden: false]! !

!DrGMathItemCostume methodsFor: 'updating' stamp: 'HilaireFernandes 7/19/2011 21:30'!
update
	mathItem exist
		ifTrue: [
			exist ifFalse: [self updateToVisible].
			self redraw]
		ifFalse: [exist ifTrue: [self updateToInvisible]]! !

!DrGMathItemCostume methodsFor: 'updating' stamp: 'HilaireFernandes 3/21/2006 11:10'!
updateToInvisible
	drawable removeMorph: morph.
	textMorph ifNotNil:
		[drawable removeMorph: textMorph].
	exist := false! !

!DrGMathItemCostume methodsFor: 'updating' stamp: 'HilaireFernandes 3/12/2006 21:52'!
updateToVisible
	style isNotHidden 
		ifTrue: [drawable addMorph: morph].
	textMorph ifNotNil:
		[drawable addMorph: textMorph.
		textMorph position: self textPosition].
	exist := true! !

!DrGMathItemCostume methodsFor: 'accessing' stamp: 'HilaireFernandes 7/17/2011 10:31'!
view
	^drawable view! !

!DrGMathItemCostume methodsFor: 'updating' stamp: 'HilaireFernandes 3/3/2006 14:43'!
width: anInteger
	morph borderWidth: anInteger! !

!DrGMathItemCostume methodsFor: 'xml writing' stamp: 'HilaireFernandes 1/19/2008 11:54'!
writeAsXmlTo: aNode 
	"We wrote from the costume because it knows both the model and the
	view of the math item"
	self writeAttributesTo: (mathItem writeAsXmlTo: aNode)! !

!DrGMathItemCostume methodsFor: 'xml writing' stamp: 'HilaireFernandes 11/6/2011 22:06'!
writeAttributesTo: aNode 
	aNode	attributeAt: #delta put: textPositionDelta asString.
	self isLockable ifTrue: [
		aNode attributeAt: #locked put: mathItem isLocked asString capitalized].
	style writeAttributesTo: aNode.
! !

DrGMathItemCostume subclass: #DrGPointCostume
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'DrGeoII-Core-Item-View'!

!DrGPointCostume methodsFor: 'testing' stamp: 'HilaireFernandes 12/31/2005 10:04'!
basicType
	^#point! !

!DrGPointCostume methodsFor: 'layout' stamp: 'HilaireFernandes 3/12/2006 22:13'!
constraintTextPosition: aPoint
|center vector|
	center := drawable worldToMorph: mathItem point.
	vector := aPoint - center.
	^textPositionDelta := self computeTextPositionDelta: vector
	! !

!DrGPointCostume methodsFor: 'layout' stamp: 'HilaireFernandes 3/8/2006 14:45'!
distanceTo: aPoint
	^morph center dist: aPoint  ! !

!DrGPointCostume methodsFor: 'updating' stamp: 'HilaireFernandes 10/27/2011 11:48'!
editFreePointOnCurveProperty
	| answer |
	answer := self view 
		textEntry:  'Edit this curvilinear abscissa in [0;1]' translated 
		title: 'Edit this value' translated 
		entryText: mathItem abscissa asString.
	answer isEmptyOrNil ifTrue: [^self]. " cancelled"
	[answer := answer asNumber]
		on: Error
		do: [^ self view alert: 'I can''t read your value.' translated].
	self domain propertyCommandFor: mathItem with: answer! !

!DrGPointCostume methodsFor: 'updating' stamp: 'HilaireFernandes 10/27/2011 23:41'!
editFreePointProperty
	| answer |
	answer := self view 
		textEntry:  'Edit the coordinates' translated 
		title: 'Edit the coordinates' translated 
		xEntryText: mathItem point x
		yEntryText: mathItem point y.
	(answer anySatisfy: [:each | each isNil]) ifTrue: [^self]. " cancelled"
	[answer := answer first asNumber @ answer second asNumber]
		on: Error
		do: [^ self view alert: 'I can''t read your value.' translated].
	self domain propertyCommandFor: mathItem with: answer! !

!DrGPointCostume methodsFor: 'updating' stamp: 'HilaireFernandes 10/27/2011 11:23'!
editMyProperty
	| answer |
	mathItem isPropertyEditable ifFalse: [^self].
	mathItem isPointItemOnCurve 
		ifTrue: [self editFreePointOnCurveProperty]
		ifFalse: [self editFreePointProperty]! !

!DrGPointCostume methodsFor: 'initialize-release' stamp: 'HilaireFernandes 12/27/2011 00:02'!
initialize 
	super initialize.
	self textMorph: ' '.
	DrGDefault ifTablet: [^ self]. 
	(ActiveEvent isNil or: [ActiveEvent hand isNil]) 
		ifFalse: [self textMorph launchMiniEditor: ActiveEvent].    ! !

!DrGPointCostume methodsFor: 'initialize-release' stamp: 'HilaireFernandes 1/24/2006 23:03'!
instantiateMorph
	morph := DrGPointMorph new! !

!DrGPointCostume methodsFor: 'updating' stamp: 'HilaireFernandes 1/24/2008 12:25'!
pointSize: anExtent
	morph extent: anExtent.
	exist ifTrue: [morph center: (drawable worldToMorph: mathItem point)].! !

!DrGPointCostume methodsFor: 'updating' stamp: 'HilaireFernandes 5/8/2011 09:17'!
redraw
	super redraw 
		ifFalse: [^ self].
	morph
		center: (drawable worldToMorph: mathItem point)! !

!DrGPointCostume methodsFor: 'updating' stamp: 'HilaireFernandes 3/3/2006 14:57'!
shape: aShape! !

!DrGPointCostume methodsFor: 'style' stamp: 'HilaireFernandes 1/24/2006 23:04'!
styleClass
	^DrGPointCostumeStyle! !

!DrGPointCostume methodsFor: 'layout' stamp: 'HilaireFernandes 1/25/2006 22:55'!
textPosition
	^(drawable worldToMorph: mathItem point) + textPositionDelta! !

DrGMathItemCostume subclass: #DrGTextCostume
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'DrGeoII-Core-Item-View'!

!DrGTextCostume methodsFor: 'testing' stamp: 'HilaireFernandes 11/5/2011 16:46'!
basicType
	^#text! !

!DrGTextCostume methodsFor: 'updating' stamp: 'HilaireFernandes 11/9/2011 12:08'!
editMyProperty
	| answer |
	answer := self view 
		textEditor: 'Edit this text' translated 
		title: 'Edit this text' translated 
		entryText: mathItem text.
	answer isEmptyOrNil ifTrue: [^self]. " cancelled"
	self domain propertyCommandFor: mathItem with: answer string! !

!DrGTextCostume methodsFor: 'initialize-release' stamp: 'HilaireFernandes 11/5/2011 18:55'!
instantiateMorph 
	morph := DrGTextMorph new autoFit: true.
	morph contents: mathItem text;
		font: DrGDefault font! !

!DrGTextCostume methodsFor: 'updating' stamp: 'HilaireFernandes 11/5/2011 22:02'!
redraw
	exist
		ifFalse: [^ self].
	morph topLeft: self textPosition! !

!DrGTextCostume methodsFor: 'updating' stamp: 'HilaireFernandes 11/5/2011 22:07'!
registerEvents
	mathItem
		ifNotNil: [
			mathItem
				when: #deletedMathItem
				send: #deleteCostume:
				to: drawable app
				with: self.
			mathItem 
				when: #updateMyView
				send: #updateMyText
				to: self]! !

!DrGTextCostume methodsFor: 'style' stamp: 'HilaireFernandes 11/5/2011 16:56'!
styleClass
	^ DrGValueCostumeStyle! !

!DrGTextCostume methodsFor: 'accessing' stamp: 'HilaireFernandes 11/5/2011 16:57'!
textMorph: aString
	"no label for text object"
	mathItem name: aString! !

!DrGTextCostume methodsFor: 'layout' stamp: 'HilaireFernandes 11/5/2011 17:53'!
textPosition
	^(drawable worldToMorph: mathItem position)! !

!DrGTextCostume methodsFor: 'updating' stamp: 'HilaireFernandes 11/5/2011 22:09'!
updateMyText
	morph 
		contents: mathItem text;
		color: style color.! !

DrGMathItemCostume subclass: #DrGValueCostume
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'DrGeoII-Core-Item-View'!

DrGValueCostume subclass: #DrGAngleCostume
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'DrGeoII-Core-Item-View'!

!DrGAngleCostume commentStamp: '<historical>' prior: 0!
origin, length, center, vector1 and vector are data to drwa the arc or square for the geometric angle!

DrGAngleCostume subclass: #DrGAngle3PointsCostume
	instanceVariableNames: 'shapeMorph origin length center vector1 vector2 screenCenter'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'DrGeoII-Core-Item-View'!

!DrGAngle3PointsCostume commentStamp: '<historical>' prior: 0!
origin, length, center, vector1 and vector are data to drwa the arc or square for the geometric angle!

!DrGAngle3PointsCostume methodsFor: 'as yet unclassified' stamp: 'HilaireFernandes 7/5/2010 22:22'!
color: aColor
	super color: aColor.
	shapeMorph color: aColor! !

!DrGAngle3PointsCostume methodsFor: 'updating' stamp: 'HilaireFernandes 2/4/2006 22:33'!
forceVisible: aBoolean
	(super forceVisible: aBoolean) ifFalse: [^false].
	aBoolean 
		ifTrue: 
			[drawable addMorph: shapeMorph]
		ifFalse: 
			[drawable removeMorph: shapeMorph].
	^true! !

!DrGAngle3PointsCostume methodsFor: 'initialize-release' stamp: 'HilaireFernandes 6/4/2011 16:20'!
initialize
	super initialize.
	drawable addMorphBack: shapeMorph! !

!DrGAngle3PointsCostume methodsFor: 'initialize-release' stamp: 'HilaireFernandes 11/30/2011 17:39'!
instantiateMorph
	super instantiateMorph.
	morph font: DrGDefault smallFont.
	shapeMorph := DrGArcMorph new borderColor: Color black;
				 borderWidth: 1.
	shapeMorph costume: self.! !

!DrGAngle3PointsCostume methodsFor: 'initialize-release' stamp: 'HilaireFernandes 6/4/2011 16:19'!
reInitialize
	super reInitialize.
	drawable addMorphBack: shapeMorph! !

!DrGAngle3PointsCostume methodsFor: 'updating' stamp: 'HilaireFernandes 5/8/2011 09:18'!
redraw
	exist  
		ifFalse: [^ self].
	self redrawShape.
	"do it last cause we have some needed computation in redrawShape"
	super redraw! !

!DrGAngle3PointsCostume methodsFor: 'updating' stamp: 'HilaireFernandes 7/5/2010 22:21'!
redrawArc
	shapeMorph class ~= DrGArcMorph
		ifTrue:
			[drawable removeMorph: shapeMorph.
			shapeMorph := DrGArcMorph new borderColor: style color; 
				borderWidth: 1. 
			drawable addMorph: shapeMorph].
	shapeMorph 
			center: (drawable worldToMorph: center)
			radius: 15
			origin: origin negated
			length: length negated! !

!DrGAngle3PointsCostume methodsFor: 'updating' stamp: 'HilaireFernandes 7/5/2010 22:19'!
redrawRec
	| vertices sum |
	shapeMorph class = DrGPolylineMorph 
		ifFalse: 
			[drawable removeMorph: shapeMorph.
			shapeMorph := (DrGPolylineMorph new)
						borderColor: style color;
						borderWidth: 1.
			drawable addMorph: shapeMorph].
	vertices := OrderedCollection new.
	vector1 := vector1 + screenCenter.
	sum := vector1 + vector2.
	vector2 := vector2 + screenCenter.
	vertices
		add: vector1;
		add: sum;
		add: vector2.
	shapeMorph setVertices: vertices! !

!DrGAngle3PointsCostume methodsFor: 'updating' stamp: 'HilaireFernandes 2/5/2006 12:10'!
redrawShape
	self updateArcData.
	(mathItem radianAngle closeTo: Float halfPi)
		ifTrue:
			[self updateRecData.
			self redrawRec]
		ifFalse:
			[self redrawArc].! !

!DrGAngle3PointsCostume methodsFor: 'initialize-release' stamp: 'HilaireFernandes 8/3/2008 10:59'!
release
	drawable removeMorph: shapeMorph.
	super release.! !

!DrGAngle3PointsCostume methodsFor: 'updating' stamp: 'HilaireFernandes 10/31/2010 15:31'!
textPosition
|angle|
	angle := length negated / 2 + origin negated.
	^ (drawable worldToMorph: center) 
		+ ((30 * (angle cos)) @ (30 * (angle sin)))! !

!DrGAngle3PointsCostume methodsFor: 'updating' stamp: 'HilaireFernandes 6/22/2006 08:47'!
updateArcData
|end parents |
	parents := mathItem parents.
	center := parents second point.
	vector1 := parents first point - center.
	vector2 := parents third point - center.
	(vector1 = (0@0) or: [vector2 = (0@0)]) 
		ifTrue: [^self].
	origin := (vector1 y asFloat) arcTan2: vector1 x.
	end := (vector2 y asFloat) arcTan2: vector2 x.
	origin < 0 ifTrue: [origin := origin + DrGDirectionItem twoPi].
	end < 0 ifTrue: [end := end + DrGDirectionItem twoPi].
	length := (end - origin) cos arcCos.
	(end - origin > Float pi 
		or: [end - origin < 0 and: [end - origin > Float pi negated]])	
			ifTrue: [length := length negated].! !

!DrGAngle3PointsCostume methodsFor: 'updating' stamp: 'HilaireFernandes 2/5/2006 01:31'!
updateRecData
|size|
	(vector1 = (0@0) or: [vector2 = (0@0)]) 
		ifTrue: [^self].
	size := drawable pixelToWorld: 10.
	vector1 := drawable vectorToMorph: (size * vector1 / vector1 r).
	vector2 := drawable vectorToMorph: (size * vector2 / vector2 r).
	screenCenter := drawable worldToMorph: center.! !

!DrGAngle3PointsCostume methodsFor: 'updating' stamp: 'HilaireFernandes 2/4/2006 22:35'!
updateToInvisible
	super updateToInvisible.
	drawable removeMorph: shapeMorph.! !

!DrGAngle3PointsCostume methodsFor: 'updating' stamp: 'HilaireFernandes 2/4/2006 22:28'!
updateToVisible
	super updateToVisible.
	style isNotHidden
		ifTrue: [drawable addMorph: shapeMorph]! !

!DrGAngleCostume class methodsFor: 'instance creation' stamp: 'HilaireFernandes 10/25/2010 23:32'!
basicNew: mathItem
	^ mathItem nodeType == #'3pts' 
		ifTrue: [DrGAngle3PointsCostume basicNew]
		ifFalse: [DrGAngleCostume basicNew]! !

!DrGAngleCostume class methodsFor: 'as yet unclassified' stamp: 'HilaireFernandes 10/25/2010 23:14'!
mathItem: item
	|costume|
	item nodeType == #'3pts' 
		ifTrue: [DrGAngle3PointsCostume basicNew mathItem: item; yourself]
		ifFalse: [self basicNew mathItem: item; yourself]! !

!DrGAngleCostume methodsFor: 'testing' stamp: 'HilaireFernandes 2/5/2006 12:33'!
basicType
	^#angle! !

!DrGAngleCostume methodsFor: 'as yet unclassified' stamp: 'HilaireFernandes 3/16/2006 17:10'!
roundedValueString
	^ mathItem degreeAngle rounded asString, ''.! !

!DrGAngleCostume methodsFor: 'as yet unclassified' stamp: 'HilaireFernandes 9/20/2010 22:13'!
textMorph: string
	"we don't have label"
	mathItem name: string! !

!DrGValueCostume methodsFor: 'testing' stamp: 'HilaireFernandes 2/2/2006 11:38'!
basicType
	^#value! !

!DrGValueCostume methodsFor: 'layout' stamp: 'HilaireFernandes 10/31/2010 15:40'!
constraintTextPosition: aPoint
	^ aPoint! !

!DrGValueCostume methodsFor: 'accessing' stamp: 'HilaireFernandes 3/14/2006 15:25'!
contents: aString
	morph contents: aString! !

!DrGValueCostume methodsFor: 'updating' stamp: 'HilaireFernandes 10/27/2011 11:56'!
editFreeValueProperty
	| answer |
	answer := self view 
		textEntry:  'Edit this value' translated 
		title: 'Edit this value' translated 
		entryText: self valueString.
	answer isEmptyOrNil ifTrue: [^self]. " cancelled"
	[answer := answer asNumber]
		on: Error
		do: [^ self view alert: 'I can''t read your value.' translated].
	self domain propertyCommandFor: mathItem with: answer! !

!DrGValueCostume methodsFor: 'updating' stamp: 'HilaireFernandes 10/27/2011 10:54'!
editMyProperty
	| answer |
	mathItem isPropertyEditable ifFalse: [^self].
	mathItem isScriptItem
		ifTrue: [self editScriptProperty]
		ifFalse: [self editFreeValueProperty]! !

!DrGValueCostume methodsFor: 'updating' stamp: 'HilaireFernandes 10/27/2011 11:12'!
editScriptProperty 
	| browser |
	browser := Browser newOnClass: DrGeoUserScripts selector: mathItem script.
	browser addDependent: self domain.
	browser textMorph owner owner extent: 450@350.
! !

!DrGValueCostume methodsFor: 'initialize-release' stamp: 'HilaireFernandes 8/5/2011 15:36'!
instantiateMorph
	morph := DrGValueMorph new.
	(mathItem class = DrGValueValueItem  and: [self app isLoading not]) ifTrue: [
		^self launchMiniEditor.].
	morph contents: self roundedValueString! !

!DrGValueCostume methodsFor: 'updating' stamp: 'HilaireFernandes 4/20/2011 22:00'!
launchMiniEditor
	morph contents: self valueString.
	morph launchMiniEditor: ActiveHand lastEvent! !

!DrGValueCostume methodsFor: 'updating' stamp: 'HilaireFernandes 4/20/2011 22:29'!
redraw
	exist
		ifFalse: [^ self].
	(mathItem name isNil
	or: [mathItem name size = 0])
	ifTrue: [self contents: self roundedValueString]
	ifFalse: [self contents: mathItem name , ' = ' , self roundedValueString].
	morph topLeft: self textPosition! !

!DrGValueCostume methodsFor: 'updating' stamp: 'HilaireFernandes 4/8/2011 19:40'!
roundedValueString
	^ (mathItem valueItem isPoint
			or: [mathItem valueItem isNumber])
		ifTrue: [
			mathItem valueItem isFraction 
				ifTrue: [mathItem valueItem printString] 
				ifFalse: [mathItem valueItem printShowingDecimalPlaces: 2]]
		ifFalse: [mathItem valueItem asString]! !

!DrGValueCostume methodsFor: 'style' stamp: 'HilaireFernandes 10/27/2011 10:37'!
styleClass
	^DrGValueCostumeStyle! !

!DrGValueCostume methodsFor: 'accessing' stamp: 'HilaireFernandes 8/5/2011 13:34'!
textMorph: aString 
	"We don't have label for ValueCostume"
	mathItem name: aString.! !

!DrGValueCostume methodsFor: 'updating' stamp: 'HilaireFernandes 10/31/2010 15:30'!
textPosition
	^(drawable worldToMorph: mathItem position)! !

!DrGValueCostume methodsFor: 'updating' stamp: 'HilaireFernandes 3/14/2006 15:30'!
valueString
	^mathItem valueItem asString! !

Object subclass: #DrGWrappedItem
	instanceVariableNames: 'costume'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'DrGeoII-Core-Script'!

!DrGWrappedItem commentStamp: 'HilaireFernandes 1/22/2008 12:35' prior: 0!
A class to encapsulate costume for user friendly use when doing Smalltalk figure!

DrGWrappedItem subclass: #DrGWrappedCurve
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'DrGeoII-Core-Script'!

!DrGWrappedCurve methodsFor: 'updating' stamp: 'HilaireFernandes 2/4/2011 17:08'!
dashed
	costume style lineStyle: #dash

! !

!DrGWrappedCurve methodsFor: 'updating' stamp: 'HilaireFernandes 2/4/2011 17:08'!
dotted
	costume style lineStyle: #dot! !

!DrGWrappedCurve methodsFor: 'updating' stamp: 'HilaireFernandes 1/22/2008 15:46'!
large
	costume style thickness: #large! !

!DrGWrappedCurve methodsFor: 'updating' stamp: 'HilaireFernandes 2/4/2011 17:13'!
normal
	costume style thickness: #medium! !

!DrGWrappedCurve methodsFor: 'updating' stamp: 'HilaireFernandes 2/4/2011 17:11'!
plain
	costume style lineStyle: #plain
! !

!DrGWrappedCurve methodsFor: 'updating' stamp: 'HilaireFernandes 1/22/2008 15:46'!
small
	costume style thickness: #small! !

!DrGWrappedItem class methodsFor: 'as yet unclassified' stamp: 'HilaireFernandes 1/22/2008 14:03'!
wrap: aCostume 
	^ self new costume: aCostume! !

!DrGWrappedItem methodsFor: 'updating' stamp: 'HilaireFernandes 1/22/2008 13:58'!
color: aColor
	costume style color: aColor! !

!DrGWrappedItem methodsFor: 'accessing' stamp: 'HilaireFernandes 1/22/2008 14:04'!
costume: aCostume
	costume := aCostume ! !

!DrGWrappedItem methodsFor: 'updating' stamp: 'HilaireFernandes 1/22/2008 15:48'!
hide
	costume style isHidden: true.
	costume forceVisible: false.! !

!DrGWrappedItem methodsFor: 'accessing' stamp: 'HilaireFernandes 1/22/2008 14:20'!
mathItem
	^ costume mathItem ! !

!DrGWrappedItem methodsFor: 'updating' stamp: 'HilaireFernandes 4/13/2011 16:57'!
moveTo: aPoint! !

!DrGWrappedItem methodsFor: 'updating' stamp: 'HilaireFernandes 1/22/2008 13:59'!
name: aString
	costume  textMorph: aString ! !

!DrGWrappedItem methodsFor: 'updating' stamp: 'HilaireFernandes 1/24/2008 11:12'!
show
	costume forceVisible: true.
	costume style isHidden: false! !

DrGWrappedItem subclass: #DrGWrappedPoint
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'DrGeoII-Core-Script'!

!DrGWrappedPoint methodsFor: 'accessing' stamp: 'HilaireFernandes 4/9/2011 10:31'!
coordinates
	^ self mathItem point! !

!DrGWrappedPoint methodsFor: 'updating' stamp: 'HilaireFernandes 1/22/2008 22:06'!
cross
	costume style shape: #cross! !

!DrGWrappedPoint methodsFor: 'updating' stamp: 'HilaireFernandes 1/22/2008 15:45'!
large
	costume style thickness: #large! !

!DrGWrappedPoint methodsFor: 'updating' stamp: 'HilaireFernandes 1/22/2008 15:46'!
medium
	costume style thickness: #medium! !

!DrGWrappedPoint methodsFor: 'updating' stamp: 'HilaireFernandes 4/13/2011 16:57'!
moveTo: aPoint
	self mathItem moveTo: aPoint    ! !

!DrGWrappedPoint methodsFor: 'updating' stamp: 'HilaireFernandes 1/22/2008 22:06'!
round
	costume style shape: #round! !

!DrGWrappedPoint methodsFor: 'updating' stamp: 'HilaireFernandes 1/22/2008 15:46'!
small
	costume style thickness: #small! !

!DrGWrappedPoint methodsFor: 'updating' stamp: 'HilaireFernandes 1/22/2008 22:06'!
square
	costume style shape: #square! !

DrGWrappedItem subclass: #DrGWrappedValue
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'DrGeoII-Core-Script'!

!DrGWrappedValue methodsFor: 'updating' stamp: 'HilaireFernandes 4/13/2011 16:58'!
moveTo: aPoint
	self mathItem position: aPoint     ! !

!DrGWrappedValue methodsFor: 'updating' stamp: 'HilaireFernandes 1/22/2008 14:18'!
name: aString 
	costume mathItem name: aString.
	costume redraw! !

Object subclass: #DrGeo
	instanceVariableNames: 'view domain costumes undoneCostumes presenter isLoading'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'DrGeoII-Core-App'!

!DrGeo commentStamp: 'HilaireFernandes 6/15/2010 21:29' prior: 0!
The application model. It is encapsulating our sub model: geometry model, tree model and script model.
Two viewers are proposed:
 - a service is basicaly a canvas with wheel morphs, it can be embedded in bookmorph ,...
 - a polymorph window with a friendly graphic user interface!

!DrGeo class methodsFor: 'user interface' stamp: 'HilaireFernandes 2/16/2011 21:54'!
aboutDrgeo
	|dialog width text |
	text := self drgeoInformationString.
	width := 0.
	text linesDo: [:l | 
		width := width max: (UITheme current textFont widthOfStringOrText: l)].
	dialog := LongMessageDialogWindow new entryText: text.
	dialog iconMorph image: DrGIcons drgeo.
	dialog title: 'About Dr. Geo II'.
	dialog open.
	dialog textMorph minWidth: 20;
		 minHeight: 20.
	dialog
		width: (width + 120 min: Display width - 50).
	dialog position: 25 @ 25! !

!DrGeo class methodsFor: 'system startup' stamp: 'HilaireFernandes 7/27/2010 19:59'!
addStartUp
"DrGeo addStartUp"
	Smalltalk addToStartUpList: self.! !

!DrGeo class methodsFor: 'user interface' stamp: 'HilaireFernandes 2/16/2011 22:13'!
drgeoInformationString
	|text |
	text := 'Dr. Geo II is about interactive geometry and interactive programming.
{1}
	
It allows one to create geometric figure plus the interactive manipulation 
of such figure in respect with their geometric constraints. 
It is usable in teaching situation with students from primary or secondary level. 

It is simple and effective with some unique features as Smalltalk scripting 
and programmed interactive sketches. 

Copyright 1996-2011 Hilaire Fernandes

LICENSE : LGPL' format: {self drgeoVersion}.
	^ text! !

!DrGeo class methodsFor: 'user interface' stamp: 'HilaireFernandes 2/16/2011 22:11'!
drgeoVersion
	^ 'Installed version: ', ConfigurationOfDrGeo new project latestVersion versionNumber versionString ! !

!DrGeo class methodsFor: 'canvas' stamp: 'HilaireFernandes 12/21/2011 10:21'!
emptyPreview
"compute an empty PNG preiview and put it in a stream"
	|form stream textForm|
	form := Form extent: self previewSize depth: 8.
	form fill: form boundingBox rule: Form over fillColor: Color white.
	textForm := (StringMorph contents: 'No preview' translated font: DrGDefault font) imageForm.
	form copy: form boundingBox from: textForm to: (form center - (textForm extent / 2)) rule: Form over.
	stream := ByteArray new writeStream.
	PNGReadWriter putForm: form onStream: stream.
	^ stream! !

!DrGeo class methodsFor: 'objects from disk' stamp: 'HilaireFernandes 1/8/2012 14:19'!
file: filename 
	|answer|
	DrGLoader start.
	answer := DrGeoXml new load: filename for: #new.
	DrGLoader stop.
	^ answer! !

!DrGeo class methodsFor: 'objects from disk' stamp: 'HilaireFernandes 1/8/2012 14:19'!
fileFullscreen: filename 
	|answer|
	DrGLoader start.
	answer := DrGeoXml new load: filename for: #newFullScreen,
	DrGLoader stop.
	^ answer! !

!DrGeo class methodsFor: 'objects from disk' stamp: 'HilaireFernandes 1/8/2012 14:19'!
fullscreenOn: stream
	|answer |
	DrGLoader start.
	answer := DrGeoXml new loadFrom: stream for: #newFullScreen.
	DrGLoader stop.
	^ answer ! !

!DrGeo class methodsFor: 'instance creation' stamp: 'HilaireFernandes 4/18/2010 20:20'!
in: aDomain 
	"A new model with an existing domain. For multiple MVP on a same
	Domain"
	^ super basicNew domain: aDomain;
		 initializeApplication! !

!DrGeo class methodsFor: 'instance creation' stamp: 'HilaireFernandes 1/8/2012 14:18'!
new
	"instanciate a standard DrGeo application"
	|answer|
	DrGLoader start.
	answer := DrGDefault isEtoys 
		ifFalse: [self in: DrGeoDomain new]
		ifTrue: [DrGeo sample openInWorld].
	DrGLoader stop.
	^ answer! !

!DrGeo class methodsFor: 'instance creation' stamp: 'HilaireFernandes 6/26/2011 12:05'!
newFullScreen
	|app|
	app := DrGeo new.
	app view 
		fullScreen;
		position: 0@0.
	DrGDefault isOlpc ifTrue: 
		[app view extent: 1200@861].
	^ app! !

!DrGeo class methodsFor: 'objects from disk' stamp: 'HilaireFernandes 8/5/2011 15:32'!
newXmlOn: stream 
	^ DrGeoXml new loadFrom: stream for: #new! !

!DrGeo class methodsFor: 'instance creation' stamp: 'HilaireFernandes 10/10/2010 10:23'!
openFigure
	" DrGeo openFigure "
	| stream apps title|
	stream := DrGeoWindow basicNew
		fileOpen: 'Pick a Dr. Geo file name' translated 
		extensions: #('fgeo')
		path:  DrGDefault examplesPath 
		preview: #drgeo.
	stream ifNotNil: [
		apps := DrGeo newXmlOn: stream.
		title := (FileDirectory baseNameFor: (FileDirectory localNameFor: stream name)) capitalized]! !

!DrGeo class methodsFor: 'instance creation' stamp: 'HilaireFernandes 12/18/2011 11:39'!
openFigureThumbnail: parentView
	" DrGeo openFigureThumbnail "
	| dialog |
	dialog := DrGFileThumbnailWindow new.
	parentView 
		ifNil: [dialog openInWorld;
				center: World center;
				delete.
			self currentWorld openModal: dialog]
		ifNotNil: [dialog openInWorld.
			dialog center: World center.
			parentView openModal: dialog].
	(dialog cancelled or: [dialog selectedFile isNil]) ifTrue: [^self].
	((DrGeo fullscreenOn: (dialog selectedFileStream)) notEmpty & parentView notNil)  ifTrue: [
		parentView deleteWithoutConfirmation].! !

!DrGeo class methodsFor: 'canvas' stamp: 'HilaireFernandes 12/21/2011 09:23'!
preview
	"class side: return an empty PNG  preview in a stream"
	^ self emptyPreview! !

!DrGeo class methodsFor: 'canvas' stamp: 'HilaireFernandes 12/21/2011 09:09'!
previewSize
	^ 320@240! !

!DrGeo class methodsFor: 'user interface' stamp: 'HilaireFernandes 12/4/2011 16:31'!
quit
	(self confirm: 'Are you sure to quit Dr. Geo environment?' translated) ifTrue: [
		Smalltalk quitPrimitive]! !

!DrGeo class methodsFor: 'system startup' stamp: 'HilaireFernandes 7/27/2010 19:59'!
removeStartUp
"DrGeo removeStartUp"
	Smalltalk removeFromStartUpList: self.! !

!DrGeo class methodsFor: 'instance creation' stamp: 'HilaireFernandes 8/8/2010 22:35'!
sample
	"DrGeo sample openInWorld"
	^ self service view! !

!DrGeo class methodsFor: 'instance creation' stamp: 'HilaireFernandes 4/18/2010 22:38'!
service
	"instanciate a DrGeo service, to embbed in third party application or for the widgetless Etoys. It is just the canvas with suround menu and icon"
	^ self serviceIn: DrGeoDomain new! !

!DrGeo class methodsFor: 'instance creation' stamp: 'HilaireFernandes 6/18/2007 16:50'!
serviceIn: aDomain 
	"A new model with an existing domain. For multiple MVP on a same
	Domain"
	^ super basicNew domain: aDomain;
		 initializeService! !

!DrGeo class methodsFor: 'objects from disk' stamp: 'HilaireFernandes 8/8/2010 22:36'!
serviceViewXml: filename 
	(self serviceXml: filename) do: [:each | 
		each view
			center: Display center;
			openInWorld]! !

!DrGeo class methodsFor: 'objects from disk' stamp: 'HilaireFernandes 8/5/2011 15:32'!
serviceXml: filename 
	^ DrGeoXml new load: filename for: #service
! !

!DrGeo class methodsFor: 'system startup' stamp: 'HilaireFernandes 8/8/2011 00:34'!
startUp
	(World submorphs anySatisfy: [:morph | morph class == DrGeoWindow]) ifFalse:
		[self newFullScreen]! !

!DrGeo class methodsFor: 'user interface' stamp: 'HilaireFernandes 8/7/2011 11:53'!
toolsMenuOn: aBuilder
	<drgeoMenu>
	(aBuilder item: #'System Browser') 
		parent: #'Dev. Tools';
		action: [Browser  open]; 
		icon: Browser taskbarIcon;
		help: 'A source code browser to view, edit and save Dr. Geo code'.
	(aBuilder item: #Workspace) 
		parent: #'Dev. Tools';
		action: [Workspace open]; 
		icon: Workspace taskbarIcon;
		help: 'An editor to write and execute Dr. Geo programmed figure'.
	(aBuilder item: #'Method finder') 
		parent: #'Dev. Tools';
		action: [Finder new openInWorld]; 
		icon: UITheme current smallInfoIcon;
		help: 'A tool to discover method'.
		
	(aBuilder item: #'Open a file')
		parent: #'Dev. Tools';
		action: [FileList open];
		icon: UITheme current smallOpenIcon;
		help: 'Open a text, a picture or a script'.	
				
	DrGDefault isWorkstation ifTrue: [
		(aBuilder item: #'Update Dr. Geo')
			parent: #'Dev. Tools';
			action: 
				[Gofer new
					url: 'http://www.squeaksource.com/DrGeoII';
					package: 'ConfigurationOfDrGeo';
					load.
				(Smalltalk at: #ConfigurationOfDrGeo ) perform:#load];
			icon: UITheme current smallUpdateIcon].  

	(aBuilder item: #'Save session') 
		parent: #'Dev. Tools';
		action: [Smalltalk saveSession]; 
		icon: UITheme current smallSaveIcon;
		help: 'Save this Dr. Geo session'.! !

!DrGeo class methodsFor: 'user interface' stamp: 'HilaireFernandes 12/21/2011 10:56'!
worldMenu: aBuilder
	<drgeoMenu>
	(aBuilder item: 'New' translated) 
		action: [DrGeo newFullScreen]; 
		icon: UITheme current smallNewIcon;
		order: 0.
	(aBuilder item: 'Open' translated) 
		action: [DrGeo openFigureThumbnail: nil];
		icon: UITheme current smallOpenIcon;
		order: 1.
	(aBuilder item: 'Settings' translated) 
		action: [(SettingBrowser forKeywords: #('drgeosettings')) open];
		icon: UITheme current smallConfigurationIcon;
		order: 2.
	(aBuilder item: 'About' translated) 
		action: [DrGeo aboutDrgeo];
		icon: UITheme current smallLanguageIcon;
		order: 3.
	(aBuilder item: #'Dev. Tools') icon: UITheme current smallDoItIcon; order: 5.0.
	(aBuilder item: 'Quit' translated) action: [DrGeo quit]; order: 999
! !

!DrGeo methodsFor: 'costume' stamp: 'HilaireFernandes 4/18/2010 16:33'!
add: aCostume
	costumes add: aCostume.! !

!DrGeo methodsFor: 'accessing' stamp: 'HilaireFernandes 4/18/2010 20:14'!
area
	^ view area! !

!DrGeo methodsFor: 'costume' stamp: 'HilaireFernandes 10/25/2010 23:14'!
costume: aMathItem 
	"new math item event : A new math item has been created, create its costume"

	self add: (DrGMathItemCostume for: aMathItem in: self area)! !

!DrGeo methodsFor: 'costume' stamp: 'HilaireFernandes 4/18/2010 16:33'!
costumeOf: aMathItem
"Find the costume of the given mathItem"
	costumes do: [:costume|
		costume mathItem = aMathItem
			ifTrue: [^costume]].
	^nil
! !

!DrGeo methodsFor: 'costume' stamp: 'HilaireFernandes 4/18/2010 16:34'!
costumes
	^costumes! !

!DrGeo methodsFor: 'costume' stamp: 'HilaireFernandes 4/18/2010 16:34'!
costumesOf: aMathItemCollection
"Find the costumes of the given mathItem collection"
	^ aMathItemCollection collect: [:aMathItem | self costumeOf: aMathItem]
! !

!DrGeo methodsFor: 'costume' stamp: 'HilaireFernandes 11/27/2011 11:51'!
deleteCostume: aCostume
	aCostume release.
	undoneCostumes add: aCostume.
	costumes := costumes copyWithout: aCostume.
	! !

!DrGeo methodsFor: 'accessing' stamp: 'HilaireFernandes 12/10/2005 12:12'!
domain
	^domain ! !

!DrGeo methodsFor: 'accessing' stamp: 'HilaireFernandes 12/10/2005 12:12'!
domain: aDomain
	domain := aDomain ! !

!DrGeo methodsFor: 'costume' stamp: 'HilaireFernandes 10/28/2011 18:05'!
fastLastCostume
"Find the costume of the factory last mathItem"
	^ costumes last.
! !

!DrGeo methodsFor: 'costume' stamp: 'HilaireFernandes 7/20/2011 13:08'!
forceVisible: aBoolean
"Force or not the visibility of hidden costume"
	aBoolean ifTrue: [self redrawCostumes. self area changed ].
	costumes do: [:costume |
		costume forceVisible: aBoolean].! !

!DrGeo methodsFor: 'as yet unclassified' stamp: 'HilaireFernandes 8/4/2010 09:58'!
fullScreen 
	view fullScreen ! !

!DrGeo methodsFor: 'costume' stamp: 'HilaireFernandes 4/18/2010 20:14'!
groupCostumes: aCompositeItem 
|compositeCostume subCostumes|
	subCostumes := OrderedCollection new.
	aCompositeItem subItems do: [:item |
		self remove: (subCostumes add: (self costumeOf: item))].
	compositeCostume := DrGCompositeCostume 
		for: aCompositeItem 
		in: self area 
		with: subCostumes .
	self add: compositeCostume.
! !

!DrGeo methodsFor: 'initialize-release' stamp: 'HilaireFernandes 8/5/2011 15:23'!
initialize
	super initialize.
	costumes := OrderedCollection new.
	undoneCostumes := OrderedCollection new.
	isLoading := false.
	self registerEvents! !

!DrGeo methodsFor: 'initialize-release' stamp: 'HilaireFernandes 4/19/2010 10:24'!
initializeApplication
	self initialize.
	presenter := DrGeoPresenter with: self.
	view := DrGeoWindow in: self.
	view openInWorld! !

!DrGeo methodsFor: 'initialize-release' stamp: 'HilaireFernandes 4/19/2010 10:24'!
initializeService
	self initialize.
	presenter := DrGeoPresenter with: self.
	view := DrGService in: self.! !

!DrGeo methodsFor: 'accessing' stamp: 'HilaireFernandes 8/5/2011 15:23'!
isLoading
	^ isLoading! !

!DrGeo methodsFor: 'accessing' stamp: 'HilaireFernandes 8/5/2011 15:23'!
isLoading: boolean
	isLoading := boolean! !

!DrGeo methodsFor: 'costume' stamp: 'HilaireFernandes 10/28/2011 17:59'!
lastCostume
"Find the costume of the factory last mathItem"
|lastMathItem costume|
	costume := costumes last.
	costume mathItem = domain factory last ifTrue: [^costume].
	lastMathItem := domain factory last.
	costumes do: [:aCostume|
		aCostume mathItem = lastMathItem ifTrue: [^aCostume]].
	^nil
! !

!DrGeo methodsFor: 'costume' stamp: 'HilaireFernandes 4/18/2010 16:34'!
pointCostumeClosestTo: aPoint atRange: anInteger
|closestCostume smallestDistance|
	smallestDistance := anInteger. 
	costumes do: [:aCostume | | dist |
		(aCostume isPointCostume and: [(dist := aCostume distanceTo: aPoint)< smallestDistance])
			ifTrue:
				[closestCostume := aCostume.
				smallestDistance := dist]].
	^closestCostume ! !

!DrGeo methodsFor: 'costume' stamp: 'HilaireFernandes 4/18/2010 16:34'!
pointCostumes
	^ costumes select: [:aCostume | 
		aCostume isPointCostume]
! !

!DrGeo methodsFor: 'accessing' stamp: 'HilaireFernandes 4/18/2010 20:16'!
presenter
	^presenter ! !

!DrGeo methodsFor: 'canvas' stamp: 'HilaireFernandes 12/19/2011 11:37'!
preview
"compute an PNG preiview and put it in a stream"
	|form stream|
	form := self area imageForm scaledToSize: self previewSize.
	form := form asFormOfDepth: 8.
	stream := ByteArray new writeStream.
	PNGReadWriter putForm: form onStream: stream.
	^ stream! !

!DrGeo methodsFor: 'canvas' stamp: 'HilaireFernandes 12/21/2011 09:09'!
previewSize
	^ DrGeo previewSize! !

!DrGeo methodsFor: 'costume' stamp: 'HilaireFernandes 5/14/2011 23:28'!
recoverCostume: aMathItem 
	"a redo math item event : a math item has been recalled, find its
	costume "
	| costume |
	costume := self undoneCostumeOf: aMathItem.
	costume
		ifNil: [self costume: aMathItem]
		ifNotNil: [costume mathItem: aMathItem.
			costume reInitialize.
			undoneCostumes := undoneCostumes copyWithout: costume.
			self add: costume]! !

!DrGeo methodsFor: 'costume' stamp: 'HilaireFernandes 7/19/2011 21:19'!
redrawCostumes
"View resized, update the area"
	costumes do: [:costume |
		costume redraw].
	self updateGrid.! !

!DrGeo methodsFor: 'initialize-release' stamp: 'HilaireFernandes 7/20/2011 16:59'!
registerEvents
	domain
		when: #redoMathItem
		send: #recoverCostume:
		to: self.
	domain
		when: #newMathItem
		send: #costume:
		to: self.
	domain
		when: #updatedItems
		send: #updateCostumes
		to: self.
	domain
		when: #updatedDirtyItems
		send: #updateDirtyCostumes
		to: self.
	domain
		when: #groupCostumes
		send: #groupCostumes:
		to: self.
	domain
		when: #ungroupCostumes
		send: #ungroupCostumes:
		to: self! !

!DrGeo methodsFor: 'initialize-release' stamp: 'HilaireFernandes 10/4/2010 21:28'!
release
	super release.
	self area release.
	self removeEvents.
	view release.
	presenter release.
	view:= nil.
	presenter := nil! !

!DrGeo methodsFor: 'costume' stamp: 'HilaireFernandes 4/18/2010 16:33'!
remove: aCostume
	aCostume ifNotNil:
	 [costumes remove: aCostume].! !

!DrGeo methodsFor: 'initialize-release' stamp: 'HilaireFernandes 4/18/2010 20:12'!
removeEvents
	domain removeActionsWithReceiver: self.! !

!DrGeo methodsFor: 'objects from disk' stamp: 'HilaireFernandes 4/22/2010 07:45'!
saveAs: filename 
	DrGeoXml new
		app: self;
		saveAs: filename! !

!DrGeo methodsFor: 'objects from disk' stamp: 'HilaireFernandes 4/26/2010 13:13'!
saveOn: stream
	DrGeoXml new
		app: self;
		saveOn: stream! !

!DrGeo methodsFor: 'canvas' stamp: 'HilaireFernandes 4/20/2010 08:46'!
scale
	self error: 'Implemented in the drawable'! !

!DrGeo methodsFor: 'accessing' stamp: 'HilaireFernandes 10/10/2010 09:17'!
title
	^ view 
		ifNotNil: [ view title]
		ifNil: ['no name']! !

!DrGeo methodsFor: 'accessing' stamp: 'HilaireFernandes 8/4/2010 09:46'!
title: aTitle
	view title: aTitle! !

!DrGeo methodsFor: 'costume' stamp: 'HilaireFernandes 9/20/2010 22:47'!
undoneCostumeOf: aMathItem
"Find the costume of the given mathItem if any"
	undoneCostumes do: [:costume|
		costume mathItem = aMathItem
			ifTrue: [^costume]].
	^nil
! !

!DrGeo methodsFor: 'costume' stamp: 'HilaireFernandes 4/18/2010 16:35'!
ungroupCostumes: aCompositeItem 
|compositeCostume |
	compositeCostume := self costumeOf: aCompositeItem.
	compositeCostume subCostumes do: [:aCostume |
		self add: aCostume].
	self remove: compositeCostume.
! !

!DrGeo methodsFor: 'costume' stamp: 'HilaireFernandes 7/19/2011 21:27'!
updateCostumes
	costumes do: [ :each | each update ]! !

!DrGeo methodsFor: 'costume' stamp: 'HilaireFernandes 8/3/2011 10:51'!
updateDirtyCostumes
	self presenter tool costumes do: [ :each | each update ].
	self area changed! !

!DrGeo methodsFor: 'canvas' stamp: 'HilaireFernandes 7/19/2011 21:17'!
updateGrid
	self area ifNotNil: [
		self area updateGrid.
		self area changed]! !

!DrGeo methodsFor: 'accessing' stamp: 'HilaireFernandes 4/19/2010 10:23'!
view
	^ view! !

Object subclass: #DrGeoCanvas
	instanceVariableNames: 'drgeo'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'DrGeoII-Core-Script'!

!DrGeoCanvas commentStamp: 'HilaireFernandes 1/22/2008 08:55' prior: 0!
A canvas to construct programmatically interactive figure!

!DrGeoCanvas class methodsFor: 'as yet unclassified' stamp: 'HilaireFernandes 4/9/2011 09:56'!
viewLess
	^ self basicNew initializeViewLess! !

!DrGeoCanvas methodsFor: 'value' stamp: 'HilaireFernandes 1/24/2008 10:42'!
aPosition
	^ 10 atRandom - 5 @ (10 atRandom - 5)! !

!DrGeoCanvas methodsFor: 'point' stamp: 'HilaireFernandes 10/16/2010 16:11'!
altIntersectionOf: wrpCurve1 and: wrpCurve2 
"use this one when you want the intersection point when non straight line are involved"
	| item |
	self preProcessing.
	item := DrGPointIntersectionItem newWith: {wrpCurve1 mathItem. wrpCurve2 mathItem. 0 @ 0}.
	item k: 1.
	^ self finalizePoint: item! !

!DrGeoCanvas methodsFor: 'value' stamp: 'HilaireFernandes 10/16/2010 15:53'!
angle: wrpVect1 to: wrpVect2
	self preProcessing.
	^ self
		finalizeValue: (DrGAngleVectorsItem newWith: {self aPosition. wrpVect1 mathItem. wrpVect2 mathItem})! !

!DrGeoCanvas methodsFor: 'value' stamp: 'HilaireFernandes 10/16/2010 15:53'!
angle: wrpPt1 to: wrpPt2 to: wrpPt3 
	self preProcessing.
	^ self
		finalizeValue: (DrGAngle3ptsItem newWith: {self aPosition . wrpPt1 mathItem. wrpPt2 mathItem. wrpPt3 mathItem})! !

!DrGeoCanvas methodsFor: 'curve' stamp: 'HilaireFernandes 5/15/2011 10:53'!
arc: wrpPt1 to: wrpPt2 to: wrpPt3
	self preProcessing.
	^ self
		finalizeCurve: (DrGArc3ptsItem newWith: 
			{(self wrappedPoint: wrpPt1) mathItem. 
			(self wrappedPoint: wrpPt2) mathItem. 
			(self wrappedPoint: wrpPt3) mathItem})! !

!DrGeoCanvas methodsFor: 'transformations' stamp: 'HilaireFernandes 1/22/2008 17:02'!
canTransform: wrptItem
	wrptItem mathItem isGeometricItem
		ifFalse: [self error: 'Only geometric object can be transformed.' translated].! !

!DrGeoCanvas methodsFor: 'curve' stamp: 'HilaireFernandes 5/15/2011 11:00'!
circleCenter: wrpCenter radius: wrpValue
	self preProcessing.
	^ self
		finalizeCurve: (DrGCircleRadiusItem  newWith: {
			(self wrappedPoint: wrpCenter) mathItem. 
			(self wrappedValue: wrpValue) mathItem})! !

!DrGeoCanvas methodsFor: 'curve' stamp: 'HilaireFernandes 5/15/2011 11:03'!
circleCenter: wrpCenter to: wrpPt
	self preProcessing.
	^ self
		finalizeCurve: (DrGCircle2ptsItem newWith: {
			(self wrappedPoint: wrpCenter) mathItem. 
			(self wrappedPoint: wrpPt) mathItem})! !

!DrGeoCanvas methodsFor: 'accessing' stamp: 'HilaireFernandes 4/19/2010 13:09'!
costumeOf: anItem 
	^ drgeo costumeOf: anItem! !

!DrGeoCanvas methodsFor: 'value' stamp: 'HilaireFernandes 10/16/2010 16:38'!
distance: wrpItem1 to: wrpItem2 
	"Distance from a point to a line or another point"
	| arg class |
	arg := OrderedCollection newFrom: {wrpItem1 mathItem. wrpItem2 mathItem}.
	(arg first isLineItem
			or: [arg second isLineItem])
		ifTrue: [class := DrGValueDistanceptlineItem.
			arg first isLineItem
				ifTrue: [arg := arg reverse]]
		ifFalse: [class := DrGValueDistance2ptsItem].
	arg addFirst: self aPosition.
	self preProcessing.
	^ self
		finalizeValue: (class newWith: arg)! !

!DrGeoCanvas methodsFor: 'accessing' stamp: 'HilaireFernandes 1/23/2008 10:13'!
domain
	^ drgeo domain! !

!DrGeoCanvas methodsFor: 'curve' stamp: 'HilaireFernandes 4/9/2011 09:45'!
finalizeCurve: item 
	^ self
		postProcessing: (DrGWrappedCurve
				wrap: (self
						costumeOf: (self domain createFromMathItemNoStack: item)))! !

!DrGeoCanvas methodsFor: 'point' stamp: 'HilaireFernandes 4/9/2011 09:45'!
finalizePoint: item 
	^ self
		postProcessing: (DrGWrappedPoint
				wrap: (self
						costumeOf: (self domain createFromMathItemNoStack: item)))! !

!DrGeoCanvas methodsFor: 'transformations' stamp: 'HilaireFernandes 2/7/2011 22:17'!
finalizeTransformation: string with: arguments 
	| class item |
	class := self class environment classNamed: 'DrG', arguments first basicType capitalized, string, 'Item'.
	item := class newWith: arguments.
	item isPointItem
		ifTrue: [^ self finalizePoint: item].
	^ self finalizeCurve: item! !

!DrGeoCanvas methodsFor: 'value' stamp: 'HilaireFernandes 4/9/2011 09:45'!
finalizeValue: item 
	^ self postProcessing: (DrGWrappedValue
			wrap: (self
					costumeOf: (self domain createFromMathItemNoStack: item))) hide! !

!DrGeoCanvas methodsFor: 'value' stamp: 'HilaireFernandes 10/16/2010 16:39'!
freeValue: aNumber 
	self preProcessing.
	^ self
		finalizeValue: ((DrGValueValueItem newWith: {self aPosition})
				valueItem: aNumber)! !

!DrGeoCanvas methodsFor: 'initialize-release' stamp: 'HilaireFernandes 6/19/2010 17:38'!
initialize
	super initialize.
	drgeo := DrGeo service.
	drgeo view extent: 400@300.
	drgeo view openInWorld! !

!DrGeoCanvas methodsFor: 'initialize-release' stamp: 'HilaireFernandes 4/9/2011 09:56'!
initializeViewLess
	super initialize.
	drgeo := DrGeo service.
! !

!DrGeoCanvas methodsFor: 'point' stamp: 'HilaireFernandes 5/15/2011 10:21'!
intersectionOf: wrpCurve1 and: wrpCurve2 
|item|
	self preProcessing.
	item := DrGPointIntersectionItem newWith: {wrpCurve1 mathItem. wrpCurve2 mathItem. 0 @ 0}.
	item k: -1.
	^ self
		finalizePoint: (item)! !

!DrGeoCanvas methodsFor: 'value' stamp: 'HilaireFernandes 10/16/2010 16:41'!
lengthOf: wrpSegmentOrCircleOrArcOrVector 
	| class |
	self preProcessing.
	wrpSegmentOrCircleOrArcOrVector mathItem isSegmentItem
		ifTrue: [class := DrGValueSegmentlengthItem].
	wrpSegmentOrCircleOrArcOrVector mathItem isCircleItem
		ifTrue: [class := DrGValueCircleperimeterItem].
	wrpSegmentOrCircleOrArcOrVector mathItem isArcItem
		ifTrue: [class := DrGValueArclengthItem].
	wrpSegmentOrCircleOrArcOrVector mathItem isVectorItem
		ifTrue: [class := DrGValueVectornormItem].
	^ self
		finalizeValue: (class newWith: {self aPosition. wrpSegmentOrCircleOrArcOrVector mathItem})! !

!DrGeoCanvas methodsFor: 'curve' stamp: 'HilaireFernandes 5/15/2011 11:07'!
line: wrpPt1 to: wrpPt2
	self preProcessing.
	^ self
		finalizeCurve: (DrGLine2ptsItem newWith: {
			(self wrappedPoint: wrpPt1) mathItem. 
			(self wrappedPoint: wrpPt2) mathItem })! !

!DrGeoCanvas methodsFor: 'curve' stamp: 'HilaireFernandes 2/4/2011 15:25'!
locusOf: wrpPt1 when: wrpPt2 
	self preProcessing.
	wrpPt2 mathItem isPointItemOnCurve 
		ifFalse: [self error: 'The second point provided in the call must be on a curve.'].
	^ self
		finalizeCurve: (DrGLocus2ptsItem newWith: {wrpPt2 mathItem. wrpPt1 mathItem})! !

!DrGeoCanvas methodsFor: 'point' stamp: 'HilaireFernandes 10/16/2010 16:13'!
middleOf: wrpSegment
	self preProcessing.
	^ self
		finalizePoint: (DrGPointMiddlesegmentItem newWith: {wrpSegment mathItem})! !

!DrGeoCanvas methodsFor: 'point' stamp: 'HilaireFernandes 5/15/2011 10:45'!
middleOf: wrappedPoint1 and: wrappedPoint2 
	self preProcessing .
	^ self finalizePoint: (DrGPointMiddle2ptsItem newWith: 
		{(self wrappedPoint: wrappedPoint1) mathItem . (self wrappedPoint: wrappedPoint2) mathItem})! !

!DrGeoCanvas methodsFor: 'curve' stamp: 'HilaireFernandes 5/15/2011 11:14'!
parallel: wrpDirection at: wrpPtOrPoint
	self preProcessing.
	^ self
		finalizeCurve: (DrGLineParallelItem newWith: {
			wrpDirection mathItem. 
			(self wrappedPoint: wrpPtOrPoint) mathItem})! !

!DrGeoCanvas methodsFor: 'curve' stamp: 'HilaireFernandes 5/15/2011 11:23'!
perpendicular: wrpDirection at: wrpPtOrPoint 
	self preProcessing.
	^ self
		finalizeCurve: (DrGLinePerpendicularItem newWith: {
			wrpDirection mathItem. 
			(self wrappedPoint: wrpPtOrPoint) mathItem})! !

!DrGeoCanvas methodsFor: 'point' stamp: 'HilaireFernandes 10/16/2010 16:10'!
point: aPoint 
	self preProcessing.
	^self finalizePoint: (DrGPointFreeItem newWith: {aPoint})! !

!DrGeoCanvas methodsFor: 'point' stamp: 'HilaireFernandes 2/4/2011 15:15'!
point: block parent: wrpValue
	|collection|
	self preProcessing.
	collection := OrderedCollection with: block.
	collection add: wrpValue mathItem.
	^ self finalizePoint: (DrGPointBlockItem newWith: collection)! !

!DrGeoCanvas methodsFor: 'point' stamp: 'HilaireFernandes 2/4/2011 15:11'!
point: block parents: wrpValueArray
	|collection|
	self preProcessing.
	collection := OrderedCollection with: block.
	collection addAll: (wrpValueArray collect: [:wrp | wrp mathItem]).
	^ self finalizePoint: (DrGPointBlockItem newWith: collection)! !

!DrGeoCanvas methodsFor: 'point' stamp: 'HilaireFernandes 4/9/2011 09:45'!
pointOnCurve: wrpCurve at: abscissa 
	| item a |
	self preProcessing.
	item := DrGPointOncurveItem newWith: {wrpCurve mathItem. 0 @ 0}.
	a := abscissa.
	abscissa < 0
		ifTrue: [a := 0].
	abscissa > 1
		ifTrue: [a := 1].
	item setCurveAbscissa: a.
	item := self domain createFromMathItemNoStack: item.
	^ self finalizePoint: item! !

!DrGeoCanvas methodsFor: 'point' stamp: 'HilaireFernandes 10/16/2010 16:09'!
pointX: wrpValue1 Y: wrpValue2
	self preProcessing.
	^ self
		finalizePoint: (DrGPointCoordinateItem  newWith: {wrpValue1 mathItem . wrpValue2 mathItem })! !

!DrGeoCanvas methodsFor: 'curve' stamp: 'HilaireFernandes 5/15/2011 11:23'!
polygon: arguments
	self preProcessing.
	^ self
		finalizeCurve: (DrGPolygonNptsItem  newWith: (arguments collect: [:wrp | 
				(self wrappedPoint: wrp) mathItem]))! !

!DrGeoCanvas methodsFor: 'processing' stamp: 'HilaireFernandes 1/22/2008 15:00'!
postProcessing: aCostume 
	"Called after each item creation command, return aCostume "
	^ aCostume! !

!DrGeoCanvas methodsFor: 'processing' stamp: 'HilaireFernandes 1/22/2008 14:54'!
preProcessing
! !

!DrGeoCanvas methodsFor: 'curve' stamp: 'HilaireFernandes 5/15/2011 11:27'!
ray: wrpPt1 to: wrpPt2
	self preProcessing.
	^ self
		finalizeCurve: (DrGRay2ptsItem newWith: {
			(self wrappedPoint: wrpPt1) mathItem. 
			(self wrappedPoint: wrpPt2) mathItem })! !

!DrGeoCanvas methodsFor: 'transformations' stamp: 'HilaireFernandes 1/22/2008 17:02'!
reflect: wrpItem axe: wrpLine 
	self canTransform: wrpItem.
	self preProcessing.
	^ self finalizeTransformation: 'Reflexion' with: {wrpItem mathItem. wrpLine mathItem}! !

!DrGeoCanvas methodsFor: 'initialize-release' stamp: 'HilaireFernandes 4/9/2011 10:10'!
release
	drgeo release.! !

!DrGeoCanvas methodsFor: 'transformations' stamp: 'HilaireFernandes 5/15/2011 11:51'!
rotate: wrpItem center: wrpCenter angle: wrpValue 
	self canTransform: wrpItem.
	self preProcessing.
	^ self finalizeTransformation: 'Rotation' with:  {
		wrpItem mathItem. 
		(self wrappedPoint: wrpCenter) mathItem. 
		(self wrappedValue: wrpValue) mathItem}.
	! !

!DrGeoCanvas methodsFor: 'transformations' stamp: 'HilaireFernandes 5/15/2011 11:59'!
scale: wrpItem center: wrpCenter factor: wrpValue 
	self canTransform: wrpItem.
	self preProcessing.
	^ self finalizeTransformation: 'Homothety' with: {
		wrpItem mathItem. 
		(self wrappedPoint: wrpCenter) mathItem.
		(self wrappedValue: wrpValue) mathItem} 
! !

!DrGeoCanvas methodsFor: 'curve' stamp: 'HilaireFernandes 5/15/2011 11:41'!
segment: wrpPt1 to: wrpPt2
	self preProcessing.
	^ self
		finalizeCurve: (DrGSegment2ptsItem newWith: {
			(self wrappedPoint: wrpPt1) mathItem. 
			(self wrappedPoint: wrpPt2) mathItem})! !

!DrGeoCanvas methodsFor: 'value' stamp: 'HilaireFernandes 2/4/2011 16:42'!
slopeOf: wrpLine
	self preProcessing.
	^ self
		finalizeValue: (DrGValueSlopeItem newWith: {wrpLine mathItem. self aPosition})! !

!DrGeoCanvas methodsFor: 'transformations' stamp: 'HilaireFernandes 5/15/2011 12:04'!
symmetry: wrpItem center: wrpCenter 
	self canTransform: wrpItem.
	self preProcessing.
	^ self finalizeTransformation: 'Symmetry' with: {
		wrpItem mathItem. 
		(self wrappedPoint: wrpCenter) mathItem}! !

!DrGeoCanvas methodsFor: 'transformations' stamp: 'HilaireFernandes 5/15/2011 12:09'!
translate: wrpItem vector: wrpVector 
	self canTransform: wrpItem.
	self preProcessing.
	^ self finalizeTransformation: 'Translation' with: 
		{wrpItem mathItem. 
		(self wrappedVector: wrpVector) mathItem}! !

!DrGeoCanvas methodsFor: 'as yet unclassified' stamp: 'HilaireFernandes 4/9/2011 14:54'!
update
	self domain updateAllMathItems! !

!DrGeoCanvas methodsFor: 'curve' stamp: 'HilaireFernandes 5/15/2011 11:46'!
vector: wrpPt
	self preProcessing.
	^ self
		finalizeCurve: (DrGVector2ptsItem newWith: {
			(self wrappedPoint: 0@0) mathItem. 
			(self wrappedPoint: wrpPt) mathItem})! !

!DrGeoCanvas methodsFor: 'curve' stamp: 'HilaireFernandes 5/15/2011 11:44'!
vector: wrpPt1 to: wrpPt2 
	self preProcessing.
	^ self
		finalizeCurve: (DrGVector2ptsItem newWith: {
			(self wrappedPoint: wrpPt1) mathItem. 
			(self wrappedPoint: wrpPt2) mathItem})! !

!DrGeoCanvas methodsFor: 'point' stamp: 'HilaireFernandes 5/15/2011 12:09'!
wrappedPoint: aPointOrWrappedPointItem
"ensure a wrapped point"
	^ aPointOrWrappedPointItem isPoint 
		ifTrue: [(self point: aPointOrWrappedPointItem) hide] 
		ifFalse:   [aPointOrWrappedPointItem]! !

!DrGeoCanvas methodsFor: 'value' stamp: 'HilaireFernandes 5/15/2011 12:09'!
wrappedValue: aNumberOrWrappedValueItem
"ensure a wrapped value"
	^ aNumberOrWrappedValueItem  isNumber 
		ifTrue: [(self freeValue: aNumberOrWrappedValueItem ) hide] 
		ifFalse:   [aNumberOrWrappedValueItem]! !

!DrGeoCanvas methodsFor: 'curve' stamp: 'HilaireFernandes 5/15/2011 12:08'!
wrappedVector: aVectorOrWrappedVectorItem
"ensure a wrapped vector"
	^ aVectorOrWrappedVectorItem isPoint 
		ifTrue: [(self vector: aVectorOrWrappedVectorItem) hide] 
		ifFalse:   [aVectorOrWrappedVectorItem]! !

Object subclass: #DrGeoPresenter
	instanceVariableNames: 'model tool gridStuck isCreateMultipleMode selectedTool selection app oy ox scale resource filename'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'DrGeoII-Core-App'!

!DrGeoPresenter commentStamp: 'HilaireFernandes 12/19/2011 14:57' prior: 0!
A DrGPresenter is a presenter for the DrGeo app.
It is the interface to callback and button definition.

Instance Variables
	resource: 			a locale or remote resource repository where was recently saved a sketch
	selectedTool:		a symbol representing the selected UI tool
!

!DrGeoPresenter class methodsFor: 'instance creation' stamp: 'HilaireFernandes 4/19/2010 07:47'!
with: aDrgeo 
	| presenter |
	presenter := self basicNew app: aDrgeo.
"	presenter
		model: ((DrGGeometryModel in: aDrgeoModel domain)
				presenter: presenter).
"
	presenter initialize.
	^ presenter! !

!DrGeoPresenter methodsFor: 'UI definition' stamp: 'HilaireFernandes 3/13/2011 18:31'!
angleButtonData
	^ {DrGAngleBuilder title. #angle. #createAngle. DrGAngleBuilder description}! !

!DrGeoPresenter methodsFor: 'accessing' stamp: 'HilaireFernandes 4/18/2010 22:56'!
app
	^ app! !

!DrGeoPresenter methodsFor: 'accessing' stamp: 'HilaireFernandes 4/18/2010 22:56'!
app: aDrgeo
	app := aDrgeo! !

!DrGeoPresenter methodsFor: 'UI definition' stamp: 'HilaireFernandes 3/13/2011 18:33'!
arcButtonData
	^ {DrGArcBuilder title. #arc. #createArc. DrGArcBuilder description}! !

!DrGeoPresenter methodsFor: 'accessing' stamp: 'HilaireFernandes 2/9/2011 17:18'!
area
	^ self view ifNotNil: [self view area]! !

!DrGeoPresenter methodsFor: 'callback UI' stamp: 'HilaireFernandes 11/19/2011 09:37'!
askForBitmapToExport
	| answer|
	answer := UIManager default request: 'Name the bitmap to export:' translated.
	^ answer 
		ifNotNil: [MultiByteFileStream forceNewFileNamed: DrGDefault myExportsPath, FileDirectory slash, answer, '.png']! !

!DrGeoPresenter methodsFor: 'callback UI' stamp: 'HilaireFernandes 10/7/2010 22:28'!
askForFileToSave
| stream |
	stream := self view
		fileSave: 'Pick a Dr. Geo file name' translated 
		extensions: #('fgeo')
		path: (filename ifNil: [DrGDefault examplesPath]).
	^ stream! !

!DrGeoPresenter methodsFor: 'callback UI' stamp: 'HilaireFernandes 12/19/2011 22:33'!
askForSketchToKeep
| dialog newResource answer|
	dialog := DrGFilenameDialogWindow new
		textFont: UITheme new textFont;
		title: 'Filename' translated;
		text: 'Name the sketch to keep.' translated.
	dialog openInWorld.
	self view openModal: dialog.
	(dialog cancelled or: [dialog entryText isNil]) ifTrue: [^ nil].
	dialog network 
		ifTrue: [newResource := DrGDefault networkResources] 
		ifFalse: [newResource := DrGDefault mySketchResources].
	newResource filename: dialog entryText, '.fgeo'.
	^ newResource! !

!DrGeoPresenter methodsFor: 'helpers' stamp: 'HilaireFernandes 12/20/2011 16:07'!
cachePreviewFilename: string
	"the name of the cahed preview if it exist"
	^ ((string allButLast: 4), 'png')! !

!DrGeoPresenter methodsFor: 'callback create' stamp: 'HilaireFernandes 4/18/2010 23:00'!
captureSelection
|mathItems|
	self selection: self area selection.
	self selection ifNotNil:
		[mathItems := self selection collect: [:aMorph |
				aMorph costume mathItem].
		self domain createCompositeItem: mathItems].
	self area selection: nil! !

!DrGeoPresenter methodsFor: 'UI definition' stamp: 'HilaireFernandes 3/13/2011 18:34'!
circleButtonData
	^ {DrGCircleBuilder title. #circle. #createCircle. DrGCircleBuilder description}! !

!DrGeoPresenter methodsFor: 'UI definition' stamp: 'HilaireFernandes 5/14/2010 16:04'!
coordinatesButtonData
	^ {'Coordinates' translated.
	#coordinates .
	#createCoordinates .
	'Vector or point coordinates, circle or line equation.' translated}! !

!DrGeoPresenter methodsFor: 'callback create' stamp: 'HilaireFernandes 7/3/2010 11:35'!
createAngle
	self selectTool: DrGBuildTool with: DrGAngleBuilder.
	selectedTool := #angle.
	self triggerEvent: #numericToolbar with: 2.
	self changed! !

!DrGeoPresenter methodsFor: 'callback create' stamp: 'HilaireFernandes 7/17/2011 11:42'!
createArc
	self selectTool: DrGFlyBuildTool with: DrGArcBuilder.
	selectedTool := #arc.
	self triggerEvent: #curveToolbar with: 8.
	self changed! !

!DrGeoPresenter methodsFor: 'callback create' stamp: 'HilaireFernandes 7/4/2011 19:29'!
createCircle
	self selectTool: DrGFlyBuildTouchTool with: DrGCircleBuilder.
	selectedTool := #circle.
	self triggerEvent: #curveToolbar with: 7.
	self changed! !

!DrGeoPresenter methodsFor: 'callback create' stamp: 'HilaireFernandes 7/3/2010 11:37'!
createCoordinatePoint
	self selectTool: DrGBuildTool with: DrGPointByCoordinatesBuilder.
	selectedTool := #pointDefinedByCoordindates.
	self triggerEvent: #pointToolbar with: 3.
	self changed! !

!DrGeoPresenter methodsFor: 'callback create' stamp: 'HilaireFernandes 7/3/2010 11:37'!
createCoordinates
" No implemented yet

	self selectTool: DrGBuildTool with: DrGCoordinateBuilder.
	selectedTool := #coordindates.
	self triggerEvent: #numericToolbar with: 3.
	self changed
"! !

!DrGeoPresenter methodsFor: 'callback create' stamp: 'HilaireFernandes 7/3/2010 11:37'!
createDistance
	self selectTool: DrGBuildTool with: DrGValueBuilder.
	selectedTool := #distance.
	self triggerEvent: #numericToolbar with: 1.
	self changed! !

!DrGeoPresenter methodsFor: 'callback create' stamp: 'HilaireFernandes 7/3/2010 11:38'!
createFreePoint
	self selectTool: DrGFlyPointBuildTool with: DrGFlyPointBuilder.
	selectedTool := #freePoint.
	self triggerEvent: #pointToolbar with: 1.
	self changed! !

!DrGeoPresenter methodsFor: 'callback create' stamp: 'HilaireFernandes 7/3/2010 11:38'!
createHomothety
	self selectTool: DrGBuildTool with: DrGHomothetyBuilder.
	selectedTool := #scale.
	self triggerEvent: #transformationToolbar with: 5.
	self changed! !

!DrGeoPresenter methodsFor: 'callback create' stamp: 'HilaireFernandes 7/4/2011 19:29'!
createLine
	self selectTool: DrGFlyBuildTouchTool with: DrGLineBuilder.
	selectedTool := #line.
	self triggerEvent: #curveToolbar with: 1.
	self changed! !

!DrGeoPresenter methodsFor: 'callback create' stamp: 'HilaireFernandes 7/6/2010 17:52'!
createLocus
	self selectTool: DrGBuildTool with: DrGLocusBuilder.
	selectedTool := #locus.
	self triggerEvent: #curveToolbar with: 10.
	self changed! !

!DrGeoPresenter methodsFor: 'callback create' stamp: 'HilaireFernandes 7/3/2010 11:38'!
createMacro
	self selectTool: DrGMacroBuildTool with: DrGMacroBuilder.
	selectedTool := #macroBuild.
	self triggerEvent: #macroToolbar with: 1.
	self changed! !

!DrGeoPresenter methodsFor: 'callback create' stamp: 'HilaireFernandes 7/3/2010 11:38'!
createMiddlePoint
	self selectTool: DrGBuildTool with: DrGMiddleBuilder.
	selectedTool := #middle.
	self triggerEvent: #pointToolbar with: 2.
	self changed! !

!DrGeoPresenter methodsFor: 'callback canvas' stamp: 'HilaireFernandes 3/25/2011 21:41'!
createMultipleOnOff
	isCreateMultipleMode := isCreateMultipleMode not.
	self changed! !

!DrGeoPresenter methodsFor: 'accessing' stamp: 'HilaireFernandes 4/10/2010 15:45'!
createMultipleString
	^ (self isCreateMultipleMode
		ifTrue: ['<on>']
		ifFalse: ['<off>'])
		, 'create multiple' translated! !

!DrGeoPresenter methodsFor: 'callback create' stamp: 'HilaireFernandes 8/3/2011 10:12'!
createParallel
	DrGDefault ifNotLiteDo: [
		self selectTool: DrGBuildTool with: DrGParallelBuilder.
		selectedTool := #parallel.
		self triggerEvent: #curveToolbar with: 2.
		self changed]! !

!DrGeoPresenter methodsFor: 'callback create' stamp: 'HilaireFernandes 8/3/2011 10:12'!
createPerpendicular
	DrGDefault ifNotLiteDo: [
		self selectTool: DrGBuildTool with: DrGPerpendicularBuilder.
		selectedTool := #perpendicular.
		self triggerEvent: #curveToolbar with: 3.
		self changed]! !

!DrGeoPresenter methodsFor: 'callback create' stamp: 'HilaireFernandes 10/30/2011 16:44'!
createPointByCoordinates
	self selectTool: DrGBuildTool with: DrGPointByCoordinatesBuilder.
	selectedTool := #pointDefinedByCoordinates.
	self triggerEvent: #pointToolbar with: 4.
	self changed! !

!DrGeoPresenter methodsFor: 'callback create' stamp: 'HilaireFernandes 10/30/2011 17:51'!
createPointIntersection
	self selectTool: DrGBuildTool with: DrGPointIntersectionBuilder.
	selectedTool := #intersection.
	self triggerEvent: #pointToolbar with: 3.
	self changed! !

!DrGeoPresenter methodsFor: 'callback create' stamp: 'HilaireFernandes 7/17/2011 11:42'!
createPolygon
	self selectTool: DrGFlyBuildTool with: DrGPolygonBuilder.
	selectedTool := #polygon.
	self triggerEvent: #curveToolbar with: 9.
	self changed! !

!DrGeoPresenter methodsFor: 'callback create' stamp: 'HilaireFernandes 7/4/2011 19:29'!
createRay
	self selectTool: DrGFlyBuildTouchTool with: DrGRayBuilder.
	selectedTool := #ray.
	self triggerEvent: #curveToolbar with: 4.
	self changed! !

!DrGeoPresenter methodsFor: 'callback create' stamp: 'HilaireFernandes 7/3/2010 11:39'!
createReflection
	self selectTool: DrGBuildTool with: DrGReflectionBuilder.
	selectedTool := #axialSymmetry.
	self triggerEvent: #transformationToolbar with: 2.
	self changed! !

!DrGeoPresenter methodsFor: 'callback create' stamp: 'HilaireFernandes 7/3/2010 11:39'!
createRotation
	self selectTool: DrGBuildTool with: DrGRotationBuilder.
	selectedTool := #rotation.
	self triggerEvent: #transformationToolbar with: 4.
	self changed! !

!DrGeoPresenter methodsFor: 'callback create' stamp: 'HilaireFernandes 7/4/2011 18:30'!
createSegment
	self selectTool: DrGFlyBuildTouchTool with: DrGSegmentBuilder.
	selectedTool := #segment.
	self triggerEvent: #curveToolbar with: 5.
	self changed! !

!DrGeoPresenter methodsFor: 'callback create' stamp: 'HilaireFernandes 7/3/2010 11:39'!
createSymmetry
	self selectTool: DrGBuildTool with: DrGSymmetryBuilder.
	selectedTool := #centralSymmetry.
	self triggerEvent: #transformationToolbar with: 1.
	self changed! !

!DrGeoPresenter methodsFor: 'callback create' stamp: 'HilaireFernandes 11/6/2011 12:05'!
createText
	self selectTool: DrGBuildTool with: DrGTextBuilder.
	selectedTool := #text.
	self triggerEvent: #numericToolbar with: 6.
	self changed! !

!DrGeoPresenter methodsFor: 'callback create' stamp: 'HilaireFernandes 7/3/2010 11:39'!
createTranslation
	self selectTool: DrGBuildTool with: DrGTranslationBuilder.
	selectedTool := #translation.
	self triggerEvent: #transformationToolbar with: 3.
	self changed! !

!DrGeoPresenter methodsFor: 'callback create' stamp: 'HilaireFernandes 7/4/2011 19:29'!
createVector
	self selectTool: DrGFlyBuildTouchTool with: DrGVectorBuilder.
	selectedTool := #vector.
	self triggerEvent: #curveToolbar with: 6.
	self changed! !

!DrGeoPresenter methodsFor: 'callback create' stamp: 'HilaireFernandes 7/3/2010 11:39'!
deleteMacro
	self selectTool: DrGSelectTool.
	selectedTool := #deleteMacro.
	self triggerEvent: #macroToolbar with: 3.
	self changed! !

!DrGeoPresenter methodsFor: 'updating' stamp: 'HilaireFernandes 4/19/2010 07:44'!
deleteMathItem: aCostume
	self domain deleteMathItem: aCostume mathItem ! !

!DrGeoPresenter methodsFor: 'UI definition' stamp: 'HilaireFernandes 3/13/2011 18:36'!
distanceButtonData
	^ {DrGValueBuilder title. #distance. #createDistance. DrGValueBuilder description}! !

!DrGeoPresenter methodsFor: 'accessing' stamp: 'HilaireFernandes 4/18/2010 22:59'!
domain
	^app domain ! !

!DrGeoPresenter methodsFor: 'UI definition' stamp: 'HilaireFernandes 5/14/2010 16:05'!
editGroupButtonData
	^ {'Edit group' translated.
	#editGroup .
	#editSelection .
	'Distance between objects, curve length, free value' translated}! !

!DrGeoPresenter methodsFor: 'callback create' stamp: 'HilaireFernandes 11/11/2010 14:41'!
editScript
	|browser|
	"We force to select mode, then open a browser on the script classes"
	self move.
	browser := (Browser newOnClass: DrGeoUserScripts label: 'Edit or create scripts' translated) 		selectMessageCategoryNamed: #examples.
	browser addDependent: self domain.
	browser topView setProperty: #morphicLayerNumber toValue: 5.
	self triggerEvent: #numericToolbar with: 4.
	self changed! !

!DrGeoPresenter methodsFor: 'callback create' stamp: 'HilaireFernandes 4/13/2010 13:01'!
editSelection
	self selectTool: DrGEditGroupTool! !

!DrGeoPresenter methodsFor: 'callback create' stamp: 'HilaireFernandes 10/28/2011 17:00'!
eraser
	selectedTool := #eraser.
	self selectTool: DrGDeleteTool.
	self changed: #selectedTool.
	self triggerEvent: #miscToolbar with: 1.
	app redrawCostumes.! !

!DrGeoPresenter methodsFor: 'UI definition' stamp: 'HilaireFernandes 3/13/2011 21:20'!
eraserButtonData
	^ {DrGDeleteTool title. #eraser. #eraser. DrGDeleteTool description}! !

!DrGeoPresenter methodsFor: 'callback create' stamp: 'HilaireFernandes 7/3/2010 11:40'!
executeMacro
	self selectTool: DrGMacroPlayTool with: DrGMacroPlayer.
	selectedTool := #macroExecute.
	self triggerEvent: #macroToolbar with: 2.
	self changed! !

!DrGeoPresenter methodsFor: 'callback UI' stamp: 'HilaireFernandes 11/19/2011 09:45'!
exportToBitmap
	| stream |
	stream := self askForBitmapToExport.
	stream ifNotNil: [
		PNGReadWriter putForm: self area imageForm onStream: stream.
		stream close].! !

!DrGeoPresenter methodsFor: 'callback UI' stamp: 'HilaireFernandes 11/19/2010 19:31'!
figuresArray
	|items activeFigures|
	activeFigures := OrderedCollection new.
	DrGeoWindow allInstances do: [:each | 
		each owner isNil ifFalse: [activeFigures add: each app]].
	items := OrderedCollection new.
	activeFigures do: [:figure |
		items add: (Array 
			with:  figure 
			with:  'Fig. - ' translated, figure title 
			with: (ValueHolder new contents: false))].
	^ items
! !

!DrGeoPresenter methodsFor: 'UI definition' stamp: 'HilaireFernandes 3/13/2011 18:40'!
freePointButtonData
	^ {DrGFlyPointBuilder title. #freePoint. #createFreePoint. DrGFlyPointBuilder description}! !

!DrGeoPresenter methodsFor: 'UI definition' stamp: 'HilaireFernandes 3/25/2011 21:22'!
gridButtonData
	^ {'Grid' translated. #grid. #gridOnOff. 'Show or hide grid.' translated. #isGridVisible}! !

!DrGeoPresenter methodsFor: 'callback canvas' stamp: 'HilaireFernandes 7/6/2011 10:52'!
gridOnOff
	self area gridVisibleOnOff.
	self area griddingOnOff.
	self area updateGrid.
	self changed! !

!DrGeoPresenter methodsFor: 'accessing' stamp: 'HilaireFernandes 4/10/2010 15:43'!
gridString
	^ (self isGridVisible
		ifTrue: ['<on>']
		ifFalse: ['<off>'])
		, 'grid' translated! !

!DrGeoPresenter methodsFor: 'UI definition' stamp: 'HilaireFernandes 3/25/2011 21:22'!
gridStuckButtonData
	^ {'Magnetic Grid' translated. #gridMagnet. #gridStuckOnOff. 'Snap to grid.' translated. #isGridded}! !

!DrGeoPresenter methodsFor: 'callback canvas' stamp: 'HilaireFernandes 7/3/2010 11:41'!
gridStuckOnOff
	gridStuck := gridStuck not.
	self changed! !

!DrGeoPresenter methodsFor: 'accessing' stamp: 'HilaireFernandes 4/10/2010 15:44'!
griddedString
	^ (self isGridded
		ifTrue: ['<on>']
		ifFalse: ['<off>'])
		, 'snap to grid' translated! !

!DrGeoPresenter methodsFor: 'UI definition' stamp: 'HilaireFernandes 5/14/2010 16:08'!
groupButtonData
	^ {'Create Group' translated.
	#group .
	#captureSelection .
	'Create a selection group. To do a selection: i. set moving object mode, ii. select a zone by shift+drag over a backgroudn area.' translated}! !

!DrGeoPresenter methodsFor: 'UI definition' stamp: 'HilaireFernandes 3/13/2011 21:23'!
homothetyButtonData
	^ {DrGHomothetyBuilder title. #scale. #createHomothety. DrGHomothetyBuilder description}! !

!DrGeoPresenter methodsFor: 'UI definition' stamp: 'HilaireFernandes 4/20/2010 08:12'!
horizontalWheel 
	 ^ WheelMorph new beHorizontal; 
		factor: 0.05; 
		setBalloonText: 'Horizontal shift' translated; 
		target: self; 
		actionSelector: #moveOx:; 
		color: Color green twiceDarker! !

!DrGeoPresenter methodsFor: 'initialize-release' stamp: 'HilaireFernandes 4/9/2010 20:01'!
initialize
	super initialize.
	self move.
	isCreateMultipleMode := false.
	gridStuck := false.! !

!DrGeoPresenter methodsFor: 'UI definition' stamp: 'HilaireFernandes 10/30/2011 16:46'!
intersectionButtonData
	^ {DrGPointIntersectionBuilder title. #intersection. #createPointIntersection. DrGPointIntersectionBuilder description}! !

!DrGeoPresenter methodsFor: 'testing' stamp: 'HilaireFernandes 3/10/2006 18:38'!
isCreateMultipleMode
	^isCreateMultipleMode! !

!DrGeoPresenter methodsFor: 'testing' stamp: 'HilaireFernandes 10/28/2011 17:15'!
isForceVisible
	"We force unvisible costume to be visible when in style mode only"
	^ (self isTool: #style) or: [self isTool: #eraser]! !

!DrGeoPresenter methodsFor: 'testing' stamp: 'HilaireFernandes 2/9/2011 17:20'!
isGridVisible
	^ self area 
		ifNotNil: [self area gridVisible]
		ifNil: [false]! !

!DrGeoPresenter methodsFor: 'testing' stamp: 'HilaireFernandes 4/9/2010 21:36'!
isGridded
	^ gridStuck! !

!DrGeoPresenter methodsFor: 'testing' stamp: 'HilaireFernandes 7/2/2010 18:12'!
isTool: symbolTool
"Do we have this symbolTool selected"
	^ selectedTool == symbolTool ! !

!DrGeoPresenter methodsFor: 'UI definition' stamp: 'HilaireFernandes 8/3/2011 09:39'!
keepButtonData
	^ {'Keep' translated. #keep. #saveFigure. 'Keep a sketch permanently.' translated}! !

!DrGeoPresenter methodsFor: 'UI definition' stamp: 'HilaireFernandes 3/13/2011 18:42'!
lineButtonData
	^ {DrGLineBuilder title. #line. #createLine. DrGLineBuilder description}! !

!DrGeoPresenter methodsFor: 'UI definition' stamp: 'HilaireFernandes 3/13/2011 20:45'!
locusButtonData
	^ {DrGLocusBuilder title. #locus. #createLocus. DrGLocusBuilder description}! !

!DrGeoPresenter methodsFor: 'UI definition' stamp: 'HilaireFernandes 3/13/2011 20:46'!
macroBuildButtonData
	^ {DrGMacroBuilder title. #macroBuild. #createMacro. DrGMacroBuilder description}! !

!DrGeoPresenter methodsFor: 'UI definition' stamp: 'HilaireFernandes 5/14/2010 16:11'!
macroDeleteButtonData
	^ {'Delete macro' translated.
	#eraser .
	#deleteMacro .
	'Delete a macro-construction from the registry.' translated}! !

!DrGeoPresenter methodsFor: 'UI definition' stamp: 'HilaireFernandes 3/13/2011 20:47'!
macroExecuteButtonData
	^ {DrGMacroPlayer title. #macroExecute. #executeMacro. DrGMacroPlayer description}! !

!DrGeoPresenter methodsFor: 'callback UI' stamp: 'HilaireFernandes 10/17/2010 10:30'!
macrosArray
	|items|
	items := OrderedCollection new.
	DrGMacroFactory new pool do: [:macro |
		items add: (Array 
			with: macro 
			with: 'Macro - ' translated, macro title 
			with: (ValueHolder new contents: false) )].
	^ items! !

!DrGeoPresenter methodsFor: 'UI definition' stamp: 'HilaireFernandes 3/13/2011 20:48'!
middleButtonData
	^ {DrGMiddleBuilder title. #middle. #createMiddlePoint. DrGMiddleBuilder description}! !

!DrGeoPresenter methodsFor: 'accessing' stamp: 'HilaireFernandes 12/9/2005 23:19'!
model
	^model! !

!DrGeoPresenter methodsFor: 'accessing' stamp: 'HilaireFernandes 12/9/2005 23:19'!
model: aModel
	model := aModel! !

!DrGeoPresenter methodsFor: 'callback canvas' stamp: 'HilaireFernandes 7/3/2010 11:41'!
move
	self selectTool: DrGSelectTool.
	selectedTool := #move.
	self changed! !

!DrGeoPresenter methodsFor: 'UI definition' stamp: 'HilaireFernandes 3/13/2011 21:15'!
moveButtonData
	^ {DrGSelectTool title. #move. #move. DrGSelectTool description}! !

!DrGeoPresenter methodsFor: 'callback canvas' stamp: 'HilaireFernandes 4/20/2010 13:26'!
moveOx: increment
	self area moveOx: increment! !

!DrGeoPresenter methodsFor: 'callback canvas' stamp: 'HilaireFernandes 4/20/2010 13:27'!
moveOy: increment
	self area moveOy: increment ! !

!DrGeoPresenter methodsFor: 'UI definition' stamp: 'HilaireFernandes 3/25/2011 21:38'!
multipleModeButtonData
	^ {'Create Multiple' translated. #createMultiple. #createMultipleOnOff. 'Toggle between the build of several geometric objects and the build of one geometric object then move it.' translated. #isCreateMultipleMode}! !

!DrGeoPresenter methodsFor: 'UI definition' stamp: 'HilaireFernandes 8/5/2011 12:09'!
newButtonData
	^ {'New' translated. #blankFigure. #newFigure. 'Open a new blank sketch.' translated}! !

!DrGeoPresenter methodsFor: 'callback UI' stamp: 'HilaireFernandes 12/27/2011 00:02'!
newFigure
	DrGDefault ifTablet: [
		(UIManager default 
			confirm: 'Close the existing sketch and create an empty one?' translated 
			trueChoice: 'New sketch' translated
			falseChoice: 'No, keep this sketch' translated) ifTrue: [
				self newFigureAndForgetPrevious].
		^self].
	DrGeo new! !

!DrGeoPresenter methodsFor: 'callback UI' stamp: 'HilaireFernandes 6/29/2011 08:20'!
newFigureAndForgetPrevious
	DrGeo newFullScreen.
	self view deleteWithoutConfirmation! !

!DrGeoPresenter methodsFor: 'UI definition' stamp: 'HilaireFernandes 6/26/2011 12:02'!
openButtonData
	^ {'Open' translated. #open. #openFigureThumbnail. 'Open a sketch.' translated}! !

!DrGeoPresenter methodsFor: 'callback UI' stamp: 'HilaireFernandes 9/9/2010 16:07'!
openFigure
	DrGeo openFigure ! !

!DrGeoPresenter methodsFor: 'callback UI' stamp: 'HilaireFernandes 12/26/2011 13:12'!
openFigureThumbnail
	(DrGDefault isTablet and: [
		UIManager default 
			confirm: 'Close the existing sketch and open a new one?' translated 
			trueChoice: 'Open sketch' translated
			falseChoice: 'No, keep this sketch' translated]) ifTrue: [
				DrGeo openFigureThumbnail: self view.
				^self].
	DrGeo openFigureThumbnail: nil. "we don't close the existing view when not in tablet"! !

!DrGeoPresenter methodsFor: 'callback create' stamp: 'HilaireFernandes 7/3/2010 11:40'!
openItemViewer
	self selectTool: DrGViewerTool.
	self changed! !

!DrGeoPresenter methodsFor: 'callback UI' stamp: 'HilaireFernandes 7/28/2011 10:18'!
openNewFigureThumbnail
	DrGeo openFigureThumbnail: nil! !

!DrGeoPresenter methodsFor: 'tool' stamp: 'HilaireFernandes 4/2/2006 11:13'!
openViewerForCostume: aCostume
	aCostume openViewer.
	self selectTool: DrGSelectTool! !

!DrGeoPresenter methodsFor: 'UI definition' stamp: 'HilaireFernandes 3/13/2011 20:50'!
parallelButtonData
	^ {DrGParallelBuilder title. #parallel. #createParallel. DrGParallelBuilder description}! !

!DrGeoPresenter methodsFor: 'UI definition' stamp: 'HilaireFernandes 3/13/2011 20:51'!
perpendicularButtonData
	^ {DrGPerpendicularBuilder title. #perpendicular. #createPerpendicular. DrGPerpendicularBuilder description}! !

!DrGeoPresenter methodsFor: 'UI definition' stamp: 'HilaireFernandes 3/13/2011 20:52'!
pointByCoordinatesButtonData
	^ {DrGPointByCoordinatesBuilder title. #pointDefinedByCoordinates. #createPointByCoordinates. DrGPointByCoordinatesBuilder description}! !

!DrGeoPresenter methodsFor: 'UI definition' stamp: 'HilaireFernandes 3/13/2011 20:53'!
polygonButtonData
	^ {DrGPolygonBuilder title. #polygon. #createPolygon. DrGPolygonBuilder description}! !

!DrGeoPresenter methodsFor: 'callback create' stamp: 'HilaireFernandes 10/27/2011 09:53'!
property 
	self selectTool: DrGPropertyTool.
	selectedTool := #property.
	self triggerEvent: #miscToolbar with: 3.
	self changed! !

!DrGeoPresenter methodsFor: 'UI definition' stamp: 'HilaireFernandes 10/26/2011 23:06'!
propertyButtonData
	^ {DrGPropertyTool title.
	#property .
	#property .
	DrGPropertyTool description}! !

!DrGeoPresenter methodsFor: 'UI definition' stamp: 'HilaireFernandes 3/13/2011 20:55'!
rayButtonData
	^ {DrGRayBuilder title. #ray. #createRay. DrGRayBuilder description}! !

!DrGeoPresenter methodsFor: 'updating' stamp: 'HilaireFernandes 11/1/2010 20:21'!
readyToBuild
	|answer|
	"readyToBuild event, the builder is ready to build, request to the domain the construction of the math item under the builder"
	answer := self domain createMathItem: self tool builder.
	answer last ifFalse: [(app costumeOf: answer first) unhide].
	self selectionCleanUp.! !

!DrGeoPresenter methodsFor: 'updating' stamp: 'HilaireFernandes 4/19/2010 07:56'!
readyToBuildMacro
	"readyToBuild event, the builder is ready to build a macro, request to the domain the construction of the math item under the builder"
	self domain createMacro: self tool builder.
	self move! !

!DrGeoPresenter methodsFor: 'updating' stamp: 'HilaireFernandes 4/19/2010 13:11'!
readyToPlayMacro
	"readyToBuild event, the builder is ready to play a macro (enought input items), request to the domain the construction of the math items under the builder"
	self domain playMacro: self tool builder in: app.
	self move! !

!DrGeoPresenter methodsFor: 'callback canvas' stamp: 'HilaireFernandes 4/18/2010 23:00'!
redo
	self domain redo! !

!DrGeoPresenter methodsFor: 'UI definition' stamp: 'HilaireFernandes 5/14/2010 16:18'!
redoButtonData
	^ {'Redo' translated.
	#redo .
	#redo .
	'Redo last action.' translated}! !

!DrGeoPresenter methodsFor: 'UI definition' stamp: 'HilaireFernandes 3/13/2011 20:56'!
reflectionButtonData
	^ {DrGReflectionBuilder title. #axialSymmetry. #createReflection.DrGReflectionBuilder description}! !

!DrGeoPresenter methodsFor: 'initialize-release' stamp: 'HilaireFernandes 10/4/2010 21:37'!
release
	tool release.
	model release.
	self domain release.
	app := nil! !

!DrGeoPresenter methodsFor: 'UI definition' stamp: 'HilaireFernandes 3/13/2011 20:57'!
rotationButtonData
	^ {DrGRotationBuilder title. #rotation. #createRotation. DrGRotationBuilder description}! !

!DrGeoPresenter methodsFor: 'callback UI' stamp: 'HilaireFernandes 12/20/2011 16:21'!
saveFigure
| stream title |
	DrGDefault ifNotLiteDo: [
		resource ifNil: [^ self saveFigureAs].
		stream := RWBinaryOrTextStream on: (String new: 4000).
		self app saveOn: stream.
		resource put: stream.
		resource put: self app preview into: (self cachePreviewFilename: resource filename )].! !

!DrGeoPresenter methodsFor: 'callback UI' stamp: 'HilaireFernandes 12/20/2011 16:21'!
saveFigureAs
| stream |
	self askForSketchToKeep ifNotNilDo: [:rsrc |
		resource := rsrc.
		self app title: (FileDirectory baseNameFor: resource filename) capitalized.
		stream := RWBinaryOrTextStream on: (String new: 4000).
		self app saveOn: stream.
		resource put: stream.
		resource put: self app preview into: (self cachePreviewFilename: resource filename )].! !

!DrGeoPresenter methodsFor: 'callback UI' stamp: 'HilaireFernandes 12/21/2011 09:19'!
saveMultiple
"Make a selection of figures and macro-constructions to save"
	|items dialog stream|
	items := OrderedCollection new.
	items addAll: self figuresArray;	addAll: self macrosArray.
	dialog := (SelectItemDialogWindow items: items)
		textFont: StandardFonts defaultFont;
		title:  'Save Multiple' translated;
		text: 'Select the sketches and macros you want to save:' translated.
	(self view openModal: dialog) cancelled ifTrue: [ ^self].
	self askForSketchToKeep ifNotNilDo: [:rsrc |
		items := items select: [:item | item third contents = true] thenCollect: [:item | item first].
		stream := RWBinaryOrTextStream on: (String new: 4000).
		DrGeoXml new saveMultiple: items on: stream.
		rsrc put: stream.
		rsrc 
			put:  (items detect: [:item | item class = DrGeo] ifNone: [DrGeo]) preview 
			into: (self cachePreviewFilename: rsrc filename )]
! !

!DrGeoPresenter methodsFor: 'callback canvas' stamp: 'HilaireFernandes 4/20/2010 13:24'!
scale: increment
	self area scale: increment ! !

!DrGeoPresenter methodsFor: 'UI definition' stamp: 'HilaireFernandes 3/13/2011 20:58'!
scriptButtonData
	^ {DrGScriptPlayer title. #script. #useScript. DrGScriptPlayer description}! !

!DrGeoPresenter methodsFor: 'UI definition' stamp: 'HilaireFernandes 5/14/2010 16:19'!
scriptEditButtonData
	^ {'Edit a script' translated.
	#script .
	#editScript .
	'Create or edit a Smalltalk script.' translated}! !

!DrGeoPresenter methodsFor: 'UI definition' stamp: 'HilaireFernandes 3/13/2011 20:59'!
segmentButtonData
	^ {DrGSegmentBuilder title. #segment. #createSegment. DrGSegmentBuilder description}! !

!DrGeoPresenter methodsFor: 'tool' stamp: 'HilaireFernandes 1/22/2006 17:05'!
selectTool: aToolClassName
	self tool: nil.
	self tool: (aToolClassName new presenter: self).! !

!DrGeoPresenter methodsFor: 'tool' stamp: 'HilaireFernandes 3/13/2011 12:21'!
selectTool: aToolClassName with: aBuilderClassName 
	self
		tool: ((aToolClassName with: aBuilderClassName)
				presenter: self).
	self domain changed: #selectedTool! !

!DrGeoPresenter methodsFor: 'tool' stamp: 'HilaireFernandes 4/9/2010 19:34'!
selectToolAfterBuilding
	self isCreateMultipleMode
		ifFalse:
			[self move]! !

!DrGeoPresenter methodsFor: 'accessing' stamp: 'HilaireFernandes 4/18/2010 16:31'!
selection
	^selection! !

!DrGeoPresenter methodsFor: 'accessing' stamp: 'HilaireFernandes 4/18/2010 16:31'!
selection: aSelection
	aSelection 
		ifNil: [selection := nil]
		ifNotNil:
			[selection := aSelection select: [:aMorph |
				aMorph isMorphCostume]]! !

!DrGeoPresenter methodsFor: 'tool' stamp: 'HilaireFernandes 10/30/2011 18:08'!
selectionCleanUp
|createdItem|
	"a mathItem is just been created, do some clean up"
	self tool reset.
	createdItem := self domain factory last.
	"eventually if not in createMultipleMode we escape from this tool"
	(createdItem isFreePointItem 
		or: [createdItem isIntersectionPointItem 
		or: [createdItem isPointItemOnCurve]])
			ifFalse: [self selectToolAfterBuilding].
	((tool class = DrGFlyPointBuildTool) or: [selectedTool = #intersection]) ifTrue: [
		self selectToolAfterBuilding].
! !

!DrGeoPresenter methodsFor: 'accessing' stamp: 'HilaireFernandes 3/13/2011 21:12'!
statusMessage
	^ self tool description! !

!DrGeoPresenter methodsFor: 'callback canvas' stamp: 'HilaireFernandes 10/27/2011 09:41'!
style
	selectedTool := #style.
	self selectTool: DrGStyleTool.
	self changed: #selectedTool.
	self triggerEvent: #miscToolbar with: 2.
	app redrawCostumes.
! !

!DrGeoPresenter methodsFor: 'UI definition' stamp: 'HilaireFernandes 3/13/2011 21:18'!
styleButtonData
	^ {DrGStyleTool title. #style. #style. DrGStyleTool description}! !

!DrGeoPresenter methodsFor: 'UI definition' stamp: 'HilaireFernandes 3/13/2011 21:00'!
symmetryButtonData
	^ {DrGSymmetryBuilder title. #centralSymmetry. #createSymmetry. DrGSymmetryBuilder description}! !

!DrGeoPresenter methodsFor: 'UI definition' stamp: 'HilaireFernandes 11/6/2011 12:02'!
textButtonData
	^ {DrGTextBuilder title. #text. #createText. DrGTextBuilder description}! !

!DrGeoPresenter methodsFor: 'accessing' stamp: 'HilaireFernandes 12/16/2005 22:41'!
tool
	^tool! !

!DrGeoPresenter methodsFor: 'accessing' stamp: 'HilaireFernandes 3/13/2011 21:40'!
tool: aTool 
	tool
		ifNotNil: [tool release].
	tool := aTool.
	tool
		ifNotNil: [tool registerDependant: self.
			self changed: #statusMessage]! !

!DrGeoPresenter methodsFor: 'UI definition' stamp: 'HilaireFernandes 3/13/2011 21:02'!
translationButtonData
	^ {DrGTranslationBuilder title. #translation. #createTranslation. DrGTranslationBuilder description}! !

!DrGeoPresenter methodsFor: 'callback canvas' stamp: 'HilaireFernandes 10/25/2011 18:15'!
undo
	self tool resetForFreshStart.
	self domain undo! !

!DrGeoPresenter methodsFor: 'UI definition' stamp: 'HilaireFernandes 5/14/2010 16:21'!
undoButtonData
	^ {'Undo' translated.
	#undo .
	#undo .
	'Undo last action' translated}! !

!DrGeoPresenter methodsFor: 'callback create' stamp: 'HilaireFernandes 7/3/2010 11:40'!
useScript
	self selectTool: DrGScriptPlayTool with: DrGScriptPlayer.
	selectedTool := #script.
	self triggerEvent: #numericToolbar with: 5.
	self changed! !

!DrGeoPresenter methodsFor: 'UI definition' stamp: 'HilaireFernandes 3/13/2011 21:02'!
vectorButtonData
	^ {DrGVectorBuilder title. #vector. #createVector. DrGVectorBuilder description}! !

!DrGeoPresenter methodsFor: 'UI definition' stamp: 'HilaireFernandes 4/20/2010 08:11'!
verticalWheel 
	 ^ WheelMorph new 
		beVertical; 
		factor: 0.05; 
		setBalloonText: 'Vertical shift' translated; 
		target: self; 
		actionSelector: #moveOy:; 
		color: Color green twiceDarker! !

!DrGeoPresenter methodsFor: 'accessing' stamp: 'HilaireFernandes 2/9/2011 17:17'!
view
	^ app ifNotNil: [app view]! !

!DrGeoPresenter methodsFor: 'UI definition' stamp: 'HilaireFernandes 3/13/2011 21:20'!
viewerButtonData
	^ {DrGViewerTool title. #viewer. #openItemViewer. DrGViewerTool description}! !

!DrGeoPresenter methodsFor: 'UI definition' stamp: 'HilaireFernandes 4/20/2010 08:31'!
zoomWheel 
	 ^ WheelMorph new 
		beVertical; 
		factor: 1; 
		setBalloonText: 'Zooming' translated; 
		target: self; 
		actionSelector: #scale:; 
		color: Color orange darker! !

Object subclass: #DrGeoScripts
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'DrGeoII-Core-Script'!

DrGeoScripts class
	instanceVariableNames: 'uniqueInstance'!

DrGeoScripts class
	instanceVariableNames: 'uniqueInstance'!

!DrGeoScripts class methodsFor: 'as yet unclassified' stamp: 'HilaireFernandes 8/2/2008 20:31'!
instance
	^self new! !

!DrGeoScripts class methodsFor: 'instance creation' stamp: 'HilaireFernandes 8/2/2008 20:31'!
new
	^ uniqueInstance ifNil: [uniqueInstance := super new]! !

!DrGeoScripts methodsFor: 'accessing' stamp: 'HilaireFernandes 8/24/2008 17:54'!
list
	"compiledMethod list, for this class and subclasses
	DrGeoUserScripts instance list
	"
	| exclude |
	exclude := DrGeoUserScripts allMethodsInCategory: #private.
	^DrGeoUserScripts methodDict values reject: [:compiledMethod |
		exclude includes: compiledMethod selector].
! !

!DrGeoScripts methodsFor: 'accessing' stamp: 'HilaireFernandes 8/2/2008 21:29'!
selectors
	"the selector name for this class and subclasses"
	^self list collect: [:method | method selector asString]! !

Object subclass: #DrGeoXml
	instanceVariableNames: 'app'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'DrGeoII-Core-App'!

!DrGeoXml commentStamp: 'HilaireFernandes 10/9/2010 22:04' prior: 0!
A DrGeoXml is an helper to load or save as an XML representation a DrGeo instance.
!

!DrGeoXml methodsFor: 'accessing' stamp: 'HilaireFernandes 4/21/2010 15:15'!
app: anObject
	app := anObject! !

!DrGeoXml methodsFor: 'accessing' stamp: 'HilaireFernandes 4/21/2010 15:16'!
area
	^ app view area! !

!DrGeoXml methodsFor: 'accessing' stamp: 'HilaireFernandes 4/21/2010 15:16'!
domain
	^ app domain! !

!DrGeoXml methodsFor: 'accessing' stamp: 'HilaireFernandes 4/21/2010 15:49'!
factory
	^ self domain factory! !

!DrGeoXml methodsFor: 'xml read item' stamp: 'HilaireFernandes 10/25/2010 19:57'!
instantiateItemFrom: aNode
 "instantiate an item from this node"
	^ self 
		perform: (self methodToParse: aNode) 
		with: aNode.! !

!DrGeoXml methodsFor: 'xml read item' stamp: 'HilaireFernandes 10/24/2010 20:23'!
instantiateStyle: aNode for: item
 "instantiate the style for this item"
	self 
		perform: (#parseStyleOf , aNode tag capitalized , #':with:') asSymbol
		with: item
		with: aNode! !

!DrGeoXml methodsFor: 'testing' stamp: 'HilaireFernandes 10/21/2010 22:35'!
isCreatingMacro
	^ app isNil! !

!DrGeoXml methodsFor: 'xml read' stamp: 'HilaireFernandes 10/17/2010 11:06'!
load: filename for: constructor
	^ self 
		loadFrom: (FileStream readOnlyFileNamed: filename) 
		for: constructor ! !

!DrGeoXml methodsFor: 'xml read' stamp: 'HilaireFernandes 10/17/2010 11:03'!
loadFigureFrom: doc for: constructor
"One XML document may contain several drgeo figure instances, parse it all,
and return a DrGeo for each one"
	|instances |
	instances := OrderedCollection new.
	(doc elementAt: #drgenius)
		tagsNamed: #drgeo
		do: [:each | instances add: (self parseOneFigure: each for: constructor)].
	^ instances! !

!DrGeoXml methodsFor: 'xml read' stamp: 'HilaireFernandes 10/17/2010 11:13'!
loadFrom: stream for: constructor
"One XML document may contain several drgeo figure instances, parse it all,
and return a DrGeo for each one"
	| doc instances |
	doc := XMLDOMParser parseDocumentFrom: stream.
	stream close.
	self loadMacroFrom: doc.
	^ self loadFigureFrom: doc for: constructor.! !

!DrGeoXml methodsFor: 'xml read' stamp: 'HilaireFernandes 10/17/2010 18:48'!
loadMacroFrom: doc
	(doc elementAt: #drgenius)
		tagsNamed: #macro
		do: [:each | DrGMacroFactory new addUnique: (self parseOneMacro: each)].
! !

!DrGeoXml methodsFor: 'xml read' stamp: 'HilaireFernandes 10/17/2010 11:05'!
loadOnlyFigureFrom: stream for: constructor
"One XML document may contain several drgeo figure instances, parse it all,
and return a DrGeo for each one"
	| doc |
	doc := XMLDOMParser parseDocumentFrom: stream.
	stream close.
	^ self loadFigureFrom: doc	for: constructor! !

!DrGeoXml methodsFor: 'xml read item' stamp: 'HilaireFernandes 10/24/2010 20:11'!
methodToParse: aNode
 "find which method to use to parse this node"
	|type|
	type := ((aNode attributeAt: #type) copyWithoutAll: '_' ) capitalized asSymbol.
	 ^(#parse , aNode tag capitalized , type , #:) asSymbol ! !

!DrGeoXml methodsFor: 'accessing' stamp: 'HilaireFernandes 4/21/2010 15:42'!
ox
	 ^ self area ox! !

!DrGeoXml methodsFor: 'accessing' stamp: 'HilaireFernandes 4/21/2010 15:42'!
ox: aNumber
	self area ox: aNumber! !

!DrGeoXml methodsFor: 'accessing' stamp: 'HilaireFernandes 4/21/2010 15:42'!
oy
	 ^ self area oy! !

!DrGeoXml methodsFor: 'accessing' stamp: 'HilaireFernandes 4/21/2010 15:42'!
oy: aNumber
	self area oy: aNumber! !

!DrGeoXml methodsFor: 'xml read macro' stamp: 'HilaireFernandes 10/16/2010 23:47'!
parentMacroNodesIn: macro from: aNode 
	"return a collection of parent macro nodes  given theirs id in the attribute
	elements <parent id=xxx>"
	|collection|
	collection := OrderedCollection new.
	aNode	elementsDo: [:each | 
		(each tag = #parent) ifTrue: [
			collection add: (macro detectId: (each attributeAt: 'ref') asSymbol)]].
	^collection ! !

!DrGeoXml methodsFor: 'parse item' stamp: 'HilaireFernandes 4/21/2010 15:50'!
parentsById: aNode 
	"return a collection of the item parent given theirs id in the attribute
	elements <parent id=xxx>"
	|collection|
	collection := OrderedCollection new.
	aNode	elementsDo: [:each | (each tag = #parent) ifTrue: [collection
				add: (self factory findInPoolById: (each attributeAt: 'ref') asSymbol)]].
	^collection ! !

!DrGeoXml methodsFor: 'parse item' stamp: 'HilaireFernandes 4/21/2010 15:50'!
parseAngle3pts: aNode
	^ self parseItem: aNode with: DrGAngleBuilder.! !

!DrGeoXml methodsFor: 'parse item' stamp: 'HilaireFernandes 10/25/2010 08:49'!
parseAngleVectors: aNode
	^ self parseItem: aNode with: DrGAngleBuilder.
! !

!DrGeoXml methodsFor: 'parse item' stamp: 'HilaireFernandes 10/16/2010 09:39'!
parseArc3pts: aNode
	^ self parseItem: aNode with: DrGArcBuilder.! !

!DrGeoXml methodsFor: 'parse item' stamp: 'HilaireFernandes 10/16/2010 09:41'!
parseArcCircle3pts: aNode
"Dr.Geo 1.1 compatibility"
	^ self parseArc3pts: aNode! !

!DrGeoXml methodsFor: 'parse item' stamp: 'HilaireFernandes 10/16/2010 09:41'!
parseArcCircleReflexion: aNode
"Dr.Geo 1.1 compatibility"
	^ self parseArcReflexion: aNode ! !

!DrGeoXml methodsFor: 'parse item' stamp: 'HilaireFernandes 10/16/2010 09:41'!
parseArcCircleRotation: aNode
"Dr.Geo 1.1 compatibility"
	^ self parseArcRotation: aNode! !

!DrGeoXml methodsFor: 'parse item' stamp: 'HilaireFernandes 10/16/2010 09:47'!
parseArcCircleScale: aNode 
"Dr.Geo 1.1 compatibility"
	^self parseArcHomothety: aNode ! !

!DrGeoXml methodsFor: 'parse item' stamp: 'HilaireFernandes 10/16/2010 09:50'!
parseArcCircleSymmetry: aNode 
"Dr.Geo 1.1 compatibility"
	^ self parseArcSymmetry: aNode! !

!DrGeoXml methodsFor: 'parse item' stamp: 'HilaireFernandes 10/16/2010 09:51'!
parseArcCircleTranslation: aNode
"Dr.Geo 1.1 compatibility"
	^self parseArcTranslation: aNode ! !

!DrGeoXml methodsFor: 'parse item' stamp: 'HilaireFernandes 10/16/2010 09:46'!
parseArcHomothety: aNode 
	^self parseHomothety: aNode ! !

!DrGeoXml methodsFor: 'parse item' stamp: 'HilaireFernandes 10/16/2010 09:40'!
parseArcReflexion: aNode
	^ self parseReflexion: aNode ! !

!DrGeoXml methodsFor: 'parse item' stamp: 'HilaireFernandes 10/16/2010 09:41'!
parseArcRotation: aNode
	^ self parseRotation: aNode! !

!DrGeoXml methodsFor: 'parse item' stamp: 'HilaireFernandes 10/16/2010 09:50'!
parseArcSymmetry: aNode 
	^ self parseSymmetry: aNode! !

!DrGeoXml methodsFor: 'parse item' stamp: 'HilaireFernandes 10/16/2010 09:51'!
parseArcTranslation: aNode
	^self parseTranslation: aNode ! !

!DrGeoXml methodsFor: 'parse item' stamp: 'HilaireFernandes 4/21/2010 15:51'!
parseCircle2pts: aNode
	^ self parseItem: aNode with: DrGCircleBuilder.! !

!DrGeoXml methodsFor: 'parse item' stamp: 'HilaireFernandes 10/16/2010 09:43'!
parseCircleHomothety: aNode
	^self parseHomothety: aNode ! !

!DrGeoXml methodsFor: 'parse item' stamp: 'HilaireFernandes 4/21/2010 15:51'!
parseCircleRadius: aNode
	^ self parseItem: aNode with: DrGCircleBuilder.! !

!DrGeoXml methodsFor: 'parse item' stamp: 'HilaireFernandes 4/21/2010 15:51'!
parseCircleReflexion: aNode
	^ self parseReflexion: aNode! !

!DrGeoXml methodsFor: 'parse item' stamp: 'HilaireFernandes 4/21/2010 15:51'!
parseCircleRotation: aNode
	^ self parseRotation: aNode! !

!DrGeoXml methodsFor: 'parse item' stamp: 'HilaireFernandes 10/16/2010 09:47'!
parseCircleScale: aNode
"Dr.Geo 1.1 compatibility"
	^self parseCircleHomothety: aNode ! !

!DrGeoXml methodsFor: 'parse item' stamp: 'HilaireFernandes 10/25/2010 08:49'!
parseCircleSegment: aNode
	^ self parseItem: aNode with: DrGCircleBuilder.! !

!DrGeoXml methodsFor: 'parse item' stamp: 'HilaireFernandes 4/21/2010 15:51'!
parseCircleSymmetry: aNode 
	^ self parseSymmetry:  aNode! !

!DrGeoXml methodsFor: 'parse item' stamp: 'HilaireFernandes 4/21/2010 15:51'!
parseCircleTranslation: aNode
	^ self parseTranslation: aNode ! !

!DrGeoXml methodsFor: 'xml read' stamp: 'HilaireFernandes 11/9/2010 10:27'!
parseFigureFrom: drgeoNode
"We parse from the view, because only the view knows both the domain and the view"
	| type itemStatut |
	self setScaleTo: (drgeoNode attributeAt: #scale ifAbsent: [self scale]).
	self ox: (drgeoNode attributeAt: #'origin_x' ifAbsent: [self ox]).
	self oy: (drgeoNode attributeAt: #'origin_y' ifAbsent: [self oy]).
	app title: (drgeoNode attributeAt: 'name' ifAbsent: ['no name' translated]). 
	(drgeoNode attributeAt: #grid) = 'True'	ifTrue: [self presenter gridOnOff].
	self parseScript: drgeoNode.
	drgeoNode	elementsDo: [:aNode | 
		itemStatut := self instantiateItemFrom: aNode.
		itemStatut second ifTrue: [self instantiateStyle: aNode for: itemStatut first]].! !

!DrGeoXml methodsFor: 'parse item' stamp: 'HilaireFernandes 10/16/2010 09:59'!
parseHalfLine2pts: aNode
"Dr.Geo 1.1 compatibility"
	^self parseRay2pts: aNode! !

!DrGeoXml methodsFor: 'parse item' stamp: 'HilaireFernandes 10/16/2010 10:00'!
parseHalfLineReflexion: aNode
"Dr.Geo 1.1 compatibility"
	^self parseRayReflexion: aNode! !

!DrGeoXml methodsFor: 'parse item' stamp: 'HilaireFernandes 10/16/2010 10:00'!
parseHalfLineRotation: aNode
"Dr.Geo 1.1 compatibility"
	^self parseRayRotation: aNode! !

!DrGeoXml methodsFor: 'parse item' stamp: 'HilaireFernandes 10/16/2010 09:47'!
parseHalfLineScale: aNode 
"Dr.Geo 1.1 compatibility"
	^self parseRayHomothety: aNode ! !

!DrGeoXml methodsFor: 'parse item' stamp: 'HilaireFernandes 10/16/2010 10:00'!
parseHalfLineSymmetry: aNode 
"Dr.Geo 1.1 compatibility"
	^self parseRaySymmetry: aNode! !

!DrGeoXml methodsFor: 'parse item' stamp: 'HilaireFernandes 10/16/2010 10:01'!
parseHalfLineTranslation:aNode
"Dr.Geo 1.1 compatibility"
	^ self parseRayTranslation: aNode ! !

!DrGeoXml methodsFor: 'parse item' stamp: 'HilaireFernandes 10/16/2010 09:42'!
parseHomothety: aNode 
	^ self parseItem: aNode with: DrGHomothetyBuilder! !

!DrGeoXml methodsFor: 'xml read macro' stamp: 'HilaireFernandes 10/16/2010 23:52'!
parseInput: node inMacro: macro
	(node elementAt: #input) elementsDo: [:each | |item|
		item := DrGMacroMathItem new
			id: (each attributeAt: #id) asSymbol; 
			basicType: each tag;
			yourself.
		macro addNode: (DrGMacroInputNode new itemTemplate: item; yourself)]! !

!DrGeoXml methodsFor: 'parse item' stamp: 'HilaireFernandes 10/27/2010 10:47'!
parseItem: aNode with: builderClass 
	^ self isCreatingMacro 
		ifFalse: [builderClass itemFrom: aNode in: app "an itemStatut"]
		ifTrue: [builderClass basicItemFrom: aNode "an item"].
! !

!DrGeoXml methodsFor: 'parse item' stamp: 'HilaireFernandes 4/21/2010 15:51'!
parseLine2pts: aNode
	^self parseItem: aNode with: DrGLineBuilder.! !

!DrGeoXml methodsFor: 'parse item' stamp: 'HilaireFernandes 10/16/2010 09:45'!
parseLineHomothety: aNode
	^self parseHomothety: aNode ! !

!DrGeoXml methodsFor: 'parse item' stamp: 'HilaireFernandes 4/21/2010 15:46'!
parseLineParallel: aNode
	^self parseItem: aNode with: DrGParallelBuilder.! !

!DrGeoXml methodsFor: 'parse item' stamp: 'HilaireFernandes 4/21/2010 15:51'!
parseLinePerpendicular: aNode 
	^ self parseItem: aNode with: DrGPerpendicularBuilder.
! !

!DrGeoXml methodsFor: 'parse item' stamp: 'HilaireFernandes 10/25/2010 08:52'!
parseLineReflexion: aNode 
	^ self parseItem: aNode with: DrGReflectionBuilder.
! !

!DrGeoXml methodsFor: 'parse item' stamp: 'HilaireFernandes 4/21/2010 15:52'!
parseLineRotation: aNode
	^self parseRotation: aNode ! !

!DrGeoXml methodsFor: 'parse item' stamp: 'HilaireFernandes 10/16/2010 09:47'!
parseLineScale: aNode
"Dr.Geo 1.1 compatibility"
	^self parseLineHomothety: aNode ! !

!DrGeoXml methodsFor: 'parse item' stamp: 'HilaireFernandes 4/21/2010 15:52'!
parseLineSymmetry: aNode
	^ self parseSymmetry: aNode ! !

!DrGeoXml methodsFor: 'parse item' stamp: 'HilaireFernandes 4/21/2010 15:52'!
parseLineTranslation: aNode
	^self parseTranslation: aNode ! !

!DrGeoXml methodsFor: 'parse item' stamp: 'HilaireFernandes 10/16/2010 16:07'!
parseLocus2pts: aNode
	^ self parseItem: aNode with: DrGLocusBuilder.! !

!DrGeoXml methodsFor: 'parse item' stamp: 'HilaireFernandes 10/16/2010 16:08'!
parseLocusNone: aNode
"DrGeo 1.1 compatibility"
	 ^ self parseLocus2pts: aNode ! !

!DrGeoXml methodsFor: 'parse item' stamp: 'HilaireFernandes 10/16/2010 10:07'!
parseNumericArclength: aNode
"Dr.Geo 1.1 compatibility"
	^ self parseValueArclength: aNode ! !

!DrGeoXml methodsFor: 'parse item' stamp: 'HilaireFernandes 10/16/2010 10:07'!
parseNumericCircleperimeter: aNode
"Dr.Geo 1.1 compatibility"
	^ self parseValueCircleperimeter: aNode ! !

!DrGeoXml methodsFor: 'parse item' stamp: 'HilaireFernandes 10/16/2010 10:08'!
parseNumericDistance2pts: aNode
"Dr.Geo 1.1 compatibility"
	^ self parseValueDistance2pts: aNode! !

!DrGeoXml methodsFor: 'parse item' stamp: 'HilaireFernandes 10/16/2010 10:08'!
parseNumericDistanceptline: aNode
"Dr.Geo 1.1 compatibility"
	^ self parseValueDistanceptline: aNode ! !

!DrGeoXml methodsFor: 'parse item' stamp: 'HilaireFernandes 10/16/2010 10:09'!
parseNumericSegmentlength: aNode
"Dr.Geo 1.1 compatibility"
	^ self parseValueSegmentlength: aNode ! !

!DrGeoXml methodsFor: 'parse item' stamp: 'HilaireFernandes 10/16/2010 10:09'!
parseNumericSlope: aNode
"Dr.Geo 1.1 compatibility"
	^ self parseValueSlope: aNode ! !

!DrGeoXml methodsFor: 'parse item' stamp: 'HilaireFernandes 10/16/2010 10:09'!
parseNumericValue: aNode
"Dr.Geo 1.1 compatibility"
	^ self parseValueValue: aNode ! !

!DrGeoXml methodsFor: 'parse item' stamp: 'HilaireFernandes 10/16/2010 10:09'!
parseNumericVectornorm: aNode
"Dr.Geo 1.1 compatibility"
	^ self parseValueVectornorm: aNode ! !

!DrGeoXml methodsFor: 'xml read' stamp: 'HilaireFernandes 8/5/2011 15:36'!
parseOneFigure: node for: constructor
" Instantiate this node in a DrGeo app and return it"
	self app: (DrGeo perform: constructor).
	app isLoading: true.
	self parseFigureFrom: node.
	app 
		forceVisible: false;
		isLoading: false;
		redrawCostumes.
	^ app! !

!DrGeoXml methodsFor: 'xml read macro' stamp: 'HilaireFernandes 10/27/2010 12:03'!
parseOneMacro: node
" parse one macro and return it"
	| macro|
	macro := DrGMacro new.
	macro title: (node attributeAt: #name).
	macro description: (node elementAt: #description) contentString.
	self parseInput: node inMacro: macro.
	self parseOutput: node inMacro: macro.
	^ macro! !

!DrGeoXml methodsFor: 'xml read macro' stamp: 'HilaireFernandes 10/27/2010 12:24'!
parseOutput: node inMacro: macro
	(node elementAt: #output) elementsDo: [:aNode | |macroNodeClass|
		aNode tag = #hidden ifTrue: [macroNodeClass := DrGMacroHiddenNode].
		aNode tag = #final ifTrue: [macroNodeClass := DrGMacroOutputNode].
		aNode elementsDo: [:child | 
			macro addNode: (
				macroNodeClass new 
					itemTemplate: (self instantiateItemFrom: child); 
					parentNodes: (self parentMacroNodesIn: macro from: child);
					yourself)]]! !

!DrGeoXml methodsFor: 'parse item' stamp: 'HilaireFernandes 2/4/2011 16:11'!
parsePointBlock: aNode
	self notYetImplemented.
	"^ self parseItem: aNode with: DrGPointBlockBuilder."! !

!DrGeoXml methodsFor: 'parse item' stamp: 'HilaireFernandes 10/25/2010 08:53'!
parsePointCoordinate: aNode
	^ self parseItem: aNode with: DrGPointByCoordinatesBuilder.! !

!DrGeoXml methodsFor: 'parse item' stamp: 'HilaireFernandes 10/25/2010 09:37'!
parsePointFree: aNode
	^ self parseItem: aNode with: DrGFreePointBuilder 
		! !

!DrGeoXml methodsFor: 'parse item' stamp: 'HilaireFernandes 10/25/2010 11:37'!
parsePointHomothety: aNode
	^ self parseItem: aNode with: DrGHomothetyBuilder ! !

!DrGeoXml methodsFor: 'parse item' stamp: 'HilaireFernandes 11/1/2011 18:29'!
parsePointIntersection: aNode
	^ self parseItem: aNode with: DrGPointIntersectionBuilder! !

!DrGeoXml methodsFor: 'parse item' stamp: 'HilaireFernandes 4/21/2010 15:52'!
parsePointMiddle2pts: aNode
^ self parseItem: aNode with: DrGMiddleBuilder.! !

!DrGeoXml methodsFor: 'parse item' stamp: 'HilaireFernandes 4/21/2010 15:52'!
parsePointMiddlesegment: aNode
	^self parseItem: aNode with: DrGMiddleBuilder.! !

!DrGeoXml methodsFor: 'parse item' stamp: 'HilaireFernandes 10/25/2010 09:38'!
parsePointOncurve: aNode
	^ self parseItem: aNode with: DrGFreePointBuilder ! !

!DrGeoXml methodsFor: 'parse item' stamp: 'HilaireFernandes 4/21/2010 15:52'!
parsePointReflexion: aNode 
	^ self parseReflexion:  aNode! !

!DrGeoXml methodsFor: 'parse item' stamp: 'HilaireFernandes 10/25/2010 11:10'!
parsePointRotation: aNode
	^ self parseItem: aNode with: DrGRotationBuilder! !

!DrGeoXml methodsFor: 'parse item' stamp: 'HilaireFernandes 10/16/2010 09:48'!
parsePointScale: aNode
"Dr.Geo 1.1 compatibility"
	^ self parsePointHomothety: aNode ! !

!DrGeoXml methodsFor: 'parse item' stamp: 'HilaireFernandes 11/8/2010 01:32'!
parsePointScript: aNode
	^ self parseItem: aNode with: DrGPointByCoordinatesBuilder.! !

!DrGeoXml methodsFor: 'parse item' stamp: 'HilaireFernandes 10/25/2010 11:11'!
parsePointSymmetry: aNode 
	^ self parseItem: aNode with: DrGSymmetryBuilder! !

!DrGeoXml methodsFor: 'parse item' stamp: 'HilaireFernandes 4/21/2010 15:53'!
parsePointTranslation: aNode
	^self parseTranslation: aNode ! !

!DrGeoXml methodsFor: 'parse item' stamp: 'HilaireFernandes 10/16/2010 09:45'!
parsePolygonHomothety: aNode 
	^self parseHomothety: aNode ! !

!DrGeoXml methodsFor: 'parse item' stamp: 'HilaireFernandes 10/25/2010 11:17'!
parsePolygonNpts: aNode
	^ self parseItem: aNode with: DrGPolygonBuilder! !

!DrGeoXml methodsFor: 'parse item' stamp: 'HilaireFernandes 4/21/2010 15:53'!
parsePolygonReflexion: aNode
	^ self parseReflexion: aNode ! !

!DrGeoXml methodsFor: 'parse item' stamp: 'HilaireFernandes 4/21/2010 15:53'!
parsePolygonRotation: aNode
	^ self parseRotation: aNode ! !

!DrGeoXml methodsFor: 'parse item' stamp: 'HilaireFernandes 10/16/2010 09:49'!
parsePolygonScale: aNode 
"Dr.Geo 1.1 compatibility"
	^self parsePolygonHomothety: aNode ! !

!DrGeoXml methodsFor: 'parse item' stamp: 'HilaireFernandes 4/21/2010 15:53'!
parsePolygonSymmetry: aNode 
	^self parseSymmetry: aNode! !

!DrGeoXml methodsFor: 'parse item' stamp: 'HilaireFernandes 4/21/2010 15:53'!
parsePolygonTranslation: aNode
	^self parseTranslation: aNode ! !

!DrGeoXml methodsFor: 'parse item' stamp: 'HilaireFernandes 10/16/2010 09:58'!
parseRay2pts: aNode
	^self parseItem: aNode with: DrGRayBuilder.! !

!DrGeoXml methodsFor: 'parse item' stamp: 'HilaireFernandes 10/16/2010 09:45'!
parseRayHomothety: aNode 
	^self parseHomothety: aNode ! !

!DrGeoXml methodsFor: 'parse item' stamp: 'HilaireFernandes 10/16/2010 09:59'!
parseRayReflexion: aNode
	^self parseReflexion: aNode! !

!DrGeoXml methodsFor: 'parse item' stamp: 'HilaireFernandes 10/16/2010 10:00'!
parseRayRotation: aNode
	^self parseRotation: aNode! !

!DrGeoXml methodsFor: 'parse item' stamp: 'HilaireFernandes 10/16/2010 10:00'!
parseRaySymmetry: aNode 
	^self parseSymmetry: aNode! !

!DrGeoXml methodsFor: 'parse item' stamp: 'HilaireFernandes 10/16/2010 10:01'!
parseRayTranslation:aNode
	^ self parseTranslation: aNode ! !

!DrGeoXml methodsFor: 'parse item' stamp: 'HilaireFernandes 4/21/2010 15:53'!
parseReflexion: aNode 
	^ self parseItem: aNode with: DrGReflectionBuilder! !

!DrGeoXml methodsFor: 'parse item' stamp: 'HilaireFernandes 4/21/2010 15:53'!
parseRotation: aNode 
	^ self parseItem: aNode with: DrGRotationBuilder! !

!DrGeoXml methodsFor: 'xml read' stamp: 'Toto 11/13/2010 16:12'!
parseScript: drgeoNode
"First do one pass to compile all the script Smalltalk source code"
	drgeoNode elementsDo: [: node |
		(node tag = #value and: [(node attributeAt: #type) = #script]) ifTrue:
			[Author 
				useAuthor:  'DrGeoUser'
				during:  [DrGeoUserScripts
					compile: (node elementAt: #code) contentString
					classified: 'scripts']]]! !

!DrGeoXml methodsFor: 'parse item' stamp: 'HilaireFernandes 2/4/2011 16:00'!
parseScriptNitems: aNode
	"Do nothing!!"! !

!DrGeoXml methodsFor: 'parse item' stamp: 'HilaireFernandes 4/21/2010 15:53'!
parseSegment2pts: aNode 
	^ self parseItem: aNode with: DrGSegmentBuilder! !

!DrGeoXml methodsFor: 'parse item' stamp: 'HilaireFernandes 10/16/2010 09:44'!
parseSegmentHomothety: aNode
	^self parseHomothety: aNode ! !

!DrGeoXml methodsFor: 'parse item' stamp: 'HilaireFernandes 4/21/2010 15:53'!
parseSegmentReflexion: aNode
	^ self parseReflexion: aNode! !

!DrGeoXml methodsFor: 'parse item' stamp: 'HilaireFernandes 4/21/2010 15:53'!
parseSegmentRotation: aNode
	^self parseRotation: aNode! !

!DrGeoXml methodsFor: 'parse item' stamp: 'HilaireFernandes 10/16/2010 09:46'!
parseSegmentScale: aNode
"Dr.Geo 1.1 compatibility"
	^self parseSegmentHomothety: aNode ! !

!DrGeoXml methodsFor: 'parse item' stamp: 'HilaireFernandes 4/21/2010 15:53'!
parseSegmentSymmetry: aNode
	^self parseSymmetry: aNode ! !

!DrGeoXml methodsFor: 'parse item' stamp: 'HilaireFernandes 4/21/2010 15:53'!
parseSegmentTranslation: aNode
	^self parseTranslation: aNode ! !

!DrGeoXml methodsFor: 'parse style' stamp: 'HilaireFernandes 10/31/2010 22:07'!
parseStyleOf: costume with: aNode 
	| attribute |
	costume textMorph: (aNode attributeAt: #name).
	costume textPositionDelta: (Point readFromString: (aNode attributeAt: #delta ifAbsent: ['7@7'])).
	(aNode attributeAt: #masked) = 'True'
		ifTrue: [costume style isHidden: true].
	attribute := (aNode attributeAt: #color) asLowercase.
	attribute = 'darkgrey'
		ifTrue: [attribute := #darkGray].
	attribute = 'grey'
		ifTrue: [attribute := #gray].
	attribute = 'darkgreen'
		ifTrue: [attribute := #green].
	attribute = 'darkblue'
		ifTrue: [attribute := #blue].
	attribute = 'bordeaux'
		ifTrue: [attribute := #red].
	costume style
		color: (Color perform: attribute asSymbol)! !

!DrGeoXml methodsFor: 'parse style' stamp: 'HilaireFernandes 4/21/2010 17:50'!
parseStyleOfAngle: item with: aNode 
	self parseStyleOfNumeric: item  with: aNode ! !

!DrGeoXml methodsFor: 'parse style' stamp: 'HilaireFernandes 11/16/2011 12:48'!
parseStyleOfArc: item with: aNode 
	|costume|
	costume := self parseStyleOfCurve: item with: aNode.
	self parseStyleOfFilled: costume with: aNode! !

!DrGeoXml methodsFor: 'parse style' stamp: 'HilaireFernandes 11/16/2011 12:38'!
parseStyleOfArcCircle: item with: aNode 
	self parseStyleOfArc: item with: aNode! !

!DrGeoXml methodsFor: 'parse style' stamp: 'HilaireFernandes 11/16/2011 12:48'!
parseStyleOfCircle: item with: aNode 
	|costume|
	costume := self parseStyleOfCurve: item with: aNode.
	self parseStyleOfFilled: costume with: aNode! !

!DrGeoXml methodsFor: 'parse style' stamp: 'HilaireFernandes 10/28/2011 18:05'!
parseStyleOfCurve: item with: aNode 
	| costume attribute |
	costume := app fastLastCostume.
	self parseStyleOf: costume with: aNode.
	costume anchorAbscissa: (aNode attributeAt: #anchor ifAbsent: ['0.5']) asNumber.
	attribute := aNode attributeAt: #thickness.
	attribute = 'Dashed' ifTrue: [costume style lineStyle: #dash].
	attribute = 'Thick' ifTrue: [costume style thickness: #medium].
	attribute := aNode attributeAt: #style ifAbsent: [^ costume].
	costume style lineStyle: attribute.
	^ costume! !

!DrGeoXml methodsFor: 'parse style' stamp: 'HilaireFernandes 11/16/2011 12:37'!
parseStyleOfFilled: costume with: aNode 
	| attribute |
	(aNode attributeAt: #filled) = 'True'
		ifTrue: [costume style isFilled: true].
	(aNode attributeAt: #translucent) = 'True'
		ifTrue: [costume style isTranslucent: true].
! !

!DrGeoXml methodsFor: 'parse style' stamp: 'HilaireFernandes 11/1/2010 21:56'!
parseStyleOfHalfLine: item with: aNode 
"DrGeo 1.1 xml compatibility"
	self parseStyleOfRay: item with: aNode! !

!DrGeoXml methodsFor: 'parse style' stamp: 'HilaireFernandes 4/21/2010 17:51'!
parseStyleOfLine: item with: aNode 
	self parseStyleOfCurve: item with: aNode! !

!DrGeoXml methodsFor: 'parse style' stamp: 'HilaireFernandes 4/21/2010 17:51'!
parseStyleOfLocus: item with: aNode 
	self parseStyleOfCurve: item with: aNode! !

!DrGeoXml methodsFor: 'parse style' stamp: 'HilaireFernandes 10/31/2010 14:20'!
parseStyleOfNumeric: item with: aNode 
 "DrGeo 1.1 xml compatibility"
	self parseStyleOfValue: item with: aNode! !

!DrGeoXml methodsFor: 'parse style' stamp: 'HilaireFernandes 10/28/2011 18:05'!
parseStyleOfPoint: item with: aNode 
	| costume attribute |
	costume := app fastLastCostume.
	self parseStyleOf: costume with: aNode.
	attribute := aNode attributeAt: #thickness.
	attribute = 'Dashed'
		ifTrue: [attribute := #small].
	attribute = 'Normal'
		ifTrue: [attribute := #medium].
	attribute = 'Thick'
		ifTrue: [attribute := #large].
	costume style thickness: attribute.
	attribute := aNode attributeAt: #style.
	(#('Rectangular' 'RectangularEmpty' ) includes: attribute)
		ifTrue: [attribute := #square].
	attribute = 'RoundEmpty'
		ifTrue: [attribute := #round].
	costume style shape: attribute asLowercase asSymbol! !

!DrGeoXml methodsFor: 'parse style' stamp: 'HilaireFernandes 4/21/2010 17:51'!
parseStyleOfPolygon: item with: aNode 
	self parseStyleOfCurve: item with: aNode! !

!DrGeoXml methodsFor: 'parse style' stamp: 'HilaireFernandes 11/1/2010 21:56'!
parseStyleOfRay: item with: aNode 
	self parseStyleOfCurve: item with: aNode! !

!DrGeoXml methodsFor: 'parse style' stamp: 'HilaireFernandes 11/16/2011 21:47'!
parseStyleOfSegment: item with: aNode 
	| costume |
	costume := self parseStyleOfCurve: item with: aNode.
	costume style arrow: (aNode attributeAt: #arrow ifAbsent: [#makeNoArrows]) asSymbol.
	costume style mark: (aNode attributeAt: #mark ifAbsent: [#none]) asSymbol! !

!DrGeoXml methodsFor: 'parse style' stamp: 'HilaireFernandes 11/6/2011 21:08'!
parseStyleOfText: item with: aNode 
	| costume |
	costume := app fastLastCostume.
	self parseStyleOf: costume with: aNode.
	costume morph crAction: nil "We don't want on place edit"! !

!DrGeoXml methodsFor: 'parse style' stamp: 'HilaireFernandes 10/28/2011 18:05'!
parseStyleOfValue: item with: aNode 
	| costume |
	costume := app fastLastCostume.
	self parseStyleOf: costume with: aNode.! !

!DrGeoXml methodsFor: 'parse style' stamp: 'HilaireFernandes 11/1/2010 22:13'!
parseStyleOfVector: item with: aNode 
	self parseStyleOfCurve: item with: aNode.
	item origin: (Point readFromString: (aNode attributeAt: #origin ifAbsent: [^self]))! !

!DrGeoXml methodsFor: 'parse item' stamp: 'HilaireFernandes 4/21/2010 15:53'!
parseSymmetry: aNode 
	^ self parseItem: aNode with: DrGSymmetryBuilder! !

!DrGeoXml methodsFor: 'parse item' stamp: 'HilaireFernandes 11/5/2011 15:55'!
parseTextFree: aNode
	^ self parseItem: aNode with: DrGTextBuilder! !

!DrGeoXml methodsFor: 'parse item' stamp: 'HilaireFernandes 4/21/2010 15:53'!
parseTranslation: aNode 
	^ self parseItem: aNode with: DrGTranslationBuilder! !

!DrGeoXml methodsFor: 'parse item' stamp: 'HilaireFernandes 10/16/2010 10:04'!
parseValueArclength: aNode
	^ self parseValueComputation: aNode ! !

!DrGeoXml methodsFor: 'parse item' stamp: 'HilaireFernandes 10/16/2010 10:04'!
parseValueCircleperimeter: aNode
	^ self parseValueComputation: aNode ! !

!DrGeoXml methodsFor: 'parse item' stamp: 'HilaireFernandes 10/25/2010 11:19'!
parseValueComputation: aNode
	^ self parseItem: aNode with: DrGValueBuilder.! !

!DrGeoXml methodsFor: 'parse item' stamp: 'HilaireFernandes 10/16/2010 10:04'!
parseValueDistance2pts: aNode
	^ self parseValueComputation: aNode ! !

!DrGeoXml methodsFor: 'parse item' stamp: 'HilaireFernandes 10/16/2010 10:04'!
parseValueDistanceptline: aNode
	^ self parseValueComputation: aNode ! !

!DrGeoXml methodsFor: 'parse item' stamp: 'HilaireFernandes 11/5/2010 22:18'!
parseValueScript: aNode
	^ self parseItem: aNode with: DrGScriptPlayer.! !

!DrGeoXml methodsFor: 'parse item' stamp: 'HilaireFernandes 10/16/2010 10:06'!
parseValueSegmentlength: aNode
	^ self parseValueComputation: aNode ! !

!DrGeoXml methodsFor: 'parse item' stamp: 'HilaireFernandes 10/16/2010 10:06'!
parseValueSlope: aNode
	^ self parseValueComputation: aNode ! !

!DrGeoXml methodsFor: 'parse item' stamp: 'HilaireFernandes 10/25/2010 11:35'!
parseValueValue: aNode
	^ self parseValueComputation: aNode ! !

!DrGeoXml methodsFor: 'parse item' stamp: 'HilaireFernandes 10/16/2010 10:07'!
parseValueVectornorm: aNode
	^ self parseValueComputation: aNode ! !

!DrGeoXml methodsFor: 'parse item' stamp: 'HilaireFernandes 4/21/2010 15:54'!
parseVector2pts: aNode
	^self parseItem: aNode with: DrGVectorBuilder.! !

!DrGeoXml methodsFor: 'parse item' stamp: 'HilaireFernandes 10/16/2010 09:44'!
parseVectorHomothety: aNode 
	^self parseHomothety: aNode ! !

!DrGeoXml methodsFor: 'parse item' stamp: 'HilaireFernandes 4/21/2010 15:54'!
parseVectorReflexion: aNode
	^self parseReflexion: aNode! !

!DrGeoXml methodsFor: 'parse item' stamp: 'HilaireFernandes 4/21/2010 15:54'!
parseVectorRotation: aNode
	^self parseRotation: aNode! !

!DrGeoXml methodsFor: 'parse item' stamp: 'HilaireFernandes 10/16/2010 09:49'!
parseVectorScale: aNode 
"Dr.Geo 1.1 compatibility"
	^self parseVectorHomothety: aNode ! !

!DrGeoXml methodsFor: 'parse item' stamp: 'HilaireFernandes 4/21/2010 15:54'!
parseVectorSymmetry: aNode 
	^ self parseSymmetry: aNode! !

!DrGeoXml methodsFor: 'accessing' stamp: 'HilaireFernandes 4/21/2010 15:43'!
presenter
	^ app presenter! !

!DrGeoXml methodsFor: 'xml write' stamp: 'HilaireFernandes 4/26/2010 13:13'!
saveAs: filename
	self saveOn: (FileStream fileNamed: filename).! !

!DrGeoXml methodsFor: 'xml write' stamp: 'HilaireFernandes 10/9/2010 22:22'!
saveMultiple: sketchesAndMacros on: stream
	| doc writer root |
	doc := XMLDocument new version: '1.0'.
	writer := XMLWriter on: stream.
	root := XMLElement named: #drgenius.
	(sketchesAndMacros select: [:each | each class = DrGeo]) do: [:figure |
		self app: figure.
		self writeFigureAsXmlTo: root.].
	(sketchesAndMacros select: [:each | each class = DrGMacro]) do: [:macro |
		macro writeAsXmlTo: root].
	doc addElement: root.
	doc printXMLOn: writer.
	stream close! !

!DrGeoXml methodsFor: 'xml write' stamp: 'HilaireFernandes 8/5/2011 17:18'!
saveOn: stream
	| doc writer root |
	doc := XMLDocument new version: '1.0'.
	writer := XMLWriter on: stream.
	root := XMLElement named: #drgenius.
	self writeFigureAsXmlTo: root.
	doc addElement: root.
	doc printXMLOn: writer.
	stream close! !

!DrGeoXml methodsFor: 'accessing' stamp: 'HilaireFernandes 4/21/2010 15:42'!
scale
	 ^ self area scale! !

!DrGeoXml methodsFor: 'accessing' stamp: 'HilaireFernandes 4/21/2010 15:41'!
setScaleTo: aNumber
	self area setScaleTo: aNumber! !

!DrGeoXml methodsFor: 'xml write' stamp: 'HilaireFernandes 10/10/2010 09:05'!
writeFigureAsXmlTo: aNode 
	| drgeoNode attr |
	attr := Dictionary new.
	attr at: #scale put: self scale asString;
		 at: #'origin_x' put: self ox asString;
		 at: #'origin_y' put: self oy asString;
		 at: #grid put: self presenter isGridVisible asString capitalized;
		at: #name put: app title.
	drgeoNode := XMLElement named: #drgeo attributes: attr.
	app costumes do: [:each | each writeAsXmlTo: drgeoNode].
	aNode addElement: drgeoNode! !

!Object methodsFor: '*DrGeoII-Core-testing' stamp: 'HilaireFernandes 2/4/2006 09:51'!
isAngleItem
	^false! !

!Object methodsFor: '*DrGeoII-Core-testing' stamp: 'HilaireFernandes 1/16/2006 21:17'!
isArcItem
	^false! !

!Object methodsFor: '*DrGeoII-Core-testing' stamp: 'HilaireFernandes 1/2/2006 16:17'!
isCircleItem
	^false! !

!Object methodsFor: '*DrGeoII-Core-testing' stamp: 'HilaireFernandes 3/1/2006 10:01'!
isCompositeItem
	^false! !

!Object methodsFor: '*DrGeoII-Core-testing' stamp: 'HilaireFernandes 11/13/2007 20:27'!
isConstrainedPointItem
	^false! !

!Object methodsFor: '*DrGeoII-Core-testing' stamp: 'HilaireFernandes 1/2/2006 16:17'!
isCurveItem
	^false! !

!Object methodsFor: '*DrGeoII-Core-testing' stamp: 'HilaireFernandes 1/2/2006 16:17'!
isDirectionItem
	^false! !

!Object methodsFor: '*DrGeoII-Core-testing' stamp: 'HilaireFernandes 3/10/2006 19:24'!
isFreePointItem
	^false! !

!Object methodsFor: '*DrGeoII-Core-testing' stamp: 'HilaireFernandes 1/3/2006 10:53'!
isGeometricItem
	^false! !

!Object methodsFor: '*DrGeoII-Core-testing' stamp: 'HilaireFernandes 3/10/2006 19:26'!
isIntersectionPointItem
	^false! !

!Object methodsFor: '*DrGeoII-Core-testing' stamp: 'HilaireFernandes 1/2/2006 16:17'!
isLineItem
	^false! !

!Object methodsFor: '*DrGeoII-Core-testing' stamp: 'HilaireFernandes 1/25/2008 22:59'!
isLocusItem
	^false! !

!Object methodsFor: '*DrGeoII-Core-testing' stamp: 'HilaireFernandes 1/2/2006 18:24'!
isNumberItem
	^false! !

!Object methodsFor: '*DrGeoII-Core-testing' stamp: 'HilaireFernandes 1/2/2006 18:24'!
isPointItem
	^false! !

!Object methodsFor: '*DrGeoII-Core-testing' stamp: 'HilaireFernandes 3/10/2006 19:25'!
isPointItemOnCurve
	^false! !

!Object methodsFor: '*DrGeoII-Core-testing' stamp: 'HilaireFernandes 2/6/2006 21:16'!
isPolygonItem
	^false! !

!Object methodsFor: '*DrGeoII-Core-testing' stamp: 'HilaireFernandes 1/2/2006 16:17'!
isRayItem
	^false! !

!Object methodsFor: '*DrGeoII-Core-testing' stamp: 'HilaireFernandes 8/3/2008 18:29'!
isScriptItem
	^false! !

!Object methodsFor: '*DrGeoII-Core-testing' stamp: 'HilaireFernandes 1/2/2006 16:17'!
isSegmentItem
	^false! !

!Object methodsFor: '*DrGeoII-Core-testing' stamp: 'HilaireFernandes 11/5/2011 14:35'!
isTextItem
	^false! !

!Object methodsFor: '*DrGeoII-Core-testing' stamp: 'HilaireFernandes 2/2/2006 11:18'!
isValueItem
	^false! !

!Object methodsFor: '*DrGeoII-Core-testing' stamp: 'HilaireFernandes 1/2/2006 16:17'!
isVectorItem
	^false! !

BorderedMorph subclass: #DrGMorph
	instanceVariableNames: 'costume blink'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'DrGeoII-Core-Item-View'!

DrGMorph subclass: #DrGCircleMorph
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'DrGeoII-Core-Item-View'!

!DrGCircleMorph methodsFor: 'accessing' stamp: 'HilaireFernandes 1/2/2006 14:23'!
center: aPoint radius: aFloat
	self bounds: (aPoint - aFloat corner: aPoint + aFloat)! !

!DrGCircleMorph methodsFor: 'geometry testing' stamp: 'HilaireFernandes 6/29/2011 08:43'!
containsPoint: aPoint
	"it is a circle so height and width is the diameter"
	(bounds containsPoint: aPoint) ifFalse: [^false].
	^((self center dist: aPoint) - (self height / 2)) abs < 15! !

!DrGCircleMorph methodsFor: 'as yet unclassified' stamp: 'HilaireFernandes 1/21/2006 15:51'!
dashedBorder: dashSpec! !

!DrGCircleMorph methodsFor: 'drawing' stamp: 'HilaireFernandes 11/16/2011 10:49'!
drawOn: aCanvas
	costume style isFilled 
		ifTrue: [
			aCanvas 
				fillOval: self bounds 
				color: (costume style isTranslucent ifTrue: [self color alpha: 0.3] ifFalse: [self color])]
	ifFalse:	[aCanvas frameOval: self bounds width: self borderWidth color: self color]! !

!DrGMorph methodsFor: 'accessing' stamp: 'HilaireFernandes 12/31/2005 10:30'!
costume
	^costume! !

!DrGMorph methodsFor: 'accessing' stamp: 'HilaireFernandes 12/31/2005 10:30'!
costume: aCostume
	costume := aCostume! !

!DrGMorph methodsFor: 'event handling' stamp: 'HilaireFernandes 12/15/2005 15:43'!
handlesMouseDown: evt
	^ false! !

!DrGMorph methodsFor: 'initialization' stamp: 'HilaireFernandes 1/11/2006 10:41'!
initialize
	super initialize.
	blink := false.
! !

!DrGMorph methodsFor: '*DrGeoII-Core-testing' stamp: 'HilaireFernandes 3/9/2006 17:08'!
isMorphCostume
	^true! !

!DrGMorph methodsFor: 'stepping and presenter' stamp: 'HilaireFernandes 12/30/2005 12:14'!
startBlinking
	blink := true.
	self startStepping ! !

!DrGMorph methodsFor: 'stepping and presenter' stamp: 'HilaireFernandes 1/24/2006 23:02'!
step
	blink ifFalse: [^self].
	super step.
	self color = DrGCostumeStyle blinkingColor 
		ifTrue: [self color: costume style color]
		ifFalse: [self color: DrGCostumeStyle blinkingColor]! !

!DrGMorph methodsFor: 'stepping and presenter' stamp: 'HilaireFernandes 1/21/2006 11:45'!
stepTime
	^500! !

!DrGMorph methodsFor: 'stepping and presenter' stamp: 'HilaireFernandes 1/20/2006 16:01'!
stopBlinking
	blink := false.
	self stopStepping.! !

!DrGMorph methodsFor: 'stepping and presenter' stamp: 'HilaireFernandes 1/21/2006 10:41'!
stopStepping
	super stopStepping.
	self color: costume style color.! !

DrGMorph subclass: #DrGPointMorph
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'DrGeoII-Core-Item-View'!

!DrGPointMorph methodsFor: 'geometry testing' stamp: 'HilaireFernandes 6/29/2011 08:43'!
containsPoint: aPoint
	^(self bounds expandBy: 10) containsPoint: aPoint! !

!DrGPointMorph methodsFor: 'drawing' stamp: 'HilaireFernandes 1/21/2006 14:34'!
drawOn: aCanvas
|shape|
	shape := costume style shape.
	shape = #square ifTrue:
		[aCanvas fillRectangle: self bounds color: self color].
	shape = #round ifTrue:
		[aCanvas fillOval: self bounds color: self color].
	shape = #cross ifTrue:
		[aCanvas 
			line: self topLeft + (1@1) 
			to: self bottomRight - (1@1) 
			width: 2 
			color: self color.
		aCanvas 
			line: self topRight + (-1@1) 
			to: self bottomLeft - (-1@1) 
			width: 2	
			color: self color]! !

!DrGPointMorph methodsFor: 'initializing' stamp: 'HilaireFernandes 3/13/2006 15:32'!
initialize
	super initialize.
	self extent: 3@3! !

BorderedMorph subclass: #DrGService
	instanceVariableNames: 'app area'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'DrGeoII-Core-App'!

!DrGService class methodsFor: 'parts bin' stamp: 'HilaireFernandes 5/13/2010 20:38'!
descriptionForPartsBin
	^ (DescriptionForPartsBin
		formalName: 'Dr. Geo'
		categoryList: #('Graphics' 'Tools' )
		documentation: 'An interactive geometry canvas to draw dynamic geometric sketch. Open the Morph menu to get access to the construction tools and more.' translated
		globalReceiverSymbol: #DrGeo
		nativitySelector: #sample)
		sampleImageForm: DrGIcons angle! !

!DrGService class methodsFor: 'instance creation' stamp: 'HilaireFernandes 4/19/2010 21:55'!
in: aDrgeo
	^ self basicNew app: aDrgeo;
		initialize! !

!DrGService class methodsFor: 'instance creation' stamp: 'HilaireFernandes 6/18/2007 17:09'!
with: aDrgeoModel 
	^ self basicNew initialize: aDrgeoModel! !

!DrGService methodsFor: 'menu' stamp: 'HilaireFernandes 5/14/2010 16:39'!
add: btnSelector menuItemBtn: array to: menu
	menu 
		add: array first 
		target: self
		selector: #giveMeButton:
		argument: btnSelector.
	menu lastItem 
		icon: (DrGIcons scaled16: array second);
		setBalloonText: array fourth  
		! !

!DrGService methodsFor: 'menu' stamp: 'HilaireFernandes 4/20/2010 07:54'!
addCustomMenuItems: menu hand: aHandMorph 
	"super addCustomMenuItems: menu hand: aHandMorph."
	| submenu |
	menu addLine.
	submenu := MenuMorph new.
	self addToolsMenuItems: submenu hand: aHandMorph.
	menu add: 'Dr. Geo tools' translated subMenu: submenu.
	submenu := MenuMorph new.
	self addGetButtonsMenuItems: submenu hand: aHandMorph.
	menu add: 'Give me Dr. Geo buttons' translated subMenu: submenu! !

!DrGService methodsFor: 'menu' stamp: 'HilaireFernandes 4/20/2010 07:54'!
addGetButtonsMenuItems: menu hand: aHandMorph 
	"from this menu we can ask for buttons to plug arround"
	self addPointBtnMenuItems: menu hand: aHandMorph.
	self addLineBtnMenuItems: menu hand: aHandMorph.
	self addTransformationBtnMenuItems: menu hand: aHandMorph.
	self addNumericBtnMenuItems: menu hand: aHandMorph.
	self addMacroBtnMenuItems: menu hand: aHandMorph.
	self addOtherBtnMenuItems: menu hand: aHandMorph.! !

!DrGService methodsFor: 'menu' stamp: 'HilaireFernandes 5/13/2010 10:39'!
addLineBtnMenuItems: menu hand: aHandMorph 
	| subMenu |
	subMenu := MenuMorph new.
	self add: #lineBtn menuItemBtn: self drgeoPresenter lineButtonData to: subMenu.
	self add: #parallelBtn menuItemBtn: self drgeoPresenter parallelButtonData to: subMenu.
	self add: #perpendicularBtn menuItemBtn: self drgeoPresenter perpendicularButtonData to: subMenu.
	self add: #rayBtn menuItemBtn: self drgeoPresenter rayButtonData to: subMenu.
	self add: #segmentBtn menuItemBtn: self drgeoPresenter segmentButtonData to: subMenu.
	self add: #vectorBtn menuItemBtn: self drgeoPresenter vectorButtonData to: subMenu.
	self add: #circleBtn menuItemBtn: self drgeoPresenter circleButtonData to: subMenu.
	self add: #arcBtn menuItemBtn: self drgeoPresenter arcButtonData to: subMenu.	
	self add: #polygonBtn menuItemBtn: self drgeoPresenter polygonButtonData to: subMenu.	
	self add: #locusBtn menuItemBtn: self drgeoPresenter locusButtonData to: subMenu.
	menu add: ('Curves' translated) subMenu: subMenu! !

!DrGService methodsFor: 'menu' stamp: 'HilaireFernandes 5/13/2010 10:39'!
addLineMenuItems: menu hand: aHandMorph 
	| subMenu |
	subMenu := MenuMorph new.
	self addMenuItem: self drgeoPresenter lineButtonData to: subMenu.
	self addMenuItem: self drgeoPresenter parallelButtonData to: 	subMenu.
	self addMenuItem: self drgeoPresenter perpendicularButtonData to: subMenu.
	self addMenuItem: self drgeoPresenter rayButtonData to: subMenu.
	self addMenuItem: self drgeoPresenter segmentButtonData to: subMenu.
	self addMenuItem: self drgeoPresenter vectorButtonData to: subMenu.
	self addMenuItem: self drgeoPresenter circleButtonData to: subMenu.
	self addMenuItem: self drgeoPresenter arcButtonData to: subMenu.
	self addMenuItem: self drgeoPresenter polygonButtonData to: subMenu.
	self addMenuItem: self drgeoPresenter locusButtonData to: subMenu.
	menu add: 'Curves' translated subMenu: subMenu! !

!DrGService methodsFor: 'menu' stamp: 'HilaireFernandes 5/13/2010 10:39'!
addMacroBtnMenuItems: menu hand: aHandMorph 
	| subMenu |
	subMenu := MenuMorph new.
	self add: #macroBuildBtn menuItemBtn: self drgeoPresenter macroBuildButtonData to: subMenu.
	self add: #executeBtn menuItemBtn: self drgeoPresenter macroExecuteButtonData to: subMenu.
	menu add: 'Macro-construction' translated subMenu: subMenu! !

!DrGService methodsFor: 'menu' stamp: 'HilaireFernandes 5/13/2010 10:40'!
addMacroMenuItems: menu hand: aHandMorph 
	| subMenu |
	subMenu := MenuMorph new.
	self addMenuItem: self drgeoPresenter macroBuildButtonData to: subMenu.
	self addMenuItem: self drgeoPresenter macroExecuteButtonData to: subMenu.
	self addMenuItem: self drgeoPresenter macroDeleteButtonData to: subMenu.
	menu add: 'Macro-construction' translated subMenu: subMenu! !

!DrGService methodsFor: 'menu' stamp: 'HilaireFernandes 5/14/2010 16:39'!
addMenuItem: array to: menu
	menu 
		add: array first 
		target: self drgeoPresenter
		selector: array third.
	menu lastItem
		setBalloonText: array fourth  
		! !

!DrGService methodsFor: 'menu' stamp: 'HilaireFernandes 5/13/2010 10:40'!
addNumericBtnMenuItems: menu hand: aHandMorph 
	| subMenu |
	subMenu := MenuMorph new.
	self add: #distanceBtn menuItemBtn: self drgeoPresenter distanceButtonData to: subMenu.
	self add: #angleBtn menuItemBtn: self drgeoPresenter angleButtonData to: subMenu.
	self add: #coordinatesBtn menuItemBtn: self drgeoPresenter coordinatesButtonData to: subMenu.
	self add: #scriptBtn menuItemBtn: self drgeoPresenter scriptButtonData to: subMenu.
	self add: #scriptEditBtn menuItemBtn: self drgeoPresenter scriptEditButtonData to: subMenu.
	menu add: 'Numerics' translated subMenu: subMenu! !

!DrGService methodsFor: 'menu' stamp: 'HilaireFernandes 5/13/2010 10:40'!
addNumericMenuItems: menu hand: aHandMorph 
	| subMenu |
	subMenu := MenuMorph new.
	self addMenuItem: self drgeoPresenter distanceButtonData to: subMenu.
	self addMenuItem: self drgeoPresenter angleButtonData to: subMenu.
	self addMenuItem: self drgeoPresenter coordinatesButtonData to: subMenu.
	self addMenuItem: self drgeoPresenter scriptEditButtonData to: subMenu.
	self addMenuItem: self drgeoPresenter scriptButtonData to: subMenu.
	menu add: 'Numerics' translated subMenu: subMenu! !

!DrGService methodsFor: 'menu' stamp: 'HilaireFernandes 5/13/2010 10:40'!
addOtherBtnMenuItems: menu hand: aHandMorph 
	| subMenu |
	subMenu := MenuMorph new.
	self add: #undoBtn menuItemBtn: self drgeoPresenter undoButtonData to: subMenu.
	self add: #redoBtn menuItemBtn: self drgeoPresenter redoButtonData to: subMenu.
	self add: #eraserBtn menuItemBtn: self drgeoPresenter eraserButtonData to: subMenu.
	self add: #styleBtn menuItemBtn: self drgeoPresenter styleButtonData to: subMenu.
	self add: #propertyBtn menuItemBtn: self drgeoPresenter propertyButtonData to: subMenu.
	self add: #moveBtn menuItemBtn: self drgeoPresenter moveButtonData to: subMenu.		
	self add: #gridBtn menuItemBtn: self drgeoPresenter gridButtonData to: subMenu.
	self add: #gridStuckBtn menuItemBtn: self drgeoPresenter gridStuckButtonData to: subMenu.
	self add: #viewerBtn menuItemBtn: self drgeoPresenter viewerButtonData to: subMenu.
	menu add: 'Other' translated subMenu: subMenu! !

!DrGService methodsFor: 'menu' stamp: 'HilaireFernandes 5/13/2010 10:41'!
addOtherMenuItems: menu hand: aHandMorph 
	| subMenu |
	subMenu := MenuMorph new.
	self addMenuItem: self drgeoPresenter undoButtonData to: subMenu.
	self addMenuItem: self drgeoPresenter redoButtonData to: subMenu.
	self addMenuItem: self drgeoPresenter eraserButtonData to: subMenu.
	self addMenuItem: self drgeoPresenter styleButtonData to: subMenu.
	self addMenuItem: self drgeoPresenter propertyButtonData to: subMenu.
	self addMenuItem: self drgeoPresenter moveButtonData to: subMenu.
	subMenu 
		addUpdating: #gridString
		target: self drgeoPresenter
		selector: #gridOnOff
		argumentList: #();
		
		addUpdating: #griddedString
		target: self drgeoPresenter
		selector: #gridStuckOnOff
		argumentList: #().
	self addMenuItem: self drgeoPresenter viewerButtonData to: subMenu.
	self addMenuItem: self drgeoPresenter multipleModeButtonData to: subMenu.
	subMenu 		
		addUpdating: #createMultipleString
		target: self drgeoPresenter
		selector: #createMultipleOnOff
		argumentList: #();
		
		add: 'default position and scale' translated
		target: self area
		selector: #defaultScalePosition.
	menu add: 'Other' translated subMenu: subMenu! !

!DrGService methodsFor: 'menu' stamp: 'HilaireFernandes 10/30/2011 16:49'!
addPointBtnMenuItems: menu hand: aHandMorph 
	| subMenu |
	subMenu := MenuMorph new.
	self add: #freePointBtn menuItemBtn: self drgeoPresenter freePointButtonData to: subMenu.
	self add: #middlePointBtn menuItemBtn: self drgeoPresenter middleButtonData to: subMenu.	
	self add: #intersectionPointBtn menuItemBtn: self drgeoPresenter intersectionButtonData to: subMenu.	
	self add: #pointByCoordinatesBtn menuItemBtn: self drgeoPresenter pointByCoordinatesButtonData to: subMenu.
	menu add: 'Points' translated subMenu: subMenu! !

!DrGService methodsFor: 'menu' stamp: 'HilaireFernandes 5/13/2010 10:41'!
addPointMenuItems: menu hand: aHandMorph 
	| subMenu |
	subMenu := MenuMorph new.
	self addMenuItem: self drgeoPresenter freePointButtonData to: subMenu.
	self addMenuItem: self drgeoPresenter middleButtonData to: subMenu.
	self addMenuItem: self drgeoPresenter pointByCoordinatesButtonData to: subMenu.
	menu add: 'Points' translated subMenu: subMenu! !

!DrGService methodsFor: 'menu' stamp: 'HilaireFernandes 4/20/2010 07:55'!
addToolsMenuItems: menu hand: aHandMorph 
	self addPointMenuItems: menu hand: aHandMorph.
	self addLineMenuItems: menu hand: aHandMorph.
	self addTransformationMenuItems: menu hand: aHandMorph.
	self addNumericMenuItems: menu hand: aHandMorph.
	self addMacroMenuItems: menu hand: aHandMorph.
	self addOtherMenuItems: menu hand: aHandMorph! !

!DrGService methodsFor: 'menu' stamp: 'HilaireFernandes 5/13/2010 10:41'!
addTransformationBtnMenuItems: menu hand: aHandMorph 
	| subMenu |
	subMenu := MenuMorph new.
	self add: #symmetryBtn menuItemBtn: self drgeoPresenter symmetryButtonData to: subMenu.
	self add: #reflectionBtn menuItemBtn: self drgeoPresenter reflectionButtonData to: subMenu.
	self add: #translationBtn menuItemBtn: self drgeoPresenter translationButtonData to: subMenu.
	self add: #rotationBtn menuItemBtn: self drgeoPresenter rotationButtonData to: subMenu.
	self add: #homothetyBtn menuItemBtn: self drgeoPresenter homothetyButtonData to: subMenu.
	menu add: 'Transformations' translated subMenu: subMenu! !

!DrGService methodsFor: 'menu' stamp: 'HilaireFernandes 5/13/2010 10:41'!
addTransformationMenuItems: menu hand: aHandMorph 
	| subMenu |
	subMenu := MenuMorph new.
	self addMenuItem: self drgeoPresenter symmetryButtonData to: subMenu.
	self addMenuItem: self drgeoPresenter reflectionButtonData to: subMenu.
	self addMenuItem: self drgeoPresenter translationButtonData to: subMenu.
	self addMenuItem: self drgeoPresenter rotationButtonData to: subMenu.
	self addMenuItem: self drgeoPresenter homothetyButtonData to: subMenu.
	menu add: 'Transformations' translated subMenu: subMenu! !

!DrGService methodsFor: 'button' stamp: 'HilaireFernandes 5/13/2010 10:41'!
angleBtn
	^ self buttonAtHand: self drgeoPresenter angleButtonData ! !

!DrGService methodsFor: 'accessing' stamp: 'HilaireFernandes 4/19/2010 21:33'!
app
	^ app! !

!DrGService methodsFor: 'accessing' stamp: 'HilaireFernandes 4/19/2010 21:33'!
app: anObject
	app := anObject! !

!DrGService methodsFor: 'button' stamp: 'HilaireFernandes 5/13/2010 10:41'!
arcBtn
	^ self buttonAtHand: self drgeoPresenter arcButtonData ! !

!DrGService methodsFor: 'accessing' stamp: 'HilaireFernandes 4/19/2010 21:33'!
area
	^ area! !

!DrGService methodsFor: 'button' stamp: 'HilaireFernandes 5/13/2010 10:41'!
buttonAtHand: array
"
array first = button form
array second = symbol callback
array third = help string
"
	^ IconicButton new
		labelGraphic: (DrGIcons scaled24: array second);
		color: self colorButton;
		setBalloonText: array fourth translated;
		target: self drgeoPresenter;
		actionSelector: array third! !

!DrGService methodsFor: 'button' stamp: 'HilaireFernandes 5/13/2010 10:42'!
circleBtn
	^ self buttonAtHand: self drgeoPresenter circleButtonData ! !

!DrGService methodsFor: 'button' stamp: 'HilaireFernandes 4/20/2010 07:56'!
colorButton
	^ Color green alpha: 0.2! !

!DrGService methodsFor: 'button' stamp: 'HilaireFernandes 5/13/2010 10:42'!
coordinatesBtn
	^ self buttonAtHand: self drgeoPresenter coordinatesButtonData ! !

!DrGService methodsFor: 'button' stamp: 'HilaireFernandes 5/13/2010 10:42'!
distanceBtn
	^ self buttonAtHand: self drgeoPresenter distanceButtonData ! !

!DrGService methodsFor: 'accessing' stamp: 'HilaireFernandes 5/13/2010 10:38'!
drgeoPresenter
	^ app presenter! !

!DrGService methodsFor: 'button' stamp: 'HilaireFernandes 5/13/2010 10:42'!
editGroupBtn
	^ self buttonAtHand: self drgeoPresenter editGroupButtonData ! !

!DrGService methodsFor: 'button' stamp: 'HilaireFernandes 5/13/2010 10:42'!
eraserBtn
	^ self buttonAtHand: self drgeoPresenter eraserButtonData ! !

!DrGService methodsFor: 'button' stamp: 'HilaireFernandes 5/13/2010 10:42'!
freePointBtn
	^ self buttonAtHand: self drgeoPresenter freePointButtonData! !

!DrGService methodsFor: 'accessing' stamp: 'HilaireFernandes 8/4/2010 10:00'!
fullScreen
	self position: 0@0.
	self extent: World extent! !

!DrGService methodsFor: 'button' stamp: 'HilaireFernandes 4/20/2010 07:56'!
giveMeButton: aSymbol
	self currentHand attachMorph: (self perform: aSymbol)! !

!DrGService methodsFor: 'button' stamp: 'HilaireFernandes 5/13/2010 10:42'!
gridBtn
	^ self buttonAtHand: self drgeoPresenter gridButtonData ! !

!DrGService methodsFor: 'button' stamp: 'HilaireFernandes 5/13/2010 10:42'!
gridStuckBtn
	^ self buttonAtHand: self drgeoPresenter gridStuckButtonData ! !

!DrGService methodsFor: 'button' stamp: 'HilaireFernandes 5/13/2010 10:42'!
groupBtn
	^ self buttonAtHand: self drgeoPresenter groupButtonData ! !

!DrGService methodsFor: 'button' stamp: 'HilaireFernandes 5/13/2010 10:42'!
homothetyBtn
	^ self buttonAtHand: self drgeoPresenter homothetyButtonData  ! !

!DrGService methodsFor: 'initialize-release' stamp: 'HilaireFernandes 8/8/2010 22:34'!
initialize
	super initialize.
	self color: Color veryLightGray muchLighter .
	self borderWidth: 1.
	self clipSubmorphs: true.
	self layoutPolicy: ProportionalLayout new.
	area := DrGDrawable newOn: app.
	self installTools.
	self extent: 600 @ 400! !

!DrGService methodsFor: 'initialize-release' stamp: 'HilaireFernandes 5/13/2010 10:42'!
installTools 
	self 
		addMorph: area
	 	fullFrame: (LayoutFrame 
			fractions: (0@0 corner: 1@1) 
			offsets: (0@0 corner: -25@ -25)).
	self
		addMorph: self drgeoPresenter verticalWheel
		fullFrame: (LayoutFrame 
			fractions: (1 @ 1 corner: 1 @ 1) 
			offsets: (-25@ -120 corner: 0@ -20)).
	self
		addMorph: self drgeoPresenter horizontalWheel
		fullFrame: (LayoutFrame 
			fractions: (1 @ 1 corner: 1 @ 1) 
			offsets: (-120@ -25 corner: -20@0)).
	self
	 	addMorph: self drgeoPresenter zoomWheel 
		fullFrame: (LayoutFrame 
			fractions: (1 @ 0 corner: 1 @ 0) 
			offsets: (-25@0 corner: 0@100)).
	area layoutChanged! !

!DrGService methodsFor: 'testing' stamp: 'HilaireFernandes 4/19/2010 10:32'!
isApplication
	^false! !

!DrGService methodsFor: 'testing' stamp: 'HilaireFernandes 6/21/2007 14:53'!
isService
	^true! !

!DrGService methodsFor: 'button' stamp: 'HilaireFernandes 5/13/2010 10:42'!
lineBtn
	^ self buttonAtHand: self drgeoPresenter lineButtonData  ! !

!DrGService methodsFor: 'button' stamp: 'HilaireFernandes 5/13/2010 10:42'!
locusBtn
	^ self buttonAtHand: self drgeoPresenter locusButtonData  ! !

!DrGService methodsFor: 'button' stamp: 'HilaireFernandes 5/13/2010 10:42'!
macroBuildBtn
	^ self buttonAtHand: self drgeoPresenter macroBuildButtonData   ! !

!DrGService methodsFor: 'button' stamp: 'HilaireFernandes 5/13/2010 10:42'!
macroDeleteBtn
	^ self buttonAtHand: self drgeoPresenter macroDeleteButtonData    ! !

!DrGService methodsFor: 'button' stamp: 'HilaireFernandes 5/13/2010 10:42'!
macroExecuteBtn
	^ self buttonAtHand: self drgeoPresenter macroExecuteButtonData    ! !

!DrGService methodsFor: 'button' stamp: 'HilaireFernandes 5/13/2010 10:42'!
middlePointBtn
	^ self buttonAtHand: self drgeoPresenter middleButtonData    ! !

!DrGService methodsFor: 'button' stamp: 'HilaireFernandes 5/13/2010 10:42'!
moveBtn
	^ self buttonAtHand: self drgeoPresenter moveButtonData     ! !

!DrGService methodsFor: 'button' stamp: 'HilaireFernandes 5/13/2010 10:42'!
multipleModeBtn          
	^ self buttonAtHand: self drgeoPresenter multipleModeButtonData        ! !

!DrGService methodsFor: 'button' stamp: 'HilaireFernandes 5/13/2010 10:42'!
parallelBtn
	^ self buttonAtHand: self drgeoPresenter parallelButtonData      ! !

!DrGService methodsFor: 'button' stamp: 'HilaireFernandes 5/13/2010 10:42'!
perpendicularBtn
	^ self buttonAtHand: self drgeoPresenter parallelButtonData! !

!DrGService methodsFor: 'button' stamp: 'HilaireFernandes 5/13/2010 10:42'!
pointByCoordinatesBtn 
	^ self buttonAtHand: self drgeoPresenter pointByCoordinatesButtonData ! !

!DrGService methodsFor: 'button' stamp: 'HilaireFernandes 5/13/2010 10:42'!
polygonBtn 
	^ self buttonAtHand: self drgeoPresenter polygonButtonData ! !

!DrGService methodsFor: 'menu' stamp: 'HilaireFernandes 4/20/2010 07:55'!
popupMenu
	"super addCustomMenuItems: menu hand: aHandMorph."
	| menu |
	menu := MenuMorph new title: 'Dr. Geo tools' translated.
	self addToolsMenuItems: menu hand: nil.
	menu addStayUpItem.
	menu popUpInWorld: World! !

!DrGService methodsFor: 'button' stamp: 'HilaireFernandes 5/13/2010 10:42'!
propertyBtn  
	^ self buttonAtHand: self drgeoPresenter propertyButtonData ! !

!DrGService methodsFor: 'button' stamp: 'HilaireFernandes 5/13/2010 10:42'!
rayBtn   
	^ self buttonAtHand: self drgeoPresenter rayButtonData! !

!DrGService methodsFor: 'button' stamp: 'HilaireFernandes 5/13/2010 10:42'!
redoBtn    
	^ self buttonAtHand: self drgeoPresenter redoButtonData ! !

!DrGService methodsFor: 'button' stamp: 'HilaireFernandes 5/13/2010 10:43'!
reflectionBtn     
	^ self buttonAtHand: self drgeoPresenter reflectionButtonData  ! !

!DrGService methodsFor: 'button' stamp: 'HilaireFernandes 5/13/2010 10:43'!
rotationBtn      
	^ self buttonAtHand: self drgeoPresenter rotationButtonData  ! !

!DrGService methodsFor: 'button' stamp: 'HilaireFernandes 5/13/2010 10:43'!
scriptBtn      
	^ self buttonAtHand: self drgeoPresenter scriptButtonData   ! !

!DrGService methodsFor: 'button' stamp: 'HilaireFernandes 5/13/2010 10:43'!
scriptEditBtn      
	^ self buttonAtHand: self drgeoPresenter scriptEditButtonData   ! !

!DrGService methodsFor: 'button' stamp: 'HilaireFernandes 5/13/2010 10:43'!
segmentBtn      
	^ self buttonAtHand: self drgeoPresenter segmentButtonData    ! !

!DrGService methodsFor: 'button' stamp: 'HilaireFernandes 5/13/2010 10:43'!
styleBtn      
	^ self buttonAtHand: self drgeoPresenter styleButtonData    ! !

!DrGService methodsFor: 'button' stamp: 'HilaireFernandes 5/13/2010 10:43'!
symmetryBtn       
	^ self buttonAtHand: self drgeoPresenter symmetryButtonData     ! !

!DrGService methodsFor: 'accessing' stamp: 'HilaireFernandes 12/6/2011 15:28'!
title
	^ 'Unnamed Dr. Geo service' translated! !

!DrGService methodsFor: 'accessing' stamp: 'HilaireFernandes 8/4/2010 09:46'!
title: aString! !

!DrGService methodsFor: 'button' stamp: 'HilaireFernandes 5/13/2010 10:43'!
translationBtn        
	^ self buttonAtHand: self drgeoPresenter translationButtonData     ! !

!DrGService methodsFor: 'button' stamp: 'HilaireFernandes 5/13/2010 10:43'!
undoBtn         
	^ self buttonAtHand: self drgeoPresenter undoButtonData      ! !

!DrGService methodsFor: 'button' stamp: 'HilaireFernandes 5/13/2010 10:43'!
vectorBtn         
	^ self buttonAtHand: self drgeoPresenter vectorButtonData      ! !

!DrGService methodsFor: 'button' stamp: 'HilaireFernandes 5/13/2010 10:43'!
viewerBtn         
	^ self buttonAtHand: self drgeoPresenter viewerButtonData       ! !

BorderedMorph subclass: #WheelMorph
	instanceVariableNames: 'target actionSelector factor circular old angle maxAngle lastRedraw'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'DrGeoII-Core-UI'!

!WheelMorph commentStamp: '<historical>' prior: 0!
A thumb wheel widget control!

!WheelMorph methodsFor: 'accessing' stamp: 'ti 3/13/2000 12:56'!
actionSelector
	^ actionSelector
! !

!WheelMorph methodsFor: 'accessing' stamp: 'CedrickBeler 4/23/2010 17:43'!
actionSelector: aSymbolOrString

	(nil = aSymbolOrString or:
	 ['nil' = aSymbolOrString or:
	 [aSymbolOrString isEmpty]])
		ifTrue: [^ actionSelector := nil].

	actionSelector := aSymbolOrString asSymbol.

! !

!WheelMorph methodsFor: 'private' stamp: 'ti 3/20/2000 15:58'!
addAngle: aNumber
	| new |
	new := self angle + aNumber.
	self isCircular ifTrue: [new := new \\ self maxAngle].
	self angle: new.
	lastRedraw := lastRedraw + aNumber.
	(lastRedraw abs > 2) ifTrue: [
		lastRedraw := 0.
		self changed]! !

!WheelMorph methodsFor: 'accessing' stamp: 'ti 3/19/2000 11:32'!
angle
	^angle! !

!WheelMorph methodsFor: 'accessing' stamp: 'ti 3/19/2000 11:33'!
angle: aNumber
	angle := aNumber! !

!WheelMorph methodsFor: 'properties' stamp: 'ti 3/20/2000 13:18'!
beCircular
	circular := true! !

!WheelMorph methodsFor: 'properties' stamp: 'bf 3/23/2000 18:34'!
beHorizontal
	self isHorizontal
		ifFalse: [self extent: self extent transposed] ! !

!WheelMorph methodsFor: 'properties' stamp: 'ti 3/20/2000 13:19'!
beLinear
	circular := false! !

!WheelMorph methodsFor: 'properties' stamp: 'bf 3/23/2000 18:34'!
beVertical
	self isHorizontal
		ifTrue: [self extent: self extent transposed] ! !

!WheelMorph methodsFor: 'event handling' stamp: 'ti 3/19/2000 17:03'!
doTargetAction: cursorPoint
	| new delta |
	self isHorizontal
		ifTrue: [new := cursorPoint x]
		ifFalse: [new := cursorPoint y].
	delta := new - old.
	delta isZero ifTrue: [^self].
	self addAngle: delta.
	(target ~~ nil and: [actionSelector ~~ nil]) ifTrue: [
		Cursor normal showWhile: [
			target perform: actionSelector withArguments: (Array with: (delta * self factor))]].
	old := new.
! !

!WheelMorph methodsFor: 'drawing' stamp: 'HilaireFernandes 4/23/2010 16:33'!
drawOn: aCanvas
	| interval delta frame x0 x1 y0 y1 deltaX deltaY  hilitColor shadowColor |
	super drawOn: aCanvas.		"border & background"
	borderColor isColor
		ifTrue: [hilitColor := shadowColor := borderColor]
		ifFalse: [hilitColor := color lighter lighter.
			shadowColor := color darker darker].
	frame := self innerBounds insetBy: 2.
	aCanvas frameAndFillRectangle: frame fillColor: color
		borderWidth: 1 topLeftColor: hilitColor bottomRightColor: shadowColor.
	frame := frame insetBy: (borderColor isColor ifTrue: [1] ifFalse: [2]).
	x0 := frame origin x.
	x1 := frame corner x.
	y0 := frame origin y.
	y1 := frame corner y.
	deltaX := (x1 - x0) / 2.
	deltaY := (y1 - y0) / 2.
	interval := 10.
	delta := self angle \\ interval.
	1 to: (self maxAngle / (2 * interval)) rounded do: [ :i |
		| x y |
		self isHorizontal ifTrue: [
			x := x0 + deltaX - (((i * interval + delta) * (2 * Float pi / 360.0)) cos * deltaX).
			aCanvas fillRectangle: (Rectangle origin: (x-1)@y0 corner: x@y1) color: hilitColor.
			borderColor isColor ifFalse:
				 [aCanvas fillRectangle: (Rectangle origin: x@y0 corner: (x+1)@y1) color: shadowColor]]
		ifFalse: [
			y := y0 + deltaY - (((i * interval + delta) * (2 * Float pi / 360.0)) cos * deltaY).
			aCanvas fillRectangle: (Rectangle origin: x0@(y-1) corner: x1@y) color: hilitColor.
			borderColor isColor ifFalse:
				 [aCanvas fillRectangle: (Rectangle origin: x0@y corner: x1@(y+1)) color: shadowColor]]]! !

!WheelMorph methodsFor: 'accessing' stamp: 'ti 3/19/2000 17:02'!
factor
	^factor! !

!WheelMorph methodsFor: 'accessing' stamp: 'ti 3/19/2000 17:03'!
factor: aFloat
	factor := aFloat! !

!WheelMorph methodsFor: 'event handling' stamp: 'ti 3/19/2000 14:50'!
handlesMouseDown: evt
	^true
! !

!WheelMorph methodsFor: 'initialization' stamp: 'HilaireFernandes 1/27/2008 10:22'!
initialize
	super initialize.
	self target: nil.
	self actionSelector: #flash.
	self angle: 0.
	self maxAngle: 360.
	self factor: 1.0.
	self extent: 100@23.
	self beCircular.
	self borderColor: #raised.
	self borderWidth: 1.
	self color: Color lightGray.
	lastRedraw := 0.! !

!WheelMorph methodsFor: 'properties' stamp: 'ti 3/20/2000 13:19'!
isCircular
	^circular! !

!WheelMorph methodsFor: 'properties' stamp: 'bf 3/23/2000 18:26'!
isHorizontal
	^ bounds isWide! !

!WheelMorph methodsFor: 'properties' stamp: 'ti 3/20/2000 13:19'!
isLinear
	^circular not! !

!WheelMorph methodsFor: 'properties' stamp: 'ti 3/22/2000 19:12'!
isVertical
	^self isHorizontal not! !

!WheelMorph methodsFor: 'accessing' stamp: 'ti 3/19/2000 11:35'!
maxAngle
	^maxAngle! !

!WheelMorph methodsFor: 'accessing' stamp: 'ti 3/19/2000 11:35'!
maxAngle: aNumber
	maxAngle := aNumber! !

!WheelMorph methodsFor: 'event handling' stamp: 'ti 3/14/2000 18:09'!
mouseDown: evt
	self isHorizontal
		ifTrue: [old := evt cursorPoint x]
		ifFalse: [old := evt cursorPoint y].! !

!WheelMorph methodsFor: 'event handling' stamp: 'ti 3/19/2000 14:51'!
mouseMove: evt
	(evt anyButtonPressed)
		ifTrue: [self doTargetAction: evt cursorPoint]! !

!WheelMorph methodsFor: 'accessing' stamp: 'ti 3/13/2000 12:47'!
target
	^target! !

!WheelMorph methodsFor: 'accessing' stamp: 'ti 3/13/2000 12:48'!
target: aTargetObject
	target := aTargetObject! !

PasteUpMorph subclass: #DrGDrawable
	instanceVariableNames: 'balloonTarget tipOn selection app ox oy scale'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'DrGeoII-Core-App'!

!DrGDrawable commentStamp: '<historical>' prior: 0!
I represent the area/place where a Drgeo figure is drawn. It is a subpart of the GeometricView
!

!DrGDrawable class methodsFor: 'instance creation' stamp: 'HilaireFernandes 4/19/2010 10:34'!
newOn: aDrgeo
	^self basicNew app: aDrgeo ; initialize ! !

!DrGDrawable methodsFor: 'accessing' stamp: 'HilaireFernandes 4/19/2010 10:33'!
app
	^app! !

!DrGDrawable methodsFor: 'accessing' stamp: 'HilaireFernandes 4/19/2010 10:33'!
app: aDrgeo
	app := aDrgeo! !

!DrGDrawable methodsFor: 'genie-drawing' stamp: 'HilaireFernandes 10/16/2010 15:59'!
circle: aCRGesture
| center aPoint circle|
	center := self getCircleCenter: aCRGesture.
	aPoint := self getCirclePoint: aCRGesture.
	circle := DrGCircle2ptsItem newWith: {center . aPoint}.
	app domain createFromMathItem: circle! !

!DrGDrawable methodsFor: 'geometry' stamp: 'HilaireFernandes 4/20/2010 08:44'!
defaultScalePosition
	ox := oy := 0.0.
	scale := 34.0.
	app redrawCostumes ! !

!DrGDrawable methodsFor: 'meta-actions' stamp: 'Hilaire 4/20/2010 07:51'!
dismissMorph
	super dismissMorph.
	app view dismissMorph ! !

!DrGDrawable methodsFor: 'genie-command' stamp: 'HilaireFernandes 4/20/2010 08:36'!
down: aNumber
	oy := oy + (self pixelToWorld: aNumber).
	app redrawCostumes.
! !

!DrGDrawable methodsFor: 'accessing' stamp: 'HilaireFernandes 5/13/2010 10:33'!
drgeoPresenter
	^app presenter! !

!DrGDrawable methodsFor: 'layout' stamp: 'HilaireFernandes 4/19/2010 12:37'!
extent: aPoint 
	super extent: aPoint.
	app redrawCostumes.
	" XXX we should not need it as I will turn DrGService as a Morph and wheel widgets will be pluggued with laout
	app isService ifTrue: 
	[app horizontalWheel right: self right - app view wheelWidth;
		 bottom: self bottom.
	app verticalWheel right: self right;
		 bottom: self bottom - app view wheelWidth.
	app zoomWheel right: self right;
		 top: self top]"! !

!DrGDrawable methodsFor: 'genie-drawing' stamp: 'HilaireFernandes 10/16/2010 16:10'!
getCircleCenter: CRGesture
|center|
	center := app pointCostumeClosestTo: CRGesture bounds center atRange: 15.
	center 
		ifNotNil: [center := center mathItem]
		ifNil: 
			[center := DrGPointFreeItem newWith: {(self morphToWorld: CRGesture bounds center)}.
			app domain createFromMathItem: center].
	^center! !

!DrGDrawable methodsFor: 'genie-drawing' stamp: 'HilaireFernandes 10/16/2010 16:10'!
getCirclePoint: aCRGesture
|closestCostume smallestDistance capturedPoints pointItem|
	smallestDistance := 15.
	capturedPoints := aCRGesture capturedPointsInScreenCoord.
	app pointCostumes do: [:aCostume |
		capturedPoints do: [:aPoint | |dist|
			(dist := aCostume distanceTo: aPoint) < smallestDistance 
				ifTrue:
					[closestCostume :=  aCostume.
					smallestDistance := dist]]].
	closestCostume 
		ifNotNil: [pointItem := closestCostume mathItem]
		ifNil:
			[pointItem := DrGPointFreeItem newWith: 
				{(self morphToWorld: aCRGesture coordinates start)}.
			app domain createFromMathItem: pointItem].
	^pointItem! !

!DrGDrawable methodsFor: 'gridding' stamp: 'HilaireFernandes 5/13/2010 10:33'!
gridPoint: ungriddedPoint
	self drgeoPresenter isGridded ifFalse: [^ungriddedPoint].
	"do it the easy way"
	^self worldToMorph: ((self morphToWorld: ungriddedPoint) rounded)! !

!DrGDrawable methodsFor: 'gridding' stamp: 'HilaireFernandes 5/13/2010 10:34'!
gridWorldPoint: ungriddedPoint
	self drgeoPresenter isGridded ifFalse: [^ self morphToWorld: ungriddedPoint].
	"do it the easy way"
	^(self morphToWorld: ungriddedPoint) rounded! !

!DrGDrawable methodsFor: 'event handling' stamp: 'HilaireFernandes 11/9/2011 11:37'!
handleMouseMove: anEvent
	
	"System level event handling."
	anEvent wasHandled ifTrue:[^self]. "not interested"
	"Our drawable gets #mouseMove iff
		* the hand is not dragging anything,
			+ and the receiver is the current mouse focus."
	(anEvent hand hasSubmorphs) ifTrue:[^self].
	self mouseMove: anEvent.
! !

!DrGDrawable methodsFor: 'event handling' stamp: 'HilaireFernandes 11/9/2011 22:40'!
handlesMouseWheel: evt
	^ true! !

!DrGDrawable methodsFor: 'balloon target' stamp: 'HilaireFernandes 1/12/2006 11:01'!
hideTip
	self removeMorph: balloonTarget! !

!DrGDrawable methodsFor: 'initialization' stamp: 'HilaireFernandes 4/21/2010 07:42'!
initialize
	super initialize.
	self color: Color white.
	self borderWidth: 0.
	tipOn := true.
	self defaultScalePosition.
	self hResizing: #spacefill;
		 vResizing: #spaceFill.
	self clipSubmorphs: true.
	balloonTarget := RectangleMorph new color: Color transparent;
				 extent: 6 @ 6;
				 borderWidth: 0.

! !

!DrGDrawable methodsFor: 'genie-command' stamp: 'HilaireFernandes 4/20/2010 08:36'!
left: aNumber 
	ox := ox + (self pixelToWorld: aNumber).
	app redrawCostumes
! !

!DrGDrawable methodsFor: 'geometry' stamp: 'HilaireFernandes 4/20/2010 08:43'!
morphAbscissaToWorld: abscissa
	"Transform abscissa from DrGeo-Morph to World  system"
	^ ox + ((abscissa - self position x - (self extent x / 2)) / scale)
! !

!DrGDrawable methodsFor: 'geometry' stamp: 'HilaireFernandes 4/20/2010 08:43'!
morphOrdinateToWorld: ordinate
	"Transform point coordinate from DrGeo-Morph to World  system"
	^ oy - ((ordinate  - self position y - (self extent y / 2)) / scale)! !

!DrGDrawable methodsFor: 'geometry' stamp: 'HilaireFernandes 4/20/2010 08:43'!
morphToVector: aPoint
"Transform vector coordinates in the morph system to the DrGeo system"
	^ Point 
		x: aPoint x asFloat / scale
		y: aPoint y negated asFloat / scale.
! !

!DrGDrawable methodsFor: 'geometry' stamp: 'HilaireFernandes 4/20/2010 08:43'!
morphToWorld: aPoint
	"Transform point coordinate from DrGeo-Morph to World  system"
	^ (Point 
		x: ox + ((aPoint x - self position x - (self extent x / 2)) / scale)
		y: oy - ((aPoint y - self position y - (self extent y / 2)) / scale))! !

!DrGDrawable methodsFor: 'event handling' stamp: 'HilaireFernandes 11/12/2011 22:40'!
mouseDown: evt 
	evt yellowButtonPressed ifTrue: [^self]. "we don't capture it"
	evt shiftPressed 
		ifTrue: [self tool handlePressShiftKey: evt cursorPoint]
		ifFalse: 	[(self tool handlePress: evt cursorPoint)
			ifFalse: [self view popupMenu]]! !

!DrGDrawable methodsFor: 'event handling' stamp: 'HilaireFernandes 11/12/2011 23:05'!
mouseMove: evt
	evt yellowButtonPressed ifTrue: [|delta| 
		delta := evt endPoint - evt startPoint.
		self moveOxy: (self pixelVectorToWorldVector: delta)].
	evt shiftPressed 
		ifTrue: [self tool handleShiftKeyMouseAt: evt cursorPoint]
		ifFalse: [self tool handleMouseAt: evt cursorPoint]
! !

!DrGDrawable methodsFor: 'event handling' stamp: 'HilaireFernandes 11/12/2011 22:48'!
mouseUp: evt
	evt shiftPressed 
		ifTrue: [self tool handleReleaseShiftKey: evt cursorPoint]
		ifFalse: [self tool handleRelease: evt cursorPoint]
! !

!DrGDrawable methodsFor: 'event handling' stamp: 'HilaireFernandes 11/9/2011 22:45'!
mouseWheel: evt
	|increment|
	evt shiftPressed 
		ifTrue: [increment := 6]
		ifFalse: [increment := 2].
	evt direction = #up ifTrue: [	self scale: increment ].
	evt direction = #down ifTrue: [	self scale: increment negated].! !

!DrGDrawable methodsFor: 'callback canvas' stamp: 'HilaireFernandes 4/20/2010 13:26'!
moveOx: increment
	ox := ox - increment.
	app redrawCostumes! !

!DrGDrawable methodsFor: 'callback canvas' stamp: 'HilaireFernandes 11/9/2011 21:27'!
moveOxy: pointIncrement
	ox := ox + pointIncrement x.
	oy := oy + pointIncrement y.
	app redrawCostumes! !

!DrGDrawable methodsFor: 'callback canvas' stamp: 'HilaireFernandes 4/20/2010 13:27'!
moveOy: increment
	oy := oy + increment.
	app redrawCostumes! !

!DrGDrawable methodsFor: 'accessing' stamp: 'HilaireFernandes 4/20/2010 08:41'!
ox
	^ ox! !

!DrGDrawable methodsFor: 'accessing' stamp: 'HilaireFernandes 4/21/2010 17:55'!
ox: integer
	ox := integer asNumber! !

!DrGDrawable methodsFor: 'accessing' stamp: 'HilaireFernandes 4/20/2010 08:41'!
oy
	^ oy! !

!DrGDrawable methodsFor: 'accessing' stamp: 'HilaireFernandes 4/21/2010 17:55'!
oy: integer
	oy := integer asNumber! !

!DrGDrawable methodsFor: 'geometry' stamp: 'HilaireFernandes 4/20/2010 08:43'!
pixelToWorld: pixels
"Transform morph pixel unit to Dr. Geo world unit"
	^ pixels asFloat / scale

! !

!DrGDrawable methodsFor: 'geometry' stamp: 'HilaireFernandes 11/9/2011 21:47'!
pixelVectorToWorldVector: pixelVector
"Transform morph pixel unit to Dr. Geo world unit"
	^ (pixelVector / scale) * (-1 @ 1)
! !

!DrGDrawable methodsFor: 'genie-command' stamp: 'HilaireFernandes 5/13/2010 10:34'!
redo
	self drgeoPresenter redo
! !

!DrGDrawable methodsFor: 'dependents access' stamp: 'HilaireFernandes 10/4/2010 21:32'!
release
	balloonTarget dismissMorph.
	self submorphsDo: [:m | 
		self removeMorph: m.
		m dismissMorph].
	super release.
	app := nil! !

!DrGDrawable methodsFor: 'genie-command' stamp: 'HilaireFernandes 4/20/2010 08:37'!
right: aNumber 
	ox := ox - (self pixelToWorld: aNumber).
	app redrawCostumes! !

!DrGDrawable methodsFor: 'accessing' stamp: 'HilaireFernandes 4/20/2010 08:42'!
scale
	^ scale! !

!DrGDrawable methodsFor: 'accessing' stamp: 'HilaireFernandes 4/21/2010 07:44'!
scale: increment
	scale := (scale + increment) asInteger.
	scale < 5 ifTrue: [scale := 4].
	scale > 200 ifTrue: [scale := 200].
	app redrawCostumes.! !

!DrGDrawable methodsFor: 'genie-drawing' stamp: 'HilaireFernandes 3/10/2006 10:54'!
segment: aCRGesture
	Transcript show: 'segment'; cr.! !

!DrGDrawable methodsFor: 'accessing' stamp: 'HilaireFernandes 4/21/2010 17:54'!
setScaleTo: number
	scale := number asNumber! !

!DrGDrawable methodsFor: 'balloon target' stamp: 'HilaireFernandes 3/6/2006 11:25'!
tip: aText at: aPoint
	tipOn ifFalse: [^self].
	self addMorphBack: balloonTarget.
	balloonTarget center: aPoint.
	balloonTarget setBalloonText: aText.
	balloonTarget showBalloon: (balloonTarget balloonText)! !

!DrGDrawable methodsFor: 'balloon target' stamp: 'HilaireFernandes 1/31/2006 12:13'!
toggleTip
	tipOn := tipOn not! !

!DrGDrawable methodsFor: 'accessing' stamp: 'HilaireFernandes 7/19/2011 19:45'!
tool
	^self drgeoPresenter tool! !

!DrGDrawable methodsFor: 'genie-command' stamp: 'HilaireFernandes 5/13/2010 10:34'!
undo
	self drgeoPresenter undo
! !

!DrGDrawable methodsFor: 'genie-command' stamp: 'HilaireFernandes 4/20/2010 08:39'!
unzoom: extent
	scale := scale - ((extent x + extent y) / 5).
	scale < 5 ifTrue: [scale := 5].
	app redrawCostumes! !

!DrGDrawable methodsFor: 'genie-command' stamp: 'HilaireFernandes 4/20/2010 08:40'!
up: aNumber
	oy := oy - (self pixelToWorld: aNumber).
	app redrawCostumes! !

!DrGDrawable methodsFor: 'gridding' stamp: 'HilaireFernandes 7/6/2011 11:21'!
updateGrid
	self gridVisible ifTrue:
		[self gridSpecPut: ((self worldToMorph: 0@0) - self position extent: scale@scale)]! !

!DrGDrawable methodsFor: 'geometry' stamp: 'HilaireFernandes 4/20/2010 08:44'!
vectorToMorph: aVector
"Transform vector coordinates in the Dr. Geo system to the morph system"
	^ Point 
		x: aVector x asFloat * scale
		y: aVector y negated asFloat * scale.
! !

!DrGDrawable methodsFor: 'accessing' stamp: 'HilaireFernandes 4/20/2010 07:59'!
view
	^app view! !

!DrGDrawable methodsFor: 'as yet unclassified' stamp: 'HilaireFernandes 4/21/2010 07:50'!
wantsHalo
	^false! !

!DrGDrawable methodsFor: 'geometry' stamp: 'HilaireFernandes 4/20/2010 08:44'!
worldAbscissaToMorph: x
	"Transform point abscissa from DrGeo-World to Morph system"
	^ (self extent x // 2 + (scale * (x - ox))) asInteger 
		+ self position x! !

!DrGDrawable methodsFor: 'geometry' stamp: 'HilaireFernandes 4/2/2011 09:55'!
worldBound
	^ Rectangle
		origin: (self morphToWorld: self bottomLeft)
		corner: (self morphToWorld: self topRight)! !

!DrGDrawable methodsFor: 'geometry' stamp: 'HilaireFernandes 4/20/2010 08:44'!
worldOrdinateToMorph: y
	"Transform point ordinate from DrGeo-World to Morph system"
	^ (self extent y // 2 + (scale * (oy - y))) asInteger
		+ self position y! !

!DrGDrawable methodsFor: 'geometry' stamp: 'HilaireFernandes 4/20/2010 08:44'!
worldToMorph: aPoint
	"Transform point coordinate from DrGeo-World to Morph system"
	^ (Point 
		x: ((self extent x // 2 + (scale * (aPoint x - ox))) asInteger) 
		y: ((self extent y // 2 + (scale * (oy - aPoint y)))) asInteger)
	+ self position! !

!DrGDrawable methodsFor: 'geometry' stamp: 'HilaireFernandes 4/20/2010 08:44'!
worldToPixel: aFloat
"Transform Dr. Geo world unit to morph unit"
	^ (aFloat * scale) asInteger 

! !

!DrGDrawable methodsFor: 'callback canvas' stamp: 'HilaireFernandes 4/20/2010 13:32'!
zoom: extent
	scale := (scale + (extent x + extent y / 5)) asInteger.
	scale > 200 ifTrue: [scale := 200].
	app redrawCostumes ! !

!Float methodsFor: '*DrGeoII-Core-fixing' stamp: 'HilaireFernandes 1/16/2006 22:00'!
arcTan2: denominator
	"Answer the angle in radians.
	 Optional. See Object documentation whatIsAPrimitive."

	| result |

	(self = 0.0) ifTrue: [ (denominator > 0.0) ifTrue: [ result := 0 ]
										    ifFalse: [ result := Pi ]
						]
			    ifFalse: [(denominator = 0.0)
					ifTrue: [ (self > 0.0) ifTrue: [ result := Halfpi ]
												ifFalse: [ result := Halfpi negated ]
							]
					ifFalse: [ (denominator > 0) ifTrue: [ result := (self / denominator) arcTan ]
								 ifFalse: [ (self > 0) ifTrue: [result := ((self / denominator) arcTan) + Pi ]
															ifFalse: [result := ((self / denominator) arcTan) - Pi]
											]
							].
						].
	
	^ result.! !

ReadWriteStream subclass: #DrGCommandStack
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'DrGeoII-Core-Command'!

!DrGCommandStack class methodsFor: 'instance creation' stamp: 'HilaireFernandes 12/21/2005 14:20'!
new
	^self on: Array new! !

!DrGCommandStack methodsFor: 'accessing' stamp: 'HilaireFernandes 12/21/2005 13:57'!
nextPut: aCommand
	super nextPut: aCommand.
	self truncate.
	^ aCommand ! !

!DrGCommandStack methodsFor: 'accessing' stamp: 'HilaireFernandes 12/21/2005 14:08'!
previous
	self position = 0 ifTrue: [^nil].
	self position: self position - 1.
	^self peek.! !

!DrGCommandStack methodsFor: 'private' stamp: 'HilaireFernandes 12/21/2005 13:57'!
truncate
	|oldReadLimit|
	oldReadLimit := readLimit.
	readLimit := position.
	oldReadLimit > readLimit ifTrue:
			[readLimit to: oldReadLimit do:
				[:index| collection at: index + 1 put: nil]]! !
DrGDirectionItem initialize!
DrGLoader initialize!
DrGDefault initialize!
